@@ -1,29 +1,14 @@
-// FGAIBallistic - FGAIBase-derived class creates a ballistic object
-//
-// Written by David Culp, started November 2003.
-// - davidculp2@comcast.net
-//
-// With major additions by Mathias Froehlich & Vivian Meazza 2004-2008
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+/*
+ * SPDX-FileName: AIBallistic.cxx
+ * SPDX-FileComment: GAIBase-derived class creates a ballistic object
+ * SPDX-FileCopyrightText: Copyright (C) 2003  David P. Culp - davidculp2@comcast.net
+ * SPDX-FileContributor: With major additions by Mathias Froehlich & Vivian Meazza 2004-2008
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
-#include <simgear/math/sg_random.h>
+#include <simgear/math/sg_random.hxx>
 #include <simgear/math/sg_geodesy.hxx>
 #include <simgear/scene/model/modellib.hxx>
 
@@ -33,6 +18,7 @@
 
 #include <Main/util.hxx>
 #include <Environment/gravity.hxx>
+#include <Environment/atmosphere.hxx>
 #include <Main/fg_props.hxx>
 
 using namespace simgear;
@@ -80,8 +66,6 @@ _impact_report_node(fgGetNode("/ai/models/model-impact", true))
     no_roll = false;
 }
 
-FGAIBallistic::~FGAIBallistic() {
-}
 
 void FGAIBallistic::readFromScenario(SGPropertyNode* scFileNode) {
     if (!scFileNode){
@@ -131,8 +115,9 @@ void FGAIBallistic::readFromScenario(SGPropertyNode* scFileNode) {
     setParentName(scFileNode->getStringValue("parent"));
 }
 
-bool FGAIBallistic::init(bool search_in_AI_path) {
-    FGAIBase::init(search_in_AI_path);
+bool FGAIBallistic::init(ModelSearchOrder searchOrder)
+{
+    FGAIBase::init(searchOrder);
     reinit();
     return true;
 }
@@ -224,7 +209,7 @@ void FGAIBallistic::bind() {
         tie("position/global-z",
             SGRawValueMethods<FGAIBase,double>(*this, &FGAIBase::_getCartPosZ, 0));
         tie("velocities/vertical-speed-fps",
-            SGRawValuePointer<double>(&vs));
+            SGRawValueMethods<FGAIBase,double>(*this, &FGAIBase::_getVS_fps, &FGAIBase::_setVS_fps));
         tie("velocities/true-airspeed-kt",
             SGRawValuePointer<double>(&speed));
         tie("velocities/horizontal-speed-fps",
@@ -696,6 +681,10 @@ void FGAIBallistic::Run(double dt) {
     // for a conventional shell/bullet (no boat-tail).
     double Cdm;
 
+    const double Mach = FGAtmo::machFromKnotsAtAltitudeFt(speed, altitude_ft);
+    const double rhoKgM3 =  FGAtmo::densityAtAltitudeFt(altitude_ft);
+    const double rho = rhoKgM3 / SG_SLUGFT3_TO_KGPM3;
+    
     if (Mach < 0.7)
         Cdm = 0.0125 * Mach + _cd;
     else if (Mach < 1.2)
@@ -711,7 +700,10 @@ void FGAIBallistic::Run(double dt) {
     // using Standard Atmosphere (sealevel temperature 15C)
     // acceleration = drag/mass;
     // adjust speed by drag
-    speed -= (Cdm * 0.5 * rho * speed * speed * _drag_area/_mass) * dt;
+    //
+    // speed is in kt, rest is in ft and sec, so we convert speed^2 to fps (* SG_KT_TO_FPS^2)
+    // then the result back to kt (/ SG_KT_TO_FPS), which is the same as just * SG_KT_TO_FPS.
+    speed -= (Cdm * 0.5 * rho * speed * speed * _drag_area/_mass) * dt * SG_KT_TO_FPS;
 
     // don't let speed become negative
     if (speed < 0.0)
@@ -737,56 +729,53 @@ void FGAIBallistic::Run(double dt) {
     }
 
     // Calculate velocity due to external force
-    double force_speed_north_deg_sec = 0;
-    double force_speed_east_deg_sec = 0;
-    double hs_force_fps = 0;
-    double v_force_acc_fpss = 0;
-    double force_speed_north_fps = 0;
-    double force_speed_east_fps = 0;
-    double h_force_lbs = 0;
-    double normal_force_lbs = 0;
-    double normal_force_fpss = 0;
-    double static_friction_force_lbs = 0;
-    double dynamic_friction_force_lbs = 0;
-    double friction_force_speed_north_fps = 0;
-    double friction_force_speed_east_fps = 0;
-    double friction_force_speed_north_deg_sec = 0;
-    double friction_force_speed_east_deg_sec = 0;
-    double force_elevation_deg = 0;
-    double force_azimuth_deg  = 0;
-    double force_lbs = 0;
+    double force_speed_north_deg_sec = 0.0;
+    double force_speed_east_deg_sec = 0.0;
+    double v_force_acc_fpss = 0.0;
+    double force_speed_north_fps = 0.0;
+    double force_speed_east_fps = 0.0;
+    double h_force_lbs = 0.0;
+    double normal_force_fpss = 0.0;
+    double friction_force_speed_north_fps = 0.0;
+    double friction_force_speed_east_fps = 0.0;
+    double friction_force_speed_north_deg_sec = 0.0;
+    double friction_force_speed_east_deg_sec = 0.0;
+    double force_elevation_deg = 0.0;
 
     if (_external_force) {
         //cout << _name << " external force " <<  hdg << " az " << _azimuth << endl;
 
         SGPropertyNode *n = fgGetNode(_force_path.c_str(), true);
-        force_lbs            = n->getChild("force-lb", 0, true)->getDoubleValue();
+        double force_lbs = n->getChild("force-lb", 0, true)->getDoubleValue();
         force_elevation_deg  = n->getChild("force-elevation-deg", 0, true)->getDoubleValue();
-        force_azimuth_deg    = n->getChild("force-azimuth-deg", 0, true)->getDoubleValue();
+        double force_azimuth_deg = n->getChild("force-azimuth-deg", 0, true)->getDoubleValue();
         
         // Resolve force into vertical and horizontal components:
         double v_force_lbs = force_lbs * sin( force_elevation_deg * SG_DEGREES_TO_RADIANS );
         h_force_lbs = force_lbs * cos( force_elevation_deg * SG_DEGREES_TO_RADIANS );
 
+        double normal_force_lbs = 0.0;
+        double dynamic_friction_force_lbs = 0.0;
+
         // Perform ground interaction if impacts are not calculated
         if (!_report_impact && getHtAGL(10000)) {
             double deadzone = 0.1;
 
-            if (_ht_agl_ft <= (0 + _ground_offset + deadzone) && _solid) {
+            if (_ht_agl_ft <= (_ground_offset + deadzone) && _solid) {
                 normal_force_lbs = (_mass * slugs_to_lbs) - v_force_lbs;
 
-                if (normal_force_lbs < 0)
-                    normal_force_lbs = 0;
+                if (normal_force_lbs < 0.0)
+                    normal_force_lbs = 0.0;
 
                 pos.setElevationFt(0 + _ground_offset);
-                if (vs < 0)
-                    vs = -vs * 0.5;
+                if (vs_fps < 0.0)
+                    vs_fps *= -0.5;
 
                 // Calculate friction. We assume a static coefficient of
                 // friction (mu) of 0.62 (wood on concrete)
                 double mu = 0.62;
 
-                static_friction_force_lbs = mu * normal_force_lbs * _frictionFactor;
+                double static_friction_force_lbs = mu * normal_force_lbs * _frictionFactor;
 
                 // Adjust horizontal force. We assume that a speed of <= 5 fps is static
                 if (h_force_lbs <= static_friction_force_lbs && hs <= 5) {
@@ -794,10 +783,10 @@ void FGAIBallistic::Run(double dt) {
                     _speed_north_fps = _speed_east_fps = 0;
                 }
                 else
-                    dynamic_friction_force_lbs = (static_friction_force_lbs * 0.95);
+                    dynamic_friction_force_lbs = static_friction_force_lbs * 0.95;
 
                 // Ignore wind when on the ground for now
-                //TODO fix this
+                // TODO: fix this
                 _wind_from_north = 0;
                 _wind_from_east = 0;
             }
@@ -810,7 +799,7 @@ void FGAIBallistic::Run(double dt) {
         double dynamic_friction_acc_fpss = dynamic_friction_force_lbs / _mass;
 
         // velocity = acceleration * dt
-        hs_force_fps = h_force_acc_fpss * dt;
+        double hs_force_fps = h_force_acc_fpss * dt;
         double friction_force_fps = dynamic_friction_acc_fpss * dt;
 
         //resolve horizontal speeds into north and east components:
@@ -843,23 +832,22 @@ void FGAIBallistic::Run(double dt) {
 
     // adjust vertical speed for acceleration of gravity, buoyancy, and vertical force
     double gravity = SG_METER_TO_FEET * (Environment::Gravity::instance()->getGravity(pos));
-    vs -= (gravity - _buoyancy - v_force_acc_fpss - normal_force_fpss) * dt;
+    vs_fps -= (gravity - _buoyancy - v_force_acc_fpss - normal_force_fpss) * dt;
 
-    if (vs <= 0.00001 && vs >= -0.00001)
-        vs = 0;
+    if (vs_fps <= 0.00001 && vs_fps >= -0.00001)
+        vs_fps = 0;
 
     // set new position
     if (_slave_load_to_ac) {
         setOffsetPos(pos, 
             manager->get_user_heading(),
             manager->get_user_pitch(), 
-            manager->get_user_roll()
-            );
+            manager->get_user_roll());
         pos.setLatitudeDeg(_offsetpos.getLatitudeDeg());
         pos.setLongitudeDeg(_offsetpos.getLongitudeDeg());
         pos.setElevationFt(_offsetpos.getElevationFt());
 
-        if (getHtAGL(10000)) {
+        if (getHtAGL(10000.0)) {
             double deadzone = 0.1;
 
             if (_ht_agl_ft <= (0 + _ground_offset + deadzone) && _solid) {
@@ -877,19 +865,19 @@ void FGAIBallistic::Run(double dt) {
         pos.setLongitudeDeg( pos.getLongitudeDeg()
             + (speed_east_deg_sec - wind_speed_from_east_deg_sec 
             + force_speed_east_deg_sec + friction_force_speed_east_deg_sec) * dt );
-        pos.setElevationFt(pos.getElevationFt() + vs * dt);
+        pos.setElevationFt(pos.getElevationFt() + vs_fps * dt);
     }
 
 //    cout << _name << " run hs " << hs << " vs " << vs << endl;
 
     // recalculate total speed
-    if ( vs == 0 && hs == 0)
+    if ( vs_fps == 0 && hs == 0)
         speed = 0;
     else
-        speed = sqrt( vs * vs + hs * hs) / SG_KT_TO_FPS;
+        speed = sqrt( vs_fps * vs_fps + hs * hs) / SG_KT_TO_FPS;
 
     // recalculate elevation and azimuth (velocity vectors)
-    _elevation = atan2( vs, hs ) * SG_RADIANS_TO_DEGREES;
+    _elevation = atan2( vs_fps, hs ) * SG_RADIANS_TO_DEGREES;
     _azimuth =  atan2((_speed_east_fps + force_speed_east_fps + friction_force_speed_east_fps), 
         (_speed_north_fps + force_speed_north_fps + friction_force_speed_north_fps))
         * SG_RADIANS_TO_DEGREES;
@@ -902,7 +890,7 @@ void FGAIBallistic::Run(double dt) {
         //cout<< "_aero_stabilised " << hdg << " az " << _azimuth << endl;
         const double coeff = 0.9;
 
-        // we assume a symetrical MI about the pitch and yaw axis
+        // we assume a symmetrical MI about the pitch and yaw axis
         setPch(_elevation, dt, coeff);
         setHdg(_azimuth, dt, coeff);
     }
@@ -920,7 +908,7 @@ void FGAIBallistic::Run(double dt) {
         if (force_pitch <= force_elevation_deg)
             force_pitch = force_elevation_deg;
 
-        // we assume a symetrical MI about the pitch and yaw axis
+        // we assume a symmetrical MI about the pitch and yaw axis
         setPch(force_pitch,dt, coeff);
         setHdg(_azimuth, dt, coeff);
     }
@@ -1001,7 +989,7 @@ void FGAIBallistic::report_impact(double elevation, const FGAIBase *object)
     SGPropertyNode *n = props->getNode("impact", true);
 
     if (object)
-        n->setStringValue("type", object->getTypeString());
+        n->setStringValue("type", static_cast<std::string>(object->getTypeString()));
     else
         n->setStringValue("type", "terrain");
 
@@ -1146,10 +1134,10 @@ void FGAIBallistic::calcVSHS() {
     double speed_fps = speed * SG_KT_TO_FPS;
 
     if (speed == 0.0) {
-        hs = vs = 0.0;
+        hs = vs_fps = 0.0;
     }
     else {
-        vs = sin( _elevation * SG_DEGREES_TO_RADIANS ) * speed_fps;
+        vs_fps = sin( _elevation * SG_DEGREES_TO_RADIANS ) * speed_fps;
         hs = cos( _elevation * SG_DEGREES_TO_RADIANS ) * speed_fps;
     }
 }
