@@ -43,7 +43,8 @@ using std::string;
 
 // TODO: Use James Turner's createOnGround functions.
 bool FGAIFlightPlan::createPushBack(FGAIAircraft *ac,
-                                    bool firstFlight, FGAirport *dep,
+                                    bool firstFlight,
+                                    FGAirport *dep,
                                     double radius,
                                     const string& fltType,
                                     const string& aircraftType,
@@ -52,66 +53,73 @@ bool FGAIFlightPlan::createPushBack(FGAIAircraft *ac,
     double vTaxi = ac->getPerformance()->vTaxi();
     double vTaxiBackward = vTaxi * (-2.0/3.0);
     double vTaxiReduced  = vTaxi * (2.0/3.0);
-    
+
     // Active runway can be conditionally set by ATC, so at the start of a new flight, this
     // must be reset.
     activeRunway.clear();
 
     if (!(dep->getDynamics()->getGroundController()->exists())) {
         //cerr << "Push Back fallback" << endl;
+        SG_LOG(SG_AI, SG_DEV_WARN, "No groundcontroller createPushBackFallBack at " << dep->getId());
         createPushBackFallBack(ac, firstFlight, dep,
                                radius, fltType, aircraftType, airline);
       return true;
     }
-  
-  // establish the parking position / gate if required
-    if (firstFlight) {
-        // if the airprot has no parking positions defined, don't log
-        // the warning below.
-        if (!dep->getDynamics()->hasParkings()) {
-            return false;
-        }
 
+    if (firstFlight || !dep->getDynamics()->hasParking(gate.parking())) {
+      // establish the parking position / gate
+      // if the airport has no parking positions defined, don't log
+      // the warning below.
+      if (!dep->getDynamics()->hasParkings()) {
+          return false;
+      }
       gate = dep->getDynamics()->getAvailableParking(radius, fltType,
-                                                       aircraftType, airline);
+                                                        aircraftType, airline);
       if (!gate.isValid()) {
         SG_LOG(SG_AI, SG_DEV_WARN, "Could not find parking for a " <<
-               aircraftType <<
-               " of flight type " << fltType <<
-               " of airline     " << airline <<
-               " at airport     " << dep->getId());
+                aircraftType <<
+                " of flight type " << fltType <<
+                " of airline     " << airline <<
+                " at airport     " << dep->getId());
         return false;
       }
     }
-  
+
+
     if (!gate.isValid()) {
+        SG_LOG(SG_AI, SG_DEV_WARN, "Gate " << gate.parking()->ident() << " not valid createPushBackFallBack at " << dep->getId());
         createPushBackFallBack(ac, firstFlight, dep,
                                radius, fltType, aircraftType, airline);
         return true;
 
     }
-  
+
     FGGroundNetwork* groundNet = dep->groundNetwork();
     FGParking *parking = gate.parking();
-    if (parking && parking->getPushBackPoint() > 0) {
+    if (parking && parking->getPushBackPoint() != nullptr) {
         FGTaxiRoute route = groundNet->findShortestRoute(parking, parking->getPushBackPoint(), false);
-      
+        SG_LOG(SG_AI, SG_BULK, "Creating Pushback from " << parking->ident() << " to " << parking->getPushBackPoint()->getIndex());
+
         int size = route.size();
         if (size < 2) {
-            SG_LOG(SG_AI, SG_DEV_WARN, "Push back route from gate " << parking->ident() << " has only " << size << " nodes.");
-            SG_LOG(SG_AI, SG_DEV_WARN, "Using  " << parking->getPushBackPoint());
+            SG_LOG(SG_AI, SG_DEV_WARN, "Push back route from gate " << parking->ident() << " has only " << size << " nodes.\n" << "Using  " << parking->getPushBackPoint());
         }
-        
+
         route.first();
         FGTaxiNodeRef node;
         int rte;
-      
+
+        if (waypoints.size()>0) {
+          // This will be a parking from a previous leg which still contains the forward speed
+          waypoints.back()->setSpeed(vTaxiBackward);
+        }
+
         while (route.next(node, &rte))
         {
-            char buffer[10];
-            snprintf (buffer, 10, "%d",  node->getIndex());
+            char buffer[20];
+            snprintf (buffer, sizeof(buffer), "pushback-%03d",  (short)node->getIndex());
             FGAIWaypoint *wpt = createOnGround(ac, string(buffer), node->geod(), dep->getElevation(), vTaxiBackward);
-            
+
             /*
             if (previous) {
               FGTaxiSegment* segment = groundNet->findSegment(previous, node);
@@ -121,12 +129,13 @@ bool FGAIFlightPlan::createPushBack(FGAIAircraft *ac,
               int x = (int) tn->guid();
               wpt->setRouteIndex(x);
             }*/
-            
-            wpt->setRouteIndex(rte);          
+
+            wpt->setRouteIndex(rte);
             pushBackWaypoint(wpt);
             //previous = node;
         }
         // some special considerations for the last point:
+        // This will trigger the release of parking
         waypoints.back()->setName(string("PushBackPoint"));
         waypoints.back()->setSpeed(vTaxi);
         ac->setTaxiClearanceRequest(true);
@@ -134,34 +143,50 @@ bool FGAIFlightPlan::createPushBack(FGAIAircraft *ac,
         ac->setTaxiClearanceRequest(false);
         double az2 = 0.0;
 
-      FGTaxiSegment* pushForwardSegment = dep->groundNetwork()->findSegment(parking, 0);
-      // there aren't any routes for this parking.
-      if (!pushForwardSegment) {
-          SG_LOG(SG_AI, SG_DEV_WARN, "Gate " << parking->ident() << "/" << parking->getName()
-                 << " at " << dep->getId()
-                 << " doesn't seem to have routes associated with it.");
-          return false;
-      }
+        FGTaxiSegment* pushForwardSegment = dep->groundNetwork()->findSegmentByHeading(parking, parking->getHeading());
 
-      lastNodeVisited = pushForwardSegment->getEnd();
-      double distance = pushForwardSegment->getLength();
+        if (!pushForwardSegment) {
+            // there aren't any routes for this parking, so create a simple segment straight ahead for 2 meters based on the parking heading
+            SG_LOG(SG_AI, SG_DEV_WARN, "Gate " << parking->ident() << " at " << dep->getId()
+                 << " doesn't seem to have pushforward routes associated with it.");
 
-      double parkingHeading = parking->getHeading();
-    
-      for (int i = 1; i < 10; i++) {
-          SGGeod pushForwardPt;
-          SGGeodesy::direct(parking->geod(), parkingHeading,
-                            ((i / 10.0) * distance), pushForwardPt, az2);
-          char buffer[16];
-          snprintf(buffer, 16, "pushback-%02d", i);
-          FGAIWaypoint *wpt = createOnGround(ac, string(buffer), pushForwardPt, dep->getElevation(), vTaxiReduced);
+            FGAIWaypoint *wpt = createOnGround(ac, string("park"), parking->geod(), dep->getElevation(), vTaxiReduced);
+            pushBackWaypoint(wpt);
 
-          wpt->setRouteIndex(pushForwardSegment->getIndex());
-          pushBackWaypoint(wpt);
-      }
+            SGGeod coord;
+            SGGeodesy::direct(parking->geod(), parking->getHeading(), 2.0, coord, az2);
+            wpt = createOnGround(ac, string("taxiStart"), coord, dep->getElevation(), vTaxiReduced);
+            pushBackWaypoint(wpt);
+            return true;
+        }
 
-      waypoints.back()->setName(string("PushBackPoint"));
-      // cerr << "Done assinging new name" << endl;
+        lastNodeVisited = pushForwardSegment->getEnd();
+        double distance = pushForwardSegment->getLength();
+
+        double parkingHeading = parking->getHeading();
+
+        SG_LOG(SG_AI, SG_BULK, "Creating Pushforward from ID " << pushForwardSegment->getEnd()->getIndex() << " Length : \t" << distance);
+// Add the parking if on first leg and not repeat
+        if (waypoints.size() == 0) {
+          pushBackWaypoint( createOnGround(ac, parking->getName(), parking->geod(), dep->getElevation(), vTaxiReduced));
+        }
+// Make sure we have at least three WPs
+        int numSegments = distance>15?(distance/5.0):3;
+        for (int i = 1; i < numSegments; i++) {
+            SGGeod pushForwardPt;
+
+            SGGeodesy::direct(parking->geod(), parkingHeading,
+                              (((double)i / numSegments) * distance), pushForwardPt, az2);
+            char buffer[20];
+            snprintf(buffer, sizeof(buffer), "pushforward-%03d", (short)i);
+            FGAIWaypoint *wpt = createOnGround(ac, string(buffer), pushForwardPt, dep->getElevation(), vTaxiReduced);
+
+            wpt->setRouteIndex(pushForwardSegment->getIndex());
+            pushBackWaypoint(wpt);
+        }
+
+        // This will trigger the release of parking
+        waypoints.back()->setName(string("PushBackPoint-pushforward"));
     }
 
     return true;
@@ -193,7 +218,7 @@ void FGAIFlightPlan::createPushBackFallBack(FGAIAircraft *ac, bool firstFlight,
     wpt = createOnGround(ac, string("park2"), coord, dep->getElevation(), vTaxiBackward);
 
     pushBackWaypoint(wpt);
-  
+
     SGGeodesy::direct(dep->geod(), heading, 2.2 * radius, coord, az2);
     wpt = createOnGround(ac, string("taxiStart"), coord, dep->getElevation(), vTaxiReduced);
     pushBackWaypoint(wpt);
