@@ -18,10 +18,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
 #include <simgear/sg_inlines.h>
 
 #include <Viewer/view.hxx>
@@ -33,29 +29,17 @@
 
 using std::string;
 
-FGAIGroundVehicle::FGAIGroundVehicle() :
-FGAIShip(otGroundVehicle),
-
-_pitch(0),
-_pitch_deg(0),
-_speed_kt(0),
-_range_ft(0),
-_relbrg (0),
-_parent_speed(0),
-_parent_x_offset(0),
-_parent_y_offset(0),
-_parent_z_offset(0),
-_dt_count(0),
-_next_run(0),
-_break_count(0)
-
+FGAIGroundVehicle::FGAIGroundVehicle() : FGAIShip(object_type::otGroundVehicle)
 {
+    _dt_count = 0.0;
+    _next_run = 0.0;
+    _x_offset = 0.0;
+    _y_offset = 0.0;
+
     invisible = false;
     _parent = "";
 }
 
-FGAIGroundVehicle::~FGAIGroundVehicle() {}
-
 void FGAIGroundVehicle::readFromScenario(SGPropertyNode* scFileNode) {
     if (!scFileNode)
         return;
@@ -78,6 +62,7 @@ void FGAIGroundVehicle::readFromScenario(SGPropertyNode* scFileNode) {
     setTowAngleGain(scFileNode->getDoubleValue("tow-angle-gain", 1.0));
     setTowAngleLimit(scFileNode->getDoubleValue("tow-angle-limit-deg", 2.0));
     setInitialTunnel(scFileNode->getBoolValue("tunnel", false));
+
     //we may need these later for towed vehicles
     //    setSubID(scFileNode->getIntValue("SubID", 0));
     //    setGroundOffset(scFileNode->getDoubleValue("ground-offset", 0.0));
@@ -121,8 +106,8 @@ void FGAIGroundVehicle::bind() {
         SGRawValuePointer<double>(&_contact_x2_offset));
 }
 
-bool FGAIGroundVehicle::init(bool search_in_AI_path) {
-    if (!FGAIShip::init(search_in_AI_path))
+bool FGAIGroundVehicle::init(ModelSearchOrder searchOrder) {
+    if (!FGAIShip::init(searchOrder))
         return false;
     reinit();
     return true;
@@ -218,7 +203,6 @@ void FGAIGroundVehicle::setTowAngle(double ta, double dt, double coeff){
 }
 
 bool FGAIGroundVehicle::getPitch() {
-
     if (!_tunnel){
         double vel = props->getDoubleValue("velocities/true-airspeed-kt", 0);
         double contact_offset_x1_m = _contact_x1_offset * SG_FEET_TO_METER;
@@ -237,7 +221,6 @@ bool FGAIGroundVehicle::getPitch() {
         double rear_elev_m = 0;
         double elev_front = 0;
         double elev_rear = 0;
-        //double max_alt = 10000;
 
         if (globals->get_scenery()->get_elevation_m(SGGeod::fromGeodM(geodFront, 3000),
             elev_front, NULL, 0)){
@@ -265,42 +248,25 @@ bool FGAIGroundVehicle::getPitch() {
         }
 
     } else {
-
-        if (prev->getAltitude() == 0 || curr->getAltitude() == 0) return false;
-
-        static double distance;
-        static double d_alt;
-        static double curr_alt;
-        static double prev_alt;
+        if (prev->getAltitude() == 0 || curr->getAltitude() == 0)
+            return false;
 
         if (_new_waypoint){
-            //cout << "new waypoint, calculating pitch " << endl;
-            curr_alt = curr->getAltitude();
-            prev_alt = prev->getAltitude();
-            //cout << "prev_alt" <<prev_alt << endl;
-            d_alt = (curr_alt - prev_alt) * SG_METER_TO_FEET;
-            //_elevation = prev->altitude;
-            distance = SGGeodesy::distanceM(SGGeod::fromDeg(prev->getLongitude(), prev->getLatitude()),
-            SGGeod::fromDeg(curr->getLongitude(), curr->getLatitude()));
-            _pitch = atan2(d_alt, distance * SG_METER_TO_FEET) * SG_RADIANS_TO_DEGREES;
-            //cout << "new waypoint, calculating pitch " <<  _pitch << 
-            //    " " << _pitch_offset << " " << _elevation <<endl;
+            _tunnel_start_alt = prev->getAltitude();
+            _tunnel_end_alt = curr->getAltitude();
+            _tunnel_distance = SGGeodesy::distanceM(SGGeod::fromDeg(prev->getLongitude(), prev->getLatitude()),
+                                                    SGGeod::fromDeg(curr->getLongitude(), curr->getLatitude()));
+            double d_alt = (_tunnel_end_alt - _tunnel_start_alt) * SG_METER_TO_FEET;
+            _pitch = atan2(d_alt, _tunnel_distance * SG_METER_TO_FEET) * SG_RADIANS_TO_DEGREES;
         }
 
         double distance_to_go = SGGeodesy::distanceM(SGGeod::fromDeg(pos.getLongitudeDeg(), pos.getLatitudeDeg()),
-            SGGeod::fromDeg(curr->getLongitude(), curr->getLatitude()));
+                                                     SGGeod::fromDeg(curr->getLongitude(), curr->getLatitude()));
 
-        /*cout << "tunnel " << _tunnel
-             << " distance prev & curr " << prev->name << " " << curr->name << " " << distance * SG_METER_TO_FEET
-             << " distance to go " << distance_to_go * SG_METER_TO_FEET
-             << " d_alt ft " << d_alt
-             << endl;*/
-
-        if (distance_to_go > distance)
-            _elevation = prev_alt;
+        if (distance_to_go > _tunnel_distance)
+            _elevation = _tunnel_start_alt;
         else
-            _elevation = curr_alt - (tan(_pitch * SG_DEGREES_TO_RADIANS) * distance_to_go * SG_METER_TO_FEET);
-
+            _elevation = _tunnel_end_alt - (tan(_pitch * SG_DEGREES_TO_RADIANS) * distance_to_go * SG_METER_TO_FEET);
     }
 
     return true;
@@ -409,7 +375,7 @@ void FGAIGroundVehicle::AdvanceFP(){
         }
 
         prev = curr;
-        fp->DecrementWaypoint(false);
+        fp->DecrementWaypoint();
         curr = fp->getCurrentWaypoint();
         next = fp->getNextWaypoint();
 
@@ -431,14 +397,14 @@ void FGAIGroundVehicle::setTowSpeed(){
 
     if (_range_ft > _x_offset * 3) x = 50;
 
-    if (_relbrg < -90 || _relbrg > 90){
+    if (_relbrg < -90 || _relbrg > 90) {
         setSpeed(_parent_speed - 5 - x);
         //cout << _name << " case 1r _relbrg spd - 5 " << _relbrg << " " << diff << endl;
-    }else if (_range_ft > _x_offset + 0.25 && _relbrg >= -90 && _relbrg <= 90){
+    } else if (_range_ft > _x_offset + 0.25) {
         setSpeed(_parent_speed + 1 + x);
         //cout << _name << " case 2r _relbrg spd + 1 " << _relbrg << " "
         //    << diff << " range " << _range_ft << endl;
-    } else if (_range_ft < _x_offset - 0.25 && _relbrg >= -90 && _relbrg <= 90){
+    } else if (_range_ft < _x_offset - 0.25) {
         setSpeed(_parent_speed - 1 - x);
         //cout << _name << " case 3r _relbrg spd - 2 " << _relbrg << " "
         //    << diff << " " << _range_ft << endl;
@@ -446,7 +412,6 @@ void FGAIGroundVehicle::setTowSpeed(){
         setSpeed(_parent_speed);
         //cout << _name << " else r _relbrg " << _relbrg << " " << diff << endl;
     }
-
 }
 
 void FGAIGroundVehicle::RunGroundVehicle(double dt){
