@@ -27,7 +27,7 @@
 
 #include "AIWingman.hxx"
 
-FGAIWingman::FGAIWingman() : FGAIBallistic(otWingman),
+FGAIWingman::FGAIWingman() : FGAIBallistic(object_type::otWingman),
 _formate_to_ac(true),
 _break(false),
 _join(false),
@@ -44,8 +44,6 @@ _coeff_spd(2.0)
 
 }
 
-FGAIWingman::~FGAIWingman() {}
-
 void FGAIWingman::readFromScenario(SGPropertyNode* scFileNode) {
     if (!scFileNode)
         return;
@@ -136,7 +134,7 @@ void FGAIWingman::bind() {
         (*this, &FGAIBallistic::getElevHitchToUser));
 
     tie("velocities/vertical-speed-fps",
-        SGRawValuePointer<double>(&vs));
+        SGRawValueMethods<FGAIBase,double>(*this, &FGAIBase::_getVS_fps, &FGAIBase::_setVS_fps));
     tie("velocities/true-airspeed-kt",
         SGRawValuePointer<double>(&speed));
     tie("velocities/speed-east-fps",
@@ -158,8 +156,9 @@ void FGAIWingman::bind() {
         SGRawValueMethods<FGAIBallistic,double>(*this, &FGAIBallistic::getTgtZOffset, &FGAIBallistic::setTgtZOffset));
 }
 
-bool FGAIWingman::init(bool search_in_AI_path) {
-    if (!FGAIBallistic::init(search_in_AI_path))
+bool FGAIWingman::init(ModelSearchOrder searchOrder)
+{
+    if (!FGAIBallistic::init(searchOrder))
         return false;
     reinit();
     return true;
@@ -279,16 +278,12 @@ void FGAIWingman::formateToAC(double dt){
 
     if(p_agl <= 10 || p_wow == 1) {
         _height = p_ht;
-        //cout << "ht case1 " ;
-    } else if (p_agl > 10 && p_agl <= 150 ) {
+    } else if (p_agl <= 150 ) {
         setHt(p_ht, dt, 1.0);
-        //cout << "ht case2 " ;
-    } else if (p_agl > 150 && p_agl <= 250) {
-        setHt(_offsetpos.getElevationFt()+ h_feet, dt, 0.75);
-        //cout << "ht case3 " ;
+    } else if (p_agl <= 250) {
+        setHt(_offsetpos.getElevationFt() + h_feet, dt, 0.75);
     } else{
-        setHt(_offsetpos.getElevationFt()+ h_feet, dt, 0.5);
-        //cout << "ht case4 " ;
+        setHt(_offsetpos.getElevationFt() + h_feet, dt, 0.5);
     }
 
     pos.setElevationFt(_height);
@@ -403,7 +398,7 @@ void FGAIWingman::Join(double dt) {
             setPch(angle, dt, _coeff_pch);
             //cout << _name << " backing up HEADING " << hdg
             //    << " RANGE " << range;
-        } else if (rel_brg >= -5 || rel_brg <= 5) {
+        } else if (rel_brg >= -5 && rel_brg <= 5) {
             // station is in front of us - slow down
             setSpeed(parent_spd + ((frm_spd/100) * range));
             //SGMiscd::clip
@@ -474,9 +469,9 @@ void FGAIWingman::Run(double dt) {
 
     // calculate vertical and horizontal speed components
     if (speed == 0.0) {
-        hs = vs = 0.0;
+        hs = vs_fps = 0.0;
     } else {
-        vs = sin( pitch * SG_DEGREES_TO_RADIANS ) * speed_fps;
+        vs_fps = sin( pitch * SG_DEGREES_TO_RADIANS ) * speed_fps;
         hs = cos( pitch * SG_DEGREES_TO_RADIANS ) * speed_fps;
     }
 
@@ -505,8 +500,8 @@ void FGAIWingman::Run(double dt) {
     if (hs <= 0.00001)
         hs = 0;
 
-    if (vs <= 0.00001 && vs >= -0.00001)
-        vs = 0;
+    if (vs_fps <= 0.00001 && vs_fps >= -0.00001)
+        vs_fps = 0;
 
     //cout << "lat " << pos.getLatitudeDeg()<< endl;
     // set new position
@@ -514,18 +509,18 @@ void FGAIWingman::Run(double dt) {
             + (speed_north_deg_sec - wind_speed_from_north_deg_sec) * dt );
     pos.setLongitudeDeg( pos.getLongitudeDeg()
             + (speed_east_deg_sec - wind_speed_from_east_deg_sec ) * dt );
-    pos.setElevationFt(pos.getElevationFt() + vs * dt);
+    pos.setElevationFt(pos.getElevationFt() + vs_fps * dt);
 
     //cout << _name << " run hs " << hs << " vs " << vs << endl;
 
     // recalculate total speed
-    if ( vs == 0 && hs == 0)
+    if ( vs_fps == 0 && hs == 0)
         speed = 0;
     else
-        speed = sqrt( vs * vs + hs * hs) / SG_KT_TO_FPS;
+        speed = sqrt( vs_fps * vs_fps + hs * hs) / SG_KT_TO_FPS;
 
     // recalculate elevation and azimuth (velocity vectors)
-    pitch = atan2( vs, hs ) * SG_RADIANS_TO_DEGREES;
+    pitch = atan2( vs_fps, hs ) * SG_RADIANS_TO_DEGREES;
     hdg   = atan2((speed_east_fps),(speed_north_fps))* SG_RADIANS_TO_DEGREES;
 
     // rationalise heading
