@@ -1,40 +1,24 @@
-// airport.cxx -- Classes representing airports, seaports and helipads
-//
-// Written by Curtis Olson, started April 1998.
-// Updated by Durk Talsma, started December, 2004.
-//
-// Copyright (C) 1998  Curtis L. Olson  - http://www.flightgear.org/~curt
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// $Id$
+/*
+ * SPDX-FileName: airport.cxx
+ * SPDX-FileComment: Classes representing airports, seaports and helipads
+ * SPDX-FileCopyrightText: Copyright (C) 1998  Curtis L. Olson  - http://www.flightgear.org/~curt
+ * SPDX-FileContributor: Updated by Durk Talsma, started December, 2004
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #include "airport.hxx"
 
 #include <algorithm>
 #include <cassert>
-#include <boost/foreach.hpp>
+#include <utility>
 
+#include <simgear/debug/ErrorReportingCallback.hxx>
+#include <simgear/debug/logstream.hxx>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/props/props_io.hxx>
-#include <simgear/debug/logstream.hxx>
 #include <simgear/sg_inlines.h>
 #include <simgear/structure/exception.hxx>
 
@@ -71,10 +55,12 @@ FGAirport::FGAirport( PositionedID aGuid,
                       const SGGeod& location,
                       const std::string &name,
                       bool has_metar,
-                      Type aType ):
+                      Type aType,
+                      SGPath sceneryPath):
     FGPositioned(aGuid, aType, id, location),
     _name(name),
     _has_metar(has_metar),
+    _sceneryPath(std::move(sceneryPath)),
     mTowerDataLoaded(false),
     mHasTower(false),
     mRunwaysLoaded(false),
@@ -107,6 +93,11 @@ bool FGAirport::isHeliport() const
   return type() == HELIPORT;
 }
 
+SGPath FGAirport::sceneryPath() const
+{
+  return _sceneryPath;
+}
+
 //------------------------------------------------------------------------------
 unsigned int FGAirport::numRunways() const
 {
@@ -143,10 +134,9 @@ FGRunwayMap FGAirport::getRunwayMap() const
 
   double minLengthFt = fgGetDouble("/sim/navdb/min-runway-length-ft");
 
-  BOOST_FOREACH(FGRunwayRef rwy, mRunways)
-  {
+  for (auto rwy : mRunways) {
     // ignore unusably short runways
-    // TODO other methods don't check this...
+    // TODO: other methods don't check this...
     if( rwy->lengthFt() >= minLengthFt )
       map[ rwy->ident() ] = rwy;
   }
@@ -160,8 +150,7 @@ FGHelipadMap FGAirport::getHelipadMap() const
   loadHelipads();
   FGHelipadMap map;
 
-  BOOST_FOREACH(PositionedID id, mHelipads)
-  {
+  for (auto id : mHelipads) {
     FGHelipad* rwy = loadById<FGHelipad>(id);
     map[ rwy->ident() ] = rwy;
   }
@@ -173,7 +162,7 @@ FGHelipadMap FGAirport::getHelipadMap() const
 bool FGAirport::hasRunwayWithIdent(const std::string& aIdent) const
 {
   loadRunways();
-  BOOST_FOREACH(FGRunwayRef rwy, mRunways) {
+  for (auto rwy : mRunways) {
     if (rwy->ident() == aIdent) {
       return true;
     }
@@ -192,13 +181,16 @@ bool FGAirport::hasHelipadWithIdent(const std::string& aIdent) const
 //------------------------------------------------------------------------------
 FGRunwayRef FGAirport::getRunwayByIdent(const std::string& aIdent) const
 {
-  loadRunways();
-  BOOST_FOREACH(FGRunwayRef rwy, mRunways) {
-    if (rwy->ident() == aIdent) {
-      return rwy;
-    }
+    if (aIdent.empty())
+        return {};
+
+    loadRunways();
+    for (auto rwy : mRunways) {
+        if (rwy->ident() == aIdent) {
+            return rwy;
+        }
   }
-  
+
   SG_LOG(SG_GENERAL, SG_ALERT, "no such runway '" << aIdent << "' at airport " << ident());
   throw sg_range_exception("unknown runway " + aIdent + " at airport:" + ident(), "FGAirport::getRunwayByIdent");
 }
@@ -219,10 +211,10 @@ FGHelipadRef FGAirport::getHelipadByIdent(const std::string& aIdent) const
 FGRunwayRef FGAirport::findBestRunwayForHeading(double aHeading, struct FindBestRunwayForHeadingParams * parms ) const
 {
   loadRunways();
-  
+
   FGRunway* result = NULL;
   double currentBestQuality = 0.0;
-  
+
   struct FindBestRunwayForHeadingParams fbrfhp;
   if( NULL != parms ) fbrfhp = *parms;
 
@@ -234,14 +226,14 @@ FGRunwayRef FGAirport::findBestRunwayForHeading(double aHeading, struct FindBest
     fbrfhp.deviationWeight = searchNode->getDoubleValue("deviation-weight", fbrfhp.deviationWeight );
     fbrfhp.ilsWeight = searchNode->getDoubleValue("ils-weight", fbrfhp.ilsWeight );
   }
-    
-  BOOST_FOREACH(FGRunwayRef rwy, mRunways) {
+
+  for (auto rwy : mRunways) {
     double good = rwy->score( fbrfhp.lengthWeight,  fbrfhp.widthWeight,  fbrfhp.surfaceWeight,  fbrfhp.ilsWeight );
     double dev = aHeading - rwy->headingDeg();
     SG_NORMALIZE_RANGE(dev, -180.0, 180.0);
     double bad = fabs( fbrfhp.deviationWeight * dev) + 1e-20;
     double quality = good / bad;
-    
+
     if (quality > currentBestQuality) {
       currentBestQuality = quality;
       result = rwy;
@@ -255,11 +247,11 @@ FGRunwayRef FGAirport::findBestRunwayForHeading(double aHeading, struct FindBest
 FGRunwayRef FGAirport::findBestRunwayForPos(const SGGeod& aPos) const
 {
   loadRunways();
-  
+
   FGRunway* result = NULL;
   double currentLowestDev = 180.0;
-  
-  BOOST_FOREACH(FGRunwayRef rwy, mRunways) {
+
+  for (auto rwy : mRunways) {
     double inboundCourse = SGGeodesy::courseDeg(aPos, rwy->end());
     double dev = inboundCourse - rwy->headingDeg();
     SG_NORMALIZE_RANGE(dev, -180.0, 180.0);
@@ -270,7 +262,7 @@ FGRunwayRef FGAirport::findBestRunwayForPos(const SGGeod& aPos) const
       result = rwy;
     }
   } // of runway iteration
-  
+
   return result;
 
 }
@@ -279,8 +271,8 @@ FGRunwayRef FGAirport::findBestRunwayForPos(const SGGeod& aPos) const
 bool FGAirport::hasHardRunwayOfLengthFt(double aLengthFt) const
 {
   loadRunways();
-  
-  BOOST_FOREACH(FGRunwayRef rwy, mRunways) {
+
+  for (auto rwy : mRunways) {
     if (rwy->isHardSurface() && (rwy->lengthFt() >= aLengthFt)) {
       return true; // we're done!
     }
@@ -294,7 +286,7 @@ FGRunwayRef FGAirport::longestRunway() const
     FGRunwayRef r;
     loadRunways();
 
-    BOOST_FOREACH(FGRunwayRef rwy, mRunways) {
+    for (auto rwy : mRunways) {
         if (!r || (r->lengthFt() < rwy->lengthFt())) {
              r = rwy;
         }
@@ -315,10 +307,10 @@ FGRunwayList FGAirport::getRunways() const
 FGRunwayList FGAirport::getRunwaysWithoutReciprocals() const
 {
   loadRunways();
-  
+
   FGRunwayList r;
-  
-  BOOST_FOREACH(FGRunwayRef rwy, mRunways) {
+
+  for (auto rwy : mRunways) {
     FGRunway* recip = rwy->reciprocalRunway();
     if (recip) {
       FGRunwayList::iterator it = std::find(r.begin(), r.end(), recip);
@@ -326,10 +318,10 @@ FGRunwayList FGAirport::getRunwaysWithoutReciprocals() const
         continue; // reciprocal already in result set, don't include us
       }
     }
-    
+
     r.push_back(rwy);
   }
-  
+
   return r;
 }
 
@@ -357,42 +349,74 @@ FGTaxiwayList FGAirport::getTaxiways() const
 //------------------------------------------------------------------------------
 unsigned int FGAirport::numPavements() const
 {
-  loadTaxiways();
   return mPavements.size();
 }
 
-//------------------------------------------------------------------------------
-FGPavementRef FGAirport::getPavementByIndex(unsigned int aIndex) const
-{
-  loadTaxiways();
-  return loadById<FGPavement>(mPavements, aIndex);
-}
-
 //------------------------------------------------------------------------------
 FGPavementList FGAirport::getPavements() const
 {
-  loadTaxiways();
-  return loadAllById<FGPavement>(mPavements);
+  return mPavements;
+}
+
+void FGAirport::addPavement(FGPavementRef pavement)
+{
+  mPavements.push_back(pavement);
+}
+
+//------------------------------------------------------------------------------
+unsigned int FGAirport::numBoundary() const
+{
+  return mBoundary.size();
+}
+
+//------------------------------------------------------------------------------
+FGPavementList FGAirport::getBoundary() const
+{
+  return mBoundary;
+}
+
+void FGAirport::addBoundary(FGPavementRef boundary)
+{
+  mBoundary.push_back(boundary);
+}
+
+//------------------------------------------------------------------------------
+unsigned int FGAirport::numLineFeatures() const
+{
+  return mLineFeatures.size();
+}
+
+//------------------------------------------------------------------------------
+FGPavementList FGAirport::getLineFeatures() const
+{
+  return mLineFeatures;
+}
+
+void FGAirport::addLineFeature(FGPavementRef linefeature)
+{
+  mLineFeatures.push_back(linefeature);
 }
 
 //------------------------------------------------------------------------------
 FGRunwayRef FGAirport::getActiveRunwayForUsage() const
 {
-  FGEnvironmentMgr* envMgr = (FGEnvironmentMgr *) globals->get_subsystem("environment");
+  auto envMgr = globals->get_subsystem<FGEnvironmentMgr>();
   
   // This forces West-facing rwys to be used in no-wind situations
   // which is consistent with Flightgear's initial setup.
   double hdg = 270;
-  
+
   if (envMgr) {
-    FGEnvironment stationWeather(envMgr->getEnvironment(geod()));
-  
-    double windSpeed = stationWeather.get_wind_speed_kt();
+      // FIXME : this should use the weather at the airport, not the player's
+      // location.
+      const auto stationWeather = envMgr->getAircraftEnvironment();
+
+    double windSpeed = stationWeather->get_wind_speed_kt();
     if (windSpeed > 0.0) {
-      hdg = stationWeather.get_wind_from_heading_deg();
+      hdg = stationWeather->get_wind_from_heading_deg();
     }
   }
-  
+
   return findBestRunwayForHeading(hdg);
 }
 
@@ -404,7 +428,7 @@ FGAirportRef FGAirport::findClosest( const SGGeod& aPos,
   AirportFilter aptFilter;
   if( !filter )
     filter = &aptFilter;
-  
+
   return static_pointer_cast<FGAirport>
   (
     FGPositioned::findClosest(aPos, aCuttofNm, filter)
@@ -455,6 +479,11 @@ bool FGAirport::TypeRunwayFilter::pass(FGPositioned* pos) const
   return true;
 }
 
+void FGAirport::clearAirportsCache()
+{
+    airportCache.clear();
+}
+
 //------------------------------------------------------------------------------
 FGAirportRef FGAirport::findByIdent(const std::string& aIdent)
 {
@@ -495,7 +524,7 @@ const FGAirport *fgFindAirportID( const std::string& id)
     if ( id.empty() ) {
         return NULL;
     }
-    
+
     return FGAirport::findByIdent(id);
 }
 
@@ -510,12 +539,12 @@ void FGAirport::loadRunways() const
   if (mRunwaysLoaded) {
     return; // already loaded, great
   }
-  
+
   loadSceneryDefinitions();
-  
+
   mRunwaysLoaded = true;
   PositionedIDVec rwys(itemsOfType(FGPositioned::RUNWAY));
-  BOOST_FOREACH(PositionedID id, rwys) {
+  for (auto id : rwys) {
     mRunways.push_back(loadById<FGRunway>(id));
   }
 }
@@ -535,7 +564,7 @@ void FGAirport::loadTaxiways() const
   if (mTaxiwaysLoaded) {
     return; // already loaded, great
   }
-  
+
   mTaxiwaysLoaded =  true;
   mTaxiways = itemsOfType(FGPositioned::TAXIWAY);
 }
@@ -545,37 +574,62 @@ void FGAirport::loadProcedures() const
   if (mProceduresLoaded) {
     return;
   }
-  
+
   mProceduresLoaded = true;
   SGPath path;
   if (!XMLLoader::findAirportData(ident(), "procedures", path)) {
     SG_LOG(SG_GENERAL, SG_INFO, "no procedures data available for " << ident());
     return;
   }
-  
+
   SG_LOG(SG_GENERAL, SG_INFO, ident() << ": loading procedures from " << path);
   RouteBase::loadAirportProcedures(path, const_cast<FGAirport*>(this));
 }
 
+void FGAirport::loadRunwayRenames() const
+{
+    if (mRunwayRenamesLoaded) {
+        return;
+    }
+
+    SGPath path;
+    if (!XMLLoader::findAirportData(ident(), "runway_rename", path)) {
+        // No rename for airport; ignore
+        mRunwayRenamesLoaded = true;
+        return;
+    }
+
+    try {
+        SGPropertyNode_ptr rootNode = new SGPropertyNode;
+        readProperties(path, rootNode);
+        const_cast<FGAirport*>(this)->parseRunwayRenameData(rootNode);
+        mRunwayRenamesLoaded = true;
+    } catch (sg_exception& e) {
+        SG_LOG(SG_NAVAID, SG_WARN, ident() << "loading runway rename XML failed:" << e.getFormattedMessage());
+    }
+}
+
 void FGAirport::loadSceneryDefinitions() const
 {
   if (mThresholdDataLoaded) {
     return;
   }
-  
+
   mThresholdDataLoaded = true;
-  
+
   SGPath path;
   if (!XMLLoader::findAirportData(ident(), "threshold", path)) {
     return; // no XML threshold data
   }
-  
+
   try {
     SGPropertyNode_ptr rootNode = new SGPropertyNode;
     readProperties(path, rootNode);
     const_cast<FGAirport*>(this)->readThresholdData(rootNode);
   } catch (sg_exception& e) {
-    SG_LOG(SG_NAVAID, SG_WARN, ident() << "loading threshold XML failed:" << e.getFormattedMessage());
+    simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::TerraSync,
+                           "Airport threshold data could not be loaded:" + e.getFormattedMessage(),
+                           path);
   }
 }
 
@@ -586,9 +640,11 @@ void FGAirport::readThresholdData(SGPropertyNode* aRoot)
   for (; (runway = aRoot->getChild("runway", runwayIndex)) != NULL; ++runwayIndex) {
     SGPropertyNode* t0 = runway->getChild("threshold", 0),
       *t1 = runway->getChild("threshold", 1);
-    assert(t0);
-    assert(t1); // too strict? maybe we should finally allow single-ended runways
-    
+    if (!t0 || !t1) {
+      throw sg_io_exception("Mis-configured runway threshold data: exactly two thresholds must be defined",
+                            sg_location{runway}, "FGAirport::readThresholdData", false);
+    }
+
     processThreshold(t0);
     processThreshold(t1);
   } // of runways iteration
@@ -598,23 +654,25 @@ void FGAirport::processThreshold(SGPropertyNode* aThreshold)
 {
   // first, let's identify the current runway
   std::string rwyIdent(aThreshold->getStringValue("rwy"));
-  NavDataCache* cache = NavDataCache::instance(); 
+  NavDataCache* cache = NavDataCache::instance();
   PositionedID id = cache->airportItemWithIdent(guid(), FGPositioned::RUNWAY, rwyIdent);
-  
+
   double lon = aThreshold->getDoubleValue("lon"),
   lat = aThreshold->getDoubleValue("lat");
   SGGeod newThreshold(SGGeod::fromDegM(lon, lat, elevationM()));
-  
+
   double newHeading = aThreshold->getDoubleValue("hdg-deg");
   double newDisplacedThreshold = aThreshold->getDoubleValue("displ-m");
   double newStopway = aThreshold->getDoubleValue("stopw-m");
-  
+
   if (id == 0) {
-    SG_LOG(SG_GENERAL, SG_DEBUG, "FGAirport::processThreshold: "
-           "found runway not defined in the global data:" << ident() << "/" << rwyIdent);
+    const auto runwayIdent = ident() + "/" + rwyIdent;
     // enable this code when threshold.xml contains sufficient data to
     // fully specify a new runway, *and* we figure out how to assign runtime
     // Positioned IDs and insert temporary items into the spatial map.
+    throw sg_io_exception("Found runway not defined in global data:" + runwayIdent,
+                          sg_location{aThreshold}, "FGAirport::processThreshold", false);
+
 #if 0
     double newLength = 0.0, newWidth = 0.0;
     int surfaceCode = 0;
@@ -645,7 +703,7 @@ void FGAirport::validateTowerData() const
   if (mTowerDataLoaded) {
     return;
   }
-  
+
   mTowerDataLoaded = true;
 
 // first, load data from the cache (apt.dat)
@@ -672,12 +730,12 @@ void FGAirport::validateTowerData() const
     mTowerPosition = tower->geod();
     mHasTower = true;
   }
-  
+
   SGPath path;
   if (!XMLLoader::findAirportData(ident(), "twr", path)) {
     return; // no XML tower data, base position is fine
   }
-  
+
   try {
     SGPropertyNode_ptr rootNode = new SGPropertyNode;
     readProperties(path, rootNode);
@@ -691,9 +749,9 @@ void FGAirport::validateTowerData() const
 void FGAirport::readTowerData(SGPropertyNode* aRoot)
 {
   SGPropertyNode* twrNode = aRoot->getChild("tower")->getChild("twr");
-  double lat = twrNode->getDoubleValue("lat"), 
-    lon = twrNode->getDoubleValue("lon"), 
-    elevM = twrNode->getDoubleValue("elev-m");  
+  double lat = twrNode->getDoubleValue("lat"),
+    lon = twrNode->getDoubleValue("lon"),
+    elevM = twrNode->getDoubleValue("elev-m");
 // tower elevation is AGL, not AMSL. Since we don't want to depend on the
 // scenery for a precise terrain elevation, we use the field elevation
 // (this is also what the apt.dat code does)
@@ -701,21 +759,53 @@ void FGAirport::readTowerData(SGPropertyNode* aRoot)
   mTowerPosition = SGGeod::fromDegM(lon, lat, fieldElevationM + elevM);
 }
 
+void FGAirport::parseRunwayRenameData(SGPropertyNode* aRoot)
+{
+    SGPropertyNode* overrideNode = aRoot->getChild("runway-rename");
+    for (auto rnm : overrideNode->getChildren("runway")) {
+        const std::string oldIdent = rnm->getStringValue("old-ident");
+        const std::string newIdent = rnm->getStringValue("new-ident");
+        if (oldIdent.empty() || newIdent.empty()) {
+            SG_LOG(SG_NAVAID, SG_WARN, ident() << ": runway rename: Skipping bad runway rename entry");
+            continue;
+        }
+
+        if (!hasRunwayWithIdent(oldIdent)) {
+            SG_LOG(SG_NAVAID, SG_WARN, ident() << ": no old runway with ident:" << oldIdent);
+            continue;
+        }
+
+        _renamedRunways[newIdent] = oldIdent;
+    }
+}
+
+std::string FGAirport::findAPTRunwayForNewName(const std::string& newIdent) const
+{
+    loadRunwayRenames();
+
+    auto it = _renamedRunways.find(newIdent);
+    if (it == _renamedRunways.end())
+        return {};
+
+    return it->second;
+}
+
+
 void FGAirport::validateILSData()
 {
   if (mILSDataLoaded) {
     return;
   }
-  
+
   // to avoid re-entrancy on this code-path, ensure we set loaded
   // immediately.
   mILSDataLoaded = true;
-    
+
   SGPath path;
   if (!XMLLoader::findAirportData(ident(), "ils", path)) {
     return; // no XML ILS data
   }
-  
+
   try {
       SGPropertyNode_ptr rootNode = new SGPropertyNode;
       readProperties(path, rootNode);
@@ -732,7 +822,7 @@ bool FGAirport::hasTower() const
 }
 
 void FGAirport::readILSData(SGPropertyNode* aRoot)
-{  
+{
   NavDataCache* cache = NavDataCache::instance();
   // find the entry matching the runway
   SGPropertyNode* runwayNode, *ilsNode;
@@ -750,12 +840,12 @@ void FGAirport::readILSData(SGPropertyNode* aRoot)
                ilsNode->getStringValue("nav-id"));
         continue;
       }
-      
+
       double hdgDeg = ilsNode->getDoubleValue("hdg-deg"),
         lon = ilsNode->getDoubleValue("lon"),
         lat = ilsNode->getDoubleValue("lat"),
         elevM = ilsNode->getDoubleValue("elev-m");
- 
+
       FGNavRecordRef nav(FGPositioned::loadById<FGNavRecord>(ils));
       assert(nav.valid());
       nav->updateFromXML(SGGeod::fromDegM(lon, lat, elevM), hdgDeg);
@@ -801,7 +891,7 @@ flightgear::SID* FGAirport::findSIDWithIdent(const std::string& aIdent) const
       return mSIDs[i];
     }
   }
-  
+
   return NULL;
 }
 
@@ -835,7 +925,7 @@ STAR* FGAirport::findSTARWithIdent(const std::string& aIdent) const
       return mSTARs[i];
     }
   }
-  
+
   return NULL;
 }
 
@@ -868,7 +958,7 @@ Approach* FGAirport::findApproachWithIdent(const std::string& aIdent) const
       return mApproaches[i];
     }
   }
-  
+
   return NULL;
 }
 
@@ -893,13 +983,12 @@ FGAirport::commStations() const
 {
   NavDataCache* cache = NavDataCache::instance();
   CommStationList result;
-  BOOST_FOREACH(PositionedID pos, cache->airportItemsOfType(guid(),
-                                                            FGPositioned::FREQ_GROUND,
-                                                            FGPositioned::FREQ_UNICOM))
-  {
+  for (auto pos : cache->airportItemsOfType(guid(),
+                                            FGPositioned::FREQ_GROUND,
+                                            FGPositioned::FREQ_UNICOM)) {
     result.push_back( loadById<CommStation>(pos) );
   }
-  
+
   return result;
 }
 
@@ -908,48 +997,48 @@ FGAirport::commStationsOfType(FGPositioned::Type aTy) const
 {
   NavDataCache* cache = NavDataCache::instance();
   CommStationList result;
-  BOOST_FOREACH(PositionedID pos, cache->airportItemsOfType(guid(), aTy)) {
+  for (auto pos : cache->airportItemsOfType(guid(), aTy)) {
     result.push_back( loadById<CommStation>(pos) );
   }
-  
+
   return result;
 }
 
 class AirportWithSize
 {
 public:
-    AirportWithSize(FGPositionedRef pos) :
+    explicit AirportWithSize(FGPositionedRef pos) :
         _pos(pos),
         _sizeMetric(0)
     {
         assert(pos->type() == FGPositioned::AIRPORT);
         FGAirport* apt = static_cast<FGAirport*>(pos.get());
-        BOOST_FOREACH(FGRunway* rwy, apt->getRunwaysWithoutReciprocals()) {
+        for (auto rwy : apt->getRunwaysWithoutReciprocals()) {
             _sizeMetric += static_cast<int>(rwy->lengthFt());
         }
     }
-    
+
     bool operator<(const AirportWithSize& other) const
     {
         return _sizeMetric < other._sizeMetric;
     }
-    
+
     FGPositionedRef pos() const
     { return _pos; }
 private:
     FGPositionedRef _pos;
     unsigned int _sizeMetric;
-    
+
 };
 
 void FGAirport::sortBySize(FGPositionedList& airportList)
 {
     std::vector<AirportWithSize> annotated;
-    BOOST_FOREACH(FGPositionedRef p, airportList) {
+    for (auto p : airportList) {
         annotated.push_back(AirportWithSize(p));
     }
     std::sort(annotated.begin(), annotated.end());
-    
+
     for (unsigned int i=0; i<annotated.size(); ++i) {
         airportList[i] = annotated[i].pos();
     }
@@ -964,15 +1053,67 @@ FGGroundNetwork *FGAirport::groundNetwork() const
 {
     if (!_groundNetwork.get()) {
         _groundNetwork.reset(new FGGroundNetwork(const_cast<FGAirport*>(this)));
-
         XMLLoader::load(_groundNetwork.get());
-
         _groundNetwork->init();
     }
 
     return _groundNetwork.get();
 }
 
+flightgear::Transition* FGAirport::selectSIDByEnrouteTransition(FGPositioned* enroute) const
+{
+    loadProcedures();
+    for (auto sid : mSIDs) {
+        auto trans = sid->findTransitionByEnroute(enroute);
+        if (trans) {
+            return trans;
+        }
+    }
+    return nullptr;
+}
+
+Transition *FGAirport::selectSIDByTransition(const FGRunway* runway,  const std::string &aIdent) const
+{
+    loadProcedures();
+    for (auto sid : mSIDs) {
+        if (runway && !sid->isForRunway(runway))
+            continue;
+
+        auto trans = sid->findTransitionByName(aIdent);
+        if (trans) {
+            return trans;
+        }
+    }
+    return nullptr;
+}
+
+flightgear::Transition* FGAirport::selectSTARByEnrouteTransition(FGPositioned* enroute) const
+{
+    loadProcedures();
+    for (auto star : mSTARs) {
+        auto trans = star->findTransitionByEnroute(enroute);
+        if (trans) {
+            return trans;
+        }
+    }
+    return nullptr;
+}
+
+Transition *FGAirport::selectSTARByTransition(const FGRunway* runway, const std::string &aIdent) const
+{
+    loadProcedures();
+    for (auto star : mSTARs) {
+        if (runway && !star->isForRunway(runway))
+            continue;
+
+        auto trans = star->findTransitionByName(aIdent);
+        if (trans) {
+            return trans;
+        }
+    }
+    return nullptr;
+}
+
 // get airport elevation
 double fgGetAirportElev( const std::string& id )
 {
