@@ -38,22 +38,17 @@ using std::string;
  ****************************************************************************/
 static double processPosition(const string &pos)
 {
-  string prefix;
-  string subs;
-  string degree;
-  string decimal;
-  int sign = 1;
-  double value;
-  subs = pos;
-  prefix= subs.substr(0,1);
-  if (prefix == string("S") || (prefix == string("W")))
-    sign = -1;
-  subs    = subs.substr(1, subs.length());
-  degree  = subs.substr(0, subs.find(" ",0));
-  decimal = subs.substr(subs.find(" ",0), subs.length());
+    string subs{pos};
+    int sign = 1;
+    const auto prefix = subs.substr(0, 1);
+    if (prefix == "S" || (prefix == "W"))
+        sign = -1;
+    subs = subs.substr(1, subs.length()); // drop first character
+    const auto spacePos = subs.find(" ", 0);
+    const auto degree = subs.substr(0, spacePos);
+    const auto decimal = subs.substr(spacePos, subs.length());
 
-  value = sign * (atof(degree.c_str()) + atof(decimal.c_str())/60.0);
-  return value;
+    return sign * (stoi(degree) + (stof(decimal) / 60.0));
 }
 
 FGGroundNetXMLLoader::FGGroundNetXMLLoader(FGGroundNetwork* net):
@@ -72,14 +67,19 @@ void  FGGroundNetXMLLoader::endXML ()
   for (it = _parkingPushbacks.begin(); it != _parkingPushbacks.end(); ++it) {
     NodeIndexMap::const_iterator j = _indexMap.find(it->second);
     if (j == _indexMap.end()) {
-      SG_LOG(SG_NAVAID, SG_DEV_WARN, "bad groundnet, no node for index:" << it->first);
-      continue;
+        _hasErrors = true;
+        SG_LOG(SG_NAVAID, SG_DEV_WARN,  "bad groundnet " << _groundNetwork->airport()->getId() << " , no node for index:" << it->first.get()->getIndex());
+        continue;
     }
 
     it->first->setPushBackPoint(j->second);
 
   }
 
+  if (!_unreferencedNodes.empty()) {
+      _hasErrors = true;
+  }
+
   for (const FGTaxiNodeRef& node: _unreferencedNodes) {
     SG_LOG(SG_NAVAID, SG_DEV_WARN,
            "unreferenced groundnet node: " << node->getIndex());
@@ -128,11 +128,16 @@ void FGGroundNetXMLLoader::startParking(const XMLAttributes &atts)
     else if (attname == "pushBackRoute") {
       const string attrVal = atts.getValue(i);
       try {
-        pushBackRoute = simgear::strutils::readNonNegativeInt<int>(attrVal);
+          if (attrVal.empty() || (attrVal == "None")) {
+              pushBackRoute = -2;
+          } else {
+              pushBackRoute = simgear::strutils::readNonNegativeInt<int>(attrVal);
+          }
       } catch (const sg_exception& e) {
         SG_LOG(SG_NAVAID, SG_DEV_WARN,
                getPath() << ":" << getLine() << ": " <<
                "invalid value for 'pushBackRoute': " << e.what());
+        _hasErrors = true;
         pushBackRoute = -2;
       }
     }
@@ -188,10 +193,11 @@ void FGGroundNetXMLLoader::startNode(const XMLAttributes &atts)
 
   if (_indexMap.find(index) != _indexMap.end()) {
     SG_LOG(SG_NAVAID, SG_DEV_WARN, "duplicate ground-net index:" << index);
+    _hasErrors = true;
   }
 
   SGGeod pos(SGGeod::fromDeg(processPosition(lon), processPosition(lat)));
-  FGTaxiNodeRef node(new FGTaxiNode(index, pos, onRunway, holdPointType));
+  FGTaxiNodeRef node(new FGTaxiNode(FGPositioned::TAXI_NODE, index, pos, onRunway, holdPointType));
   _indexMap[index] = node;
   _unreferencedNodes.insert(node);
 }
@@ -215,6 +221,7 @@ void FGGroundNetXMLLoader::startArc(const XMLAttributes &atts)
   IntPair e(begin, end);
   if (_arcSet.find(e) != _arcSet.end()) {
     SG_LOG(SG_NAVAID, SG_DEV_WARN, _groundNetwork->airport()->ident() << " ground-net: skipping duplicate edge:" << begin << "->" << end);
+    _hasErrors = true;
     return;
   }
 
@@ -223,6 +230,7 @@ void FGGroundNetXMLLoader::startArc(const XMLAttributes &atts)
   it = _indexMap.find(begin);
   if (it == _indexMap.end()) {
       SG_LOG(SG_NAVAID, SG_DEV_WARN, "ground-net: bad edge:" << begin << "->" << end << ", begin index unknown");
+      _hasErrors = true;
       return;
   } else {
       _unreferencedNodes.erase(it->second);
@@ -232,6 +240,7 @@ void FGGroundNetXMLLoader::startArc(const XMLAttributes &atts)
   it = _indexMap.find(end);
   if (it == _indexMap.end()) {
       SG_LOG(SG_NAVAID, SG_DEV_WARN, "ground-net: bad edge:" << begin << "->" << end << ", end index unknown");
+      _hasErrors = true;
       return;
   } else {
       _unreferencedNodes.erase(it->second);
@@ -290,9 +299,9 @@ void  FGGroundNetXMLLoader::pi (const char * target, const char * data) {
 }
 
 void  FGGroundNetXMLLoader::warning (const char * message, int line, int column) {
-  SG_LOG(SG_IO, SG_WARN, "Warning: " << message << " (" << line << ',' << column << ')');
+    SG_LOG(SG_IO, SG_DEV_WARN, "Warning: " << message << " (" << line << ',' << column << ')');
 }
 
 void  FGGroundNetXMLLoader::error (const char * message, int line, int column) {
-  SG_LOG(SG_IO, SG_ALERT, "Error: " << message << " (" << line << ',' << column << ')');
+    SG_LOG(SG_IO, SG_DEV_ALERT, "Error: " << message << " (" << line << ',' << column << ')');
 }
