@@ -1,25 +1,10 @@
-// autopilotgroup.cxx - an even more flexible, generic way to build autopilots
-//
-// Written by Torsten Dreyer
-// Based heavily on work created by Curtis Olson, started January 2004.
-//
-// Copyright (C) 2004  Curtis L. Olson  - http://www.flightgear.org/~curt
-// Copyright (C) 2010  Torsten Dreyer - Torsten (at) t3r (dot) de
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
+/*
+ * SPDX-FileName: autopilotgroup.cxx
+ * SPDX-FileComment: an even more flexible, generic way to build autopilots
+ * SPDX-FileCopyrightText: Copyright (C) 2010  Torsten Dreyer - Torsten (at) t3r (dot) de
+ * SPDX-FileContributor: Copyright (C) 2004  Curtis L. Olson  - http://www.flightgear.org/~curt
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -31,37 +16,42 @@
 #include <string>
 #include <vector>
 
+#include <simgear/debug/ErrorReportingCallback.hxx>
 #include <simgear/props/props_io.hxx>
-#include <simgear/structure/subsystem_mgr.hxx>
 #include <simgear/structure/exception.hxx>
+#include <simgear/structure/subsystem_mgr.hxx>
+
 #include <Main/fg_props.hxx>
-#include <boost/foreach.hpp>
+#include <Main/sentryIntegration.hxx>
 
 using std::vector;
 using simgear::PropertyList;
 using FGXMLAutopilot::Autopilot;
 
-class FGXMLAutopilotGroupImplementation:
-  public FGXMLAutopilotGroup
+class FGXMLAutopilotGroupImplementation : public FGXMLAutopilotGroup
 {
-  public:
+public:
     FGXMLAutopilotGroupImplementation(const std::string& nodeName):
-      FGXMLAutopilotGroup(),
-      _nodeName(nodeName)
+        FGXMLAutopilotGroup(),
+        _nodeName(nodeName)
     {}
 
+    // Subsystem API.
+    void init() override;
+    InitStatus incrementalInit() override;
+    void reinit() override;
+
+    // Subsystem identification.
+    static const char* staticSubsystemClassId() { return "xml-autopilot-group"; }
+
     virtual void addAutopilot( const std::string& name,
                                SGPropertyNode_ptr apNode,
                                SGPropertyNode_ptr config );
     virtual void removeAutopilot( const std::string & name );
-    void init();
-    InitStatus incrementalInit();
-    void reinit();
 
-  private:
+private:
     void initFrom( SGPropertyNode_ptr rootNode, const char * childName );
     std::string _nodeName;
-
 };
 
 //------------------------------------------------------------------------------
@@ -104,7 +94,12 @@ void FGXMLAutopilotGroupImplementation::reinit()
 {
   SGSubsystemGroup::unbind();
   clearSubsystems();
-
+  
+  // ensure we bind again, so the SGSubsystemGroup state is correct before
+  // we call init. Since there's no actual group members at this point (we
+  // cleared them just above) this is purely to ensure SGSubsystemGroup::_state
+  // is BIND, so that ::init doesn't assert
+  SGSubsystemGroup::bind();
   init();
 }
 
@@ -119,8 +114,6 @@ SGSubsystem::InitStatus FGXMLAutopilotGroupImplementation::incrementalInit()
 void FGXMLAutopilotGroupImplementation::init()
 {
   initFrom(fgGetNode("/sim/systems"), _nodeName.c_str());
-
-  SGSubsystemGroup::bind();
   SGSubsystemGroup::init();
 }
 
@@ -131,8 +124,7 @@ void FGXMLAutopilotGroupImplementation::initFrom( SGPropertyNode_ptr rootNode,
   if( !rootNode )
     return;
 
-  BOOST_FOREACH( SGPropertyNode_ptr autopilotNode,
-                 rootNode->getChildren(childName) )
+  for( auto autopilotNode : rootNode->getChildren(childName) )
   {
     SGPropertyNode_ptr pathNode = autopilotNode->getNode("path");
     if( !pathNode )
@@ -168,7 +160,7 @@ void FGXMLAutopilotGroupImplementation::initFrom( SGPropertyNode_ptr rootNode,
         SG_LOG
         (
           SG_AUTOPILOT,
-          SG_WARN,
+          SG_DEV_WARN,
           "Duplicate " << childName << " configuration name " << name
                                     << ", renamed to " << apName
         );
@@ -180,18 +172,18 @@ void FGXMLAutopilotGroupImplementation::initFrom( SGPropertyNode_ptr rootNode,
 
 void FGXMLAutopilotGroup::addAutopilotFromFile( const std::string& name,
                                                 SGPropertyNode_ptr apNode,
-                                                const char* path )
+                                                const std::string& path )
 {
   SGPath config = globals->resolve_maybe_aircraft_path(path);
   if( config.isNull() )
   {
-    SG_LOG
-    (
-      SG_AUTOPILOT,
-      SG_ALERT,
-      "Cannot find property-rule configuration file '" << path << "'."
-    );
-    return;
+      simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::AircraftSystems,
+                             std::string{"Autopilot XML not found:"} + path, sg_location{path});
+      SG_LOG(
+          SG_AUTOPILOT,
+          SG_ALERT,
+          "Cannot find property-rule configuration file '" << path << "'.");
+      return;
   }
   SG_LOG
   (
@@ -202,11 +194,11 @@ void FGXMLAutopilotGroup::addAutopilotFromFile( const std::string& name,
 
   try
   {
-    SGPropertyNode_ptr configNode = new SGPropertyNode();
-    readProperties( config, configNode );
+      SGPropertyNode_ptr configNode = new SGPropertyNode();
+      readProperties(config, configNode);
 
-    SG_LOG(SG_AUTOPILOT, SG_INFO, "adding  property-rule subsystem " << name);
-    addAutopilot(name, apNode, configNode);
+      SG_LOG(SG_AUTOPILOT, SG_INFO, "adding  property-rule subsystem " << name);
+      addAutopilot(name, apNode, configNode);
   }
   catch (const sg_exception& e)
   {
@@ -217,6 +209,8 @@ void FGXMLAutopilotGroup::addAutopilotFromFile( const std::string& name,
       "Failed to load property-rule configuration: " << config
                                              << ": " << e.getMessage()
     );
+    simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::AircraftSystems,
+                           std::string{"Autopilot XML failed to load:"} + e.getFormattedMessage(), e.getLocation());
     return;
   }
 }
