@@ -26,7 +26,13 @@
 //
 
 #include "digitalfilter.hxx"
+#include <GUI/Highlight.hxx>
+#include <Main/globals.hxx>
+
 #include <deque>
+#include <algorithm>
+
+#include <simgear/misc/strutils.hxx>
 
 namespace FGXMLAutopilot
 {
@@ -48,22 +54,26 @@ class DigitalFilterImplementation:
                             SGPropertyNode& prop_root ) = 0;
 
     void setDigitalFilter( DigitalFilter * digitalFilter ) { _digitalFilter = digitalFilter; }
-
+    virtual void collectDependentProperties(std::set<const SGPropertyNode*>& props) const = 0;
   protected:
-    DigitalFilter * _digitalFilter;
+    DigitalFilter * _digitalFilter = nullptr;
 };
 
 /* --------------------------------------------------------------------------------- */
 /* --------------------------------------------------------------------------------- */
 class GainFilterImplementation : public DigitalFilterImplementation {
 protected:
-  InputValueList _gainInput;
-  bool configure( SGPropertyNode& cfg_node,
-                  const std::string& cfg_name,
-                  SGPropertyNode& prop_root );
+    simgear::ValueList _gainInput;
+    bool configure(SGPropertyNode& cfg_node,
+                   const std::string& cfg_name,
+                   SGPropertyNode& prop_root);
 public:
   GainFilterImplementation() : _gainInput(1.0) {}
   double compute(  double dt, double input );
+  virtual void collectDependentProperties(std::set<const SGPropertyNode*>& props) const
+  {
+    _gainInput.collectDependentProperties(props);
+  }
 };
 
 class ReciprocalFilterImplementation : public GainFilterImplementation {
@@ -72,7 +82,7 @@ public:
 };
 
 class DerivativeFilterImplementation : public GainFilterImplementation {
-  InputValueList _TfInput;
+  simgear::ValueList _TfInput;
   double _input_1;
   bool configure( SGPropertyNode& cfg_node,
                   const std::string& cfg_name,
@@ -81,40 +91,54 @@ public:
   DerivativeFilterImplementation();
   double compute(  double dt, double input );
   virtual void initialize( double initvalue );
+  virtual void collectDependentProperties(std::set<const SGPropertyNode*>& props) const
+  {
+    GainFilterImplementation::collectDependentProperties(props);
+    _TfInput.collectDependentProperties(props);
+  }
 };
 
 class ExponentialFilterImplementation : public GainFilterImplementation {
 protected:
-  InputValueList _TfInput;
-  bool configure( SGPropertyNode& cfg_node,
-                  const std::string& cfg_name,
-                  SGPropertyNode& prop_root );
-  bool _isSecondOrder;
-  double _output_1, _output_2;
+    simgear::ValueList _TfInput;
+    bool configure(SGPropertyNode& cfg_node,
+                   const std::string& cfg_name,
+                   SGPropertyNode& prop_root);
+    bool _isSecondOrder;
+    double _output_1, _output_2;
 public:
   ExponentialFilterImplementation();
   double compute(  double dt, double input );
   virtual void initialize( double initvalue );
+  virtual void collectDependentProperties(std::set<const SGPropertyNode*>& props) const
+  {
+    GainFilterImplementation::collectDependentProperties(props);
+    _TfInput.collectDependentProperties(props);
+  }
 };
 
 class MovingAverageFilterImplementation : public DigitalFilterImplementation {
 protected:
-  InputValueList _samplesInput;
-  double _output_1;
-  std::deque <double> _inputQueue;
-  bool configure( SGPropertyNode& cfg_node,
-                  const std::string& cfg_name,
-                  SGPropertyNode& prop_root );
+    simgear::ValueList _samplesInput;
+    double _output_1;
+    std::deque<double> _inputQueue;
+    bool configure(SGPropertyNode& cfg_node,
+                   const std::string& cfg_name,
+                   SGPropertyNode& prop_root);
 public:
   MovingAverageFilterImplementation();
   double compute(  double dt, double input );
   virtual void initialize( double initvalue );
+  virtual void collectDependentProperties(std::set<const SGPropertyNode*>& props) const
+  {
+    _samplesInput.collectDependentProperties(props);
+  }
 };
 
 class NoiseSpikeFilterImplementation : public DigitalFilterImplementation {
 protected:
   double _output_1;
-  InputValueList _rateOfChangeInput;
+  simgear::ValueList _rateOfChangeInput;
   bool configure( SGPropertyNode& cfg_node,
                   const std::string& cfg_name,
                   SGPropertyNode& prop_root );
@@ -122,13 +146,17 @@ public:
   NoiseSpikeFilterImplementation();
   double compute(  double dt, double input );
   virtual void initialize( double initvalue );
+  virtual void collectDependentProperties(std::set<const SGPropertyNode*>& props) const
+  {
+    _rateOfChangeInput.collectDependentProperties(props);
+  }
 };
 
 class RateLimitFilterImplementation : public DigitalFilterImplementation {
 protected:
   double _output_1;
-  InputValueList _rateOfChangeMax;
-  InputValueList _rateOfChangeMin ;
+  simgear::ValueList _rateOfChangeMax;
+  simgear::ValueList _rateOfChangeMin;
   bool configure( SGPropertyNode& cfg_node,
                   const std::string& cfg_name,
                   SGPropertyNode& prop_root );
@@ -136,69 +164,123 @@ public:
   RateLimitFilterImplementation();
   double compute(  double dt, double input );
   virtual void initialize( double initvalue );
+  virtual void collectDependentProperties(std::set<const SGPropertyNode*>& props) const
+  {
+    _rateOfChangeMax.collectDependentProperties(props);
+    _rateOfChangeMin.collectDependentProperties(props);
+  }
 };
 
 class IntegratorFilterImplementation : public GainFilterImplementation {
 protected:
-  InputValueList _TfInput;
-  InputValueList _minInput;
-  InputValueList _maxInput;
-  double _input_1;
-  double _output_1;
-  bool configure( SGPropertyNode& cfg_node,
-                  const std::string& cfg_name,
-                  SGPropertyNode& prop_root );
+    simgear::ValueList _TfInput;
+    simgear::ValueList _minInput;
+    simgear::ValueList _maxInput;
+    double _input_1;
+    double _output_1;
+    bool configure(SGPropertyNode& cfg_node,
+                   const std::string& cfg_name,
+                   SGPropertyNode& prop_root);
 public:
   IntegratorFilterImplementation();
   double compute(  double dt, double input );
   virtual void initialize( double initvalue );
+  virtual void collectDependentProperties(std::set<const SGPropertyNode*>& props) const
+  {
+    GainFilterImplementation::collectDependentProperties(props);
+    _TfInput.collectDependentProperties(props);
+    _minInput.collectDependentProperties(props);
+    _maxInput.collectDependentProperties(props);
+  }
 };
 
 // integrates x" + ax' + bx + c = 0
 class DampedOscillationFilterImplementation : public GainFilterImplementation {
 protected:
-  InputValueList _aInput;
-  InputValueList _bInput;
-  InputValueList _cInput;
-  double _x2;
-  double _x1;
-  double _x0;
-  bool configure( SGPropertyNode& cfg_node,
-                  const std::string& cfg_name,
-                  SGPropertyNode& prop_root );
+    simgear::ValueList _aInput;
+    simgear::ValueList _bInput;
+    simgear::ValueList _cInput;
+    double _x2;
+    double _x1;
+    double _x0;
+    bool configure(SGPropertyNode& cfg_node,
+                   const std::string& cfg_name,
+                   SGPropertyNode& prop_root);
 public:
   DampedOscillationFilterImplementation();
   double compute(  double dt, double input );
   virtual void initialize( double initvalue );
+  virtual void collectDependentProperties(std::set<const SGPropertyNode*>& props) const
+  {
+    GainFilterImplementation::collectDependentProperties(props);
+    _aInput.collectDependentProperties(props);
+    _bInput.collectDependentProperties(props);
+    _cInput.collectDependentProperties(props);
+  }
 };
 
 class HighPassFilterImplementation : public GainFilterImplementation {
 protected:
-  InputValueList _TfInput;
-  double _input_1;
-  double _output_1;
-  bool configure( SGPropertyNode& cfg_node,
-                  const std::string& cfg_name,
-                  SGPropertyNode& prop_root );
+    simgear::ValueList _TfInput;
+    double _input_1;
+    double _output_1;
+    bool configure(SGPropertyNode& cfg_node,
+                   const std::string& cfg_name,
+                   SGPropertyNode& prop_root);
 public:
   HighPassFilterImplementation();
   double compute(  double dt, double input );
   virtual void initialize( double initvalue );
+  virtual void collectDependentProperties(std::set<const SGPropertyNode*>& props) const
+  {
+    GainFilterImplementation::collectDependentProperties(props);
+    _TfInput.collectDependentProperties(props);
+  }
 };
 class LeadLagFilterImplementation : public GainFilterImplementation {
 protected:
-  InputValueList _TfaInput;
-  InputValueList _TfbInput;
-  double _input_1;
-  double _output_1;
-  bool configure( SGPropertyNode& cfg_node,
-                  const std::string& cfg_name,
-                  SGPropertyNode& prop_root );
+    simgear::ValueList _TfaInput;
+    simgear::ValueList _TfbInput;
+    double _input_1;
+    double _output_1;
+    bool configure(SGPropertyNode& cfg_node,
+                   const std::string& cfg_name,
+                   SGPropertyNode& prop_root);
 public:
   LeadLagFilterImplementation();
   double compute(  double dt, double input );
   virtual void initialize( double initvalue );
+  virtual void collectDependentProperties(std::set<const SGPropertyNode*>& props) const
+  {
+    GainFilterImplementation::collectDependentProperties(props);
+    _TfaInput.collectDependentProperties(props);
+    _TfbInput.collectDependentProperties(props);
+  }
 };
+
+class CoherentNoiseFilterImplementation : public DigitalFilterImplementation
+{
+protected:
+    simgear::ValueList _amplitude;
+
+    std::vector<double> _discreteValues;
+    bool _absoluteVal = false;
+    size_t _numDiscreteValues = 1024;
+
+    bool configure(SGPropertyNode& cfg_node,
+                   const std::string& cfg_name,
+                   SGPropertyNode& prop_root) override;
+
+public:
+    CoherentNoiseFilterImplementation();
+    double compute(double dt, double input) override;
+    void initialize(double initvalue) override;
+  virtual void collectDependentProperties(std::set<const SGPropertyNode*>& props) const
+  {
+    _amplitude.collectDependentProperties(props);
+  }
+};
+
 /* --------------------------------------------------------------------------------- */
 /* --------------------------------------------------------------------------------- */
 
@@ -224,7 +306,7 @@ bool GainFilterImplementation::configure( SGPropertyNode& cfg_node,
                                           SGPropertyNode& prop_root )
 {
   if (cfg_name == "gain" ) {
-    _gainInput.push_back( new InputValue(prop_root, cfg_node, 1) );
+    _gainInput.push_back(new simgear::Value(prop_root, cfg_node, 1));
     return true;
   }
 
@@ -265,7 +347,7 @@ bool DerivativeFilterImplementation::configure( SGPropertyNode& cfg_node,
     return true;
 
   if (cfg_name == "filter-time" ) {
-    _TfInput.push_back( new InputValue(prop_root, cfg_node, 1) );
+    _TfInput.push_back(new simgear::Value(prop_root, cfg_node, 1));
     return true;
   }
 
@@ -323,7 +405,7 @@ bool MovingAverageFilterImplementation::configure( SGPropertyNode& cfg_node,
                                                    SGPropertyNode& prop_root )
 {
   if (cfg_name == "samples" ) {
-    _samplesInput.push_back( new InputValue(prop_root, cfg_node, 1) );
+    _samplesInput.push_back(new simgear::Value(prop_root, cfg_node, 1));
     return true;
   }
 
@@ -349,7 +431,7 @@ double NoiseSpikeFilterImplementation::compute(  double dt, double input )
   if( fabs(delta) <= SGLimitsd::min() ) return input; // trivial
 
   double maxChange = _rateOfChangeInput.get_value() * dt;
-  const PeriodicalValue * periodical = _digitalFilter->getPeriodicalValue();
+  const auto periodical = _digitalFilter->getPeriodicalValue();
   if( periodical ) delta = periodical->normalizeSymmetric( delta );
 
   if( fabs(delta) <= maxChange )
@@ -364,7 +446,7 @@ bool NoiseSpikeFilterImplementation::configure( SGPropertyNode& cfg_node,
                                                 SGPropertyNode& prop_root )
 {
   if (cfg_name == "max-rate-of-change" ) {
-    _rateOfChangeInput.push_back( new InputValue(prop_root, cfg_node, 1) );
+    _rateOfChangeInput.push_back(new simgear::Value(prop_root, cfg_node, 1));
     return true;
   }
 
@@ -409,11 +491,11 @@ bool RateLimitFilterImplementation::configure( SGPropertyNode& cfg_node,
 {
 //  std::cout << "RateLimitFilterImplementation " << cfg_name << std::endl;
   if (cfg_name == "max-rate-of-change" ) {
-    _rateOfChangeMax.push_back( new InputValue(prop_root, cfg_node, 1) );
+    _rateOfChangeMax.push_back(new simgear::Value(prop_root, cfg_node, 1));
     return true;
   }
   if (cfg_name == "min-rate-of-change" ) {
-    _rateOfChangeMin.push_back( new InputValue(prop_root, cfg_node, 1) );
+    _rateOfChangeMin.push_back(new simgear::Value(prop_root, cfg_node, 1));
     return true;
   }
 
@@ -467,12 +549,12 @@ bool ExponentialFilterImplementation::configure( SGPropertyNode& cfg_node,
     return true;
 
   if (cfg_name == "filter-time" ) {
-    _TfInput.push_back( new InputValue(prop_root, cfg_node, 1) );
+    _TfInput.push_back(new simgear::Value(prop_root, cfg_node, 1));
     return true;
   }
 
   if (cfg_name == "type" ) {
-    std::string type(cfg_node.getStringValue());
+      std::string type = simgear::strutils::strip(cfg_node.getStringValue());
     _isSecondOrder = type == "double-exponential";
   }
 
@@ -501,11 +583,11 @@ bool IntegratorFilterImplementation::configure( SGPropertyNode& cfg_node,
     return true;
 
   if (cfg_name == "u_min" ) {
-    _minInput.push_back( new InputValue(prop_root, cfg_node, 1) );
+    _minInput.push_back(new simgear::Value(prop_root, cfg_node, 1));
     return true;
   }
   if (cfg_name == "u_max" ) {
-    _maxInput.push_back( new InputValue(prop_root, cfg_node, 1) );
+    _maxInput.push_back(new simgear::Value(prop_root, cfg_node, 1));
     return true;
   }
   return false;
@@ -543,15 +625,15 @@ bool DampedOscillationFilterImplementation::configure( SGPropertyNode& cfg_node,
     return true;
 
   if (cfg_name == "a" ) {
-    _aInput.push_back( new InputValue(prop_root, cfg_node, 1) );
+    _aInput.push_back(new simgear::Value(prop_root, cfg_node, 1));
     return true;
   }
   if (cfg_name == "b" ) {
-    _bInput.push_back( new InputValue(prop_root, cfg_node, 1) );
+    _bInput.push_back(new simgear::Value(prop_root, cfg_node, 1));
     return true;
   }
   if (cfg_name == "c" ) {
-    _cInput.push_back( new InputValue(prop_root, cfg_node, 1) );
+    _cInput.push_back(new simgear::Value(prop_root, cfg_node, 1));
     return true;
   }
   return false;
@@ -642,7 +724,7 @@ bool HighPassFilterImplementation::configure( SGPropertyNode& cfg_node,
     return true;
 
   if (cfg_name == "filter-time" ) {
-    _TfInput.push_back( new InputValue(prop_root, cfg_node, 1) );
+    _TfInput.push_back(new simgear::Value(prop_root, cfg_node, 1));
     return true;
   }
 
@@ -692,15 +774,76 @@ bool LeadLagFilterImplementation::configure( SGPropertyNode& cfg_node,
     return true;
 
   if (cfg_name == "filter-time-a" ) {
-    _TfaInput.push_back( new InputValue(prop_root, cfg_node, 1) );
+    _TfaInput.push_back(new simgear::Value(prop_root, cfg_node, 1));
     return true;
   }
   if (cfg_name == "filter-time-b" ) {
-    _TfbInput.push_back( new InputValue(prop_root, cfg_node, 1) );
+    _TfbInput.push_back(new simgear::Value(prop_root, cfg_node, 1));
     return true;
   }
   return false;
 }
+
+
+/* --------------------------------------------------------------------------------- */
+
+CoherentNoiseFilterImplementation::CoherentNoiseFilterImplementation() : _amplitude(1.0),
+                                                                         _numDiscreteValues(1024)
+{
+}
+
+void CoherentNoiseFilterImplementation::initialize(double initvalue)
+{
+    // allocate the array one bigger so we don't need to worry about
+    // wrapping.bound checking the discrete +1 lookup
+    _discreteValues.resize(_numDiscreteValues + 1);
+    std::generate(_discreteValues.begin(), _discreteValues.end(), []() {
+        return (sg_random() * 2.0) - 1.0;
+    });
+}
+
+static double lerp(double a, double b, double t)
+{
+    return (a * (1.0 - t)) + (b * t);
+}
+
+double CoherentNoiseFilterImplementation::compute(double dt, double input)
+{
+    const double a = _amplitude.get_value();
+
+    const double t = input * _numDiscreteValues;
+    const int i = static_cast<int>(floor(t)) % _numDiscreteValues;
+    const auto v0 = _discreteValues.at(i);
+    const auto v1 = _discreteValues.at(i + 1);
+
+    const auto weight = t - floor(t);
+    const double output = lerp(v0, v1, weight);
+
+    return _absoluteVal ? fabs(output) * a : output * a;
+}
+
+//------------------------------------------------------------------------------
+bool CoherentNoiseFilterImplementation::configure(SGPropertyNode& cfg_node,
+                                                  const std::string& cfg_name,
+                                                  SGPropertyNode& prop_root)
+{
+    if (cfg_name == "discrete-resolution") {
+        _numDiscreteValues = cfg_node.getIntValue();
+        return true;
+    }
+
+    if (cfg_name == "amplitude") {
+        _amplitude.push_back(new simgear::Value(prop_root, cfg_node, 1.0));
+        return true;
+    }
+
+    if (cfg_name == "absolute") {
+        _absoluteVal = cfg_node.getBoolValue();
+        return true;
+    }
+    return false;
+}
+
 /* -------------------------------------------------------------------------- */
 /* Digital Filter Component Implementation                                    */
 /* -------------------------------------------------------------------------- */
@@ -744,9 +887,10 @@ bool DigitalFilter::configure( SGPropertyNode& prop_root,
     componentForge["lead-lag"           ] = digitalFilterFactory<LeadLagFilterImplementation>;
     componentForge["integrator"         ] = digitalFilterFactory<IntegratorFilterImplementation>;
     componentForge["damped-oscillation" ] = digitalFilterFactory<DampedOscillationFilterImplementation>;
+    componentForge["coherent-noise"] = digitalFilterFactory<CoherentNoiseFilterImplementation>;
   }
 
-  const std::string type = cfg.getStringValue("type");
+  const auto type = simgear::strutils::strip(cfg.getStringValue("type"));
   DigitalFilterMap::iterator component_factory = componentForge.find(type);
   if( component_factory == componentForge.end() )
   {
@@ -760,21 +904,40 @@ bool DigitalFilter::configure( SGPropertyNode& prop_root,
   for( int i = 0; i < cfg.nChildren(); ++i )
   {
     SGPropertyNode_ptr child = cfg.getChild(i);
-    std::string cname(child->getName());
-
-    if(    !_implementation->configure(*child, cname, prop_root)
-        && !configure(*child, cname, prop_root)
-        && cname != "type"
-        && cname != "params" ) // 'params' is usually used to specify parameters
-                               // in PropertList files.
+    std::string cname(child->getNameString());
+    bool ok = false;
+    if (!ok) ok = _implementation->configure(*child, cname, prop_root);
+    if (!ok) ok = configure(*child, cname, prop_root);
+    if (!ok) ok = (cname == "type");
+    if (!ok) ok = (cname == "params");   // 'params' is usually used to specify parameters in PropertList files.
+    if (!ok) {
       SG_LOG
       (
         SG_AUTOPILOT,
-        SG_WARN,
+        SG_ALERT,
         "DigitalFilter: unknown config node: " << cname
       );
+    }
   }
-
+  
+  /* Send information about associations between our input and output
+  properties to Highlight. */
+  std::set<const SGPropertyNode*>    inputs;
+  _implementation->collectDependentProperties(inputs);
+  collectDependentProperties(inputs);
+  
+  auto highlight = globals->get_subsystem<Highlight>();
+  if (highlight) {
+    for (auto in: inputs) {
+      for (auto& out: _output_list) {
+          highlight->addPropertyProperty(
+                  in->getPath(true /*simplify*/),
+                  out->getPath(true /*simplify*/)
+                  );
+      }
+    }
+  }
+  
   return true;
 }
 
@@ -785,7 +948,7 @@ bool DigitalFilter::configure( SGPropertyNode& cfg_node,
 {
   if( cfg_name == "initialize-to" )
   {
-    std::string s( cfg_node.getStringValue() );
+      const auto s = simgear::strutils::strip(cfg_node.getStringValue());
     if( s == "input" )
       _initializeTo = INITIALIZE_INPUT;
     else if( s == "output" )
@@ -814,17 +977,17 @@ void DigitalFilter::update( bool firstTime, double dt)
     switch( _initializeTo ) {
 
       case INITIALIZE_INPUT:
-        SG_LOG(SG_AUTOPILOT,SG_DEBUG, "First time initialization of " << get_name() << " to " << _valueInput.get_value() );
+        SG_LOG(SG_AUTOPILOT,SG_DEBUG, "First time initialization of " << subsystemId() << " to " << _valueInput.get_value() );
         _implementation->initialize( _valueInput.get_value() );
         break;
 
       case INITIALIZE_OUTPUT:
-        SG_LOG(SG_AUTOPILOT,SG_DEBUG, "First time initialization of " << get_name() << " to " << get_output_value() );
+        SG_LOG(SG_AUTOPILOT,SG_DEBUG, "First time initialization of " << subsystemId() << " to " << get_output_value() );
         _implementation->initialize( get_output_value() );
         break;
 
       default:
-        SG_LOG(SG_AUTOPILOT,SG_DEBUG, "First time initialization of " << get_name() << " to (uninitialized)" );
+        SG_LOG(SG_AUTOPILOT,SG_DEBUG, "First time initialization of " << subsystemId() << " to (uninitialized)" );
         break;
     }
   }
@@ -837,7 +1000,11 @@ void DigitalFilter::update( bool firstTime, double dt)
   set_output_value( output );
 
   if(_debug) {
-    std::cout << "input:" << input
-              << "\toutput:" << output << std::endl;
+    std::cout << subsystemId() << ": input=" << input
+              << "\toutput=" << output << std::endl;
   }
 }
+
+
+// Register the subsystem.
+SGSubsystemMgr::Registrant<DigitalFilter> registrantDigitalFilter;
