@@ -21,7 +21,7 @@
 
 #include "flipflop.hxx"
 #include "functor.hxx"
-#include "inputvalue.hxx"
+#include <simgear/misc/inputvalue.hxx>
 #include <Main/fg_props.hxx>
 
 using std::map;
@@ -267,8 +267,14 @@ protected:
   virtual bool configure( SGPropertyNode& cfg_node,
                           const std::string& cfg_name,
                           SGPropertyNode& prop_root );
-  InputValueList _time;
+  simgear::ValueList _time;
   double _t;
+
+  bool isConfigProperty(const std::string& cfg_name) const override
+  {
+    return cfg_name == "time";
+  }
+
 public:
   /**
    * @brief constructor for a MonoFlopImplementation
@@ -299,7 +305,7 @@ bool MonoFlopImplementation::configure( SGPropertyNode& cfg_node,
     return true;
 
   if (cfg_name == "time") {
-    _time.push_back( new InputValue(prop_root, cfg_node) );
+    _time.push_back(new simgear::Value(prop_root, cfg_node));
     return true;
   } 
 
@@ -383,7 +389,7 @@ bool FlipFlopImplementation::configure( SGPropertyNode& prop_root,
   for( int i = 0; i < cfg.nChildren(); ++i )
   {
     SGPropertyNode_ptr child = cfg.getChild(i);
-    string cname(child->getName());
+    string cname(child->getNameString());
 
     if( configure(*child, cname, prop_root) )
       continue;
@@ -457,13 +463,19 @@ bool FlipFlop::configure( SGPropertyNode& cfg_node,
     return true;
   }
 
+  if (_implementation && _implementation->isConfigProperty(cfg_name)) {
+    // componentForge CreateAndConfigureFunctor<> already called configure with all the nodes
+    // on our implementation, but we need to ensure we don't return false here for such nodes
+    return true;
+  }
+
   return false; 
 }
 
 void FlipFlop::update( bool firstTime, double dt )
 {
   if( _implementation == NULL ) {
-    SG_LOG( SG_AUTOPILOT, SG_ALERT, "No flip-flop implementation for " << get_name() << endl );
+    SG_LOG( SG_AUTOPILOT, SG_ALERT, "No flip-flop implementation for " << subsystemId() << endl );
     return;
   }
 
@@ -475,7 +487,7 @@ void FlipFlop::update( bool firstTime, double dt )
     set_output( q );
 
     if(_debug) {
-      cout << "updating flip-flop \"" << get_name() << "\"" << endl;
+      cout << "updating flip-flop \"" << subsystemId() << "\"" << endl;
       cout << "prev. Output:" << q0 << endl;
       for( InputMap::const_iterator it = _input.begin(); it != _input.end(); ++it ) 
         cout << "Input \"" << (*it).first << "\":" << (*it).second->test() << endl;
@@ -485,3 +497,5 @@ void FlipFlop::update( bool firstTime, double dt )
 }
 
 
+// Register the subsystem.
+SGSubsystemMgr::Registrant<FlipFlop> registrantFlipFlop;
