@@ -1,46 +1,22 @@
 // route_mgr.cxx - manage a route (i.e. a collection of waypoints)
-//
-// Written by Curtis Olson, started January 2004.
-//            Norman Vine
-//            Melchior FRANZ
-//
-// Copyright (C) 2004  Curtis L. Olson  - http://www.flightgear.org/~curt
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// $Id$
+/*
+ * SPDX-FileCopyrightText: (C) 2004 Curtis L. Olson http://www.flightgear.org/~curt
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #include <cstdio>
 
 #include <simgear/compiler.h>
-
 #include "route_mgr.hxx"
 
-#include <boost/algorithm/string/case_conv.hpp>
-#include <boost/tuple/tuple.hpp>
-#include <boost/foreach.hpp>
-
-#include <simgear/misc/strutils.hxx>
-#include <simgear/structure/exception.hxx>
-#include <simgear/structure/commands.hxx>
 #include <simgear/misc/sg_path.hxx>
+#include <simgear/misc/strutils.hxx>
+#include <simgear/structure/commands.hxx>
+#include <simgear/structure/exception.hxx>
+
 #include <simgear/timing/sg_time.hxx>
 #include <simgear/sg_inlines.h>
 
@@ -55,26 +31,26 @@
 #include "Airports/runways.hxx"
 #include <GUI/new_gui.hxx>
 #include <GUI/dialog.hxx>
-#include <Main/util.hxx>        // fgValidatePath()
 #include <GUI/MessageBox.hxx>
 
 #define RM "/autopilot/route-manager/"
 
 using namespace flightgear;
 using std::string;
+namespace su = simgear::strutils;
 
 static bool commandLoadFlightPlan(const SGPropertyNode* arg, SGPropertyNode *)
 {
-  FGRouteMgr* self = (FGRouteMgr*) globals->get_subsystem("route-manager");
+  auto self = globals->get_subsystem<FGRouteMgr>();
   SGPath path = SGPath::fromUtf8(arg->getStringValue("path"));
   return self->loadRoute(path);
 }
 
 static bool commandSaveFlightPlan(const SGPropertyNode* arg, SGPropertyNode *)
 {
-  FGRouteMgr* self = (FGRouteMgr*) globals->get_subsystem("route-manager");
+  auto self = globals->get_subsystem<FGRouteMgr>();
   SGPath path = SGPath::fromUtf8(arg->getStringValue("path"));
-  SGPath authorizedPath = fgValidatePath(path, true /* write */);
+  const SGPath authorizedPath = SGPath(path).validate(true /* write */);
 
   if (!authorizedPath.isNull()) {
     return self->saveRoute(authorizedPath);
@@ -94,7 +70,7 @@ static bool commandSaveFlightPlan(const SGPropertyNode* arg, SGPropertyNode *)
 
 static bool commandActivateFlightPlan(const SGPropertyNode* arg, SGPropertyNode *)
 {
-  FGRouteMgr* self = (FGRouteMgr*) globals->get_subsystem("route-manager");
+  auto self = globals->get_subsystem<FGRouteMgr>();
   bool activate = arg->getBoolValue("activate", true);
   if (activate) {
     self->activate();
@@ -107,14 +83,14 @@ static bool commandActivateFlightPlan(const SGPropertyNode* arg, SGPropertyNode
 
 static bool commandClearFlightPlan(const SGPropertyNode*, SGPropertyNode *)
 {
-  FGRouteMgr* self = (FGRouteMgr*) globals->get_subsystem("route-manager");
+  auto self = globals->get_subsystem<FGRouteMgr>();
   self->clearRoute();
   return true;
 }
 
 static bool commandSetActiveWaypt(const SGPropertyNode* arg, SGPropertyNode *)
 {
-  FGRouteMgr* self = (FGRouteMgr*) globals->get_subsystem("route-manager");
+  auto self = globals->get_subsystem<FGRouteMgr>();
   int index = arg->getIntValue("index");
   if ((index < 0) || (index >= self->numLegs())) {
     return false;
@@ -126,27 +102,40 @@ static bool commandSetActiveWaypt(const SGPropertyNode* arg, SGPropertyNode *)
 
 static bool commandInsertWaypt(const SGPropertyNode* arg, SGPropertyNode *)
 {
-  FGRouteMgr* self = (FGRouteMgr*) globals->get_subsystem("route-manager");
-  int index = arg->getIntValue("index");
-  std::string ident(arg->getStringValue("id"));
-  int alt = arg->getIntValue("altitude-ft", -999);
-  int ias = arg->getIntValue("speed-knots", -999);
-  
-  WayptRef wp;
-// lat/lon may be supplied to narrow down navaid search, or to specify
-// a raw waypoint
-  SGGeod pos;
-  if (arg->hasChild("longitude-deg")) {
-    pos = SGGeod::fromDeg(arg->getDoubleValue("longitude-deg"),
-                               arg->getDoubleValue("latitude-deg"));
+    auto self = globals->get_subsystem<FGRouteMgr>();
+    const bool haveIndex = arg->hasChild("index");
+    int index = arg->getIntValue("index");
+
+    std::string ident(arg->getStringValue("id"));
+    int alt = arg->getIntValue("altitude-ft", -999);
+    int ias = arg->getIntValue("speed-knots", -999);
+
+    WayptRef wp;
+    // lat/lon may be supplied to narrow down navaid search, or to specify
+    // a raw waypoint
+    SGGeod pos = SGGeod::invalid();
+    if (arg->hasChild("longitude-deg")) {
+        pos = SGGeod::fromDeg(arg->getDoubleValue("longitude-deg"),
+                              arg->getDoubleValue("latitude-deg"));
   }
   
   if (arg->hasChild("navaid")) {
-    FGPositionedRef p = FGPositioned::findClosestWithIdent(arg->getStringValue("navaid"), pos);
-    
+      if (!pos.isValid()) {
+          pos = self->flightPlan()->vicinityForInsertIndex(haveIndex ? index : -1 /* append */);
+      }
+
+      FGPositioned::TypeFilter filter({FGPositioned::Type::NDB, FGPositioned::Type::VOR,
+                                       FGPositioned::Type::FIX, FGPositioned::Type::WAYPOINT});
+
+      FGPositionedRef p = FGPositioned::findClosestWithIdent(arg->getStringValue("navaid"), pos, &filter);
+      if (!p) {
+          SG_LOG(SG_AUTOPILOT, SG_WARN, "Unable to find navaid with ident:" << arg->getStringValue("navaid"));
+          return false;
+      }
+
     if (arg->hasChild("navaid", 1)) {
       // intersection of two radials
-      FGPositionedRef p2 = FGPositioned::findClosestWithIdent(arg->getStringValue("navaid[1]"), pos);
+      FGPositionedRef p2 = FGPositioned::findClosestWithIdent(arg->getStringValue("navaid[1]"), pos, &filter);
       if (!p2) {
         SG_LOG( SG_AUTOPILOT, SG_INFO, "Unable to find FGPositioned with ident:" << arg->getStringValue("navaid[1]"));
         return false;
@@ -193,12 +182,24 @@ static bool commandInsertWaypt(const SGPropertyNode* arg, SGPropertyNode *)
       wp = new NavaidWaypoint((FGAirport*) apt, NULL);
     }
   } else if (arg->hasChild("text")) {
-    wp = self->waypointFromString(arg->getStringValue("text"));
-  } else if (!(pos == SGGeod())) {
-    // just a raw lat/lon
-    wp = new BasicWaypt(pos, ident, NULL);
+      const auto t = su::strip(arg->getStringValue("text"));
+      if (pos.isValid()) {
+          // if 'pos' is valid, use it as the search vicinity
+          wp = self->flightPlan()->waypointFromString(t, pos);
+      } else {
+          const int searchIndex = haveIndex ? index : -1;
+          wp = self->waypointFromString(t, searchIndex);
+      }
+
+      if (!wp) { // failed to build waypoint
+          SG_LOG(SG_AUTOPILOT, SG_WARN, "insert-waypoint failed: couldn't parse waypoint from '" << t << "'");
+          return false;
+      }
+  } else if (pos.isValid()) {
+      // just a raw lat/lon
+      wp = new BasicWaypt(pos, ident, NULL);
   } else {
-    return false; // failed to build waypoint
+      return false; // failed to build waypoint
   }
 
   FlightPlan::Leg* leg = self->flightPlan()->insertWayptAtIndex(wp, index);
@@ -215,7 +216,7 @@ static bool commandInsertWaypt(const SGPropertyNode* arg, SGPropertyNode *)
 
 static bool commandDeleteWaypt(const SGPropertyNode* arg, SGPropertyNode *)
 {
-  FGRouteMgr* self = (FGRouteMgr*) globals->get_subsystem("route-manager");
+  auto self = globals->get_subsystem<FGRouteMgr>();
   int index = arg->getIntValue("index");
   self->removeLegAtIndex(index);
   return true;
@@ -224,7 +225,6 @@ static bool commandDeleteWaypt(const SGPropertyNode* arg, SGPropertyNode *)
 /////////////////////////////////////////////////////////////////////////////
 
 FGRouteMgr::FGRouteMgr() :
-  _plan(NULL),
   input(fgGetNode( RM "input", true )),
   mirror(fgGetNode( RM "route", true ))
 {
@@ -284,9 +284,9 @@ void FGRouteMgr::init() {
                                                          &FGRouteMgr::setSID));
   
   departure->tie("name", SGStringValueMethods<FGRouteMgr>(*this, 
-    &FGRouteMgr::getDepartureName, NULL));
+    &FGRouteMgr::getDepartureName, nullptr));
   departure->tie("field-elevation-ft", SGRawValueMethods<FGRouteMgr, double>(*this, 
-                                                                             &FGRouteMgr::getDepartureFieldElevation, NULL));
+                                                                             &FGRouteMgr::getDepartureFieldElevation, nullptr));
   departure->getChild("etd", 0, true);
   departure->getChild("takeoff-time", 0, true);
 
@@ -306,25 +306,35 @@ void FGRouteMgr::init() {
                                                                 &FGRouteMgr::setApproach));
   
   destination->tie("name", SGStringValueMethods<FGRouteMgr>(*this, 
-    &FGRouteMgr::getDestinationName, NULL));
+    &FGRouteMgr::getDestinationName, nullptr));
   destination->tie("field-elevation-ft", SGRawValueMethods<FGRouteMgr, double>(*this, 
-                                                                      &FGRouteMgr::getDestinationFieldElevation, NULL));
+                                                                      &FGRouteMgr::getDestinationFieldElevation, nullptr));
   
   destination->getChild("eta", 0, true);
   destination->getChild("eta-seconds", 0, true);
   destination->getChild("touchdown-time", 0, true);
 
   alternate = fgGetNode(RM "alternate", true);
-  alternate->getChild("airport", 0, true);
-  alternate->getChild("runway", 0, true);
-  
+  alternate->tie("airport", SGStringValueMethods<FGRouteMgr>(*this,
+                             &FGRouteMgr::getAlternate,
+                            &FGRouteMgr::setAlternate));
+  alternate->tie("name", SGStringValueMethods<FGRouteMgr>(*this,
+    &FGRouteMgr::getAlternateName, nullptr));
+
   cruise = fgGetNode(RM "cruise", true);
-  cruise->getChild("altitude-ft", 0, true);
-  cruise->setDoubleValue("altitude-ft", 10000.0);
-  cruise->getChild("flight-level", 0, true);
-  cruise->getChild("speed-kts", 0, true);
-  cruise->setDoubleValue("speed-kts", 160.0);
-  
+  cruise->tie("altitude-ft", SGRawValueMethods<FGRouteMgr, int>(*this,
+                                                           &FGRouteMgr::getCruiseAltitudeFt,
+                                                           &FGRouteMgr::setCruiseAltitudeFt));
+  cruise->tie("flight-level", SGRawValueMethods<FGRouteMgr, int>(*this,
+                                                            &FGRouteMgr::getCruiseFlightLevel,
+                                                            &FGRouteMgr::setCruiseFlightLevel));
+  cruise->tie("speed-kts", SGRawValueMethods<FGRouteMgr, int>(*this,
+                                                         &FGRouteMgr::getCruiseSpeedKnots,
+                                                         &FGRouteMgr::setCruiseSpeedKnots));
+  cruise->tie("mach", SGRawValueMethods<FGRouteMgr, double>(*this,
+                                                       &FGRouteMgr::getCruiseSpeedMach,
+                                                       &FGRouteMgr::setCruiseSpeedMach));
+
   totalDistance = fgGetNode(RM "total-distance", true);
   totalDistance->setDoubleValue(0.0);
   distanceToGo = fgGetNode(RM "distance-remaining-nm", true);
@@ -344,8 +354,10 @@ void FGRouteMgr::init() {
     
   _edited = fgGetNode(RM "signals/edited", true);
   _flightplanChanged = fgGetNode(RM "signals/flightplan-changed", true);
-  
+  _isRoute = fgGetNode(RM "is-route", true);
+
   _currentWpt = fgGetNode(RM "current-wp", true);
+  _currentWpt->setAttribute(SGPropertyNode::LISTENER_SAFE, true);
   _currentWpt->tie(SGRawValueMethods<FGRouteMgr, int>
     (*this, &FGRouteMgr::currentIndex, &FGRouteMgr::jumpToIndex));
       
@@ -373,7 +385,7 @@ void FGRouteMgr::init() {
 
 void FGRouteMgr::postinit()
 {
-  setFlightPlan(new FlightPlan());
+  setFlightPlan(FlightPlan::create());
   _plan->setIdent("default-flightplan");
     
   SGPath path = SGPath::fromUtf8(_pathNode->getStringValue());
@@ -382,20 +394,22 @@ void FGRouteMgr::postinit()
     loadRoute(path);
   }
   
+  _isRoute->setBoolValue(_plan->isRoute());
+
 // this code only matters for the --wp option now - perhaps the option
 // should be deprecated in favour of an explicit flight-plan file?
 // then the global initial waypoint list could die.
   string_list *waypoints = globals->get_initial_waypoints();
   if (waypoints) {
-    string_list::iterator it;
-    for (it = waypoints->begin(); it != waypoints->end(); ++it) {
-      WayptRef w = waypointFromString(*it);
-      if (w) {
-        _plan->insertWayptAtIndex(w, -1);
-      }
+    for (const auto& wpStr : *waypoints) {
+        WayptRef w = waypointFromString(wpStr, -1);
+        if (w) {
+            _plan->insertWayptAtIndex(w, -1);
+        } else {
+            SG_LOG(SG_AUTOPILOT, SG_WARN, "Failed to create waypoint from '" << wpStr << "'");
+        }
     }
     
-    SG_LOG(SG_AUTOPILOT, SG_INFO, "loaded initial waypoints:" << numLegs());
     update_mirror();
   }
 
@@ -424,7 +438,7 @@ bool FGRouteMgr::saveRoute(const SGPath& p)
 
 bool FGRouteMgr::loadRoute(const SGPath& p)
 {
-  FlightPlan* fp = new FlightPlan;
+  FlightPlan* fp = FlightPlan::create();
   if (!fp->load(p)) {
     delete fp;
     return false;
@@ -457,7 +471,7 @@ void FGRouteMgr::setFlightPlan(const FlightPlanRef& plan)
   
   _plan = plan;
   _plan->addDelegate(this);
-  
+  _isRoute->setBoolValue(_plan->isRoute());
   _flightplanChanged->fireValueChanged();
   
 // fire all the callbacks!
@@ -467,6 +481,25 @@ void FGRouteMgr::setFlightPlan(const FlightPlanRef& plan)
   currentWaypointChanged();
 }
 
+void FGRouteMgr::departureChanged()
+{
+    auto gui = globals->get_subsystem<NewGUI>();
+    FGDialog* rmDlg = gui ? gui->getDialog("route-manager") : NULL;
+    if (rmDlg) {
+        rmDlg->runCallback("departure-changed");
+    }
+}
+
+void FGRouteMgr::arrivalChanged()
+{
+    auto gui = globals->get_subsystem<NewGUI>();
+    FGDialog* rmDlg = gui ? gui->getDialog("route-manager") : NULL;
+    if (rmDlg) {
+        rmDlg->runCallback("arrival-changed");
+    }
+}
+
+
 void FGRouteMgr::update( double dt )
 {
   if (dt <= 0.0) {
@@ -506,8 +539,11 @@ void FGRouteMgr::update( double dt )
   }
   
   // use RoutePath to compute location of active WP
-  RoutePath path(_plan);
-  SGGeod wpPos = path.positionForIndex(_plan->currentIndex());
+  if (!_routePath) {
+    _routePath.reset(new RoutePath{_plan});
+  }
+
+  SGGeod wpPos = _routePath->positionForIndex(_plan->currentIndex());
   double courseDeg, az2, distanceM;
   SGGeodesy::inverse(currentPos, wpPos, courseDeg, az2, distanceM);
 
@@ -532,7 +568,7 @@ void FGRouteMgr::update( double dt )
   
   FlightPlan::Leg* nextLeg = _plan->nextLeg();
   if (nextLeg) {
-    wpPos = path.positionForIndex(_plan->currentIndex() + 1);
+    wpPos = _routePath->positionForIndex(_plan->currentIndex() + 1);
     SGGeodesy::inverse(currentPos, wpPos, courseDeg, az2, distanceM);
 
     wp1->setDoubleValue("dist", distanceM * SG_METER_TO_NM);
@@ -554,8 +590,9 @@ void FGRouteMgr::update( double dt )
 
 void FGRouteMgr::clearRoute()
 {
+  _routePath.reset();
   if (_plan) {
-    _plan->clear();
+      _plan->clearLegs();
   }
 }
 
@@ -638,8 +675,9 @@ void FGRouteMgr::waypointsChanged()
 // mirror internal route to the property system for inspection by other subsystems
 void FGRouteMgr::update_mirror()
 {
+    _routePath.reset(); // wipe this so we re-compute on next update()
   mirror->removeChildren("wp");
-  NewGUI * gui = (NewGUI *)globals->get_subsystem("gui");
+  auto gui = globals->get_subsystem<NewGUI>();
   FGDialog* rmDlg = gui ? gui->getDialog("route-manager") : NULL;
 
   if (!_plan) {
@@ -701,7 +739,7 @@ void FGRouteMgr::update_mirror()
   
   // set number as listener attachment point
   mirror->setIntValue("num", _plan->numLegs());
-    
+
   if (rmDlg) {
     rmDlg->updateValues();
   }
@@ -719,54 +757,62 @@ void FGRouteMgr::update_mirror()
 //
 void FGRouteMgr::InputListener::valueChanged(SGPropertyNode *prop)
 {
-    const char *s = prop->getStringValue();
-    if (strlen(s) == 0) {
-      return;
+    const auto input = su::uppercase(su::strip(prop->getStringValue()));
+    if (input.empty()) {
+        return;
     }
-    
-    if (!strcmp(s, "@CLEAR"))
+
+    if (input == "@CLEAR") {
         mgr->clearRoute();
-    else if (!strcmp(s, "@ACTIVATE"))
+    } else if (input == "@ACTIVATE") {
         mgr->activate();
-    else if (!strcmp(s, "@LOAD")) {
-      SGPath path = SGPath::fromUtf8(mgr->_pathNode->getStringValue());
-      mgr->loadRoute(path);
-    } else if (!strcmp(s, "@SAVE")) {
-      SGPath path = SGPath::fromUtf8(mgr->_pathNode->getStringValue());
-      SGPath authorizedPath = fgValidatePath(path, true /* write */);
+    } else if (input == "@LOAD") {
+        SGPath path = SGPath::fromUtf8(mgr->_pathNode->getStringValue());
+        mgr->loadRoute(path);
+    } else if (input == "@SAVE") {
+        SGPath path = SGPath::fromUtf8(mgr->_pathNode->getStringValue());
+        const SGPath authorizedPath = SGPath(path).validate(true /* write */);
 
-      if (!authorizedPath.isNull()) {
-        mgr->saveRoute(authorizedPath);
-      } else {
-        std::string msg =
-          "The route manager was asked to write the flightplan to '" +
-          path.utf8Str() + "', but this path is not authorized for writing. " +
-          "Please choose another location, for instance in the $FG_HOME/Export "
-          "folder (" + (globals->get_fg_home() / "Export").utf8Str() + ").";
+        if (!authorizedPath.isNull()) {
+            mgr->saveRoute(authorizedPath);
+        } else {
+            std::string msg =
+                "The route manager was asked to write the flightplan to '" +
+                path.utf8Str() + "', but this path is not authorized for writing. " +
+                "Please choose another location, for instance in the $FG_HOME/Export "
+                "folder (" +
+                (globals->get_fg_home() / "Export").utf8Str() + ").";
 
-        SG_LOG(SG_AUTOPILOT, SG_ALERT, msg);
-        modalMessageBox("FlightGear", "Unable to write to the specified file",
-                        msg);
-      }
-    } else if (!strcmp(s, "@NEXT")) {
-      mgr->jumpToIndex(mgr->currentIndex() + 1);
-    } else if (!strcmp(s, "@PREVIOUS")) {
-      mgr->jumpToIndex(mgr->currentIndex() - 1);
-    } else if (!strncmp(s, "@JUMP", 5)) {
-      mgr->jumpToIndex(atoi(s + 5));
-    } else if (!strncmp(s, "@DELETE", 7))
-        mgr->removeLegAtIndex(atoi(s + 7));
-    else if (!strncmp(s, "@INSERT", 7)) {
-        char *r;
-        int pos = strtol(s + 7, &r, 10);
-        if (*r++ != ':')
+            SG_LOG(SG_AUTOPILOT, SG_ALERT, msg);
+            modalMessageBox("FlightGear", "Unable to write to the specified file",
+                            msg);
+        }
+    } else if (input == "@NEXT") {
+        mgr->jumpToIndex(mgr->currentIndex() + 1);
+    } else if (input == "@PREVIOUS") {
+        mgr->jumpToIndex(mgr->currentIndex() - 1);
+    } else if (su::starts_with(input, "@JUMP")) {
+        const int index = stoi(input.substr(5));
+        mgr->jumpToIndex(index);
+    } else if (su::starts_with(input, "@DELETE")) {
+        const int index = stoi(input.substr(7));
+        mgr->removeLegAtIndex(index);
+    } else if (su::starts_with(input, "@INSERT")) {
+        size_t pos = 0;
+        const auto arg = input.substr(7); // input without prefix
+        const int index = stoi(arg, &pos, 10);
+        if (arg.at(pos) != ':') {
+            SG_LOG(SG_AUTOPILOT, SG_WARN, "@INSERT: couldn't parse index from:'" << input << "'");
             return;
-        while (isspace(*r))
-            r++;
-        if (*r)
-            mgr->flightPlan()->insertWayptAtIndex(mgr->waypointFromString(r), pos);
-    } else
-      mgr->flightPlan()->insertWayptAtIndex(mgr->waypointFromString(s), -1);
+        }
+
+        const auto wpString = su::strip(arg.substr(pos + 1));
+        const auto newWp = mgr->waypointFromString(wpString, index);
+        mgr->flightPlan()->insertWayptAtIndex(newWp, index);
+    } else {
+        const auto newWp = mgr->waypointFromString(input, -1);
+        mgr->flightPlan()->insertWayptAtIndex(newWp, -1);
+    }
 }
 
 bool FGRouteMgr::activate()
@@ -802,6 +848,14 @@ void FGRouteMgr::jumpToIndex(int index)
   if (!_plan) {
     return;
   }
+
+  // this method is tied() to current-wp property, but FlightPlan::setCurrentIndex
+  // will throw on invalid input, so guard against invalid values here.
+  // See Sentry FLIGHTGEAR-71
+  if ((index < -1) || (index >= _plan->numLegs())) {
+    SG_LOG(SG_AUTOPILOT, SG_WARN, "FGRouteMgr::jumpToIndex: ignoring invalid index:" << index);
+    return;
+  }
   
   _plan->setCurrentIndex(index);
 }
@@ -866,7 +920,7 @@ void FGRouteMgr::setDepartureICAO(const std::string& aIdent)
     }
     
   if (aIdent.length() < 3) {
-    _plan->setDeparture((FGAirport*) NULL);
+    _plan->setDeparture((FGAirport*) nullptr);
   } else {
     _plan->setDeparture(FGAirport::findByIdent(aIdent));
   }
@@ -936,7 +990,7 @@ flightgear::SID* createDefaultSID(FGRunway* aRunway, double enrouteCourse)
     wpts.push_back(w);
   }
   
-  BOOST_FOREACH(Waypt* w, wpts) {
+  for (Waypt* w : wpts) {
     w->setFlag(WPT_DEPARTURE);
     w->setFlag(WPT_GENERATED);
   }
@@ -1094,7 +1148,7 @@ flightgear::Approach* createDefaultApproach(FGRunway* aRunway, double aEnrouteCo
   w->setAltitude(thresholdElevFt + approachHeightFt, RESTRICT_AT);
   wpts.push_back(w);
     
-  BOOST_FOREACH(Waypt* w, wpts) {
+  for (Waypt* w : wpts) {
     w->setFlag(WPT_APPROACH);
     w->setFlag(WPT_GENERATED);
   }
@@ -1120,7 +1174,7 @@ void FGRouteMgr::setApproach(const std::string& aIdent)
   }
   
   if (!apt || aIdent.empty()) {
-    _plan->setApproach(NULL);
+      _plan->setApproach(static_cast<Approach*>(nullptr));
   } else {
     _plan->setApproach(apt->findApproachWithIdent(aIdent));
   }
@@ -1161,9 +1215,9 @@ void FGRouteMgr::setSTAR(const std::string& aIdent)
   }
 }
 
-WayptRef FGRouteMgr::waypointFromString(const std::string& target)
+WayptRef FGRouteMgr::waypointFromString(const std::string& target, int insertPosition)
 {
-  return _plan->waypointFromString(target);
+    return _plan->waypointFromString(target, _plan->vicinityForInsertIndex(insertPosition));
 }
 
 double FGRouteMgr::getDepartureFieldElevation() const
@@ -1184,6 +1238,95 @@ double FGRouteMgr::getDestinationFieldElevation() const
   return _plan->destinationAirport()->elevation();
 }
 
+int FGRouteMgr::getCruiseAltitudeFt() const
+{
+    if (!_plan)
+        return 0;
+
+    return _plan->cruiseAltitudeFt();
+}
+
+void FGRouteMgr::setCruiseAltitudeFt(int ft)
+{
+    if (!_plan)
+        return;
+
+    _plan->setCruiseAltitudeFt(ft);
+}
+
+int FGRouteMgr::getCruiseFlightLevel() const
+{
+    if (!_plan)
+        return 0;
+
+    return _plan->cruiseFlightLevel();
+}
+
+void FGRouteMgr::setCruiseFlightLevel(int fl)
+{
+    if (!_plan)
+        return;
+
+    _plan->setCruiseFlightLevel(fl);
+}
+
+int FGRouteMgr::getCruiseSpeedKnots() const
+{
+    if (!_plan)
+        return 0;
+
+    return _plan->cruiseSpeedKnots();
+}
+
+void FGRouteMgr::setCruiseSpeedKnots(int kts)
+{
+    if (!_plan)
+        return;
+
+    _plan->setCruiseSpeedKnots(kts);
+}
+
+double FGRouteMgr::getCruiseSpeedMach() const
+{
+    if (!_plan)
+        return 0.0;
+
+    return _plan->cruiseSpeedMach();
+}
+
+void FGRouteMgr::setCruiseSpeedMach(double m)
+{
+    if (!_plan)
+        return;
+
+    _plan->setCruiseSpeedMach(m);
+}
+
+string FGRouteMgr::getAlternate() const
+{
+    if (!_plan || !_plan->alternate())
+        return {};
+
+    return _plan->alternate()->ident();
+}
+
+std::string FGRouteMgr::getAlternateName() const
+{
+    if (!_plan || !_plan->alternate())
+        return {};
+
+    return _plan->alternate()->name();
+}
+
+void FGRouteMgr::setAlternate(const string &icao)
+{
+    if (!_plan)
+        return;
+
+    _plan->setAlternate(FGAirport::findByIdent(icao));
+    alternate->fireValueChanged();
+}
+
 SGPropertyNode_ptr FGRouteMgr::wayptNodeAtIndex(int index) const
 {
   if ((index < 0) || (index >= numWaypts())) {
@@ -1209,9 +1352,12 @@ bool FGRouteMgr::commandDefineUserWaypoint(const SGPropertyNode * arg, SGPropert
         return false;
     }
 
+    const bool temporary = arg->getBoolValue("temporary");
     SGGeod pos(SGGeod::fromDeg(arg->getDoubleValue("longitude-deg"),
                                arg->getDoubleValue("latitude-deg")));
-    FGPositioned::createUserWaypoint(ident, pos);
+    const auto name = arg->getStringValue("name");
+    FGPositioned::createWaypoint(FGPositioned::WAYPOINT, ident, pos,
+                                 temporary, name);
     return true;
 }
 
@@ -1222,7 +1368,19 @@ bool FGRouteMgr::commandDeleteUserWaypoint(const SGPropertyNode * arg, SGPropert
         SG_LOG(SG_AUTOPILOT, SG_WARN, "missing ident deleting user waypoint");
         return false;
     }
-    
-    return FGPositioned::deleteUserWaypoint(ident);
+
+    FGPositioned::TypeFilter f(FGPositioned::WAYPOINT);
+    auto existing = FGPositioned::findFirstWithIdent(ident, &f);
+    if (!existing) {
+      SG_LOG(SG_AUTOPILOT, SG_WARN, "no user waypoint with ident:" << ident);
+      return false;
+    }
+
+    return FGPositioned::deleteWaypoint(existing);
 }
 
+
+// Register the subsystem.
+SGSubsystemMgr::Registrant<FGRouteMgr> registrantFGRouteMgr(
+    SGSubsystemMgr::GENERAL,
+    {{"gui", SGSubsystemMgr::Dependency::HARD}});
