@@ -19,13 +19,15 @@
 #include "FGCanvasSystemAdapter.hxx"
 
 #include <Main/globals.hxx>
-#include <Main/util.hxx>
 #include <Network/HTTPClient.hxx>
 #include <Viewer/renderer.hxx>
 
+#include <osg/Version>
 #include <osgDB/ReadFile>
 #include <stdexcept>
 
+#include <simgear/scene/util/SGReaderWriterOptions.hxx>
+
 namespace canvas
 {
   //----------------------------------------------------------------------------
@@ -51,7 +53,7 @@ namespace canvas
       "canvas::Text: using font file " << path
     );
 
-    simgear::canvas::FontPtr font = osgText::readFontFile(path.local8BitStr());
+    simgear::canvas::FontPtr font = osgText::readFontFile(path.utf8Str());
     if( !font )
       SG_LOG
       (
@@ -79,12 +81,15 @@ namespace canvas
   //----------------------------------------------------------------------------
   osg::ref_ptr<osg::Image> FGCanvasSystemAdapter::getImage(const std::string& path) const
   {
-      SGPath p(SGPath::fromUtf8(path));
+    SGPath p(SGPath::fromUtf8(path));
+    osg::ref_ptr<simgear::SGReaderWriterOptions> localReaderWriterOptions = simgear::SGReaderWriterOptions::copyOrCreate(osgDB::Registry::instance()->getOptions());
+    localReaderWriterOptions->setLoadOriginHint(simgear::SGReaderWriterOptions::LoadOriginHint::ORIGIN_CANVAS);
+
     if( p.isAbsolute() )
     {
-      SGPath valid_path = fgValidatePath(p, false);
+      const SGPath valid_path = SGPath(p).validate(false);
       if( !valid_path.isNull() )
-        return osgDB::readImageFile(valid_path.local8BitStr());
+          return osgDB::readRefImageFile(valid_path.utf8Str(), localReaderWriterOptions);
 
       SG_LOG(SG_IO, SG_ALERT, "canvas::Image: reading '" << path << "' denied");
     }
@@ -92,7 +97,7 @@ namespace canvas
     {
       SGPath tpath = globals->resolve_resource_path(path);
       if( !tpath.isNull() )
-        return osgDB::readRefImageFile(tpath.local8BitStr());
+          return osgDB::readRefImageFile(tpath.utf8Str(), localReaderWriterOptions);
 
       SG_LOG(SG_IO, SG_ALERT, "canvas::Image: No such image: '" << path << "'");
     }
@@ -104,13 +109,13 @@ namespace canvas
   SGSubsystem*
   FGCanvasSystemAdapter::getSubsystem(const std::string& name) const
   {
-    return globals->get_subsystem(name.c_str());
+    return globals->get_subsystem_mgr()->get_subsystem(name.c_str());
   }
 
   //----------------------------------------------------------------------------
   simgear::HTTP::Client* FGCanvasSystemAdapter::getHTTPClient() const
   {
-    FGHTTPClient* http = globals->get_subsystem<FGHTTPClient>();
+    auto http = globals->get_subsystem<FGHTTPClient>();
 
     if( http )
       return http->client();
@@ -118,7 +123,7 @@ namespace canvas
     SG_LOG( SG_IO,
             SG_ALERT,
             "FGCanvasSystemAdapter: Failed to get HTTP subsystem" );
-    return 0;
+    return nullptr;
   }
 
 }
