@@ -19,15 +19,11 @@
 //
 //
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+#include "config.h"
 
 #include "NavDisplay.hxx"
 
 #include <cassert>
-#include <boost/foreach.hpp>
-#include <boost/algorithm/string/case_conv.hpp>
 #include <algorithm>
 
 #include <osg/Array>
@@ -45,6 +41,7 @@
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/misc/strutils.hxx>
 #include <simgear/math/sg_geodesy.hxx>
+#include <simgear/scene/util/OsgUtils.hxx>
 
 #include <cstdio>
 #include <sstream>
@@ -74,6 +71,8 @@ using std::string;
 #include <Airports/runways.hxx>
 #include "od_gauge.hxx"
 
+using namespace flightgear;
+
 static const char *DEFAULT_FONT = "typewriter.txf";
 
 static
@@ -92,37 +91,6 @@ static osg::Vec4 readColor(SGPropertyNode* colorNode, const osg::Vec4& c)
     return result;
 }
 
-static osgText::Text::AlignmentType readAlignment(const std::string& t)
-{
-    if (t == "left-top") {
-        return osgText::Text::LEFT_TOP;
-    } else if (t == "left-center") {
-        return osgText::Text::LEFT_CENTER;
-    } else if (t == "left-bottom") {
-        return osgText::Text::LEFT_BOTTOM;
-    } else if (t == "center-top") {
-        return osgText::Text::CENTER_TOP;
-    } else if (t == "center-center") {
-        return osgText::Text::CENTER_CENTER;
-    } else if (t == "center-bottom") {
-        return osgText::Text::CENTER_BOTTOM;
-    } else if (t == "right-top") {
-        return osgText::Text::RIGHT_TOP;
-    } else if (t == "right-center") {
-        return osgText::Text::RIGHT_CENTER;
-    } else if (t == "right-bottom") {
-        return osgText::Text::RIGHT_BOTTOM;
-    } else if (t == "left-baseline") {
-        return osgText::Text::LEFT_BASE_LINE;
-    } else if (t == "center-baseline") {
-        return osgText::Text::CENTER_BASE_LINE;
-    } else if (t == "right-baseline") {
-        return osgText::Text::RIGHT_BASE_LINE;
-    }
-    
-    return osgText::Text::BASE_LINE;
-}
-
 static string formatPropertyValue(SGPropertyNode* nd, const string& format)
 {
     assert(nd);
@@ -133,7 +101,7 @@ static string formatPropertyValue(SGPropertyNode* nd, const string& format)
     }
     
     if (format.find('s') != string::npos) {
-        ::snprintf(buf, 512, format.c_str(), nd->getStringValue());
+        ::snprintf(buf, 512, format.c_str(), nd->getStringValue().c_str());
         return buf;
     }
     
@@ -155,7 +123,7 @@ public:
         _nd(nd)
     {}
     
-    virtual void valueChanged (SGPropertyNode * prop)
+    void valueChanged (SGPropertyNode * prop) override
     {
         _nd->invalidatePositionedCache();
     }
@@ -170,7 +138,7 @@ public:
     _nd(nd)
   {}
   
-  virtual void valueChanged (SGPropertyNode * prop)
+  void valueChanged (SGPropertyNode * prop) override
   {
     _nd->forceUpdate();
   }
@@ -194,8 +162,7 @@ public:
             return false;
         }
         
-        type = node->getStringValue("type");
-        boost::to_lower(type);
+        type = simgear::strutils::lowercase(node->getStringValue("type"));
         SGPropertyNode* enableNode = node->getChild("enable");
         if (enableNode) { 
             enable.reset(sgReadCondition(fgGetNode("/"), enableNode));
@@ -225,13 +192,13 @@ public:
     
     bool matches(const string_set& states) const
     {
-        BOOST_FOREACH(const string& s, required_states) {
+        for (auto s : required_states) {
             if (states.count(s) == 0) {
                 return false;
             }
         }
         
-        BOOST_FOREACH(const string& s, excluded_states) {
+        for (auto s : excluded_states) {
             if (states.count(s) != 0) {
                 return false;
             }
@@ -259,7 +226,7 @@ public:
   // record instances for limiting by count
     int instanceCount;
 private:
-    SymbolDef* definition;
+    SymbolDef* definition = nullptr;
     
     std::unique_ptr<SGCondition> enable;
     string_set required_states;
@@ -309,7 +276,7 @@ public:
         hasText = false;
         if (node->hasChild("text")) {
             hasText = true;
-            alignment = readAlignment(node->getStringValue("text-align"));
+            alignment = simgear::osgutils::mapAlignment(node->getStringValue("text-align"));
             textTemplate = node->getStringValue("text");
             textOffset.x() = node->getFloatValue("text-offset-x", 0);
             textOffset.y() = node->getFloatValue("text-offset-y", 0);
@@ -462,30 +429,30 @@ NavDisplay::NavDisplay(SGPropertyNode *node) :
     SGPropertyNode* symbol;
 
     map<string, SymbolDef*> definitionDict;
-    for (int i = 0; (symbol = symbolsNode->getChild("symbol", i)) != NULL; ++i) {
+    for (int i = 0; (symbol = symbolsNode->getChild("symbol", i)) != nullptr; ++i) {
         SymbolDef* def = new SymbolDef;
         if (!def->initFromNode(symbol, this)) {
           delete def;
           continue;
         }
         
-        const char* id = symbol->getStringValue("id");
-        if (id && strlen(id)) {
+        string id = symbol->getStringValue("id", "");
+        if (!id.empty()) {
             definitionDict[id] = def;
         }
         
         _definitions.push_back(def);
     } // of symbol definition parsing
     
-    BOOST_FOREACH(SGPropertyNode* rule, symbolsNode->getChildren("rule")) {
+    for (SGPropertyNode* rule : symbolsNode->getChildren("rule")) {
         SymbolRule* r = new SymbolRule;
         if (!r->initFromNode(rule, this)) {
             delete r;
             continue;
         }
         
-        const char* id = rule->getStringValue("symbol");
-        if (id && strlen(id) && (definitionDict.find(id) != definitionDict.end())) {
+        string id = rule->getStringValue("symbol", "");
+        if (!id.empty() && (definitionDict.find(id) != definitionDict.end())) {
             r->setDefinition(definitionDict[id]);
         } else {
             SG_LOG(SG_INSTR, SG_WARN, "symbol rule has missing/unknown definition id:" << id);
@@ -542,12 +509,12 @@ NavDisplay::init ()
     }
   
     // no mipmap or else alpha will mix with pixels on the border of shapes, ruining the effect
-    _symbolTexture = SGLoadTexture2D(tpath, NULL, false, false);
+    _symbolTexture = SGLoadTexture2D(tpath, nullptr, false, false);
 
     _odg = new FGODGauge;
     _odg->setSize(_Instrument->getIntValue("texture-size", 512));
 
-    _route = static_cast<FGRouteMgr*>(globals->get_subsystem("route-manager"));
+    _route = globals->get_subsystem<FGRouteMgr>();
     
     _navRadio1Node = fgGetNode("/instrumentation/nav[0]", true);
     _navRadio2Node = fgGetNode("/instrumentation/nav[1]", true);
@@ -717,18 +684,18 @@ NavDisplay::update (double delta_time_sec)
   _texCoords->clear();
   _textGeode->removeDrawables(0, _textGeode->getNumDrawables());
   
-  BOOST_FOREACH(SymbolInstance* si, _symbols) {
+  for (SymbolInstance* si : _symbols) {
       delete si;
   }
   _symbols.clear();
   
-  BOOST_FOREACH(SymbolDef* d, _definitions) {
+  for (SymbolDef* d : _definitions) {
     d->instanceCount = 0;
     d->textEnabled = d->textEnable.get() ? d->textEnable->test() : true;
   }
   
   bool enableChanged = false;
-  BOOST_FOREACH(SymbolRule* r, _rules) {
+  for (SymbolRule* r : _rules) {
       enableChanged |= r->checkEnabled();
   }
   
@@ -780,15 +747,20 @@ NavDisplay::updateFont()
     }
 
     osg::ref_ptr<osgDB::ReaderWriter::Options> fontOptions = new osgDB::ReaderWriter::Options("monochrome");
-    osg::ref_ptr<osgText::Font> font = osgText::readFontFile(tpath.local8BitStr(), fontOptions.get());
+    osg::ref_ptr<osgText::Font> font = osgText::readFontFile(tpath.utf8Str(), fontOptions.get());
 
     if (font != 0) {
         _font = font;
         _font->setMinFilterHint(osg::Texture::NEAREST);
         _font->setMagFilterHint(osg::Texture::NEAREST);
+
+#if OSG_VERSION_LESS_THAN(3,5,6)
         _font->setGlyphImageMargin(0);
         _font->setGlyphImageMarginRatio(0);
-    }
+#else
+//OSG:TODO: need 3.5.7 version of this.
+#endif
+}
 }
 
 void NavDisplay::addSymbolToScene(SymbolInstance* sym)
@@ -921,7 +893,7 @@ public:
 void NavDisplay::addSymbolsToScene()
 {
     std::sort(_symbols.begin(), _symbols.end(), OrderByZ());
-    BOOST_FOREACH(SymbolInstance* sym, _symbols) {
+    for (SymbolInstance* sym : _symbols) {
         addSymbolToScene(sym);
     }
 }
@@ -956,7 +928,7 @@ public:
   
     double minRunwayLengthFt;
   
-    virtual bool pass(FGPositioned* aPos) const
+    bool pass(FGPositioned* aPos) const override
     {
         if (aPos->type() == FGPositioned::FIX) {
             string ident(aPos->ident());
@@ -977,11 +949,13 @@ public:
         return _owner->isPositionedShown(aPos);
     }
 
-    virtual FGPositioned::Type minType() const {
+    FGPositioned::Type minType() const override
+    {
         return FGPositioned::AIRPORT;
     }
 
-    virtual FGPositioned::Type maxType() const {
+    FGPositioned::Type maxType() const override
+    {
         return FGPositioned::OBSTACLE;
     }
   
@@ -1008,8 +982,7 @@ void NavDisplay::findItems()
     
   // sort by distance from pos, so symbol limits are accurate
     FGPositioned::sortByRange(_itemsInRange, _pos);
-  
-    BOOST_FOREACH(FGPositioned* pos, _itemsInRange) {
+    for (FGPositioned* pos : _itemsInRange) {
         foundPositionedItem(pos);
     }
 }
@@ -1061,7 +1034,7 @@ void NavDisplay::processRoute()
         computeWayptPropsAndHeading(wpt, g, vars, heading);
 
         osg::Vec2 projected = projectGeod(g);
-        BOOST_FOREACH(SymbolRule* r, rules) {
+        for (SymbolRule* r : rules) {
             addSymbolInstance(projected, heading, r->getDefinition(), vars);
             
             if (r->getDefinition()->drawRouteLeg) {
@@ -1104,7 +1077,7 @@ FGNavRecord* NavDisplay::processNavRadio(const SGPropertyNode_ptr& radio)
   FGNavRecord* nav = FGNavList::findByFreq(mhz, _pos, FGNavList::navFilter());
     if (!nav || (nav->ident() != radio->getStringValue("nav-id"))) {
         // station was not found
-        return NULL;
+        return nullptr;
     }
     
     
@@ -1113,7 +1086,7 @@ FGNavRecord* NavDisplay::processNavRadio(const SGPropertyNode_ptr& radio)
 
 bool NavDisplay::anyRuleForType(const string& type) const
 {
-    BOOST_FOREACH(SymbolRule* r, _rules) {
+    for (SymbolRule* r : _rules) {
         if (!r->enabled) {
             continue;
         }
@@ -1128,7 +1101,7 @@ bool NavDisplay::anyRuleForType(const string& type) const
 
 void NavDisplay::findRules(const string& type, const string_set& states, SymbolRuleVector& rules)
 {
-    BOOST_FOREACH(SymbolRule* candidate, _rules) {
+    for (SymbolRule* candidate : _rules) {
         if (!candidate->enabled || (candidate->type != type)) {
             continue;
         }
@@ -1148,8 +1121,7 @@ bool NavDisplay::isPositionedShown(FGPositioned* pos)
 
 void NavDisplay::isPositionedShownInner(FGPositioned* pos, SymbolRuleVector& rules)
 {
-  string type = FGPositioned::nameForType(pos->type());
-  boost::to_lower(type);
+  string type = simgear::strutils::lowercase(FGPositioned::nameForType(pos->type()));
   if (!anyRuleForType(type)) {
     return; // not diplayed at all, we're done
   }
@@ -1182,7 +1154,7 @@ void NavDisplay::foundPositionedItem(FGPositioned* pos)
         projected = projectGeod(rwy->threshold());
     }
     
-    BOOST_FOREACH(SymbolRule* r, rules) {
+    for (SymbolRule* r : rules) {
         SymbolInstance* ins = addSymbolInstance(projected, heading, r->getDefinition(), vars);
         if ((ins)&&(pos->type() == FGPositioned::RUNWAY)) {
             FGRunway* rwy = (FGRunway*) pos;
@@ -1303,11 +1275,11 @@ void NavDisplay::computePositionedState(FGPositioned* pos, string_set& states)
 static string mapAINodeToType(SGPropertyNode* model)
 {
   // assume all multiplayer items are aircraft for the moment. Not ideal.
-  if (!strcmp(model->getName(), "multiplayer")) {
+  if (model->getNameString() == "multiplayer") {
     return "ai-aircraft";
   }
   
-  return string("ai-") + model->getName();
+  return string("ai-") + model->getNameString();
 }
 
 void NavDisplay::processAI()
@@ -1338,7 +1310,7 @@ void NavDisplay::processAI()
         model->setIntValue("flight-level", fl * 10);
                                             
         osg::Vec2 projected = projectGeod(aiModelPos);
-        BOOST_FOREACH(SymbolRule* r, rules) {
+        for (SymbolRule* r : rules) {
             addSymbolInstance(projected, heading, r->getDefinition(), (SGPropertyNode*) model);
         }
     } // of ai models iteration
@@ -1367,11 +1339,11 @@ void NavDisplay::computeAIStates(const SGPropertyNode* ai, string_set& states)
 SymbolInstance* NavDisplay::addSymbolInstance(const osg::Vec2& proj, double heading, SymbolDef* def, SGPropertyNode* vars)
 {
     if (isProjectedClipped(proj)) {
-        return NULL;
+        return nullptr;
     }
     
     if ((def->limitCount > 0) && (def->instanceCount >= def->limitCount)) {
-      return NULL;
+      return nullptr;
     }
   
     ++def->instanceCount;
@@ -1392,7 +1364,7 @@ bool NavDisplay::isProjectedClipped(const osg::Vec2& projected) const
 void NavDisplay::addTestSymbol(const std::string& type, const std::string& states, const SGGeod& pos, double heading, SGPropertyNode* vars)
 {
   string_set stateSet;
-  BOOST_FOREACH(std::string s, simgear::strutils::split(states, ",")) {
+  for (auto s : simgear::strutils::split(states, ",")) {
     stateSet.insert(s);
   }
   
@@ -1403,7 +1375,7 @@ void NavDisplay::addTestSymbol(const std::string& type, const std::string& state
   }
     
   osg::Vec2 projected = projectGeod(pos);
-  BOOST_FOREACH(SymbolRule* r, rules) {
+  for (SymbolRule* r : rules) {
     addSymbolInstance(projected, heading, r->getDefinition(), vars);
   }
 }
@@ -1416,28 +1388,28 @@ void NavDisplay::addTestSymbols()
   double dummy;
   SGGeodesy::direct(_pos, 45.0, 20.0 * SG_NM_TO_METER, a1, dummy);
   
-  addTestSymbol("airport", "", a1, 0.0, NULL);
+  addTestSymbol("airport", "", a1, 0.0, nullptr);
   
   SGGeodesy::direct(_pos, 95.0, 40.0 * SG_NM_TO_METER, a1, dummy);
   
-  addTestSymbol("vor", "", a1, 0.0, NULL);
+  addTestSymbol("vor", "", a1, 0.0, nullptr);
   
   SGGeodesy::direct(_pos, 120, 80.0 * SG_NM_TO_METER, a1, dummy);
   
-  addTestSymbol("airport", "destination", a1, 0.0, NULL);
+  addTestSymbol("airport", "destination", a1, 0.0, nullptr);
   
   SGGeodesy::direct(_pos, 80.0, 20.0 * SG_NM_TO_METER, a1, dummy);  
-  addTestSymbol("fix", "", a1, 0.0, NULL);
+  addTestSymbol("fix", "", a1, 0.0, nullptr);
 
   
   SGGeodesy::direct(_pos, 140.0, 20.0 * SG_NM_TO_METER, a1, dummy);  
-  addTestSymbol("fix", "", a1, 0.0, NULL);
+  addTestSymbol("fix", "", a1, 0.0, nullptr);
   
   SGGeodesy::direct(_pos, 110.0, 10.0 * SG_NM_TO_METER, a1, dummy);  
-  addTestSymbol("fix", "", a1, 0.0, NULL);
+  addTestSymbol("fix", "", a1, 0.0, nullptr);
   
   SGGeodesy::direct(_pos, 110.0, 5.0 * SG_NM_TO_METER, a1, dummy);  
-  addTestSymbol("fix", "", a1, 0.0, NULL);
+  addTestSymbol("fix", "", a1, 0.0, nullptr);
 }
 
 void NavDisplay::addRule(SymbolRule* r)
@@ -1447,7 +1419,7 @@ void NavDisplay::addRule(SymbolRule* r)
 
 void NavDisplay::computeCustomSymbolStates(const SGPropertyNode* sym, string_set& states)
 {
-  BOOST_FOREACH(SGPropertyNode* st, sym->getChildren("state")) {
+  for (SGPropertyNode* st : sym->getChildren("state")) {
     states.insert(st->getStringValue());
   }
 }
@@ -1462,7 +1434,7 @@ void NavDisplay::processCustomSymbols()
     string_set ss;
     computeCustomSymbolStates(symNode, ss);
     SymbolRuleVector rules;
-    findRules(symNode->getName(), ss, rules);
+    findRules(symNode->getNameString(), ss, rules);
     if (rules.empty()) {
       return; // no rules matched, we can skip this item
     }
@@ -1474,10 +1446,16 @@ void NavDisplay::processCustomSymbols()
  
     
     osg::Vec2 projected = projectGeod(pos);
-    BOOST_FOREACH(SymbolRule* r, rules) {
+    for (SymbolRule* r : rules) {
       addSymbolInstance(projected, heading, r->getDefinition(), symNode);
     }
   } // of custom symbols iteration
 }
 
 
+// Register the subsystem.
+#if 0
+SGSubsystemMgr::Registrant<NavDisplay> registrantNavDisplay(
+    SGSubsystemMgr::GENERAL,
+    {{"route-manager", SGSubsystemMgr::Dependency::HARD}});
+#endif
