@@ -17,9 +17,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #include "cockpitDisplayManager.hxx"
 
@@ -27,8 +25,7 @@
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/sg_inlines.h>
 #include <simgear/props/props_io.hxx>
-
-#include <boost/foreach.hpp>
+#include <simgear/debug/ErrorReportingCallback.hxx>
 
 #include <Main/fg_props.hxx>
 #include <Main/globals.hxx>
@@ -41,8 +38,8 @@
 namespace flightgear
 {
 
-CockpitDisplayManager::CockpitDisplayManager () 
-{    
+CockpitDisplayManager::CockpitDisplayManager ()
+{
 }
 
 CockpitDisplayManager::~CockpitDisplayManager ()
@@ -60,11 +57,16 @@ void CockpitDisplayManager::init()
   SGPropertyNode_ptr config_props = new SGPropertyNode;
   SGPropertyNode* path_n = fgGetNode("/sim/instrumentation/path");
   if (!path_n) {
-    SG_LOG(SG_COCKPIT, SG_WARN, "No instrumentation model specified for this model!");
     return;
   }
 
   SGPath config = globals->resolve_aircraft_path(path_n->getStringValue());
+  if (!config.exists()) {
+      simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::AircraftSystems,
+                             "CockpitDisplaysManager: Missing instrumentation file", config);
+    return;
+  }
+
   SG_LOG( SG_COCKPIT, SG_INFO, "Reading cockpit displays from " << config );
 
   try {
@@ -75,15 +77,11 @@ void CockpitDisplayManager::init()
                     "system specification file.  See earlier errors for details.");
     }
   } catch (const sg_exception& e) {
-    SG_LOG(SG_COCKPIT, SG_ALERT, "Failed to load instrumentation system model: "
-                    << config << ":" << e.getFormattedMessage() );
+      simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::AircraftSystems,
+                             "Failed to load cockpit displays system:" + e.getFormattedMessage(),
+                             e.getLocation());
   }
 
-  // bind() created instruments before init.
-  BOOST_FOREACH(std::string s, _displays) {
-      get_subsystem(s)->bind();
-  }
-  
   SGSubsystemGroup::init();
 }
 
@@ -91,7 +89,7 @@ bool CockpitDisplayManager::build (SGPropertyNode* config_props)
 {
     for ( int i = 0; i < config_props->nChildren(); ++i ) {
         SGPropertyNode *node = config_props->getChild(i);
-        std::string name = node->getName();
+        std::string name = node->getNameString();
 
         std::ostringstream subsystemname;
         subsystemname << "instrument-" << i << '-'
@@ -100,7 +98,7 @@ bool CockpitDisplayManager::build (SGPropertyNode* config_props)
         if (index > 0)
             subsystemname << '['<< index << ']';
         std::string id = subsystemname.str();
-      
+
         if ( name == "radar" ) {
             set_subsystem( id, new wxRadarBg ( node ) );
 
@@ -109,10 +107,10 @@ bool CockpitDisplayManager::build (SGPropertyNode* config_props)
 
         } else if ( name == "air-ground-radar" ) {
             set_subsystem( id, new agRadar( node ) );
-      
+
         } else if ( name == "navigation-display" ) {
             set_subsystem( id, new NavDisplay( node ) );
-            
+
         } else {
             // probably a regular instrument
             continue;
@@ -123,4 +121,9 @@ bool CockpitDisplayManager::build (SGPropertyNode* config_props)
     return true;
 }
 
+
+// Register the subsystem.
+SGSubsystemMgr::Registrant<CockpitDisplayManager> registrantCockpitDisplayManager(
+    SGSubsystemMgr::DISPLAY);
+
 } // of namespace flightgear
