@@ -1,26 +1,11 @@
-//  panel_io.cxx - I/O for 2D panel.
-//
-//  Written by David Megginson, started January 2000.
-//
-//  This program is free software; you can redistribute it and/or
-//  modify it under the terms of the GNU General Public License as
-//  published by the Free Software Foundation; either version 2 of the
-//  License, or (at your option) any later version.
-// 
-//  This program is distributed in the hope that it will be useful, but
-//  WITHOUT ANY WARRANTY; without even the implied warranty of
-//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//  General Public License for more details.
-//
-//  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-//  $Id$
+/*
+ * SPDX-FileName: panel_io.cxx
+ * SPDX-FileComment: I/O for 2D panel
+ * SPDX-FileCopyrightText: Written by David Megginson, started January 2000
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #include <string.h>		// for strcmp()
 
@@ -30,6 +15,7 @@
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/props/props_io.hxx>
+#include <simgear/debug/ErrorReportingCallback.hxx>
 
 #include <istream>
 #include <fstream>
@@ -37,6 +23,7 @@
 
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
+#include <Main/sentryIntegration.hxx>
 
 #include <GUI/gui.h>
 
@@ -52,7 +39,7 @@ using std::ifstream;
 using std::string;
 
 
-
+
 ////////////////////////////////////////////////////////////////////////
 // Read and construct a panel.
 //
@@ -115,7 +102,7 @@ readTexture (const SGPropertyNode * node)
 			     node->getFloatValue("y1"),
 			     node->getFloatValue("x2", 1.0),
 			     node->getFloatValue("y2", 1.0));
-    SG_LOG(SG_COCKPIT, SG_DEBUG, "Read texture " << node->getName());
+    SG_LOG(SG_COCKPIT, SG_DEBUG, "Read texture " << node->getNameString());
     return texture;
 }
 
@@ -123,7 +110,7 @@ readTexture (const SGPropertyNode * node)
 /**
  * Test for a condition in the current node.
  */
-
+
 ////////////////////////////////////////////////////////////////////////
 // Read a condition and use it if necessary.
 ////////////////////////////////////////////////////////////////////////
@@ -160,7 +147,7 @@ readConditions (SGConditional *component, const SGPropertyNode *node)
  * For the adjust action, it is possible to specify an increment
  * (use a negative number for a decrement), a minimum allowed value,
  * a maximum allowed value, and a flag to indicate whether the value
- * should freeze or wrap-around when it reachs the minimum or maximum.
+ * should freeze or wrap-around when it reaches the minimum or maximum.
  *
  * The action will be scaled automatically if the instrument is not
  * being drawn at its regular size.
@@ -168,17 +155,16 @@ readConditions (SGConditional *component, const SGPropertyNode *node)
 static FGPanelAction *
 readAction (const SGPropertyNode * node, float w_scale, float h_scale)
 {
-  unsigned int i, j;
-  SGPropertyNode *binding;
-  vector<SGPropertyNode_ptr>bindings = node->getChildren("binding");
+    auto propRoot = globals->get_props();
 
-  // button-less actions are fired initially
-  if (!node->hasValue("w") || !node->hasValue("h")) {
-    for (i = 0; i < bindings.size(); i++) {
-      SGBinding b(bindings[i], globals->get_props());
-      b.fire();
-    }
-    return 0;
+    // button-less actions are fired initially
+    if (!node->hasValue("w") || !node->hasValue("h")) {
+        for (auto bn : node->getChildren("binding")) {
+            SGBinding b(bn, propRoot);
+            b.fire();
+        }
+
+        return nullptr;
   }
 
   string name = node->getStringValue("name");
@@ -192,32 +178,9 @@ readAction (const SGPropertyNode * node, float w_scale, float h_scale)
 
   FGPanelAction * action = new FGPanelAction(button, x, y, w, h, repeatable);
 
-  SGPropertyNode * dest = fgGetNode("/sim/bindings/panel", true);
-
-  for (i = 0; i < bindings.size(); i++) {
-    SG_LOG(SG_INPUT, SG_BULK, "Reading binding "
-      << bindings[i]->getStringValue("command"));
-
-    j = 0;
-    while (dest->getChild("binding", j))
-      j++;
-
-    binding = dest->getChild("binding", j, true);
-    copyProperties(bindings[i], binding);
-    action->addBinding(new SGBinding(binding, globals->get_props()), 0);
-  }
-
+  action->setBindings(readBindingList(node->getChildren("binding"), propRoot), 0);
   if (node->hasChild("mod-up")) {
-    bindings = node->getChild("mod-up")->getChildren("binding");
-    for (i = 0; i < bindings.size(); i++) {
-      j = 0;
-      while (dest->getChild("binding", j))
-        j++;
-
-      binding = dest->getChild("binding", j, true);
-      copyProperties(bindings[i], binding);
-      action->addBinding(new SGBinding(binding, globals->get_props()), 1);
-    }
+      action->setBindings(readBindingList(node->getChild("mod-up")->getChildren("binding"), propRoot), 1);
   }
 
   readConditions(action, node);
@@ -255,7 +218,7 @@ readTransformation (const SGPropertyNode * node, float w_scale, float h_scale)
 {
   FGPanelTransformation * t = new FGPanelTransformation;
 
-  string name = node->getName();
+  string name = node->getNameString();
   string type = node->getStringValue("type");
   string propName = node->getStringValue("property", "");
   SGPropertyNode * target = 0;
@@ -454,7 +417,7 @@ readLayer (const SGPropertyNode * node, float w_scale, float h_scale)
     layer = new FGGroupLayer();
     for (int i = 0; i < node->nChildren(); i++) {
       const SGPropertyNode * child = node->getChild(i);
-      if (!strcmp(child->getName(), "layer"))
+      if (child->getNameString() == "layer")
 	((FGGroupLayer *)layer)->addLayer(readLayer(child, w_scale, h_scale));
     }
   }
@@ -483,12 +446,12 @@ readLayer (const SGPropertyNode * node, float w_scale, float h_scale)
       int nChunks = chunk_group->nChildren();
       for (int i = 0; i < nChunks; i++) {
 	const SGPropertyNode * node = chunk_group->getChild(i);
-	if (!strcmp(node->getName(), "chunk")) {
+	if (node->getNameString() == "chunk") {
 	  FGTextLayer::Chunk * chunk = readTextChunk(node);
 	  if (chunk != 0)
 	    tlayer->addChunk(chunk);
 	} else {
-	  SG_LOG( SG_COCKPIT, SG_INFO, "Skipping " << node->getName()
+	  SG_LOG( SG_COCKPIT, SG_INFO, "Skipping " << node->getNameString()
                   << " in chunks" );
 	}
       }
@@ -501,7 +464,7 @@ readLayer (const SGPropertyNode * node, float w_scale, float h_scale)
     layer = new FGSwitchLayer();
     for (int i = 0; i < node->nChildren(); i++) {
       const SGPropertyNode * child = node->getChild(i);
-      if (!strcmp(child->getName(), "layer"))
+      if (child->getNameString() == "layer")
 	((FGGroupLayer *)layer)->addLayer(readLayer(child, w_scale, h_scale));
     }
   }
@@ -542,12 +505,12 @@ readLayer (const SGPropertyNode * node, float w_scale, float h_scale)
     int nTransformations = trans_group->nChildren();
     for (int i = 0; i < nTransformations; i++) {
       const SGPropertyNode * node = trans_group->getChild(i);
-      if (!strcmp(node->getName(), "transformation")) {
+      if (node->getNameString() == "transformation") {
 	FGPanelTransformation * t = readTransformation(node, w_scale, h_scale);
 	if (t != 0)
 	  layer->addTransformation(t);
       } else {
-	SG_LOG( SG_COCKPIT, SG_INFO, "Skipping " << node->getName()
+        SG_LOG( SG_COCKPIT, SG_INFO, "Skipping " << node->getNameString()
                 << " in transformations" );
       }
     }
@@ -571,7 +534,7 @@ readLayer (const SGPropertyNode * node, float w_scale, float h_scale)
  * scaled automatically if the instrument is not at its preferred size.
  */
 static FGPanelInstrument *
-readInstrument (const SGPropertyNode * node)
+readInstrument (const SGPropertyNode * node, const SGPath& path)
 {
   const string name = node->getStringValue("name");
   int x = node->getIntValue("x", -1);
@@ -582,8 +545,8 @@ readInstrument (const SGPropertyNode * node)
   int h = node->getIntValue("h-base", -1);
 
   if (x == -1 || y == -1) {
-    SG_LOG( SG_COCKPIT, SG_ALERT,
-            "x and y positions must be specified and > 0" );
+      simgear::reportFailure(simgear::LoadFailure::Misconfigured, simgear::ErrorCode::AircraftSystems,
+                             "Panel x and y positions must be specified and > 0", path);
     return 0;
   }
 
@@ -611,12 +574,12 @@ readInstrument (const SGPropertyNode * node)
     int nActions = action_group->nChildren();
     for (int i = 0; i < nActions; i++) {
       const SGPropertyNode * node = action_group->getChild(i);
-      if (!strcmp(node->getName(), "action")) {
+      if (node->getNameString() == "action") {
 	FGPanelAction * action = readAction(node, w_scale, h_scale);
 	if (action != 0)
 	  instrument->addAction(action);
       } else {
-	SG_LOG( SG_COCKPIT, SG_INFO, "Skipping " << node->getName()
+        SG_LOG( SG_COCKPIT, SG_INFO, "Skipping " << node->getNameString()
                 << " in actions" );
       }
     }
@@ -630,12 +593,12 @@ readInstrument (const SGPropertyNode * node)
     int nLayers = layer_group->nChildren();
     for (int i = 0; i < nLayers; i++) {
       const SGPropertyNode * node = layer_group->getChild(i);
-      if (!strcmp(node->getName(), "layer")) {
+      if (node->getNameString() == "layer") {
 	FGInstrumentLayer * layer = readLayer(node, w_scale, h_scale);
 	if (layer != 0)
 	  instrument->addLayer(layer);
       } else {
-	SG_LOG( SG_COCKPIT, SG_INFO, "Skipping " << node->getName()
+        SG_LOG( SG_COCKPIT, SG_INFO, "Skipping " << node->getNameString()
                 << " in layers" );
       }
     }
@@ -650,10 +613,11 @@ readInstrument (const SGPropertyNode * node)
  * Construct the panel from a property tree.
  */
 FGPanel *
-readPanel (const SGPropertyNode * root)
+readPanel (const SGPropertyNode * root, const SGPath& path)
 {
   SG_LOG( SG_COCKPIT, SG_INFO, "Reading properties for panel " <<
           root->getStringValue("name", "[Unnamed Panel]") );
+  simgear::ErrorReportContext ec("2d-panel", path.utf8Str());
 
   FGPanel * panel = new FGPanel();
   panel->setWidth(root->getIntValue("w", 1024));
@@ -741,11 +705,11 @@ readPanel (const SGPropertyNode * root)
     int nInstruments = instrument_group->nChildren();
     for (int i = 0; i < nInstruments; i++) {
       const SGPropertyNode * node = instrument_group->getChild(i);
-      if (!strcmp(node->getName(), "instrument")) {
-        FGPanelInstrument * instrument = readInstrument(node);
+      if (node->getNameString() == "instrument") {
+        FGPanelInstrument * instrument = readInstrument(node, path);
         if (instrument != 0)
           panel->addInstrument(instrument);
-      } else if (!strcmp(node->getName(), "special-instrument")) {
+      } else if (node->getNameString() == "special-instrument") {
         //cout << "Special instrument found in instruments section!\n";
         const string name = node->getStringValue("name");
         if (name == "KLN89 GPS") {
@@ -779,19 +743,20 @@ readPanel (const SGPropertyNode * root)
           
           // Warning - hardwired size!!!
           RenderArea2D* instrument = new RenderArea2D(158, 40, 158, 40, x, y);
-          KLN89* gps = (KLN89*)globals->get_subsystem("kln89");
+          auto gps = globals->get_subsystem<KLN89>();
 		  if (gps == NULL) {
 			  gps = new KLN89(instrument);
-			  globals->add_subsystem("kln89", gps);
+			  globals->get_subsystem_mgr()->add("kln89", gps);
 		  }
 		  //gps->init();  // init seems to get called automagically.
 		  FGSpecialInstrument* gpsinst = new FGSpecialInstrument(gps);
           panel->addInstrument(gpsinst);
         } else {
-          SG_LOG( SG_COCKPIT, SG_WARN, "Unknown special instrument found" );
+          simgear::reportFailure(simgear::LoadFailure::Misconfigured, simgear::ErrorCode::AircraftSystems,
+             "Unknown special instrument found:" + name, path);
         }
       } else {
-        SG_LOG( SG_COCKPIT, SG_WARN, "Skipping " << node->getName()
+        SG_LOG( SG_COCKPIT, SG_WARN, "Skipping " << node->getNameString()
         << " in instruments section" );
       }
     }
@@ -824,10 +789,11 @@ fgReadPanel (istream &input)
   try {
     readProperties(input, &root);
   } catch (const sg_exception &e) {
-    guiErrorMessage("Error reading panel: ", e);
+          simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::AircraftSystems,
+                             "Failed to load aircraft panel:" + e.getFormattedMessage());
     return 0;
   }
-  return readPanel(&root);
+    return readPanel(&root, SGPath{});
 }
 
 
@@ -840,16 +806,25 @@ fgReadPanel (istream &input)
 FGPanel *
 fgReadPanel (const string &relative_path)
 {
+  simgear::ErrorReportContext ec("2d-panel", relative_path);
+
   SGPath path = globals->resolve_aircraft_path(relative_path);
   SGPropertyNode root;
-  
+
+  if (!path.exists()) {
+      simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::AircraftSystems,
+                             "Missing panel file:" + relative_path, path);
+      return nullptr;
+  }
+
   try {
     readProperties(path, &root);
   } catch (const sg_exception &e) {
-    guiErrorMessage("Error reading panel: ", e);
+      simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::AircraftSystems,
+                             "Error parsing panel file:" + e.getFormattedMessage(), path);
     return 0;
   }
-  return readPanel(&root);
+  return readPanel(&root, path);
 }
 
 
