@@ -96,7 +96,7 @@ wxRadarBg::wxRadarBg(SGPropertyNode *node) :
     branch = "/instrumentation/" + _name;
     _Instrument = fgGetNode(branch.c_str(), _num, true);
 
-    const char *tacan_source = node->getStringValue("tacan-source", "/instrumentation/tacan");
+    string tacan_source = node->getStringValue("tacan-source", "/instrumentation/tacan");
     _Tacan = fgGetNode(tacan_source, true);
 
     _font_node = _Instrument->getNode("font", true);
@@ -210,10 +210,12 @@ wxRadarBg::init ()
     // Initially allocate space for 128 quads
     _vertices = new osg::Vec2Array;
     _vertices->setDataVariance(osg::Object::DYNAMIC);
+    // REVIEW: Memory Leak - 4,096 bytes in 1 blocks are indirectly lost
     _vertices->reserve(128 * 4);
     _geom->setVertexArray(_vertices);
     _texCoords = new osg::Vec2Array;
     _texCoords->setDataVariance(osg::Object::DYNAMIC);
+    // REVIEW: Memory Leak - 4,096 bytes in 1 blocks are indirectly lost
     _texCoords->reserve(128 * 4);
     _geom->setTexCoordArray(0, _texCoords);
     osg::Vec3Array *colors = new osg::Vec3Array;
@@ -242,6 +244,7 @@ wxRadarBg::init ()
 
     osg::Camera *camera = _odg->getCamera();
     camera->addChild(_radarGeode.get());
+    // REVIEW: Memory Leak - 277,296 (16 direct, 277,280 indirect) bytes in 1 blocks are definitely lost
     camera->addChild(_textGeode.get());
 
     updateFont();
@@ -380,7 +383,7 @@ wxRadarBg::update (double delta_time_sec)
         _textGeode->removeDrawables(0, _textGeode->getNumDrawables());
 
 #if 0
-        //TODO FIXME Mask below (only used for ARC mode) isn't properly aligned, i.e.
+        // TODO: FIXME Mask below (only used for ARC mode) isn't properly aligned, i.e.
         // it assumes the a/c position at the center of the display - though it's somewhere at
         // bottom part for ARC mode.
         // The mask hadn't worked at all for a while (probably since the OSG port) due to
@@ -593,8 +596,8 @@ wxRadarBg::update_data(const SGPropertyNode *ac, double altitude, double heading
     callsign->setAlignment(osgText::Text::LEFT_BOTTOM_BASE_LINE);
     callsign->setLineSpacing(_font_spacing);
 
-    const char *identity = ac->getStringValue("transponder-id");
-    if (!identity[0])
+    string identity = ac->getStringValue("transponder-id", "");
+    if (identity.empty())
         identity = ac->getStringValue("callsign");
 
     stringstream text;
@@ -727,7 +730,7 @@ wxRadarBg::update_aircraft()
             continue;
 
         double echo_radius, sigma;
-        const string name = model->getName();
+        const string name = model->getNameString();
 
         //cout << "name "<<name << endl;
         if (name == "aircraft" || name == "tanker")
@@ -1008,7 +1011,7 @@ wxRadarBg::inRadarRange(double sigma, double range_nm)
     // Here, we will use a normalised rcs (sigma) for a standard taget and assume that this
     // will provide a maximum range of 35nm;
     //
-    // TODO - make the maximum range adjustable at runtime
+    // TODO: make the maximum range adjustable at runtime
 
     double constant = _radar_ref_rng;
 
@@ -1084,14 +1087,18 @@ wxRadarBg::updateFont()
     }
     
     osg::ref_ptr<osgDB::ReaderWriter::Options> fontOptions = new osgDB::ReaderWriter::Options("monochrome");
-    osg::ref_ptr<osgText::Font> font = osgText::readFontFile(tpath.local8BitStr(), fontOptions.get());
+    osg::ref_ptr<osgText::Font> font = osgText::readFontFile(tpath.utf8Str(), fontOptions.get());
 
     if (font != 0) {
         _font = font;
         _font->setMinFilterHint(osg::Texture::NEAREST);
         _font->setMagFilterHint(osg::Texture::NEAREST);
+#if OSG_VERSION_LESS_THAN(3,5,6)
         _font->setGlyphImageMargin(0);
         _font->setGlyphImageMarginRatio(0);
+#else
+//OSG:TODO: need 3.5.7 version of this.
+#endif
     }
 
     for (int i=0;i<4;i++)
@@ -1113,3 +1120,8 @@ wxRadarBg::valueChanged(SGPropertyNode*)
     _time = _interval;
 }
 
+
+// Register the subsystem.
+#if 0
+SGSubsystemMgr::Registrant<wxRadarBg> registrantwxRadarBg;
+#endif
