@@ -1,24 +1,10 @@
-// metarproperties.cxx -- Parse a METAR and write properties
-//
-// Written by David Megginson, started May 2002.
-// Rewritten by Torsten Dreyer, August 2010
-//
-// Copyright (C) 2002  David Megginson - david@megginson.com
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
+/*
+ * SPDX-FileName: metarproperties.cxx
+ * SPDX-FileComment: Parse a METAR and write properties
+ * SPDX-FileCopyrightText: Copyright (C) 2002  David Megginson - david@megginson.com
+ * SPDX-FileContributor: Rewritten by Torsten Dreyer, August 2010
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -27,6 +13,7 @@
 #include <cstring> // for strlen
 
 #include "metarproperties.hxx"
+#include "environment_mgr.hxx"
 #include "fgmetar.hxx"
 #include "environment.hxx"
 #include "atmosphere.hxx"
@@ -42,7 +29,7 @@ using std::string;
 
 namespace Environment {
 
-static vector<string> coverage_string;
+static std::vector<string> coverage_string;
 
 /**
  * @brief Helper class to wrap SGMagVar functionality and cache the variation and dip for
@@ -183,6 +170,10 @@ MetarProperties::MetarProperties( SGPropertyNode_ptr rootNode ) :
   _tiedProperties.Tie("minute", &_minute );
   _tiedProperties.Tie("decoded", this, &MetarProperties::get_decoded );
   _tiedProperties.Tie("cavok", &_cavok );
+  _tiedProperties.Tie("description", this, &MetarProperties::get_description );
+    
+    // mark proeprties as listener-safe, we invoke valueChanged explicitly
+    _tiedProperties.setAttribute(SGPropertyNode::LISTENER_SAFE, true);
 }
 
 MetarProperties::~MetarProperties()
@@ -190,28 +181,40 @@ MetarProperties::~MetarProperties()
   delete _magneticVariation;
 }
 
+void MetarProperties::invalidate()
+{
+  if( _metarValidNode->getBoolValue() )
+    _metarValidNode->setBoolValue(false);
+}
 
 static const double thickness_value[] = { 0, 65, 600, 750, 1000 };
 
 const char* MetarProperties::get_metar() const
 {
-    if (!_metar) return "";
-    return _metar->getData();
+    if (!_metar || _metarData.empty())
+        return "";
+    return _metarData.c_str();
 }
     
 void MetarProperties::set_metar( const char * metarString )
 {
     SGSharedPtr<FGMetar> m;
-    if ((metarString == NULL) || (strlen(metarString) == 0)) {
+    if (!metarString) {
         setMetar(m);
         return;
     }
     
-    try {
-        m = new FGMetar( metarString );
+    std::string trimmedMetar = simgear::strutils::strip(std::string{metarString});
+    if (trimmedMetar.empty()) {
+        setMetar(m);
+        return;
     }
-    catch( sg_io_exception ) {
-        SG_LOG( SG_ENVIRONMENT, SG_WARN, "Can't parse metar: " << metarString );
+
+    try {
+        m = new FGMetar(trimmedMetar);
+    }
+    catch( sg_io_exception& ) {
+        SG_LOG( SG_ENVIRONMENT, SG_WARN, "Can't parse metar:'" << trimmedMetar << "'");
         _metarValidNode->setBoolValue(false);
         return;
     }
@@ -224,12 +227,16 @@ void MetarProperties::setMetar( SGSharedPtr<FGMetar> m )
     _metar = m;
     _decoded.clear();
     if (!m) {
+        _metarData.clear();
         return;
     }
     
-    const vector<string> weather = m->getWeather();
-    for( vector<string>::const_iterator it = weather.begin(); it != weather.end(); ++it ) {
-        if( false == _decoded.empty() ) _decoded.append(", ");
+    // copy the string so we have guranteed storage for get_metar tied property API
+    _metarData = _metar->getDataString();
+    
+    const std::vector<string> weather = m->getWeather();
+    for( std::vector<string>::const_iterator it = weather.begin(); it != weather.end(); ++it ) {
+        if( !_decoded.empty() ) _decoded.append(", ");
         _decoded.append(*it);
     }
 
@@ -285,6 +292,7 @@ void MetarProperties::setMetar( SGSharedPtr<FGMetar> m )
 
     {    // calculate sea level temperature, dewpoint and pressure
         FGEnvironment dummy; // instantiate a dummy so we can leech a method
+        dummy.set_is_isa( globals->get_subsystem<FGEnvironmentMgr>()->getEnvironment().get_is_isa() );
         dummy.set_elevation_ft( _station_elevation );
         dummy.set_temperature_degc( _temperature );
         dummy.set_dewpoint_degc( _dewpoint );
@@ -305,7 +313,7 @@ void MetarProperties::setMetar( SGSharedPtr<FGMetar> m )
     {
         for( unsigned i = 0; i < 3; i++ ) {
             SGPropertyNode_ptr n = _rootNode->getChild("weather", i, true );
-            vector<struct SGMetar::Weather> weather = m->getWeather2();
+            std::vector<struct SGMetar::Weather> weather = m->getWeather2();
             struct SGMetar::Weather * w = i < weather.size() ? &weather[i] : NULL;
             n->getNode("intensity",true)->setIntValue( w != NULL ? w->intensity : 0 );
             n->getNode("vincinity",true)->setBoolValue( w != NULL ? w->vincinity : false );
@@ -331,13 +339,17 @@ void MetarProperties::setMetar( SGSharedPtr<FGMetar> m )
     {
         static const char * LAYER = "layer";
         SGPropertyNode_ptr cloudsNode = _rootNode->getNode("clouds", true );
-        const vector<SGMetarCloud> & metarClouds = m->getClouds();
+        const std::vector<SGMetarCloud> & metarClouds = m->getClouds();
         unsigned layerOffset = 0; // Oh, this is ugly!
 
         // fog/mist/haze cloud layer does not work with 3d clouds yet :-(
         bool setGroundCloudLayer = _rootNode->getBoolValue("set-ground-cloud-layer", false ) &&
-              false == (fgGetBool("/sim/rendering/shader-effects", false ) && 
-                        fgGetBool("/sim/rendering/clouds3d-enable", false ) );
+              !fgGetBool("/sim/rendering/clouds3d-enable", false);
+
+        // track the coverage of the previous layer, so we can use it
+        // for higher layers which don't have coverage set
+        // see: https://sourceforge.net/p/flightgear/codetickets/2765/
+        SGMetarCloud::Coverage coverageBelow = SGMetarCloud::COVERAGE_NIL;
 
         if( setGroundCloudLayer ) {
             // create a cloud layer #0 starting at the ground if its fog, mist or haze
@@ -387,12 +399,25 @@ void MetarProperties::setMetar( SGSharedPtr<FGMetar> m )
                 _min_visibility = _max_visibility =
                   fgGetDouble("/environment/params/fog-mist-haze-layer/visibility-above-layer-m",20000.0); // assume good visibility above the fog
                 layerOffset = 1;  // shudder
+                
+                coverageBelow = coverage;
             }
         } 
 
         for( unsigned i = 0; i < 5-layerOffset; i++ ) {
             SGPropertyNode_ptr layerNode = cloudsNode->getChild(LAYER, i+layerOffset, true );
             SGMetarCloud::Coverage coverage = i < metarClouds.size() ? metarClouds[i].getCoverage() : SGMetarCloud::COVERAGE_CLEAR;
+            if (coverage == SGMetarCloud::COVERAGE_NIL) {
+                coverage = coverageBelow; // invalid coverage, use value of layer below
+            } else {
+                coverageBelow = coverage; // valid coverage, save for future layers
+            }
+            
+            if (coverage == SGMetarCloud::COVERAGE_NIL) {
+                SG_LOG(SG_ENVIRONMENT, SG_WARN, "METAR: skipping cloud layer " << i << " because no coverage is set");
+                continue;
+            }
+            
             double elevation = 
                 i >= metarClouds.size() || coverage == SGMetarCloud::COVERAGE_CLEAR ? 
                 -9999.0 : 
@@ -416,7 +441,9 @@ void MetarProperties::setMetar( SGSharedPtr<FGMetar> m )
     _hour = m->getHour();
     _minute = m->getMinute();
     _cavok = m->getCAVOK();
+    _tiedProperties.fireValueChanged();
     _metarValidNode->setBoolValue(true);
+    _description = m->getDescription(-1);
 }
 
 void MetarProperties::setStationId( const std::string & value )
