@@ -1,25 +1,9 @@
-// JSBsim.cxx -- interface to the JSBsim flight model
-//
-// Written by Curtis Olson, started February 1999.
-//
-// Copyright (C) 1999  Curtis L. Olson  - curt@flightgear.org
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Lesser General Public License for more details.
-//
-// You should have received a copy of the GNU Lesser General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// $Id: FlightGear.cxx,v 1.15 2014/01/28 09:42:20 ehofman Exp $
-
+/*
+ * SPDX-FileName: JSBsim.cxx
+ * SPDX-FileComment: interface to the JSBsim flight model
+ * SPDX-FileCopyrightText: Copyright (C) 1999  Curtis L. Olson  - curt@flightgear.org
+ * SPDX-License-Identifier: LGPL-2.0-or-later
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -39,6 +23,7 @@
 #include <simgear/bvh/BVHMaterial.hxx>
 
 #include <FDM/flight.hxx>
+#include <FDM/groundreactions.hxx>
 
 #include <Aircraft/controls.hxx>
 #include <Main/globals.hxx>
@@ -55,13 +40,13 @@
 #include <FDM/JSBSim/models/FGPropagate.h>
 #include <FDM/JSBSim/models/FGAuxiliary.h>
 #include <FDM/JSBSim/models/FGInertial.h>
-#include <FDM/JSBSim/models/FGAtmosphere.h>
 #include <FDM/JSBSim/models/FGMassBalance.h>
 #include <FDM/JSBSim/models/FGAerodynamics.h>
 #include <FDM/JSBSim/models/FGLGear.h>
 #include <FDM/JSBSim/models/FGGroundReactions.h>
 #include <FDM/JSBSim/models/FGPropulsion.h>
 #include <FDM/JSBSim/models/FGAccelerations.h>
+#include <FDM/JSBSim/models/atmosphere/FGStandardAtmosphere.h>
 #include <FDM/JSBSim/models/atmosphere/FGWinds.h>
 #include <FDM/JSBSim/models/propulsion/FGEngine.h>
 #include <FDM/JSBSim/models/propulsion/FGPiston.h>
@@ -100,9 +85,9 @@ public:
   }
 
   /** Compute the altitude above ground. */
-  virtual double GetAGLevel(double t, const FGLocation& l,
-                            FGLocation& cont, FGColumnVector3& n,
-                            FGColumnVector3& v, FGColumnVector3& w) const {
+  double GetAGLevel(double t, const FGLocation& l, FGLocation& cont,
+                    FGColumnVector3& n, FGColumnVector3& v, FGColumnVector3& w)
+      const override {
     double contact[3], normal[3], vel[3], angularVel[3];
     double agl = mInterface->get_agl_ft(t, l, SG_METER_TO_FEET*2, contact,
                                         normal, vel, angularVel);
@@ -113,30 +98,12 @@ public:
     return agl;
   }
 
-  virtual double GetTerrainGeoCentRadius(double t, const FGLocation& l) const {
-    double contact[3], normal[3], vel[3], angularVel[3];
-    mInterface->get_agl_ft(t, l, SG_METER_TO_FEET*2, contact,
-                           normal, vel, angularVel);
-    return sqrt(contact[0]*contact[0]+contact[1]*contact[1]+contact[2]*contact[2]);
-  }
-
-  virtual double GetSeaLevelRadius(const FGLocation& l) const {
-    double seaLevelRadius, latGeoc;
-
-    sgGeodToGeoc(l.GetGeodLatitudeRad(), l.GetGeodAltitude(),
-                 &seaLevelRadius, &latGeoc);
-
-    return seaLevelRadius * SG_METER_TO_FEET;
-  }
-
-  virtual void SetTerrainGeoCentRadius(double radius) {}
-  virtual void SetSeaLevelRadius(double radius) {}
 private:
   FGJSBsim* mInterface;
 };
 
 // FG uses a squared normalized magnitude for turbulence
-// this lookup table maps fg's severity levels 
+// this lookup table maps fg's severity levels
 // none(0), light(1/3), moderate(2/3) and severe(3/3)
 // to the POE table indexes 0, 3, 4 and 7
 class FGTurbulenceSeverityTable : public FGTable {
@@ -199,9 +166,7 @@ FGJSBsim::FGJSBsim( double dt )
 
     PropertyManager = new FGPropertyManager( (FGPropertyNode*)globals->get_props() );
     fdmex = new FGFDMExec( PropertyManager );
-
-    // Register ground callback.
-    fdmex->SetGroundCallback( new FGFSGroundCallback(this) );
+    fdmex->Hold();
 
     Atmosphere      = fdmex->GetAtmosphere();
     Winds           = fdmex->GetWinds();
@@ -216,6 +181,9 @@ FGJSBsim::FGJSBsim( double dt )
     GroundReactions = fdmex->GetGroundReactions();
     Accelerations   = fdmex->GetAccelerations();
 
+    // Register ground callback.
+    Inertial->SetGroundCallback( new FGFSGroundCallback(this) );
+
     fgic=fdmex->GetIC();
     needTrim=true;
 
@@ -230,8 +198,8 @@ FGJSBsim::FGJSBsim( double dt )
 // deprecate sim-time-sec for simulation/sim-time-sec
 // remove alias with increased configuration file version number (2.1 or later)
     SGPropertyNode * node = fgGetNode("/fdm/jsbsim/simulation/sim-time-sec");
-    fgGetNode("/fdm/jsbsim/sim-time-sec", true)->alias( node );
-// end of sim-time-sec deprecation patch
+    fgGetNode("/fdm/jsbsim/sim-time-sec", true)->alias(node, false);
+    // end of sim-time-sec deprecation patch
 
     _ai_wake_enabled = fgGetNode("fdm/ai-wake/enabled", true);
 
@@ -273,19 +241,29 @@ FGJSBsim::FGJSBsim( double dt )
       double d;
       SGPropertyNode * node = fgGetNode("/consumables/fuel/tank", i, true);
       FGTank* tank = Propulsion->GetTank(i);
+      SGPropertyNode * prop = node->getNode( "density-ppg", true );
 
-      d = node->getNode( "density-ppg", true )->getDoubleValue();
+      d = prop->getDoubleValue();
       if( d > 0.0 ) {
         tank->SetDensity( d );
       } else {
-        node->getNode( "density-ppg", true )->setDoubleValue( SG_MAX2<double>(tank->GetDensity(), 0.1) );
+        prop->setDoubleValue( SG_MAX2<double>(tank->GetDensity(), 0.1) );
       }
 
-      d = node->getNode( "level-lbs", true )->getDoubleValue();
+      prop = node->getNode( "level-lbs", true );
+      d = prop->getDoubleValue();
       if( d > 0.0 ) {
         tank->SetContents( d );
       } else {
-        node->getNode( "level-lbs", true )->setDoubleValue( tank->GetContents() );
+        prop->setDoubleValue( tank->GetContents() );
+      }
+
+      prop = node->getNode( "unusable-gal_us", true );
+      d = prop->getDoubleValue();
+      if ( d > 0.0 ) {
+        tank->SetUnusableVolume( d );
+      } else {
+        prop->setDoubleValue(tank->GetUnusableVolume());
       }
       /* Capacity is read-only in FGTank and can't be overwritten from FlightGear */
       node->getNode("capacity-gal_us", true )->setDoubleValue( tank->GetCapacityGallons() );
@@ -332,16 +310,21 @@ FGJSBsim::FGJSBsim( double dt )
     speedbrake_pos_pct->setDoubleValue(0);
     spoilers_pos_pct->setDoubleValue(0);
 
+    override_fg_brake_prop = fgGetNode("/fdm/jsbsim/systems/override-fg-brake-properties", true);
+    override_fg_brake_prop->setBoolValue(false);
     ab_brake_engaged = fgGetNode("/autopilot/autobrake/engaged", true);
     ab_brake_left_pct = fgGetNode("/autopilot/autobrake/brake-left-output", true);
     ab_brake_right_pct = fgGetNode("/autopilot/autobrake/brake-right-output", true);
+
     arrestor_wire_engaged_hook = fgGetNode("/fdm/jsbsim/systems/hook/arrestor-wire-engaged-hook", true);
     release_hook = fgGetNode("/fdm/jsbsim/systems/hook/tailhook-release-cmd", true);
 
     altitude = fgGetNode("/position/altitude-ft");
+    precipitation = fgGetNode("/environment/rain-norm", true);
     temperature = fgGetNode("/environment/temperature-degc",true);
     pressure = fgGetNode("/environment/pressure-inhg",true);
     pressureSL = fgGetNode("/environment/pressure-sea-level-inhg",true);
+    dew_point = fgGetNode("/environment/dewpoint-degc", true);
     ground_wind = fgGetNode("/environment/config/boundary/entry[0]/wind-speed-kt",true);
     turbulence_gain = fgGetNode("/environment/turbulence/magnitude-norm",true);
     turbulence_rate = fgGetNode("/environment/turbulence/rate-hz",true);
@@ -369,7 +352,17 @@ FGJSBsim::FGJSBsim( double dt )
     crashed = false;
 
     mesh = new AircraftMesh(fgGetDouble("/fdm/jsbsim/metrics/bw-ft"),
-                            fgGetDouble("/fdm/jsbsim/metrics/cbarw-ft"));
+                            fgGetDouble("/fdm/jsbsim/metrics/cbarw-ft"),
+                            fdmex->GetModelName());
+
+    // Trim once to initialize all output parameters
+    FGTrim *fgtrim = new FGTrim(fdmex,tFull);
+    fgtrim->DoTrim();
+    delete fgtrim;
+
+    std::string directive_file = fgGetString("/sim/jsbsim/output-directive-file");
+    if (!directive_file.empty())
+      fdmex->SetOutputDirectives(directive_file);
 }
 
 /******************************************************************************/
@@ -414,6 +407,7 @@ void FGJSBsim::init()
     common_init();
 
     copy_to_JSBsim();
+    fdmex->Resume();
     fdmex->RunIC();     //loop JSBSim once w/o integrating
     if (fgGetBool("/sim/presets/running")) {
       Propulsion->InitRunning(-1);
@@ -618,23 +612,24 @@ bool FGJSBsim::copy_to_JSBsim()
     FCS->SetDsbCmd( globals->get_controls()->get_speedbrake() );
     FCS->SetDspCmd( globals->get_controls()->get_spoilers() );
 
+    if (!override_fg_brake_prop->getBoolValue()) {
         // Parking brake sets minimum braking
         // level for mains.
-    double parking_brake = globals->get_controls()->get_brake_parking();
-    double left_brake = globals->get_controls()->get_brake_left();
-    double right_brake = globals->get_controls()->get_brake_right();
-    
-    if (ab_brake_engaged->getBoolValue()) {
-      left_brake = ab_brake_left_pct->getDoubleValue();
-      right_brake = ab_brake_right_pct->getDoubleValue(); 
+        double parking_brake = globals->get_controls()->get_brake_parking();
+        double left_brake = globals->get_controls()->get_brake_left();
+        double right_brake = globals->get_controls()->get_brake_right();
+
+        if (ab_brake_engaged->getBoolValue()) {
+            left_brake = ab_brake_left_pct->getDoubleValue();
+            right_brake = ab_brake_right_pct->getDoubleValue();
+        }
+
+        FCS->SetLBrake(FMAX(left_brake, parking_brake));
+        FCS->SetRBrake(FMAX(right_brake, parking_brake));
+
+        FCS->SetCBrake( 0.0 );
+        // FCS->SetCBrake( globals->get_controls()->get_brake(2) );
     }
-    
-    FCS->SetLBrake(FMAX(left_brake, parking_brake));
-    FCS->SetRBrake(FMAX(right_brake, parking_brake));
-    
-    
-    FCS->SetCBrake( 0.0 );
-    // FCS->SetCBrake( globals->get_controls()->get_brake(2) );
 
     FCS->SetGearCmd( globals->get_controls()->get_gear_down());
     for (i = 0; i < Propulsion->GetNumEngines(); i++) {
@@ -692,6 +687,8 @@ bool FGJSBsim::copy_to_JSBsim()
 
     Atmosphere->SetTemperature(temperature->getDoubleValue(), get_Altitude(), FGAtmosphere::eCelsius);
     Atmosphere->SetPressureSL(FGAtmosphere::eInchesHg, pressureSL->getDoubleValue());
+    static_cast<FGStandardAtmosphere*>(Atmosphere)->SetDewPoint(FGAtmosphere::eCelsius,
+                                                                dew_point->getDoubleValue());
 
     Winds->SetTurbType((FGWinds::tType)TURBULENCE_TYPE_NAMES[turbulence_model->getStringValue()]);
     switch( Winds->GetTurbType() ) {
@@ -1089,9 +1086,7 @@ void FGJSBsim::set_V_calibrated_kts(double vc)
     fgic->SetVcalibratedKtsIC(vc);
   else {
     double p=pressure->getDoubleValue();
-    double psl=fdmex->GetAtmosphere()->GetPressureSL();
-    double rhosl=fdmex->GetAtmosphere()->GetDensitySL();
-    double mach = FGJSBBase::MachFromVcalibrated(vc, p, psl, rhosl);
+    double mach = FGJSBBase::MachFromVcalibrated(vc, p);
     double temp = 1.8*(temperature->getDoubleValue()+273.15);
     double soundSpeed = sqrt(1.4*1716.0*temp);
     FGColumnVector3 vUVW = Propagate->GetUVW();
@@ -1236,18 +1231,32 @@ void FGJSBsim::init_gear(void )
 {
     FGGroundReactions* gr=fdmex->GetGroundReactions();
     int Ngear=GroundReactions->GetNumGearUnits();
+    double max = 1.0;
     for (int i=0;i<Ngear;i++) {
       FGLGear *gear = gr->GetGearUnit(i);
       SGPropertyNode * node = fgGetNode("gear/gear", i, true);
+      // coordinates are in feet so convert
       node->setDoubleValue("xoffset-in", gear->GetBodyLocation()(1) * 12);
       node->setDoubleValue("yoffset-in", gear->GetBodyLocation()(2) * 12);
       node->setDoubleValue("zoffset-in", gear->GetBodyLocation()(3) * 12);
+
+      node->setDoubleValue("xoffset-m", gear->GetBodyLocation()(1) * SG_FEET_TO_METER);
+      node->setDoubleValue("yoffset-m", gear->GetBodyLocation()(2) * SG_FEET_TO_METER);
+      node->setDoubleValue("zoffset-m", gear->GetBodyLocation()(3) * SG_FEET_TO_METER);
+
       node->setBoolValue("wow", gear->GetWOW());
-      node->setDoubleValue("rollspeed-ms", gear->GetWheelRollVel()*0.3043);
+      node->setDoubleValue("rollspeed-ms", gear->GetWheelRollVel() * SG_FEET_TO_METER);
       node->setBoolValue("has-brake", gear->GetBrakeGroup() > 0);
       node->setDoubleValue("position-norm", gear->GetGearUnitPos());
 //    node->setDoubleValue("tire-pressure-norm", gear->GetTirePressure());
-      node->setDoubleValue("compression-norm", gear->GetCompLen());
+      max = node->getDoubleValue("max-compression-ft");
+      if (max < 0.00001) {
+        max = node->getDoubleValue("max-compression-m");
+        if (max < 0.00001) max = 1.0;
+        else max /= SG_FEET_TO_METER;
+      }
+      node->setDoubleValue("compression-norm", gear->GetCompLen() / max);
+      node->setDoubleValue("compression-m", gear->GetCompLen() * SG_FEET_TO_METER);
       node->setDoubleValue("compression-ft", gear->GetCompLen());
       if ( gear->GetSteerable() )
         node->setDoubleValue("steering-norm", gear->GetSteerNorm());
@@ -1258,14 +1267,22 @@ void FGJSBsim::update_gear(void)
 {
     FGGroundReactions* gr=fdmex->GetGroundReactions();
     int Ngear=GroundReactions->GetNumGearUnits();
+    double max = 1.0;
     for (int i=0;i<Ngear;i++) {
       FGLGear *gear = gr->GetGearUnit(i);
       SGPropertyNode * node = fgGetNode("gear/gear", i, true);
       node->getChild("wow", 0, true)->setBoolValue( gear->GetWOW());
-      node->getChild("rollspeed-ms", 0, true)->setDoubleValue(gear->GetWheelRollVel()*0.3043);
+      node->getChild("rollspeed-ms", 0, true)->setDoubleValue(gear->GetWheelRollVel() * SG_FEET_TO_METER);
       node->getChild("position-norm", 0, true)->setDoubleValue(gear->GetGearUnitPos());
 //    gear->SetTirePressure(node->getDoubleValue("tire-pressure-norm"));
-      node->setDoubleValue("compression-norm", gear->GetCompLen());
+      max = node->getDoubleValue("max-compression-ft");
+      if (max < 0.00001) {
+        max = node->getDoubleValue("max-compression-m");
+        if (max < 0.00001) max = 1.0;
+        else max /= SG_FEET_TO_METER;
+      }
+      node->setDoubleValue("compression-norm", gear->GetCompLen() / max);
+      node->setDoubleValue("compression-m", gear->GetCompLen() * SG_FEET_TO_METER);
       node->setDoubleValue("compression-ft", gear->GetCompLen());
       if ( gear->GetSteerable() )
         node->setDoubleValue("steering-norm", gear->GetSteerNorm());
@@ -1329,20 +1346,20 @@ bool FGJSBsim::update_ground_cache(const FGLocation& cart, double dt)
   bool cache_ok = prepare_ground_cache_ft( t0, t0 + dt, cart_pos,
                                            groundCacheRadius );
   if (!cache_ok) {
-    SG_LOG(SG_FLIGHT, SG_WARN,
-           "FGInterface is being called without scenery below the aircraft!");
+    //SG_LOG(SG_FLIGHT, SG_WARN,
+    //       "FGInterface is being called without scenery below the aircraft!");
 
-    SG_LOG(SG_FLIGHT, SG_WARN, "altitude         = "
-                      << fgic->GetAltitudeASLFtIC());
+    //SG_LOG(SG_FLIGHT, SG_WARN, "altitude         = "
+    //                  << fgic->GetAltitudeASLFtIC());
 
-    SG_LOG(SG_FLIGHT, SG_WARN, "sea level radius = "
-                      << get_Sea_level_radius());
+    //SG_LOG(SG_FLIGHT, SG_WARN, "sea level radius = "
+    //                  << get_Sea_level_radius());
 
-    SG_LOG(SG_FLIGHT, SG_WARN, "latitude         = "
-                      << fgic->GetLatitudeRadIC());
+    //SG_LOG(SG_FLIGHT, SG_WARN, "latitude         = "
+    //                  << fgic->GetLatitudeRadIC());
 
-    SG_LOG(SG_FLIGHT, SG_WARN, "longitude        = "
-                      << fgic->GetLongitudeRadIC());
+    //SG_LOG(SG_FLIGHT, SG_WARN, "longitude        = "
+    //                  << fgic->GetLongitudeRadIC());
   }
 
   return cache_ok;
@@ -1353,14 +1370,13 @@ FGJSBsim::get_agl_ft(double t, const FGColumnVector3& loc, double alt_off,
                      double contact[3], double normal[3], double vel[3],
                      double angularVel[3])
 {
-  const simgear::BVHMaterial* material;
+  const simgear::BVHMaterial* material = nullptr;
   simgear::BVHNode::Id id;
   double pt[3] {loc(1), loc(2), loc(3)};
 
-  // don't check the return value and continue above scenery discontinuity
-  // see http://osdir.com/ml/flightgear-sim/2014-04/msg00145.html
-  FGInterface::get_agl_ft(t, pt, alt_off, contact, normal, vel,
-                          angularVel, material, id);
+  if (!FGInterface::get_agl_ft(t, pt, alt_off, contact, normal, vel,
+                          angularVel, material, id))
+    material = nullptr; // Discard the material data when FGInterface reports an problem.
 
   SGGeod geodPt = SGGeod::fromCart(SG_FEET_TO_METER*SGVec3d(pt));
   SGQuatd hlToEc = SGQuatd::fromLonLat(geodPt);
@@ -1372,15 +1388,15 @@ FGJSBsim::get_agl_ft(double t, const FGColumnVector3& loc, double alt_off,
   if (terrain_active)
   {
     static bool material_valid = false;
-    if (material) {
-      GroundReactions->SetStaticFFactor((*material).get_friction_factor());
-      GroundReactions->SetRollingFFactor((*material).get_rolling_friction()/0.02);
-      // 1 Pascal = 0.00014503773800721815 lbs/in^2
-      double pressure = (*material).get_load_resistance(); // N/m^2 (or Pascal)
-      GroundReactions->SetMaximumForce(pressure*0.00014503773800721815);
 
-      GroundReactions->SetBumpiness((*material).get_bumpiness());
-      GroundReactions->SetSolid((*material).get_solid());
+    if (updateGroundReactions(material)) {
+      setPosition(pt);
+      setHeading(Propagate->GetEuler(FGJSBBase::ePsi));
+      GroundReactions->SetRollingFFactor(getRolingFrictionFactor());
+      GroundReactions->SetStaticFFactor(getStaticFrictionFactor());
+      GroundReactions->SetMaximumForce(getPressure()*0.00014503773800721815);
+      GroundReactions->SetBumpiness(getBumpiness());
+      GroundReactions->SetSolid(getSolid());
       GroundReactions->SetPosition(pt);
       material_valid = true;
     } else {
@@ -1393,7 +1409,8 @@ FGJSBsim::get_agl_ft(double t, const FGColumnVector3& loc, double alt_off,
 #else
   terrain->setBoolValue("valid", false);
 #endif
-  return dot(hlToEc.rotate(SGVec3d(0, 0, 1)), SGVec3d(contact) - SGVec3d(pt));
+  return dot(hlToEc.rotate(SGVec3d(0, 0, 1)), SGVec3d(contact) - SGVec3d(pt)) +
+         getGroundDisplacement();
 }
 
 inline static double sqr(double x)
@@ -1405,7 +1422,7 @@ static double angle_diff(double a, double b)
 {
     double diff = fabs(a - b);
     if (diff > 180) diff = 360 - diff;
-    
+
     return diff;
 }
 
@@ -1418,7 +1435,7 @@ static void check_hook_solution(const FGColumnVector3& ground_normal_body, doubl
 	cos_fis[*points] = cos_fi_guess;
 	fis[*points] = atan2(sin_fi_guess, cos_fi_guess) * SG_RADIANS_TO_DEGREES;
 	(*points)++;
-    } 
+    }
 }
 
 
@@ -1439,15 +1456,15 @@ void FGJSBsim::update_external_forces(double t_off)
     const FGMatrix33& Tl2b = Propagate->GetTl2b();
     const FGLocation& Location = Propagate->GetLocation();
     const FGMatrix33& Tec2l = Location.GetTec2l();
-        
+
     double hook_area[4][3];
-    
+
     FGColumnVector3 hook_root_body = MassBalance->StructuralToBody(hook_root_struct);
     FGColumnVector3 hook_root = Location.LocalToLocation(Tb2l *   hook_root_body);
     hook_area[1][0] = hook_root(1);
     hook_area[1][1] = hook_root(2);
     hook_area[1][2] = hook_root(3);
-    
+
     hook_length = fgGetDouble("/fdm/jsbsim/systems/hook/tailhook-length-ft", 6.75);
     double fi_min = fgGetDouble("/fdm/jsbsim/systems/hook/tailhook-pos-min-deg", -18);
     double fi_max = fgGetDouble("/fdm/jsbsim/systems/hook/tailhook-pos-max-deg", 30);
@@ -1457,7 +1474,7 @@ void FGJSBsim::update_external_forces(double t_off)
 
     FGColumnVector3 hook_tip_body = hook_root_body;
     hook_tip_body(1) -= hook_length * cos_fi;
-    hook_tip_body(3) += hook_length * sin_fi;    
+    hook_tip_body(3) += hook_length * sin_fi;
     if (!arrestor_wire_engaged_hook->getBoolValue()) {
       double contact[3];
       double ground_normal[3];
@@ -1572,7 +1589,7 @@ void FGJSBsim::update_external_forces(double t_off)
                 arrestor_wire_engaged_hook->setBoolValue(true);
         }
     }
-    
+
     // save actual position as old position ...
     last_hook_tip[0] = hook_area[0][0];
     last_hook_tip[1] = hook_area[0][1];
@@ -1580,7 +1597,7 @@ void FGJSBsim::update_external_forces(double t_off)
     last_hook_root[0] = hook_area[1][0];
     last_hook_root[1] = hook_area[1][1];
     last_hook_root[2] = hook_area[1][2];
-    
+
     fgSetDouble("/fdm/jsbsim/systems/hook/tailhook-pos-deg", fi);
 
     if (_ai_wake_enabled->getBoolValue()) {
@@ -1614,3 +1631,9 @@ void FGJSBsim::update_external_forces(double t_off)
       if (_mmag) _mmag->setDoubleValue(0.0);
     }
 }
+
+
+// Register the subsystem.
+#if 0
+SGSubsystemMgr::Registrant<FGJSBsim> registrantFGJSBsim;
+#endif
