@@ -55,14 +55,12 @@ INCLUDES
 #include "models/FGFCS.h"
 #include "models/atmosphere/FGWinds.h"
 #include "input_output/FGXMLElement.h"
+#include "math/FGPropertyValue.h"
 
 using namespace std;
 
 namespace JSBSim {
 
-IDENT(IdSrc,"$Id: FGOutputSocket.cpp,v 1.9 2014/02/17 05:01:22 jberndt Exp $");
-IDENT(IdHdr,ID_OUTPUTSOCKET);
-
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
@@ -106,7 +104,7 @@ void FGOutputSocket::SetOutputName(const string& fname)
   
   SockPort = atoi(port.c_str());
   
-  if (proto == "UDP")
+  if (to_upper(proto) == "UDP")
     SockProtocol = FGfdmSocket::ptUDP;
   else // Default to TCP
     SockProtocol = FGfdmSocket::ptTCP;
@@ -123,6 +121,12 @@ bool FGOutputSocket::Load(Element* el)
                 el->GetAttributeValue("protocol") + "/" +
                 el->GetAttributeValue("port"));
 
+  // Check if output precision for doubles has been specified, default to 7 if not
+  if(el->HasAttribute("precision"))
+    precision = (int)el->GetAttributeValueAsNumber("precision");
+  else
+    precision = 7;
+
   return true;
 }
 
@@ -132,7 +136,7 @@ bool FGOutputSocket::InitModel(void)
 {
   if (FGOutputType::InitModel()) {
     delete socket;
-    socket = new FGfdmSocket(SockName, SockPort, SockProtocol);
+    socket = new FGfdmSocket(SockName, SockPort, SockProtocol, precision);
 
     if (socket == 0) return false;
     if (!socket->GetConnectStatus()) return false;
@@ -260,13 +264,11 @@ void FGOutputSocket::PrintHeaders(void)
   if (SubSystems & ssPropulsion && Propulsion->GetNumEngines() > 0)
     socket->Append(Propulsion->GetPropulsionStrings(","));
 
-  if (OutputProperties.size() > 0) {
-    for (unsigned int i=0;i<OutputProperties.size();i++)
-      if (OutputCaptions[i].size() > 0) {
-        socket->Append(OutputCaptions[i]);
-      } else {
-        socket->Append(OutputProperties[i]->GetPrintableName());
-      }
+  for (unsigned int i=0;i<OutputParameters.size();++i) {
+    if (!OutputCaptions[i].empty())
+      socket->Append(OutputCaptions[i]);
+    else
+      socket->Append(OutputParameters[i]->GetPrintableName());
   }
 
   socket->Send();
@@ -378,8 +380,8 @@ void FGOutputSocket::Print(void)
     socket->Append(Propulsion->GetPropulsionValues(","));
   }
 
-  for (unsigned int i=0;i<OutputProperties.size();i++) {
-    socket->Append(OutputProperties[i]->getDoubleValue());
+  for (unsigned int i=0;i<OutputParameters.size();++i) {
+    socket->Append(OutputParameters[i]->GetValue());
   }
 
   socket->Send();
