@@ -40,12 +40,6 @@ INCLUDES
 
 #include "models/FGModel.h"
 
-/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-DEFINITIONS
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
-
-#define ID_OUTPUTTYPE "$Id: FGOutputType.h,v 1.11 2015/08/23 09:43:31 bcoconni Exp $"
-
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
@@ -67,6 +61,7 @@ class FGFCS;
 class FGGroundReactions;
 class FGExternalReactions;
 class FGBuoyantForces;
+class FGPropertyValue;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS DOCUMENTATION
@@ -101,7 +96,7 @@ public:
   FGOutputType(FGFDMExec* fdmex);
 
   /// Destructor
-  virtual ~FGOutputType();
+  ~FGOutputType() override;
 
   /** Set the idx for this output instance
       @param idx ID of the output instance that is constructed
@@ -124,10 +119,7 @@ public:
   /** Set the list of properties that should be output for this output instance.
       @param outputProperties list of properties that should be output
   */
-  void SetOutputProperties(std::vector<FGPropertyNode_ptr> & outputProperties)
-  {
-    OutputProperties = outputProperties;
-  }
+  void SetOutputProperties(std::vector<FGPropertyNode_ptr> & outputProperties);
 
   /** Overwrites the name identifier under which the output will be logged.
       This method is taken into account if it is called before
@@ -143,10 +135,10 @@ public:
   /** Init the output directives from an XML file (implement the FGModel interface).
       @param element XML Element that is pointing to the output directives
   */
-  virtual bool Load(Element* el);
+  bool Load(Element* el) override;
 
   /// Init the output model according to its configitation.
-  virtual bool InitModel(void);
+  bool InitModel(void) override;
 
   /** Executes the output directives (implement the FGModel interface).
       This method checks that the current time step matches the output
@@ -199,7 +191,7 @@ public:
 protected:
   unsigned int OutputIdx;
   int SubSystems;
-  std::vector <FGPropertyNode_ptr> OutputProperties;
+  std::vector <FGPropertyValue*> OutputParameters;
   std::vector <std::string> OutputCaptions;
   bool enabled;
 
@@ -217,7 +209,7 @@ protected:
   FGExternalReactions* ExternalReactions;
   FGBuoyantForces* BuoyantForces;
 
-  void Debug(int from);
+  void Debug(int from) override;
 };
 }
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
