@@ -28,6 +28,7 @@
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
+#include <assert.h>
 #include "FGPropertyManager.h"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -52,12 +53,10 @@ namespace JSBSim {
 
 void FGPropertyManager::Unbind(void)
 {
-    vector<SGPropertyNode_ptr>::iterator it;
+  for(auto& prop: tied_properties)
+    prop->untie();
 
-    for (it = tied_properties.begin();it < tied_properties.end();it++)
-        (*it)->untie();
-
-    tied_properties.clear();
+  tied_properties.clear();
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -73,7 +72,7 @@ string FGPropertyManager::mkPropertyName(string name, bool lowercase) {
     else if( isspace(name[i]) )
       name[i]='-';
   }
-  
+
   return name;
 }
 
@@ -112,16 +111,9 @@ bool FGPropertyNode::HasNode (const string &path)
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-string FGPropertyNode::GetName( void ) const
-{
-  return string( getName() );
-}
-
-//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-
 string FGPropertyNode::GetPrintableName( void ) const
 {
-  string temp_string(getName());
+  string temp_string(getNameString());
   size_t initial_location=0;
   size_t found_location;
 
@@ -303,7 +295,7 @@ void FGPropertyNode::SetWritable (const string &name, bool state )
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-void FGPropertyManager::Untie (const string &name)
+void FGPropertyManager::Untie(const string &name)
 {
   SGPropertyNode* property = root->getNode(name.c_str());
   if (!property) {
@@ -311,8 +303,18 @@ void FGPropertyManager::Untie (const string &name)
     return;
   }
 
-  vector <SGPropertyNode_ptr>::iterator it;
-  for (it = tied_properties.begin(); it != tied_properties.end(); ++it) {
+  Untie(property);
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+void FGPropertyManager::Untie(SGPropertyNode *property)
+{
+  const string& name = property->getNameString();
+
+  assert(property->isTied());
+
+  for (auto it = tied_properties.begin(); it != tied_properties.end(); ++it) {
     if (*it == property) {
       property->untie();
       tied_properties.erase(it);
@@ -324,98 +326,4 @@ void FGPropertyManager::Untie (const string &name)
   cerr << "Failed to untie property " << name << endl
        << "JSBSim is not the owner of this property." << endl;
 }
-
-//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-
-void FGPropertyManager::Tie (const string &name, bool *pointer, bool useDefault)
-{
-  SGPropertyNode* property = root->getNode(name.c_str(), true);
-  if (!property) {
-    cerr << "Could not get or create property " << name << endl;
-    return;
-  }
-
-  if (!property->tie(SGRawValuePointer<bool>(pointer), useDefault))
-    cerr << "Failed to tie property " << name << " to a pointer" << endl;
-  else {
-    tied_properties.push_back(property);
-    if (FGJSBBase::debug_lvl & 0x20) cout << name << endl;
-  }
-}
-
-//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-
-void FGPropertyManager::Tie (const string &name, int *pointer,
-                                          bool useDefault )
-{
-  SGPropertyNode* property = root->getNode(name.c_str(), true);
-  if (!property) {
-    cerr << "Could not get or create property " << name << endl;
-    return;
-  }
-
-  if (!property->tie(SGRawValuePointer<int>(pointer), useDefault))
-    cerr << "Failed to tie property " << name << " to a pointer" << endl;
-  else {
-    tied_properties.push_back(property);
-    if (FGJSBBase::debug_lvl & 0x20) cout << name << endl;
-  }
-}
-
-//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-
-void FGPropertyManager::Tie (const string &name, long *pointer,
-                                          bool useDefault )
-{
-  SGPropertyNode* property = root->getNode(name.c_str(), true);
-  if (!property) {
-    cerr << "Could not get or create property " << name << endl;
-    return;
-  }
-
-  if (!property->tie(SGRawValuePointer<long>(pointer), useDefault))
-    cerr << "Failed to tie property " << name << " to a pointer" << endl;
-  else {
-    tied_properties.push_back(property);
-    if (FGJSBBase::debug_lvl & 0x20) cout << name << endl;
-  }
-}
-
-//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-
-void FGPropertyManager::Tie (const string &name, float *pointer,
-                                          bool useDefault )
-{
-  SGPropertyNode* property = root->getNode(name.c_str(), true);
-  if (!property) {
-    cerr << "Could not get or create property " << name << endl;
-    return;
-  }
-
-  if (!property->tie(SGRawValuePointer<float>(pointer), useDefault))
-    cerr << "Failed to tie property " << name << " to a pointer" << endl;
-  else {
-    tied_properties.push_back(property);
-    if (FGJSBBase::debug_lvl & 0x20) cout << name << endl;
-  }
-}
-
-//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-
-void FGPropertyManager::Tie (const string &name, double *pointer, bool useDefault)
-{
-  SGPropertyNode* property = root->getNode(name.c_str(), true);
-  if (!property) {
-    cerr << "Could not get or create property " << name << endl;
-    return;
-  }
-
-  if (!property->tie(SGRawValuePointer<double>(pointer), useDefault))
-    cerr << "Failed to tie property " << name << " to a pointer" << endl;
-  else {
-    tied_properties.push_back(property);
-    if (FGJSBBase::debug_lvl & 0x20) cout << name << endl;
-  }
-}
-
 } // namespace JSBSim
