@@ -8,21 +8,21 @@
  ------------- Copyright (C) 1999  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
 --------------------------------------------------------------------------------
@@ -41,30 +41,20 @@ COMMENTS, REFERENCES,  and NOTES
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include <iostream>
-#include <cstdlib>
 #include <iomanip>
 
 #include "FGScript.h"
 #include "FGFDMExec.h"
-#include "input_output/FGXMLElement.h"
 #include "input_output/FGXMLFileRead.h"
 #include "initialization/FGInitialCondition.h"
 #include "models/FGInput.h"
 #include "math/FGCondition.h"
-#include "math/FGFunction.h"
+#include "math/FGFunctionValue.h"
 
 using namespace std;
 
 namespace JSBSim {
 
-IDENT(IdSrc,"$Id: FGScript.cpp,v 1.65 2017/02/25 14:23:18 bcoconni Exp $");
-IDENT(IdHdr,ID_FGSCRIPT);
-
-/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-GLOBAL DECLARATIONS
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
-
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
@@ -88,6 +78,8 @@ FGScript::~FGScript()
     delete Events[i].Condition;
     for (j=0; j<Events[i].Functions.size(); j++)
       delete Events[i].Functions[j];
+    for (j=0; j<Events[i].NotifyProperties.size(); j++)
+      delete Events[i].NotifyProperties[j];
   }
   Events.clear();
 
@@ -146,9 +138,6 @@ bool FGScript::LoadScript(const SGPath& script, double default_dT,
     return false;
   }
 
-  // Make sure that the desired time is reached and executed.
-  EndTime += 0.99*FDMExec->GetDeltaT();
-
   if (default_dT == 0.0)
     dt = run_element->GetAttributeValueAsNumber("dt");
   else {
@@ -158,7 +147,10 @@ bool FGScript::LoadScript(const SGPath& script, double default_dT,
   }
 
   FDMExec->Setdt(dt);
-  
+
+  // Make sure that the desired time is reached and executed.
+  EndTime += 0.99*FDMExec->GetDeltaT();
+
   // read aircraft and initialization files
 
   element = document->FindElement("use");
@@ -179,8 +171,9 @@ bool FGScript::LoadScript(const SGPath& script, double default_dT,
         return false;
       }
     } else {
-      cout << endl << "The initialization file specified in the script file (" << initialize
-                   << ") has been overridden with a specified file (" << initfile << ")." << endl;
+      cout << endl << "The initialization file specified in the script file ("
+           << initialize << ") has been overridden with a specified file ("
+           << initfile << ")." << endl;
       initialize = initfile;
     }
 
@@ -200,16 +193,20 @@ bool FGScript::LoadScript(const SGPath& script, double default_dT,
   while (element) {
     if (!FDMExec->GetInput()->Load(element))
       return false;
- 
+
     element = document->FindNextElement("input");
   }
 
   // Now, read output spec if given.
   element = document->FindElement("output");
+  SGPath scriptDir = SGPath(script.dir());
+  if (scriptDir.isNull())
+    scriptDir = SGPath(".");
+
   while (element) {
-    if (!FDMExec->GetOutput()->Load(element))
+    if (!FDMExec->GetOutput()->Load(element, scriptDir))
       return false;
- 
+
     element = document->FindNextElement("output");
   }
 
@@ -253,13 +250,14 @@ bool FGScript::LoadScript(const SGPath& script, double default_dT,
       }
       newEvent->Condition = newCondition;
     } else {
-      cerr << "No condition specified in script event " << newEvent->Name << endl;
+      cerr << "No condition specified in script event " << newEvent->Name
+           << endl;
       delete newEvent;
       return false;
     }
 
-    // Is there a delay between the time this event is triggered, and when the event
-    // actions are executed?
+    // Is there a delay between the time this event is triggered, and when the
+    // event actions are executed?
 
     Element* delay_element = event_element->FindElement("delay");
     if (delay_element)
@@ -282,20 +280,35 @@ bool FGScript::LoadScript(const SGPath& script, double default_dT,
       while (notify_property_element) {
         notifyPropertyName = notify_property_element->GetDataLine();
 
-        newEvent->NotifyPropertyNames.push_back(notifyPropertyName);
-        newEvent->NotifyProperties.push_back(0);
-          string caption_attribute = notify_property_element->GetAttributeValue("caption");
-          if (caption_attribute.empty()) {
-            newEvent->DisplayString.push_back(notifyPropertyName);
-          } else {
-            newEvent->DisplayString.push_back(caption_attribute);
+        if (notify_property_element->HasAttribute("apply")) {
+          string function_str = notify_property_element->GetAttributeValue("apply");
+          FGTemplateFunc* f = FDMExec->GetTemplateFunc(function_str);
+          if (f)
+            newEvent->NotifyProperties.push_back(new FGFunctionValue(notifyPropertyName, PropertyManager, f));
+          else {
+            cerr << notify_property_element->ReadFrom()
+              << fgred << highint << "  No function by the name "
+              << function_str << " has been defined. This property will "
+              << "not be logged. You should check your configuration file."
+              << reset << endl;
           }
+        }
+        else
+          newEvent->NotifyProperties.push_back(new FGPropertyValue(notifyPropertyName, PropertyManager));
+
+        string caption_attribute = notify_property_element->GetAttributeValue("caption");
+        if (caption_attribute.empty()) {
+          newEvent->DisplayString.push_back(notifyPropertyName);
+        } else {
+          newEvent->DisplayString.push_back(caption_attribute);
+        }
 
         notify_property_element = notify_element->FindNextElement("property");
       }
     }
 
-    // Read set definitions (these define the actions to be taken when the event is triggered).
+    // Read set definitions (these define the actions to be taken when the event
+    // is triggered).
     set_element = event_element->FindElement("set");
     while (set_element) {
       prop_name = set_element->GetAttributeValue("name");
@@ -306,14 +319,14 @@ bool FGScript::LoadScript(const SGPath& script, double default_dT,
       }
       newEvent->SetParamName.push_back( prop_name );
 
-      //Todo - should probably do some safety checking here to make sure one or the other
-      //of value or function is specified.
+      // Todo - should probably do some safety checking here to make sure one or
+      // the other of value or function is specified.
       if (!set_element->GetAttributeValue("value").empty()) {
         value = set_element->GetAttributeValueAsNumber("value");
-        newEvent->Functions.push_back((FGFunction*)0L);
+        newEvent->Functions.push_back(nullptr);
       } else if (set_element->FindElement("function")) {
         value = 0.0;
-        newEvent->Functions.push_back(new FGFunction(PropertyManager, set_element->FindElement("function")));
+        newEvent->Functions.push_back(new FGFunction(FDMExec, set_element->FindElement("function")));
       }
       newEvent->SetValue.push_back(value);
       newEvent->OriginalValue.push_back(0.0);
@@ -379,14 +392,15 @@ bool FGScript::RunScript(void)
     struct event &thisEvent = Events[ev_ctr];
 
     // Determine whether the set of conditional tests for this condition equate
-    // to true and should cause the event to execute. If the conditions evaluate 
+    // to true and should cause the event to execute. If the conditions evaluate
     // to true, then the event is triggered. If the event is not persistent,
-    // then this trigger will remain set true. If the event is persistent,
-    // the trigger will reset to false when the condition evaluates to false.
+    // then this trigger will remain set true. If the event is persistent, the
+    // trigger will reset to false when the condition evaluates to false.
     if (thisEvent.Condition->Evaluate()) {
       if (!thisEvent.Triggered) {
 
-        // The conditions are true, do the setting of the desired Event parameters
+        // The conditions are true, do the setting of the desired Event
+        // parameters
         for (i=0; i<thisEvent.SetValue.size(); i++) {
           if (thisEvent.SetParam[i] == 0L) { // Late bind property if necessary
             if (PropertyManager->HasNode(thisEvent.SetParamName[i])) {
@@ -473,28 +487,25 @@ bool FGScript::RunScript(void)
       if (thisEvent.Notify && !thisEvent.Notified) {
         if (thisEvent.NotifyKML) {
           cout << endl << "<Placemark>" << endl;
-          cout << "  <name> " << currentTime << " seconds" << " </name>" << endl;
+          cout << "  <name> " << currentTime << " seconds" << " </name>"
+               << endl;
           cout << "  <description>" << endl;
           cout << "  <![CDATA[" << endl;
-          cout << "  <b>" << thisEvent.Name << " (Event " << event_ctr << ")" << " executed at time: " << currentTime << "</b><br/>" << endl;
+          cout << "  <b>" << thisEvent.Name << " (Event " << event_ctr << ")"
+               << " executed at time: " << currentTime << "</b><br/>" << endl;
         } else  {
           cout << endl << underon
                << highint << thisEvent.Name << normint << underoff
-               << " (Event " << event_ctr << ")" 
-               << " executed at time: " << highint << currentTime << normint << endl;
+               << " (Event " << event_ctr << ")"
+               << " executed at time: " << highint << currentTime << normint
+               << endl;
         }
         if (!thisEvent.Description.empty()) {
           cout << "    " << thisEvent.Description << endl;
         }
         for (j=0; j<thisEvent.NotifyProperties.size();j++) {
-          if (thisEvent.NotifyProperties[j] == 0) {
-            if (PropertyManager->HasNode(thisEvent.NotifyPropertyNames[j])) {
-              thisEvent.NotifyProperties[j] = PropertyManager->GetNode(thisEvent.NotifyPropertyNames[j]);
-            } else {
-              throw("Could not find property named "+thisEvent.NotifyPropertyNames[j]+" in script.");
-            }
-          }
-          cout << "    " << thisEvent.DisplayString[j] << " = " << thisEvent.NotifyProperties[j]->getDoubleValue();
+          cout << "    " << thisEvent.DisplayString[j] << " = "
+               << thisEvent.NotifyProperties[j]->getDoubleValue();
           if (thisEvent.NotifyKML) cout << " <br/>";
           cout << endl;
         }
@@ -504,9 +515,11 @@ bool FGScript::RunScript(void)
           cout << "  <Point>" << endl;
           cout << "    <altitudeMode> absolute </altitudeMode>" << endl;
           cout << "    <extrude> 1 </extrude>" << endl;
-          cout << "    <coordinates>" << FDMExec->GetPropagate()->GetLongitudeDeg()
-            << "," << FDMExec->GetPropagate()->GetGeodLatitudeDeg()
-            << "," << FDMExec->GetPropagate()->GetAltitudeASLmeters() << "</coordinates>" << endl;
+          cout << "    <coordinates>"
+               << FDMExec->GetPropagate()->GetLongitudeDeg() << ","
+               << FDMExec->GetPropagate()->GetGeodLatitudeDeg() << ","
+               << FDMExec->GetPropagate()->GetAltitudeASLmeters()
+               << "</coordinates>" << endl;
           cout << "  </Point>" << endl;
           cout << "</Placemark>" << endl;
         }
@@ -551,18 +564,16 @@ void FGScript::Debug(int from)
       cout << endl;
       cout << "Script: \"" << ScriptName << "\"" << endl;
       cout << "  begins at " << StartTime << " seconds and runs to " << EndTime
-        << " seconds with dt = " << setprecision(6) << FDMExec->GetDeltaT() << " (" <<
-        ceil(1.0/FDMExec->GetDeltaT()) << " Hz)" << endl;
+           << " seconds with dt = " << setprecision(6) << FDMExec->GetDeltaT()
+           << " (" << ceil(1.0/FDMExec->GetDeltaT()) << " Hz)" << endl;
       cout << endl;
 
-      FGPropertyReader::const_iterator it;
-      for (it = LocalProperties.begin(); it != LocalProperties.end(); ++it) {
-        FGPropertyNode* node = *it;
+      for (auto node: LocalProperties) {
         cout << "Local property: " << node->GetName()
              << " = " << node->getDoubleValue()
              << endl;
       }
-      
+
       if (LocalProperties.empty()) cout << endl;
 
       for (unsigned i=0; i<Events.size(); i++) {
@@ -586,36 +597,38 @@ void FGScript::Debug(int from)
         for (unsigned j=0; j<Events[i].SetValue.size(); j++) {
           if (Events[i].SetValue[j] == 0.0 && Events[i].Functions[j] != 0L) {
             if (Events[i].SetParam[j] == 0) {
-              if (Events[i].SetParamName[j].size() == 0) {
-              cerr << fgred << highint << endl
-                   << "  An attempt has been made to access a non-existent property" << endl
-                   << "  in this event. Please check the property names used, spelling, etc."
-                   << reset << endl;
-              exit(-1);
+              if (Events[i].SetParamName[j].empty()) {
+                stringstream s;
+                s << "  An attempt has been made to access a non-existent property" << endl
+                  << "  in this event. Please check the property names used, spelling, etc.";
+                cerr << fgred << highint << endl << s.str() << reset << endl;
+                throw BaseException(s.str());
               } else {
                 cout << endl << "      set " << Events[i].SetParamName[j]
                      << " to function value (Late Bound)";
-            }
+              }
             } else {
-            cout << endl << "      set " << Events[i].SetParam[j]->GetRelativeName("/fdm/jsbsim/")
-                 << " to function value";
+              cout << endl << "      set "
+                   << Events[i].SetParam[j]->GetRelativeName("/fdm/jsbsim/")
+                   << " to function value";
             }
           } else {
             if (Events[i].SetParam[j] == 0) {
-              if (Events[i].SetParamName[j].size() == 0) {
-              cerr << fgred << highint << endl
-                   << "  An attempt has been made to access a non-existent property" << endl
-                   << "  in this event. Please check the property names used, spelling, etc."
-                   << reset << endl;
-              exit(-1);
+              if (Events[i].SetParamName[j].empty()) {
+                stringstream s;
+                s << "  An attempt has been made to access a non-existent property" << endl
+                  << "  in this event. Please check the property names used, spelling, etc.";
+                cerr << fgred << highint << endl << s.str() << reset << endl;
+                throw BaseException(s.str());
               } else {
                 cout << endl << "      set " << Events[i].SetParamName[j]
                      << " to function value (Late Bound)";
-            }
+              }
             } else {
-            cout << endl << "      set " << Events[i].SetParam[j]->GetRelativeName("/fdm/jsbsim/")
-                 << " to " << Events[i].SetValue[j];
-          }
+              cout << endl << "      set "
+                   << Events[i].SetParam[j]->GetRelativeName("/fdm/jsbsim/")
+                   << " to " << Events[i].SetValue[j];
+            }
           }
 
           switch (Events[i].Type[j]) {
@@ -653,14 +666,15 @@ void FGScript::Debug(int from)
         if (Events[i].Notify) {
           if (Events[i].NotifyProperties.size() > 0) {
             if (Events[i].NotifyKML) {
-              cout << "  Notifications (KML Format):" << endl << "    {" << endl;
+              cout << "  Notifications (KML Format):" << endl << "    {"
+                   << endl;
             } else {
               cout << "  Notifications:" << endl << "    {" << endl;
             }
-            for (unsigned j=0; j<Events[i].NotifyPropertyNames.size();j++) {
+            for (unsigned j=0; j<Events[i].NotifyProperties.size();j++) {
               cout << "      "
-                   << Events[i].NotifyPropertyNames[j]
-                   << endl;
+                   << Events[i].NotifyProperties[j]->GetPrintableName()
+                    << endl;
             }
             cout << "    }" << endl;
           }
@@ -681,8 +695,6 @@ void FGScript::Debug(int from)
   }
   if (debug_lvl & 64) {
     if (from == 0) { // Constructor
-      cout << IdSrc << endl;
-      cout << IdHdr << endl;
     }
   }
 }
