@@ -9,21 +9,21 @@
  ------------- Copyright (C) 1999  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
 --------------------------------------------------------------------------------
@@ -39,16 +39,20 @@ INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 #if defined(_MSC_VER) || defined(__MINGW32__)
-#include <WS2tcpip.h>
+#include <ws2tcpip.h>
+#elif defined(__OpenBSD__)
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <netdb.h>
+#include <unistd.h>
+#include <fcntl.h>
 #else
 #include <fcntl.h>
+#include <unistd.h>
 #endif
-#include <iostream>
 #include <iomanip>
 #include <cstring>
-#include <cstdio>
 #include "FGfdmSocket.h"
-#include "string_utilities.h"
 
 using std::cout;
 using std::cerr;
@@ -57,211 +61,163 @@ using std::string;
 
 namespace JSBSim {
 
-IDENT(IdSrc,"$Id: FGfdmSocket.cpp,v 1.32 2017/03/11 12:12:12 bcoconni Exp $");
-IDENT(IdHdr,ID_FDMSOCKET);
-
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 #if defined(_MSC_VER) || defined(__MINGW32__)
-static bool LoadWinSockDLL(void)
+static bool LoadWinSockDLL(int debug_lvl)
 {
   WSADATA wsaData;
   if (WSAStartup(MAKEWORD(1, 1), &wsaData)) {
-	cout << "Winsock DLL not initialized ..." << endl;
-	return false;
+    cerr << "Winsock DLL not initialized ..." << endl;
+    return false;
   }
 
-  cout << "Winsock DLL loaded ..." << endl;
+  if (debug_lvl > 0)
+    cout << "Winsock DLL loaded ..." << endl;
+
   return true;
 }
 #endif
 
-FGfdmSocket::FGfdmSocket(const string& address, int port, int protocol)
+FGfdmSocket::FGfdmSocket(const string& address, int port, int protocol, int precision)
 {
   sckt = sckt_in = 0;
   Protocol = (ProtocolType)protocol;
   connected = false;
+  struct addrinfo *addr = nullptr;
+  this->precision = precision;
 
   #if defined(_MSC_VER) || defined(__MINGW32__)
-  if (!LoadWinSockDLL()) return;
+  if (!LoadWinSockDLL(debug_lvl)) return;
   #endif
 
-  if (!is_number(address)) {
-    if ((host = gethostbyname(address.c_str())) == NULL) {
-      cout << "Could not get host net address by name..." << endl;
-    }
-  } else {
-    unsigned long ip;
-    ip = inet_addr(address.c_str());
-    if ((host = gethostbyaddr((char*)&ip, sizeof(ip), PF_INET)) == NULL) {
-      cout << "Could not get host net address by number..." << endl;
-    }
+  struct addrinfo hints;
+  memset(&hints, 0, sizeof(struct addrinfo));
+  hints.ai_family = AF_INET;
+  if (protocol == ptUDP)
+    hints.ai_socktype = SOCK_DGRAM;
+  else
+    hints.ai_socktype = SOCK_STREAM;
+  hints.ai_protocol = 0;
+  if (!is_number(address))
+    hints.ai_flags = AI_ADDRCONFIG;
+  else
+    hints.ai_flags = AI_NUMERICHOST;
+
+  int failure = getaddrinfo(address.c_str(), NULL, &hints, &addr);
+  if (failure || !addr) {
+    cerr << "Could not get host net address " << address;
+
+    if (hints.ai_flags == AI_NUMERICHOST)
+       cerr << " by number..." << endl;
+    else
+      cerr << " by name..." << endl;
+
+    cerr  << gai_strerror(failure) << endl;
+
+    freeaddrinfo(addr);
+    return;
   }
 
-  if (host != NULL) {
-    if (protocol == ptUDP) {  //use udp protocol
-       sckt = socket(AF_INET, SOCK_DGRAM, 0);
-       cout << "Creating UDP socket on port " << port << endl;
-    }
-    else { //use tcp protocol
-       sckt = socket(AF_INET, SOCK_STREAM, 0);
-       cout << "Creating TCP socket on port " << port << endl;
-    }
+  sckt = socket(addr->ai_family, addr->ai_socktype, addr->ai_protocol);
 
-    if (sckt >= 0) {  // successful
-      memset(&scktName, 0, sizeof(struct sockaddr_in));
-      scktName.sin_family = AF_INET;
-      scktName.sin_port = htons(port);
-      memcpy(&scktName.sin_addr, host->h_addr_list[0], host->h_length);
-      int len = sizeof(struct sockaddr_in);
-      if (connect(sckt, (struct sockaddr*)&scktName, len) == 0) {   // successful
+  if (debug_lvl > 0) {
+    if (protocol == ptUDP)  //use udp protocol
+      cout << "Creating UDP socket on port " << port << endl;
+    else //use tcp protocol
+      cout << "Creating TCP socket on port " << port << endl;
+  }
+
+  if (sckt >= 0) {  // successful
+    int len = sizeof(struct sockaddr_in);
+    memcpy(&scktName, addr->ai_addr, len);
+    scktName.sin_port = htons(port);
+
+    if (connect(sckt, (struct sockaddr*)&scktName, len) == 0) {   // successful
+      if (debug_lvl > 0)
         cout << "Successfully connected to socket for output ..." << endl;
-        connected = true;
-      } else {                // unsuccessful
-        cout << "Could not connect to socket for output ..." << endl;
-      }
-    } else {          // unsuccessful
-      cout << "Could not create socket for FDM output, error = " << errno << endl;
-    }
+      connected = true;
+    } else                // unsuccessful
+      cerr << "Could not connect to socket for output ..." << endl;
+  } else          // unsuccessful
+    cerr << "Could not create socket for FDM output, error = " << errno << endl;
+
+  freeaddrinfo(addr);
 
-  }
   Debug(0);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-// assumes UDP socket on localhost, for inbound datagrams
-FGfdmSocket::FGfdmSocket(int port, int protocol, int direction) // assumes UDP
+// assumes TCP or UDP socket on localhost, for inbound datagrams
+FGfdmSocket::FGfdmSocket(int port, int protocol, int precision)
 {
   sckt = -1;
   connected = false;
   Protocol = (ProtocolType)protocol;
-  Direction = (DirectionType) direction;
- 
+  string ProtocolName;
+  this->precision = precision;
+
 #if defined(_MSC_VER) || defined(__MINGW32__)
-  if (!LoadWinSockDLL()) return;
+  if (!LoadWinSockDLL(debug_lvl)) return;
 #endif
 
   if (Protocol == ptUDP) {  //use udp protocol
+    ProtocolName = "UDP";
     sckt = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
 #if defined(_MSC_VER) || defined(__MINGW32__)
-	u_long NonBlock = 1; // True
-	ioctlsocket(sckt, FIONBIO, &NonBlock);
+    u_long NonBlock = 1; // True
+    ioctlsocket(sckt, FIONBIO, &NonBlock);
 #else
-	fcntl(sckt, F_SETFL, O_NONBLOCK);
+    fcntl(sckt, F_SETFL, O_NONBLOCK);
 #endif
-    cout << "Creating UDP input socket on port " << port << endl;
   }
-  
-    if (sckt != -1) { 
-      memset(&scktName, 0, sizeof(struct sockaddr_in));
-      scktName.sin_family = AF_INET;
-      scktName.sin_port = htons(port);
-      scktName.sin_addr.s_addr = htonl(INADDR_ANY);
-      int len = sizeof(struct sockaddr_in);
-      if (bind(sckt, (struct sockaddr*)&scktName, len) != -1) { 
-        cout << "Successfully bound to UDP input socket on port " << port << endl <<endl;
-        connected = true;
-      } else {                // unsuccessful
-        cout << "Could not bind to UDP input socket, error = " << errno << endl;
-      }
-    } else {          // unsuccessful
-      cout << "Could not create socket for UDP input, error = " << errno << endl;
-    }
-    
-  
-  Debug(0);
-}
-
-//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-
-FGfdmSocket::FGfdmSocket(const string& address, int port) // assumes TCP
-{
-  sckt = sckt_in = 0;
-  connected = false;
-  Protocol = ptTCP;
-
-  #if defined(_MSC_VER) || defined(__MINGW32__)
-  if (!LoadWinSockDLL()) return;
-  #endif
-
-  cout << "... Socket Configuration Sanity Check ..." << endl;
-  cout << "Host name...   " << address << ",  Port...  " << port << "." << endl;
-  cout << "Host name... (char)  " << address.c_str() << "." << endl;
-
-  if (!is_number(address)) {
-    if ((host = gethostbyname(address.c_str())) == NULL) {
-      cout << "Could not get host net address by name..." << endl;
-    }
-  } else {
-    if ((host = gethostbyaddr(address.c_str(), address.size(), PF_INET)) == NULL) {
-      cout << "Could not get host net address by number..." << endl;
-    }
-  }
-
-  if (host != NULL) {
-    cout << "Got host net address..." << endl;
+  else {
+    ProtocolName = "TCP";
     sckt = socket(AF_INET, SOCK_STREAM, 0);
-
-    if (sckt >= 0) {  // successful
-      memset(&scktName, 0, sizeof(struct sockaddr_in));
-      scktName.sin_family = AF_INET;
-      scktName.sin_port = htons(port);
-      memcpy(&scktName.sin_addr, host->h_addr_list[0], host->h_length);
-      int len = sizeof(struct sockaddr_in);
-      if (connect(sckt, (struct sockaddr*)&scktName, len) == 0) {   // successful
-        cout << "Successfully connected to socket for output ..." << endl;
-        connected = true;
-      } else {                // unsuccessful
-        cout << "Could not connect to socket for output ..." << endl;
-      }
-    } else {          // unsuccessful
-      cout << "Could not create socket for FDM output, error = " << errno << endl;
-    }
   }
-  Debug(0);
-}
 
-//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+  if (debug_lvl > 0)
+    cout << "Creating input " << ProtocolName << " socket on port " << port
+         << endl;
 
-FGfdmSocket::FGfdmSocket(int port) // assumes TCP
-{
-  connected = false;
-  unsigned long NoBlock = true;
-  Protocol = ptTCP;
-
-  #if defined(_MSC_VER) || defined(__MINGW32__)
-  if (!LoadWinSockDLL()) return;
-  #endif
-
-  sckt = socket(AF_INET, SOCK_STREAM, 0);
-
-  if (sckt >= 0) {  // successful
+  if (sckt != -1) {
     memset(&scktName, 0, sizeof(struct sockaddr_in));
     scktName.sin_family = AF_INET;
     scktName.sin_port = htons(port);
+
+    if (Protocol == ptUDP)
+      scktName.sin_addr.s_addr = htonl(INADDR_ANY);
+
     int len = sizeof(struct sockaddr_in);
-    if (bind(sckt, (struct sockaddr*)&scktName, len) == 0) {   // successful
-      cout << "Successfully bound to socket for input on port " << port << endl;
-      if (listen(sckt, 5) >= 0) { // successful listen()
-        #if defined(_MSC_VER) || defined(__MINGW32__)
+    if (bind(sckt, (struct sockaddr*)&scktName, len) != -1) {
+      if (debug_lvl > 0)
+        cout << "Successfully bound to " << ProtocolName
+             << " input socket on port " << port << endl << endl;
+
+      if (Protocol == ptTCP) {
+        if (listen(sckt, 5) >= 0) { // successful listen()
+#if defined(_MSC_VER) || defined(__MINGW32__)
+          u_long NoBlock = 1;
           ioctlsocket(sckt, FIONBIO, &NoBlock);
           sckt_in = accept(sckt, (struct sockaddr*)&scktName, &len);
-        #else
-          ioctl(sckt, FIONBIO, &NoBlock);
+#else
+          int flags = fcntl(sckt, F_GETFL, 0);
+          fcntl(sckt, F_SETFL, flags | O_NONBLOCK);
           sckt_in = accept(sckt, (struct sockaddr*)&scktName, (socklen_t*)&len);
-        #endif
-      } else {
-        cerr << "Could not listen ..." << endl;
-      }
-      connected = true;
-    } else {                // unsuccessful
-      cerr << "Could not bind to socket for input ..." << endl;
-    }
-  } else {          // unsuccessful
-    cerr << "Could not create socket for FDM input, error = " << errno << endl;
-  }
+#endif
+          connected = true;
+        } else
+          cerr << "Could not listen ..." << endl;
+      } else
+        connected = true;
+    } else                // unsuccessful
+        cerr << "Could not bind to " << ProtocolName << " input socket, error = "
+             << errno << endl;
+  } else          // unsuccessful
+      cerr << "Could not create " << ProtocolName << " socket for input, error = "
+           << errno << endl;
 
   Debug(0);
 }
@@ -282,7 +238,6 @@ string FGfdmSocket::Receive(void)
   char buf[1024];
   int len = sizeof(struct sockaddr_in);
   int num_chars=0;
-  unsigned long NoBlock = true;
   string data;      // todo: should allocate this with a standard size as a
                     // class attribute and pass as a reference?
 
@@ -294,9 +249,11 @@ string FGfdmSocket::Receive(void)
     #endif
     if (sckt_in > 0) {
       #if defined(_MSC_VER) || defined(__MINGW32__)
-         ioctlsocket(sckt_in, FIONBIO,&NoBlock);
+        u_long NoBlock = 1;
+        ioctlsocket(sckt_in, FIONBIO, &NoBlock);
       #else
-         ioctl(sckt_in, FIONBIO, &NoBlock);
+        int flags = fcntl(sckt_in, F_GETFL, 0);
+        fcntl(sckt_in, F_SETFL, flags | O_NONBLOCK);
       #endif
       send(sckt_in, "Connected to JSBSim server\nJSBSim> ", 35, 0);
     }
@@ -320,15 +277,15 @@ string FGfdmSocket::Receive(void)
     }
 #endif
   }
-  
+
   // this is for FGUDPInputSocket
   if (sckt >= 0 && Protocol == ptUDP) {
     struct sockaddr addr;
     socklen_t fromlen = sizeof addr;
     num_chars = recvfrom(sckt, buf, sizeof buf, 0, (struct sockaddr*)&addr, &fromlen);
-    if (num_chars != -1) data.append(buf, num_chars); 
+    if (num_chars != -1) data.append(buf, num_chars);
   }
-  
+
   return data;
 }
 
@@ -383,7 +340,7 @@ void FGfdmSocket::Append(const char* item)
 void FGfdmSocket::Append(double item)
 {
   if (buffer.tellp() > 0) buffer << ',';
-  buffer << std::setw(12) << std::setprecision(7) << item;
+  buffer << std::setw(12) << std::setprecision(precision) << item;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -414,6 +371,29 @@ void FGfdmSocket::Send(const char *data, int length)
   }
 }
 
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+void FGfdmSocket::WaitUntilReadable(void)
+{
+  if (sckt_in <= 0)
+    return;
+
+  fd_set fds;
+  FD_ZERO(&fds);
+  FD_SET(sckt_in, &fds);
+  select(sckt_in+1, &fds, NULL, NULL, NULL);
+
+  /*
+    If you want to check select return status:
+
+    int recVal = select(sckt_in+1, &fds, NULL, NULL, NULL);
+    recVal: received value
+    0,             if socket timeout
+    -1,            if socket error
+    anithing else, if socket is readable
+  */
+}
+
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 //    The bitmasked value choices are as follows:
 //    unset: In this case (the default) JSBSim would only print
@@ -453,8 +433,6 @@ void FGfdmSocket::Debug(int from)
   }
   if (debug_lvl & 64) {
     if (from == 0) { // Constructor
-      cout << IdSrc << endl;
-      cout << IdHdr << endl;
     }
   }
 }
