@@ -7,21 +7,21 @@ Date started: Unknown
  ------------- Copyright (C) 2001  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 HISTORY
 --------------------------------------------------------------------------------
@@ -43,40 +43,17 @@ INCLUDES
 #include <string>
 #include <iosfwd>
 
+#include "FGJSBBase.h"
 #include "FGColumnVector3.h"
 
-/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-DEFINITIONS
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
-
-#define ID_MATRIX33 "$Id: FGMatrix33.h,v 1.17 2012/11/22 22:04:06 bcoconni Exp $"
-
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 namespace JSBSim {
 
-class FGColumnVector3;
 class FGQuaternion;
 
-/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-CLASS DOCUMENTATION
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
-
-/** Exception convenience class.
-  */
-
-/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-DECLARATION: MatrixException
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
-
-class MatrixException //: public FGJSBBase
-{
-public:
-  std::string Message;
-};
-
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS DOCUMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
@@ -163,7 +140,8 @@ public:
 
   /** Prints the contents of the matrix.
       @param delimeter the item separator (tab or comma, etc.)
-      @param prefix an additional prefix that is used to indent the 3X3 matrix printout
+      @param prefix an additional prefix that is used to indent the 3X3 matrix
+             printout
       @return a string with the delimeter-separated contents of the matrix  */
   std::string Dump(const std::string& delimiter, const std::string& prefix) const;
 
@@ -236,8 +214,8 @@ public:
    unsigned int Cols(void) const { return eColumns; }
 
   /** Transposed matrix.
-      This function only returns the transpose of this matrix. This matrix itself
-      remains unchanged.
+      This function only returns the transpose of this matrix. This matrix
+      itself remains unchanged.
       @return the transposed matrix.
    */
   FGMatrix33 Transposed(void) const {
@@ -324,6 +302,24 @@ public:
     return *this;
   }
 
+  /** Assignment operator.
+
+      @param lv initializer list of at most 9 values.
+
+      Copy the content of the list into *this. */
+  FGMatrix33& operator=(std::initializer_list<double> lv)
+  {
+    double *v = data;
+    for(auto& x: lv) {
+      *v = x;
+      v += 3;
+      if (v-data > 8)
+        v -= 8;
+    }
+
+    return *this;
+  }
+
   /** Matrix vector multiplication.
 
       @param v vector to multiply with.
