@@ -8,21 +8,21 @@
  ------------- Copyright (C) 2001  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
 --------------------------------------------------------------------------------
@@ -36,25 +36,21 @@ JSB  1/9/00          Created
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
+#include <assert.h>
+
 #include "FGTable.h"
 #include "input_output/FGXMLElement.h"
-#include "input_output/FGPropertyManager.h"
-#include <iostream>
-#include <sstream>
-#include <cstdlib>
 
 using namespace std;
 
 namespace JSBSim {
 
-IDENT(IdSrc,"$Id: FGTable.cpp,v 1.33 2017/03/11 19:31:48 bcoconni Exp $");
-IDENT(IdHdr,ID_TABLE);
-
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-FGTable::FGTable(int NRows) : nRows(NRows), nCols(1), PropertyManager(0)
+FGTable::FGTable(int NRows)
+  : nRows(NRows), nCols(1), PropertyManager(nullptr)
 {
   Type = tt1D;
   colCounter = 0;
@@ -68,7 +64,8 @@ FGTable::FGTable(int NRows) : nRows(NRows), nCols(1), PropertyManager(0)
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-FGTable::FGTable(int NRows, int NCols) : nRows(NRows), nCols(NCols), PropertyManager(0)
+FGTable::FGTable(int NRows, int NCols)
+  : nRows(NRows), nCols(NCols), PropertyManager(nullptr)
 {
   Type = tt2D;
   colCounter = 1;
@@ -112,22 +109,29 @@ FGTable::FGTable(const FGTable& t) : PropertyManager(t.PropertyManager)
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
+unsigned int FindNumColumns(const string& test_line)
+{
+  // determine number of data columns in table (first column is row lookup - don't count)
+  size_t position=0;
+  unsigned int nCols=0;
+  while ((position = test_line.find_first_not_of(" \t", position)) != string::npos) {
+    nCols++;
+    position = test_line.find_first_of(" \t", position);
+  }
+  return nCols;
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
 FGTable::FGTable(FGPropertyManager* propMan, Element* el,
-                 const std::string& prefix)
-  : PropertyManager(propMan), Prefix(prefix)
+                 const std::string& Prefix)
+  : PropertyManager(propMan)
 {
   unsigned int i;
 
   stringstream buf;
-  string property_string;
-  string lookup_axis;
-  string call_type;
-  string parent_type;
   string brkpt_string;
-  FGPropertyNode* node;
-  Element *tableData=0;
-  Element *parent_element=0;
-  Element *axisElement=0;
+  Element *tableData = nullptr;
   string operation_types = "function, product, sum, difference, quotient,"
                            "pow, abs, sin, cos, asin, acos, tan, atan, table";
 
@@ -137,59 +141,57 @@ FGTable::FGTable(FGPropertyManager* propMan, Element* el,
 
   internal = false;
   Name = el->GetAttributeValue("name"); // Allow this table to be named with a property
-  call_type = el->GetAttributeValue("type");
+  string call_type = el->GetAttributeValue("type");
   if (call_type == string("internal")) {
-    parent_element = el->GetParent();
-    parent_type = parent_element->GetName();
+    Element* parent_element = el->GetParent();
+    string parent_type = parent_element->GetName();
     if (operation_types.find(parent_type) == string::npos) {
       internal = true;
     } else {
       // internal table is a child element of a restricted type
-      throw("  An internal table cannot be nested within another type,"
-            " such as a function. The 'internal' keyword is ignored.");
+      std::cerr << el->ReadFrom()
+                << "  An internal table cannot be nested within another type,"
+                << " such as a function. The 'internal' keyword of table "
+                << Name << "is ignored." << endl;
     }
   } else if (!call_type.empty()) {
-    throw("  An unknown table type attribute is listed: "  
-          ". Execution cannot continue.");
+    std::cerr << el->ReadFrom()
+              <<"  An unknown table type attribute is listed: " << call_type
+              << endl;
+    throw BaseException("Unknown table type.");
   }
 
   // Determine and store the lookup properties for this table unless this table
-  // is part of a 3D table, in which case its independentVar property indexes will
-  // be set by a call from the owning table during creation
+  // is part of a 3D table, in which case its independentVar property indexes
+  // will be set by a call from the owning table during creation
 
   dimension = 0;
 
-  axisElement = el->FindElement("independentVar");
+  Element* axisElement = el->FindElement("independentVar");
   if (axisElement) {
 
     // The 'internal' attribute of the table element cannot be specified
     // at the same time that independentVars are specified.
     if (internal) {
-      cerr << endl << fgred << "  This table specifies both 'internal' call type" << endl;
-      cerr << "  and specific lookup properties via the 'independentVar' element." << endl;
-      cerr << "  These are mutually exclusive specifications. The 'internal'" << endl;
-      cerr << "  attribute will be ignored." << fgdef << endl << endl;
+      cerr  << el->ReadFrom()
+            << fgred << "  This table specifies both 'internal' call type" << endl
+            << "  and specific lookup properties via the 'independentVar' element." << endl
+            << "  These are mutually exclusive specifications. The 'internal'" << endl
+            << "  attribute will be ignored." << fgdef << endl << endl;
       internal = false;
     }
 
-    for (i=0; i<3; i++) lookupProperty[i] = 0;
-
     while (axisElement) {
-      property_string = axisElement->GetDataLine();
+      string property_string = axisElement->GetDataLine();
       if (property_string.find("#") != string::npos) {
         if (is_number(Prefix)) {
           property_string = replace(property_string,"#",Prefix);
         }
       }
-      // The property string passed into GetNode() must have no spaces or tabs.
-      node = PropertyManager->GetNode(property_string);
 
-      if (node == 0) {
-        cerr << axisElement->ReadFrom();
-        throw("IndependentVar property, " + property_string + " in Table definition is not defined.");
-      }
-
-      lookup_axis = axisElement->GetAttributeValue("lookup");
+      FGPropertyValue_ptr node = new FGPropertyValue(property_string,
+                                                     PropertyManager);
+      string lookup_axis = axisElement->GetAttributeValue("lookup");
       if (lookup_axis == string("row")) {
         lookupProperty[eRow] = node;
       } else if (lookup_axis == string("column")) {
@@ -197,7 +199,7 @@ FGTable::FGTable(FGPropertyManager* propMan, Element* el,
       } else if (lookup_axis == string("table")) {
         lookupProperty[eTable] = node;
       } else if (!lookup_axis.empty()) {
-        throw("Lookup table axis specification not understood: " + lookup_axis);
+        throw BaseException("Lookup table axis specification not understood: " + lookup_axis);
       } else { // assumed single dimension table; row lookup
         lookupProperty[eRow] = node;
       }
@@ -217,15 +219,20 @@ FGTable::FGTable(FGPropertyManager* propMan, Element* el,
       if (FindNumColumns(test_line) == 2) dimension = 1;    // 1D table
       else if (FindNumColumns(test_line) > 2) dimension = 2; // 2D table
       else {
-        cerr << "Invalid number of columns in table" << endl;
+        std::cerr << tableData->ReadFrom()
+                  << "Invalid number of columns in table" << endl;
       }
     }
 
   } else {
     brkpt_string = el->GetAttributeValue("breakPoint");
     if (brkpt_string.empty()) {
-     // no independentVars found, and table is not marked as internal, nor is it a 3D table
-      throw("No independent variable found for table.");
+     // no independentVars found, and table is not marked as internal, nor is it
+     // a 3D table
+      std::cerr << el->ReadFrom()
+                << "No independentVars found, and table is not marked as internal,"
+                << " nor is it a 3D table." << endl;
+      throw BaseException("No independent variable found for table.");
     }
   }
   // end lookup property code
@@ -238,8 +245,16 @@ FGTable::FGTable(FGPropertyManager* propMan, Element* el,
   }
 
   for (i=0; i<tableData->GetNumDataLines(); i++) {
-    buf << tableData->GetDataLine(i) << string(" ");
+    string line = tableData->GetDataLine(i);
+    if (line.find_first_not_of("0123456789.-+eE \t\n") != string::npos) {
+      cerr << " In file " << tableData->GetFileName() << endl
+           << "   Illegal character found in line "
+           << tableData->GetLineNumber() + i + 1 << ": " << endl << line << endl;
+      throw BaseException("Illegal character");
+    }
+    buf << line << " ";
   }
+
   switch (dimension) {
   case 1:
     nRows = tableData->GetNumDataLines();
@@ -257,9 +272,15 @@ FGTable::FGTable(FGPropertyManager* propMan, Element* el,
 
     if (nRows >= 2) {
       nCols = FindNumColumns(tableData->GetDataLine(0));
-      if (nCols < 2) throw(string("Not enough columns in table data."));
+      if (nCols < 2) {
+        std::cerr << tableData->ReadFrom()
+                  << "Not enough columns in table data" << endl;
+        throw BaseException("Not enough columns in table data.");
+      }
     } else {
-      throw(string("Not enough rows in the table data."));
+      std::cerr << tableData->ReadFrom()
+                << "Not enough rows in table data" << endl;
+      throw BaseException("Not enough rows in the table data.");
     }
 
     Type = tt2D;
@@ -285,8 +306,8 @@ FGTable::FGTable(FGPropertyManager* propMan, Element* el,
     for (i=0; i<nTables; i++) {
       Tables.push_back(new FGTable(PropertyManager, tableData));
       Data[i+1][1] = tableData->GetAttributeValueAsNumber("breakPoint");
-      Tables[i]->SetRowIndexProperty(lookupProperty[eRow]);
-      Tables[i]->SetColumnIndexProperty(lookupProperty[eColumn]);
+      Tables[i]->lookupProperty[eRow] = lookupProperty[eRow];
+      Tables[i]->lookupProperty[eColumn] = lookupProperty[eColumn];
       tableData = el->FindNextElement("tableData");
     }
 
@@ -310,14 +331,14 @@ FGTable::FGTable(FGPropertyManager* propMan, Element* el,
   if (dimension > 2) {
     for (b=2; b<=nTables; ++b) {
       if (Data[b][1] <= Data[b-1][1]) {
-        stringstream errormsg;
-        errormsg << fgred << highint << endl
-             << "  FGTable: breakpoint lookup is not monotonically increasing" << endl
-             << "  in breakpoint " << b;
-        if (nameel != 0) errormsg << " of table in " << nameel->GetAttributeValue("name");
-        errormsg << ":" << reset << endl
-                 << "  " << Data[b][1] << "<=" << Data[b-1][1] << endl;
-        throw(errormsg.str());
+        std::cerr << el->ReadFrom()
+                  << fgred << highint 
+                  << "  FGTable: breakpoint lookup is not monotonically increasing" << endl
+                  << "  in breakpoint " << b;
+        if (nameel != 0) std::cerr << " of table in " << nameel->GetAttributeValue("name");
+        std::cerr << ":" << reset << endl
+                  << "  " << Data[b][1] << "<=" << Data[b-1][1] << endl;
+        throw BaseException("Breakpoint lookup is not monotonically increasing");
       }
     }
   }
@@ -326,14 +347,14 @@ FGTable::FGTable(FGPropertyManager* propMan, Element* el,
   if (dimension > 1) {
     for (c=2; c<=nCols; ++c) {
       if (Data[0][c] <= Data[0][c-1]) {
-        stringstream errormsg;
-        errormsg << fgred << highint << endl
-             << "  FGTable: column lookup is not monotonically increasing" << endl
-             << "  in column " << c;
-        if (nameel != 0) errormsg << " of table in " << nameel->GetAttributeValue("name");
-        errormsg << ":" << reset << endl
-                 << "  " << Data[0][c] << "<=" << Data[0][c-1] << endl;
-        throw(errormsg.str());
+        std::cerr << el->ReadFrom()
+                  << fgred << highint 
+                  << "  FGTable: column lookup is not monotonically increasing" << endl
+                  << "  in column " << c;
+        if (nameel != 0) std::cerr << " of table in " << nameel->GetAttributeValue("name");
+        std::cerr << ":" << reset << endl
+                  << "  " << Data[0][c] << "<=" << Data[0][c-1] << endl;
+        throw BaseException("FGTable: column lookup is not monotonically increasing");
       }
     }
   }
@@ -342,19 +363,19 @@ FGTable::FGTable(FGPropertyManager* propMan, Element* el,
   if (dimension < 3) { // in 3D tables, check only rows of subtables
     for (r=2; r<=nRows; ++r) {
       if (Data[r][0]<=Data[r-1][0]) {
-        stringstream errormsg;
-        errormsg << fgred << highint << endl
-             << "  FGTable: row lookup is not monotonically increasing" << endl
-             << "  in row " << r;
-        if (nameel != 0) errormsg << " of table in " << nameel->GetAttributeValue("name");
-        errormsg << ":" << reset << endl
-                 << "  " << Data[r][0] << "<=" << Data[r-1][0] << endl;
-        throw(errormsg.str());
+        std::cerr << el->ReadFrom()
+                  << fgred << highint 
+                  << "  FGTable: row lookup is not monotonically increasing" << endl
+                  << "  in row " << r;
+        if (nameel != 0) std::cerr << " of table in " << nameel->GetAttributeValue("name");
+        std::cerr << ":" << reset << endl
+                  << "  " << Data[r][0] << "<=" << Data[r-1][0] << endl;
+        throw BaseException("FGTable: row lookup is not monotonically increasing");
       }
     }
   }
 
-  bind(el);
+  bind(el, Prefix);
 
   if (debug_lvl & 1) Print();
 }
@@ -377,6 +398,15 @@ double** FGTable::Allocate(void)
 
 FGTable::~FGTable()
 {
+  // Untie the bound property so that it makes no further reference to this
+  // instance of FGTable after the destruction is completed.
+  if (!Name.empty() && !internal) {
+    string tmp = PropertyManager->mkPropertyName(Name, false);
+    FGPropertyNode* node = PropertyManager->GetNode(tmp);
+    if (node && node->isTied())
+      PropertyManager->Untie(node);
+  }
+
   if (nTables > 0) {
     for (unsigned int i=0; i<nTables; i++) delete Tables[i];
     Tables.clear();
@@ -389,20 +419,6 @@ FGTable::~FGTable()
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-unsigned int FGTable::FindNumColumns(const string& test_line)
-{
-  // determine number of data columns in table (first column is row lookup - don't count)
-  size_t position=0;
-  unsigned int nCols=0;
-  while ((position = test_line.find_first_not_of(" \t", position)) != string::npos) {
-    nCols++;
-    position = test_line.find_first_of(" \t", position);
-  }
-  return nCols;
-}
-
-//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-
 double FGTable::GetValue(void) const
 {
   double temp = 0;
@@ -410,13 +426,19 @@ double FGTable::GetValue(void) const
 
   switch (Type) {
   case tt1D:
+    assert(lookupProperty[eRow]);
     temp = lookupProperty[eRow]->getDoubleValue();
     temp2 = GetValue(temp);
     return temp2;
   case tt2D:
+    assert(lookupProperty[eRow]);
+    assert(lookupProperty[eColumn]);
     return GetValue(lookupProperty[eRow]->getDoubleValue(),
                     lookupProperty[eColumn]->getDoubleValue());
   case tt3D:
+    assert(lookupProperty[eRow]);
+    assert(lookupProperty[eColumn]);
+    assert(lookupProperty[eTable]);
     return GetValue(lookupProperty[eRow]->getDoubleValue(),
                     lookupProperty[eColumn]->getDoubleValue(),
                     lookupProperty[eTable]->getDoubleValue());
@@ -446,7 +468,7 @@ double FGTable::GetValue(double key) const
   }
 
   // the key is somewhere in the middle, search for the right breakpoint
-  // The search is particularly efficient if 
+  // The search is particularly efficient if
   // the correct breakpoint has not changed since last frame or
   // has only changed very little
 
@@ -522,7 +544,7 @@ double FGTable::GetValue(double rowKey, double colKey, double tableKey) const
   }
 
   // the key is somewhere in the middle, search for the right breakpoint
-  // The search is particularly efficient if 
+  // The search is particularly efficient if
   // the correct breakpoint has not changed since last frame or
   // has only changed very little
 
@@ -639,28 +661,26 @@ void FGTable::Print(void)
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-void FGTable::bind(Element* el)
+void FGTable::bind(Element* el, const string& Prefix)
 {
   typedef double (FGTable::*PMF)(void) const;
+
   if ( !Name.empty() && !internal) {
-    string tmp;
-    if (Prefix.empty())
-      tmp  = PropertyManager->mkPropertyName(Name, false); // Allow upper
-    else {
+    if (!Prefix.empty()) {
       if (is_number(Prefix)) {
         if (Name.find("#") != string::npos) { // if "#" is found
-          Name = replace(Name,"#",Prefix);
-          tmp  = PropertyManager->mkPropertyName(Name, false); // Allow upper
+          Name = replace(Name, "#", Prefix);
         } else {
           cerr << el->ReadFrom()
-               << "Malformed table name with number: " << Prefix
-               << " and property name: " << Name
-               << " but no \"#\" sign for substitution." << endl;
+                << "Malformed table name with number: " << Prefix
+                << " and property name: " << Name
+                << " but no \"#\" sign for substitution." << endl;
         }
       } else {
-        tmp  = PropertyManager->mkPropertyName(Prefix + "/" + Name, false);
+        Name = Prefix + "/" + Name;
       }
     }
+    string tmp = PropertyManager->mkPropertyName(Name, false);
 
     if (PropertyManager->HasNode(tmp)) {
       FGPropertyNode* _property = PropertyManager->GetNode(tmp);
@@ -670,7 +690,8 @@ void FGTable::bind(Element* el)
         throw("Failed to bind the property to an existing already tied node.");
       }
     }
-    PropertyManager->Tie( tmp, this, (PMF)&FGTable::GetValue);
+
+    PropertyManager->Tie(tmp, this, (PMF)&FGTable::GetValue);
   }
 }
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -713,8 +734,6 @@ void FGTable::Debug(int from)
   }
   if (debug_lvl & 64) {
     if (from == 0) { // Constructor
-      cout << IdSrc << endl;
-      cout << IdHdr << endl;
     }
   }
 }
