@@ -9,21 +9,21 @@
  ------------- Copyright (C) 1999  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
 --------------------------------------------------------------------------------
@@ -46,14 +46,12 @@ INCLUDES
 #include "initialization/FGInitialCondition.h"
 #include "FGFDMExec.h"
 #include "input_output/FGPropertyManager.h"
+#include "FGInertial.h"
 
 using namespace std;
 
 namespace JSBSim {
 
-IDENT(IdSrc,"$Id: FGAuxiliary.cpp,v 1.72 2016/05/21 11:45:22 bcoconni Exp $");
-IDENT(IdHdr,ID_AUXILIARY);
-
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
@@ -62,31 +60,28 @@ CLASS IMPLEMENTATION
 FGAuxiliary::FGAuxiliary(FGFDMExec* fdmex) : FGModel(fdmex)
 {
   Name = "FGAuxiliary";
-  pt = 1.0;
-  tat = 1.0;
-  tatc = RankineToCelsius(tat);
+  pt = 2116.23; // ISA SL pressure
+  tatc = 15.0; // ISA SL temperature
+  tat = 518.67;
 
   vcas = veas = 0.0;
   qbar = qbarUW = qbarUV = 0.0;
-  Mach = MachU = MachPitot = 0.0;
+  Mach = MachU = 0.0;
   alpha = beta = 0.0;
   adot = bdot = 0.0;
-  gamma = Vt = Vground = Vpitot = 0.0;
+  gamma = Vt = Vground = 0.0;
   psigt = 0.0;
   day_of_year = 1;
   seconds_in_day = 0.0;
   hoverbmac = hoverbcg = 0.0;
   Re = 0.0;
-  Nz = Ny = 0.0;
+  Nx = Ny = Nz = 0.0;
 
   vPilotAccel.InitMatrix();
   vPilotAccelN.InitMatrix();
   vAeroUVW.InitMatrix();
   vAeroPQR.InitMatrix();
   vMachUVW.InitMatrix();
-  vWindUVW.InitMatrix();
-  vPitotUVW.InitMatrix();
-  vEuler.InitMatrix();
   vEulerRates.InitMatrix();
 
   bind();
@@ -106,10 +101,10 @@ bool FGAuxiliary::InitModel(void)
 
   vcas = veas = 0.0;
   qbar = qbarUW = qbarUV = 0.0;
-  Mach = MachU = MachPitot = 0.0;
+  Mach = MachU = 0.0;
   alpha = beta = 0.0;
   adot = bdot = 0.0;
-  gamma = Vt = Vground = Vpitot = 0.0;
+  gamma = Vt = Vground = 0.0;
   psigt = 0.0;
   day_of_year = 1;
   seconds_in_day = 0.0;
@@ -122,7 +117,6 @@ bool FGAuxiliary::InitModel(void)
   vAeroUVW.InitMatrix();
   vAeroPQR.InitMatrix();
   vMachUVW.InitMatrix();
-  vEuler.InitMatrix();
   vEulerRates.InitMatrix();
 
   return true;
@@ -154,29 +148,22 @@ bool FGAuxiliary::Run(bool Holding)
   vAeroPQR = in.vPQR - in.TurbPQR;
   vAeroUVW = in.vUVW - in.Tl2b * in.TotalWindNED;
 
-  Vt = vAeroUVW.Magnitude();
   alpha = beta = adot = bdot = 0;
   double AeroU2 = vAeroUVW(eU)*vAeroUVW(eU);
   double AeroV2 = vAeroUVW(eV)*vAeroUVW(eV);
   double AeroW2 = vAeroUVW(eW)*vAeroUVW(eW);
   double mUW = AeroU2 + AeroW2;
 
-  double Vt2 = Vt*Vt;
+  double Vt2 = mUW + AeroV2;
+  Vt = sqrt(Vt2);
 
   if ( Vt > 0.001 ) {
-    if (vAeroUVW(eW) != 0.0)
-      alpha = AeroU2 > 0.0 ? atan2(vAeroUVW(eW), vAeroUVW(eU)) : 0.0;
-    if (vAeroUVW(eV) != 0.0)
-      beta  =    mUW > 0.0 ? atan2(vAeroUVW(eV), sqrt(mUW)) : 0.0;
+    beta = atan2(vAeroUVW(eV), sqrt(mUW));
 
-    //double signU=1;
-    //if (vAeroUVW(eU) < 0.0) signU=-1;
-
-    if ( mUW >= 0.001 ) {
+    if ( mUW >= 1E-6 ) {
+      alpha = atan2(vAeroUVW(eW), vAeroUVW(eU));
       double Vtdot = (vAeroUVW(eU)*in.vUVWdot(eU) + vAeroUVW(eV)*in.vUVWdot(eV) + vAeroUVW(eW)*in.vUVWdot(eW))/Vt;
       adot = (vAeroUVW(eU)*in.vUVWdot(eW) - vAeroUVW(eW)*in.vUVWdot(eU))/mUW;
-      // bdot = (signU*mUW*in.vUVWdot(eV)
-      //        - vAeroUVW(eV)*(vAeroUVW(eU)*in.vUVWdot(eU) + vAeroUVW(eW)*in.vUVWdot(eW)))/(Vt2*sqrt(mUW));
       bdot = (in.vUVWdot(eV)*Vt - vAeroUVW(eV)*Vtdot)/(Vt*sqrt(mUW));
     }
   }
@@ -206,35 +193,28 @@ bool FGAuxiliary::Run(bool Holding)
   tat = in.Temperature*(1 + 0.2*Mach*Mach); // Total Temperature, isentropic flow
   tatc = RankineToCelsius(tat);
 
-  // Pitot
+  pt = PitotTotalPressure(Mach, in.Pressure);
 
-  vWindUVW(eU) = Vt;
-  vPitotUVW = mTw2p * vWindUVW;
-  Vpitot = vPitotUVW(eU);
-  if (Vpitot < 0.0) Vpitot = 0.0;
-  MachPitot = Vpitot / in.SoundSpeed;
-  pt = PitotTotalPressure(MachPitot, in.Pressure);
-
-  if (abs(MachPitot) > 0.0) {
-    vcas = VcalibratedFromMach(MachPitot, in.Pressure, in.PressureSL, in.DensitySL);
+  if (abs(Mach) > 0.0) {
+    vcas = VcalibratedFromMach(Mach, in.Pressure);
     veas = sqrt(2 * qbar / in.DensitySL);
-    vtrue = 1116.43559 * Mach * sqrt(in.Temperature / 518.67);
-  } else {
-    vcas = veas = vtrue = 0.0;
   }
+  else
+    vcas = veas = 0.0;
 
   vPilotAccel.InitMatrix();
-  vNcg = in.vBodyAccel/in.SLGravity;
+  vNcg = in.vBodyAccel/in.StandardGravity;
   // Nz is Acceleration in "g's", along normal axis (-Z body axis)
   Nz = -vNcg(eZ);
   Ny =  vNcg(eY);
+  Nx =  vNcg(eX);
   vPilotAccel = in.vBodyAccel + in.vPQRidot * in.ToEyePt;
   vPilotAccel += in.vPQRi * (in.vPQRi * in.ToEyePt);
 
   vNwcg = mTb2w * vNcg;
   vNwcg(eZ) = 1.0 - vNwcg(eZ);
 
-  vPilotAccelN = vPilotAccel / in.SLGravity;
+  vPilotAccelN = vPilotAccel / in.StandardGravity;
 
   // VRP computation
   vLocationVRP = in.vLocation.LocalToLocation( in.Tb2l * in.VRPBody );
@@ -243,7 +223,7 @@ bool FGAuxiliary::Run(bool Holding)
   hoverbcg = in.DistanceAGL / in.Wingspan;
 
   FGColumnVector3 vMac = in.Tb2l * in.RPBody;
-  hoverbmac = (in.DistanceAGL + vMac(3)) / in.Wingspan;
+  hoverbmac = (in.DistanceAGL - vMac(3)) / in.Wingspan;
 
   return false;
 }
@@ -284,45 +264,6 @@ void FGAuxiliary::UpdateWindMatrices(void)
   mTw2b(3,3) =  ca;
 
   mTb2w = mTw2b.Transposed();
-
-  // The pitot frame is the same as the body frame except rotated about the
-  // Y axis by the pitot attachment angle.
-
-  ca = cos(alpha + in.PitotAngle);
-  sa = sin(alpha + in.PitotAngle);
-
-  mTw2p(1,1) =  ca*cb;
-  mTw2p(1,2) = -ca*sb;
-  mTw2p(1,3) = -sa;
-  mTw2p(2,1) =  sb;
-  mTw2p(2,2) =  cb;
-  mTw2p(2,3) =  0.0;
-  mTw2p(3,1) =  sa*cb;
-  mTw2p(3,2) = -sa*sb;
-  mTw2p(3,3) =  ca;
-
-}
-
-//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-//
-// A positive headwind is blowing with you, a negative headwind is blowing against you.
-// psi is the direction the wind is blowing *towards*.
-// ToDo: should this simply be in the atmosphere class? Same with Get Crosswind.
-
-double FGAuxiliary::GetHeadWind(void) const
-{
-  return in.Vwind * cos(in.WindPsi - in.Psi);
-}
-
-//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-//
-// A positive crosswind is blowing towards the right (from teh perspective of the
-// pilot). A negative crosswind is blowing towards the -Y direction (left).
-// psi is the direction the wind is blowing *towards*.
-
-double FGAuxiliary::GetCrossWind(void) const
-{
-  return in.Vwind * sin(in.WindPsi - in.Psi);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -330,7 +271,7 @@ double FGAuxiliary::GetCrossWind(void) const
 double FGAuxiliary::GetNlf(void) const
 {
   if (in.Mass != 0)
-    return (-in.vFw(3))/(in.Mass*slugtolb);
+    return (in.vFw(3))/(in.Mass*slugtolb);
   else
     return 0.;
 }
@@ -339,33 +280,25 @@ double FGAuxiliary::GetNlf(void) const
 
 double FGAuxiliary::GetLongitudeRelativePosition(void) const
 {
-  FGLocation source(FDMExec->GetIC()->GetLongitudeRadIC(),
-                    FDMExec->GetIC()->GetLatitudeRadIC(),
-                    in.vLocation.GetSeaLevelRadius());
-  return source.GetDistanceTo(in.vLocation.GetLongitude(),
-                              FDMExec->GetIC()->GetLatitudeRadIC()) * fttom;
+  return in.vLocation.GetDistanceTo(FDMExec->GetIC()->GetLongitudeRadIC(),
+                                    in.vLocation.GetLatitude())* fttom;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 double FGAuxiliary::GetLatitudeRelativePosition(void) const
 {
-  FGLocation source(FDMExec->GetIC()->GetLongitudeRadIC(),
-                    FDMExec->GetIC()->GetLatitudeRadIC(),
-                    in.vLocation.GetSeaLevelRadius());
-  return source.GetDistanceTo(FDMExec->GetIC()->GetLongitudeRadIC(),
-                              in.vLocation.GetLatitude()) * fttom;
+  return in.vLocation.GetDistanceTo(in.vLocation.GetLongitude(),
+                                    FDMExec->GetIC()->GetLatitudeRadIC())* fttom;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 double FGAuxiliary::GetDistanceRelativePosition(void) const
 {
-  FGLocation source(FDMExec->GetIC()->GetLongitudeRadIC(),
-                    FDMExec->GetIC()->GetLatitudeRadIC(),
-                    in.vLocation.GetSeaLevelRadius());
-  return source.GetDistanceTo(in.vLocation.GetLongitude(),
-                              in.vLocation.GetLatitude()) * fttom;
+  FGInitialCondition *ic = FDMExec->GetIC();
+  return in.vLocation.GetDistanceTo(ic->GetLongitudeRadIC(),
+                                    ic->GetLatitudeRadIC())* fttom;
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -402,11 +335,10 @@ void FGAuxiliary::bind(void)
   PropertyManager->Tie("accelerations/n-pilot-x-norm", this, eX, (PMF)&FGAuxiliary::GetNpilot);
   PropertyManager->Tie("accelerations/n-pilot-y-norm", this, eY, (PMF)&FGAuxiliary::GetNpilot);
   PropertyManager->Tie("accelerations/n-pilot-z-norm", this, eZ, (PMF)&FGAuxiliary::GetNpilot);
-  PropertyManager->Tie("accelerations/Nz", this, &FGAuxiliary::GetNz);
+  PropertyManager->Tie("accelerations/Nx", this, &FGAuxiliary::GetNx);
   PropertyManager->Tie("accelerations/Ny", this, &FGAuxiliary::GetNy);
+  PropertyManager->Tie("accelerations/Nz", this, &FGAuxiliary::GetNz);
   PropertyManager->Tie("forces/load-factor", this, &FGAuxiliary::GetNlf);
-  /* PropertyManager->Tie("atmosphere/headwind-fps", this, &FGAuxiliary::GetHeadWind, true);
-  PropertyManager->Tie("atmosphere/crosswind-fps", this, &FGAuxiliary::GetCrossWind, true); */
   PropertyManager->Tie("aero/alpha-rad", this, (PF)&FGAuxiliary::Getalpha);
   PropertyManager->Tie("aero/beta-rad", this, (PF)&FGAuxiliary::Getbeta);
   PropertyManager->Tie("aero/mag-beta-rad", this, (PF)&FGAuxiliary::GetMagBeta);
@@ -424,6 +356,7 @@ void FGAuxiliary::bind(void)
   PropertyManager->Tie("aero/h_b-cg-ft", this, &FGAuxiliary::GetHOverBCG);
   PropertyManager->Tie("aero/h_b-mac-ft", this, &FGAuxiliary::GetHOverBMAC);
   PropertyManager->Tie("flight-path/gamma-rad", this, &FGAuxiliary::GetGamma);
+  PropertyManager->Tie("flight-path/gamma-deg", this, inDegrees, (PMF)&FGAuxiliary::GetGamma);
   PropertyManager->Tie("flight-path/psi-gt-rad", this, &FGAuxiliary::GetGroundTrack);
 
   PropertyManager->Tie("position/distance-from-start-lon-mt", this, &FGAuxiliary::GetLongitudeRelativePosition);
@@ -485,8 +418,6 @@ void FGAuxiliary::Debug(int from)
   }
   if (debug_lvl & 64) {
     if (from == 0) { // Constructor
-      cout << IdSrc << endl;
-      cout << IdHdr << endl;
     }
   }
 }
