@@ -7,21 +7,21 @@
  ------------- Copyright (C) 1999  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 HISTORY
 --------------------------------------------------------------------------------
@@ -40,16 +40,8 @@ INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 #include "FGModel.h"
-#include "math/FGColumnVector3.h"
-#include "math/FGMatrix33.h"
 #include "math/FGLocation.h"
 
-/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-DEFINITIONS
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
-
-#define ID_AUXILIARY "$Id: FGAuxiliary.h,v 1.28 2013/06/10 01:56:27 jberndt Exp $"
-
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
@@ -99,7 +91,6 @@ CLASS DOCUMENTATION
     to the JSBSim vPQRdot vector, and the w parameter is equivalent to vPQR.
 
     @author Tony Peden, Jon Berndt
-    @version $Id: FGAuxiliary.h,v 1.28 2013/06/10 01:56:27 jberndt Exp $
 */
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -110,21 +101,22 @@ class FGAuxiliary : public FGModel {
 public:
   /** Constructor
       @param Executive a pointer to the parent executive object */
-  FGAuxiliary(FGFDMExec* Executive);
+  explicit FGAuxiliary(FGFDMExec* Executive);
 
   /// Destructor
   ~FGAuxiliary();
 
-  bool InitModel(void);
+  bool InitModel(void) override;
 
   /** Runs the Auxiliary routines; called by the Executive
-      Can pass in a value indicating if the executive is directing the simulation to Hold.
-      @param Holding if true, the executive has been directed to hold the sim from 
-                     advancing time. Some models may ignore this flag, such as the Input
-                     model, which may need to be active to listen on a socket for the
-                     "Resume" command to be given.
-      @return false if no error */
-  bool Run(bool Holding);
+      Can pass in a value indicating if the executive is directing the
+      simulation to Hold.
+      @param Holding if true, the executive has been directed to hold the sim
+                     from advancing time. Some models may ignore this flag, such
+                     as the Input model, which may need to be active to listen
+                     on a socket for the "Resume" command to be given.  @return
+                     false if no error */
+  bool Run(bool Holding) override;
 
 // GET functions
 
@@ -138,9 +130,9 @@ public:
   /** Returns equivalent airspeed in knots. */
   double GetVequivalentKTS(void) const { return veas*fpstokts; }
   /** Returns the true airspeed in feet per second. */
-  double GetVtrueFPS() const { return vtrue; }
+  double GetVtrueFPS() const { return Vt; }
   /** Returns the true airspeed in knots. */
-  double GetVtrueKTS() const { return vtrue * fpstokts; }
+  double GetVtrueKTS() const { return Vt * fpstokts; }
 
   /** Returns the total pressure.
       Total pressure is freestream total pressure for
@@ -153,7 +145,8 @@ public:
     @code
     tat = in.Temperature*(1 + 0.2*Mach*Mach)
     @endcode
-    (where "in.Temperature" is standard temperature calculated by the atmosphere model) */
+    (where "in.Temperature" is standard temperature calculated by the atmosphere
+    model) */
 
   double GetTotalTemperature(void) const { return tat; }
   double GetTAT_C(void) const { return tatc; }
@@ -173,7 +166,6 @@ public:
   const FGColumnVector3& GetAeroUVW    (void) const { return vAeroUVW;     }
   const FGLocation&      GetLocationVRP(void) const { return vLocationVRP; }
 
-  double GethVRP(void) const;
   double GetAeroUVW (int idx) const { return vAeroUVW(idx); }
   double Getalpha   (void) const { return alpha;      }
   double Getbeta    (void) const { return beta;       }
@@ -195,19 +187,20 @@ public:
   /** Calculates and returns the wind-to-body axis transformation matrix.
       @return a reference to the wind-to-body transformation matrix.
       */
-  const FGMatrix33& GetTw2b(void) { return mTw2b; }
+  const FGMatrix33& GetTw2b(void) const { return mTw2b; }
 
   /** Calculates and returns the body-to-wind axis transformation matrix.
       @return a reference to the wind-to-body transformation matrix.
       */
-  const FGMatrix33& GetTb2w(void) { return mTb2w; }
+  const FGMatrix33& GetTb2w(void) const { return mTb2w; }
 
   double Getqbar          (void) const { return qbar;       }
   double GetqbarUW        (void) const { return qbarUW;     }
   double GetqbarUV        (void) const { return qbarUV;     }
   double GetReynoldsNumber(void) const { return Re;         }
 
-  /** Gets the magnitude of total vehicle velocity including wind effects in feet per second. */
+  /** Gets the magnitude of total vehicle velocity including wind effects in
+      feet per second. */
   double GetVt            (void) const { return Vt;         }
 
   /** Gets the ground speed in feet per second.
@@ -222,12 +215,15 @@ public:
   /** The mach number calculated using the vehicle X axis velocity. */
   double GetMachU         (void) const { return MachU;      }
 
-  /** The vertical acceleration in g's of the aircraft center of gravity. */
-  double GetNz            (void) const { return Nz;         }
+  /** The longitudinal acceleration in g's of the aircraft center of gravity. */
+  double GetNx            (void) const { return Nx;         }
 
   /** The lateral acceleration in g's of the aircraft center of gravity. */
   double GetNy            (void) const { return Ny;         }
 
+  /** The vertical acceleration in g's of the aircraft center of gravity. */
+  double GetNz            (void) const { return Nz;         }
+
   const FGColumnVector3& GetNwcg(void) const { return vNwcg; }
 
   double GetHOverBCG(void) const { return hoverbcg; }
@@ -236,8 +232,10 @@ public:
   double GetGamma(void)              const { return gamma;         }
   double GetGroundTrack(void)        const { return psigt;         }
 
-  double GetHeadWind(void) const;
-  double GetCrossWind(void) const;
+  double GetGamma(int unit) const {
+    if (unit == inDegrees) return gamma*radtodeg;
+    else return BadUnits();
+  }
 
 // Time routines, SET and GET functions, used by FGMSIS atmosphere
 
@@ -247,9 +245,9 @@ public:
   int    GetDayOfYear    (void) const { return day_of_year;    }
   double GetSecondsInDay (void) const { return seconds_in_day; }
 
-  double GetLongitudeRelativePosition (void) const { return lon_relative_position; }
-  double GetLatitudeRelativePosition  (void) const { return lat_relative_position; }
-  double GetDistanceRelativePosition  (void) const { return relative_position; }
+  double GetLongitudeRelativePosition (void) const;
+  double GetLatitudeRelativePosition  (void) const;
+  double GetDistanceRelativePosition  (void) const;
 
   void SetAeroPQR(const FGColumnVector3& tt) { vAeroPQR = tt; }
 
@@ -264,12 +262,13 @@ public:
     double DistanceAGL;
     double Wingspan;
     double Wingchord;
-    double SLGravity;
+    double StandardGravity;
     double Mass;
     FGMatrix33 Tl2b;
     FGMatrix33 Tb2l;
     FGColumnVector3 vPQR;
-    FGColumnVector3 vPQRdot;
+    FGColumnVector3 vPQRi;
+    FGColumnVector3 vPQRidot;
     FGColumnVector3 vUVW;
     FGColumnVector3 vUVWdot;
     FGColumnVector3 vVel;
@@ -279,20 +278,16 @@ public:
     FGColumnVector3 VRPBody;
     FGColumnVector3 vFw;
     FGLocation vLocation;
-    double ReferenceRadius;
     double CosTht;
     double SinTht;
     double CosPhi;
     double SinPhi;
-    double Psi;
     FGColumnVector3 TotalWindNED;
     FGColumnVector3 TurbPQR;
-    double WindPsi;
-    double Vwind;
   } in;
 
 private:
-  double vcas, veas, vtrue;
+  double vcas, veas;
   double pt, tat, tatc; // Don't add a getter for pt!
 
   FGMatrix33 mTw2b;
@@ -304,36 +299,30 @@ private:
   FGColumnVector3 vNwcg;
   FGColumnVector3 vAeroPQR;
   FGColumnVector3 vAeroUVW;
-  FGColumnVector3 vEuler;
   FGColumnVector3 vEulerRates;
   FGColumnVector3 vMachUVW;
   FGLocation vLocationVRP;
 
-  double Vt, Vground, Mach, MachU;
+  double Vt, Vground;
+  double Mach, MachU;
   double qbar, qbarUW, qbarUV;
   double Re; // Reynolds Number = V*c/mu
   double alpha, beta;
   double adot,bdot;
   double psigt, gamma;
-  double Nz, Ny;
+  double Nx, Ny, Nz;
   double seconds_in_day;  // seconds since current GMT day began
   int    day_of_year;     // GMT day, 1 .. 366
 
   double hoverbcg, hoverbmac;
 
-  // helper data, calculation of distance from initial position
-
-  double lon_relative_position;
-  double lat_relative_position;
-  double relative_position;
-
   void UpdateWindMatrices(void);
 
   void CalculateRelativePosition(void);
 
   void bind(void);
   double BadUnits(void) const;
-  void Debug(int from);
+  void Debug(int from) override;
 };
 
 } // namespace JSBSim
