@@ -7,21 +7,21 @@
  ------------- Copyright (C) 2007  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 
  HISTORY
@@ -39,19 +39,8 @@ SENTRY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include <string>
-#include "FGFDMExec.h"
-#include "FGJSBBase.h"
 #include "models/propulsion/FGForce.h"
-#include "input_output/FGPropertyManager.h"
-#include "math/FGColumnVector3.h"
-#include "math/FGFunction.h"
-
-/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-DEFINITIONS
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
-
-#define ID_EXTERNALFORCE "$Id: FGExternalForce.h,v 1.13 2014/11/25 01:44:17 dpculp Exp $"
+#include "simgear/props/propertyObject.hxx"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
@@ -59,14 +48,55 @@ FORWARD DECLARATIONS
 
 namespace JSBSim {
 
+class FGParameter;
+class Element;
+class FGPropertyManager;
+
+/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+CLASS DECLARATION
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
+
+class FGPropertyVector3
+{
+public:
+  FGPropertyVector3(void) {}
+  FGPropertyVector3(FGPropertyManager* pm, const std::string& baseName,
+                    const std::string& xcmp, const std::string& ycmp,
+                    const std::string& zcmp);
+
+  FGPropertyVector3& operator=(const FGColumnVector3& v) {
+    data[0] = v(1);
+    data[1] = v(2);
+    data[2] = v(3);
+
+    return *this;
+  }
+
+  operator FGColumnVector3() const {
+    return FGColumnVector3(data[0], data[1], data[2]);
+  }
+
+  FGColumnVector3 operator*(double a) const {
+    return FGColumnVector3(a * data[0], a * data[1], a * data[2]);
+  }
+
+private:
+  SGPropObjDouble data[3];
+};
+
+inline FGColumnVector3 operator*(double a, const FGPropertyVector3& v) {
+  return v*a;
+}
+
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS DOCUMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-/** Encapsulates code that models an individual arbitrary force.
-    This class encapsulates an individual force applied at the specified
+/** Encapsulates code that models an individual arbitrary force, moment or a
+    combination thereof.
+    This class encapsulates an individual reaction applied at the specified
     location on the vehicle, and oriented as specified in one of three frames:
-    
+
     - BODY frame is defined with the X axis positive forward, the Y axis
            positive out the right wing, and the Z axis completing the set
            positive downward out the belly of the aircraft.
@@ -80,14 +110,14 @@ CLASS DOCUMENTATION
            wind frame definition, which is rotated about the Y axis 180 degrees
            from this WIND frame.
 
-    Much of the substance of this class is located in the FGForce base class, from
-    which this class is derived.
-    
+    Much of the substance of this class is located in the FGForce base class,
+    from which this class is derived.
+
     Here is the XML definition of a force (optional items are in []):
-    
+
     @code
     <force name="name" frame="BODY | LOCAL | WIND">
-      
+
       [<function> ... </function>]
 
       <location unit="{IN | M}"> 
@@ -103,54 +133,88 @@ CLASS DOCUMENTATION
     </force>
     @endcode
 
+    The location of the force vector, in structural coordinates, can be set at
+    runtime through the following properties:
+
+    @code
+    external_reactions/{force name}/location-x-in
+    external_reactions/{force name}/location-y-in
+    external_reactions/{force name}/location-z-in
+    @endcode
+
+    The XML definition of a moment (optional items are in []) is a bit simpler
+    because you do not need to specify the location:
+
+    @code
+    <moment name="name" frame="BODY | LOCAL | WIND">
+
+      [<function> ... </function>]
+
+      [<direction> <!-- optional initial direction vector -->
+        <x> {number} </x>
+        <y> {number} </y>
+        <z> {number} </z>
+      </direction>]
+    </moment>
+    @endcode
+
     The initial direction can optionally be set by specifying a unit vector
-    in the chosen frame (body, local, or wind). The direction is specified
-    at runtime through setting any/all of the following properties:
-    
+    in the chosen frame (body, local, or wind).
+
+    As an example, a parachute can be defined oriented in the wind axis frame
+    so the drag always acts in the drag direction - opposite the positive X
+    axis. That does not include the effects of parachute oscillations, but
+    those could be handled in the calling application.
+
+    The force (or moment) direction is not actually required to be specified as
+    a unit vector, but prior to the force (or moment) vector being calculated,
+    the direction vector is normalized when initialized.
+
+    The force direction can be specified at runtime through setting any/all of
+    the following properties:
+
     @code
     external_reactions/{force name}/x
     external_reactions/{force name}/y
     external_reactions/{force name}/z
     @endcode
-    
-    As an example, a parachute can be defined oriented in the wind axis frame
-    so the drag always acts in the drag direction - opposite the positive X
-    axis. That does not include the effects of parachute oscillations, but
-    those could be handled in the calling application.
-     
-    The force direction is not actually required to be specified as a unit
-    vector, but prior to the force vector being calculated, the direction
-    vector is normalized.
 
-    The location of the force vector, in structural coordinates, can be set at
-    runtime through the following properties:
+    The moment direction can be specified at runtime through setting any/all of
+    the following properties:
 
     @code
-    external_reactions/{force name}/locx
-    external_reactions/{force name}/locy
-    external_reactions/{force name}/locz
+    external_reactions/{moment name}/l
+    external_reactions/{moment name}/m
+    external_reactions/{moment name}/n
     @endcode
-    
-*/
 
-/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-CLASS DECLARATION
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
+    However in that case, the direction is no longer normalized.
+
+    When no <function> has been provided in the force definition, its magnitude
+    can be specified through the following property:
+
+    @code
+    external_reactions/{force name}/magnitude
+    @endcode
+
+    When no <function> has been provided in the moment definition, its magnitude
+    can be specified through the following property:
+
+    @code
+    external_reactions/{moment name}/magnitude-lbsft
+    @endcode
+*/
 
 class FGExternalForce : public FGForce
 {
 public:
-  /** Constructor.
-      @param FDMExec pointer to the main executive class.
-  */
-  FGExternalForce(FGFDMExec *FDMExec);
-
   /** Constructor.
       @param FDMExec pointer to the main executive class.
       @param el pointer to the XML element defining an individual force.
-      @param index the position of this force object in the whole list.
   */
-  FGExternalForce(FGFDMExec *FDMExec, Element *el, int index);
+  explicit FGExternalForce(FGFDMExec *FDMExec)
+    : FGForce(FDMExec), forceMagnitude(NULL), momentMagnitude(NULL)
+  { Debug(0); }
 
   /** Copy Constructor
       @param extForce a reference to an existing FGExternalForce object
@@ -158,37 +222,20 @@ public:
   FGExternalForce(const FGExternalForce& extForce);
 
   /// Destructor
-  ~FGExternalForce();
+  ~FGExternalForce() override;
 
-  void SetMagnitude(double mag);
-  void SetAzimuth(double az) {azimuth = az;}
+  void setForce(Element* el);
+  void setMoment(Element* el);
+  const FGColumnVector3& GetBodyForces(void) override;
 
-  const FGColumnVector3& GetBodyForces(void);
-  double GetMagnitude(void) const {return magnitude;}
-  double GetAzimuth(void) const {return azimuth;}
-  double GetX(void) const {return vDirection(eX);}
-  double GetY(void) const {return vDirection(eY);}
-  double GetZ(void) const {return vDirection(eZ);}
-  void SetX(double x) {vDirection(eX) = x;}
-  void SetY(double y) {vDirection(eY) = y;}
-  void SetZ(double z) {vDirection(eZ) = z;}
-  double GetLocX(void) const {return vActingXYZn(eX);}
-  double GetLocY(void) const {return vActingXYZn(eY);}
-  double GetLocZ(void) const {return vActingXYZn(eZ);}
-  void SetLocX(double x) {vXYZn(eX) = x; vActingXYZn(eX) = x;}
-  void SetLocY(double y) {vXYZn(eY) = y; vActingXYZn(eY) = y;}
-  void SetLocZ(double z) {vXYZn(eZ) = z; vActingXYZn(eZ) = z;}  
-  
 private:
+  FGParameter* bind(Element* el, const std::string& baseName,
+                    FGPropertyVector3& v);
 
-  std::string Frame;
   std::string Name;
-  FGFunction* Magnitude_Function;
-  FGColumnVector3 vDirection;
-  double magnitude;
-  double azimuth;
+  FGParameter *forceMagnitude, *momentMagnitude;
+  FGPropertyVector3 forceDirection, momentDirection;
   void Debug(int from);
 };
 }
 #endif
-
