@@ -9,21 +9,21 @@
  ------------- Copyright (C) 1999  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
 --------------------------------------------------------------------------------
@@ -37,14 +37,9 @@ HISTORY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include <fstream>
-#include <sstream>
 #include <iomanip>
 
 #include "FGFCS.h"
-#include "FGFDMExec.h"
-#include "FGGroundReactions.h"
-#include "input_output/FGXMLElement.h"
 #include "input_output/FGModelLoader.h"
 
 #include "models/flight_control/FGFilter.h"
@@ -55,7 +50,6 @@ INCLUDES
 #include "models/flight_control/FGSummer.h"
 #include "models/flight_control/FGKinemat.h"
 #include "models/flight_control/FGFCSFunction.h"
-#include "models/flight_control/FGSensor.h"
 #include "models/flight_control/FGActuator.h"
 #include "models/flight_control/FGAccelerometer.h"
 #include "models/flight_control/FGMagnetometer.h"
@@ -63,6 +57,7 @@ INCLUDES
 #include "models/flight_control/FGWaypoint.h"
 #include "models/flight_control/FGAngles.h"
 #include "models/flight_control/FGDistributor.h"
+#include "models/flight_control/FGLinearActuator.h"
 
 #include "FGFCSChannel.h"
 
@@ -70,9 +65,6 @@ using namespace std;
 
 namespace JSBSim {
 
-IDENT(IdSrc,"$Id: FGFCS.cpp,v 1.98 2017/02/25 14:23:18 bcoconni Exp $");
-IDENT(IdHdr,ID_FCS);
-
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
@@ -490,7 +482,7 @@ bool FGFCS::Load(Element* document)
   }
 
   // Load interface properties from document
-  if (!FGModel::Load(document))
+  if (!FGModel::Upload(document, true))
     return false;
 
   Name += document->GetAttributeValue("name");
@@ -506,9 +498,10 @@ bool FGFCS::Load(Element* document)
     string sOnOffProperty = channel_element->GetAttributeValue("execute");
     string sChannelName = channel_element->GetAttributeValue("name");
     
-    int Rate = 0;
     if (!channel_element->GetAttributeValue("execrate").empty())
-      Rate = channel_element->GetAttributeValueAsNumber("execrate");
+      ChannelRate = channel_element->GetAttributeValueAsNumber("execrate");
+    else
+      ChannelRate = 1;
 
     if (sOnOffProperty.length() > 0) {
       FGPropertyNode* OnOffPropertyNode = PropertyManager->GetNode(sOnOffProperty);
@@ -519,10 +512,10 @@ bool FGFCS::Load(Element* document)
              << "understood. The simulation will abort" << reset << endl;
         throw("Bad system definition");
       } else
-        newChannel = new FGFCSChannel(this, sChannelName, Rate,
+        newChannel = new FGFCSChannel(this, sChannelName, ChannelRate,
                                       OnOffPropertyNode);
     } else
-      newChannel = new FGFCSChannel(this, sChannelName, Rate);
+      newChannel = new FGFCSChannel(this, sChannelName, ChannelRate);
 
     SystemChannels.push_back(newChannel);
 
@@ -536,8 +529,7 @@ bool FGFCS::Load(Element* document)
         if ((component_element->GetName() == string("lag_filter")) ||
             (component_element->GetName() == string("lead_lag_filter")) ||
             (component_element->GetName() == string("washout_filter")) ||
-            (component_element->GetName() == string("second_order_filter")) ||
-            (component_element->GetName() == string("integrator")) )
+            (component_element->GetName() == string("second_order_filter")) )
         {
           newChannel->Add(new FGFilter(this, component_element));
         } else if ((component_element->GetName() == string("pure_gain")) ||
@@ -557,6 +549,18 @@ bool FGFCS::Load(Element* document)
           newChannel->Add(new FGFCSFunction(this, component_element));
         } else if (component_element->GetName() == string("pid")) {
           newChannel->Add(new FGPID(this, component_element));
+        } else if (component_element->GetName() == string("integrator")) {
+          // <integrator> is equivalent to <pid type="trap">
+          Element* c1_el = component_element->FindElement("c1");
+          if (!c1_el) {
+            cerr << component_element->ReadFrom();
+            throw("INTEGRATOR component " + component_element->GetAttributeValue("name")
+                  + " does not provide the parameter <c1>");
+          }
+          c1_el->ChangeName("ki");
+          if (!c1_el->HasAttribute("type"))
+            c1_el->AddAttribute("type", "trap");
+          newChannel->Add(new FGPID(this, component_element));
         } else if (component_element->GetName() == string("actuator")) {
           newChannel->Add(new FGActuator(this, component_element));
         } else if (component_element->GetName() == string("sensor")) {
@@ -575,6 +579,8 @@ bool FGFCS::Load(Element* document)
           newChannel->Add(new FGAngles(this, component_element));
         } else if (component_element->GetName() == string("distributor")) {
           newChannel->Add(new FGDistributor(this, component_element));
+        } else if (component_element->GetName() == string("linear_actuator")) {
+          newChannel->Add(new FGLinearActuator(this, component_element));
         } else {
           cerr << "Unknown FCS component: " << component_element->GetName() << endl;
         }
@@ -588,7 +594,7 @@ bool FGFCS::Load(Element* document)
     channel_element = document->FindNextElement("channel");
   }
 
-  PostLoad(document, PropertyManager);
+  PostLoad(document, FDMExec);
 
   return true;
 }
@@ -817,8 +823,6 @@ void FGFCS::Debug(int from)
   }
   if (debug_lvl & 64) {
     if (from == 0) { // Constructor
-      cout << IdSrc << endl;
-      cout << IdHdr << endl;
     }
   }
 }
