@@ -11,21 +11,21 @@
  ------------- Copyright (C) 1999  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
 --------------------------------------------------------------------------------
@@ -34,20 +34,18 @@ HISTORY
 --------------------------------------------------------------------------------
 11/18/99   JSB   Created
 01/30/01   NHP   Extended gear model to properly simulate steering and braking
-07/08/09   BC    Modified gear model to support large angles between aircraft and ground
+07/08/09   BC    Modified gear model to support large angles between aircraft
+                 and ground
 
-/%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+/%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include <cstdlib>
-#include <cstring>
-
-#include "math/FGFunction.h"
 #include "FGLGear.h"
 #include "models/FGGroundReactions.h"
 #include "math/FGTable.h"
 #include "input_output/FGXMLElement.h"
+#include "models/FGInertial.h"
 
 using namespace std;
 
@@ -61,11 +59,8 @@ DEFINITIONS
 GLOBAL DATA
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-IDENT(IdSrc,"$Id: FGLGear.cpp,v 1.125 2017/02/21 21:14:13 bcoconni Exp $");
-IDENT(IdHdr,ID_LGEAR);
-
-// Body To Structural (body frame is rotated 180 deg about Y and lengths are given in
-// ft instead of inches)
+// Body To Structural (body frame is rotated 180 deg about Y and lengths are
+// given in ft instead of inches)
 const FGMatrix33 FGLGear::Tb2s(-1./inchtoft, 0., 0., 0., 1./inchtoft, 0., 0., 0., -1./inchtoft);
 const FGMatrix33 FGLGear::Ts2b(-inchtoft, 0., 0., 0., inchtoft, 0., 0., 0., -inchtoft);
 
@@ -114,7 +109,7 @@ FGLGear::FGLGear(Element* el, FGFDMExec* fdmex, int number, const struct Inputs&
   Element* strutForce = el->FindElement("strut_force");
   if (strutForce) {
     Element* springFunc = strutForce->FindElement("function");
-    fStrutForce = new FGFunction(PropertyManager, springFunc);
+    fStrutForce = new FGFunction(fdmex, springFunc);
   }
   else {
     if (el->FindElement("spring_coeff"))
@@ -173,7 +168,7 @@ FGLGear::FGLGear(Element* el, FGFDMExec* fdmex, int number, const struct Inputs&
   ForceY_Table = 0;
   Element* force_table = el->FindElement("table");
   while (force_table) {
-    string force_type = force_table->GetAttributeValue("type");
+    string force_type = force_table->GetAttributeValue("name");
     if (force_type == "CORNERING_COEFF") {
       ForceY_Table = new FGTable(PropertyManager, force_table);
       break;
@@ -185,7 +180,12 @@ FGLGear::FGLGear(Element* el, FGFDMExec* fdmex, int number, const struct Inputs&
 
   Element* element = el->FindElement("location");
   if (element) vXYZn = element->FindElementTripletConvertTo("IN");
-  else {cerr << "No location given for contact " << name << endl; exit(-1);}
+  else {
+    stringstream s;
+    s << "No location given for contact " << name;
+    cerr << endl << s.str() << endl;
+    throw BaseException(s.str());
+  }
   SetTransformType(FGForce::tCustom);
 
   element = el->FindElement("orientation");
@@ -267,7 +267,7 @@ void FGLGear::ResetToIC(void)
   // Initialize Lagrange multipliers
   for (int i=0; i < 3; i++) {
     LMultiplier[i].ForceJacobian.InitMatrix();
-    LMultiplier[i].MomentJacobian.InitMatrix();
+    LMultiplier[i].LeverArm.InitMatrix();
     LMultiplier[i].Min = 0.0;
     LMultiplier[i].Max = 0.0;
     LMultiplier[i].value = 0.0;
@@ -294,7 +294,9 @@ const FGColumnVector3& FGLGear::GetBodyForces(FGSurface *surface)
 
     // Compute the height of the theoretical location of the wheel (if strut is
     // not compressed) with respect to the ground level
-    double height = gearLoc.GetContactPoint(contact, normal, terrainVel, dummy);
+    double height = fdmex->GetInertial()->GetContactPoint(gearLoc, contact,
+                                                          normal, terrainVel,
+                                                          dummy);
 
     // Does this surface contact point interact with another surface?
     if (surface) {
@@ -314,9 +316,9 @@ const FGColumnVector3& FGLGear::GetBodyForces(FGSurface *surface)
       vGroundNormal = in.Tec2b * normal;
 
       // The height returned by GetGroundCallback() is the AGL and is expressed
-      // in the Z direction of the local coordinate frame. We now need to transform
-      // this height in actual compression of the strut (BOGEY) or in the normal
-      // direction to the ground (STRUCTURE)
+      // in the Z direction of the local coordinate frame. We now need to
+      // transform this height in actual compression of the strut (BOGEY) or in
+      // the normal direction to the ground (STRUCTURE)
       double normalZ = (in.Tec2l*normal)(eZ);
       LGearProj = -(mTGear.Transposed() * vGroundNormal)(eZ);
 
@@ -354,12 +356,19 @@ const FGColumnVector3& FGLGear::GetBodyForces(FGSurface *surface)
 
       vGroundWhlVel = mT.Transposed() * vBodyWhlVel;
 
-      if (fdmex->GetTrimStatus())
+      if (fdmex->GetTrimStatus() || in.TotalDeltaT == 0.0)
         compressSpeed = 0.0; // Steady state is sought during trimming
       else {
         compressSpeed = -vGroundWhlVel(eZ);
         if (eContactType == ctBOGEY)
           compressSpeed /= LGearProj;
+
+        // If the gear is entering in contact with the ground during the current
+        // time step, the compression speed might actually be lower than the
+        // aircraft velocity projected along the gear leg (compressSpeed).
+        double maxCompressSpeed = compressLength/in.TotalDeltaT;
+        if (fabs(compressSpeed) > maxCompressSpeed)
+          compressSpeed = sign(compressSpeed)*maxCompressSpeed;
       }
 
       ComputeVerticalStrutForce();
@@ -385,10 +394,6 @@ const FGColumnVector3& FGLGear::GetBodyForces(FGSurface *surface)
       LMultiplier[ftSide].value = 0.0;
       LMultiplier[ftDynamic].value = 0.0;
 
-      // Let wheel spin down slowly
-      vWhlVelVec(eX) -= 13.0 * in.TotalDeltaT;
-      if (vWhlVelVec(eX) < 0.0) vWhlVelVec(eX) = 0.0;
-
       // Return to neutral position between 1.0 and 0.8 gear pos.
       SteerAngle *= max(gearPos-0.8, 0.0)/0.2;
 
@@ -396,6 +401,12 @@ const FGColumnVector3& FGLGear::GetBodyForces(FGSurface *surface)
     }
   }
 
+  if (!WOW) {
+    // Let wheel spin down slowly
+    vWhlVelVec(eX) -= 13.0 * in.TotalDeltaT;
+    if (vWhlVelVec(eX) < 0.0) vWhlVelVec(eX) = 0.0;
+  }
+
   if (!fdmex->GetTrimStatus()) {
     ReportTakeoffOrLanding();
 
@@ -600,13 +611,11 @@ void FGLGear::ComputeSideForceCoefficient(void)
 
 void FGLGear::ComputeVerticalStrutForce()
 {
-  double springForce = 0;
-  double dampForce = 0;
-
   if (fStrutForce)
     StrutForce = min(fStrutForce->GetValue(), (double)0.0);
   else {
-    springForce = -compressLength * kSpring;
+    double springForce = -compressLength * kSpring;
+    double dampForce = 0;
 
     if (compressSpeed >= 0.0) {
 
@@ -678,29 +687,31 @@ void FGLGear::ComputeJacobian(const FGColumnVector3& vWhlContactVec)
     velocityDirection.Normalize();
 
     LMultiplier[ftDynamic].ForceJacobian = mT * velocityDirection;
-    LMultiplier[ftDynamic].MomentJacobian = vWhlContactVec * LMultiplier[ftDynamic].ForceJacobian;
     LMultiplier[ftDynamic].Max = 0.;
     LMultiplier[ftDynamic].Min = -fabs(staticFFactor * dynamicFCoeff * vFn(eZ));
+    LMultiplier[ftDynamic].LeverArm = vWhlContactVec;
 
-    // The Lagrange multiplier value obtained from the previous iteration is kept
-    // This is supposed to accelerate the convergence of the projected Gauss-Seidel
-    // algorithm. The code just below is to make sure that the initial value
-    // is consistent with the current friction coefficient and normal reaction.
+    // The Lagrange multiplier value obtained from the previous iteration is
+    // kept. This is supposed to accelerate the convergence of the projected
+    // Gauss-Seidel algorithm. The code just below is to make sure that the
+    // initial value is consistent with the current friction coefficient and
+    // normal reaction.
     LMultiplier[ftDynamic].value = Constrain(LMultiplier[ftDynamic].Min, LMultiplier[ftDynamic].value, LMultiplier[ftDynamic].Max);
 
     GroundReactions->RegisterLagrangeMultiplier(&LMultiplier[ftDynamic]);
   }
   else {
-    // Static friction is used for ctSTRUCTURE when the contact point is not moving.
-    // It is always used for ctBOGEY elements because the friction coefficients
-    // of a tyre depend on the direction of the movement (roll & side directions).
-    // This cannot be handled properly by the so-called "dynamic friction".
+    // Static friction is used for ctSTRUCTURE when the contact point is not
+    // moving. It is always used for ctBOGEY elements because the friction
+    // coefficients of a tyre depend on the direction of the movement (roll &
+    // side directions). This cannot be handled properly by the so-called
+    // "dynamic friction".
     StaticFriction = true;
 
     LMultiplier[ftRoll].ForceJacobian = mT * FGColumnVector3(1.,0.,0.);
     LMultiplier[ftSide].ForceJacobian = mT * FGColumnVector3(0.,1.,0.);
-    LMultiplier[ftRoll].MomentJacobian = vWhlContactVec * LMultiplier[ftRoll].ForceJacobian;
-    LMultiplier[ftSide].MomentJacobian = vWhlContactVec * LMultiplier[ftSide].ForceJacobian;
+    LMultiplier[ftRoll].LeverArm = vWhlContactVec;
+    LMultiplier[ftSide].LeverArm = vWhlContactVec;
 
     switch(eContactType) {
     case ctBOGEY:
@@ -716,10 +727,11 @@ void FGLGear::ComputeJacobian(const FGColumnVector3& vWhlContactVec)
     LMultiplier[ftRoll].Min = -LMultiplier[ftRoll].Max;
     LMultiplier[ftSide].Min = -LMultiplier[ftSide].Max;
 
-    // The Lagrange multiplier value obtained from the previous iteration is kept
-    // This is supposed to accelerate the convergence of the projected Gauss-Seidel
-    // algorithm. The code just below is to make sure that the initial value
-    // is consistent with the current friction coefficient and normal reaction.
+    // The Lagrange multiplier value obtained from the previous iteration is
+    // kept. This is supposed to accelerate the convergence of the projected
+    // Gauss-Seidel algorithm. The code just below is to make sure that the
+    // initial value is consistent with the current friction coefficient and
+    // normal reaction.
     LMultiplier[ftRoll].value = Constrain(LMultiplier[ftRoll].Min, LMultiplier[ftRoll].value, LMultiplier[ftRoll].Max);
     LMultiplier[ftSide].value = Constrain(LMultiplier[ftSide].Min, LMultiplier[ftSide].value, LMultiplier[ftSide].Max);
 
@@ -935,8 +947,6 @@ void FGLGear::Debug(int from)
   }
   if (debug_lvl & 64) {
     if (from == 0) { // Constructor
-      cout << IdSrc << endl;
-      cout << IdHdr << endl;
     }
   }
 }
