@@ -8,21 +8,21 @@
  ------------- Copyright (C) 2000  Jon S. Berndt (jon@jsbsim.org) --------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
 --------------------------------------------------------------------------------
@@ -38,40 +38,36 @@ HISTORY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include <iostream>
 #include <iomanip>
-#include <cstdlib>
 
 #include "FGMassBalance.h"
 #include "FGFDMExec.h"
-#include "input_output/FGPropertyManager.h"
 #include "input_output/FGXMLElement.h"
 
 using namespace std;
 
 namespace JSBSim {
 
-IDENT(IdSrc,"$Id: FGMassBalance.cpp,v 1.55 2016/03/26 18:54:27 bcoconni Exp $");
-IDENT(IdHdr,ID_MASSBALANCE);
-
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 
-FGMassBalance::FGMassBalance(FGFDMExec* fdmex) : FGModel(fdmex)
+FGMassBalance::FGMassBalance(FGFDMExec* fdmex)
+  : FGModel(fdmex)
 {
   Name = "FGMassBalance";
   Weight = EmptyWeight = Mass = 0.0;
 
-  vbaseXYZcg.InitMatrix(0.0);
-  vXYZcg.InitMatrix(0.0);
-  vLastXYZcg.InitMatrix(0.0);
-  vDeltaXYZcg.InitMatrix(0.0);
+  vbaseXYZcg.InitMatrix();
+  vXYZcg.InitMatrix();
+  vLastXYZcg.InitMatrix();
+  vDeltaXYZcg.InitMatrix();
   baseJ.InitMatrix();
   mJ.InitMatrix();
   mJinv.InitMatrix();
   pmJ.InitMatrix();
+  Propagate = fdmex->GetPropagate();
 
   bind();
 
@@ -82,8 +78,7 @@ FGMassBalance::FGMassBalance(FGFDMExec* fdmex) : FGModel(fdmex)
 
 FGMassBalance::~FGMassBalance()
 {
-  for (unsigned int i=0; i<PointMasses.size(); i++) delete PointMasses[i];
-  PointMasses.clear();
+  for(auto pm: PointMasses) delete pm;
 
   Debug(1);
 }
@@ -94,8 +89,8 @@ bool FGMassBalance::InitModel(void)
 {
   if (!FGModel::InitModel()) return false;
 
-  vLastXYZcg.InitMatrix(0.0);
-  vDeltaXYZcg.InitMatrix(0.0);
+  vLastXYZcg.InitMatrix();
+  vDeltaXYZcg.InitMatrix();
 
   return true;
 }
@@ -120,9 +115,16 @@ static FGMatrix33 ReadInertiaMatrix(Element* document)
   if (document->FindElement("iyz"))
     biyz = document->FindElementValueAsNumberConvertTo("iyz", "SLUG*FT2");
 
-  return FGMatrix33(  bixx,  -bixy,  bixz,
-                      -bixy,  biyy,  -biyz,
-                      bixz,  -biyz,  bizz );
+  // Transform the inertia products from the structural frame to the body frame
+  // and create the inertia matrix.
+  if (document->GetAttributeValue("negated_crossproduct_inertia") == string("false"))
+    return FGMatrix33( bixx,  bixy, -bixz,
+                       bixy,  biyy,  biyz,
+                      -bixz,  biyz,  bizz );
+  else
+    return FGMatrix33( bixx, -bixy,  bixz,
+                      -bixy,  biyy, -biyz,
+                       bixz, -biyz,  bizz );
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -134,7 +136,7 @@ bool FGMassBalance::Load(Element* document)
   Name = "Mass Properties Model: " + document->GetAttributeValue("name");
 
   // Perform base class Pre-Load
-  if (!FGModel::Load(document))
+  if (!FGModel::Upload(document, true))
     return false;
 
   SetAircraftBaseInertias(ReadInertiaMatrix(document));
@@ -168,7 +170,7 @@ bool FGMassBalance::Load(Element* document)
 
   Mass = lbtoslug*Weight;
 
-  PostLoad(document, PropertyManager);
+  PostLoad(document, FDMExec);
 
   Debug(2);
   return true;
@@ -209,7 +211,11 @@ bool FGMassBalance::Run(bool Holding)
   vDeltaXYZcg = vXYZcg - vLastXYZcg;
   vDeltaXYZcgBody = StructuralToBody(vLastXYZcg) - StructuralToBody(vXYZcg);
   vLastXYZcg = vXYZcg;
-  FDMExec->GetPropagate()->NudgeBodyLocation(vDeltaXYZcgBody);
+
+  // Compensate displacements of the structural frame when the mass distribution
+  // is modified while the aircraft is in contact with the ground.
+  if (FDMExec->GetHoldDown() || in.WOW)
+    Propagate->NudgeBodyLocation(vDeltaXYZcgBody);
 
 // Calculate new total moments of inertia
 
@@ -229,7 +235,8 @@ bool FGMassBalance::Run(bool Holding)
   Ixz = -mJ(1,3);
   Iyz = -mJ(2,3);
 
-// Calculate inertia matrix inverse (ref. Stevens and Lewis, "Flight Control & Simulation")
+// Calculate inertia matrix inverse (ref. Stevens and Lewis, "Flight Control &
+// Simulation")
 
   k1 = (Iyy*Izz - Iyz*Iyz);
   k2 = (Iyz*Ixz + Ixy*Izz);
@@ -243,9 +250,9 @@ bool FGMassBalance::Run(bool Holding)
   k5 = (Ixy*Ixz + Iyz*Ixx)*denom;
   k6 = (Ixx*Iyy - Ixy*Ixy)*denom;
 
-  mJinv.InitMatrix( k1, k2, k3,
-                    k2, k4, k5,
-                    k3, k5, k6 );
+  mJinv = { k1, k2, k3,
+            k2, k4, k5,
+            k3, k5, k6 };
 
   RunPostFunctions();
 
@@ -261,9 +268,11 @@ void FGMassBalance::AddPointMass(Element* el)
   Element* loc_element = el->FindElement("location");
   string pointmass_name = el->GetAttributeValue("name");
   if (!loc_element) {
-    cerr << el->ReadFrom() << "Pointmass " << pointmass_name
-         << " has no location." << endl;
-    exit(-1);
+    stringstream s;
+    s << el->ReadFrom() << "Pointmass " << pointmass_name
+         << " has no location.";
+    cerr << endl << s.str() << endl;
+    throw BaseException(s.str());
   }
 
   double w = el->FindElementValueAsNumberConvertTo("weight", "LBS");
@@ -316,9 +325,9 @@ double FGMassBalance::GetTotalPointMassWeight(void) const
 {
   double PM_total_weight = 0.0;
 
-  for (unsigned int i=0; i<PointMasses.size(); i++) {
-    PM_total_weight += PointMasses[i]->Weight;
-  }
+  for(auto pm: PointMasses)
+    PM_total_weight += pm->Weight;
+
   return PM_total_weight;
 }
 
@@ -328,9 +337,9 @@ const FGColumnVector3& FGMassBalance::GetPointMassMoment(void)
 {
   PointMassCG.InitMatrix();
 
-  for (unsigned int i=0; i<PointMasses.size(); i++) {
-    PointMassCG += PointMasses[i]->Weight*PointMasses[i]->Location;
-  }
+  for (auto pm: PointMasses)
+    PointMassCG += pm->Weight * pm->Location;
+
   return PointMassCG;
 }
 
@@ -338,15 +347,13 @@ const FGColumnVector3& FGMassBalance::GetPointMassMoment(void)
 
 const FGMatrix33& FGMassBalance::CalculatePMInertias(void)
 {
-  size_t size = PointMasses.size();
+  if (PointMasses.empty()) return pmJ;
 
-  if (size == 0) return pmJ;
+  pmJ.InitMatrix();
 
-  pmJ = FGMatrix33();
-
-  for (unsigned int i=0; i<size; i++) {
-    pmJ += GetPointmassInertia( lbtoslug * PointMasses[i]->Weight, PointMasses[i]->Location );
-    pmJ += PointMasses[i]->GetPointMassInertia();
+  for (auto pm: PointMasses) {
+    pmJ += GetPointmassInertia( lbtoslug * pm->Weight, pm->Location );
+    pmJ += pm->GetPointMassInertia();
   }
 
   return pmJ;
@@ -416,7 +423,7 @@ void FGMassBalance::bind(void)
                        &FGMassBalance::GetIyz);
   typedef int (FGMassBalance::*iOPV)() const;
   PropertyManager->Tie("inertia/print-mass-properties", this, (iOPV)0,
-                       &FGMassBalance::GetMassPropertiesReport, false);
+                       &FGMassBalance::GetMassPropertiesReport);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -444,11 +451,11 @@ void FGMassBalance::PointMass::bind(FGPropertyManager* PropertyManager,
 
 void FGMassBalance::GetMassPropertiesReport(int i)
 {
-  cout << endl << fgblue << highint 
+  cout << endl << fgblue << highint
        << "  Mass Properties Report (English units: lbf, in, slug-ft^2)"
        << reset << endl;
   cout << "                                  " << underon << "    Weight    CG-X    CG-Y"
-       << "    CG-Z         Ixx         Iyy         Izz" 
+       << "    CG-Z         Ixx         Iyy         Izz"
        << "         Ixy         Ixz         Iyz" << underoff << endl;
   cout.precision(1);
   cout << highint << setw(34) << left << "    Base Vehicle " << normint
@@ -464,13 +471,13 @@ void FGMassBalance::GetMassPropertiesReport(int i)
          << right << setw(10) << pmweight << setw(8) << pm->GetLocation()(eX)
          << setw(8) << pm->GetLocation()(eY) << setw(8) << pm->GetLocation()(eZ)
          << setw(12) << pm->GetPointMassMoI(1,1) << setw(12) << pm->GetPointMassMoI(2,2) << setw(12) << pm->GetPointMassMoI(3,3)
-         << setw(12) << pm->GetPointMassMoI(1,2) << setw(12) << pm->GetPointMassMoI(1,3) << setw(12) << pm->GetPointMassMoI(2,3) << endl;         
+         << setw(12) << pm->GetPointMassMoI(1,2) << setw(12) << pm->GetPointMassMoI(1,3) << setw(12) << pm->GetPointMassMoI(2,3) << endl;
   }
 
   cout << FDMExec->GetPropulsionTankReport();
 
   cout << "    " << underon << setw(136) << " " << underoff << endl;
-  cout << highint << left << setw(30) << "    Total: " << right << setw(14) << Weight 
+  cout << highint << left << setw(30) << "    Total: " << right << setw(14) << Weight
        << setw(8) << vXYZcg(eX)
        << setw(8) << vXYZcg(eY)
        << setw(8) << vXYZcg(eZ)
@@ -548,8 +555,6 @@ void FGMassBalance::Debug(int from)
   }
   if (debug_lvl & 64) {
     if (from == 0) { // Constructor
-      cout << IdSrc << endl;
-      cout << IdHdr << endl;
     }
   }
 }
