@@ -7,21 +7,21 @@
  ------------- Copyright (C) 1999  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 HISTORY
 --------------------------------------------------------------------------------
@@ -39,17 +39,8 @@ INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 #include "models/FGModel.h"
-#include "math/FGColumnVector3.h"
 #include "math/FGLocation.h"
 #include "math/FGQuaternion.h"
-#include "math/FGMatrix33.h"
-#include <deque>
-
-/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-DEFINITIONS
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
-
-#define ID_PROPAGATE "$Id: FGPropagate.h,v 1.81 2014/05/17 15:15:53 jberndt Exp $"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
@@ -57,8 +48,8 @@ FORWARD DECLARATIONS
 
 namespace JSBSim {
 
-using std::deque;
 class FGInitialCondition;
+class FGInertial;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS DOCUMENTATION
@@ -93,7 +84,6 @@ CLASS DOCUMENTATION
     @endcode
 
     @author Jon S. Berndt, Mathias Froehlich, Bertrand Coconnier
-    @version $Id: FGPropagate.h,v 1.81 2014/05/17 15:15:53 jberndt Exp $
   */
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -134,14 +124,16 @@ public:
         body frame relative to the inertial (ECI) frame. */
     FGQuaternion qAttitudeECI;
 
+    FGQuaternion vQtrndot;
+
     FGColumnVector3 vInertialVelocity;
 
     FGColumnVector3 vInertialPosition;
 
-    deque <FGColumnVector3> dqPQRidot;
-    deque <FGColumnVector3> dqUVWidot;
-    deque <FGColumnVector3> dqInertialVelocity;
-    deque <FGQuaternion>    dqQtrndot;
+    std::deque <FGColumnVector3> dqPQRidot;
+    std::deque <FGColumnVector3> dqUVWidot;
+    std::deque <FGColumnVector3> dqInertialVelocity;
+    std::deque <FGQuaternion>    dqQtrndot;
   };
 
   /** Constructor.
@@ -152,7 +144,7 @@ public:
       - integrator, rotational position = Trapezoidal
       - integrator, translational position = Trapezoidal
       @param Executive a pointer to the parent executive object */
-  FGPropagate(FGFDMExec* Executive);
+  explicit FGPropagate(FGFDMExec* Executive);
 
   /// Destructor
   ~FGPropagate();
@@ -191,7 +183,7 @@ public:
   const FGColumnVector3& GetVel(void) const { return vVel; }
 
   /** Retrieves the body frame vehicle velocity vector.
-      The vector returned is represented by an FGColumnVector reference. The vector
+      The vector returned is represented by an FGColumnVector3 reference. The vector
       for the velocity in Body frame is organized (Vx, Vy, Vz). The vector
       is 1-based, so that the first element can be retrieved using the "()" operator.
       In other words, vUVW(1) is Vx. Various convenience enumerators are defined
@@ -205,7 +197,7 @@ public:
   /** Retrieves the body angular rates vector, relative to the ECEF frame.
       Retrieves the body angular rates (p, q, r), which are calculated by integration
       of the angular acceleration.
-      The vector returned is represented by an FGColumnVector reference. The vector
+      The vector returned is represented by an FGColumnVector3 reference. The vector
       for the angular velocity in Body frame is organized (P, Q, R). The vector
       is 1-based, so that the first element can be retrieved using the "()" operator.
       In other words, vPQR(1) is P. Various convenience enumerators are defined
@@ -230,6 +222,17 @@ public:
   */
   const FGColumnVector3& GetPQRi(void) const {return VState.vPQRi;}
 
+  /** Retrieves the time derivative of the body orientation quaternion.
+      Retrieves the time derivative of the body orientation quaternion based on
+      the rate of change of the orientation between the body and the ECI frame.
+      The quaternion returned is represented by an FGQuaternion reference. The
+      quaternion is 1-based, so that the first element can be retrieved using
+      the "()" operator.
+      units rad/sec^2
+      @return The time derivative of the body orientation quaternion.
+  */
+  const FGQuaternion& GetQuaterniondot(void) const {return VState.vQtrndot;}
+
   /** Retrieves the Euler angles that define the vehicle orientation.
       Extracts the Euler angles from the quaternion that stores the orientation
       in the Local frame. The order of rotation used is Yaw-Pitch-Roll. The
@@ -322,7 +325,7 @@ public:
       units ft
       @return The current altitude above sea level in feet.
   */
-  double GetAltitudeASL(void) const { return VState.vLocation.GetAltitudeASL(); }
+  double GetAltitudeASL(void) const;
 
   /** Returns the current altitude above sea level.
       This function returns the altitude above sea level.
@@ -420,15 +423,21 @@ public:
       */
   double GetLocalTerrainRadius(void) const;
 
-  double GetEarthPositionAngle(void) const { return VState.vLocation.GetEPA(); }
+  /** Returns the Earth position angle.
+      @return Earth position angle in radians.
+   */
+  double GetEarthPositionAngle(void) const { return epa; }
 
-  double GetEarthPositionAngleDeg(void) const { return GetEarthPositionAngle()*radtodeg;}
+  /** Returns the Earth position angle in degrees.
+      @return Earth position angle in degrees.
+  */
+  double GetEarthPositionAngleDeg(void) const { return epa*radtodeg;}
 
   const FGColumnVector3& GetTerrainVelocity(void) const { return LocalTerrainVelocity; }
   const FGColumnVector3& GetTerrainAngularVelocity(void) const { return LocalTerrainAngularVelocity; }
   void RecomputeLocalTerrainVelocity();
 
-  double GetTerrainElevation(void) const { return GetLocalTerrainRadius() - VState.vLocation.GetSeaLevelRadius(); }
+  double GetTerrainElevation(void) const;
   double GetDistanceAGL(void)  const;
   double GetDistanceAGLKm(void)  const;
   double GetRadius(void) const {
@@ -478,11 +487,13 @@ public:
   const FGMatrix33& GetTb2i(void) const { return Tb2i; }
 
   /** Retrieves the ECEF-to-ECI transformation matrix.
-      @return a reference to the ECEF-to-ECI transformation matrix.  */
+      @return a reference to the ECEF-to-ECI transformation matrix.
+      @see SetEarthPositionAngle */
   const FGMatrix33& GetTec2i(void) const { return Tec2i; }
 
   /** Retrieves the ECI-to-ECEF transformation matrix.
-      @return a reference to the ECI-to-ECEF matrix.  */
+      @return a reference to the ECI-to-ECEF matrix.
+      @see SetEarthPositionAngle */
   const FGMatrix33& GetTi2ec(void) const { return Ti2ec; }
 
   /** Retrieves the ECEF-to-local transformation matrix.
@@ -498,18 +509,25 @@ public:
   const FGMatrix33& GetTl2ec(void) const { return Tl2ec; }
 
   /** Retrieves the local-to-inertial transformation matrix.
-      @return a reference to the local-to-inertial transformation matrix.  */
+      @return a reference to the local-to-inertial transformation matrix.
+      @see SetEarthPositionAngle */
   const FGMatrix33& GetTl2i(void) const { return Tl2i; }
 
   /** Retrieves the inertial-to-local transformation matrix.
-      @return a reference to the inertial-to-local matrix.  */
+      @return a reference to the inertial-to-local matrix.
+      @see SetEarthPositionAngle */
   const FGMatrix33& GetTi2l(void) const { return Ti2l; }
 
   const VehicleState& GetVState(void) const { return VState; }
 
   void SetVState(const VehicleState& vstate);
 
-  void SetEarthPositionAngle(double epa) {VState.vLocation.SetEarthPositionAngle(epa);}
+  /** Sets the Earth position angle.
+      This is the relative angle around the Z axis of the ECEF frame with
+      respect to the inertial frame.
+      @param EPA Earth position angle in radians.
+   */
+  void SetEarthPositionAngle(double EPA) {epa = EPA;}
 
   void SetInertialOrientation(const FGQuaternion& Qi);
   void SetInertialVelocity(const FGColumnVector3& Vi);
@@ -551,23 +569,17 @@ public:
   void SetRadius(double r)
   {
     VState.vLocation.SetRadius(r);
-    VehicleRadius = r;
     VState.vInertialPosition = Tec2i * VState.vLocation;
   }
 
-  void SetAltitudeASL(double altASL)
-  {
-    VState.vLocation.SetAltitudeASL(altASL);
-    UpdateVehicleState();
-  }
+  void SetAltitudeASL(double altASL);
   void SetAltitudeASLmeters(double altASL) { SetAltitudeASL(altASL/fttom); }
 
-  void SetSeaLevelRadius(double tt);
   void SetTerrainElevation(double tt);
   void SetDistanceAGL(double tt);
   void SetDistanceAGLKm(double tt);
 
-  void SetInitialState(const FGInitialCondition *);
+  void SetInitialState(const FGInitialCondition*);
   void SetLocation(const FGLocation& l);
   void SetLocation(const FGColumnVector3& lv)
   {
@@ -585,11 +597,16 @@ public:
     VState.vLocation -= Tb2ec*deltaLoc;
   }
 
+  /** Sets the property forces/hold-down. This allows to do hard 'hold-down'
+      such as for rockets on a launch pad with engines ignited.
+      @param hd enables the 'hold-down' function if non-zero
+  */
+  void SetHoldDown(bool hd);
+
   void DumpState(void);
 
   struct Inputs {
     FGColumnVector3 vPQRidot;
-    FGQuaternion vQtrndot;
     FGColumnVector3 vUVWidot;
     FGColumnVector3 vOmegaPlanet;
     double SemiMajor;
@@ -603,6 +620,7 @@ private:
 
   struct VehicleState VState;
 
+  FGInertial* Inertial = nullptr;
   FGColumnVector3 vVel;
   FGMatrix33 Tec2b;
   FGMatrix33 Tb2ec;
@@ -616,10 +634,10 @@ private:
   FGMatrix33 Tb2i;   // body to ECI frame rotation matrix
   FGMatrix33 Ti2l;
   FGMatrix33 Tl2i;
+  double epa;        // Earth Position Angle
 
   FGQuaternion Qec2b;
 
-  double VehicleRadius;
   FGColumnVector3 LocalTerrainVelocity, LocalTerrainAngularVelocity;
 
   eIntegrateType integrator_rotational_rate;
@@ -629,16 +647,17 @@ private:
 
   void CalculateInertialVelocity(void);
   void CalculateUVW(void);
+  void CalculateQuatdot(void);
 
   void Integrate( FGColumnVector3& Integrand,
                   FGColumnVector3& Val,
-                  deque <FGColumnVector3>& ValDot,
+                  std::deque <FGColumnVector3>& ValDot,
                   double dt,
                   eIntegrateType integration_type);
 
   void Integrate( FGQuaternion& Integrand,
                   FGQuaternion& Val,
-                  deque <FGQuaternion>& ValDot,
+                  std::deque <FGQuaternion>& ValDot,
                   double dt,
                   eIntegrateType integration_type);
 
