@@ -9,21 +9,21 @@
  ------------- Copyright (C) 2000  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
 --------------------------------------------------------------------------------
@@ -44,9 +44,6 @@ HISTORY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include <iostream>
-#include <sstream>
-#include <cstdlib>
 #include <iomanip>
 
 #include "FGFDMExec.h"
@@ -59,18 +56,13 @@ INCLUDES
 #include "models/propulsion/FGTurboProp.h"
 #include "models/propulsion/FGTank.h"
 #include "input_output/FGModelLoader.h"
-#include "math/FGColumnVector3.h"
 
 using namespace std;
 
 namespace JSBSim {
 
-IDENT(IdSrc,"$Id: FGPropulsion.cpp,v 1.88 2017/02/25 14:23:19 bcoconni Exp $");
-IDENT(IdHdr,ID_PROPULSION);
-
 extern short debug_lvl;
 
-
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
@@ -212,7 +204,7 @@ void FGPropulsion::ConsumeFuel(FGEngine* engine)
       if (TankPriority != 0) {
         switch(Tank->GetType()) {
         case FGTank::ttFUEL:
-          if ((Tank->GetContents() > 0.0) && Tank->GetSelected() && (TankPriority == CurrentFuelTankPriority)) {
+          if ((Tank->GetContents() > Tank->GetUnusable()) && Tank->GetSelected() && (TankPriority == CurrentFuelTankPriority)) {
             TanksWithFuel++;
             Starved = false;
             FeedListFuel.push_back(TankId);
@@ -244,7 +236,7 @@ void FGPropulsion::ConsumeFuel(FGEngine* engine)
             break;
           case FGTank::ttOXIDIZER:
             hasOxTanks = true;
-            if (Tank->GetContents() > 0.0 && Tank->GetSelected() && TankPriority == CurrentOxidizerTankPriority) {
+            if (Tank->GetContents() > Tank->GetUnusable() && Tank->GetSelected() && TankPriority == CurrentOxidizerTankPriority) {
               TanksWithOxidizer++;
               if (TanksWithFuel > 0) Starved = false;
               FeedListOxi.push_back(TankId);
@@ -373,7 +365,7 @@ bool FGPropulsion::Load(Element* el)
   Name = "Propulsion Model: " + el->GetAttributeValue("name");
 
   // Perform base class Pre-Load
-  if (!FGModel::Load(el))
+  if (!FGModel::Upload(el, true))
     return false;
 
   // Process tank definitions first to establish the number of fuel tanks
@@ -456,7 +448,7 @@ bool FGPropulsion::Load(Element* el)
   }
 
 
-  PostLoad(el, PropertyManager);
+  PostLoad(el, FDMExec);
 
   return true;
 }
@@ -589,7 +581,7 @@ const FGMatrix33& FGPropulsion::CalculateTankInertias(void)
 
   if (size == 0) return tankJ;
 
-  tankJ = FGMatrix33();
+  tankJ.InitMatrix();
 
   for (unsigned int i=0; i<size; i++) {
     tankJ += FDMExec->GetMassBalance()->GetPointmassInertia( lbtoslug * Tanks[i]->GetContents(),
@@ -640,6 +632,21 @@ void FGPropulsion::SetStarter(int setting)
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
+int FGPropulsion::GetStarter(void) const
+{
+  if (ActiveEngine < 0) {
+    bool starter = true;
+
+    for (unsigned i=0; i<Engines.size(); i++)
+      starter &= Engines[i]->GetStarter();
+
+    return starter ? 1 : 0;
+  } else
+    return Engines[ActiveEngine]->GetStarter() ? 1: 0;
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
 void FGPropulsion::SetCutoff(int setting)
 {
   bool bsetting = setting == 0 ? false : true;
@@ -673,6 +680,41 @@ void FGPropulsion::SetCutoff(int setting)
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
+int FGPropulsion::GetCutoff(void) const
+{
+  if (ActiveEngine < 0) {
+    bool cutoff = true;
+
+    for (unsigned i=0; i<Engines.size(); i++) {
+      switch (Engines[i]->GetType()) { 
+      case FGEngine::etTurbine:
+        cutoff &= ((FGTurbine*)Engines[i])->GetCutoff();
+        break;
+      case FGEngine::etTurboprop:
+        cutoff &= ((FGTurboProp*)Engines[i])->GetCutoff();
+        break;
+      default:
+        return -1;
+      }
+    }
+
+    return cutoff ? 1 : 0;
+  } else {
+    switch (Engines[ActiveEngine]->GetType()) {
+    case FGEngine::etTurbine:
+      return ((FGTurbine*)Engines[ActiveEngine])->GetCutoff() ? 1 : 0;
+    case FGEngine::etTurboprop:
+      return ((FGTurboProp*)Engines[ActiveEngine])->GetCutoff() ? 1 : 0;
+    default:
+      break;
+    }
+  }
+
+  return -1;
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
 void FGPropulsion::SetActiveEngine(int engine)
 {
   if (engine >= (int)Engines.size() || engine < 0)
@@ -760,21 +802,22 @@ void FGPropulsion::bind(void)
 {
   typedef double (FGPropulsion::*PMF)(int) const;
   typedef int (FGPropulsion::*iPMF)(void) const;
+  typedef bool (FGPropulsion::*bPMF)(void) const;
 
   IsBound = true;
-  PropertyManager->Tie("propulsion/set-running", this, (iPMF)0, &FGPropulsion::InitRunning, false);
+  PropertyManager->Tie("propulsion/set-running", this, (iPMF)0, &FGPropulsion::InitRunning);
   if (HaveTurbineEngine || HaveTurboPropEngine) {
-    PropertyManager->Tie("propulsion/starter_cmd", this, (iPMF)0, &FGPropulsion::SetStarter,  false);
-    PropertyManager->Tie("propulsion/cutoff_cmd", this,  (iPMF)0, &FGPropulsion::SetCutoff,   false);
+    PropertyManager->Tie("propulsion/starter_cmd", this, &FGPropulsion::GetStarter, &FGPropulsion::SetStarter);
+    PropertyManager->Tie("propulsion/cutoff_cmd", this,  &FGPropulsion::GetCutoff, &FGPropulsion::SetCutoff);
   }
 
   if (HavePistonEngine) {
-    PropertyManager->Tie("propulsion/starter_cmd", this, (iPMF)0, &FGPropulsion::SetStarter,  false);
-    PropertyManager->Tie("propulsion/magneto_cmd", this, (iPMF)0, &FGPropulsion::SetMagnetos, false);
+    PropertyManager->Tie("propulsion/starter_cmd", this, &FGPropulsion::GetStarter, &FGPropulsion::SetStarter);
+    PropertyManager->Tie("propulsion/magneto_cmd", this, (iPMF)0, &FGPropulsion::SetMagnetos);
   }
 
   PropertyManager->Tie("propulsion/active_engine", this, (iPMF)&FGPropulsion::GetActiveEngine,
-                        &FGPropulsion::SetActiveEngine, true);
+                        &FGPropulsion::SetActiveEngine);
   PropertyManager->Tie("forces/fbx-prop-lbs", this, eX, (PMF)&FGPropulsion::GetForces);
   PropertyManager->Tie("forces/fby-prop-lbs", this, eY, (PMF)&FGPropulsion::GetForces);
   PropertyManager->Tie("forces/fbz-prop-lbs", this, eZ, (PMF)&FGPropulsion::GetForces);
@@ -785,6 +828,7 @@ void FGPropulsion::bind(void)
   TotalOxidizerQuantity = PropertyManager->CreatePropertyObject<double>("propulsion/total-oxidizer-lbs");
   refuel = PropertyManager->CreatePropertyObject<bool>("propulsion/refuel");
   dump = PropertyManager->CreatePropertyObject<bool>("propulsion/fuel_dump");
+  PropertyManager->Tie("propulsion/fuel_freeze", this, (bPMF)nullptr, &FGPropulsion::SetFuelFreeze);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -827,8 +871,6 @@ void FGPropulsion::Debug(int from)
   }
   if (debug_lvl & 64) {
     if (from == 0) { // Constructor
-      cout << IdSrc << endl;
-      cout << IdHdr << endl;
     }
   }
 }
