@@ -4,27 +4,28 @@
  Author:       Jon Berndt, Tony Peden, Andreas Gaeb
  Date started: Extracted from FGAtmosphere, which originated in 1998
                5/2011
- Purpose:      Models winds, gusts, turbulence, and other atmospheric disturbances
+ Purpose:      Models winds, gusts, turbulence, and other atmospheric
+               disturbances
  Called by:    FGFDMExec
 
  ------------- Copyright (C) 2011  Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
 --------------------------------------------------------------------------------
@@ -42,18 +43,14 @@ COMMENTS, REFERENCES,  and NOTES
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include <iostream>
-#include <cstdlib>
 #include "FGWinds.h"
 #include "FGFDMExec.h"
+#include "math/FGTable.h"
 
 using namespace std;
 
 namespace JSBSim {
 
-IDENT(IdSrc,"$Id: FGWinds.cpp,v 1.15 2015/02/27 20:49:36 bcoconni Exp $");
-IDENT(IdHdr,ID_WINDS);
-
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
@@ -72,7 +69,7 @@ static inline double square_signed (double value)
 */
 
 /// simply square a value
-static inline double sqr(double x) { return x*x; }
+constexpr double sqr(double x) { return x*x; }
 
 FGWinds::FGWinds(FGFDMExec* fdmex) : FGModel(fdmex)
 {
@@ -282,7 +279,7 @@ void FGWinds::Turbulence(double h)
     }
 
     // keep values from last timesteps
-    // TODO maybe use deque?
+    // TODO: maybe use deque?
     static double
       xi_u_km1 = 0, nu_u_km1 = 0,
       xi_v_km1 = 0, xi_v_km2 = 0, nu_v_km1 = 0, nu_v_km2 = 0,
@@ -441,7 +438,7 @@ void FGWinds::CosineGust()
   if (profile.elapsedTime > (profile.startupDuration + profile.steadyDuration + profile.endDuration)) {
     profile.Running = false;
     profile.elapsedTime = 0.0;
-    oneMinusCosineGust.vWindTransformed.InitMatrix(0.0);
+    oneMinusCosineGust.vWindTransformed.InitMatrix();
     vCosineGust.InitMatrix(0);
   }
 }
@@ -524,7 +521,7 @@ void FGWinds::bind(void)
   PropertyManager->Tie("atmosphere/cosine-gust/X-velocity-ft_sec", this, (Ptr)0L, &FGWinds::GustXComponent);
   PropertyManager->Tie("atmosphere/cosine-gust/Y-velocity-ft_sec", this, (Ptr)0L, &FGWinds::GustYComponent);
   PropertyManager->Tie("atmosphere/cosine-gust/Z-velocity-ft_sec", this, (Ptr)0L, &FGWinds::GustZComponent);
-  PropertyManager->Tie("atmosphere/cosine-gust/start", this, (PMFt)0L, (PMFi)&FGWinds::StartGust);
+  PropertyManager->Tie("atmosphere/cosine-gust/start", this, static_cast<bool (FGWinds::*)(void) const>(nullptr), &FGWinds::StartGust);
 
   // User-specified Up- Down-burst parameters
   PropertyManager->Tie("atmosphere/updownburst/number-of-cells", this, (PMFt)0L, &FGWinds::NumberOfUpDownburstCells);
@@ -609,8 +606,6 @@ void FGWinds::Debug(int from)
   }
   if (debug_lvl & 64) {
     if (from == 0) { // Constructor
-      cout << IdSrc << endl;
-      cout << IdHdr << endl;
     }
   }
 }
