@@ -7,21 +7,21 @@
  ------------- Copyright (C) 2013 Jon S. Berndt (jon@jsbsim.org) -------------
 
  This program is free software; you can redistribute it and/or modify it under
- the terms of the GNU Lesser General Public License as published by the Free Software
- Foundation; either version 2 of the License, or (at your option) any later
- version.
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2 of the License, or (at your option) any
+ later version.
 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  details.
 
- You should have received a copy of the GNU Lesser General Public License along with
- this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- Place - Suite 330, Boston, MA  02111-1307, USA.
+ You should have received a copy of the GNU Lesser General Public License along
+ with this program; if not, write to the Free Software Foundation, Inc., 59
+ Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
- Further information about the GNU Lesser General Public License can also be found on
- the world wide web at http://www.gnu.org.
+ Further information about the GNU Lesser General Public License can also be
+ found on the world wide web at http://www.gnu.org.
 
 FUNCTIONAL DESCRIPTION
 --------------------------------------------------------------------------------
@@ -39,23 +39,23 @@ INCLUDES
 
 #include "FGWaypoint.h"
 #include "input_output/FGXMLElement.h"
-#include "input_output/FGPropertyManager.h"
 #include "math/FGLocation.h"
+#include "models/FGFCS.h"
+#include "models/FGInertial.h"
+#include "initialization/FGInitialCondition.h"
 
 using namespace std;
 
 namespace JSBSim {
 
-IDENT(IdSrc,"$Id: FGWaypoint.cpp,v 1.8 2016/05/05 15:32:42 bcoconni Exp $");
-IDENT(IdHdr,ID_WAYPOINT);
-
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-FGWaypoint::FGWaypoint(FGFCS* fcs, Element* element) : FGFCSComponent(fcs, element)
+FGWaypoint::FGWaypoint(FGFCS* fcs, Element* element)
+  : FGFCSComponent(fcs, element)
 {
   if      (Type == "WAYPOINT_HEADING")  WaypointType = eHeading;
   else if (Type == "WAYPOINT_DISTANCE") WaypointType = eDistance;
@@ -64,9 +64,11 @@ FGWaypoint::FGWaypoint(FGFCS* fcs, Element* element) : FGFCSComponent(fcs, eleme
   target_longitude_unit = 1.0;
   source_latitude_unit = 1.0;
   source_longitude_unit = 1.0;
+  source = fcs->GetExec()->GetIC()->GetPosition();
 
   if (element->FindElement("target_latitude") ) {
-    target_latitude = PropertyManager->CreatePropertyObject<double>(element->FindElementValue("target_latitude"));
+    target_latitude.reset(new FGPropertyValue(element->FindElementValue("target_latitude"),
+                                              PropertyManager));
     if (element->FindElement("target_latitude")->HasAttribute("unit")) {
       if (element->FindElement("target_latitude")->GetAttributeValue("unit") == "DEG") {
         target_latitude_unit = 0.017453293;
@@ -80,7 +82,8 @@ FGWaypoint::FGWaypoint(FGFCS* fcs, Element* element) : FGFCSComponent(fcs, eleme
   }
 
   if (element->FindElement("target_longitude") ) {
-    target_longitude = PropertyManager->CreatePropertyObject<double>(element->FindElementValue("target_longitude"));
+    target_longitude.reset(new FGPropertyValue(element->FindElementValue("target_longitude"),
+                                               PropertyManager));
     if (element->FindElement("target_longitude")->HasAttribute("unit")) {
       if (element->FindElement("target_longitude")->GetAttributeValue("unit") == "DEG") {
         target_longitude_unit = 0.017453293;
@@ -94,7 +97,8 @@ FGWaypoint::FGWaypoint(FGFCS* fcs, Element* element) : FGFCSComponent(fcs, eleme
   }
 
   if (element->FindElement("source_latitude") ) {
-    source_latitude = PropertyManager->CreatePropertyObject<double>(element->FindElementValue("source_latitude"));
+    source_latitude.reset(new FGPropertyValue(element->FindElementValue("source_latitude"),
+                                              PropertyManager));
     if (element->FindElement("source_latitude")->HasAttribute("unit")) {
       if (element->FindElement("source_latitude")->GetAttributeValue("unit") == "DEG") {
         source_latitude_unit = 0.017453293;
@@ -108,7 +112,8 @@ FGWaypoint::FGWaypoint(FGFCS* fcs, Element* element) : FGFCSComponent(fcs, eleme
   }
 
   if (element->FindElement("source_longitude") ) {
-    source_longitude = PropertyManager->CreatePropertyObject<double>(element->FindElementValue("source_longitude"));
+    source_longitude.reset(new FGPropertyValue(element->FindElementValue("source_longitude"),
+                                               PropertyManager));
     if (element->FindElement("source_longitude")->HasAttribute("unit")) {
       if (element->FindElement("source_longitude")->GetAttributeValue("unit") == "DEG") {
         source_longitude_unit = 0.017453293;
@@ -124,9 +129,7 @@ FGWaypoint::FGWaypoint(FGFCS* fcs, Element* element) : FGFCSComponent(fcs, eleme
   if (element->FindElement("radius"))
     radius = element->FindElementValueAsNumberConvertTo("radius", "FT");
   else {
-    FGLocation source(source_longitude * source_latitude_unit,
-                      source_latitude * source_longitude_unit, 1.0);
-    radius = source.GetSeaLevelRadius(); // Radius of Earth in feet.
+    radius = 20925646.32546; // Radius of Earth in feet.
   }
 
   unit = element->GetAttributeValue("unit");
@@ -158,7 +161,7 @@ FGWaypoint::FGWaypoint(FGFCS* fcs, Element* element) : FGFCSComponent(fcs, eleme
     }
   }
 
-  FGFCSComponent::bind();
+  bind(element);
   Debug(0);
 }
 
@@ -173,12 +176,30 @@ FGWaypoint::~FGWaypoint()
 
 bool FGWaypoint::Run(void )
 {
-  double target_latitude_rad = target_latitude * target_latitude_unit;
-  double target_longitude_rad = target_longitude * target_longitude_unit;
-  FGLocation source(source_longitude * source_latitude_unit,
-                    source_latitude * source_longitude_unit, radius);
+  double source_latitude_rad = source_latitude->GetValue() * source_latitude_unit;
+  double source_longitude_rad = source_longitude->GetValue() * source_longitude_unit;
+  double target_latitude_rad = target_latitude->GetValue() * target_latitude_unit;
+  double target_longitude_rad = target_longitude->GetValue() * target_longitude_unit;
+  source.SetPosition(source_longitude_rad, source_latitude_rad, radius);
+
+  if (fabs(target_latitude_rad) > M_PI/2.0) {
+    cerr << endl;
+    cerr << "Target latitude in waypoint \"" << Name << "\" must be less than or equal to 90 degrees." << endl;
+    cerr << "(is longitude being mistakenly supplied?)" << endl;
+    cerr << endl;
+    throw("Waypoint target latitude exceeded 90 degrees.");
+  }
+
+  if (fabs(source_latitude_rad) > M_PI/2.0) {
+    cerr << endl;
+    cerr << "Source latitude in waypoint \"" << Name << "\" must be less than or equal to 90 degrees." << endl;
+    cerr << "(is longitude being mistakenly supplied?)" << endl;
+    cerr << endl;
+    throw("Source latitude exceeded 90 degrees.");
+  }
 
   if (WaypointType == eHeading) {     // Calculate Heading
+
     double heading_to_waypoint_rad = source.GetHeadingTo(target_longitude_rad,
                                                          target_latitude_rad);
 
@@ -186,15 +207,16 @@ bool FGWaypoint::Run(void )
     else               Output = heading_to_waypoint_rad;
 
   } else {                            // Calculate Distance
+
     double wp_distance = source.GetDistanceTo(target_longitude_rad,
                                               target_latitude_rad);
-
     if (eUnit == eMeters) Output = FeetToMeters(wp_distance);
     else                  Output = wp_distance;
+
   }
 
   Clip();
-  if (IsOutput) SetOutput();
+  SetOutput();
 
   return true;
 }
@@ -238,8 +260,6 @@ void FGWaypoint::Debug(int from)
   }
   if (debug_lvl & 64) {
     if (from == 0) { // Constructor
-      cout << IdSrc << endl;
-      cout << IdHdr << endl;
     }
   }
 }
