@@ -52,9 +52,6 @@ using namespace std;
 
 namespace JSBSim {
 
-IDENT(IdSrc,"$Id: FGPiston.cpp,v 1.84 2017/06/03 19:49:20 bcoconni Exp $");
-IDENT(IdHdr,ID_PISTON);
-
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
@@ -127,7 +124,7 @@ FGPiston::FGPiston(FGFDMExec* exec, Element* el, int engine_number, struct Input
   bTakeoffBoost = false;
   TakeoffBoost = 0.0;   // Default to no extra takeoff-boost
   BoostLossFactor = 0.0;   // Default to free boost
-  
+
   int i;
   for (i=0; i<FG_MAX_BOOST_SPEEDS; i++) {
     RatedBoost[i] = 0.0;
@@ -145,7 +142,7 @@ FGPiston::FGPiston(FGFDMExec* exec, Element* el, int engine_number, struct Input
 
   // Read inputs from engine data file where present.
 
-  if (el->FindElement("minmp")) 
+  if (el->FindElement("minmp"))
     MinManifoldPressure_inHg = el->FindElementValueAsNumberConvertTo("minmp","INHG");
   if (el->FindElement("maxmp"))
     MaxManifoldPressure_inHg = el->FindElementValueAsNumberConvertTo("maxmp","INHG");
@@ -312,7 +309,7 @@ FGPiston::FGPiston(FGFDMExec* exec, Element* el, int engine_number, struct Input
     Z_airbox = (standard_pressure *Ze / maxMAP) - Ze; // impedence of airbox
   }
   // Constant for Throttle impedence
-  Z_throttle=(PeakMeanPistonSpeed_fps/((IdleRPM * Stroke) / 360))*(standard_pressure/minMAP - 1) - Z_airbox; 
+  Z_throttle=(PeakMeanPistonSpeed_fps/((IdleRPM * Stroke) / 360))*(standard_pressure/minMAP - 1) - Z_airbox;
   //  Z_throttle=(MaxRPM/IdleRPM )*(standard_pressure/minMAP+2); // Constant for Throttle impedence
 
 // Default tables if not provided in the configuration file
@@ -391,6 +388,8 @@ FGPiston::FGPiston(FGFDMExec* exec, Element* el, int engine_number, struct Input
     property_name = base_property_name + "/boostloss-hp";
     PropertyManager->Tie(property_name, &BoostLossHP);
   }
+  property_name = base_property_name + "/AFR";
+  PropertyManager->Tie(property_name, this, &FGPiston::getAFR);
 
   // Set up and sanity-check the turbo/supercharging configuration based on the input values.
   if (TakeoffBoost > RatedBoost[0]) bTakeoffBoost = true;
@@ -403,13 +402,13 @@ FGPiston::FGPiston(FGFDMExec* exec, Element* el, int engine_number, struct Input
     if (bad) {
       // We can't recover from the above - don't use this supercharger speed.
       BoostSpeeds--;
-      // TODO - put out a massive error message!
+      // TODO: put out a massive error message!
       break;
     }
     // Now sanity-check stuff that is recoverable.
     if (i < BoostSpeeds - 1) {
       if (BoostSwitchAltitude[i] < RatedAltitude[i]) {
-        // TODO - put out an error message
+        // TODO: put out an error message
         // But we can also make a reasonable estimate, as below.
         BoostSwitchAltitude[i] = RatedAltitude[i] + 1000;
       }
@@ -519,7 +518,10 @@ void FGPiston::Calculate(void)
   }
 
   LoadThrusterInputs();
-  Thruster->Calculate(HP * hptoftlbssec);
+  // Filters out negative powers when the propeller is not rotating.
+  double power = HP * hptoftlbssec;
+  if (RPM <= 0.1) power = max(power, 0.0);
+  Thruster->Calculate(power);
 
   RunPostFunctions();
 }
@@ -529,7 +531,7 @@ void FGPiston::Calculate(void)
 double FGPiston::CalcFuelNeed(void)
 {
   FuelExpended = FuelFlowRate * in.TotalDeltaT;
-  if (!Starved) FuelUsedLbs += FuelExpended; 
+  if (!Starved) FuelUsedLbs += FuelExpended;
   return FuelExpended;
 }
 
@@ -590,7 +592,7 @@ void FGPiston::doEngineStartup(void)
   if ( Running ) {
     if (!spark || !fuel)    Running = false;
     if (RPM < IdleRPM*0.8 ) Running = false;
-  } else { // !Running  
+  } else { // !Running
     if ( spark && fuel) {     // start the engine if revs high enough
       if (RPM > IdleRPM*0.8)  // This allows us to in-air start
         Running = true;       // when windmilling
@@ -701,7 +703,7 @@ void FGPiston::doMAP(void)
   } else {
       BoostLossHP = 0;
   }
-  
+
   // And set the value in American units as well
   ManifoldPressure_inHg = MAP / inhgtopa;
 }
@@ -752,8 +754,6 @@ void FGPiston::doFuelFlow(void)
 {
   double thi_sea_level = 1.3 * in.MixturePos[EngineNumber]; // Allows an AFR of infinity:1 to 11.3075:1
   equivalence_ratio = thi_sea_level * 101325.0 / p_amb;
-//  double AFR = 10+(12*(1-in.Mixture[EngineNumber]));// mixture 10:1 to 22:1
-//  m_dot_fuel = m_dot_air / AFR;
   m_dot_fuel = (m_dot_air * equivalence_ratio) / 14.7;
   FuelFlowRate =  m_dot_fuel * 2.2046;  // kg to lb
   if(Starved) // There is no fuel, so zero out the flows we've calculated so far
@@ -798,14 +798,14 @@ void FGPiston::doEnginePower(void)
   } else {
     // Power output when the engine is not running
     double torque, k_torque, rpm;  // Convienience term for use in the calculations
-    
+
     rpm = RPM < 1.0 ? 1.0 : RPM;
     if (Cranking) {
       if(RPM<StarterRPM) k_torque = 1.0-RPM/(StarterRPM);
       else k_torque = 0;
       torque = StarterTorque*k_torque*StarterGain;
       IndicatedHorsePower = torque * rpm / 5252;
-     } 
+     }
   }
 
   // Constant is (1/2) * 60 * 745.7
@@ -1072,8 +1072,6 @@ void FGPiston::Debug(int from)
   }
   if (debug_lvl & 64) {
     if (from == 0) { // Constructor
-      cout << IdSrc << endl;
-      cout << IdHdr << endl;
     }
   }
 }
