@@ -41,23 +41,16 @@ INCLUDES
 #include "FGFDMExec.h"
 #include "FGPropeller.h"
 #include "input_output/FGXMLElement.h"
+#include "models/FGMassBalance.h"
 
 using namespace std;
 
 namespace JSBSim {
 
-IDENT(IdSrc,"$Id: FGPropeller.cpp,v 1.63 2017/06/03 19:49:20 bcoconni Exp $");
-IDENT(IdHdr,ID_PROPELLER);
-
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS IMPLEMENTATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-// This class currently makes certain assumptions when calculating torque and
-// p-factor. That is, that the axis of rotation is the X axis of the aircraft -
-// not just the X-axis of the engine/propeller. This may or may not work for a
-// helicopter.
-
 FGPropeller::FGPropeller(FGFDMExec* exec, Element* prop_element, int num)
                        : FGThruster(exec, prop_element, num)
 {
@@ -78,7 +71,7 @@ FGPropeller::FGPropeller(FGFDMExec* exec, Element* prop_element, int num)
   Vinduced = 0.0;
 
   if (prop_element->FindElement("ixx"))
-    Ixx = max(prop_element->FindElementValueAsNumberConvertTo("ixx", "SLUG*FT2"), 0.001);
+    Ixx = max(prop_element->FindElementValueAsNumberConvertTo("ixx", "SLUG*FT2"), 1e-06);
 
   Sense_multiplier = 1.0;
   if (prop_element->HasAttribute("version")
@@ -168,9 +161,15 @@ FGPropeller::FGPropeller(FGFDMExec* exec, Element* prop_element, int num)
   property_name = base_property_name + "/constant-speed-mode";
   PropertyManager->Tie( property_name.c_str(), this, &FGPropeller::GetConstantSpeed,
                       &FGPropeller::SetConstantSpeed );
-  property_name = base_property_name + "/prop-induced-velocity_fps";
+  property_name = base_property_name + "/prop-induced-velocity_fps"; // [ft/sec]
   PropertyManager->Tie( property_name.c_str(), this, &FGPropeller::GetInducedVelocity,
                       &FGPropeller::SetInducedVelocity );
+  property_name = base_property_name + "/propeller-power-ftlbps"; // [ft-lbs/sec]
+  PropertyManager->Tie( property_name.c_str(), &PowerRequired );
+  property_name = base_property_name + "/propeller-torque-ftlb"; // [ft-lbs]
+  PropertyManager->Tie( property_name.c_str(), this, &FGPropeller::GetTorque);
+  property_name = base_property_name + "/propeller-sense";
+  PropertyManager->Tie( property_name.c_str(), &Sense );
 
   Debug(0);
 }
@@ -208,7 +207,13 @@ void FGPropeller::ResetToIC(void)
 
 double FGPropeller::Calculate(double EnginePower)
 {
-  FGColumnVector3 localAeroVel = Transform().Transposed() * in.AeroUVW;
+  FGColumnVector3 vDXYZ = MassBalance->StructuralToBody(vXYZn);
+  const FGMatrix33& mT = Transform();
+  // Local air velocity is obtained from Stevens & Lewis' "Aircraft Control and
+  // Simualtion (3rd edition)" eqn 8.2-1
+  // Variables in.AeroUVW and in.AeroPQR include the wind and turbulence effects
+  // as computed by FGAuxiliary.
+  FGColumnVector3 localAeroVel = mT.Transposed() * (in.AeroUVW + in.AeroPQR*vDXYZ);
   double omega, PowerAvailable;
 
   double Vel = localAeroVel(eU);
@@ -220,7 +225,7 @@ double FGPropeller::Calculate(double EnginePower)
   double Vtip = RPS * Diameter * M_PI;
   HelicalTipMach = sqrt(Vtip*Vtip + Vel*Vel) / in.Soundspeed;
 
-  if (RPS > 0.0) J = Vel / (Diameter * RPS); // Calculate J normally
+  if (RPS > 0.01) J = Vel / (Diameter * RPS); // Calculate J normally
   else           J = Vel / Diameter;
 
   PowerAvailable = EnginePower - GetPowerRequired();
@@ -258,7 +263,10 @@ double FGPropeller::Calculate(double EnginePower)
     double tangentialVel = localAeroVel.Magnitude(eV, eW);
 
     if (tangentialVel > 0.0001) {
-      double angle = atan2(tangentialVel, localAeroVel(eU));
+      // The angle made locally by the air flow with respect to the propeller
+      // axis is influenced by the induced velocity. This attenuates the
+      // influence of a string cross wind and gives a more realistic behavior.
+      double angle = atan2(tangentialVel, Vel+Vinduced);
       double factor = Sense * P_Factor * angle / tangentialVel;
       SetActingLocationY( GetLocationY() + factor * localAeroVel(eW));
       SetActingLocationZ( GetLocationZ() + factor * localAeroVel(eV));
@@ -268,6 +276,7 @@ double FGPropeller::Calculate(double EnginePower)
   omega = RPS*2.0*M_PI;
 
   vFn(eX) = Thrust;
+  vTorque(eX) = -Sense*EnginePower / max(0.01, omega);
 
   // The Ixx value and rotation speed given below are for rotation about the
   // natural axis of the engine. The transform takes place in the base class
@@ -275,7 +284,7 @@ double FGPropeller::Calculate(double EnginePower)
 
   FGColumnVector3 vH(Ixx*omega*Sense*Sense_multiplier, 0.0, 0.0);
 
-  if (omega > 0.0) ExcessTorque = PowerAvailable / omega;
+  if (omega > 0.01) ExcessTorque = PowerAvailable / omega;
   else             ExcessTorque = PowerAvailable / 1.0;
 
   RPM = (RPS + ((ExcessTorque / Ixx) / (2.0 * M_PI)) * in.TotalDeltaT) * 60.0;
@@ -284,7 +293,7 @@ double FGPropeller::Calculate(double EnginePower)
 
   // Transform Torque and momentum first, as PQR is used in this
   // equation and cannot be transformed itself.
-  vMn = in.PQRi*(Transform()*vH) + Transform()*vTorque;
+  vMn = in.PQRi*(mT*vH) + mT*vTorque;
 
   return Thrust; // return thrust in pounds
 }
@@ -354,7 +363,6 @@ double FGPropeller::GetPowerRequired(void)
   double local_RPS = RPS < 0.01 ? 0.01 : RPS; 
 
   PowerRequired = cPReq*local_RPS*local_RPS*local_RPS*D5*in.Density;
-  vTorque(eX) = -Sense*PowerRequired / (local_RPS*2.0*M_PI);
 
   return PowerRequired;
 }
@@ -363,12 +371,13 @@ double FGPropeller::GetPowerRequired(void)
 
 FGColumnVector3 FGPropeller::GetPFactor() const
 {
-  double px=0.0, py, pz;
+  // These are moments in lbf per ft : the lever arm along Z generates a moment
+  // along the pitch direction.
+  double p_pitch = Thrust * Sense * (GetActingLocationZ() - GetLocationZ()) / 12.0;
+  // The lever arm along Y generates a moment along the yaw direction.
+  double p_yaw = Thrust * Sense * (GetActingLocationY() - GetLocationY()) / 12.0;
 
-  py = Thrust * Sense * (GetActingLocationY() - GetLocationY()) / 12.0;
-  pz = Thrust * Sense * (GetActingLocationZ() - GetLocationZ()) / 12.0;
-
-  return FGColumnVector3(px, py, pz);
+  return FGColumnVector3(0.0, p_pitch, p_yaw);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -473,8 +482,6 @@ void FGPropeller::Debug(int from)
   }
   if (debug_lvl & 64) {
     if (from == 0) { // Constructor
-      cout << IdSrc << endl;
-      cout << IdHdr << endl;
     }
   }
 }
