@@ -7,6 +7,7 @@
 #include "RigidBody.hpp"
 #include <string.h>
 #include <sstream>
+#include <simgear/constants.h>
 
 
 
@@ -31,8 +32,9 @@ Hitch::Hitch(const char *name)
     _open=true;
     _oldOpen=_open;
     _towLength=60;
+    _autoReleaseAngle=0;
     _towElasticConstant=1e5;
-    _towBrakeForce=100000;
+    _towBreakForce=100000;
     _towWeightPerM=1;
     _winchMaxSpeed=40;
     _winchRelSpeed=0;
@@ -73,9 +75,10 @@ Hitch::Hitch(const char *name)
     {
 #define TIE(x,v) _tiedProperties.Tie(_node->getNode(x, true),v)
         TIE("tow/length", SGRawValuePointer<float>(&_towLength));
+        TIE("automatic-release-angle-deg",SGRawValuePointer<float>(&_autoReleaseAngle));
         TIE("tow/elastic-constant",SGRawValuePointer<float>(&_towElasticConstant));
         TIE("tow/weight-per-m-kg-m",SGRawValuePointer<float>(&_towWeightPerM));
-        TIE("tow/brake-force",SGRawValuePointer<float>(&_towBrakeForce));
+        TIE("tow/break-force",SGRawValuePointer<float>(&_towBreakForce));
         TIE("winch/max-speed-m-s",SGRawValuePointer<float>(&_winchMaxSpeed));
         TIE("winch/rel-speed",SGRawValuePointer<float>(&_winchRelSpeed));
         TIE("winch/initial-tow-length-m",SGRawValuePointer<float>(&_winchInitialTowLength));
@@ -155,7 +158,7 @@ void Hitch::setTowElasticConstant(float sc)
 
 void Hitch::setTowBreakForce(float bf)
 {
-    _towBrakeForce=bf;
+    _towBreakForce=bf;
 }
 
 void Hitch::setWinchMaxForce(float f)
@@ -278,15 +281,14 @@ void Hitch::findBestAIObject(bool doit,bool running_as_autoconnect)
     _towEndIsConnectedToProperty=false;
     SGPropertyNode * ainode = fgGetNode("/ai/models",false);
     if(!ainode) return;
-    char myCallsign[256]="***********";
+    std::string myCallsign = "***********";
     if (running_as_autoconnect)
     {
         //get own callsign
         SGPropertyNode *cs=fgGetNode("/sim/multiplay/callsign",false);
         if (cs)
         {
-            strncpy(myCallsign,cs->getStringValue(),256);
-            myCallsign[255]=0;
+            myCallsign = cs->getStringValue();
         }
         //reset tow length for search radius. Lentgh will be later copied from master 
         _towLength=_winchInitialTowLength;
@@ -296,8 +298,8 @@ void Hitch::findBestAIObject(bool doit,bool running_as_autoconnect)
     for (int i=0;i<ainode->nChildren();i++)
     {
         SGPropertyNode * n=ainode->getChild(i);
-        _nodeIsMultiplayer = strncmp("multiplayer",n->getName(),11)==0;
-        _nodeIsAiAircraft = strncmp("aircraft",n->getName(),8)==0;
+        _nodeIsMultiplayer = strncmp("multiplayer", n->getNameString().c_str(), 11) == 0;
+        _nodeIsAiAircraft = strncmp("aircraft", n->getNameString().c_str(), 8) == 0;
         if (!(_nodeIsMultiplayer || _nodeIsAiAircraft))
             continue;
         if (running_as_autoconnect)
@@ -305,7 +307,7 @@ void Hitch::findBestAIObject(bool doit,bool running_as_autoconnect)
             if (!_nodeIsMultiplayer)
                 continue;
             if(n->getBoolValue("sim/hitches/aerotow/open",true)) continue;
-            if(strncmp(myCallsign,n->getStringValue("sim/hitches/aerotow/tow/connected-to-ai-or-mp-callsign"),255)!=0)
+            if (myCallsign != n->getStringValue("sim/hitches/aerotow/tow/connected-to-ai-or-mp-callsign"))
                 continue;
         }
         double pos[3];
@@ -410,7 +412,7 @@ void Hitch::calcForce(Ground *g_cb, RigidBody* body, State* s)
                                   //With this trick, both player in aerotow get the same length
     Math::unit3(delta,deltaN);
     float lvel[3];
-    s->velGlobalToLocal(s->v,lvel);
+    s->globalToLocal(s->v,lvel);
     _speed_in_tow_direction=Math::dot3(lvel,deltaN);
     if (_towEndIsConnectedToProperty && _nodeIsMultiplayer)
     {
@@ -418,7 +420,7 @@ void Hitch::calcForce(Ground *g_cb, RigidBody* body, State* s)
         _timeLagCorrectedDist=_dist+mp_delta_dist_due_to_time_lag;
         if(_forceIsCalculatedByMaster && !_open)
         {
-            s->velGlobalToLocal(_mp_force,_force);
+            s->globalToLocal(_mp_force,_force);
             return;
         }
     }
@@ -433,10 +435,10 @@ void Hitch::calcForce(Ground *g_cb, RigidBody* body, State* s)
         if(_towLength>1e-3)
             _forceMagnitude=(_dist-_towLength)/_towLength*_towElasticConstant;
         else
-            _forceMagnitude=2*_towBrakeForce;
+            _forceMagnitude=2*_towBreakForce;
     else
         _forceMagnitude=0;
-    if(_forceMagnitude>=_towBrakeForce)
+    if(_forceMagnitude>=_towBreakForce)
     {
         _forceMagnitude=0;
         _open=true;
@@ -525,9 +527,9 @@ void Hitch::calcForce(Ground *g_cb, RigidBody* body, State* s)
     //the same for the tow end:
     Math::mul3(grav_frac_tow_end*grav_force,ground,grav_force_v);
     Math::add3(grav_force_v,_towEndForce,_towEndForce);
-    s->velLocalToGlobal(_towEndForce,_towEndForce);
+    s->localToGlobal(_towEndForce,_towEndForce);
 
-    if(_forceMagnitude>=_towBrakeForce)
+    if(_forceMagnitude>=_towBreakForce)
     {
         _forceMagnitude=0;
         _open=true;
@@ -546,6 +548,11 @@ void Hitch::getForce(float* force, float* off)
     Math::set3(_pos, off);
 }
 
+float Hitch::getRopeAngleDeg()
+{
+    return -atan2(_force[2] , _force[0] ) * SG_RADIANS_TO_DEGREES;
+}
+
 void Hitch::integrate (float dt)
 {
     //check if hitch has opened or closed, if yes: message
@@ -557,7 +564,7 @@ void Hitch::integrate (float dt)
             {
                 std::stringstream message;
                 message<<"Could not lock hitch (tow length is insufficient) on hitch "
-                       <<_node->getName()<<" "<<_node->getIndex()<<"!";
+                       <<_node->getNameString()<<" "<<_node->getIndex()<<"!";
                 fgSetString("/sim/messages/pilot", message.str().c_str());
                 _open=true;
                 return;
@@ -568,7 +575,7 @@ void Hitch::integrate (float dt)
         if (_node->getBoolValue("broken",false)&&_open)
             message<<"Oh no, the tow is broken";
         else
-            message<<(_open?"Opened hitch ":"Locked hitch ")<<_node->getName()<<" "<<_node->getIndex()<<"!";
+            message<<(_open?"Opened hitch ":"Locked hitch ")<<_node->getNameString()<<" "<<_node->getIndex()<<"!";
         fgSetString("/sim/messages/pilot", message.str().c_str());
         _oldOpen=_open;
     }
@@ -591,23 +598,21 @@ void Hitch::integrate (float dt)
         if (_node)
         {
             //_towEndNode=fgGetNode(_node->getStringValue("tow/node"), false);
-            char towNode[256];
-            strncpy(towNode,_node->getStringValue("tow/node"),256);
-            towNode[255]=0;
+            std::string towNode = _node->getStringValue("tow/node");
             _towEndNode=fgGetNode("ai/models")->getNode(towNode, false);
             //AI and multiplayer objects seem to change node?
             //Check if we have the right one by callsign
             if (_nodeIsMultiplayer || _nodeIsAiAircraft)
             {
-                char MPcallsign[256]="";
-                const char *MPc;
-                MPc=_node->getStringValue("tow/connected-to-ai-or-mp-callsign");
-                if (MPc)
+                std::string MPcallsign = "";
+                std::string MPc =_node->getStringValue("tow/connected-to-ai-or-mp-callsign");
+                if (!MPc.empty())
                 {
-                    strncpy(MPcallsign,MPc,256);
-                    MPcallsign[255]=0;
+                    MPcallsign = MPc;
                 }
-                if (((_towEndNode)&&(strncmp(_towEndNode->getStringValue("callsign"),MPcallsign,255)!=0))||!_towEndNode)
+                if ((_towEndNode
+                    && _towEndNode->getStringValue("callsign") != MPcallsign)
+                    || !_towEndNode)
                 {
                     _timeToNextReConnectTry-=dt;
                     if((_timeToNextReConnectTry<0)||(_timeToNextReConnectTry>10))
@@ -619,13 +624,17 @@ void Hitch::integrate (float dt)
                             for (int i=0;i<ainode->nChildren();i++)
                             {
                                 SGPropertyNode * n=ainode->getChild(i);
-                                if(_nodeIsMultiplayer?strncmp("multiplayer",n->getName(),11)==0:strncmp("aircraft",n->getName(),8))
-                                    if (strcmp(n->getStringValue("callsign"),MPcallsign)==0)//found
+                                if (_nodeIsMultiplayer
+                                        ? strncmp("multiplayer", n->getNameString().c_str(), 11) == 0
+                                        : strncmp("aircraft", n->getNameString().c_str(), 8)
+                                ) {
+                                    if (n->getStringValue("callsign") == MPcallsign)//found
                                     {
                                         _towEndNode=n;
                                         //_node->setStringValue("tow/node",n->getPath());
                                         _node->setStringValue("tow/node",n->getDisplayName());
                                     }
+                                }
                             }
                         }
                     }
@@ -652,7 +661,7 @@ void Hitch::integrate (float dt)
                     gf(_towLength,"sim/hitches/aerotow/tow/length");
                     gf(_towElasticConstant,"sim/hitches/aerotow/tow/elastic-constant");
                     gf(_towWeightPerM,"sim/hitches/aerotow/tow/weight-per-m-kg-m");
-                    gf(_towBrakeForce,"sim/hitches/aerotow/brake-force");
+                    gf(_towBreakForce,"sim/hitches/aerotow/break-force");
                     gb(_open,"sim/hitches/aerotow/open");
                     gb(_mp_is_slave,"sim/hitches/aerotow/is-slave");
 #undef gf
@@ -709,6 +718,17 @@ void Hitch::integrate (float dt)
 
     if (_open) return;
     if (_winchRelSpeed==0) return;
+
+    float forceMagnitude = Math::mag3(_towEndForce);
+    if (forceMagnitude > 100 && _autoReleaseAngle && (this->getRopeAngleDeg() > _autoReleaseAngle))
+    {
+        std::stringstream message;
+        message<<"Cable autoreleased";
+        fgSetString("/sim/messages/pilot", message.str().c_str());
+        _open=true;
+        return;
+    }
+
     float factor=1,offset=0;
     if (_winchActualForce>_winchMaxForce)
         offset=-(_winchActualForce-_winchMaxForce)/_winchMaxForce*20;
