@@ -54,6 +54,13 @@ YASim::~YASim()
     _gearProps.clear();
 }
 
+void YASim::property_associations(
+        std::function<void(const std::string& from, const std::string& to)> fn
+        )
+{
+    return _fdm->property_associations(fn);
+}
+
 void YASim::report()
 {
     Airplane* a = _fdm->getAirplane();
@@ -73,7 +80,7 @@ void YASim::report()
     float cg[3];
     char buf[256];
     a->getModel()->getBody()->getCG(cg);
-    sprintf(buf, "            CG: %.3f, %.3f, %.3f", cg[0], cg[1], cg[2]);
+    snprintf(buf, 256, "            CG: %.3f, %.3f, %.3f", cg[0], cg[1], cg[2]);
     SG_LOG(SG_FLIGHT, SG_INFO, buf);
 
     if(a->getFailureMsg()) {
@@ -94,11 +101,16 @@ void YASim::bind()
 
     char buf[256];
     for(int i=0; i<_fdm->getAirplane()->getModel()->numThrusters(); i++) {
-        sprintf(buf, "/engines/engine[%d]/fuel-flow-gph", i);        fgUntie(buf);
-        sprintf(buf, "/engines/engine[%d]/rpm", i);                  fgUntie(buf);
-        sprintf(buf, "/engines/engine[%d]/mp-osi", i);               fgUntie(buf);
-        sprintf(buf, "/engines/engine[%d]/egt-degf", i);             fgUntie(buf);
-        sprintf(buf, "/engines/engine[%d]/oil-temperature-degf", i); fgUntie(buf);
+        snprintf(buf, 256, "/engines/engine[%d]/fuel-flow-gph", i);
+        fgUntieIfDefined(buf);
+        snprintf(buf, 256, "/engines/engine[%d]/rpm", i);
+        fgUntieIfDefined(buf);
+        snprintf(buf, 256, "/engines/engine[%d]/mp-osi", i);
+        fgUntieIfDefined(buf);
+        snprintf(buf, 256, "/engines/engine[%d]/egt-degf", i);
+        fgUntieIfDefined(buf);
+        snprintf(buf, 256, "/engines/engine[%d]/oil-temperature-degf", i);
+        fgUntieIfDefined(buf);
     }
 }
 
@@ -191,34 +203,47 @@ void YASim::init()
         node->setDoubleValue("yoffset-in", pos[1] * M2FT * 12);
         node->setDoubleValue("zoffset-in", pos[2] * M2FT * 12);
 
+        node->setDoubleValue("xoffset-m", pos[0]);
+        node->setDoubleValue("yoffset-m", pos[1]);
+        node->setDoubleValue("zoffset-m", pos[2]);
+
+        node->setDoubleValue("wheel-radius-m", g->getWheelRadius());
+        node->setDoubleValue("tyre-radius-m", g->getTyreRadius());
+
+        node->setDoubleValue("spring", g->getSpring());
+        node->setDoubleValue("damping", g->getDamping());
+        
         _gearProps.push_back(GearProps(node));
     }
 
     // Are we at ground level?  If so, lift the plane up so the gear
     // clear the ground.
+    bool respect_external_gear_state = fgGetBool("/fdm/yasim/respect-external-gear-state");
     double runway_altitude = get_Runway_altitude();
     if(get_Altitude() - runway_altitude < 50) {
+        bool gear_state = fgGetBool("/controls/gear/gear-down");
         fgSetBool("/controls/gear/gear-down", false);
         float minGearZ = 1e18;
         for(int i=0; i<airplane->numGear(); i++) {
             Gear* g = airplane->getGear(i);
             float pos[3];
             g->getPosition(pos);
+            pos[2] -= (g->getWheelRadius() + g->getTyreRadius());
             if(pos[2] < minGearZ)
                 minGearZ = pos[2];
         }
         _set_Altitude(runway_altitude - minGearZ*M2FT);
         // ground start-up: gear down
-        fgSetBool("/controls/gear/gear-down", true);
+        fgSetBool("/controls/gear/gear-down", respect_external_gear_state ? gear_state : true);
     }
-    else
+    else if (! respect_external_gear_state)
     {
         // airborne start-up: gear up
         fgSetBool("/controls/gear/gear-down", false);
     }
 
     // Blank the state, and copy in ours
-    State s;
+    yasim::State s;
     model->setState(&s);
     copyToYASim(true);
 
@@ -300,10 +325,10 @@ void YASim::copyToYASim(bool copyState)
     atmo.setDensity(dens);
     atmo.setTemperature(temp);
     atmo.setPressure(pressure);
-                            
+
     // Convert and set:
     Model* model = _fdm->getAirplane()->getModel();
-    State s;
+    yasim::State s;
     float xyz2ned[9];
     Glue::xyz2nedMat(lat, lon, xyz2ned);
 
@@ -335,14 +360,14 @@ void YASim::copyToYASim(bool copyState)
         Math::tmul33(s.orient, v, v);
         break;
     case KNOTS:
-        v[0] = atmo.spdFromVCAS(get_V_calibrated_kts()/MPS2KTS);
+        v[0] = atmo.speedFromVCAS(get_V_calibrated_kts()/MPS2KTS);
         v[1] = 0;
         v[2] = 0;
         Math::tmul33(s.orient, v, v);
         needCopy = true;
         break;
     case MACH:
-        v[0] = atmo.spdFromMach(get_Mach_number());
+        v[0] = atmo.speedFromMach(get_Mach_number());
         v[1] = 0;
         v[2] = 0;
         Math::tmul33(s.orient, v, v);
@@ -380,9 +405,9 @@ void YASim::copyToYASim(bool copyState)
 // These are set below:
 // _set_Accels_Local
 // _set_Accels_Body
-// _set_Accels_CG_Body 
+// _set_Accels_CG_Body
 // _set_Accels_Pilot_Body
-// _set_Accels_CG_Body_N 
+// _set_Accels_CG_Body_N
 // _set_Velocities_Local
 // _set_Velocities_Ground
 // _set_Velocities_Body
@@ -424,7 +449,7 @@ void YASim::copyFromYASim()
 {
     Airplane* airplane = _fdm->getAirplane();
     Model* model = airplane->getModel();
-    State* s = model->getState();
+    yasim::State* s = model->getState();
 
     // position
     double lat, lon, alt;
@@ -574,3 +599,9 @@ void YASim::reinit()
     // get current FDM values from the property tree
     copyToYASim(true);
 }
+
+
+// Register the subsystem.
+#if 0
+SGSubsystemMgr::Registrant<YASim> registrantYASim;
+#endif
