@@ -34,6 +34,8 @@
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
 #include <Scenery/scenery.hxx>
+#include "AIModel/AIManager.hxx"
+#include "AIModel/AIAircraft.hxx"
 
 // all the FDMs, since we are the factory method
 #ifdef ENABLE_SP_FDM
@@ -61,11 +63,12 @@
 #include <FDM/YASim/YASim.hxx>
 #endif
 
+#include <GUI/MessageBox.hxx>
+#include <Main/sentryIntegration.hxx>
+
 using std::string;
 
-FDMShell::FDMShell() :
-  _tankProperties( fgGetNode("/consumables/fuel", true) ),
-  _dataLogging(false)
+FDMShell::FDMShell() : _tankProperties(fgGetNode("/consumables/fuel", true))
 {
 }
 
@@ -88,6 +91,14 @@ void FDMShell::init()
   _data_logging     = _props->getNode("/sim/temp/fdm-data-logging",         true);
   _replay_master    = _props->getNode("/sim/freeze/replay-state",           true);
 
+  // AI aerodynamic wake interaction
+  _max_radius_nm    = _props->getNode("fdm/ai-wake/max-radius-nm",          true);
+  _ai_wake_enabled  = _props->getNode("fdm/ai-wake/enabled",                true);
+
+  _nanCheckFailed = false;
+  fgSetBool("/sim/fdm-nan-failure", false);
+  _lastValidPos = SGGeod::invalid();
+
   createImplementation();
 }
 
@@ -100,6 +111,11 @@ void FDMShell::postinit()
     {
         SG_LOG(SG_FLIGHT, SG_ALERT, "Failed to save initial FDM property state");
     }
+
+    _ai_mgr = globals->get_subsystem<FGAIManager>();
+    if (!_ai_mgr) {
+        SG_LOG(SG_FLIGHT, SG_DEV_ALERT, "FDM connection to the AI manager: FAILED");
+    }
 }
 
 void FDMShell::shutdown()
@@ -154,12 +170,39 @@ void FDMShell::unbind()
   _tankProperties.unbind();
 }
 
+void FDMShell::doInitAndBind()
+{
+    SG_LOG(SG_FLIGHT, SG_INFO, "Scenery loaded, will init FDM");
+    try {
+        _impl->init();
+        if (_impl->get_bound()) {
+            _impl->unbind();
+        }
+        _impl->bind();
+
+        fgSetBool("/sim/fdm-initialized", true);
+        fgSetBool("/sim/signals/fdm-initialized", true);
+    } catch (std::exception& e) {
+        flightgear::fatalMessageBoxThenExit("Aircraft FDM initialization error",
+                                            string{"The aircraft flight dynamics model contains errors and cannot be used. ("} + e.what() + ")");
+    }
+
+    if (!copyProperties(_props->getNode("fdm", true),
+                        _initialFdmProperties)) {
+        SG_LOG(SG_FLIGHT, SG_ALERT, "Failed to save initial FDM property state");
+    }
+}
+
 void FDMShell::update(double dt)
 {
   if (!_impl) {
     return;
   }
-  
+
+  if (_nanCheckFailed) {
+      return;
+  }
+
   if (!_impl->get_inited()) {
     // Check for scenery around the aircraft.
     double lon = fgGetDouble("/sim/presets/longitude-deg");
@@ -167,16 +210,9 @@ void FDMShell::update(double dt)
         
     double range = 1000.0; // in meters
     SGGeod geod = SGGeod::fromDeg(lon, lat);
-    if (globals->get_scenery()->scenery_available(geod, range)) {
-        SG_LOG(SG_FLIGHT, SG_INFO, "Scenery loaded, will init FDM");
-        _impl->init();
-        if (_impl->get_bound()) {
-          _impl->unbind();
-        }
-        _impl->bind();
-        
-        fgSetBool("/sim/fdm-initialized", true);
-        fgSetBool("/sim/signals/fdm-initialized", true);
+    const auto startUpPositionFialized = fgGetBool("/sim/position-finalized", false);
+    if (startUpPositionFialized && globals->get_scenery()->scenery_available(geod, range)) {
+        doInitAndBind();
     }
   }
 
@@ -184,7 +220,30 @@ void FDMShell::update(double dt)
     return; // still waiting
   }
 
-// pull environmental data in, since the FDMs are lazy
+  // AI aerodynamic wake interaction
+  if (_ai_wake_enabled->getBoolValue()) {
+      for (FGAIBase* base : _ai_mgr->get_ai_list()) {
+          try {
+              if (base->isa(FGAIBase::object_type::otAircraft) ) {
+                  SGVec3d pos = _impl->getCartPosition();
+                  const SGSharedPtr<FGAIAircraft> aircraft = dynamic_cast<FGAIAircraft*>(base);
+                  double range = _ai_mgr->calcRangeFt(pos, aircraft)*SG_FEET_TO_METER;
+
+                  if (!aircraft->onGround() && aircraft->getSpeed() > 0.0
+                      && range < _max_radius_nm->getDoubleValue()*SG_NM_TO_METER) {
+                      _impl->add_ai_wake(aircraft);
+                  }
+              }
+          } catch (sg_exception& e) {
+              SG_LOG(SG_FLIGHT, SG_WARN, "caught exception updating AI model:"
+                     << base->_getName()<< ", which will be killed."
+                     "\n\tError:" << e.getFormattedMessage());
+              base->setDie(true);
+          }
+      }
+  }
+
+  // pull environmental data in, since the FDMs are lazy
   _impl->set_Velocities_Local_Airmass(
           _wind_north->getDoubleValue(),
           _wind_east->getDoubleValue(),
@@ -222,6 +281,8 @@ void FDMShell::update(double dt)
           // replay is active
           break;
   }
+
+  validateOutputProperties();
 }
 
 FGInterface* FDMShell::getInterface() const
@@ -320,8 +381,8 @@ void FDMShell::createImplementation()
         _impl = new FGBalloonSim( dt );
     } else if ( model == "magic" ) {
         _impl = new FGMagicCarpet( dt );
-//  } else if ( model == "aisim" ) {
-//      _impl = new FGAISim( dt );
+    } else if ( model == "aisim" ) {
+        _impl = new FGAISim( dt );
     }
 #else
     else if (( model == "ada" )||(model == "acms")||( model == "balloon" )||( model == "magic" )||( model == "aisim" ))
@@ -348,3 +409,38 @@ void FDMShell::createImplementation()
                    "If you still need it, please rebuild FlightGear and enable its support."));
     }
 }
+
+void FDMShell::validateOutputProperties()
+{
+    if (_nanCheckFailed)
+        return;
+
+    const auto p = globals->get_aircraft_position();
+    if (SGMisc<double>::isNaN(p.getLatitudeDeg()) ||
+        SGMisc<double>::isNaN(p.getLongitudeDeg()) ||
+        SGMisc<double>::isNaN(p.getElevationFt())) {
+        SG_LOG(SG_FLIGHT, SG_ALERT, "FDM position became invalid. Last valid position was:" << _lastValidPos);
+        _nanCheckFailed = true;
+    } else {
+        _lastValidPos = p;
+    }
+
+    if (SGMisc<double>::isNaN(_impl->get_V_true_kts())) {
+        SG_LOG(SG_FLIGHT, SG_ALERT, "FDM velocity became invalid");
+        _nanCheckFailed = true;
+    }
+
+    if (_nanCheckFailed) {
+        flightgear::sentryReportException("FDM NaN check failed");
+
+        flightgear::modalMessageBox("Flight dynamics error",
+                                    "The flight dynamics model (FDM) has become invalid. The simulation will be stopped, so you can restart at a new location.");
+        fgSetBool("/sim/fdm-nan-failure", true);
+        fgSetBool("/sim/freeze/master", true);
+        fgSetBool("/sim/freeze/clock", true);
+    }
+}
+
+// Register the subsystem.
+SGSubsystemMgr::Registrant<FDMShell> registrantFDMShell(
+    SGSubsystemMgr::FDM);
