@@ -233,6 +233,13 @@ FGInterface::common_init ()
     SG_LOG( SG_FLIGHT, SG_INFO, "End common FDM init" );
 }
 
+void FGInterface::property_associations(
+        std::function<void(const std::string& from, const std::string& to)> fn
+        )
+{
+    /* Do nothing by default. */
+}
+
 
 /**
  * Bind getters and setters to properties.
@@ -293,6 +300,10 @@ FGInterface::bind ()
                       &FGInterface::get_Psi_deg,
                       &FGInterface::set_Psi_deg, false);
   fgSetArchivable("/orientation/heading-deg");
+  _tiedProperties.Tie("/orientation/true-heading-deg", this,
+                      &FGInterface::get_Psi_deg,
+                      &FGInterface::set_Psi_deg, false);
+  fgSetArchivable("/orientation/true-heading-deg");
   _tiedProperties.Tie("/orientation/track-deg", this,
                       &FGInterface::get_Track); // read-only
   _tiedProperties.Tie("/orientation/path-deg", this,
@@ -718,14 +729,16 @@ FGInterface::get_body_m(double t, simgear::BVHNode::Id id,
 {
   SGMatrixd _bodyToWorld;
   SGVec3d _linearVel, _angularVel;
-  if (!ground_cache.get_body(t, _bodyToWorld, _linearVel, _angularVel, id))
-    return false;
+  bool _foundID = ground_cache.get_body(t, _bodyToWorld, _linearVel, _angularVel, id);
 
   assign(linearVel, _linearVel);
   assign(angularVel, _angularVel);
-  for (unsigned i = 0; i < 16; ++i)
+  for (unsigned int i = 0; i < 16; ++i)
       bodyToWorld[i] = _bodyToWorld.data()[i];
 
+  if (!_foundID)
+    return false;
+
   return true;
 }
 
