@@ -36,6 +36,8 @@
 #include <osg/MatrixTransform>
 #include <osg/PositionAttitudeTransform>
 #include <osg/CameraView>
+#include <osgTerrain/TerrainTile>
+#include <osgTerrain/Terrain>
 
 #include <simgear/sg_inlines.h>
 #include <simgear/constants.h>
@@ -110,7 +112,6 @@ public:
         
         mSubTreeCollector.popNodeList(parentNodeList);
     }
-    
     virtual void apply(osg::Transform& transform)
     { handleTransform(transform); }
     virtual void apply(osg::Camera& camera)
@@ -319,6 +320,11 @@ bool
 FGGroundCache::prepare_ground_cache(double startSimTime, double endSimTime,
                                     const SGVec3d& pt, double rad)
 {
+    if (rad > 10000.0) {
+        SG_LOG(SG_FLIGHT, SG_DEV_WARN, "FGGroundCache::prepare_ground_cache passed an excessive radius");
+        rad = 10000.0;
+    }
+    
 #ifdef GROUNDCACHE_DEBUG
     SGTimeStamp t0 = SGTimeStamp::now();
 #endif
@@ -365,7 +371,7 @@ FGGroundCache::prepare_ground_cache(double startSimTime, double endSimTime,
         found_ground = true;
     } else if (_localBvhTree) {
         // We have nothing below us, so try starting with the lowest point
-        // upwards for a croase altitude value
+        // upwards for a coarse altitude value
         SGLineSegmentd line(pt + reference_vehicle_radius*down, pt - 1e3*down);
         simgear::BVHLineSegmentVisitor lineSegmentVisitor(line, startSimTime);
         _localBvhTree->accept(lineSegmentVisitor);
@@ -388,10 +394,12 @@ FGGroundCache::prepare_ground_cache(double startSimTime, double endSimTime,
             _altitude = alt;
     }
     
-    // Still not sucessful??
-    if (!found_ground)
-        SG_LOG(SG_FLIGHT, SG_WARN, "prepare_ground_cache(): trying to build "
-               "cache without any scenery below the aircraft");
+    // RJH: 2018-12-31: Remove this message as it happens too frequently when flying over areas of missing terrain
+    //                  and realistically it doesn't really give much information to help identify or resolve a problem
+    //                  which is evident when looking out of the window.
+    //if (!found_ground)
+    //    SG_LOG(SG_FLIGHT, SG_WARN, "prepare_ground_cache(): trying to build "
+    //           "cache without any scenery below the aircraft");
 
 #ifdef GROUNDCACHE_DEBUG
     t0 = SGTimeStamp::now() - t0;
@@ -448,6 +456,7 @@ public:
         _bodyToWorld(SGMatrixd::unit()),
         _linearVelocity(0, 0, 0),
         _angularVelocity(0, 0, 0),
+        _foundId(false),
         _time(t)
     { }
     
@@ -499,6 +508,7 @@ public:
     }
     virtual void apply(BVHLineGeometry& node) { }
     virtual void apply(BVHStaticGeometry& node) { }
+    virtual void apply(BVHTerrainTile& node) { }
     
     virtual void apply(const BVHStaticBinary&, const BVHStaticData&) { }
     virtual void apply(const BVHStaticTriangle&, const BVHStaticData&) { }
@@ -536,13 +546,14 @@ FGGroundCache::get_body(double t, SGMatrixd& bodyToWorld, SGVec3d& linearVel,
     t += cache_time_offset;
     BodyFinder bodyFinder(id, t);
     _localBvhTree->accept(bodyFinder);
-    if (bodyFinder.empty())
-        return false;
 
     bodyToWorld = bodyFinder.getBodyToWorld();
     linearVel = bodyFinder.getLinearVelocity();
     angularVel = bodyFinder.getAngularVelocity();
 
+    if (bodyFinder.empty())
+        return false;
+
     return true;
 }
 
@@ -626,8 +637,8 @@ public:
         _angularVelocity = SGVec3d::zeros();
         _haveLineSegment = true;
     }
-    virtual void apply(BVHStaticGeometry& node)
-    { }
+    virtual void apply(BVHStaticGeometry& node) { }
+    virtual void apply(BVHTerrainTile& node) { }
     
     virtual void apply(const BVHStaticBinary&, const BVHStaticData&) { }
     virtual void apply(const BVHStaticTriangle&, const BVHStaticData&) { }
@@ -693,6 +704,10 @@ FGGroundCache::get_agl(double t, const SGVec3d& pt, SGVec3d& contact,
                        SGVec3d& normal, SGVec3d& linearVel, SGVec3d& angularVel,
                        simgear::BVHNode::Id& id, const simgear::BVHMaterial*& material)
 {
+    if (isNaN(pt)) {
+        throw sg_range_exception("FGGroundCache::get_agl: NaN position input");
+    }
+
 #ifdef GROUNDCACHE_DEBUG
     SGTimeStamp t0 = SGTimeStamp::now();
 #endif
@@ -864,8 +879,8 @@ public:
         _angularVelocity = SGVec3d::zeros();
         _wire = &node;
     }
-    virtual void apply(BVHStaticGeometry& node)
-    { }
+    virtual void apply(BVHStaticGeometry& node) { }
+    virtual void apply(BVHTerrainTile& node) { }
     
     virtual void apply(const BVHStaticBinary&, const BVHStaticData&) { }
     virtual void apply(const BVHStaticTriangle&, const BVHStaticData&) { }
@@ -990,6 +1005,7 @@ public:
         _haveLineSegment = true;
     }
     virtual void apply(BVHStaticGeometry&) { }
+    virtual void apply(BVHTerrainTile&) { }
     
     virtual void apply(const BVHStaticBinary&, const BVHStaticData&) { }
     virtual void apply(const BVHStaticTriangle&, const BVHStaticData&) { }
