@@ -16,9 +16,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #include "CanvasWidget.hxx"
 
@@ -35,7 +33,7 @@ CanvasWidget::CanvasWidget( int x, int y,
                             SGPropertyNode* props,
                             const std::string& module ):
   puObject(x, y, width, height),
-  _canvas_mgr( dynamic_cast<CanvasMgr*>(globals->get_subsystem("Canvas")) ),
+  _canvas_mgr(globals->get_subsystem<CanvasMgr>()),
   _last_x(0),
   _last_y(0),
   // automatically resize viewport of canvas if no size is given
@@ -71,7 +69,7 @@ CanvasWidget::CanvasWidget( int x, int y,
   if( !nasal )
     return;
 
-  FGNasalSys *nas = globals->get_subsystem<FGNasalSys>();
+  auto nas = globals->get_subsystem<FGNasalSys>();
   if( !nas )
     SG_LOG( SG_GENERAL,
             SG_ALERT,
@@ -83,8 +81,14 @@ CanvasWidget::CanvasWidget( int x, int y,
   SGPropertyNode *load = nasal->getNode("load");
   if( load )
   {
-    const char *s = load->getStringValue();
-    nas->handleCommand(module.c_str(), file.c_str(), s, cprops);
+    const std::string s = load->getStringValue();
+    // avoid crash FLIGHTGEAR-5FQ
+    if (s.empty()) {
+        SG_LOG(SG_GUI, SG_ALERT, "Empty 'load' script for Canvas widget:" << cprops->getStringValue("name"));
+        return;
+    }
+
+    nas->handleCommand(module.c_str(), file.c_str(), s.c_str(), cprops);
   }
 }
 
@@ -196,14 +200,22 @@ void CanvasWidget::setSize(int w, int h)
 //------------------------------------------------------------------------------
 void CanvasWidget::draw(int dx, int dy)
 {
+
   glEnable(GL_TEXTURE_2D);
+    // prevent alpha values in the Canvas texture ending up in the PUI
+    // compositing FBO. While that would ideally be a feature, it didn't
+    // use to work, so Canvas users don't do a default opaque fill.
+  glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_FALSE);
+
   glBindTexture(GL_TEXTURE_2D, _canvas_mgr->getCanvasTexId(_canvas));
   glBegin( GL_QUADS );
-    glColor3f(1,1,1);
+    glColor4f(1,1,1, 1.0f);
     glTexCoord2f(0,0); glVertex2f(dx + abox.min[0], dy + abox.min[1]);
     glTexCoord2f(1,0); glVertex2f(dx + abox.max[0], dy + abox.min[1]);
     glTexCoord2f(1,1); glVertex2f(dx + abox.max[0], dy + abox.max[1]);
     glTexCoord2f(0,1); glVertex2f(dx + abox.min[0], dy + abox.max[1]);
   glEnd();
+    
   glDisable(GL_TEXTURE_2D);
+  glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
 }
