@@ -23,8 +23,6 @@
 #include <AppKit/NSSavePanel.h>
 #include <AppKit/NSOpenPanel.h>
 
-#include <boost/foreach.hpp>
-
 #include <osgViewer/Viewer>
 #include <osgViewer/api/Cocoa/GraphicsWindowCocoa>
 
@@ -47,14 +45,14 @@ public:
     CocoaFileDialogPrivate() :
         panel(nil)
     {
-        
+
     }
-    
+
     ~CocoaFileDialogPrivate()
     {
         [panel release];
     }
-    
+
     NSSavePanel* panel;
 };
 
@@ -65,20 +63,20 @@ CocoaFileDialog::CocoaFileDialog(FGFileDialog::Usage use) :
     if (use == USE_SAVE_FILE) {
         d->panel = [NSSavePanel savePanel];
     } else {
-        NSOpenPanel* openPanel = [NSOpenPanel openPanel]; 
+        NSOpenPanel* openPanel = [NSOpenPanel openPanel];
         d->panel = openPanel;
-        
+
         if (use == USE_CHOOSE_DIR) {
             [openPanel setCanChooseDirectories:YES];
         }
     } // of USE_OPEN_FILE or USE_CHOOSE_DIR -> building NSOpenPanel
-    
+
     [d->panel retain];
 }
 
 CocoaFileDialog::~CocoaFileDialog()
 {
-    
+
 }
 
 void CocoaFileDialog::exec()
@@ -87,31 +85,31 @@ void CocoaFileDialog::exec()
 // it window-modal.
     NSWindow* cocoaWindow = nil;
     std::vector<osgViewer::GraphicsWindow*> windows;
-    globals->get_renderer()->getViewer()->getWindows(windows);
-    
-    BOOST_FOREACH(osgViewer::GraphicsWindow* gw, windows) {
+    globals->get_renderer()->getViewerBase()->getWindows(windows);
+
+    for (auto gw : windows) {
         // OSG doesn't use RTTI, so no dynamic cast. Let's check the class type
         // using OSG's own system, before we blindly static_cast<> and break
         // everything.
         if (strcmp(gw->className(), "GraphicsWindowCocoa")) {
-            continue; 
+            continue;
         }
-            
+
         osgViewer::GraphicsWindowCocoa* gwCocoa = static_cast<osgViewer::GraphicsWindowCocoa*>(gw);
         cocoaWindow = (NSWindow*) gwCocoa->getWindow();
         break;
     }
-    
+
 // setup the panel fields now we have collected all the data
     if (_usage == USE_SAVE_FILE) {
         [d->panel setNameFieldStringValue:stdStringToCocoa(_placeholder)];
     }
-    
+
     if (_filterPatterns.empty()) {
         [d->panel setAllowedFileTypes:nil];
     } else {
         NSMutableArray* extensions = [NSMutableArray arrayWithCapacity:0];
-        BOOST_FOREACH(std::string ext, _filterPatterns) {
+        for (const auto& ext : _filterPatterns) {
             if (!simgear::strutils::starts_with(ext, "*.")) {
                 SG_LOG(SG_GENERAL, SG_INFO, "can't use pattern on Cococa:" << ext);
                 continue;
@@ -121,20 +119,20 @@ void CocoaFileDialog::exec()
 
         [d->panel setAllowedFileTypes:extensions];
     }
-    
+
     [d->panel setTitle:stdStringToCocoa(_title)];
     if (_showHidden) {
         [d->panel setShowsHiddenFiles:YES];
     }
-    
+
     [d->panel setDirectoryURL: pathToNSURL(_initialPath)];
-    
+
     [d->panel beginSheetModalForWindow:cocoaWindow completionHandler:^(NSInteger result)
     {
         if (result == NSFileHandlingPanelOKButton) {
             completion_path = [[d->panel URL] path];
             //NSLog(@"the URL is: %@", d->panel URL]);
-            completion_sgpath = ([completion_path UTF8String]);
+            completion_sgpath = SGPath::fromUtf8([completion_path UTF8String]);
             _callback->onFileDialogDone(this, completion_sgpath);
         }
     }];
@@ -144,4 +142,3 @@ void CocoaFileDialog::close()
 {
     [d->panel close];
 }
-
