@@ -106,7 +106,7 @@ WeatherScenariosModel::WeatherScenariosModel(QObject *pr) :
         int nChildren = scenarios->nChildren();
         for (int i = 0; i < nChildren; i++) {
             SGPropertyNode_ptr scenario = scenarios->getChild(i);
-            if (strcmp(scenario->getName(), "scenario") != 0) {
+            if (scenario->getNameString() != "scenario") {
                 continue;
             }
 
@@ -117,7 +117,7 @@ WeatherScenariosModel::WeatherScenariosModel(QObject *pr) :
             }
 
             WeatherScenario ws;
-            const string wsId = scenario->getStringValue("id");
+            const std::string wsId = scenario->getStringValue("id");
             if (!wsId.empty()) {
                 // translated
                 auto locale = globals->get_locale();
@@ -146,7 +146,7 @@ int WeatherScenariosModel::rowCount(const QModelIndex&) const
 QVariant WeatherScenariosModel::data(const QModelIndex &index, int role) const
 {
     const int row = index.row();
-    if ((row < 0) || (row >= m_scenarios.size())) {
+    if ((row < 0) || (static_cast<std::vector<WeatherScenariosModel::WeatherScenario>::size_type>(row) >= m_scenarios.size())) {
            return QVariant();
     }
 
