@@ -13,20 +13,19 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
-#ifdef HAVE_WINDOWS_H
-#include <windows.h>
-#endif
+#include <config.h>
 
 #include "FGFontCache.hxx"
 
-#include <plib/fnt.h>
-#include <plib/pu.h>
+// this is our one in 3rdparty
+#include "fnt.h"
+
+#if defined(HAVE_PUI)
+#include "FlightGear_pu.h"
+#endif
 
 #include <simgear/props/props.hxx>
+#include <simgear/misc/sg_dir.hxx>
 
 #include <Main/globals.hxx>
 
@@ -36,44 +35,33 @@
 
 static std::unique_ptr<FGFontCache> global_fontCacheInstance;
 
-
+#if defined(HAVE_PUI)
 extern puFont FONT_HELVETICA_14;
 extern puFont FONT_SANS_12B;
 extern puFont FONT_HELVETICA_12;
-
+#endif
 namespace
 {
 struct GuiFont
 {
-    const char* name;
+    const std::string name;
     puFont *font;
-    struct Predicate
-        : public std::unary_function<const GuiFont, bool>
-    {
-        Predicate(const std::string& name_) : name(name_) {}
-        bool operator() (const GuiFont& f1) const
-        {
-            return (name == f1.name);
-        }
-        const std::string name;
-    };
 };
 
-const GuiFont guifonts[] = {
-    { "default",      &PUFONT_HELVETICA_12 },
-    { "FIXED_8x13",   &PUFONT_8_BY_13 },
-    { "FIXED_9x15",   &PUFONT_9_BY_15 },
-    { "TIMES_10",     &PUFONT_TIMES_ROMAN_10 },
-    { "TIMES_24",     &PUFONT_TIMES_ROMAN_24 },
-    { "HELVETICA_10", &PUFONT_HELVETICA_10 },
-    { "HELVETICA_12", &FONT_HELVETICA_12 },
-    { "HELVETICA_14", &FONT_HELVETICA_14 },
-    { "HELVETICA_18", &PUFONT_HELVETICA_18 },
-    { "SANS_12B",     &FONT_SANS_12B },
-    { 0 }
+const std::initializer_list<GuiFont> guiFonts = {
+    {"default", &PUFONT_HELVETICA_12},
+    {"FIXED_8x13", &PUFONT_8_BY_13},
+    {"FIXED_9x15", &PUFONT_9_BY_15},
+    {"TIMES_10", &PUFONT_TIMES_ROMAN_10},
+    {"TIMES_24", &PUFONT_TIMES_ROMAN_24},
+    {"HELVETICA_10", &PUFONT_HELVETICA_10},
+    {"HELVETICA_18", &PUFONT_HELVETICA_18},
+#if defined(HAVE_PUI)
+    {"HELVETICA_12", &FONT_HELVETICA_12},
+    {"HELVETICA_14", &FONT_HELVETICA_14},
+    {"SANS_12B", &FONT_SANS_12B},
+#endif
 };
-
-const GuiFont* guifontsEnd = &guifonts[sizeof(guifonts)/ sizeof(guifonts[0])-1];
 }
 
 FGFontCache* FGFontCache::instance()
@@ -97,32 +85,36 @@ FGFontCache::FGFontCache() :
 
 FGFontCache::~FGFontCache()
 {
-   PuFontMap::iterator it, end = _puFonts.end();
-   for (it = _puFonts.begin(); it != end; ++it)
-       delete it->second;
+    for (auto puFontIt : _cache) {
+        delete puFontIt.second;
+    }
+
+    // these were created i initializeFonts
+    for (auto texFontIt : _texFonts) {
+        delete texFontIt.second;
+    }
 }
 
-inline bool FGFontCache::FntParamsLess::operator()(const FntParams& f1,
-                                                   const FntParams& f2) const
+bool FGFontCache::FntParams::operator<(const FntParams& other) const
 {
-    int comp = f1.name.compare(f2.name);
+    int comp = name.compare(other.name);
     if (comp < 0)
         return true;
     else if (comp > 0)
         return false;
-    if (f1.size < f2.size)
+    if (size < other.size)
         return true;
-    else if (f1.size > f2.size)
+    else if (size > other.size)
         return false;
-    return f1.slant < f2.slant;
+    return slant < other.slant;
 }
 
-struct FGFontCache::fnt *
+FGFontCache::FontCacheEntry*
 FGFontCache::getfnt(const std::string& fontName, float size, float slant)
 {
     FntParams fntParams(fontName, size, slant);
-    PuFontMap::iterator i = _puFonts.find(fntParams);
-    if (i != _puFonts.end())
+    PuFontMap::iterator i = _cache.find(fntParams);
+    if (i != _cache.end())
         return i->second;
     // fntTexFont s are all preloaded into the _texFonts map
     TexFontMap::iterator texi = _texFonts.find(fontName);
@@ -131,23 +123,30 @@ FGFontCache::getfnt(const std::string& fontName, float size, float slant)
     if (texi != _texFonts.end()) {
         texfont = texi->second;
     } else {
-        const GuiFont* guifont = std::find_if(&guifonts[0], guifontsEnd,
-                                              GuiFont::Predicate(fontName));
-        if (guifont != guifontsEnd) {
+        auto guifont = std::find_if(guiFonts.begin(), guiFonts.end(),
+                                    [&fontName](const GuiFont& gf) {
+                                        return gf.name == fontName;
+                                    });
+
+        if (guifont != guiFonts.end()) {
             pufont = guifont->font;
         }
     }
-    fnt* f = new fnt;
+
+    auto f = new FontCacheEntry;
     if (pufont) {
         f->pufont = pufont;
     } else if (texfont) {
         f->texfont = texfont;
+        f->ownsPUFont = true; // ensure it gets cleaned up
         f->pufont = new puFont;
         f->pufont->initialize(static_cast<fntFont *>(f->texfont), size, slant);
     } else {
-        f->pufont = guifonts[0].font;
+        f->pufont = guiFonts.begin()->font;
     }
-    _puFonts[fntParams] = f;
+
+    // insert into the cache
+    _cache[fntParams] = f;
     return f;
 }
 
@@ -170,7 +169,7 @@ FGFontCache::get(SGPropertyNode *node)
     if (!node)
         return get("Helvetica.txf", 15.0, 0.0);
 
-    const char *name = node->getStringValue("name", "Helvetica.txf");
+    std::string name = node->getStringValue("name", "Helvetica.txf");
     float size = node->getFloatValue("size", 15.0);
     float slant = node->getFloatValue("slant", 0.0);
 
@@ -210,34 +209,25 @@ FGFontCache::getfntpath(const std::string& name)
 
 bool FGFontCache::initializeFonts()
 {
-    static std::string fontext("txf");
+    static std::string fontext(".txf");
     init();
-    std::string fontPath = _path.local8BitStr();
-    ulDir* fontdir = ulOpenDir(fontPath.c_str());
-    if (!fontdir)
-        return false;
-    const ulDirEnt *dirEntry;
-    while ((dirEntry = ulReadDir(fontdir)) != 0) {
-        SGPath path(_path);
-        path.append(dirEntry->d_name);
-        if (path.extension() == fontext) {
-            fntTexFont* f = new fntTexFont;
-            std::string ps = path.local8BitStr();
-            if (f->load(ps.c_str()))
-                _texFonts[std::string(dirEntry->d_name)] = f;
-            else
-                delete f;
-        }
+
+    auto dir = simgear::Dir(_path);
+    for (auto p : dir.children(simgear::Dir::TYPE_FILE, fontext)) {
+        fntTexFont* f = new fntTexFont;
+        if (f->load(p))
+            _texFonts[p.file()] = f;
+        else
+            delete f;
     }
-    ulCloseDir(fontdir);
+
     return true;
 }
 
-FGFontCache::fnt::~fnt()
+FGFontCache::FontCacheEntry::~FontCacheEntry()
 {
-    if (texfont) { 
-        delete pufont; 
-        delete texfont;
+    if (ownsPUFont) {
+        delete pufont;
     }
 }
 
