@@ -1,3 +1,5 @@
+#include "config.h"
+
 #include "LauncherController.hxx"
 
 // Qt headers
@@ -25,16 +27,18 @@
 #include <Main/options.hxx>
 #include <Main/fg_init.hxx>
 #include <Main/fg_props.hxx>
-#include "version.h"
 #include <Main/sentryIntegration.hxx>
 
-#include "AircraftModel.hxx"
-#include "AircraftSearchFilterModel.hxx"
+#include "AircraftItemModel.hxx"
+#include "AircraftProxyModel.hxx"
 #include "AirportDiagram.hxx"
 #include "CarrierDiagram.hxx"
 #include "CarriersLocationModel.hxx"
 #include "DefaultAircraftLocator.hxx"
 #include "FlightPlanController.hxx"
+#include "GettingStartedScope.hxx"
+#include "GettingStartedTip.hxx"
+#include "GettingStartedTipsController.hxx"
 #include "HoverArea.hxx"
 #include "LaunchConfig.hxx"
 #include "LauncherArgumentTokenizer.hxx"
@@ -50,19 +54,19 @@
 #include "QmlAircraftInfo.hxx"
 #include "QmlPositioned.hxx"
 #include "QmlRadioButtonHelper.hxx"
+#include "QmlStringListModel.hxx"
 #include "QtLauncher.hxx"
 #include "RecentAircraftModel.hxx"
 #include "RecentLocationsModel.hxx"
 #include "RouteDiagram.hxx"
+#include "SettingsWrapper.hxx"
 #include "SetupRootDialog.hxx"
 #include "StackController.hxx"
 #include "ThumbnailImageItem.hxx"
+#include "TipBackgroundBox.hxx"
 #include "UnitsModel.hxx"
 #include "UpdateChecker.hxx"
-#include "GettingStartedTipsController.hxx"
-#include "GettingStartedTip.hxx"
-#include "TipBackgroundBox.hxx"
-#include "GettingStartedScope.hxx"
+
 
 using namespace simgear::pkg;
 
@@ -86,8 +90,6 @@ LauncherController::LauncherController(QObject *parent, QWindow* window) :
     connect(m_location, &LocationController::descriptionChanged,
             this, &LauncherController::summaryChanged);
 
-    initQML();
-
     m_aircraftModel = new AircraftItemModel(this);
     m_installedAircraftModel = new AircraftProxyModel(this, m_aircraftModel);
     m_installedAircraftModel->setInstalledFilterEnabled(true);
@@ -115,7 +117,7 @@ LauncherController::LauncherController(QObject *parent, QWindow* window) :
             &LocalAircraftCache::scanCompleted,
             this, &LauncherController::updateSelectedAircraft);
 
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
     m_aircraftModel->setPackageRoot(globals->packageRoot());
 
     m_aircraftGridMode = settings.value("aircraftGridMode").toBool();
@@ -126,7 +128,7 @@ LauncherController::LauncherController(QObject *parent, QWindow* window) :
        {globals->get_subsystem_mgr()->update(0.0);});
     m_subsystemIdleTimer->start();
 
-    QRect winRect= settings.value("window-geometry").toRect();
+    QRect winRect = settings.value("window-geometry").toRect();
 
     if (winRect.isValid()) {
         m_window->setGeometry(winRect);
@@ -137,7 +139,14 @@ LauncherController::LauncherController(QObject *parent, QWindow* window) :
 
     if (settings.contains("window-state")) {
         const auto ws = static_cast<Qt::WindowState>(settings.value("window-state").toInt());
-        m_window->setWindowState(ws);
+        const auto options = flightgear::Options::sharedInstance();
+        if (ws == Qt::WindowState::WindowFullScreen && options->isBoolOptionDisable("fullscreen")) {
+            // Last time we used fullscreen, but now we explicitly turn off fullscreen
+            m_window->setWindowState(Qt::WindowState::WindowNoState);
+        }
+        else {
+            m_window->setWindowState(ws);
+        }
     }
 
     // count launches; we use this to trigger first-run and periodic notices
@@ -157,7 +166,7 @@ LauncherController::LauncherController(QObject *parent, QWindow* window) :
     QTimer::singleShot(2000, this, &LauncherController::checkForOldDownloadDir);
 }
 
-void LauncherController::initQML()
+void LauncherController::initQML(int& styleTypeId)
 {
     qmlRegisterUncreatableType<LauncherController>("FlightGear.Launcher", 1, 0, "LauncherController", "no");
     qmlRegisterUncreatableType<LocationController>("FlightGear.Launcher", 1, 0, "LocationController", "no");
@@ -200,8 +209,11 @@ void LauncherController::initQML()
     qmlRegisterType<StackController>("FlightGear", 1, 0, "StackController");
 
     qmlRegisterType<ModelDataExtractor>("FlightGear", 1, 0, "ModelDataExtractor");
+    qmlRegisterType<QmlStringListModel>("FlightGear", 1, 0, "StringListModel");
+
 
     qmlRegisterSingletonType(QUrl("qrc:/qml/OverlayShared.qml"), "FlightGear", 1, 0, "OverlayShared");
+    styleTypeId = qmlRegisterSingletonType(QUrl("qrc:/qml/Style.qml"), "FlightGear", 1, 0, "Style");
 
     qmlRegisterType<GettingStartedScope>("FlightGear", 1, 0, "GettingStartedScope");
     qmlRegisterType<GettingStartedTipsController>("FlightGear", 1, 0, "GettingStartedController");
@@ -274,11 +286,11 @@ void LauncherController::initialRestoreSettings()
 
 void LauncherController::saveSettings()
 {
-    QSettings settings;
-    settings.setValue("window-geometry", m_window->geometry());
-    if (m_window->windowState() != Qt::WindowNoState) {
-        settings.setValue("window-state", m_window->windowState());
+    auto settings = flightgear::getQSettings();
+    if (m_window->windowState() != Qt::WindowMaximized) {
+        settings.setValue("window-geometry", m_window->geometry());
     }
+    settings.setValue("window-state", m_window->windowState());
 
     m_config->saveConfigToINI();
     m_aircraftHistory->saveToSettings();
@@ -297,7 +309,7 @@ void LauncherController::collectAircraftArgs()
             m_config->setArg("aircraft-dir", setFileInfo.dir().absolutePath());
             QString setFile = setFileInfo.fileName();
             Q_ASSERT(setFile.endsWith("-set.xml"));
-            setFile.truncate(setFile.count() - 8); // drop the '-set.xml' portion
+            setFile.truncate(setFile.size() - 8); // drop the '-set.xml' portion
             m_config->setArg("aircraft", setFile);
         } else if (m_selectedAircraft.scheme() == "package") {
             // no need to set aircraft-dir, handled by the corresponding code
@@ -312,9 +324,6 @@ void LauncherController::collectAircraftArgs()
         QString state = m_aircraftState;
         if ((m_aircraftState == "auto") && !m_selectedAircraftInfo->haveExplicitAutoState()) {
             state = selectAircraftStateAutomatically();
-#if QT_VERSION >= 0x050600
-            qInfo() << "doing launcher auto state selection, picked:" + state;
-#endif
         }
 
         if (!state.isEmpty()) {
@@ -342,7 +351,7 @@ void LauncherController::restoreAircraft()
 
 void LauncherController::doRun()
 {
-    flightgear::addSentryBreadcrumb("Launcher: fly!", "info");
+    flightgear::addSentryBreadcrumb("Launcher: fly!:" + m_selectedAircraft.toString().toStdString(), "info");
     flightgear::Options* opt = flightgear::Options::sharedInstance();
     m_config->reset();
     m_config->collect();
@@ -352,8 +361,10 @@ void LauncherController::doRun()
     QVariant locSet = m_location->saveLocation();
     m_locationHistory->insert(locSet);
 
+    flightgear::addSentryBreadcrumb("acft path:" + m_selectedAircraftInfo->pathOnDisk().toStdString(), "info");
+
     // aircraft paths
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
     QString downloadDir = settings.value("downloadSettings/downloadDir").toString();
     if (!downloadDir.isEmpty()) {
         QDir d(downloadDir);
@@ -391,7 +402,7 @@ void LauncherController::doApply()
             QFileInfo setFileInfo(m_selectedAircraft.toLocalFile());
             QString setFile = setFileInfo.fileName();
             Q_ASSERT(setFile.endsWith("-set.xml"));
-            setFile.truncate(setFile.count() - 8); // drop the '-set.xml' portion
+            setFile.truncate(setFile.size() - 8); // drop the '-set.xml' portion
             aircraftDir = setFileInfo.dir().absolutePath().toStdString();
             aircraftPropValue = setFile.toStdString();
         } else if (m_selectedAircraft.scheme() == "package") {
@@ -763,7 +774,7 @@ QVariantList LauncherController::defaultSplashUrls() const
 
 QVariant LauncherController::loadUISetting(QString name, QVariant defaultValue) const
 {
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
     if (!settings.contains(name))
         return defaultValue;
     return settings.value(name);
@@ -771,7 +782,7 @@ QVariant LauncherController::loadUISetting(QString name, QVariant defaultValue)
 
 void LauncherController::saveUISetting(QString name, QVariant value) const
 {
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
     settings.setValue(name, value);
 }
 
@@ -818,8 +829,8 @@ void LauncherController::requestRestoreDefaults()
 	}
 
 	{
-		QSettings settings;
-		settings.clear();
+        auto settings = flightgear::getQSettings();
+        settings.clear();
 		settings.setValue("restore-defaults-on-run", true);
 	}
 
@@ -829,8 +840,8 @@ void LauncherController::requestRestoreDefaults()
 void LauncherController::requestChangeDataPath()
 {
 	QString currentLocText;
-	QSettings settings;
-	QString root = settings.value(SetupRootDialog::rootPathKey()).toString();
+    auto settings = flightgear::getQSettings();
+    QString root = settings.value(SetupRootDialog::rootPathKey()).toString();
 	if (root.isNull()) {
 		currentLocText = tr("Currently the built-in data files are being used");
 	}
@@ -886,7 +897,7 @@ void LauncherController::setAircraftGridMode(bool aircraftGridMode)
     if (m_aircraftGridMode == aircraftGridMode)
         return;
 
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
     settings.setValue("aircraftGridMode", aircraftGridMode);
     m_aircraftGridMode = aircraftGridMode;
     emit aircraftGridModeChanged(m_aircraftGridMode);
@@ -895,7 +906,7 @@ void LauncherController::setAircraftGridMode(bool aircraftGridMode)
 void LauncherController::resetGettingStartedTips()
 {
     {
-        QSettings settings;
+        auto settings = flightgear::getQSettings();
         settings.beginGroup("GettingStarted-DontShow");
         settings.remove(""); // remove all keys in the current group
         settings.endGroup();
@@ -916,14 +927,14 @@ void LauncherController::setMinWindowSize(QSize sz)
 QUrl LauncherController::flyIconUrl() const
 {
     if (m_aircraftType == Helicopter) {
-        return QUrl{"qrc:///svg/toolbox-fly-heli"};
+        return QUrl{"image://colored-icon/toolbox-fly-heli"};
     } else if (m_selectedAircraftInfo) {
         if (m_selectedAircraftInfo->hasTag("spaceship")) {
-            return QUrl{"qrc:///svg/toolbox-fly-alt"};
+            return QUrl{"image://colored-icon/toolbox-fly-alt"};
         }
     }
 
-    return QUrl{"qrc:///svg/toolbox-fly"};
+    return QUrl{"image://colored-icon/toolbox-fly"};
 }
 
 QString LauncherController::flyButtonLabel() const
