@@ -32,11 +32,13 @@
 
 #include "AirportDiagram.hxx"
 #include "CarrierDiagram.hxx"
-#include "NavaidDiagram.hxx"
-#include "LaunchConfig.hxx"
-#include "DefaultAircraftLocator.hxx"
-#include "NavaidSearchModel.hxx"
 #include "CarriersLocationModel.hxx"
+#include "DefaultAircraftLocator.hxx"
+#include "LaunchConfig.hxx"
+#include "NavaidDiagram.hxx"
+#include "NavaidSearchModel.hxx"
+#include "QmlPositionedModel.hxx"
+#include "SettingsWrapper.hxx"
 
 #include <Airports/airport.hxx>
 #include <Airports/groundnetwork.hxx>
@@ -93,10 +95,15 @@ FGPositionedList loadPositionedList(QVariant v)
 LocationController::LocationController(QObject *parent) :
     QObject(parent)
 {
+    qmlRegisterUncreatableType<QmlPositionedModel>("FlightGear.Launcher", 1, 0, "QmlPositionedModel", "no");
+
     m_searchModel = new NavaidSearchModel(this);
     m_detailQml = new QmlPositioned(this);
     m_baseQml = new QmlPositioned(this);
     m_carriersModel = new CarriersLocationModel(this);
+    m_runwaysModel = new QmlPositionedModel(this);
+    m_helipadsModel = new QmlPositionedModel(this);
+    m_parkingsModel = new QmlPositionedModel(this);
 
     m_defaultAltitude = QuantityValue{Units::FeetMSL, 6000};
     m_defaultAirspeed = QuantityValue{Units::Knots, 120};
@@ -125,7 +132,7 @@ void LocationController::setLaunchConfig(LaunchConfig *config)
 
 void LocationController::restoreSearchHistory()
 {
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
     m_recentLocations = loadPositionedList(settings.value("recent-locations"));
 }
 
@@ -236,6 +243,9 @@ void LocationController::clearLocation()
     m_baseQml->setGuid(0);
     m_carrierParkings.clear();
     m_carrierParking.clear();
+    m_runwaysModel->clear();
+    m_helipadsModel->clear();
+    m_parkingsModel->clear();
     emit baseLocationChanged();
 }
 
@@ -260,6 +270,8 @@ void LocationController::setBaseLocation(QmlPositioned* pos)
             m_onFinal = false;
             m_useActiveRunway = false;
         }
+
+        updateAirportModels();
     } else {
         m_airportLocation.clear();
     }
@@ -268,6 +280,19 @@ void LocationController::setBaseLocation(QmlPositioned* pos)
     emit baseLocationChanged();
 }
 
+void LocationController::updateAirportModels()
+{
+    m_runwaysModel->setValues(m_airportLocation->getRunways());
+
+    FGPositionedList helipads;
+    for (unsigned int r = 0; r < m_airportLocation->numHelipads(); ++r) {
+        helipads.push_back(m_airportLocation->getHelipadByIndex(r));
+    }
+
+    m_helipadsModel->setValues(helipads);
+    m_parkingsModel->setValues(m_airportLocation->groundNetwork()->allParkings());
+}
+
 void LocationController::setDetailLocation(QmlPositioned* pos)
 {
     if (pos && (pos->inner() == m_detailLocation))
@@ -320,50 +345,6 @@ void LocationController::addToRecent(QmlPositioned* pos)
     addToRecent(pos->inner());
 }
 
-QObjectList LocationController::airportRunways() const
-{
-    if (!m_airportLocation)
-        return {};
-
-    QObjectList result;
-    for (unsigned int r = 0; r < m_airportLocation->numRunways(); ++r) {
-        auto p = new QmlPositioned(m_airportLocation->getRunwayByIndex(r).ptr());
-        QQmlEngine::setObjectOwnership(p, QQmlEngine::JavaScriptOwnership);
-        result.push_back(p);
-    }
-
-    return result;
-}
-
-QObjectList LocationController::airportHelipads() const
-{
-    if (!m_airportLocation)
-        return {};
-
-    QObjectList result;
-    for (unsigned int r = 0; r < m_airportLocation->numHelipads(); ++r) {
-        auto p = new QmlPositioned(m_airportLocation->getHelipadByIndex(r).ptr());
-        QQmlEngine::setObjectOwnership(p, QQmlEngine::JavaScriptOwnership);
-        result.push_back(p);
-    }
-
-    return result;
-}
-
-QObjectList LocationController::airportParkings() const
-{
-    if (!m_airportLocation)
-        return {};
-
-    QObjectList result;
-    for (auto park : m_airportLocation->groundNetwork()->allParkings()) {
-        auto p = new QmlPositioned(park);
-        QQmlEngine::setObjectOwnership(p, QQmlEngine::JavaScriptOwnership);
-        result.push_back(p);
-    }
-    return result;
-}
-
 void LocationController::showHistoryInSearchModel()
 {
     // prepend the default location and tutorial airport
@@ -571,9 +552,18 @@ void LocationController::restoreLocation(QVariantMap l)
             const SGGeod vicinity = SGGeod::fromDeg(l.value("vicinity-lon").toDouble(),
                                                     l.value("vicinity-lat").toDouble());
 
-            FGPositioned::TypeFilter filter({FGPositioned::Type::NDB, FGPositioned::Type::VOR,
-                                             FGPositioned::Type::FIX, FGPositioned::Type::WAYPOINT});
-            m_location = FGPositioned::findClosestWithIdent(ident, vicinity, &filter);
+            // special-case TACANs so we don't get an ambiguity on VORTACs, where
+            // there would be a double hit on any ident
+            const auto isTacan = l.value("location-is-tacan").toBool();
+            if (isTacan) {
+                FGPositioned::TypeFilter filter(FGPositioned::Type::DME);
+                m_location = FGPositioned::findClosestWithIdent(ident, vicinity, &filter);
+            } else {
+                FGPositioned::TypeFilter filter({FGPositioned::Type::NDB, FGPositioned::Type::VOR,
+                                                 FGPositioned::Type::FIX, FGPositioned::Type::WAYPOINT});
+                m_location = FGPositioned::findClosestWithIdent(ident, vicinity, &filter);
+            }
+
             m_baseQml->setInner(m_location);
         } else if (l.contains("location-lat")) {
             m_locationIsLatLon = true;
@@ -616,6 +606,8 @@ void LocationController::restoreLocation(QVariantMap l)
             m_onFinal = l.value("location-on-final").toBool();
             setAbeam(l.value("abeam").toBool());
             m_offsetDistance = l.value("location-apt-final-distance", QVariant::fromValue(m_defaultOffsetDistance)).value<QuantityValue>();
+            updateAirportModels();
+
         } // of location is an airport
     } catch (const sg_exception&) {
         qWarning() << "Errors restoring saved location, clearing";
@@ -643,8 +635,6 @@ bool LocationController::shouldStartPaused() const
         // navaid, start paused
         return true;
     }
-
-    return false;
 }
 
 QVariantMap LocationController::saveLocation() const
@@ -680,8 +670,14 @@ QVariantMap LocationController::saveLocation() const
                     locationSet.insert("location-apt-parking", QString::fromStdString(m_detailLocation->ident()));
                 }
             }
+
         } else { // not an aiport, must be a navaid
             locationSet.insert("location-navaid", QString::fromStdString(m_location->ident()));
+            if (m_location->type() == FGPositioned::DME) {
+                // so we don't get ambiguous on VORTACs, explicity mark TACANs
+                // otherwise every VORTAC would be ambiguous
+                locationSet.insert("location-is-tacan", true);
+            }
             locationSet.insert("vicinity-lat", m_location->geod().getLatitudeDeg());
             locationSet.insert("vicinity-lon", m_location->geod().getLongitudeDeg());
 
@@ -716,11 +712,23 @@ void LocationController::setLocationProperties()
 {
     SGPropertyNode_ptr presets = fgGetNode("/sim/presets", true);
 
-    QStringList props = QStringList() << "vor-id" << "fix" << "ndb-id" <<
-        "runway-requested" << "navaid-id" << "offset-azimuth-deg" <<
-        "offset-distance-nm" << "glideslope-deg" <<
-        "speed-set" << "on-ground" << "airspeed-kt" <<
-        "airport-id" << "runway" << "parkpos" << "carrier" << "carrier-position";
+    QStringList props = QStringList() << "vor-id"
+                                      << "fix"
+                                      << "ndb-id"
+                                      << "tacan-id"
+                                      << "runway-requested"
+                                      << "navaid-id"
+                                      << "offset-azimuth-deg"
+                                      << "offset-distance-nm"
+                                      << "glideslope-deg"
+                                      << "speed-set"
+                                      << "on-ground"
+                                      << "airspeed-kt"
+                                      << "airport-id"
+                                      << "runway"
+                                      << "parkpos"
+                                      << "carrier"
+                                      << "carrier-position";
 
     Q_FOREACH(QString s, props) {
         SGPropertyNode* c = presets->getChild(s.toStdString());
@@ -814,6 +822,8 @@ void LocationController::setLocationProperties()
             // parking selection
             fgSetString("/sim/presets/parkpos", m_detailLocation->ident());
         }
+
+        updateAirportModels();
         // of location is an airport
     } else {
         fgSetString("/sim/presets/airport-id", "");
@@ -837,6 +847,12 @@ void LocationController::setLocationProperties()
             case FGPositioned::FIX:
                 fgSetString("/sim/presets/fix", m_location->ident());
                 break;
+
+            // assume if a DME was selected, it was actually a TACAN
+            case FGPositioned::DME:
+                fgSetString("/sim/presets/tacan-id", m_location->ident());
+                break;
+
             default:
                 break;
         }
@@ -931,7 +947,7 @@ void LocationController::applyOnFinal()
 
         const double offsetNm = m_offsetDistance.convertToUnit(Units::NauticalMiles).value;
         m_config->setArg("offset-distance", QString::number(offsetNm));
-        m_config->setArg("on-ground", std::string("false"));
+        m_config->setArg("in-air", std::string(""));
 
         applyAirspeed();
         applyAltitude();
@@ -1033,6 +1049,11 @@ void LocationController::onCollectConfig()
             case FGPositioned::FIX:
                  m_config->setArg("fix", m_location->ident());
                 break;
+
+            case FGPositioned::DME:
+                m_config->setArg("tacan", m_location->ident());
+                break;
+
             default:
                 break;
         }
@@ -1164,6 +1185,9 @@ QString LocationController::description() const
             navaidType = QString("VOR"); break;
         case FGPositioned::NDB:
             navaidType = QString("NDB"); break;
+        case FGPositioned::DME:
+            navaidType = QString("TACAN");
+            break;
         case FGPositioned::FIX:
             return tr("%2 waypoint %1").arg(ident).arg(offsetDesc);
         default:
@@ -1173,8 +1197,6 @@ QString LocationController::description() const
 
         return tr("%4 %1 %2 (%3)").arg(navaidType).arg(ident).arg(name).arg(offsetDesc);
     }
-
-    return tr("No location selected");
 }
 
 
@@ -1197,6 +1219,6 @@ void LocationController::addToRecent(FGPositionedRef pos)
     }
 
     m_recentLocations.insert(m_recentLocations.begin(), pos);
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
     settings.setValue("recent-locations", savePositionList(m_recentLocations));
 }
