@@ -1,11 +1,20 @@
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+/*
+ * SPDX-FileCopyrightText: (C) 2010 James Turner <james@flightgear.org>
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
+
+#include "config.h"
 
 #include "MapWidget.hxx"
 
 #include <sstream>
 #include <algorithm> // for std::sort
+
+// ensure we include this before puAux.h, so that 
+// #define _PU_H_ 1 has been done, and hence we don't
+// include the un-modified system pu.h
+#include "FlightGear_pu.h"
+
 #include <plib/puAux.h>
 
 #include <simgear/sg_inlines.h>
@@ -388,23 +397,29 @@ namespace
 class MapAirportFilter : public FGAirport::AirportFilter
 {
 public:
-    MapAirportFilter(SGPropertyNode_ptr nd)
+    MapAirportFilter(SGPropertyNode_ptr nd) :
+        _heliports(nd->getBoolValue("draw-heliports", false)),
+        _hardRunwaysOnly( nd->getBoolValue("hard-surfaced-airports", true)),
+        _minLengthFt(fgGetDouble("/sim/navdb/min-runway-length-ft", 2000))
     {
-        _heliports = nd->getBoolValue("show-heliports", false);
-        _hardRunwaysOnly = nd->getBoolValue("hard-surfaced-airports", true);
-        _minLengthFt = fgGetDouble("/sim/navdb/min-runway-length-ft", 2000);
     }
     
-    virtual FGPositioned::Type maxType() const {
+    FGPositioned::Type maxType() const override
+    {
         return _heliports ? FGPositioned::HELIPORT : FGPositioned::AIRPORT;
     }
     
-    virtual bool passAirport(FGAirport* aApt) const {
-        if (_hardRunwaysOnly) {
+    FGPositioned::Type minType() const override
+    {
+        return FGPositioned::AIRPORT;
+    }
+
+    bool passAirport(FGAirport* aApt) const override
+    {
+        if (_hardRunwaysOnly && !aApt->isHeliport()) {
             return aApt->hasHardRunwayOfLengthFt(_minLengthFt);
         }
-        
-        return true;
+        return (aApt->type() <= maxType()) && (aApt->type() >= minType());
     }
     
     void showAll()
@@ -413,9 +428,9 @@ public:
     }
     
 private:
-    bool _heliports;
+    const bool _heliports;
     bool _hardRunwaysOnly;
-    double _minLengthFt;
+    const double _minLengthFt;
 };
 
 class NavaidFilter : public FGPositioned::Filter
@@ -454,18 +469,19 @@ private:
 const int MAX_ZOOM = 12;
 const int SHOW_DETAIL_ZOOM = 8;
 const int SHOW_DETAIL2_ZOOM = 5;
-const int CURSOR_PAN_STEP = 32;
 
 MapWidget::MapWidget(int x, int y, int maxX, int maxY) :
   puObject(x,y,maxX, maxY)
 {
-  _route = static_cast<FGRouteMgr*>(globals->get_subsystem("route-manager"));
+  _route = globals->get_subsystem<FGRouteMgr>();
   _gps = fgGetNode("/instrumentation/gps");
 
   _width = maxX - x;
   _height = maxY - y;
   _hasPanned = false;
   _projection = PROJECTION_AZIMUTHAL_EQUIDISTANT;
+
+  _projectionCenter = globals->get_aircraft_position();
   _magneticHeadings = false;
   
   MapData::setFont(legendFont);
@@ -480,6 +496,14 @@ MapWidget::~MapWidget()
   clearData();
 }
 
+static void createPropertyIfRequired(SGPropertyNode_ptr p, const std::string& name, bool defaultValue)
+{
+    auto existing = p->getChild(name);
+    if (!existing) {
+        p->setBoolValue(name, defaultValue);
+    }
+}
+
 void MapWidget::setProperty(SGPropertyNode_ptr prop)
 {
   _root = prop;
@@ -490,10 +514,14 @@ void MapWidget::setProperty(SGPropertyNode_ptr prop)
   
 // expose MAX_ZOOM to the UI
   _root->setIntValue("max-zoom", MAX_ZOOM);
-  _root->setBoolValue("centre-on-aircraft", true);
-  _root->setBoolValue("draw-data", false);
-  _root->setBoolValue("draw-flight-history", false);
-  _root->setBoolValue("magnetic-headings", true);
+    
+  createPropertyIfRequired(_root, "centre-on-aircraft", true);
+  createPropertyIfRequired(_root, "draw-data", false);
+  createPropertyIfRequired(_root, "draw-pois", true);
+  createPropertyIfRequired(_root, "draw-aircraft", true);
+  createPropertyIfRequired(_root, "draw-flight-history", false);
+  createPropertyIfRequired(_root, "magnetic-headings", true);
+  createPropertyIfRequired(_root, "key-pan", true);
 }
 
 void MapWidget::setSize(int w, int h)
@@ -512,13 +540,15 @@ void MapWidget::doHit( int button, int updown, int x, int y )
     handlePan(x, y);
     return;
   }
-  
-  if (button == 3) { // mouse-wheel up
-    zoomIn();
-  } else if (button == 4) { // mouse-wheel down
-    zoomOut();
+  if (updown == PU_DOWN)
+  {
+      if (button == 3) { // mouse-wheel up
+          zoomIn();
+      }
+      else if (button == 4) { // mouse-wheel down
+          zoomOut();
+      }
   }
-  
   _hitLocation = SGVec2d(x - abox.min[0], y - abox.min[1]);
   if ((button == 2) && (updown == PU_DOWN)) {
     _clickGeod = unproject(_hitLocation - SGVec2d(_width>>1, _height>>1));
@@ -542,11 +572,22 @@ void MapWidget::handlePan(int x, int y)
   _hitLocation = SGVec2d(x,y);
 }
 
+#if 0
 int MapWidget::checkKey (int key, int updown )
 {
   if ((updown == PU_UP) || !isVisible () || !isActive () || (window != puGetWindow())) {
     return FALSE ;
   }
+  
+  bool key_pan = _root->getIntValue("key-pan");
+  if (!key_pan && (0
+        || key == PU_KEY_UP
+        || key == PU_KEY_DOWN
+        || key == PU_KEY_LEFT
+        || key == PU_KEY_RIGHT
+        )) {
+    return FALSE;
+  }
 
   switch (key)
   {
@@ -582,6 +623,8 @@ int MapWidget::checkKey (int key, int updown )
 
   return TRUE ;
 }
+#endif
+
 
 void MapWidget::pan(const SGVec2d& delta)
 {
@@ -616,6 +659,7 @@ void MapWidget::zoomOut()
 
 void MapWidget::update()
 {
+    _drawAircraft = _root->getBoolValue("draw-aircraft");
     _aircraft = globals->get_aircraft_position();
     
     bool mag = _root->getBoolValue("magnetic-headings");
@@ -628,7 +672,7 @@ void MapWidget::update()
         _root->setBoolValue("centre-on-aircraft", false);
         _hasPanned = false;
     }
-    else if (_root->getBoolValue("centre-on-aircraft")) {
+    else if (_root->getBoolValue("centre-on-aircraft") && _drawAircraft) {
         _projectionCenter = _aircraft;
     }
     
@@ -636,7 +680,7 @@ void MapWidget::update()
     _magVar->update(_projectionCenter, julianDate);
     
     _aircraftUp = _root->getBoolValue("aircraft-heading-up");
-    if (_aircraftUp) {
+    if (_aircraftUp && _drawAircraft) {
         _upHeading = fgGetDouble("/orientation/heading-deg");
     } else {
         _upHeading = 0.0;
@@ -654,12 +698,7 @@ void MapWidget::update()
     // symbols.
     _drawRangeNm = SGGeodesy::distanceNm(_projectionCenter, topLeft) + 10.0;
   
-    FGFlightHistory* history = (FGFlightHistory*) globals->get_subsystem("history");
-    if (history && _root->getBoolValue("draw-flight-history")) {
-        _flightHistoryPath = history->pathForHistory();
-    } else {
-        _flightHistoryPath.clear();
-    }
+    updateFlightHistory();
 
 // make spatial queries. This can trigger loading of XML files, etc, so we do
 // not want to do it in draw(), which can be called from an arbitrary OSG
@@ -682,23 +721,40 @@ void MapWidget::update()
         newItemsToDraw.insert(newItemsToDraw.end(), navs.begin(), navs.end());
     }
 
-    FGPositioned::TypeFilter tf(FGPositioned::COUNTRY);
-    if (_cachedZoom <= SHOW_DETAIL_ZOOM) {
-        tf.addType(FGPositioned::CITY);
+    const bool drawPOIs = _root->getBoolValue("draw-pois");
+    if (drawPOIs) {
+      FGPositioned::TypeFilter tf(FGPositioned::COUNTRY);
+
+      if (_cachedZoom <= SHOW_DETAIL_ZOOM) {
+          tf.addType(FGPositioned::CITY);
+      }
+      
+      if (_cachedZoom <= SHOW_DETAIL2_ZOOM) {
+          tf.addType(FGPositioned::TOWN);
+      }
+      
+      FGPositionedList poi = FGPositioned::findWithinRange(_projectionCenter, _drawRangeNm, &tf);
+      newItemsToDraw.insert(newItemsToDraw.end(), poi.begin(), poi.end());
     }
-    
-    if (_cachedZoom <= SHOW_DETAIL2_ZOOM) {
-        tf.addType(FGPositioned::TOWN);
-    }
-    
-    FGPositionedList poi = FGPositioned::findWithinRange(_projectionCenter, _drawRangeNm, &tf);
-    newItemsToDraw.insert(newItemsToDraw.end(), poi.begin(), poi.end());
-    
+
     _itemsToDraw.swap(newItemsToDraw);
     
     updateAIObjects();
 }
 
+void MapWidget::updateFlightHistory()
+{
+    if (_drawAircraft && _root->getBoolValue("draw-flight-history")) {
+        auto history = globals->get_subsystem<FGFlightHistory>();
+        if (history) {
+            _flightHistoryPath = history->pathForHistory();
+            return;
+        }
+    }
+
+    _flightHistoryPath.clear();
+}
+
 void MapWidget::updateAIObjects()
 {
     if (!_root->getBoolValue("draw-traffic") || (_cachedZoom > SHOW_DETAIL_ZOOM)) {
@@ -747,7 +803,7 @@ void MapWidget::draw(int dx, int dy)
 
   drawLatLonGrid();
 
-  if (_aircraftUp) {
+  if (_aircraftUp && _drawAircraft) {
     int textHeight = legendFont.getStringHeight() + 5;
 
     // draw heading line
@@ -767,10 +823,10 @@ void MapWidget::draw(int dx, int dy)
   drawGPSData();
   drawNavRadio(fgGetNode("/instrumentation/nav[0]", false));
   drawNavRadio(fgGetNode("/instrumentation/nav[1]", false));
-  paintAircraftLocation(_aircraft);
   drawFlightHistory();
   paintRoute();
   paintRuler();
+  paintAircraftLocation(_aircraft);
 
   drawData();
 
@@ -780,7 +836,7 @@ void MapWidget::draw(int dx, int dy)
 
 void MapWidget::paintRuler()
 {
-  if (_clickGeod == SGGeod()) {
+  if (_clickGeod == SGGeod() || !_drawAircraft) {
     return;
   }
 
@@ -807,6 +863,10 @@ void MapWidget::paintRuler()
 
 void MapWidget::paintAircraftLocation(const SGGeod& aircraftPos)
 {
+  if (!_drawAircraft) {
+    return;
+  }
+
   SGVec2d loc = project(aircraftPos);
 
   double hdg = fgGetDouble("/orientation/heading-deg");
@@ -1019,39 +1079,43 @@ void MapWidget::drawLatLonGrid()
 
 void MapWidget::drawGPSData()
 {
-  std::string gpsMode = _gps->getStringValue("mode");
-
-  SGGeod wp0Geod = SGGeod::fromDeg(
-        _gps->getDoubleValue("wp/wp[0]/longitude-deg"),
-        _gps->getDoubleValue("wp/wp[0]/latitude-deg"));
-
-  SGGeod wp1Geod = SGGeod::fromDeg(
-        _gps->getDoubleValue("wp/wp[1]/longitude-deg"),
-        _gps->getDoubleValue("wp/wp[1]/latitude-deg"));
-
 // draw track line
-  double gpsTrackDeg = _gps->getDoubleValue("indicated-track-true-deg");
-  double gpsSpeed = _gps->getDoubleValue("indicated-ground-speed-kt");
-  double az2;
+  if (_drawAircraft) {
+    double gpsSpeed = _gps->getDoubleValue("indicated-ground-speed-kt");
 
-  if (gpsSpeed > 3.0) { // only draw track line if valid
-    SGGeod trackRadial;
-    SGGeodesy::direct(_aircraft, gpsTrackDeg, _drawRangeNm * SG_NM_TO_METER, trackRadial, az2);
+    if (gpsSpeed > 3.0) { // only draw track line if valid
+      double gpsTrackDeg = _gps->getDoubleValue("indicated-track-true-deg");
+      double az2;
 
-    glColor3f(1.0, 1.0, 0.0);
-    glEnable(GL_LINE_STIPPLE);
-    glLineStipple(1, 0x00FF);
-    drawLine(project(_aircraft), project(trackRadial));
-    glDisable(GL_LINE_STIPPLE);
+      SGGeod trackRadial;
+      SGGeodesy::direct(_aircraft, gpsTrackDeg, _drawRangeNm * SG_NM_TO_METER, trackRadial, az2);
+
+      glColor3f(1.0, 1.0, 0.0);
+      glEnable(GL_LINE_STIPPLE);
+      glLineStipple(1, 0x00FF);
+      drawLine(project(_aircraft), project(trackRadial));
+      glDisable(GL_LINE_STIPPLE);
+    }
   }
 
+  std::string gpsMode = _gps->getStringValue("mode");
+
   if (gpsMode == "dto") {
+    SGGeod wp0Geod = SGGeod::fromDeg(
+        _gps->getDoubleValue("wp/wp[0]/longitude-deg"),
+        _gps->getDoubleValue("wp/wp[0]/latitude-deg")
+    );
+
+    SGGeod wp1Geod = SGGeod::fromDeg(
+        _gps->getDoubleValue("wp/wp[1]/longitude-deg"),
+        _gps->getDoubleValue("wp/wp[1]/latitude-deg")
+    );
+
     SGVec2d wp0Pos = project(wp0Geod);
     SGVec2d wp1Pos = project(wp1Geod);
 
     glColor3f(1.0, 0.0, 1.0);
     drawLine(wp0Pos, wp1Pos);
-
   }
 
   if (_gps->getBoolValue("scratch/valid")) {
@@ -1066,16 +1130,19 @@ void MapWidget::drawPositioned()
       FGPositionedRef p = _itemsToDraw[i];
       switch (p->type()) {
           case FGPositioned::AIRPORT:
-              drawAirport((FGAirport*) p.get());
+              drawAirport(fgpositioned_cast<FGAirport>(p));
+              break;
+          case FGPositioned::HELIPORT:
+              drawHeliport(fgpositioned_cast<FGAirport>(p));
               break;
           case FGPositioned::NDB:
-              drawNDB(false, (FGNavRecord*) p.get());
+              drawNDB(false, fgpositioned_cast<FGNavRecord>(p));
               break;
           case FGPositioned::VOR:
-              drawVOR(false, (FGNavRecord*) p.get());
+              drawVOR(false, fgpositioned_cast<FGNavRecord>(p));
               break;
           case FGPositioned::FIX:
-              drawFix((FGFix*) p.get());
+              drawFix(fgpositioned_cast<FGFix>(p));
               break;
          case FGPositioned::TOWN:
           case FGPositioned::CITY:
@@ -1355,6 +1422,28 @@ void MapWidget::drawAirport(FGAirport* apt)
 
 }
 
+
+void MapWidget::drawHeliport(FGAirport* apt)
+{
+  SGVec2d pos = project(apt->geod());
+  glLineWidth(1.0);
+  glColor3f(1.0, 0.0, 1.0);
+  circleAt(pos, 16, 5.0);
+
+  if (validDataForKey(apt)) {
+    setAnchorForKey(apt, pos);
+    return;
+  }
+
+  MapData* d = createDataForKey(apt);
+  d->setLabel(apt->ident());
+  d->setPriority(40);
+  d->setOffset(MapData::VALIGN_CENTER | MapData::HALIGN_LEFT, 10);
+  d->setAnchor(pos);
+
+}
+
+
 int MapWidget::scoreAirportRunways(FGAirport* apt)
 {
   bool needHardSurface = _root->getBoolValue("hard-surfaced-airports", true);
@@ -1633,7 +1722,6 @@ SGVec2d MapWidget::project(const SGGeod& geod) const
         SGQuatd orient = SGQuatd::fromLonLat(_projectionCenter);
         cartPt = orient.rotateBack(cartPt);
         return SGVec2d(cartPt.y(), cartPt.x()) * currentScale();
-        break;
     }
     } // of projection mode switch
     
@@ -1969,7 +2057,7 @@ MapWidget::DrawAIObject::DrawAIObject(SGPropertyNode* m, const SGGeod& g) :
     heading = model->getDoubleValue("orientation/true-heading-deg");
     
     if ((name == "aircraft") || (name == "multiplayer") ||
-        (name == "wingman") || (name == "tanker"))
+        (name == "wingman") || (name == "tanker") || (name == "swift"))
     {
         speedKts = static_cast<int>(model->getDoubleValue("velocities/true-airspeed-kt"));
         label = model->getStringValue("callsign", "<>");
@@ -1977,7 +2065,7 @@ MapWidget::DrawAIObject::DrawAIObject(SGPropertyNode* m, const SGGeod& g) :
         // try to access the flight-plan of the aircraft. There are several layers
         // of potential NULL-ness here, so we have to be defensive at each stage.
         std::string originICAO, destinationICAO;
-        FGAIManager* aiManager = globals->get_subsystem<FGAIManager>();
+        auto aiManager = globals->get_subsystem<FGAIManager>();
         FGAIBasePtr aircraft = aiManager ? aiManager->getObjectFromProperty(model) : NULL;
         if (aircraft) {
             FGAIAircraft* p = static_cast<FGAIAircraft*>(aircraft.get());
@@ -2012,7 +2100,7 @@ MapWidget::DrawAIObject::DrawAIObject(SGPropertyNode* m, const SGGeod& g) :
         
         char buffer[1024];
         ::snprintf(buffer, 1024, "%s\n%dkts",
-                   model->getStringValue("name", "<>"),
+                   model->getStringValue("name", "<>").c_str(),
                    speedKts);
         legend = buffer;
     }
