@@ -12,6 +12,7 @@
 #include <Main/globals.hxx>
 #include <Viewer/renderer.hxx>
 #include <GUI/new_gui.hxx>
+#include <Main/sentryIntegration.hxx>
 
 #include <osgViewer/Viewer>
 
@@ -43,6 +44,8 @@ using namespace simgear::strutils;
 
 namespace {
 
+static bool static_isHeadless = false;
+
 bool isCanvasImplementationRegistered()
 {
 	if (!globals) {
@@ -58,11 +61,11 @@ bool isCanvasImplementationRegistered()
 HWND getMainViewerHWND()
 {
 	osgViewer::Viewer::Windows windows;
-	if (!globals || !globals->get_renderer() || !globals->get_renderer()->getViewer()) {
+	if (!globals || !globals->get_renderer() || !globals->get_renderer()->getViewerBase()) {
 		return 0;
 	}
 
-    globals->get_renderer()->getViewer()->getWindows(windows);
+    globals->get_renderer()->getViewerBase()->getWindows(windows);
     osgViewer::Viewer::Windows::const_iterator it = windows.begin();
     for(; it != windows.end(); ++it) {
         if (strcmp((*it)->className(), "GraphicsWindowWin32")) {
@@ -107,10 +110,29 @@ win32MessageBox(const std::string& caption,
 namespace flightgear
 {
 
+void setHeadlessMode(bool headless)
+{
+    static_isHeadless = headless;
+}
+
+bool isHeadlessMode()
+{
+    return static_isHeadless;
+}
+
 MessageBoxResult modalMessageBox(const std::string& caption,
     const std::string& msg,
     const std::string& moreText)
 {
+    // Headless mode.
+    if (static_isHeadless) {
+        SG_LOG(SG_HEADLESS, SG_ALERT, "ModalMessageBox Caption: \"" << caption << "\"");
+        SG_LOG(SG_HEADLESS, SG_ALERT, "ModalMessageBox Message: \"" << msg << "\"");
+        if (!moreText.empty())
+            SG_LOG(SG_HEADLESS, SG_ALERT, "ModalMessageBox More text: \"" << moreText << "\"");
+        return MSG_BOX_OK;
+    }
+
     // prefer canvas
     if (isCanvasImplementationRegistered()) {
         SGPropertyNode_ptr args(new SGPropertyNode);
@@ -136,9 +158,9 @@ MessageBoxResult modalMessageBox(const std::string& caption,
         s += "\n( " +  moreText + ")";
     }
 
-    NewGUI* gui = globals->get_subsystem<NewGUI>();
+    auto gui = globals->get_subsystem<NewGUI>();
     if (!gui || (fgGetBool("/sim/rendering/initialized", false) == false)) {
-        SG_LOG(SG_GENERAL, SG_ALERT, s);
+        SG_LOG(SG_GENERAL, SG_POPUP, s);
     } else {
         SGPropertyNode_ptr dlg = gui->getDialogProperties("popup");
         dlg->setStringValue("text/label", s  );
@@ -150,8 +172,22 @@ MessageBoxResult modalMessageBox(const std::string& caption,
 
 MessageBoxResult fatalMessageBoxWithoutExit(const std::string& caption,
     const std::string& msg,
-    const std::string& moreText)
+    const std::string& moreText,
+    bool reportToSentry)
 {
+    if (reportToSentry) {
+        flightgear::sentryReportFatalError(msg, moreText);
+    }
+    
+    // Headless mode.
+    if (static_isHeadless) {
+        SG_LOG(SG_HEADLESS, SG_ALERT, "Fatal Error: \"" << caption << "\"");
+        SG_LOG(SG_HEADLESS, SG_ALERT, "Error Message: \"" << msg << "\"");
+        if (!moreText.empty())
+            SG_LOG(SG_HEADLESS, SG_ALERT, "\tMore text: \"" << moreText << "\"");
+        return MSG_BOX_OK;
+    }
+
 #if defined(SG_WINDOWS)
     return win32MessageBox(caption, msg, moreText);
 #elif defined(SG_MAC)
@@ -166,7 +202,7 @@ MessageBoxResult fatalMessageBoxWithoutExit(const std::string& caption,
     if (fgGetBool("/sim/rendering/initialized", false) == false) {
         std::cerr << s << std::endl;
     } else {
-        NewGUI* _gui = (NewGUI *)globals->get_subsystem("gui");
+        auto _gui = globals->get_subsystem<NewGUI>();
         SGPropertyNode_ptr dlg = _gui->getDialogProperties("popup");
         dlg->setStringValue("text/label", s  );
         _gui->showDialog("popup");
@@ -179,10 +215,14 @@ MessageBoxResult fatalMessageBoxWithoutExit(const std::string& caption,
     const std::string& caption,
     const std::string& msg,
     const std::string& moreText,
-    int exitStatus)
+    int exitStatus,
+    bool reportToSentry)
 {
-    fatalMessageBoxWithoutExit(caption, msg, moreText);
-    exit(exitStatus);
+    fatalMessageBoxWithoutExit(caption, msg, moreText, reportToSentry);
+    // we can't use exit() here or QGuiApplication crashes
+    // let's instead throw a sepcial exception which we catch
+    // in boostrap.
+    throw FatalErrorException{};
 }
 
 } // of namespace flightgear
