@@ -24,7 +24,6 @@
 #include "MouseCursor.hxx"
 
 #include <cstring>
-#include <boost/foreach.hpp>
 
 #include <osgViewer/GraphicsWindow>
 #include <osgViewer/Viewer>
@@ -48,7 +47,7 @@
 
 namespace
 {
-    
+
 /**
  * @brief when no native cursor implementation is available, use the osgViewer support. This
  * has several limitations but is better than nothing
@@ -62,36 +61,35 @@ public:
         mCursor(osgViewer::GraphicsWindow::InheritCursor)
     {
         mActualCursor = mCursor;
-        
-        globals->get_renderer()->getViewer()->getWindows(mWindows);
     }
 
     virtual void setCursor(Cursor aCursor)
     {
+        m_currentCursor = aCursor;
         mCursor = translateCursor(aCursor);
         updateCursor();
     }
-    
+
     virtual void setCursorVisible(bool aVis)
     {
         if (mCursorObscured == aVis) {
             return;
         }
-        
+
         mCursorVisible = aVis;
         updateCursor();
     }
-    
+
     virtual void hideCursorUntilMouseMove()
     {
         if (mCursorObscured) {
             return;
         }
-        
+
         mCursorObscured = true;
         updateCursor();
     }
-    
+
     virtual void mouseMoved()
     {
         if (mCursorObscured) {
@@ -110,37 +108,61 @@ private:
         case CURSOR_IBEAM: return osgViewer::GraphicsWindow::TextCursor;
         case CURSOR_LEFT_RIGHT: return osgViewer::GraphicsWindow::LeftRightCursor;
         case CURSOR_UP_DOWN: return osgViewer::GraphicsWindow::UpDownCursor;
+
+        case CURSOR_LEFT_SIDE: return osgViewer::GraphicsWindow::LeftSideCursor;
+        case CURSOR_RIGHT_SIDE: return osgViewer::GraphicsWindow::RightSideCursor;
+        case CURSOR_TOP_SIDE: return osgViewer::GraphicsWindow::TopSideCursor;
+        case CURSOR_BOTTOM_SIDE: return osgViewer::GraphicsWindow::BottomSideCursor;
+
+        case CURSOR_TOP_LEFT: return osgViewer::GraphicsWindow::TopLeftCorner;
+        case CURSOR_TOP_RIGHT: return osgViewer::GraphicsWindow::TopRightCorner;
+        case CURSOR_BOTTOM_LEFT: return osgViewer::GraphicsWindow::BottomLeftCorner;
+        case CURSOR_BOTTOM_RIGHT: return osgViewer::GraphicsWindow::BottomRightCorner;
+
+        case CURSOR_WAIT: return osgViewer::GraphicsWindow::WaitCursor;
+
         default:
-			return osgViewer::GraphicsWindow::RightArrowCursor;  
+			return osgViewer::GraphicsWindow::RightArrowCursor;
         }
     }
-    
+
     void updateCursor()
     {
-        osgViewer::GraphicsWindow::MouseCursor cur = osgViewer::GraphicsWindow::InheritCursor;
+        auto cur = osgViewer::GraphicsWindow::InheritCursor;
         if (mCursorObscured || !mCursorVisible) {
             cur = osgViewer::GraphicsWindow::NoCursor;
         } else {
             cur = mCursor;
         }
-        
+
         if (cur == mActualCursor) {
             return;
         }
-        
-        BOOST_FOREACH(osgViewer::GraphicsWindow* gw, mWindows) {
+
+        // can happen during shutdown, we call fgSetMouseCursor from fgExitCleanup
+        if (!globals || !globals->get_renderer()) {
+            return;
+        }
+
+        osgViewer::ViewerBase* viewer_base = globals->get_renderer()->getViewerBase();
+        if (!viewer_base)
+            return;
+
+        std::vector<osgViewer::GraphicsWindow*> windows;
+        viewer_base->getWindows(windows);
+
+        for (auto gw : windows) {
             gw->setCursor(cur);
         }
-        
+
         mActualCursor = cur;
     }
-    
+
     bool mCursorObscured;
     bool mCursorVisible;
     osgViewer::GraphicsWindow::MouseCursor mCursor, mActualCursor;
-    std::vector<osgViewer::GraphicsWindow*> mWindows;
 };
-    
+
 } // of anonymous namespace
 
 static FGMouseCursor* static_instance = NULL;
@@ -161,8 +183,8 @@ FGMouseCursor* FGMouseCursor::instance()
 	#ifdef SG_WINDOWS
 	// set osgViewer cursor inherit, otherwise it will interefere
 		std::vector<osgViewer::GraphicsWindow*> gws;
-		globals->get_renderer()->getViewer()->getWindows(gws);
-		BOOST_FOREACH(osgViewer::GraphicsWindow* gw, gws) {
+		globals->get_renderer()->getViewerBase()->getWindows(gws);
+		for (auto gw : gws) {
             gw->setCursor(osgViewer::GraphicsWindow::InheritCursor);
         }
 
@@ -173,18 +195,18 @@ FGMouseCursor* FGMouseCursor::instance()
 	// and create our real implementation
 	//	static_instance = new WindowsMouseCursor;
 	#endif
-        
+
         // X11
-                
+
         if (static_instance == NULL) {
             static_instance = new StockOSGCursor;
         }
-        
+
         // initialise mouse-hide delay from global properties
-        
+
         globals->get_commands()->addCommand("set-cursor", static_instance, &FGMouseCursor::setCursorCommand);
     }
-    
+
     return static_instance;
 }
 
@@ -193,11 +215,15 @@ void FGMouseCursor::setAutoHideTimeMsec(unsigned int aMsec)
     mAutoHideTimeMsec = aMsec;
 }
 
+FGMouseCursor::Cursor FGMouseCursor::getCursor() const
+{
+    return m_currentCursor;
+}
 
 bool FGMouseCursor::setCursorCommand(const SGPropertyNode* arg, SGPropertyNode*)
 {
     // JMT 2013 - I would prefer this was a seperate 'warp' command, but
-    // historically set-cursor has done both. 
+    // historically set-cursor has done both.
     if (arg->hasValue("x") || arg->hasValue("y")) {
         SGPropertyNode *mx = fgGetNode("/devices/status/mice/mouse/x", true);
         SGPropertyNode *my = fgGetNode("/devices/status/mice/mouse/y", true);
@@ -208,8 +234,8 @@ bool FGMouseCursor::setCursorCommand(const SGPropertyNode* arg, SGPropertyNode*)
         my->setIntValue(y);
     }
 
-    
-    Cursor c = cursorFromString(arg->getStringValue("cursor"));    
+
+    Cursor c = cursorFromString(arg->getStringValue("cursor").c_str());
     setCursor(c);
     return true;
 }
@@ -226,7 +252,7 @@ const MouseCursorMap mouse_cursor_map[] = {
     { "hand", FGMouseCursor::CURSOR_HAND },
     { "closed-hand", FGMouseCursor::CURSOR_CLOSED_HAND },
     { "text", FGMouseCursor::CURSOR_IBEAM },
-    
+
 // aliases
     { "drag-horizontal", FGMouseCursor::CURSOR_LEFT_RIGHT },
     { "drag-vertical", FGMouseCursor::CURSOR_UP_DOWN },
