@@ -76,13 +76,18 @@
 #include "LauncherMainWindow.hxx"
 #include "LocalAircraftCache.hxx"
 #include "PathListModel.hxx"
+#include "SettingsWrapper.hxx"
+#include "SetupRootDialog.hxx"
 #include "UnitsModel.hxx"
-#include "GettingStartedTip.hxx"
 
 #if defined(SG_MAC)
 #include <GUI/CocoaHelpers.h>
 #endif
 
+#if defined(Q_OS_WIN)
+#include <windows.h> // for GetAsyncKeyState and VK_ constants
+#endif
+
 using namespace flightgear;
 using namespace simgear::pkg;
 using std::string;
@@ -211,8 +216,7 @@ class NaturalEarthDataLoaderThread : public QThread
     Q_OBJECT
 public:
 
-    NaturalEarthDataLoaderThread() :
-        m_lineInsertCount(0)
+    NaturalEarthDataLoaderThread()
     {
         connect(this, &QThread::finished, this, &NaturalEarthDataLoaderThread::onFinished);
     }
@@ -224,10 +228,26 @@ public:
 protected:
     void run() override
     {
-        loadNaturalEarthFile("ne_10m_coastline.shp", flightgear::PolyLine::COASTLINE, false);
-        loadNaturalEarthFile("ne_10m_rivers_lake_centerlines.shp", flightgear::PolyLine::RIVER, false);
-        loadNaturalEarthFile("ne_10m_lakes.shp", flightgear::PolyLine::LAKE, true);
-        loadNaturalEarthFile("ne_10m_urban_areas.shp", flightgear::PolyLine::URBAN, true);
+        struct FileAndType {
+            std::string file;
+            flightgear::PolyLine::Type type;
+            bool closed = false;
+        };
+
+        const std::initializer_list<FileAndType> files = {
+            {"ne_10m_coastline.shp", flightgear::PolyLine::COASTLINE, false},
+            {"ne_10m_rivers_lake_centerlines.shp", flightgear::PolyLine::RIVER, false},
+            {"ne_10m_lakes.shp", flightgear::PolyLine::LAKE, true},
+            {"ne_10m_urban_areas.shp", flightgear::PolyLine::URBAN, true}
+        };
+
+        for (const auto& d : files) {
+            if (m_abandoned) {
+                break;
+            }
+
+            loadNaturalEarthFile(d.file, d.type, d.closed);
+        }
     }
 
 private:
@@ -255,7 +275,6 @@ private:
     }
 
     flightgear::PolyLineList m_parsedLines;
-    unsigned int m_lineInsertCount;
     bool m_abandoned = false;
 };
 
@@ -301,11 +320,6 @@ OpenGLStatus checkForWorkingOpenGL()
                 flightgear::addSentryBreadcrumb("Detected Intel < 2.1 renderer", "info");
                 result = OpenGLStatus::Intel14;
             }
-        } else if (simgear::strutils::starts_with(renderer, "S3 Graphics")) {
-            if (ctx.format().majorVersion() < 2) {
-                flightgear::addSentryBreadcrumb("Detected S2 < 2.1 renderer", "info");
-                result = OpenGLStatus::Unknown;
-            }
         }
 
         // ensure the context is no longer current on the offscreen
@@ -331,9 +345,7 @@ static void simgearMessageOutput(QtMsgType type, const QMessageLogContext &conte
     sgDebugPriority mappedPriority = SG_WARN;
     switch (type) {
     case QtDebugMsg:    mappedPriority = SG_DEBUG; break;
-#if QT_VERSION >= 0x050500
     case QtInfoMsg:     mappedPriority = SG_INFO; break;
-#endif
     case QtWarningMsg:  mappedPriority = SG_WARN; break;
     case QtCriticalMsg: mappedPriority = SG_ALERT; break;
     case QtFatalMsg:    mappedPriority = SG_POPUP; break;
@@ -343,7 +355,7 @@ static void simgearMessageOutput(QtMsgType type, const QMessageLogContext &conte
     const char* file = context.file ? context.file : nullFile;
     const auto s = msg.toStdString();
     // important we copy the file name here, since QMessageLogContext doesn't
-    sglog().logCopyingFilename(SG_GUI, mappedPriority, file, context.line, s);
+    sglog().logCopyingFilename(SG_GUI, mappedPriority, file, context.line, "" /*function*/, s);
     if (type == QtFatalMsg) {
         // if we abort() here, we get a thread crash which prevents
         // us actually seeing the error
@@ -372,7 +384,7 @@ void selectUITranslation()
     QStringList uiLanguages = QLocale::system().uiLanguages();
     //qWarning() << "UI languages:" << uiLanguages;
 
-    for (QString locale : qAsConst(uiLanguages)) {
+    for (QString locale : std::as_const(uiLanguages)) {
         // remove script if it exists, eg zh-Hans-CN -> zh-CN
         locale = QLocale(locale).name();
         locale.replace('-', '_');
@@ -399,11 +411,6 @@ void initApp(int& argc, char** argv, bool doInitQSettings)
     if (!qtInitDone) {
         qtInitDone = true;
 
-        // Disable Qt 5.15 warnings about obsolete Connections/onFoo: syntax
-        // we cannot use the new syntax
-        // as long as we have to support Qt 5.9
-        qputenv("QT_LOGGING_RULES", "qt.qml.connections.warning=false");
-        
         initQtResources(); // can't be called from a namespace
 
         s_argc = argc; // QApplication only stores a reference to argc,
@@ -424,20 +431,17 @@ void initApp(int& argc, char** argv, bool doInitQSettings)
 		// leave things unset here, so users can use env var
 		// QT_AUTO_SCREEN_SCALE_FACTOR=1 to enable it at runtime
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
 #if !defined (SG_WINDOWS)
-    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
-#endif
+        QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
 #endif
         static_qApp.reset(new QApplication(s_argc, argv));
         static_qApp->setOrganizationName("FlightGear");
         static_qApp->setApplicationName("FlightGear");
         static_qApp->setOrganizationDomain("flightgear.org");
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 7, 0)
         static_qApp->setDesktopFileName(
           QStringLiteral("org.flightgear.FlightGear.desktop"));
-#endif
+
         QTranslator* fallbackTranslator = new QTranslator(static_qApp.get());
         if (!fallbackTranslator->load(QLatin1String(":/FlightGear_en_US.qm"))) {
             qWarning() << "Failed to load default (en) translations";
@@ -466,15 +470,6 @@ void initApp(int& argc, char** argv, bool doInitQSettings)
             selectUITranslation();
         }
 
-        // temporary code: only enable getting-started tips for English, to give translators time to
-        // catch up
-        if (!lang.empty()) {
-            GettingStartedTip::setGlobalTipsEnabled(simgear::strutils::starts_with(lang, "en"));
-        } else {
-            QLocale currentLocale;
-            GettingStartedTip::setGlobalTipsEnabled(currentLocale.name().startsWith("en"));
-        }
-
         // reset numeric / collation locales as described at:
         // http://doc.qt.io/qt-5/qcoreapplication.html#details
         ::setlocale(LC_NUMERIC, "C");
@@ -515,8 +510,9 @@ void initQSettings()
 
     if (!qSettingsInitDone) {
         qRegisterMetaType<QuantityValue>();
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         qRegisterMetaTypeStreamOperators<QuantityValue>("QuantityValue");
-
+#endif
 
         qSettingsInitDone = true;
         string fgHome = globals->get_fg_home().utf8Str();
@@ -527,6 +523,28 @@ void initQSettings()
     }
 }
 
+QSettings getQSettings()
+{
+#if defined(SG_MAC)
+    return QSettings(QSettings::IniFormat, QSettings::UserScope, QCoreApplication::organizationDomain(),
+                     "FlightGear_" FLIGHTGEAR_MAJOR_MINOR_VERSION);
+#else
+    return QSettings(QSettings::IniFormat, QSettings::UserScope, QCoreApplication::organizationName(),
+                     "FlightGear_" FLIGHTGEAR_MAJOR_MINOR_VERSION);
+#endif
+}
+
+std::unique_ptr<QSettings> createQSettings()
+{
+#if defined(SG_MAC)
+    return std::make_unique<QSettings>(QSettings::IniFormat, QSettings::UserScope, QCoreApplication::organizationDomain(),
+                                       "FlightGear_" FLIGHTGEAR_MAJOR_MINOR_VERSION);
+#else
+    return std::make_unique<QSettings>(QSettings::IniFormat, QSettings::UserScope, QCoreApplication::organizationName(),
+                                       "FlightGear_" FLIGHTGEAR_MAJOR_MINOR_VERSION);
+#endif
+}
+
 bool checkKeyboardModifiersForSettingFGRoot()
 {
     initQSettings();
@@ -611,7 +629,7 @@ void launcherSetSceneryPaths()
 
 // mimic what options.cxx does, so we can find airport data for parking
 // positions
-    QSettings settings;
+    auto settings = flightgear::getQSettings();
     // append explicit scenery paths
     Q_FOREACH(QString path, PathListModel::readEnabledPaths("scenery-paths-v2")) {
         globals->append_fg_scenery(path.toStdString());
@@ -663,7 +681,7 @@ bool runLauncherDialog()
         // launcher GUI. We'll disable the UI.
         LaunchConfig::setEnableDownloadDirUI(false);
     } else {
-        QSettings settings;
+        auto settings = flightgear::getQSettings();
         QString downloadDir = settings.value("download-dir").toString();
         if (!downloadDir.isEmpty()) {
             options->setOption("download-dir", downloadDir.toStdString());
@@ -685,11 +703,7 @@ bool runLauncherDialog()
     globals->packageRoot()->setLocale(globals->get_locale()->getPreferredLanguage());
 
     // startup the HTTP system now since packages needs it
-    FGHTTPClient* http = globals->add_new_subsystem<FGHTTPClient>();
-
-    // we guard against re-init in the global phase; bind and postinit
-    // will happen as normal
-    http->init();
+    FGHTTPClient::getOrCreate();
 
     QPointer<NaturalEarthDataLoaderThread> naturalEarthLoader = new NaturalEarthDataLoaderThread;
     naturalEarthLoader->start();
@@ -699,17 +713,14 @@ bool runLauncherDialog()
     flightgear::WindowBuilder::setPoseAsStandaloneApp(false);
 
     LauncherMainWindow dlg(false);
-    if (options->isOptionSet("enable-fullscreen")) {
+
+    if (options->isOptionSet("fullscreen")) {
         dlg.showFullScreen();
     } else {
-        dlg.show();
+        dlg.setVisible(true);
     }
 
     int appResult = qApp->exec();
-    if (appResult <= 0) {
-        return false; // quit
-    }
-
     // avoid crashes / NavCache races if the loader is still running after
     // the launcher exits
     if (naturalEarthLoader) {
@@ -724,6 +735,10 @@ bool runLauncherDialog()
     globals->clear_fg_scenery();
     globals->get_locale()->clear();
 
+    if (appResult <= 0) {
+        return false; // quit
+    }
+    
     return true;
 }
 
@@ -777,6 +792,16 @@ LockFileDialogResult showLockFileDialog()
     return LockFileContinue;
 }
 
+bool showSetupRootDialog(bool usingDefaultRoot)
+{
+    return SetupRootDialog::runDialog(usingDefaultRoot);
+}
+
+SetupRootResult restoreUserSelectedRoot(SGPath& path)
+{
+    return SetupRootDialog::restoreUserSelectedRoot(path);
+}
+
 } // of namespace flightgear
 
 #include "QtLauncher.moc"
