@@ -30,6 +30,7 @@
 #include <simgear/io/sg_file.hxx>
 #include <simgear/props/props_io.hxx>
 #include <simgear/math/SGMath.hxx>
+#include <simgear/math/interpolater.hxx>
 #include <Scripting/NasalSys.hxx>
 
 using simgear::PropertyList;
@@ -43,7 +44,7 @@ FGEventSetting::FGEventSetting( SGPropertyNode_ptr base ) :
 {
   SGPropertyNode_ptr n;
 
-  if( (n = base->getNode( "value" )) != NULL ) {  
+  if( (n = base->getNode( "value" )) != NULL ) {
     valueNode = NULL;
     value = n->getDoubleValue();
   } else {
@@ -76,33 +77,33 @@ static inline bool StartsWith( string & s, const char * cp )
   return s.find( cp ) == 0;
 }
 
-FGInputEvent * FGInputEvent::NewObject( FGInputDevice * device, SGPropertyNode_ptr node )
+FGInputEvent * FGInputEvent::NewObject( FGInputDevice * device, SGPropertyNode_ptr eventNode )
 {
-  string name = node->getStringValue( "name", "" );
+  string name = eventNode->getStringValue( "name", "" );
   if( StartsWith( name, "button-" ) )
-    return new FGButtonEvent( device, node );
+    return new FGButtonEvent( device, eventNode );
 
   if( StartsWith( name, "rel-" ) )
-    return new FGRelAxisEvent( device, node );
+    return new FGRelAxisEvent( device, eventNode );
 
   if( StartsWith( name, "abs-" ) )
-    return new FGAbsAxisEvent( device, node );
+    return new FGAbsAxisEvent( device, eventNode );
 
-  return new FGInputEvent( device, node );
+  return new FGInputEvent( device, eventNode );
 }
 
-FGInputEvent::FGInputEvent( FGInputDevice * aDevice, SGPropertyNode_ptr node ) :
+FGInputEvent::FGInputEvent( FGInputDevice * aDevice, SGPropertyNode_ptr eventNode ) :
   device( aDevice ),
-  lastDt(0.0), 
+  lastDt(0.0),
   lastSettingValue(std::numeric_limits<float>::quiet_NaN())
 {
-  name = node->getStringValue( "name", "" );
-  desc = node->getStringValue( "desc", "" );
-  intervalSec = node->getDoubleValue("interval-sec",0.0);
-  
-  read_bindings( node, bindings, KEYMOD_NONE, device->GetNasalModule() );
-    
-  for (auto child : node->getChildren("setting"))
+  name = eventNode->getStringValue( "name", "" );
+  desc = eventNode->getStringValue( "desc", "" );
+  intervalSec = eventNode->getDoubleValue("interval-sec",0.0);
+
+  read_bindings( eventNode, bindings, KEYMOD_NONE, device->GetNasalModule() );
+
+  for (auto child : eventNode->getChildren("setting"))
     settings.push_back( new FGEventSetting(child) );
 }
 
@@ -110,6 +111,7 @@ FGInputEvent::~FGInputEvent()
 {
 }
 
+// send changed value to device (if condition matches)
 void FGInputEvent::update( double dt )
 {
   for (auto setting : settings) {
@@ -135,24 +137,34 @@ void FGInputEvent::fire( FGEventData & eventData )
   }
 }
 
-void FGInputEvent::fire( SGBinding * binding, FGEventData & eventData )
+void FGInputEvent::fire(SGAbstractBinding* binding, FGEventData& eventData)
 {
   binding->fire();
 }
 
 
 
-FGAxisEvent::FGAxisEvent( FGInputDevice * device, SGPropertyNode_ptr node ) :
-  FGInputEvent( device, node )
+FGAxisEvent::FGAxisEvent( FGInputDevice * device, SGPropertyNode_ptr eventNode ) :
+  FGInputEvent( device, eventNode )
 {
-  tolerance = node->getDoubleValue("tolerance", 0.002);
-  minRange = node->getDoubleValue("min-range", 0.0 );
-  maxRange = node->getDoubleValue("max-range", 0.0 );
-  center = node->getDoubleValue("center", 0.0);
-  deadband = node->getDoubleValue("dead-band", 0.0);
-  lowThreshold = node->getDoubleValue("low-threshold", -0.9);
-  highThreshold = node->getDoubleValue("high-threshold", 0.9);
+  tolerance = eventNode->getDoubleValue("tolerance", 0.002);
+  minRange = eventNode->getDoubleValue("min-range", 0.0 );
+  maxRange = eventNode->getDoubleValue("max-range", 0.0 );
+  center = eventNode->getDoubleValue("center", 0.0);
+  deadband = eventNode->getDoubleValue("dead-band", 0.0);
+  lowThreshold = eventNode->getDoubleValue("low-threshold", -0.9);
+  highThreshold = eventNode->getDoubleValue("high-threshold", 0.9);
   lastValue = 9999999;
+
+// interpolation of values
+  if (eventNode->hasChild("interpolater")) {
+    interpolater.reset(new SGInterpTable{eventNode->getChild("interpolater")});
+    mirrorInterpolater = eventNode->getBoolValue("interpolater/mirrored", false);
+  }
+}
+
+FGAxisEvent::~FGAxisEvent()
+{
 }
 
 void FGAxisEvent::fire( FGEventData & eventData )
@@ -164,40 +176,49 @@ void FGAxisEvent::fire( FGEventData & eventData )
   // We need a copy of the  FGEventData struct to set the new value and to avoid side effects
   FGEventData ed = eventData;
 
+  if( minRange != maxRange )
+    ed.value = 2.0*(eventData.value-minRange)/(maxRange-minRange)-1.0;
+
   if( fabs(ed.value) < deadband )
     ed.value = 0.0;
 
-  if( minRange != maxRange )
-    ed.value = 2.0*(eventData.value-minRange)/(maxRange-minRange)-1.0;
+  if (interpolater) {
+    if ((ed.value < 0.0) && mirrorInterpolater) {
+      // mirror the positive interpolation for negative values
+      ed.value = -interpolater->interpolate(fabs(ed.value));
+    } else {
+      ed.value = interpolater->interpolate(ed.value);
+    }
+  }
 
   FGInputEvent::fire( ed );
 }
 
-void FGAbsAxisEvent::fire( SGBinding * binding, FGEventData & eventData )
+void FGAbsAxisEvent::fire(SGAbstractBinding* binding, FGEventData& eventData)
 {
   // sets the "setting" node
   binding->fire( eventData.value );
 }
 
-FGRelAxisEvent::FGRelAxisEvent( FGInputDevice * device, SGPropertyNode_ptr node ) : 
-  FGAxisEvent( device, node ) 
+FGRelAxisEvent::FGRelAxisEvent( FGInputDevice * device, SGPropertyNode_ptr eventNode ) :
+  FGAxisEvent( device, eventNode )
 {
   // relative axes can't use tolerance
   tolerance = 0.0;
 }
 
-void FGRelAxisEvent::fire( SGBinding * binding, FGEventData & eventData )
+void FGRelAxisEvent::fire(SGAbstractBinding* binding, FGEventData& eventData)
 {
   // sets the "offset" node
   binding->fire( eventData.value, 1.0 );
 }
 
-FGButtonEvent::FGButtonEvent( FGInputDevice * device, SGPropertyNode_ptr node ) :
-  FGInputEvent( device, node ),
+FGButtonEvent::FGButtonEvent( FGInputDevice * device, SGPropertyNode_ptr eventNode ) :
+  FGInputEvent( device, eventNode ),
   repeatable(false),
   lastState(false)
 {
-  repeatable = node->getBoolValue("repeatable", repeatable);
+  repeatable = eventNode->getBoolValue("repeatable", repeatable);
 }
 
 void FGButtonEvent::fire( FGEventData & eventData )
@@ -206,24 +227,33 @@ void FGButtonEvent::fire( FGEventData & eventData )
   if (pressed) {
     // The press event may be repeated.
     if (!lastState || repeatable) {
-      SG_LOG( SG_INPUT, SG_DEBUG, "Button has been pressed" );
+      SG_LOG( SG_INPUT, SG_DEBUG, "Button '" << this->name << "' has been pressed" );
       FGInputEvent::fire( eventData );
     }
   } else {
     // The release event is never repeated.
     if (lastState) {
-      SG_LOG( SG_INPUT, SG_DEBUG, "Button has been released" );
+      SG_LOG( SG_INPUT, SG_DEBUG, "Button '" << this->name << "' has been released" );
       eventData.modifiers|=KEYMOD_RELEASED;
       FGInputEvent::fire( eventData );
     }
   }
-          
+
   lastState = pressed;
 }
 
+void FGButtonEvent::update( double dt )
+{
+  if (repeatable && lastState) {
+      // interval / dt handling is done by base ::fire method
+      FGEventData ed{1.0, dt, 0 /* modifiers */};
+      FGInputEvent::fire( ed );
+  }
+}
+
 FGInputDevice::~FGInputDevice()
 {
-  FGNasalSys *nas = (FGNasalSys *)globals->get_subsystem("nasal");
+  auto nas = globals->get_subsystem<FGNasalSys>();
   if (nas && deviceNode ) {
     SGPropertyNode_ptr nasal = deviceNode->getNode("nasal");
     if( nasal ) {
@@ -235,43 +265,49 @@ FGInputDevice::~FGInputDevice()
     }
     nas->deleteModule(nasalModule.c_str());
   }
-} 
+}
 
 void FGInputDevice::Configure( SGPropertyNode_ptr aDeviceNode )
 {
   deviceNode = aDeviceNode;
 
-  nasalModule = string("__event:") + GetName();
+  // export our class_id to property tree
+  deviceNode->getNode("_class-id", true)->setValue(class_id.c_str());
 
-  PropertyList eventNodes = deviceNode->getChildren( "event" );
-  for( PropertyList::iterator it = eventNodes.begin(); it != eventNodes.end(); ++it )
-    AddHandledEvent( FGInputEvent::NewObject( this, *it ) );
+  SG_LOG(SG_INPUT, SG_DEBUG, "FGInputDevice::Configure");
+
+  // use _uniqueName here so each loaded device gets its own Nasal module
+  nasalModule = string("__event:") + _uniqueName;
+
+  for (auto ev : deviceNode->getChildren( "event" )) {
+    AddHandledEvent( FGInputEvent::NewObject( this, ev) );
+  }
 
   debugEvents = deviceNode->getBoolValue("debug-events", debugEvents );
   grab = deviceNode->getBoolValue("grab", grab );
 
-    PropertyList reportNodes = deviceNode->getChildren("report");
-    for( PropertyList::iterator it = reportNodes.begin(); it != reportNodes.end(); ++it ) {
-        FGReportSetting_ptr r = new FGReportSetting(*it);
-        reportSettings.push_back(r);
-    }
+  PropertyList reportNodes = deviceNode->getChildren("report");
+  for( PropertyList::iterator it = reportNodes.begin(); it != reportNodes.end(); ++it ) {
+      FGReportSetting_ptr r = new FGReportSetting(*it);
+      reportSettings.push_back(r);
+  }
 
-  // TODO:
-  // add nodes for the last event:
-  // last-event/name [string]
-  // last-event/value [double]
+
+  lastEventName = deviceNode->getNode("last-event", true)->getNode("name", true);
+  lastEventName->setStringValue("");
+  lastEventValue = deviceNode->getNode("last-event")->getNode("value", true);
+  lastEventValue->setDoubleValue(0.0);
 
   SGPropertyNode_ptr nasal = deviceNode->getNode("nasal");
   if (nasal) {
     SGPropertyNode_ptr open = nasal->getNode("open");
     if (open) {
       const string s = open->getStringValue();
-      FGNasalSys *nas = (FGNasalSys *)globals->get_subsystem("nasal");
+      auto nas = globals->get_subsystem<FGNasalSys>();
       if (nas)
         nas->createModule(nasalModule.c_str(), nasalModule.c_str(), s.c_str(), s.length(), deviceNode );
     }
   }
-
 }
 
 void FGInputDevice::AddHandledEvent( FGInputEvent_ptr event )
@@ -298,12 +334,13 @@ void FGInputDevice::update( double dt )
 
 void FGInputDevice::HandleEvent( FGEventData & eventData )
 {
-  string eventName = TranslateEventName( eventData );  
+  string eventName = TranslateEventName( eventData );
   if( debugEvents ) {
-    SG_LOG(SG_INPUT, SG_INFO, GetName() << " has event " <<
+    SG_LOG(SG_INPUT, SG_INFO, class_id << " " << GetUniqueName() << " has event " <<
            eventName << " modifiers=" << eventData.modifiers << " value=" << eventData.value);
   }
-    
+  lastEventName->setStringValue(eventName);
+  lastEventValue->setValue(eventData.value);
   if( handledEvents.count( eventName ) > 0 ) {
     handledEvents[ eventName ]->fire( eventData );
   }
@@ -311,7 +348,12 @@ void FGInputDevice::HandleEvent( FGEventData & eventData )
 
 void FGInputDevice::SetName( string name )
 {
-  this->name = name; 
+  this->name = name;
+}
+
+void FGInputDevice::SetUniqueName(const std::string &name)
+{
+  _uniqueName = name;
 }
 
 void FGInputDevice::SetSerialNumber( std::string serial )
@@ -325,10 +367,13 @@ void FGInputDevice::SendFeatureReport(unsigned int reportId, const std::string&
 }
 
 
-const char * FGEventInput::PROPERTY_ROOT = "/input/event";
+FGEventInput::FGEventInput()
+{
+}
 
-FGEventInput::FGEventInput() : 
-  configMap( "Input/Event", fgGetNode(PROPERTY_ROOT, true), "device-named")
+FGEventInput::FGEventInput(const char* filePath, const char* propertyRoot):
+  filePath(filePath),
+  propertyRoot(propertyRoot)
 {
 }
 
@@ -339,15 +384,17 @@ FGEventInput::~FGEventInput()
 
 void FGEventInput::shutdown()
 {
-    for (auto it : input_devices) {
-        it.second->Close();
-        delete it.second;
+    SG_LOG(SG_INPUT, SG_DEBUG, "FGEventInput::shutdown()");
+    auto tmp = inputDevices;
+    for (auto it : tmp) {
+      RemoveDevice(it.first);
     }
-    input_devices.clear();
+    inputDevices.clear();
 }
 
 void FGEventInput::init( )
 {
+  configMap = FGDeviceConfigurationMap(filePath, fgGetNode(propertyRoot, true), "device-named");
 }
 
 void FGEventInput::postinit ()
@@ -356,47 +403,74 @@ void FGEventInput::postinit ()
 
 void FGEventInput::update( double dt )
 {
-    for (auto it : input_devices) {
+    for (auto it : inputDevices) {
         it.second->update(dt);
     }
 }
 
+std::string FGEventInput::computeDeviceIndexName(FGInputDevice* dev) const
+{
+    int count = 0;
+    const auto devName = dev->GetName();
+    for (auto it : inputDevices) {
+        if (it.second->GetName() == devName) {
+            ++count;
+        }
+    }
+
+    std::ostringstream os;
+    os << devName << "_" << count;
+    return os.str();
+}
+
 unsigned FGEventInput::AddDevice( FGInputDevice * inputDevice )
 {
-  SGPropertyNode_ptr baseNode = fgGetNode( PROPERTY_ROOT, true );
-  SGPropertyNode_ptr deviceNode;
+  SGPropertyNode_ptr baseNode = fgGetNode( propertyRoot, true );
+  SGPropertyNode_ptr deviceNode = nullptr;
 
   const string deviceName = inputDevice->GetName();
-  SGPropertyNode_ptr configNode;
-  
+  SGPropertyNode_ptr configNode = nullptr;
+
+  // if we have a serial number set, try using that to select a specfic configuration
   if (!inputDevice->GetSerialNumber().empty()) {
     const string nameWithSerial = deviceName + "::" + inputDevice->GetSerialNumber();
     if (configMap.hasConfiguration(nameWithSerial)) {
       configNode = configMap.configurationForDeviceName(nameWithSerial);
+        SG_LOG(SG_INPUT, SG_INFO, "using instance-specific configuration for device "
+               << nameWithSerial << " : " << configNode->getStringValue("source"));
+      inputDevice->SetUniqueName(nameWithSerial);
     }
   }
-  
   if (configNode == nullptr) {
-    if (!configMap.hasConfiguration(deviceName)) {
-      SG_LOG(SG_INPUT, SG_DEBUG, "No configuration found for device " << deviceName <<
-             " (with serial: " << inputDevice->GetSerialNumber() << ")");
+    const auto nameWithIndex = computeDeviceIndexName(inputDevice);
+    // try instanced (counted) name
+    if (configMap.hasConfiguration(nameWithIndex)) {
+        configNode = configMap.configurationForDeviceName(nameWithIndex);
+        SG_LOG(SG_INPUT, SG_INFO, "using instance-specific configuration for device "
+                << nameWithIndex << " : " << configNode->getStringValue("source"));
+    }
+    // otherwise try the unmodifed name for the device
+    else if (configMap.hasConfiguration(deviceName)) {
+      configNode = configMap.configurationForDeviceName(deviceName);
+    }
+    else {
+      SG_LOG(SG_INPUT, SG_INFO, "No configuration found for device " << deviceName);
       delete inputDevice;
       return INVALID_DEVICE_INDEX;
     }
-    configNode = configMap.configurationForDeviceName(deviceName);
+    inputDevice->SetUniqueName(nameWithIndex);
   }
-    
-  // found - copy to /input/event/device[n]
 
+  // found - copy to /input/event/device[n]
   // find a free index
   unsigned int index;
-  for ( index = 0; index < MAX_DEVICES; index++ ) {
-    if ( (deviceNode = baseNode->getNode( "device", index, false ) ) == nullptr )
+  for (index = 0; index < MAX_DEVICES; index++) {
+    if ((deviceNode = baseNode->getNode("device", index, false)) == nullptr)
       break;
   }
 
   if (index == MAX_DEVICES) {
-    SG_LOG(SG_INPUT, SG_WARN, "To many event devices - ignoring " << inputDevice->GetName() );
+    SG_LOG(SG_INPUT, SG_WARN, "To many event devices - ignoring " << inputDevice->GetUniqueName() );
     delete inputDevice;
     return INVALID_DEVICE_INDEX;
   }
@@ -409,36 +483,33 @@ unsigned FGEventInput::AddDevice( FGInputDevice * inputDevice )
 
   inputDevice->Configure( deviceNode );
 
-  try {	
     bool ok = inputDevice->Open();
-      if (ok) {
-          input_devices[ deviceNode->getIndex() ] = inputDevice;
-      } else {
-          throw sg_exception("Opening input device failed");
-      }
-  }
-  catch( ... ) {
-    delete  inputDevice;
-    SG_LOG(SG_INPUT, SG_ALERT, "can't open InputDevice " << inputDevice->GetName()  );
-    return INVALID_DEVICE_INDEX;
-  }
+    if (!ok) {
+        // TODO repot a better error here, to the user
+        SG_LOG(SG_INPUT, SG_ALERT, "can't open InputDevice " << inputDevice->GetUniqueName());
+        delete inputDevice;
+        return INVALID_DEVICE_INDEX;
+    }
 
-  SG_LOG(SG_INPUT, SG_DEBUG, "using InputDevice " << inputDevice->GetName()  );
-  return deviceNode->getIndex();
+    inputDevices[deviceNode->getIndex()] = inputDevice;
+
+    SG_LOG(SG_INPUT, SG_INFO, inputDevice->class_id << "::AddDevice '" << inputDevice->GetUniqueName() << "' s/n: " << inputDevice->GetSerialNumber() );
+    return deviceNode->getIndex();
 }
 
 void FGEventInput::RemoveDevice( unsigned index )
 {
   // not fully implemented yet
-  SGPropertyNode_ptr baseNode = fgGetNode( PROPERTY_ROOT, true );
+  SGPropertyNode_ptr baseNode = fgGetNode( propertyRoot, true );
   SGPropertyNode_ptr deviceNode = NULL;
 
-  FGInputDevice *inputDevice = input_devices[index];
+  SG_LOG(SG_INPUT, SG_DEBUG, "FGEventInput::RemoveDevice(" << index << ") ");
+  FGInputDevice *inputDevice = inputDevices[index];
   if (inputDevice) {
+    SG_LOG(SG_INPUT, SG_DEBUG, "\tremoving (" << index << ") " << inputDevice->GetUniqueName());
     inputDevice->Close();
-    input_devices.erase(index);
+    inputDevices.erase(index);
     delete inputDevice;
-    
   }
   deviceNode = baseNode->removeChild("device", index);
 }
@@ -467,9 +538,9 @@ bool FGReportSetting::Test()
 
 std::string FGReportSetting::reportBytes(const std::string& moduleName) const
 {
-    FGNasalSys *nas = (FGNasalSys *)globals->get_subsystem("nasal");
+    auto nas = globals->get_subsystem<FGNasalSys>();
     if (!nas) {
-        return std::string();
+        return {};
     }
 
     naRef module = nas->getModule(moduleName.c_str());
@@ -477,20 +548,46 @@ std::string FGReportSetting::reportBytes(const std::string& moduleName) const
         SG_LOG(SG_IO, SG_WARN, "No such Nasal module:" << moduleName);
         return {};
     }
-    
+
     naRef func = naHash_cget(module, (char*) nasalFunction.c_str());
     if (!naIsFunc(func)) {
         return std::string();
     }
 
     naRef result = nas->call(func, 0, 0, naNil());
-    if (!naIsString(result)) {
-        return std::string();
+    if (naIsString(result)) {
+        size_t len = naStr_len(result);
+        char* bytes = naStr_data(result);
+        return std::string(bytes, len);
     }
 
-    size_t len = naStr_len(result);
-    char* bytes = naStr_data(result);
-    return std::string(bytes, len);
+    if (naIsVector(result)) {
+      int len = naVec_size(result);
+      std::string s;
+      for (int b=0; b < len; ++b) {
+        int num = naNumValue(naVec_get(result, b)).num;
+        s.push_back(static_cast<char>(num));
+      }
+
+      // can't access FGInputDevice here to check debugEvents flag
+#if 0
+      std::ostringstream byteString;
+      static const char* hexTable = "0123456789ABCDEF";
+
+        for (int i=0; i<s.size(); ++i) {
+            uint8_t uc = static_cast<uint8_t>(s[i]);
+            byteString << hexTable[uc >> 4];
+            byteString << hexTable[uc & 0x0f];
+            byteString << " ";
+        }
+        SG_LOG(SG_INPUT, SG_INFO, "report bytes: (" << s.size() << ") " << byteString.str());
+#endif
+
+      return s;
+    }
+
+    SG_LOG(SG_INPUT, SG_DEV_WARN, "bad return data from report setting");
+    return {};
 }
 
 void FGReportSetting::valueChanged(SGPropertyNode * node)
