@@ -26,6 +26,7 @@
 #include <cstdio>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <unistd.h>
 #include "FGLinuxEventInput.hxx"
 
 extern "C" {
@@ -38,6 +39,15 @@ extern "C" {
 
 #include <string.h>
 
+/*
+ * Handle old kernel headers (< v4.16) which don't define input_event_sec or
+ * input_event_usec macros (for use with struct input_event).
+ */
+#ifndef input_event_sec
+#define input_event_sec time.tv_sec
+#define input_event_usec time.tv_usec
+#endif
+
 struct TypeCode {
   unsigned type;
   unsigned code;
@@ -182,10 +192,10 @@ static struct EventTypes {
   { { EV_SW, SW_LID },               "switch-lid" },
   { { EV_SW, SW_TABLET_MODE },       "switch-tablet-mode" },
   { { EV_SW, SW_HEADPHONE_INSERT },  "switch-headphone-insert" },
-#ifdef SW_RFKILL_ALL 
+#ifdef SW_RFKILL_ALL
   { { EV_SW, SW_RFKILL_ALL },        "switch-rfkill" },
 #endif
-#ifdef SW_MICROPHONE_INSERT 
+#ifdef SW_MICROPHONE_INSERT
   { { EV_SW, SW_MICROPHONE_INSERT }, "switch-microphone-insert" },
 #endif
 #ifdef SW_DOCK
@@ -260,18 +270,21 @@ public:
 static EventTypeByName EVENT_TYPE_BY_NAME;
 
 
-FGLinuxInputDevice::FGLinuxInputDevice( std::string aName, std::string aDevname, std::string aSerial ) :
+FGLinuxInputDevice::FGLinuxInputDevice( std::string aName, std::string aDevname, std::string aSerial, std::string aDevpath ) :
   FGInputDevice(aName,aSerial),
-  devname( aDevname ),
+  devfile(aDevname),
+  devpath(aDevpath),
   fd(-1)
 {
+  class_id = "FGLinuxInputDevice";
+  SG_LOG(SG_INPUT, SG_DEBUG, "FGLinuxInputDevice " << aName << " (s/n: " << aSerial << ") " << aDevname);
 }
 
 FGLinuxInputDevice::~FGLinuxInputDevice()
 {
   try {
     Close();
-  } 
+  }
   catch(...) {
   }
 }
@@ -289,35 +302,40 @@ static inline bool bitSet( unsigned char * buf, unsigned bit )
 bool FGLinuxInputDevice::Open()
 {
   if( fd != -1 ) return true;
-  if( (fd = ::open( devname.c_str(), O_RDWR )) == -1 ) { 
-    throw std::exception();
+  if( (fd = ::open( devfile.c_str(), O_RDWR )) == -1 ) {
+    SG_LOG( SG_INPUT, SG_ALERT, "Cannot open '" << devfile
+            << "'. errno=" << errno
+            << ": " << strerror(errno)
+            );
+    //throw std::exception();
+    return false;
   }
 
   if( GetGrab() && ioctl( fd, EVIOCGRAB, 2 ) == -1 ) {
-    SG_LOG( SG_INPUT, SG_WARN, "Can't grab " << devname << " for exclusive access" );
+    SG_LOG( SG_INPUT, SG_WARN, "Can't grab " << devfile << " for exclusive access" );
   }
 
   {
     unsigned char buf[ABS_CNT/sizeof(unsigned char)/8];
     // get axes maximums
     if( ioctl( fd, EVIOCGBIT(EV_ABS,ABS_MAX), buf ) == -1 ) {
-      SG_LOG( SG_INPUT, SG_WARN, "Can't get abs-axes for " << devname );
+      SG_LOG( SG_INPUT, SG_WARN, "Can't get abs-axes for " << devfile );
     } else {
       for( unsigned i = 0; i < ABS_MAX; i++ ) {
         if( bitSet( buf, i ) ) {
           struct input_absinfo ai;
           if( ioctl(fd, EVIOCGABS(i), &ai) == -1 ) {
-            SG_LOG( SG_INPUT, SG_WARN, "Can't get abs-axes maximums for " << devname );
+            SG_LOG( SG_INPUT, SG_WARN, "Can't get abs-axes maximums for " << devfile );
             continue;
           }
           absinfo[i] = ai;
 /*
           SG_LOG( SG_INPUT, SG_INFO, "Axis #" << i <<
-            ": value=" << ai.value << 
-            ": minimum=" << ai.minimum << 
-            ": maximum=" << ai.maximum << 
-            ": fuzz=" << ai.fuzz << 
-            ": flat=" << ai.flat << 
+            ": value=" << ai.value <<
+            ": minimum=" << ai.minimum <<
+            ": maximum=" << ai.maximum <<
+            ": fuzz=" << ai.fuzz <<
+            ": flat=" << ai.flat <<
             ": resolution=" << ai.resolution );
 */
 
@@ -340,7 +358,7 @@ bool FGLinuxInputDevice::Open()
     memset(flag,0,sizeof(flag));
     if( ioctl( fd, EVIOCGKEY(sizeof(flag)), flag ) == -1 ||
         ioctl( fd, EVIOCGBIT(EV_KEY, sizeof(mask)), mask ) == -1 ) {
-      SG_LOG( SG_INPUT, SG_WARN, "Can't get keys for " << devname );
+      SG_LOG( SG_INPUT, SG_WARN, "Can't get keys for " << devfile );
     } else {
       for( unsigned i = 0; i < KEY_MAX; i++ ) {
         if( bitSet( mask, i ) ) {
@@ -357,7 +375,7 @@ bool FGLinuxInputDevice::Open()
   {
     unsigned char buf[SW_CNT/sizeof(unsigned char)/8];
     if( ioctl( fd, EVIOCGSW(sizeof(buf)), buf ) == -1 ) {
-      SG_LOG( SG_INPUT, SG_WARN, "Can't get switches for " << devname );
+      SG_LOG( SG_INPUT, SG_WARN, "Can't get switches for " << devfile );
     } else {
       for( unsigned i = 0; i < SW_MAX; i++ ) {
         if( bitSet( buf, i ) ) {
@@ -375,7 +393,7 @@ bool FGLinuxInputDevice::Open()
   return true;
 }
 
-double FGLinuxInputDevice::Normalize( struct input_event & event ) 
+double FGLinuxInputDevice::Normalize( struct input_event & event )
 {
   if( absinfo.count(event.code) > 0 ) {
      const struct input_absinfo & ai = absinfo[(unsigned int)event.code];
@@ -391,7 +409,7 @@ void FGLinuxInputDevice::Close()
 {
   if( fd != -1 ) {
     if( GetGrab() && ioctl( fd, EVIOCGRAB, 0 ) != 0 ) {
-      SG_LOG( SG_INPUT, SG_WARN, "Can't ungrab " << devname );
+      SG_LOG( SG_INPUT, SG_WARN, "Can't ungrab " << devfile );
     }
     ::close(fd);
   }
@@ -401,7 +419,7 @@ void FGLinuxInputDevice::Close()
 void FGLinuxInputDevice::Send( const char * eventName, double value )
 {
   if( EVENT_TYPE_BY_NAME.count( eventName ) <= 0 ) {
-    SG_LOG( SG_INPUT, SG_ALERT, "Can't send unknown event " << eventName );
+    SG_LOG( SG_INPUT, SG_DEBUG, "Can't send unknown event " << eventName );
     return;
   }
 
@@ -414,24 +432,20 @@ void FGLinuxInputDevice::Send( const char * eventName, double value )
   evt.type=typeCode.type;
   evt.code = typeCode.code;
   evt.value = (long)value;
-  evt.time.tv_sec = 0;
-  evt.time.tv_usec = 0;
+  evt.input_event_sec = 0;
+  evt.input_event_usec = 0;
   size_t bytes_written = write(fd, &evt, sizeof(evt));
-
+  lastEventName->setStringValue(eventName);
+  lastEventValue->setDoubleValue(value);
   if( bytes_written == sizeof(evt) )
-    SG_LOG( SG_INPUT,
-            SG_DEBUG,
-            "Written event " << eventName << " as type=" << evt.type
-                        << ", code=" << evt.code
-                        << " value=" << evt.value );
+    SG_LOG(SG_INPUT, SG_DEBUG, "Written event " << eventName << " as type=" << evt.type
+                        << ", code=" << evt.code << " value=" << evt.value);
   else
-    SG_LOG( SG_INPUT,
-            SG_WARN,
-            "Failed to write event: written = " << bytes_written );
+    SG_LOG(SG_INPUT, SG_WARN, "Failed to write event: written = " << bytes_written);
 }
 
 static char ugly_buffer[128];
-const char * FGLinuxInputDevice::TranslateEventName( FGEventData & eventData ) 
+const char * FGLinuxInputDevice::TranslateEventName( FGEventData & eventData )
 {
   FGLinuxEventData & linuxEventData = (FGLinuxEventData&)eventData;
   TypeCode typeCode;
@@ -453,10 +467,10 @@ const char * FGLinuxInputDevice::TranslateEventName( FGEventData & eventData )
 
 void FGLinuxInputDevice::SetDevname( const std::string & name )
 {
-  this->devname = name; 
+  this->devfile = name;
 }
 
-FGLinuxEventInput::FGLinuxEventInput()
+FGLinuxEventInput::FGLinuxEventInput() : FGEventInput("Input/Event", "/input/event")
 {
 }
 
@@ -470,29 +484,41 @@ void FGLinuxEventInput::postinit()
 
   struct udev * udev = udev_new();
 
+  // REVIEW: Memory Leak - 55,004 (40 direct, 54,964 indirect) bytes in 1 blocks are definitely lost
   struct udev_enumerate *enumerate = udev_enumerate_new(udev);
   udev_enumerate_add_match_subsystem(enumerate, "input");
   udev_enumerate_scan_devices(enumerate);
+  // REVIEW: Memory Leak - 4,995 (40 direct, 4,955 indirect) bytes in 1 blocks are definitely lost
   struct udev_list_entry *devices = udev_enumerate_get_list_entry(enumerate);
   struct udev_list_entry *dev_list_entry;
 
   udev_list_entry_foreach(dev_list_entry, devices) {
-    const char * path = udev_list_entry_get_name(dev_list_entry);
-    struct udev_device *dev = udev_device_new_from_syspath(udev, path);
-    const char * node = udev_device_get_devnode(dev);
- 
-    dev = udev_device_get_parent( dev );
-    const char * name = udev_device_get_sysattr_value(dev,"name");
-    const char * serial = udev_device_get_sysattr_value(dev, "serial");
-    SG_LOG(SG_INPUT,SG_DEBUG, "name=" << (name?name:"<null>") << ", node=" << (node?node:"<null>"));
-    if( name && node ) {
-      std::string serialString = serial ? serial : std::string{};
-      AddDevice( new FGLinuxInputDevice(name, node, serialString) );
+    const char * syspath = udev_list_entry_get_name(dev_list_entry);
+    struct udev_device *dev = udev_device_new_from_syspath(udev, syspath);
+    const char * devFile = udev_device_get_devnode(dev);
+
+    // get device path by removing trailing sysname from syspath
+    std::string sysname = udev_device_get_sysname(dev);
+    std::string devpath = syspath;
+    devpath.erase(devpath.length() - sysname.length());
+
+    struct udev_device * parent_dev = udev_device_get_parent( dev );
+    if ( parent_dev != NULL ) {
+      const char * name = udev_device_get_sysattr_value(parent_dev,"name");
+      if( name && devFile ) {
+      const char * serial = udev_device_get_sysattr_value(parent_dev, "serial");
+        SG_LOG(SG_INPUT,SG_DEBUG, "FGLinuxEventInput: path=" << devpath << ": " <<
+        " name=" << (name?name:"<null>") << ", devFile=" << (devFile?devFile:"<null>") );
+
+        std::string serialString = serial ? serial : std::string{};
+        AddDevice( new FGLinuxInputDevice(name, devFile, serialString, devpath) );
+      }
     }
 
     udev_device_unref(dev);
   }
 
+  udev_enumerate_unref(enumerate); // REVIEW: this should fix the memory leak
   udev_unref(udev);
 
 }
@@ -502,11 +528,11 @@ void FGLinuxEventInput::update( double dt )
   FGEventInput::update( dt );
   // index the input devices by the associated fd and prepare
   // the pollfd array by filling in the file descriptor
-  struct pollfd fds[input_devices.size()];
+  struct pollfd fds[inputDevices.size()];
   std::map<int,FGLinuxInputDevice*> devicesByFd;
   std::map<int,FGInputDevice*>::const_iterator it;
   int i;
-  for( i=0, it = input_devices.begin(); it != input_devices.end(); ++it, i++ ) {
+  for( i=0, it = inputDevices.begin(); it != inputDevices.end(); ++it, i++ ) {
     FGInputDevice* p = (*it).second;
     int fd = ((FGLinuxInputDevice*)p)->GetFd();
     fds[i].fd = fd;
@@ -539,4 +565,3 @@ void FGLinuxEventInput::update( double dt )
     }
   }
 }
-
