@@ -30,7 +30,6 @@
 
 #include "FGMouseInput.hxx"
 
-#include <boost/foreach.hpp>
 #include <osgGA/GUIEventAdapter>
 
 #include <simgear/scene/util/SGPickCallback.hxx>
@@ -40,6 +39,7 @@
 #include "FGButton.hxx"
 #include "Main/globals.hxx"
 #include <Viewer/renderer.hxx>
+#include <Viewer/sview.hxx>
 #include <Model/panelnode.hxx>
 #include <Cockpit/panel.hxx>
 #include <Viewer/FGEventHandler.hxx>
@@ -72,7 +72,7 @@ void ActivePickCallbacks::init( int button, const osgGA::GUIEventAdapter* ea )
 {
   osg::Vec2d windowPos;
   flightgear::eventToWindowCoords(ea, windowPos.x(), windowPos.y());
-    
+
   // Get the list of hit callbacks. Take the first callback that
   // accepts the mouse button press and ignore the rest of them
   // That is they get sorted by distance and by scenegraph depth.
@@ -165,33 +165,38 @@ public:
     {
         tooltipTimeoutDone = false;
         hoverPickScheduled = false;
-        tooltipsEnabled = false;
-        
+
         fgGetNode("/sim/mouse/hide-cursor", true )->addChangeListener(this, true);
         fgGetNode("/sim/mouse/cursor-timeout-sec", true )->addChangeListener(this, true);
         fgGetNode("/sim/mouse/right-button-mode-cycle-enabled", true)->addChangeListener(this, true);
         fgGetNode("/sim/mouse/tooltip-delay-msec", true)->addChangeListener(this, true);
         fgGetNode("/sim/mouse/click-shows-tooltip", true)->addChangeListener(this, true);
         fgGetNode("/sim/mouse/tooltips-enabled", true)->addChangeListener(this, true);
+        fgGetNode("/sim/mouse/tooltip-commands-registered", true)->addChangeListener(this, true);
         fgGetNode("/sim/mouse/drag-sensitivity", true)->addChangeListener(this, true);
         fgGetNode("/sim/mouse/invert-mouse-wheel", true)->addChangeListener(this, true);
     }
-  
+
+    bool areTooltipsEnabled() const
+    {
+        return _tooltipsEnabled && _tooltipsCommandsRegistered;
+    }
+
     void centerMouseCursor(mouse& m)
-    {    
+    {
       // center the cursor
       m.x = (xSizeNode ? xSizeNode->getIntValue() : 800) / 2;
       m.y = (ySizeNode ? ySizeNode->getIntValue() : 600) / 2;
       fgWarpMouse(m.x, m.y);
       haveWarped = true;
     }
-  
+
     void constrainMouse(int x, int y)
     {
         int new_x=x,new_y=y;
         int xsize = xSizeNode ? xSizeNode->getIntValue() : 800;
         int ysize = ySizeNode ? ySizeNode->getIntValue() : 600;
-        
+
         bool need_warp = false;
         if (x <= (xsize * .25) || x >= (xsize * .75)) {
           new_x = int(xsize * .5);
@@ -215,7 +220,7 @@ public:
       hoverPickScheduled = true;
       hoverPos = windowPos;
     }
-  
+
     void doHoverPick(const osg::Vec2d& windowPos)
     {
         FGMouseCursor::Cursor cur = FGMouseCursor::CURSOR_ARROW;
@@ -234,7 +239,7 @@ public:
                 explicitCursor = true;
                 cur = FGMouseCursor::cursorFromString(curName.c_str());
             }
-            
+
             // if the callback is of higher prioirty (lower enum index),
             // record that.
             if (i->callback->getPriority() < priority) {
@@ -255,29 +260,32 @@ public:
             i->callback->mouseLeave(windowPos);
         }
         _previous_picks = pickList;
-      
+
         if (!explicitCursor && (priority == SGPickCallback::PriorityPanel)) {
             cur = FGMouseCursor::CURSOR_HAND;
         }
-        
-        FGMouseCursor::instance()->setCursor(cur);
-        if (!didPick) {
-          SGPropertyNode_ptr args(new SGPropertyNode);
-          globals->get_commands()->execute("update-hover", args, nullptr);
 
+        FGMouseCursor::instance()->setCursor(cur);
+        if (!didPick && areTooltipsEnabled()) {
+            SGPropertyNode_ptr args(new SGPropertyNode);
+            globals->get_commands()->execute("update-hover", args, nullptr);
         }
     }
-    
+
     void doMouseMoveWithCallbacks(const osgGA::GUIEventAdapter* ea)
     {
         FGMouseCursor::Cursor cur = FGMouseCursor::CURSOR_CLOSED_HAND;
-        
-        osg::Vec2d windowPos;
-        flightgear::eventToWindowCoords(ea, windowPos.x(), windowPos.y());
 
-        SGSceneryPicks pickList = globals->get_renderer()->pick(windowPos);
-        if(pickList.empty())
-          return;
+        osg::Vec2d windowPos;
+        const bool ok = flightgear::eventToWindowCoords(ea, windowPos.x(), windowPos.y());
+        if (!ok) {
+            SG_LOG(SG_GUI, SG_WARN, "doMouseMoveWithCallbacks: ignoring mouse move with missing context/traits");
+            return;
+        }
+
+        // Do not compute scenery picks unless a callback requests it, as it is costly.
+        SGSceneryPicks pickList;
+        bool did_pick = false;
 
         for( ActivePickCallbacks::iterator mi = activePickCallbacks.begin();
                                            mi != activePickCallbacks.end();
@@ -286,6 +294,12 @@ public:
           SGPickCallbackList::iterator li;
           for( li = mi->second.begin(); li != mi->second.end(); ++li )
           {
+            if (!did_pick && (*li)->needsDragPosition())
+            {
+              pickList = globals->get_renderer()->pick(windowPos);
+              did_pick = true;
+            }
+
             const SGSceneryPick* pick = getPick(pickList, *li);
             (*li)->mouseMoved(*ea, pick ? &pick->info : 0);
 
@@ -299,7 +313,7 @@ public:
     }
 
     // implement the property-change-listener interfacee
-    virtual void valueChanged( SGPropertyNode * node )
+    void valueChanged(SGPropertyNode* node) override
     {
         if (node->getNameString() == "drag-sensitivity") {
             SGKnobAnimation::setDragSensitivity(node->getDoubleValue());
@@ -316,28 +330,32 @@ public:
         } else if (node->getNameString() == "click-shows-tooltip") {
             clickTriggersTooltip = node->getBoolValue();
         } else if (node->getNameString() == "tooltips-enabled") {
-            tooltipsEnabled = node->getBoolValue();
+            _tooltipsEnabled = node->getBoolValue();
+        } else if (node->getNameString() == "tooltip-commands-registered") {
+            _tooltipsCommandsRegistered = node->getBoolValue();
         }
     }
-    
+
     ActivePickCallbacks activePickCallbacks;
     SGSceneryPicks _previous_picks;
 
     mouse mice[MAX_MICE];
-    
+
     bool hideCursor, haveWarped;
     bool tooltipTimeoutDone;
     bool clickTriggersTooltip;
     int tooltipDelayMsec, cursorTimeoutMsec;
     bool rightClickModeCycle;
-    bool tooltipsEnabled;
-    
+
+    bool _tooltipsEnabled = false;
+    bool _tooltipsCommandsRegistered = false; ///< avoid errors if the mouse moves before Nasal init
+
     SGPropertyNode_ptr xSizeNode;
     SGPropertyNode_ptr ySizeNode;
     SGPropertyNode_ptr xAccelNode;
     SGPropertyNode_ptr yAccelNode;
     SGPropertyNode_ptr mouseXNode, mouseYNode;
-  
+
     bool hoverPickScheduled;
     osg::Vec2d hoverPos;
 };
@@ -408,8 +426,8 @@ void FGMouseInput::init()
       SGPropertyNode * mode_node = mouse_node->getChild("mode", j, true);
 
     // Read the mouse cursor for this mode
-      m.modes[j].cursor = FGMouseCursor::cursorFromString(mode_node->getStringValue("cursor", "inherit"));
-        
+      m.modes[j].cursor = FGMouseCursor::cursorFromString(mode_node->getStringValue("cursor", "inherit").c_str());
+
       // Read other properties for this mode
       m.modes[j].constrained = mode_node->getBoolValue("constrained", false);
       m.modes[j].pass_through = mode_node->getBoolValue("pass-through", false);
@@ -425,7 +443,7 @@ void FGMouseInput::init()
       // Read the axis bindings for this mode
       read_bindings(mode_node->getChild("x-axis", 0, true), m.modes[j].x_bindings, KEYMOD_NONE, module );
       read_bindings(mode_node->getChild("y-axis", 0, true), m.modes[j].y_bindings, KEYMOD_NONE, module );
-      
+
       if (mode_node->hasChild("x-axis-ctrl")) {
         read_bindings(mode_node->getChild("x-axis-ctrl"), m.modes[j].x_bindings, KEYMOD_CTRL, module );
       }
@@ -435,7 +453,7 @@ void FGMouseInput::init()
       if (mode_node->hasChild("x-axis-ctrl-shift")) {
         read_bindings(mode_node->getChild("x-axis-ctrl-shift"), m.modes[j].x_bindings, KEYMOD_CTRL|KEYMOD_SHIFT, module );
       }
-      
+
       if (mode_node->hasChild("y-axis-ctrl")) {
         read_bindings(mode_node->getChild("y-axis-ctrl"), m.modes[j].y_bindings, KEYMOD_CTRL, module );
       }
@@ -474,14 +492,14 @@ void FGMouseInput::update ( double dt )
     if (!d) {
         SG_LOG(SG_INPUT, SG_WARN, "update of mouse before init");
     }
-    
+
   mouse &m = d->mice[0];
   int mode =  m.mode_node->getIntValue();
   if (mode != m.current_mode) {
     // current mode has changed
     m.current_mode = mode;
     m.timeSinceLastMove.stamp();
-      
+
     if (mode >= 0 && mode < m.nModes) {
       FGMouseCursor::instance()->setCursor(m.modes[mode].cursor);
       d->centerMouseCursor(m);
@@ -495,23 +513,22 @@ void FGMouseInput::update ( double dt )
     d->doHoverPick(d->hoverPos);
     d->hoverPickScheduled = false;
   }
-  
-  if ( !d->tooltipTimeoutDone &&
-      d->tooltipsEnabled &&
-      (m.timeSinceLastMove.elapsedMSec() > d->tooltipDelayMsec))
-  {
+
+  if (!d->tooltipTimeoutDone &&
+      d->areTooltipsEnabled() &&
+      (m.timeSinceLastMove.elapsedMSec() > d->tooltipDelayMsec)) {
       d->tooltipTimeoutDone = true;
       SGPropertyNode_ptr arg(new SGPropertyNode);
       globals->get_commands()->execute("tooltip-timeout", arg, nullptr);
   }
-  
+
   if ( d->hideCursor ) {
       if ( m.timeSinceLastMove.elapsedMSec() > d->cursorTimeoutMsec) {
           FGMouseCursor::instance()->hideCursorUntilMouseMove();
           m.timeSinceLastMove.stamp();
       }
   }
-    
+
   d->activePickCallbacks.update( dt, fgGetKeyModifiers() );
 }
 
@@ -538,7 +555,7 @@ void FGMouseInput::doMouseClick (int b, int updown, int x, int y, bool mainWindo
         // can occur during reset
         return;
     }
-    
+
   int modifiers = fgGetKeyModifiers();
 
   mouse &m = d->mice[0];
@@ -551,17 +568,29 @@ void FGMouseInput::doMouseClick (int b, int updown, int x, int y, bool mainWindo
     // in spring-loaded look mode, ignore right clicks entirely here
     return;
   }
-  
+
+
   // Pass on to PUI and the panel if
   // requested, and return if one of
   // them consumes the event.
 
   osg::Vec2d windowPos;
-  flightgear::eventToWindowCoords(ea, windowPos.x(), windowPos.y());
+  bool ok = flightgear::eventToWindowCoords(ea, windowPos.x(), windowPos.y());
+  if (!ok) {
+      SG_LOG(SG_GUI, SG_WARN, "Ignoring mouse click with null context/traits");
+      return;
+  }
 
-  SGSceneryPicks pickList = globals->get_renderer()->pick(windowPos);
+  SGSceneryPicks pickList;
 
-  if( updown != MOUSE_BUTTON_DOWN )
+  if (isRightDragLookActive() && (updown == MOUSE_BUTTON_DOWN)) {
+      // when spring-loaded mode is active, don't do scene selection for picks
+      // https://sourceforge.net/p/flightgear/codetickets/2108/
+  } else {
+      pickList = globals->get_renderer()->pick(windowPos);
+  }
+
+  if( updown == MOUSE_BUTTON_UP )
   {
     // Execute the mouse up event in any case, may be we should
     // stop processing here?
@@ -572,10 +601,16 @@ void FGMouseInput::doMouseClick (int b, int updown, int x, int y, bool mainWindo
     {
       SGPickCallbackPtr& cb = callbacks.front();
       const SGSceneryPick* pick = getPick(pickList, cb);
-      cb->buttonReleased(ea->getModKeyMask(), *ea, pick ? &pick->info : 0);
+      cb->buttonReleased(ea->getModKeyMask(), *ea, pick ? &pick->info : nullptr);
 
       callbacks.pop_front();
     }
+
+    if (ea->getHandled()) {
+        // for https://sourceforge.net/p/flightgear/codetickets/2347/
+        // we cleared the active picks, but don't do further processing
+        return;
+    }
   }
 
   if (mode.pass_through) {
@@ -583,12 +618,12 @@ void FGMouseInput::doMouseClick (int b, int updown, int x, int y, bool mainWindo
     // scenegraph intersection point corresponding to the mouse click
     if (updown == MOUSE_BUTTON_DOWN) {
       d->activePickCallbacks.init( b, ea );
-        
-      if (d->clickTriggersTooltip) {
-            SGPropertyNode_ptr args(new SGPropertyNode);
-            args->setStringValue("reason", "click");
-            globals->get_commands()->execute("tooltip-timeout", args, nullptr);
-            d->tooltipTimeoutDone = true;
+
+      if (d->clickTriggersTooltip && d->areTooltipsEnabled()) {
+          SGPropertyNode_ptr args(new SGPropertyNode);
+          args->setStringValue("reason", "click");
+          globals->get_commands()->execute("tooltip-timeout", args, nullptr);
+          d->tooltipTimeoutDone = true;
       }
     } else {
       // do a hover pick now, to fix up cursor
@@ -602,7 +637,7 @@ void FGMouseInput::doMouseClick (int b, int updown, int x, int y, bool mainWindo
     return;
   }
 
-  m.modes[m.current_mode].buttons[b].update( modifiers, 0 != updown, x, y);  
+  m.modes[m.current_mode].buttons[b].update( modifiers, 0 != updown, x, y);
 }
 
 void FGMouseInput::processMotion(int x, int y, const osgGA::GUIEventAdapter* ea)
@@ -611,27 +646,34 @@ void FGMouseInput::processMotion(int x, int y, const osgGA::GUIEventAdapter* ea)
     d->doMouseMoveWithCallbacks(ea);
     return;
   }
-  
+
+  if (SviewMouseMotion(x, y, *ea)) {
+    return;
+  }
+
   mouse &m = d->mice[0];
   int modeIndex = m.current_mode;
-  // are we in spring-loaded look mode?
-  if (!d->rightClickModeCycle && m.nModes > 3) {
-    if (m.mouse_button_nodes[2]->getBoolValue()) {
+
+  if (isRightDragLookActive()) {
       // right mouse is down, force look mode
       modeIndex = 3;
-    }
   }
 
   if (modeIndex == 0) {
     osg::Vec2d windowPos;
     flightgear::eventToWindowCoords(ea, windowPos.x(), windowPos.y());
-    d->scheduleHoverPick(windowPos);
+
+    // omly do hover picks if no buttons are down
+    if (ea->getButtonMask() == 0) {
+        d->scheduleHoverPick(windowPos);
+    }
+
     // mouse has moved, so we may need to issue tooltip-timeout command again
     d->tooltipTimeoutDone = false;
   }
-  
+
   mouse_mode &mode = m.modes[modeIndex];
- 
+
   if (d->haveWarped)
   {
     // don't fire mouse-movement events at the first update after warping the mouse,
@@ -643,7 +685,7 @@ void FGMouseInput::processMotion(int x, int y, const osgGA::GUIEventAdapter* ea)
     int modifiers = fgGetKeyModifiers();
     int xsize = d->xSizeNode ? d->xSizeNode->getIntValue() : 800;
     int ysize = d->ySizeNode ? d->ySizeNode->getIntValue() : 600;
-      
+
     // OK, PUI didn't want the event,
     // so we can play with it.
     if (x != m.x) {
@@ -659,7 +701,7 @@ void FGMouseInput::processMotion(int x, int y, const osgGA::GUIEventAdapter* ea)
         mode.y_bindings[modifiers][i]->fire(double(delta), double(ysize));
     }
   }
-  
+
   // Constrain the mouse if requested
   if (mode.constrained) {
     d->constrainMouse(x, y);
@@ -672,7 +714,7 @@ void FGMouseInput::doMouseMotion (int x, int y, const osgGA::GUIEventAdapter* ea
         // can occur during reset
         return;
     }
-    
+
   mouse &m = d->mice[0];
 
   if (m.current_mode < 0 || m.current_mode >= m.nModes) {
@@ -685,11 +727,11 @@ void FGMouseInput::doMouseMotion (int x, int y, const osgGA::GUIEventAdapter* ea
   FGMouseCursor::instance()->mouseMoved();
 
   // TODO Get rid of this as soon as soon as cursor hide timeout works globally
-  if( ea->getHandled() )
-    return;
 
-  processMotion(x, y, ea);
-    
+    if (!ea->getHandled()) {
+        processMotion(x, y, ea);
+    }
+
   m.x = x;
   m.y = y;
   d->mouseXNode->setIntValue(x);
@@ -710,12 +752,30 @@ bool FGMouseInput::isActiveModePassThrough() const
     if (!d) {
         return false;
     }
-    
+
     mouse &m = d->mice[0];
     int mode = m.current_mode;
     if (isRightDragToLookEnabled() && m.mouse_button_nodes[2]->getBoolValue()) {
         mode = 3;
     }
-    
+
     return m.modes[mode].pass_through;
 }
+
+bool FGMouseInput::isRightDragLookActive() const
+{
+    if (!d) {
+        return false;
+    }
+
+    const auto& m = d->mice[0];
+    if (!d->rightClickModeCycle && m.nModes > 3) {
+        return m.mouse_button_nodes[2]->getBoolValue();
+    }
+
+    return false;
+}
+
+
+// Register the subsystem.
+SGSubsystemMgr::Registrant<FGMouseInput> registrantFGMouseInput;
