@@ -34,7 +34,7 @@
 #include <FDM/flight.hxx>
 #include <Environment/environment.hxx>
 #include <Environment/environment_mgr.hxx>
-#include <Viewer/viewer.hxx>
+#include <Viewer/view.hxx>
 #include <Viewer/viewmgr.hxx>
 #include <Airports/airport.hxx>
 
@@ -87,11 +87,11 @@ void HUD::Runway::draw()
 //    double po = curr_view->getPitchOffset_deg();
 //    double ho = curr_view->getHeadingOffset_deg();
 
-    FGViewer* cockpitView = globals->get_viewmgr()->get_view(0);
+    flightgear::View* cockpitView = globals->get_viewmgr()->get_view(0);
     
     double yaw = -(cockpitView->getHeadingOffset_deg() - _default_heading) * SG_DEGREES_TO_RADIANS;
     double pitch = (cockpitView->getPitchOffset_deg() - _default_pitch) * SG_DEGREES_TO_RADIANS;
-    //double roll = fgGetDouble("/sim/view[0]/config/roll-offset-deg",0.0) //TODO: adjust for default roll offset
+    //double roll = fgGetDouble("/sim/view[0]/config/roll-offset-deg",0.0) // TODO: adjust for default roll offset
     double sPitch = sin(pitch), cPitch = cos(pitch),
            sYaw = sin(yaw), cYaw = cos(yaw);
 
@@ -136,10 +136,10 @@ void HUD::Runway::draw()
     glEnable(GL_LINE_STIPPLE);
     glLineStipple(1, _stipple_out);
     anyLines =
-            drawLine(_points3d[0], _points3d[1], _points2d[0], _points2d[1]) | //draw top
-            drawLine(_points3d[2], _points3d[1], _points2d[2], _points2d[1]) | //draw right
-            drawLine(_points3d[2], _points3d[3], _points2d[2], _points2d[3]) | //draw bottom
-            drawLine(_points3d[3], _points3d[0], _points2d[3], _points2d[0]);  //draw left
+        drawLine(_points3d[0], _points3d[1], _points2d[0], _points2d[1]) || //draw top
+        drawLine(_points3d[2], _points3d[1], _points2d[2], _points2d[1]) || //draw right
+        drawLine(_points3d[2], _points3d[3], _points2d[2], _points2d[3]) || //draw bottom
+        drawLine(_points3d[3], _points3d[0], _points2d[3], _points2d[0]);   //draw left
 
     glLineStipple(1, _stipple_center);
     anyLines |= drawLine(_points3d[5], _points3d[4], _points2d[5], _points2d[4]); //draw center
