@@ -27,7 +27,6 @@
 #include "newnavradio.hxx"
 
 #include <assert.h>
-#include <boost/foreach.hpp>
 
 #include <simgear/math/interpolater.hxx>
 #include <simgear/sg_inlines.h>
@@ -44,6 +43,7 @@
 
 namespace Instrumentation {
 
+using namespace std::string_literals;
 using simgear::PropertyObject;
 
 /* --------------The Navigation Indicator ----------------------------- */
@@ -144,7 +144,7 @@ public:
   virtual void   search( double frequency, const SGGeod & aircraftPosition );
   virtual double getRange_nm( const SGGeod & aircraftPosition );
   virtual void   display( NavIndicator & navIndicator ) = 0;
-  virtual bool   valid() const { return NULL != _navRecord && true == _serviceable; }
+  virtual bool   valid() const { return NULL != _navRecord && _serviceable; }
   virtual const std::string getIdent() const { return _ident; }
 
 protected:
@@ -218,7 +218,7 @@ void NavRadioComponentWithIdent::update( double dt, const SGGeod & aircraftPosit
   NavRadioComponent::update( dt, aircraftPosition );
   _audioIdent->update( dt );
 
-  if( false == ( valid() && _identEnabled && _signalQuality_norm > 0.1 ) ) {
+  if( !( valid() && _identEnabled && _signalQuality_norm > 0.1 ) ) {
       _audioIdent->setIdent("", 0.0 );
       return;
   }
@@ -273,7 +273,7 @@ void NavRadioComponent::search( double frequency, const SGGeod & aircraftPositio
 
 double NavRadioComponent::computeSignalQuality_norm( const SGGeod & aircraftPosition )
 {
-  if( false == valid() ) return 0.0;
+  if( !valid() ) return 0.0;
 
   double distance_nm = _slantDistance_m * SG_METER_TO_NM;
   double range_nm = _range_nm;
@@ -286,7 +286,7 @@ double NavRadioComponent::computeSignalQuality_norm( const SGGeod & aircraftPosi
 
 void NavRadioComponent::update( double dt, const SGGeod & aircraftPosition )
 {
-    if( false == valid() ) {
+    if( !valid() ) {
       _signalQuality_norm = 0.0;
       _trueBearingTo_deg = 0.0;
       _trueBearingFrom_deg = 0.0;
@@ -381,7 +381,7 @@ double VOR::ServiceVolume::adjustRange( double height_ft, double nominalRange_nm
 
 VOR::VOR( SGPropertyNode_ptr rootNode) :
     NavRadioComponentWithIdent("vor", rootNode,
-                               new VORAudioIdent(getIdentString(std::string("vor"),
+                               new VORAudioIdent(getIdentString("vor"s,
                                                                 rootNode->getIndex()))),
   _totalTime(0.0),
   _radial( rootNode->getNode(_name,true)->getNode("radial",true) ),
@@ -426,7 +426,7 @@ void VOR::update( double dt, const SGGeod & aircraftPosition )
   _totalTime += dt;
   NavRadioComponentWithIdent::update( dt, aircraftPosition );
 
-  if( false == valid() ) {
+  if( !valid() ) {
       _radial = 0.0;
       return;
   }
@@ -448,7 +448,7 @@ void VOR::update( double dt, const SGGeod & aircraftPosition )
 
 void VOR::display( NavIndicator & navIndicator )
 {
-  if( false == valid() ) return;
+  if( !valid() ) return;
 
   double offset = SGMiscd::normalizePeriodic(-180.0,180.0,_radial - navIndicator.getSelectedCourse());
   bool to = fabs(offset) >= 90.0;
@@ -528,7 +528,7 @@ double LOC::ServiceVolume::adjustRange( double azimuthAngle_deg, double elevatio
 }
 
 LOC::LOC( SGPropertyNode_ptr rootNode) :
-    NavRadioComponentWithIdent("loc", rootNode, new LOCAudioIdent(getIdentString(std::string("loc"),
+    NavRadioComponentWithIdent("loc", rootNode, new LOCAudioIdent(getIdentString("loc"s,
                                                                                  rootNode->getIndex()))),
   _serviceVolume(),
   _localizerOffset_norm( rootNode->getNode(_name,true)->getNode("offset-norm",true) ),
@@ -549,7 +549,7 @@ FGNavList::TypeFilter* LOC::getNavaidFilter()
 void LOC::search( double frequency, const SGGeod & aircraftPosition )
 {
   NavRadioComponentWithIdent::search( frequency, aircraftPosition );
-  if( false == valid() ) {
+  if( !valid() ) {
       _localizerWidth_deg = 0.0;
       return;
   }
@@ -587,7 +587,7 @@ void LOC::update( double dt, const SGGeod & aircraftPosition )
 {
   NavRadioComponentWithIdent::update( dt, aircraftPosition );
 
-  if( false == valid() ) {
+  if( !valid() ) {
     _localizerOffset_norm = 0.0;
     _localizerOffset_m = 0.0;
     return;
@@ -611,7 +611,7 @@ void LOC::update( double dt, const SGGeod & aircraftPosition )
 
 void LOC::display( NavIndicator & navIndicator )
 {
-  if( false == valid() ) 
+  if( !valid() ) 
     return;
 
   navIndicator.showTo( true );
@@ -738,7 +738,7 @@ SGVec3d GS::tangentVector(const SGGeod& midpoint, const double heading)
 void GS::search( double frequency, const SGGeod & aircraftPosition )
 {
   NavRadioComponent::search( frequency, aircraftPosition );
-  if( false == valid() ) {
+  if( !valid() ) {
       _gsAxis = SGVec3d::zeros();
       _gsVertical = SGVec3d::zeros();
       _targetGlideslope_deg = 3.0;
@@ -759,7 +759,7 @@ void GS::search( double frequency, const SGGeod & aircraftPosition )
 void GS::update( double dt, const SGGeod & aircraftPosition )
 {
   NavRadioComponent::update( dt, aircraftPosition );
-  if( false == valid() ) {
+  if( !valid() ) {
       _glideslopeOffset_norm = 0.0;
       return;
   }
@@ -785,7 +785,7 @@ void GS::update( double dt, const SGGeod & aircraftPosition )
   double offset = _targetGlideslope_deg - gsDirect;
   if( offset < 0.0 )
     offset = _targetGlideslope_deg/2 * sawtooth(2.0*offset/_targetGlideslope_deg);
-    assert( false == SGMisc<double>::isNaN(offset) );
+  assert( !SGMisc<double>::isNaN(offset) );
 // GS is documented to be 1.4 degrees thick, 
 // i.e. plus or minus 0.7 degrees from the midline:
   _glideslopeOffset_norm = SGMiscd::clip(offset/0.7, -1.0, 1.0);
@@ -793,7 +793,7 @@ void GS::update( double dt, const SGGeod & aircraftPosition )
 
 void GS::display( NavIndicator & navIndicator )
 {
-  if( false == valid() ) {
+  if( !valid() ) {
     navIndicator.setGS( false );
     return;
   }
@@ -803,55 +803,59 @@ void GS::display( NavIndicator & navIndicator )
 
 /* ------------- The NavRadio implementation ---------------------- */
 
-class NavRadioImpl : public NavRadio {
+class NavRadioImpl : public NavRadio
+{
 public:
-  NavRadioImpl( SGPropertyNode_ptr node );
-  virtual ~NavRadioImpl();
+    NavRadioImpl( SGPropertyNode_ptr node );
+    virtual ~NavRadioImpl();
+
+    // Subsystem API.
+    void init() override;
+    void update(double dt) override;
 
-  virtual void update( double dt );
-  virtual void init();
 private:
-  void search();
+    void search();
 
-  class Legacy {
-  public:
-      Legacy( NavRadioImpl * navRadioImpl ) : _navRadioImpl( navRadioImpl ) {}
+    class Legacy {
+    public:
+        Legacy( NavRadioImpl * navRadioImpl ) : _navRadioImpl( navRadioImpl ) {}
 
-      void init();
-      void update( double dt );
-  private:
-      NavRadioImpl * _navRadioImpl;
-      SGPropertyNode_ptr is_valid_node;
-      SGPropertyNode_ptr nav_serviceable_node;
-      SGPropertyNode_ptr nav_id_node;
-      SGPropertyNode_ptr id_c1_node;
-      SGPropertyNode_ptr id_c2_node;
-      SGPropertyNode_ptr id_c3_node;
-      SGPropertyNode_ptr id_c4_node;
-  } _legacy;
+        void init();
+        void update( double dt );
 
-  const static int VOR_COMPONENT = 0;
-  const static int LOC_COMPONENT = 1;
-  const static int GS_COMPONENT  = 2;
+    private:
+        NavRadioImpl * _navRadioImpl;
+        SGPropertyNode_ptr is_valid_node;
+        SGPropertyNode_ptr nav_serviceable_node;
+        SGPropertyNode_ptr nav_id_node;
+        SGPropertyNode_ptr id_c1_node;
+        SGPropertyNode_ptr id_c2_node;
+        SGPropertyNode_ptr id_c3_node;
+        SGPropertyNode_ptr id_c4_node;
+    } _legacy;
 
-  std::string _name;
-  int         _num;
-  SGPropertyNode_ptr _rootNode;
-  FrequencyFormatter _useFrequencyFormatter;
-  FrequencyFormatter _stbyFrequencyFormatter;
-  std::vector<NavRadioComponent*> _components;
-  NavIndicator _navIndicator;
-  double _stationTTL;
-  double _frequency;
-  PropertyObject<bool> _cdiDisconnected;
-  PropertyObject<std::string> _navType;
+    const static int VOR_COMPONENT = 0;
+    const static int LOC_COMPONENT = 1;
+    const static int GS_COMPONENT  = 2;
+
+    std::string _name;
+    int         _num;
+    SGPropertyNode_ptr _rootNode;
+    FrequencyFormatter _useFrequencyFormatter;
+    FrequencyFormatter _stbyFrequencyFormatter;
+    std::vector<NavRadioComponent*> _components;
+    NavIndicator _navIndicator;
+    double _stationTTL;
+    double _frequency;
+    PropertyObject<bool> _cdiDisconnected;
+    PropertyObject<std::string> _navType;
 };
 
 NavRadioImpl::NavRadioImpl( SGPropertyNode_ptr node ) :
   _legacy( this ),
   _name(node->getStringValue("name", "nav")),
   _num(node->getIntValue("number", 0)),
-  _rootNode(fgGetNode( std::string("/instrumentation/") + _name, _num, true)),
+  _rootNode(fgGetNode( "/instrumentation/"s + _name, _num, true)),
   _useFrequencyFormatter( _rootNode->getNode("frequencies/selected-mhz",true), _rootNode->getNode("frequencies/selected-mhz-fmt",true), 0.05, 108.0, 118.0 ),
   _stbyFrequencyFormatter( _rootNode->getNode("frequencies/standby-mhz",true), _rootNode->getNode("frequencies/standby-mhz-fmt",true), 0.05, 108.0, 118.0 ),
   _navIndicator(_rootNode),
@@ -864,7 +868,7 @@ NavRadioImpl::NavRadioImpl( SGPropertyNode_ptr node ) :
 
 NavRadioImpl::~NavRadioImpl()
 {
-  BOOST_FOREACH( NavRadioComponent * p, _components ) {
+  for( auto p : _components ) {
     delete p;
   }
 }
@@ -904,12 +908,12 @@ void NavRadioImpl::update( double dt )
       _stationTTL = 0.0;
   }
 
-  BOOST_FOREACH( NavRadioComponent * p, _components ) {
+  for( auto p : _components ) {
       if( _stationTTL <= 0.0 )
           p->search( _frequency, position );
       p->update( dt, position );
 
-      if( false == _cdiDisconnected )
+      if( !_cdiDisconnected )
           p->display( _navIndicator );
   }
 
@@ -969,5 +973,13 @@ SGSubsystem * NavRadio::createInstance( SGPropertyNode_ptr rootNode )
     return new FGNavRadio( rootNode );
 }
 
+
+// Register the subsystem.
+#if 0
+SGSubsystemMgr::InstancedRegistrant<NavRadio> registrantNavRadio(
+    SGSubsystemMgr::FDM,
+    {{"instrumentation", SGSubsystemMgr::Dependency::HARD}});
+#endif
+
 } // namespace Instrumentation
 
