@@ -60,9 +60,9 @@
 /* Module properties:
  *
  *   serviceable             enable/disable TCAS processing
- *   
+ *
  *   voice/file-prefix       path (and optional prefix) for sound sample files
- *                           (only evaluated at start-up)  
+ *                           (only evaluated at start-up)
  *
  *   inputs/mode             TCAS mode selection: 0=off,1=standby,2=TA only,3=auto(TA/RA)
  *   inputs/self-test        trigger self-test sequence
@@ -84,9 +84,7 @@
 #  pragma warning( disable: 4355 )
 #endif
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -106,8 +104,6 @@
 
 using std::string;
 
-#include <Include/version.h>
-
 ///////////////////////////////////////////////////////////////////////////////
 // debug switches /////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////
@@ -127,12 +123,12 @@ using std::string;
 // constants //////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////
 
-/** Sensitivity Level Definition and Alarm Thresholds (TCASII, Version 7) 
+/** Sensitivity Level Definition and Alarm Thresholds (TCASII, Version 7)
  *  Max Own        |    |TA-level                |RA-level
  *  Altitude(ft)   | SL |Tau(s),DMOD(nm),ALIM(ft)|Tau(s),DMOD(nm),ALIM(ft) */
 const TCAS::SensitivityLevel
 TCAS::ThreatDetector::sensitivityLevels[] = {
-   { 1000,           2,  {20,   0.30,     850},  {0,    0,        0  }}, 
+   { 1000,           2,  {20,   0.30,     850},  {0,    0,        0  }},
    { 2350,           3,  {25,   0.33,     850},  {15,   0.20,     300}},
    { 5000,           4,  {30,   0.48,     850},  {20,   0.35,     300}},
    {10000,           5,  {40,   0.75,     850},  {25,   0.55,     350}},
@@ -169,7 +165,7 @@ relAngle(float Heading1, float Heading2)
 }
 #endif
 
-/** calculate range and bearing of lat2/lon2 relative to lat1/lon1 */ 
+/** calculate range and bearing of lat2/lon2 relative to lat1/lon1 */
 static void
 calcRangeBearing(double lat1, double lon1, double lat2, double lon2,
                  double &rangeNm, double &bearing)
@@ -247,7 +243,7 @@ void
 TCAS::Annunciator::update(void)
 {
     voicePlayer.update();
-    
+
     /* [TCASII]: "The priority scheme gives ground proximity warning systems (GPWS)
      *   a higher annunciation priority than a TCAS alert. TCAS aural annunciation will
      *   be inhibited during the time that a GPWS alert is active." */
@@ -504,7 +500,7 @@ TCAS::AdvisoryCoordinator::update(int mode)
     }
 
     /* [TCASII]: "Aural annunciations are inhibited below 500+/-100 feet AGL." */
-    if ((tcas->threatDetector.getRadarAlt() > 500)&&
+    if ((tcas->threatDetector.getRadarAlt() > tcas->_TAInhbAlt) &&
         (mode >= SwitchTaOnly))
         tcas->annunciator.trigger(current, revertedRA);
     else
@@ -515,7 +511,7 @@ TCAS::AdvisoryCoordinator::update(int mode)
     }
 
     previous = current;
-    
+
     /* [TCASII] "[..] also performs the function of setting flags that control the displays.
      *   The traffic display, the RA display, [..] use these flags to alert the pilot to
      *   the presence of TAs and RAs." */
@@ -577,7 +573,7 @@ TCAS::ThreatDetector::update(void)
     checkCount = 0;
 #endif
 
-    // determine current altitude's "Sensitivity Level Definition and Alarm Thresholds" 
+    // determine current altitude's "Sensitivity Level Definition and Alarm Thresholds"
     int sl=0;
     for (sl=0;((self.radarAltFt > sensitivityLevels[sl].maxAltitude)&&
                (sensitivityLevels[sl].maxAltitude));sl++);
@@ -585,32 +581,45 @@ TCAS::ThreatDetector::update(void)
     tcas->advisoryGenerator.setAlarmThresholds(pAlarmThresholds);
 }
 
+// If plane's transponder is enabled, return true with o_altFt set to
+// altitude. Otherwise return false.
+//
+static bool checkTransponderLocal(const SGPropertyNode* pModel, float velocityKt, float& o_altFt)
+{
+    if (pModel->getBoolValue("controls/invisible", false /*default*/))
+    {
+        // For MP aircraft (name='multiplayer') that are being ignored.
+        return false;
+    }
+    if (pModel->getNameString() == "swift")
+    {
+        /* Transponder info is in ./swift/transponder/ but altitude needs to
+        come from ./position/altitude-ft. */
+        bool xpdr_on = pModel->getBoolValue("swift/transponder/c-mode", false);
+        if (!xpdr_on) return false;
+        o_altFt = pModel->getDoubleValue("position/altitude-ft");
+        return true;
+    }
+    else if (pModel->getNameString() == "aircraft")
+    {
+        /* assume all non-MP and non-Swift (i.e. AI) aircraft have their transponder switched off while taxiing/parking
+         * (at low speed) */
+        if (velocityKt < 40.0)  return false;
+        o_altFt = pModel->getDoubleValue("position/altitude-ft");
+        return true;
+    }
+    o_altFt = pModel->getIntValue("instrumentation/transponder/altitude", -9999);
+    // must have Mode C (altitude) transponder to be visible.
+    // "-9999" is a special value used by src/Instrumentation/transponder.cxx to indicate the non-transmission of a value.
+    return (o_altFt != -9999);
+}
+
 /** Check if plane's transponder is enabled. */
 bool
 TCAS::ThreatDetector::checkTransponder(const SGPropertyNode* pModel, float velocityKt)
 {
-    const string name = pModel->getName();
-    if (name != "multiplayer" && name != "aircraft")
-    {
-        // assume non-MP/non-AI planes (e.g. ships) have no transponder
-        return false;
-    }
-
-    if (velocityKt < 40)
-    {
-        /* assume all pilots have their transponder switched off while taxiing/parking
-         * (at low speed) */
-        return false;
-    }
-
-    if ((name == "multiplayer")&&
-        (pModel->getBoolValue("controls/invisible")))
-    {
-        // ignored MP plane: pretend transponder is switched off
-        return false;
-    }
-
-    return true;
+    float altFt;
+    return checkTransponderLocal(pModel, velocityKt, altFt);
 }
 
 /** Check if plane is a threat. */
@@ -625,15 +634,15 @@ TCAS::ThreatDetector::checkThreat(int mode, const SGPropertyNode* pModel)
 
     float velocityKt  = pModel->getDoubleValue("velocities/true-airspeed-kt");
 
-    if (!checkTransponder(pModel, velocityKt))
+    float altFt;
+    if (!checkTransponderLocal(pModel, velocityKt, altFt))
         return ThreatInvisible;
 
     int threatLevel = ThreatNone;
-    float altFt = pModel->getDoubleValue("position/altitude-ft");
     currentThreat.relativeAltitudeFt = altFt - self.pressureAltFt;
 
     // save computation time: don't care when relative altitude is excessive
-    if (fabs(currentThreat.relativeAltitudeFt) > 10000)
+    if (fabs(currentThreat.relativeAltitudeFt) > tcas->_verticalRange)
         return threatLevel;
 
     // position data of current intruder
@@ -645,16 +654,16 @@ TCAS::ThreatDetector::checkThreat(int mode, const SGPropertyNode* pModel)
     calcRangeBearing(self.lat, self.lon, lat, lon, distanceNm, bearing);
 
     // save computation time: don't care for excessive distances (also captures NaNs...)
-    if ((distanceNm > 10)||(distanceNm < 0))
+    if ((distanceNm > tcas->_lateralRange) || (distanceNm < 0))
         return threatLevel;
 
     currentThreat.verticalFps = pModel->getDoubleValue("velocities/vertical-speed-fps");
-    
+
     /* Detect proximity targets
      * [TCASII]: "Any target that is less than 6 nmi in range and within +/-1200ft
      *  vertically, but that does not meet the intruder or threat criteria." */
-    if ((distanceNm < 6)&&
-        (fabs(currentThreat.relativeAltitudeFt) < 1200))
+    if ((distanceNm < tcas->_proxLatRange) &&
+        (fabs(currentThreat.relativeAltitudeFt) < tcas->_proxVertRange))
     {
         // at least a proximity target
         threatLevel = ThreatProximity;
@@ -674,7 +683,7 @@ TCAS::ThreatDetector::checkThreat(int mode, const SGPropertyNode* pModel)
     }
     else
         currentThreat.isTracked = false;
-    
+
     // first stage: vertical movement
     checkVerticalThreat();
 
@@ -706,8 +715,8 @@ TCAS::ThreatDetector::checkThreat(int mode, const SGPropertyNode* pModel)
     cout << "#" << checkCount << ": " << pModel->getStringValue("callsign") << endl;
 #endif
 
-    
-    /* [TCASII]: "For either a TA or an RA to be issued, both the range and 
+
+    /* [TCASII]: "For either a TA or an RA to be issued, both the range and
      *    vertical criteria, in terms of tau or the fixed thresholds, must be
      *    satisfied only one of the criteria is satisfied, TCAS will not issue
      *    an advisory." */
@@ -720,7 +729,7 @@ TCAS::ThreatDetector::checkThreat(int mode, const SGPropertyNode* pModel)
         currentThreat.callsign = pModel->getStringValue("callsign");
 
     tcas->tracker.add(currentThreat.callsign, threatLevel);
-    
+
     // check existing threat level
     if (currentThreat.isTracked)
     {
@@ -731,8 +740,8 @@ TCAS::ThreatDetector::checkThreat(int mode, const SGPropertyNode* pModel)
 
     // find all resolution options for this conflict
     threatLevel = tcas->advisoryGenerator.resolution(mode, threatLevel, distanceNm, altFt, heading, velocityKt);
-   
-    
+
+
 #ifdef FEATURE_TCAS_DEBUG_THREAT_DETECTOR
     printf("  threat: distance: %4.1f, bearing: %4.1f, alt: %5.1f, velocity: %4.1f, heading: %4.1f, vspeed: %4.1f, "
            "own alt: %5.1f, own heading: %4.1f, own velocity: %4.1f, vertical tau: %3.2f"
@@ -816,7 +825,7 @@ TCAS::ThreatDetector::horizontalThreat(float bearing, float distanceNm, float he
     float vyKt = cos(heading*SGD_DEGREES_TO_RADIANS)*velocityKt - cos(self.heading*SGD_DEGREES_TO_RADIANS)*self.velocityKt;
 
     // calculate horizontal closing speed
-    float closingSpeedKt2 = vxKt*vxKt+vyKt*vyKt; 
+    float closingSpeedKt2 = vxKt*vxKt+vyKt*vyKt;
     float closingSpeedKt  = sqrt(closingSpeedKt2);
 
     /* [TCASII]: "The range tau is equal to the slant range (nmi) divided by the closing speed
@@ -844,7 +853,7 @@ TCAS::ThreatDetector::horizontalThreat(float bearing, float distanceNm, float he
     currentThreat.horizontalTA   = (distanceNm < TA_rangeNm);
     currentThreat.horizontalRA   = (distanceNm < RA_rangeNm);
     currentThreat.horizontalTau  = -1;
-    
+
     if ((currentThreat.horizontalRA)&&
         (currentThreat.verticalRA))
     {
@@ -866,7 +875,7 @@ TCAS::ThreatDetector::horizontalThreat(float bearing, float distanceNm, float he
          * => r2'(t) = b + a*2*t
          * at t=tau:
          *    r2'(tau) = 0 = b + 2*a*tau
-         * => tau = -b/(2*a) 
+         * => tau = -b/(2*a)
          */
         float sx = sin(bearing*SGD_DEGREES_TO_RADIANS)*distanceNm;
         float sy = cos(bearing*SGD_DEGREES_TO_RADIANS)*distanceNm;
@@ -882,7 +891,7 @@ TCAS::ThreatDetector::horizontalThreat(float bearing, float distanceNm, float he
 #endif
         if (tau > pAlarmThresholds->RA.Tau)
             tau = pAlarmThresholds->RA.Tau;
-        
+
         // remember time to horizontal CPA
         currentThreat.horizontalTau = tau;
     }
@@ -977,7 +986,7 @@ TCAS::AdvisoryGenerator::determineRAsense(int& RASense, bool& isCrossing)
              *   geometry. The first step in the process is to select the RA sense, i.e., upward or downward." */
     RASense = 0;
     isCrossing = false;
-    
+
     /* [TCASII]: "Based on the range and altitude tracks of the intruder, the CAS logic models the
      *   intruder's flight path from its present position to CPA. The CAS logic then models upward
      *   and downward sense RAs for own aircraft [..] to determine which sense provides the most
@@ -1000,7 +1009,7 @@ TCAS::AdvisoryGenerator::determineRAsense(int& RASense, bool& isCrossing)
         // threat is above and RA is crossing
         if (fabs(downSenseRelAltFt) > pAlarmThresholds->TA.ALIM)
         {
-            // non-crossing descend is sufficient 
+            // non-crossing descend is sufficient
             RASense = -1;
         }
         else
@@ -1016,7 +1025,7 @@ TCAS::AdvisoryGenerator::determineRAsense(int& RASense, bool& isCrossing)
         // threat is below and RA is crossing
         if (fabs(upSenseRelAltFt) > pAlarmThresholds->TA.ALIM)
         {
-            // non-crossing climb is sufficient 
+            // non-crossing climb is sufficient
             RASense = 1;
         }
         else
@@ -1050,21 +1059,21 @@ TCAS::AdvisoryGenerator::resolution(int mode, int threatLevel, float rangeNm, fl
     {
         /* [TCASII]: "... less than 360 feet, TCAS considers the reporting aircraft
          *   to be on the ground. If TCAS determines the intruder to be on the ground, it
-         *   inhibits the generation of advisories against this aircraft."*/
-        if (altFt < 360)
+         * New custom logic for the A320 - if enabled explicitly this will only occur if your altitude is less than 1700 */
+        if ((altFt < tcas->_intruderInhbAlt) && (!tcas->_intruderInhbSelfAltToggle || pSelf->radarAltFt < tcas->_intruderInhbSelfAlt))
             threatLevel = ThreatTA;
 
         /* [EUROACAS]: "Certain RAs are inhibited at altitudes based on inputs from the radio altimeter:
          *   [..] (c)1000ft (+/- 100ft) and below, all RAs are inhibited;" */
-        if (pSelf->radarAltFt < 1000)
+        if (pSelf->radarAltFt < tcas->_RAInhbAlt)
             threatLevel = ThreatTA;
-        
+
         // RAs only issued in mode "Auto" (= "TA/RA" mode)
         if (mode != SwitchAuto)
             threatLevel = ThreatTA;
     }
 
-    bool isCrossing = false; 
+    bool isCrossing = false;
     int RASense = 0;
     // determine suitable RAs
     if (threatLevel == ThreatRA)
@@ -1081,7 +1090,7 @@ TCAS::AdvisoryGenerator::resolution(int mode, int threatLevel, float rangeNm, fl
             if ((pSelf->verticalFps < -1000/60.0)&&
                 (!isCrossing))
             {
-                // currently descending, see if reducing current descent is sufficient 
+                // currently descending, see if reducing current descent is sufficient
                 float relAltFt = verticalSeparation(-500/60.0);
                 if (relAltFt > pAlarmThresholds->TA.ALIM)
                     RA |= AdvisoryAdjustVSpeed;
@@ -1098,7 +1107,7 @@ TCAS::AdvisoryGenerator::resolution(int mode, int threatLevel, float rangeNm, fl
             if ((pSelf->verticalFps > 1000/60.0)&&
                 (!isCrossing))
             {
-                // currently climbing, see if reducing current climb is sufficient 
+                // currently climbing, see if reducing current climb is sufficient
                 float relAltFt = verticalSeparation(500/60.0);
                 if (relAltFt < -pAlarmThresholds->TA.ALIM)
                     RA |= AdvisoryAdjustVSpeed;
@@ -1113,18 +1122,18 @@ TCAS::AdvisoryGenerator::resolution(int mode, int threatLevel, float rangeNm, fl
          *   and are currently well separated in altitude, TCAS will first issue a vertical speed
          *   limit (Negative) RA to reinforce the pilots' likely intention to level off at adjacent
          *   flight levels." */
-        
+
         //TODO
         /* [TCASII]: "[..] if the CAS logic determines that the response to a Positive RA has provided
          *   ALIM feet of vertical separation before CPA, the initial RA will be weakened to either a
          *   Do Not Descend RA (after an initial Climb RA) or a Do Not Climb RA (after an initial
          *   Descend RA)." */
-        
+
         //TODO
         /* [TCASII]: "TCAS is designed to inhibit Increase Descent RAs below 1450 feet AGL; */
-        
+
         /* [TCASII]: "Descend RAs below 1100 feet AGL;" (inhibited) */
-        if (pSelf->radarAltFt < 1100)
+        if (pSelf->radarAltFt < tcas->_DesInhbAlt)
         {
             RA &= ~AdvisoryDescend;
             //TODO Support "Do not descend" RA
@@ -1141,7 +1150,7 @@ TCAS::AdvisoryGenerator::resolution(int mode, int threatLevel, float rangeNm, fl
     newAdvisory.RA          = RA;
     newAdvisory.threatLevel = threatLevel;
     tcas->advisoryCoordinator.add(newAdvisory);
-    
+
     return threatLevel;
 }
 
@@ -1159,18 +1168,56 @@ TCAS::TCAS(SGPropertyNode* pNode) :
     tracker(this),
     advisoryCoordinator(this),
     advisoryGenerator(this),
-    annunciator(this)
+    annunciator(this),
+    _verticalRange(10000),
+    _lateralRange(10),
+    _proxVertRange(1200),
+    _proxLatRange(6),
+    _incDesInhbAlt(1450),
+    _DesInhbAlt(1100),
+    _RAInhbAlt(1000),
+    _TAInhbAlt(500),
+    _intruderInhbAlt(360),
+    _intruderInhbSelfAltToggle(false),
+    _intruderInhbSelfAlt(1700)
 {
     for (int i = 0; i < pNode->nChildren(); ++i)
     {
         SGPropertyNode* pChild = pNode->getChild(i);
-        string cname = pChild->getName();
+        string cname = pChild->getNameString();
         string cval = pChild->getStringValue();
+        int cintval = pChild->getIntValue();
+
+        if (cintval < 0 ) {
+            SG_LOG(SG_INSTR, SG_WARN, "Error in TCAS config logic: value less than zero. Skipping!");
+        }
 
         if (cname == "name")
             name = cval;
         else if (cname == "number")
-            num = pChild->getIntValue();
+            num = cintval;
+        else if (cname == "vertical-range-ft" && cintval > 0)
+            _verticalRange = cintval;
+        else if (cname == "lateral-range-nm" && cintval > 0)
+            _lateralRange = cintval;
+        else if (cname == "proximate-vertical-range-ft" && cintval > 0)
+            _proxVertRange = cintval;
+        else if (cname == "proximate-lateral-range-nm" && cintval > 0)
+            _proxLatRange = cintval;
+        else if (cname == "increase-descent-inhibit-alt-ft" && cintval > 0)
+            _incDesInhbAlt = cintval;
+        else if (cname == "descend-inhibit-alt-ft" && cintval > 0)
+            _DesInhbAlt = cintval;
+        else if (cname == "ra-inhibit-alt-ft" && cintval > 0)
+            _RAInhbAlt = cintval;
+        else if (cname == "ta-inhibit-alt-ft" && cintval > 0)
+            _TAInhbAlt = cintval;
+        else if (cname == "intruder-inhibit-alt-ft" && cintval > 0)
+            _intruderInhbAlt = cintval;
+        else if (cname == "intruder-use-own-alt")
+            _intruderInhbSelfAltToggle = pChild->getBoolValue();
+        else if (cname == "intruder-minimum-own-alt-ft" && cintval > 0)
+            _intruderInhbSelfAlt = cintval;
         else
         {
             SG_LOG(SG_INSTR, SG_WARN, "Error in TCAS config logic");
@@ -1198,11 +1245,11 @@ TCAS::reinit(void)
 void
 TCAS::bind(void)
 {
-    SGPropertyNode* node = fgGetNode(("/instrumentation/" + name).c_str(), num, true);
+    SGPropertyNode* node = fgGetNode(("/instrumentation/" + name), num, true);
 
     nodeServiceable  = node->getNode("serviceable", true);
 
-    // TCAS mode selection (0=off, 1=standby, 2=TA only, 3=auto(TA/RA) ) 
+    // TCAS mode selection (0=off, 1=standby, 2=TA only, 3=auto(TA/RA) )
     nodeModeSwitch   = node->getNode("inputs/mode", true);
     // self-test button
     nodeSelfTest     = node->getNode("inputs/self-test", true);
@@ -1489,3 +1536,12 @@ TCAS::Tracker::getThreatLevel(string callsign)
     else
         return 0;
 }
+
+
+// Register the subsystem.
+#if 0
+SGSubsystemMgr::InstancedRegistrant<TCAS> registrantTCAS(
+    SGSubsystemMgr::FDM,
+    {{"instrumentation", SGSubsystemMgr::Dependency::HARD}},
+    0.2);
+#endif
