@@ -1,4 +1,4 @@
-// transponder.cxx -- class to impliment a transponder
+// transponder.cxx -- class to implement a transponder
 //
 // Written by Roy Vegard Ovesen, started September 2004.
 //
@@ -23,12 +23,31 @@
 //      <transponder>
 //        <name>encoder</name>
 //        <number>0</number>
-//        <mode>0</mode>  // Mode A = 0, Mode C = 1, Mode S = 2
+//        <mode>2</mode>  // Mode A = 0, Mode C = 1, Mode S = 2
+//        <auto-ground>...</auto-ground>
+//        <airspeed-path>...</airspeed-path>
+//        <mach-path>...</mach-path>
 //      </altimeter>
+//
+// Mode-S transponders (configured with mode = 2) can transmit a ground bit to
+// indicate ground operation. If auto-ground is not defined, the ground bit is
+// switched manually by setting the transponder knob to GND. If auto-ground is
+// defined, the transponder simulates automatic switching of the ground bit
+// using the value of the property defined in auto-ground.
+//
+// For a squat switch, use "/gear/gear/wow". For automatic switching based on
+// airspeed, use a property rule to make a new boolean property that indicates,
+// for example, airspeed < 80, and reference your new property in auto-ground.
+//
+// Note that Mode-A and Mode-C transponders do not transmit a ground bit, even
+// if the transponder knob is set to the GND position.
+//
+// Mode-S transponders also transmit indicated airspeed and Mach number. The
+// default sources are "/instrumentation/airspeed-indicator/indicated-speed-kt"
+// and ".../indicated-mach", but this can be changed by setting "airspeed-path"
+// and "mach-path" properties respectively as shown above.
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #include "transponder.hxx"
 
@@ -44,18 +63,25 @@ using std::string;
 
 const double IDENT_TIMEOUT = 18.0; // 18 seconds
 const int INVALID_ALTITUDE = -9999;
+const int INVALID_AIRSPEED = -9999;
+const float INVALID_MACH_NUM = -1.0;
 const int INVALID_ID = -9999;
 
-Transponder::Transponder(SGPropertyNode *node)
-    :
-    _identMode(false),
-    _name(node->getStringValue("name", "transponder")),
-    _num(node->getIntValue("number", 0)),
-    _mode((Mode) node->getIntValue("mode", 1)),
-    _listener_active(0)
+Transponder::Transponder(SGPropertyNode *node) :
+    _mode((Mode) node->getIntValue("mode", 1))
 {
-    _requiredBusVolts = node->getDoubleValue("bus-volts", 8.0);
+    readConfig(node, "transponder");
+
+    // ensure older power-supply name works
+    if (node->hasChild("bus-volts")) {
+        SG_LOG(SG_INSTR, SG_DEV_WARN, "Transponder: use new supply-volatge prop: 'minimum-supply-volts' instead of 'bus-volts'");
+        setMinimumSupplyVolts(node->getDoubleValue("bus-volts", 8.0));
+    }
+    
     _altitudeSourcePath = node->getStringValue("encoder-path", "/instrumentation/altimeter");
+    _autoGroundPath = node->getStringValue("auto-ground");
+    _airspeedSourcePath = node->getStringValue("airspeed-path", "/instrumentation/airspeed-indicator/indicated-speed-kt");
+    _machSourcePath = node->getStringValue("mach-path", "/instrumentation/airspeed-indicator/indicated-mach");
     _kt70Compat = node->getBoolValue("kt70-compatibility", false);
 }
 
@@ -67,11 +93,14 @@ Transponder::~Transponder()
 
 void Transponder::init()
 {
-    SGPropertyNode *node = fgGetNode("/instrumentation/" + _name, _num, true );
+    SGPropertyNode *node = fgGetNode(nodePath(), true);
+    initServicePowerProperties(node);
 
     // Inputs
-    _busPower_node = fgGetNode("/systems/electrical/outputs/transponder", true);
     _pressureAltitude_node = fgGetNode(_altitudeSourcePath, true);
+    _autoGround_node = fgGetNode(_autoGroundPath, true);
+    _airspeedIndicator_node = fgGetNode(_airspeedSourcePath, true);
+    _machSource_node = fgGetNode(_machSourcePath, true);
 
     SGPropertyNode *in_node = node->getChild("inputs", 0, true);
     for (int i=0; i<4;++i) {
@@ -99,9 +128,6 @@ void Transponder::init()
     _identBtn_node->setBoolValue(false);
     _identBtn_node->addChangeListener(this);
     
-    _serviceable_node = node->getChild("serviceable", 0, true);
-    _serviceable_node->setBoolValue(true);
-    
     _idCode_node = node->getChild("id-code", 0, true);
     _idCode_node->addChangeListener(this);
     // set default, but don't overwrite value from defaults.xml or -set.xml
@@ -114,20 +140,23 @@ void Transponder::init()
     _altitudeValid_node = node->getChild("altitude-valid", 0, true);
     _ident_node = node->getChild("ident", 0, true);
     _transmittedId_node = node->getChild("transmitted-id", 0, true);
+    _ground_node = node->getChild("ground-bit", 0, true);
+    _airspeed_node = node->getChild("airspeed-kt", 0, true);
+    _mach_node = node->getChild("mach-number", 0, true);
     
     if (_kt70Compat) {
         // alias the properties through
         SGPropertyNode_ptr output = node->getChild("outputs", 0, true);
-        output->getChild("flight-level", 0, true)->alias(_altitude_node);
-        output->getChild("id-code", 0, true)->alias(_idCode_node);
-        in_node->getChild("func-knob", 0, true)->alias(_knob_node);
+        output->getChild("flight-level", 0, true)->alias(_altitude_node, true);
+        output->getChild("id-code", 0, true)->alias(_idCode_node, true);
+        in_node->getChild("func-knob", 0, true)->alias(_knob_node, true);
     }
 }
 
 void Transponder::bind()
 {
     if (_kt70Compat) {
-        SGPropertyNode *node = fgGetNode("/instrumentation/" + _name, _num, true );
+        SGPropertyNode *node = fgGetNode(nodePath(), true );
         _tiedProperties.setRoot(node);
         
         _tiedProperties.Tie("annunciators/fl", this,
@@ -148,15 +177,15 @@ void Transponder::bind()
 void Transponder::unbind()
 {
     _tiedProperties.Untie();
+    AbstractInstrument::unbind();
 }
 
-
 void Transponder::update(double dt)
 {
-    if (has_power() && _serviceable_node->getBoolValue())
+    if (isServiceableAndPowered())
     {
         // Mode C & S send also altitude
-        Mode effectiveMode = (_knob == KNOB_ALT) ? _mode : MODE_A;
+        Mode effectiveMode = (_knob == KNOB_ALT || _knob == KNOB_GROUND) ? _mode : MODE_A;
         SGPropertyNode* altitudeSource = NULL;
         
         switch (effectiveMode) {
@@ -192,17 +221,44 @@ void Transponder::update(double dt)
             }
         }
         
-        if (_knob >= KNOB_ON) {
+        if (_knob >= KNOB_GROUND) {
             _transmittedId_node->setIntValue(_idCode_node->getIntValue());
         } else {
             _transmittedId_node->setIntValue(INVALID_ID);
         }
+
+        if (_mode == MODE_S && _knob >= KNOB_GROUND) {
+            if (_autoGround_node->hasValue()) {
+                // Automatic ground bit based on the auto-ground property
+                _ground_node->setBoolValue(_autoGround_node->getBoolValue());
+            } else {
+                // Manual ground bit based on the transponder knob
+                _ground_node->setBoolValue(_knob == KNOB_GROUND);
+            }
+        } else {
+            _ground_node->setBoolValue(false);
+        }
+
+        if (_mode == MODE_S && _knob >= KNOB_GROUND && _airspeedIndicator_node->hasValue()) {
+            _airspeed_node->setIntValue(_airspeedIndicator_node->getIntValue());
+        } else {
+            _airspeed_node->setIntValue(INVALID_AIRSPEED);
+        }
+
+        if (_mode == MODE_S && _machSource_node->hasValue()) {
+            _mach_node->setDoubleValue(_machSource_node->getDoubleValue());
+        } else {
+            _mach_node->setDoubleValue(INVALID_MACH_NUM);
+        }
     }
     else
-    {
+    { // un-powered or u/s
       _altitude_node->setIntValue(INVALID_ALTITUDE);
       _altitudeValid_node->setBoolValue(false);
       _ident_node->setBoolValue(false);
+      _ground_node->setBoolValue(false);
+      _airspeed_node->setIntValue(INVALID_AIRSPEED);
+      _mach_node->setDoubleValue(INVALID_MACH_NUM);
       _transmittedId_node->setIntValue(INVALID_ID);
     }
 }
@@ -268,11 +324,6 @@ void Transponder::valueChanged(SGPropertyNode *prop)
     _listener_active--;
 }
 
-bool Transponder::has_power() const
-{
-    return (_knob_node->getIntValue() > KNOB_STANDBY) && (_busPower_node->getDoubleValue() > _requiredBusVolts);
-}
-
 bool Transponder::getFLAnnunciator() const
 {
     return (_knob == KNOB_ALT) || (_knob == KNOB_GROUND) || (_knob == KNOB_TEST);
@@ -303,3 +354,16 @@ bool Transponder::getReplyAnnunciator() const
     return _identMode || (_knob == KNOB_TEST);
 }
 
+bool Transponder::isPowerSwitchOn() const
+{
+    return (_knob_node->getIntValue() >= KNOB_STANDBY);
+}
+
+
+// Register the subsystem.
+#if 0
+SGSubsystemMgr::InstancedRegistrant<Transponder> registrantTransponder(
+    SGSubsystemMgr::FDM,
+    {{"instrumentation", SGSubsystemMgr::Dependency::HARD}},
+    0.2);
+#endif
