@@ -58,17 +58,16 @@
 #include <osg/Version>
 #include <osgText/Font>
 
-#include <Viewer/fgviewer.hxx>
 #include "main.hxx"
-#include <Include/version.h>
-#include <Include/build.h>
-#include <Main/globals.hxx>
-#include <Main/fg_init.hxx>
-#include <Main/options.hxx>
-#include <Main/fg_props.hxx>
-#include <Main/sentryIntegration.hxx>
 #include <GUI/MessageBox.hxx>
+#include <Main/fg_init.hxx>
+#include <Main/fg_props.hxx>
+#include <Main/globals.hxx>
+#include <Main/options.hxx>
+#include <Main/sentryIntegration.hxx>
 #include <Navaids/NavDataCache.hxx>
+#include <Viewer/fgviewer.hxx>
+#include <flightgearBuildId.h>
 
 #include "fg_os.hxx"
 
@@ -148,18 +147,56 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
 
 	main(numArgs, utf8Args.data());
 }
+
+// see https://stackoverflow.com/questions/35572792/setlocale-stuck-on-windows
+// for why we need this
+bool checkUniversalCRTVersion()
+{
+	DWORD               dwSize = 0;
+	VS_FIXEDFILEINFO    *pFileInfo = NULL;
+	UINT                puLenFileInfo = 0;
+
+	// Get the version information for the file requested
+    dwSize = GetFileVersionInfoSizeA("ucrtbase.dll", NULL);
+    if (dwSize == 0) {
+		return false;
+	}
+
+	std::vector<BYTE> versionInfo;
+	versionInfo.resize(dwSize);
+
+    if (!GetFileVersionInfoA("ucrtbase.dll", 0, dwSize, versionInfo.data())) {
+        return false;
+    }
+
+    if (!VerQueryValue(versionInfo.data(), TEXT("\\"), (LPVOID*)&pFileInfo, &puLenFileInfo)) {
+		return false;
+	}
+
+	const WORD majorVersion = pFileInfo->dwFileVersionMS >> 16;
+	const WORD minorVersion = pFileInfo->dwFileVersionMS & 0xffff;
+	const WORD buildVersion = pFileInfo->dwFileVersionLS >> 16;
+	const WORD releaseVersion = pFileInfo->dwFileVersionLS & 0xffff;
+
+	// char buffer[256];
+	// snprintf(buffer, 256, "File Version: %d.%d.%d.%d\n",
+	// 	majorVersion, minorVersion, buildVersion, releaseVersion);
+	// OutputDebugString(buffer);
+	return (buildVersion > 10586);
+}
+
 #endif
 
 #if defined(__GNUC__)
 #include <execinfo.h>
 #include <cxxabi.h>
 void segfault_handler(int signo) {
-  void *array[128];
-  size_t size;
 
   fprintf(stderr, "Error: caught signal %d:\n", signo);
 
   #ifndef __OpenBSD__
+  void *array[128];
+  size_t size;
   size = backtrace(array, 128);
   if (size) {
     char** list = backtrace_symbols(array, size);
@@ -238,7 +275,7 @@ int main ( int argc, char **argv )
     // we don't want to accidently show a GUI box and block startup in
     // non_GUI setups, so check this value early here, before options are
     // processed
-    const bool headless = flightgear::Options::checkForArg(argc, argv, "disable-gui");
+    const bool headless = flightgear::Options::checkForArgDisable(argc, argv, "gui");
     flightgear::setHeadlessMode(headless);
 
 #ifdef ENABLE_SIMD
@@ -257,6 +294,13 @@ int main ( int argc, char **argv )
   SetErrorMode(SEM_NOOPENFILEERRORBOX);
 
   hostname = ::getenv( "COMPUTERNAME" );
+
+  if (!checkUniversalCRTVersion()) {
+	  flightgear::fatalMessageBoxThenExit(
+		  "Fatal error",
+		  "The Microsoft Universal CRT on this computer is too old to run FlightGear. "
+		"Please use Windows Update to install a more recent Universal CRT version.");
+  }
 #else
   // Unix(alike) systems
   char _hostname[256];
@@ -268,18 +312,23 @@ int main ( int argc, char **argv )
   _bootstrap_OSInit = 0;
     
 #if defined(HAVE_SENTRY)
-  flightgear::initSentry();
-#endif
+  const bool noSentry = flightgear::Options::checkForArgDisable(argc, argv, "sentry");
+  if (!noSentry) {
+      flightgear::initSentry();
+  }
+  #endif
 
 // if we're not using the normal crash-reported, install our
-// custom segfault handler on Linux, in debug builds
-#if !defined(SG_WINDOWS) && !defined(NDEBUG)
+// custom segfault handler on Linux, in debug builds.
+// NB On OpenBSD this seems to lose info about where the signal
+// happened, so is disabled.
+#if !defined(SG_WINDOWS) && !defined(NDEBUG) && !defined(__OpenBSD__)
     if (!flightgear::isSentryEnabled()) {
         signal(SIGSEGV, segfault_handler);
     }
 #endif
 
-    initFPE(flightgear::Options::checkForArg(argc, argv, "enable-fpe"));
+    initFPE(flightgear::Options::checkForArgEnable(argc, argv, "fpe"));
 
     // pick up all user locale settings, but force C locale for numerical/sorting
     // conversions because we have lots of code which assumes standard
@@ -291,7 +340,7 @@ int main ( int argc, char **argv )
     if (flightgear::Options::checkForArg(argc, argv, "uninstall")) {
         return fgUninstall();
     }
-
+    
     bool fgviewer = flightgear::Options::checkForArg(argc, argv, "fgviewer");
     int exitStatus = EXIT_FAILURE;
     try {
@@ -315,6 +364,7 @@ int main ( int argc, char **argv )
 #endif
         std::set_terminate(fg_terminate);
         atexit(fgExitCleanup);
+        
         if (fgviewer) {
             exitStatus = fgviewerMain(argc, argv);
         } else {
@@ -353,10 +403,11 @@ int main ( int argc, char **argv )
 
 // do some clean up on exit.  Specifically we want to delete the sound-manager,
 // so OpenAL device and context are released cleanly
-void fgExitCleanup() {
+void fgExitCleanup() 
+{
+    flightgear::addSentryBreadcrumb("starting fgExitCleanup", "info");
 
     if (_bootstrap_OSInit != 0) {
-        fgSetMouseCursor(MOUSE_CURSOR_POINTER);
         fgOSCloseWindow();
     }
 
@@ -369,6 +420,9 @@ void fgExitCleanup() {
     // on the common exit path globals is already deleted, and NULL,
     // so this only happens on error paths.
     delete globals;
+
+    flightgear::addSentryBreadcrumb("finished deleting globals", "info");
+
     // avoid crash on exit (https://sourceforge.net/p/flightgear/codetickets/1935/)
     simgear::GroundLightManager::instance()->getRunwayLightStateSet()->clear();
     simgear::GroundLightManager::instance()->getTaxiLightStateSet()->clear();
