@@ -1,8 +1,11 @@
-// fg_commands.cxx - internal FGFS commands.
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+/*
+ * SPDX-FileName: fg_commands.hxx
+ * SPDX-FileComment: built-in commands for FlightGear.
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
+
+#include <config.h>
 
 #include <string.h>		// strcmp()
 
@@ -14,7 +17,8 @@
 #include <simgear/debug/ErrorReportingCallback.hxx>
 #include <simgear/debug/logstream.hxx>
 #include <simgear/io/iostreams/sgstream.hxx>
-#include <simgear/math/sg_random.h>
+#include <simgear/math/sg_random.hxx>
+#include <simgear/misc/simgear_optional.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/props/props_io.hxx>
 #include <simgear/sg_inlines.h>
@@ -44,7 +48,6 @@
 #include "fg_props.hxx"
 #include "globals.hxx"
 #include "logger.hxx"
-#include "util.hxx"
 #include "main.hxx"
 #include "positioninit.hxx"
 
@@ -119,41 +122,81 @@ split_value (double full_value, const char * mask,
     }
 }
 
+/**
+ * @brief Retrive a typed value from a node, either from <foo> directly or indirectly
+ * 
+ * @param node - base node from the command to look inside
+ * @param name - direct name of the argument, eg 'min' or 'max'
+ * @param indirectName - indirect name to use, eg, 'min-path'. If empty, the name is formed using the base
+ *  name and appending '-prop', eg 'min-prop' and 'max-prop'.
+ * @return std::optional<T> 
+ */
+template <class T>
+static simgear::optional<T>
+getValueIndirect(const SGPropertyNode* node, const std::string& name, const std::string& indirectName = {})
+{
+    auto indirectNode = node->getChild(indirectName.empty() ? (name + "-prop") : indirectName);
+    if (indirectNode) {
+        const auto resolvedNode = fgGetNode(indirectNode->getStringValue());
+        if (resolvedNode) {
+            return resolvedNode->getValue<T>();
+        }
+
+        // if the path is not valid, warn
+        SG_LOG(SG_GENERAL, SG_DEV_WARN, "getValueIndirect: property:" << indirectNode->getNameString() << " has value '" << indirectNode->getStringValue() << "' which was not found in the global property tree. Falling back to "
+                                                                                                                                                              "value defined by argument '"
+                                                                      << name << "'");
+
+        // deliberate fall through here, so we use the value from the direct prop
+    }
+
+    auto directNode = node->getChild(name);
+    if (directNode)
+        return directNode->getValue<T>();
+
+    return {};
+}
 
 /**
  * Clamp or wrap a value as specified.
  */
-static void
-limit_value (double * value, const SGPropertyNode * arg)
+static double
+limit_value(double value, const SGPropertyNode* arg)
 {
-    const SGPropertyNode * min_node = arg->getChild("min");
-    const SGPropertyNode * max_node = arg->getChild("max");
+    const auto minv = getValueIndirect<double>(arg, "min");
+    const auto maxv = getValueIndirect<double>(arg, "max");
 
-    bool wrap = arg->getBoolValue("wrap");
-
-    if (min_node == 0 || max_node == 0)
-        wrap = false;
+    // we can wrap only if requested, and
+    const bool canWrap = (minv && maxv);
+    const bool wantsWrap = arg->getBoolValue("wrap");
+    const bool wrap = canWrap && wantsWrap;
+    if (wantsWrap && !canWrap) {
+        SG_LOG(SG_GENERAL, SG_DEV_WARN, "limit_value: wrap requested, but no min/max values defined");
+    }
 
     if (wrap) {                 // wrap such that min <= x < max
-        double min_val = min_node->getDoubleValue();
-        double max_val = max_node->getDoubleValue();
-        double resolution = arg->getDoubleValue("resolution");
+        const double resolution = arg->getDoubleValue("resolution");
         if (resolution > 0.0) {
             // snap to (min + N*resolution), taking special care to handle imprecision
-            int n = (int)floor((*value - min_val) / resolution + 0.5);
-            int steps = (int)floor((max_val - min_val) / resolution + 0.5);
+            int n = (int)floor((value - minv.value()) / resolution + 0.5);
+            int steps = (int)floor((maxv.value() - minv.value()) / resolution + 0.5);
             SG_NORMALIZE_RANGE(n, 0, steps);
-            *value = min_val + resolution * n;
+            return minv.value() + resolution * n;
         } else {
             // plain circular wrapping
-            SG_NORMALIZE_RANGE(*value, min_val, max_val);
+            return SGMiscd::normalizePeriodic(minv.value(), maxv.value(), value);
         }
     } else {                    // clamp such that min <= x <= max
-        if ((min_node != 0) && (*value < min_node->getDoubleValue()))
-            *value = min_node->getDoubleValue();
-        else if ((max_node != 0) && (*value > max_node->getDoubleValue()))
-            *value = max_node->getDoubleValue();
+        if (minv && (value < minv.value())) {
+            return minv.value();
+        }
+
+        if (maxv && (value > maxv.value())) {
+            return maxv.value();
+        }
     }
+
+    return value; // return unmodified value
 }
 
 static bool
@@ -171,7 +214,7 @@ compare_values (SGPropertyNode * value1, SGPropertyNode * value2)
     case simgear::props::DOUBLE:
         return (value1->getDoubleValue() == value2->getDoubleValue());
     default:
-        return !strcmp(value1->getStringValue(), value2->getStringValue());
+        return value1->getStringValue() == value2->getStringValue();
     }
 }
 
@@ -212,7 +255,7 @@ do_nasal (const SGPropertyNode * arg, SGPropertyNode * root)
 static bool
 do_replay (const SGPropertyNode * arg, SGPropertyNode * root)
 {
-    FGReplay *r = (FGReplay *)(globals->get_subsystem( "replay" ));
+    auto r = globals->get_subsystem<FGReplay>();
     return r->start();
 }
 
@@ -253,12 +296,12 @@ do_pause (const SGPropertyNode * arg, SGPropertyNode * root)
 static bool
 do_load (const SGPropertyNode * arg, SGPropertyNode * root)
 {
-    SGPath file(arg->getStringValue("file", "fgfs.sav"));
+    SGPath file(arg->getStringValue("file", "fgfs.sav").c_str());
 
     if (file.extension() != "sav")
         file.concat(".sav");
 
-    SGPath validated_path = fgValidatePath(file, false);
+    const SGPath validated_path = SGPath(file).validate(false);
     if (validated_path.isNull()) {
         SG_LOG(SG_IO, SG_ALERT, "load: reading '" << file << "' denied "
                 "(unauthorized access)");
@@ -286,12 +329,12 @@ do_load (const SGPropertyNode * arg, SGPropertyNode * root)
 static bool
 do_save (const SGPropertyNode * arg, SGPropertyNode * root)
 {
-    SGPath file(arg->getStringValue("file", "fgfs.sav"));
+    SGPath file(arg->getStringValue("file", "fgfs.sav").c_str());
 
     if (file.extension() != "sav")
         file.concat(".sav");
 
-    SGPath validated_path = fgValidatePath(file, true);
+    const SGPath validated_path = SGPath(file).validate(true);
     if (validated_path.isNull()) {
         SG_LOG(SG_IO, SG_ALERT, "save: writing '" << file << "' denied "
                 "(unauthorized access)");
@@ -318,7 +361,7 @@ do_save (const SGPropertyNode * arg, SGPropertyNode * root)
 static bool
 do_save_tape (const SGPropertyNode * arg, SGPropertyNode * root)
 {
-    FGReplay* replay = (FGReplay*) globals->get_subsystem("replay");
+    auto replay = globals->get_subsystem<FGReplay>();
     replay->saveTape(arg);
 
     return true;
@@ -330,7 +373,7 @@ do_save_tape (const SGPropertyNode * arg, SGPropertyNode * root)
 static bool
 do_load_tape (const SGPropertyNode * arg, SGPropertyNode * root)
 {
-    FGReplay* replay = (FGReplay*) globals->get_subsystem("replay");
+    auto replay = globals->get_subsystem<FGReplay>();
     replay->loadTape(arg);
 
     return true;
@@ -369,6 +412,108 @@ do_view_cycle (const SGPropertyNode * arg, SGPropertyNode * root)
   return true;
 }
 
+
+/**
+ * Built-in command: view-push.
+ */
+static bool
+do_view_push (const SGPropertyNode * arg, SGPropertyNode * root)
+{
+  SG_LOG(SG_GENERAL, SG_DEBUG, "do_view_push() called");
+  globals->get_viewmgr()->view_push();
+  return true;
+}
+
+
+/**
+ * Built-in command: clone view.
+ */
+static bool
+do_view_clone (const SGPropertyNode * arg, SGPropertyNode * root)
+{
+  SG_LOG(SG_GENERAL, SG_DEBUG, "do_view_clone() called");
+  globals->get_viewmgr()->clone_current_view(arg);
+  return true;
+}
+
+
+/**
+ * Built-in command: view last pair.
+ */
+static bool
+do_view_last_pair (const SGPropertyNode * arg, SGPropertyNode * root)
+{
+  SG_LOG(SG_GENERAL, SG_DEBUG, "do_view_last_pair() called");
+  globals->get_viewmgr()->clone_last_pair(arg);
+  return true;
+}
+
+
+/**
+ * Built-in command: double view last pair.
+ */
+static bool
+do_view_last_pair_double (const SGPropertyNode * arg, SGPropertyNode * root)
+{
+  SG_LOG(SG_GENERAL, SG_DEBUG, "do_view_last_pair_double() called");
+  globals->get_viewmgr()->clone_last_pair_double(arg);
+  return true;
+}
+
+
+/**
+ * Built-in command: double view last pair.
+ */
+static bool
+do_view_new (const SGPropertyNode * arg, SGPropertyNode * root)
+{
+  SG_LOG(SG_GENERAL, SG_ALERT, "do_view_new() called");
+  globals->get_viewmgr()->view_new(arg);
+  return true;
+}
+
+/**
+ * Built-in command: video-start.
+ *
+ * If arg->name exists, we use it as the leafname of the generated video,
+ * appending '.'+{/sim/video/container} if it doesn't contain '.' already.
+ *
+ * Otherwise we use:
+ *      fgvideo-{/sim/aircraft}-YYMMDD-HHMMSS.{/sim/video/container}
+ *
+ * The video file is generated in directory {/sim/paths/screenshot-dir}.
+ *
+ * We also create a convenience link in the same directory called
+ * fgvideo-{/sim/aircraft}.<suffix> (where <suffix> is the same suffix as the
+ * recording file) that points to the video file.
+ */
+static bool
+do_video_start (const SGPropertyNode * arg, SGPropertyNode * root)
+{
+    auto view_mgr = globals->get_subsystem<FGViewMgr>();
+    if (!view_mgr) return false;
+    view_mgr->video_start(
+            arg->getStringValue("name"),
+            arg->getStringValue("codec"),
+            arg->getDoubleValue("quality", -1),
+            arg->getDoubleValue("speed", -1),
+            arg->getIntValue("bitrate", 0)
+            );
+    return true;
+}
+
+/**
+ * Built-in command: video-stop.
+ */
+static bool
+do_video_stop (const SGPropertyNode * arg, SGPropertyNode * root)
+{
+  auto view_mgr = globals->get_subsystem<FGViewMgr>();
+  if (!view_mgr) return false;
+  view_mgr->video_stop();
+  return true;
+}
+
 /**
  * Built-in command: toggle a bool property value.
  *
@@ -441,10 +586,10 @@ do_property_adjust (const SGPropertyNode * arg, SGPropertyNode * root)
                 * arg->getDoubleValue("offset"));
 
   double unmodifiable, modifiable;
-  split_value(prop->getDoubleValue(), arg->getStringValue("mask", "all"),
+  split_value(prop->getDoubleValue(), arg->getStringValue("mask", "all").c_str(),
               &unmodifiable, &modifiable);
   modifiable += amount;
-  limit_value(&modifiable, arg);
+  modifiable = limit_value(modifiable, arg);
 
   prop->setDoubleValue(unmodifiable + modifiable);
 
@@ -470,13 +615,17 @@ static bool
 do_property_multiply (const SGPropertyNode * arg, SGPropertyNode * root)
 {
   SGPropertyNode * prop = get_prop(arg,root);
-  double factor = arg->getDoubleValue("factor", 1.0);
+  auto factorValue = getValueIndirect<double>(arg, "factor");
+  if (!factorValue) {
+      SG_LOG(SG_GENERAL, SG_DEV_WARN, "property-multiply: missing factor/factor-prop argument");
+      return false;
+  }
 
   double unmodifiable, modifiable;
-  split_value(prop->getDoubleValue(), arg->getStringValue("mask", "all"),
+  split_value(prop->getDoubleValue(), arg->getStringValue("mask", "all").c_str(),
               &unmodifiable, &modifiable);
-  modifiable *= factor;
-  limit_value(&modifiable, arg);
+  modifiable *= factorValue.value();
+  modifiable = limit_value(modifiable, arg);
 
   prop->setDoubleValue(unmodifiable + modifiable);
 
@@ -706,7 +855,7 @@ do_property_interpolate (const SGPropertyNode * arg, SGPropertyNode * root)
 static bool
 do_data_logging_commit (const SGPropertyNode * arg, SGPropertyNode * root)
 {
-    FGLogger *log = (FGLogger *)globals->get_subsystem("logger");
+    auto log = globals->get_subsystem<FGLogger>();
     log->reinit();
     return true;
 }
@@ -768,9 +917,8 @@ do_load_xml_to_proptree(const SGPropertyNode * arg, SGPropertyNode * root)
         if (!XMLLoader::findAirportData(icao, file.utf8Str(), file)) {
             if (!quiet) {
                 SG_LOG(SG_IO, SG_INFO, "loadxml: failed to find airport data for " << file << " at ICAO:" << icao);
-                // disable this error, too noisy until jetways.xml is fixed
-                // simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::XMLLoadCommand,
-                //                        "loadxml: no airport data file for:" + icao, file);
+                simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::XMLLoadCommand,
+                                       "loadxml: no airport data file for:" + icao, file);
             }
           return false;
         }
@@ -783,7 +931,7 @@ do_load_xml_to_proptree(const SGPropertyNode * arg, SGPropertyNode * root)
         return false;
     }
 
-    SGPath validated_path = fgValidatePath(file, false);
+    const SGPath validated_path = SGPath(file).validate(false);
     if (validated_path.isNull()) {
         SG_LOG(SG_IO, quiet ? SG_DEV_WARN : SG_ALERT, "loadxml: reading '" << file << "' denied "
                                                                                       "(unauthorized directory - authorization no longer follows symlinks; to authorize reading additional directories, pass them to --allow-nasal-read)");
@@ -813,7 +961,7 @@ do_load_xml_to_proptree(const SGPropertyNode * arg, SGPropertyNode * root)
 static bool
 do_load_xml_from_url(const SGPropertyNode * arg, SGPropertyNode * root)
 {
-    FGHTTPClient* http = static_cast<FGHTTPClient*>(globals->get_subsystem("http"));
+    auto http = globals->get_subsystem<FGHTTPClient>();
     if (!http) {
         SG_LOG(SG_IO, SG_ALERT, "xmlhttprequest: HTTP client not running");
         return false;
@@ -870,7 +1018,7 @@ do_save_xml_from_proptree(const SGPropertyNode * arg, SGPropertyNode * root)
     if (file.extension() != "xml")
         file.concat(".xml");
 
-    SGPath validated_path = fgValidatePath(file, true);
+    const SGPath validated_path = SGPath(file).validate(true);
     if (validated_path.isNull()) {
         SG_LOG(SG_IO, SG_ALERT, "savexml: writing to '" << file << "' denied "
                 "(unauthorized directory - authorization no longer follows symlinks)");
@@ -905,7 +1053,7 @@ no_profiling_support()
   SG_LOG
   (
     SG_GENERAL,
-    SG_WARN,
+    SG_ALERT,
     "No profiling support! Install gperftools and reconfigure/rebuild fgfs."
   );
 }
@@ -936,6 +1084,17 @@ do_profiler_stop(const SGPropertyNode *arg, SGPropertyNode *root)
 #endif
 }
 
+static bool do_reload_nasal_module(const SGPropertyNode* arg, SGPropertyNode*)
+{
+    auto nasalSys = globals->get_subsystem<FGNasalSys>();
+    if (!nasalSys) {
+        SG_LOG(SG_GUI, SG_ALERT, "reloadModuleFromFile command: Nasal subsystem not found");
+        return false;
+    }
+
+    return nasalSys->reloadModuleFromFile(arg->getStringValue("module"));
+}
+
 
 ////////////////////////////////////////////////////////////////////////
 // Command setup.
@@ -951,45 +1110,54 @@ do_profiler_stop(const SGPropertyNode *arg, SGPropertyNode *root)
 static struct {
   const char * name;
   SGCommandMgr::command_t command;
-} built_ins [] = {
-    { "null", do_null },
-    { "nasal", do_nasal },
-    { "pause", do_pause },
-    { "load", do_load },
-    { "save", do_save },
-    { "save-tape", do_save_tape },
-    { "load-tape", do_load_tape },
-    { "view-cycle", do_view_cycle },
+} built_ins[] = {
+    {"null", do_null},
+    {"nasal", do_nasal},
+    {"nasal-reload", do_reload_nasal_module}, // avoid conflict with modules.nas which defines 'nasal-module-reload' 
+    {"pause", do_pause},
+    {"load", do_load},
+    {"save", do_save},
+    {"save-tape", do_save_tape},
+    {"load-tape", do_load_tape},
+    {"view-cycle", do_view_cycle},
+    {"view-push", do_view_push},
+    {"view-clone", do_view_clone},
+    {"view-last-pair", do_view_last_pair},
+    {"view-last-pair-double", do_view_last_pair_double},
+    {"view-new", do_view_new},
     /*
     { "set-sea-level-air-temp-degc", do_set_sea_level_degc },
     { "set-outside-air-temp-degc", do_set_oat_degc },
     { "set-dewpoint-sea-level-air-temp-degc", do_set_dewpoint_sea_level_degc },
     { "set-dewpoint-temp-degc", do_set_dewpoint_degc },
     */
-    { "property-toggle", do_property_toggle },
-    { "property-assign", do_property_assign },
-    { "property-adjust", do_property_adjust },
-    { "property-multiply", do_property_multiply },
-    { "property-swap", do_property_swap },
-    { "property-scale", do_property_scale },
-    { "property-cycle", do_property_cycle },
-    { "property-randomize", do_property_randomize },
-    { "property-interpolate", do_property_interpolate },
-    { "data-logging-commit", do_data_logging_commit },
-    { "log-level", do_log_level },
-    { "replay", do_replay },
+    {"property-toggle", do_property_toggle},
+    {"property-assign", do_property_assign},
+    {"property-adjust", do_property_adjust},
+    {"property-multiply", do_property_multiply},
+    {"property-swap", do_property_swap},
+    {"property-scale", do_property_scale},
+    {"property-cycle", do_property_cycle},
+    {"property-randomize", do_property_randomize},
+    {"property-interpolate", do_property_interpolate},
+    {"data-logging-commit", do_data_logging_commit},
+    {"log-level", do_log_level},
+    {"replay", do_replay},
     /*
     { "decrease-visibility", do_decrease_visibility },
     { "increase-visibility", do_increase_visibility },
     */
-    { "loadxml", do_load_xml_to_proptree},
-    { "savexml", do_save_xml_from_proptree },
-    { "xmlhttprequest", do_load_xml_from_url },
+    {"loadxml", do_load_xml_to_proptree},
+    {"savexml", do_save_xml_from_proptree},
+    {"xmlhttprequest", do_load_xml_from_url},
 
-    { "profiler-start", do_profiler_start },
-    { "profiler-stop",  do_profiler_stop },
+    {"profiler-start", do_profiler_start},
+    {"profiler-stop", do_profiler_stop},
 
-    { 0, 0 }			// zero-terminated
+    {"video-start", do_video_start},
+    {"video-stop", do_video_stop},
+
+    {0, 0} // zero-terminated
 };
 
 
