@@ -20,14 +20,12 @@
 //
 // $Id$
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+#include "config.h"
 
 #include <simgear/compiler.h>
 #include <simgear/structure/exception.hxx>
 #include <simgear/props/props_io.hxx>
-
+#include <simgear/misc/strutils.hxx>
 #include <simgear/timing/sg_time.hxx>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/scene/model/particles.hxx>
@@ -38,50 +36,11 @@
 #include "globals.hxx"
 #include "fg_props.hxx"
 
-static bool frozen = false;	// FIXME: temporary
-
 using std::string;
 ////////////////////////////////////////////////////////////////////////
 // Default property bindings (not yet handled by any module).
 ////////////////////////////////////////////////////////////////////////
 
-struct LogClassMapping {
-  sgDebugClass c;
-  string name;
-  LogClassMapping(sgDebugClass cc, string nname) { c = cc; name = nname; }
-};
-
-LogClassMapping log_class_mappings [] = {
-  LogClassMapping(SG_NONE, "none"),
-  LogClassMapping(SG_TERRAIN, "terrain"),
-  LogClassMapping(SG_ASTRO, "astro"),
-  LogClassMapping(SG_FLIGHT, "flight"),
-  LogClassMapping(SG_INPUT, "input"),
-  LogClassMapping(SG_GL, "gl"),
-  LogClassMapping(SG_VIEW, "view"),
-  LogClassMapping(SG_COCKPIT, "cockpit"),
-  LogClassMapping(SG_GENERAL, "general"),
-  LogClassMapping(SG_MATH, "math"),
-  LogClassMapping(SG_EVENT, "event"),
-  LogClassMapping(SG_AIRCRAFT, "aircraft"),
-  LogClassMapping(SG_AUTOPILOT, "autopilot"),
-  LogClassMapping(SG_IO, "io"),
-  LogClassMapping(SG_CLIPPER, "clipper"),
-  LogClassMapping(SG_NETWORK, "network"),
-  LogClassMapping(SG_INSTR, "instrumentation"),
-  LogClassMapping(SG_ATC, "atc"),
-  LogClassMapping(SG_NASAL, "nasal"),
-  LogClassMapping(SG_SYSTEMS, "systems"),
-  LogClassMapping(SG_AI, "ai"),
-  LogClassMapping(SG_ENVIRONMENT, "environment"),
-  LogClassMapping(SG_SOUND, "sound"),
-  LogClassMapping(SG_NAVAID, "navaid"),
-  LogClassMapping(SG_GUI, "gui"),
-  LogClassMapping(SG_TERRASYNC, "terrasync"),
-  LogClassMapping(SG_UNDEFD, "")
-};
-
-
 /**
  * Get the logging classes.
  */
@@ -95,30 +54,14 @@ string loggingResult;
 static const char *
 getLoggingClasses ()
 {
-  sgDebugClass classes = sglog().get_log_classes();
-  loggingResult.clear();
-  for (int i = 0; log_class_mappings[i].c != SG_UNDEFD; i++) {
-    if ((classes&log_class_mappings[i].c) > 0) {
-      if (!loggingResult.empty())
-	loggingResult += '|';
-      loggingResult += log_class_mappings[i].name;
-    }
-  }
-  return loggingResult.c_str();
+    loggingResult = sglog().getLogClassesAsString();
+    return loggingResult.c_str();
 }
 
-
 static void
 addLoggingClass (const string &name)
 {
-  sgDebugClass classes = sglog().get_log_classes();
-  for (int i = 0; log_class_mappings[i].c != SG_UNDEFD; i++) {
-    if (name == log_class_mappings[i].name) {
-      sglog().set_log_classes(sgDebugClass(classes|log_class_mappings[i].c));
-      return;
-    }
-  }
-  SG_LOG(SG_GENERAL, SG_WARN, "Unknown logging class: " << name);
+    sglog().addLogClass(name);
 }
 
 
@@ -128,40 +71,9 @@ addLoggingClass (const string &name)
 void
 setLoggingClasses (const char * c)
 {
-  string classes = c;
-  sglog().set_log_classes(SG_NONE);
-
-  if (classes == "none") {
-    SG_LOG(SG_GENERAL, SG_INFO, "Disabled all logging classes");
-    return;
-  }
-
-  if (classes.empty() || classes == "all") { // default
-    sglog().set_log_classes(SG_ALL);
-    SG_LOG(SG_GENERAL, SG_INFO, "Enabled all logging classes: "
-	   << getLoggingClasses());
-    return;
-  }
-
-  string rest = classes;
-  string name = "";
-  string::size_type sep = rest.find('|');
-  if (sep == string::npos)
-    sep = rest.find(',');
-  while (sep != string::npos) {
-    name = rest.substr(0, sep);
-    rest = rest.substr(sep+1);
-    addLoggingClass(name);
-    sep = rest.find('|');
-    if (sep == string::npos)
-      sep = rest.find(',');
-  }
-  addLoggingClass(rest);
-  SG_LOG(SG_GENERAL, SG_INFO, "Set logging classes to "
-	 << getLoggingClasses());
+    sglog().parseLogClasses(c);
 }
 
-
 /**
  * Get the logging priority.
  */
@@ -196,20 +108,18 @@ setLoggingPriority (const char * p)
 {
   if (p == 0)
       return;
+
   string priority = p;
-  if (priority == "bulk") {
-    sglog().set_log_priority(SG_BULK);
-  } else if (priority == "debug") {
-    sglog().set_log_priority(SG_DEBUG);
-  } else if (priority.empty() || priority == "info") { // default
-    sglog().set_log_priority(SG_INFO);
-  } else if (priority == "warn") {
-    sglog().set_log_priority(SG_WARN);
-  } else if (priority == "alert") {
-    sglog().set_log_priority(SG_ALERT);
+  if (priority.empty()) {
+      sglog().set_log_priority(SG_INFO);
   } else {
-    SG_LOG(SG_GENERAL, SG_WARN, "Unknown logging priority " << priority);
+      try {
+          sglog().set_log_priority(logstream::priorityFromString(priority));
+      } catch (std::exception& e) {
+          SG_LOG(SG_GENERAL, SG_WARN, "Unknown logging priority: " << priority);
+      }
   }
+
   SG_LOG(SG_GENERAL, SG_DEBUG, "Logging priority is " << getLoggingPriority());
 }
 
@@ -217,35 +127,27 @@ setLoggingPriority (const char * p)
 /**
  * Return the current frozen state.
  */
-static bool
-getFreeze ()
+bool FGProperties::getFreeze() const
 {
-  return frozen;
+  return _simFreeze;
 }
 
 
 /**
  * Set the current frozen state.
  */
-static void
-setFreeze (bool f)
+void FGProperties::setFreeze(bool f)
 {
-    frozen = f;
+  if (_simFreeze == f)
+      return;
 
-    // Stop sound on a pause
-    SGSoundMgr *smgr = globals->get_subsystem<SGSoundMgr>();
-    if ( smgr != NULL ) {
-        if ( f ) {
-            smgr->suspend();
-        } else if (fgGetBool("/sim/sound/working")) {
-            smgr->resume();
-        }
-    }
+  _simFreeze = f;
 
-#ifndef FG_TESTLIB
-    // Pause the particle system
-    simgear::Particles::setFrozen(f);
-#endif
+  // Pause the particle system
+  auto p = simgear::ParticlesGlobalManager::instance();
+  p->setFrozen(f);
+
+  _simFreezeNode->fireValueChanged();
 }
 
 
@@ -274,9 +176,9 @@ getDateString ()
   }
   
   struct tm * t = st->getGmt();
-  sprintf(buf, "%.4d-%.2d-%.2dT%.2d:%.2d:%.2d",
-	  t->tm_year + 1900, t->tm_mon + 1, t->tm_mday,
-	  t->tm_hour, t->tm_min, t->tm_sec);
+  snprintf(buf, 64, "%.4d-%.2d-%.2dT%.2d:%.2d:%.2d",
+           t->tm_year + 1900, t->tm_mon + 1, t->tm_mday,
+           t->tm_hour, t->tm_min, t->tm_sec);
   return buf;
 }
 
@@ -293,6 +195,7 @@ setDateString (const char * date_string)
 
 				// Scan for basic ISO format
 				// YYYY-MM-DDTHH:MM:SS
+  new_time.tm_isdst = 0;
   int ret = sscanf(date_string, "%d-%d-%dT%d:%d:%d",
 		   &(new_time.tm_year), &(new_time.tm_mon),
 		   &(new_time.tm_mday), &(new_time.tm_hour),
@@ -307,7 +210,6 @@ setDateString (const char * date_string)
 	   << " not in YYYY-MM-DDTHH:MM:SS format; skipped");
     return;
   }
-
 				// OK, it looks like we got six
 				// values, one way or another.
   new_time.tm_year -= 1900;
@@ -340,28 +242,6 @@ getGMTString ()
   return buf;
 }
 
-/**
- * Return the current heading in degrees.
- */
-static double
-getHeadingMag ()
-{
-  double magheading = fgGetDouble("/orientation/heading-deg") -
-    fgGetDouble("/environment/magnetic-variation-deg");
-  return SGMiscd::normalizePeriodic(0, 360, magheading );
-}
-
-/**
- * Return the current track in degrees.
- */
-static double
-getTrackMag ()
-{
-  double magtrack = fgGetDouble("/orientation/track-deg") -
-    fgGetDouble("/environment/magnetic-variation-deg");
-  return SGMiscd::normalizePeriodic(0, 360, magtrack );
-}
-
 ////////////////////////////////////////////////////////////////////////
 // Tie the properties.
 ////////////////////////////////////////////////////////////////////////
@@ -369,167 +249,33 @@ SGConstPropertyNode_ptr FGProperties::_longDeg;
 SGConstPropertyNode_ptr FGProperties::_latDeg;
 SGConstPropertyNode_ptr FGProperties::_lonLatformat;
 
-/*
- * Format the latitude and longitude floats into a character array using a variety of coordinate formats.
- */
-static void
-formatLatLongString (double deg, int format, char *buf, char c) {
-  double min, sec;
-  int sign = deg < 0.0 ? -1 : 1;
-  deg = fabs(deg);
+using namespace simgear;
 
-  if (format == 0) {
-    // d.dddddd' (DDD format).
-    snprintf(buf, 32, "%3.6f%c", deg, c);
-
-  } else if (format == 1) {
-    // d mm.mmm' (DMM format) -- uses a round-off factor tailored to the
-    // required precision of the minutes field (three decimal places),
-    // preventing minute values of 60.
-    min = (deg - int(deg)) * 60.0;
-    if (min >= 59.9995) {
-        min -= 60.0;
-        deg += 1.0;
-    }
-    snprintf(buf, 32, "%d*%06.3f'%c", int(deg), fabs(min), c);
-
-  } else if (format == 2) {
-    // d mm'ss.s" (DMS format) -- uses a round-off factor tailored to the
-    // required precision of the seconds field (one decimal place),
-    // preventing second values of 60.
-    min = (deg - int(deg)) * 60.0;
-    sec = (min - int(min)) * 60.0;
-    if (sec >= 59.95) {
-        sec -= 60.0;
-        min += 1.0;
-        if (min >= 60.0) {
-            min -= 60.0;
-            deg += 1.0;
-        }
-    }
-    snprintf(buf, 32, "%d*%02d'%04.1f\"%c", int(deg), int(min), fabs(sec), c);
-
-  } else if (format == 3) {
-    // d.dddddd' (signed DDD format).
-    snprintf(buf, 32, "%3.6f", sign*deg);
-
-  } else if (format == 4) {
-    // d mm.mmm' (signed DMM format).
-    min = (deg - int(deg)) * 60.0;
-    if (min >= 59.9995) {
-        min -= 60.0;
-        deg += 1.0;
-    }
-    if (sign == 1) {
-        snprintf(buf, 32, "%d*%06.3f'", int(deg), fabs(min));
-    } else {
-        snprintf(buf, 32, "-%d*%06.3f'", int(deg), fabs(min));
-    }
-
-  } else if (format == 5) {
-    // d mm'ss.s" (signed DMS format).
-    min = (deg - int(deg)) * 60.0;
-    sec = (min - int(min)) * 60.0;
-    if (sec >= 59.95) {
-        sec -= 60.0;
-        min += 1.0;
-        if (min >= 60.0) {
-            min -= 60.0;
-            deg += 1.0;
-        }
-    }
-    if (sign == 1) {
-        snprintf(buf, 32, "%d*%02d'%04.1f\"", int(deg), int(min), fabs(sec));
-    } else {
-        snprintf(buf, 32, "-%d*%02d'%04.1f\"", int(deg), int(min), fabs(sec));
-    }
-
-  } else if (format == 6) {
-    // dd.dddddd X, ddd.dddddd X (zero padded DDD format).
-    if (c == 'N' || c == 'S') {
-      snprintf(buf, 32, "%09.6f%c", deg, c);
-    } else {
-      snprintf(buf, 32, "%010.6f%c", deg, c);
-    }
-
-  } else if (format == 7) {
-    // dd mm.mmm' X, ddd mm.mmm' X (zero padded DMM format).
-    min = (deg - int(deg)) * 60.0;
-    if (min >= 59.9995) {
-        min -= 60.0;
-        deg += 1.0;
-    }
-    if (c == 'N' || c == 'S') {
-      snprintf(buf, 32, "%02d*%06.3f'%c", int(deg), fabs(min), c);
-    } else {
-      snprintf(buf, 32, "%03d*%06.3f'%c", int(deg), fabs(min), c);
-    }
-
-  } else if (format == 8) {
-    // dd mm'ss.s" X, dd mm'ss.s" X (zero padded DMS format).
-    min = (deg - int(deg)) * 60.0;
-    sec = (min - int(min)) * 60.0;
-    if (sec >= 59.95) {
-        sec -= 60.0;
-        min += 1.0;
-        if (min >= 60.0) {
-            min -= 60.0;
-            deg += 1.0;
-        }
-    }
-    if (c == 'N' || c == 'S') {
-      snprintf(buf, 32, "%02d*%02d'%04.1f\"%c", int(deg), int(min), fabs(sec), c);
-    } else {
-      snprintf(buf, 32, "%03d*%02d'%04.1f\"%c", int(deg), int(min), fabs(sec), c);
-    }
-
-  } else if (format == 9) {
-    // dd* mm'.mmm X, ddd* mm'.mmm X (Trinity House Navigation standard).
-    min = (deg - int(deg)) * 60.0;
-    if (min >= 59.9995) {
-        min -= 60.0;
-        deg += 1.0;
-    }
-    if (c == 'N' || c == 'S') {
-        snprintf(buf, 32, "%02d* %02d'.%03d%c", int(deg), int(min), int(SGMisc<double>::round((min-int(min))*1000)), c);
-    } else {
-        snprintf(buf, 32, "%03d* %02d'.%03d%c", int(deg), int(min), int(SGMisc<double>::round((min-int(min))*1000)), c);
-    }
-
-  } else {
-    // Empty the buffer for all invalid formats.
-    buf[0] = '\0';
-  }
-}
-
-const char *
-FGProperties::getLongitudeString ()
+const char* FGProperties::getLongitudeString ()
 {
-  static char buf[32];
-  double d = _longDeg->getDoubleValue();
-  int format = _lonLatformat->getIntValue();
-
-  char c = d < 0.0 ? 'W' : 'E';
-  formatLatLongString(d, format, buf, c);
-  return buf;
+  const double d = _longDeg->getDoubleValue();
+  auto format = static_cast<strutils::LatLonFormat>(_lonLatformat->getIntValue());
+  const char c = d < 0.0 ? 'W' : 'E';
+  
+  static char longitudeBuffer[64];
+  const auto s = strutils::formatLatLonValueAsString(d, format, c);
+  memcpy(longitudeBuffer, s.c_str(), s.size() + 1);
+  return longitudeBuffer;
 }
 
-const char *
-FGProperties::getLatitudeString ()
+const char* FGProperties::getLatitudeString ()
 {
-  static char buf[32];
-  double d = _latDeg->getDoubleValue();
-  int format = _lonLatformat->getIntValue();
-
-  char c = d < 0.0 ? 'S' : 'N';
-  formatLatLongString(d, format, buf, c);
-  return buf;
+  const double d = _latDeg->getDoubleValue();
+  auto format = static_cast<strutils::LatLonFormat>(_lonLatformat->getIntValue());
+  const char c = d < 0.0 ? 'S' : 'N';
+  
+  static char latitudeBuffer[64];
+  const auto s = strutils::formatLatLonValueAsString(d, format, c);
+  memcpy(latitudeBuffer, s.c_str(), s.size() + 1);
+  return latitudeBuffer;
 }
 
 
-
-
-
 FGProperties::FGProperties ()
 {
 }
@@ -541,6 +287,18 @@ FGProperties::~FGProperties ()
 void
 FGProperties::init ()
 {
+  memset(&_lastUtc, 0, sizeof(struct tm));
+  memset(&_lastRealTime, 0, sizeof(struct tm));
+  _simFreeze = false;
+}
+
+static SGPropertyNode_ptr initDoubleNode(const std::string& path, const double v)
+{
+    auto r = fgGetNode(path, true);
+    if (r->getType() == simgear::props::NONE) {
+        r->setDoubleValue(v);
+    }
+    return r;
 }
 
 void
@@ -576,20 +334,25 @@ FGProperties::bind ()
   // Simulation
   _tiedProperties.Tie("/sim/logging/priority", getLoggingPriority, setLoggingPriority);
   _tiedProperties.Tie("/sim/logging/classes", getLoggingClasses, setLoggingClasses);
-  _tiedProperties.Tie("/sim/freeze/master", getFreeze, setFreeze);
+  _simFreezeNode = fgGetNode("/sim/freeze/master", true);
+  _tiedProperties.Tie(_simFreezeNode, this, &FGProperties::getFreeze, &FGProperties::setFreeze);
+  _simFreezeNode->setAttribute(SGPropertyNode::LISTENER_SAFE, true);
 
   _tiedProperties.Tie<double>("/sim/time/elapsed-sec", getElapsedTime_sec);
-  _tiedProperties.Tie("/sim/time/gmt", getDateString, setDateString);
+  _timeGmtNode = _tiedProperties.Tie("/sim/time/gmt", getDateString, setDateString);
+  _timeGmtNode->setAttribute(SGPropertyNode::LISTENER_SAFE, true);
   fgSetArchivable("/sim/time/gmt");
-  _tiedProperties.Tie<const char*>("/sim/time/gmt-string", getGMTString);
-
+  _timeGmtStringNode = _tiedProperties.Tie<const char*>("/sim/time/gmt-string", getGMTString);
+  _timeGmtStringNode->setAttribute(SGPropertyNode::LISTENER_SAFE, true);
   // Position
   _tiedProperties.Tie<const char*>("/position/latitude-string", getLatitudeString);
   _tiedProperties.Tie<const char*>("/position/longitude-string", getLongitudeString);
 
-  // Orientation
-  _tiedProperties.Tie<double>("/orientation/heading-magnetic-deg", getHeadingMag);
-  _tiedProperties.Tie<double>("/orientation/track-magnetic-deg", getTrackMag);
+    _headingMagnetic = initDoubleNode("/orientation/heading-magnetic-deg", 0.0);
+    _trackMagnetic = initDoubleNode("/orientation/track-magnetic-deg", 0.0);
+    _magVar = initDoubleNode("/environment/magnetic-variation-deg", 0.0);
+    _trueHeading = initDoubleNode("/orientation/heading-deg", 0.0);
+    _trueTrack = initDoubleNode("/orientation/track-deg", 0.0);
 }
 
 void
@@ -601,6 +364,10 @@ FGProperties::unbind ()
     _longDeg = 0;
     _latDeg = 0;
     _lonLatformat = 0;
+
+    _timeGmtNode.clear();
+    _timeGmtStringNode.clear();
+    _simFreezeNode.clear();
 }
 
 void
@@ -610,29 +377,48 @@ FGProperties::update (double dt)
 
     // utc date/time
     struct tm *u = globals->get_time_params()->getGmt();
-    _uyear->setIntValue(u->tm_year + 1900);
-    _umonth->setIntValue(u->tm_mon + 1);
-    _uday->setIntValue(u->tm_mday);
-    _uhour->setIntValue(u->tm_hour);
-    _umin->setIntValue(u->tm_min);
-    _usec->setIntValue(u->tm_sec);
-    _uwday->setIntValue(u->tm_wday);
-    _udsec->setIntValue(u->tm_hour * 3600 + u->tm_min * 60 + u->tm_sec);
+    if (memcmp(u, &_lastUtc, sizeof(struct tm)) != 0) {
+        _lastUtc = *u;
+        _uyear->setIntValue(u->tm_year + 1900);
+        _umonth->setIntValue(u->tm_mon + 1);
+        _uday->setIntValue(u->tm_mday);
+        _uhour->setIntValue(u->tm_hour);
+        _umin->setIntValue(u->tm_min);
+        _usec->setIntValue(u->tm_sec);
+        _uwday->setIntValue(u->tm_wday);
+        _udsec->setIntValue(u->tm_hour * 3600 + u->tm_min * 60 + u->tm_sec);
+
+        _timeGmtNode->fireValueChanged();
+        _timeGmtStringNode->fireValueChanged();
+    }
+
 
     // real local date/time
     time_t real = time(0);
     struct tm *r = localtime(&real);
-    _ryear->setIntValue(r->tm_year + 1900);
-    _rmonth->setIntValue(r->tm_mon + 1);
-    _rday->setIntValue(r->tm_mday);
-    _rhour->setIntValue(r->tm_hour);
-    _rmin->setIntValue(r->tm_min);
-    _rsec->setIntValue(r->tm_sec);
-    _rwday->setIntValue(r->tm_wday);
+    if (memcmp(r, &_lastRealTime, sizeof(struct tm)) != 0) {
+        _lastRealTime = *r;
+        _ryear->setIntValue(r->tm_year + 1900);
+        _rmonth->setIntValue(r->tm_mon + 1);
+        _rday->setIntValue(r->tm_mday);
+        _rhour->setIntValue(r->tm_hour);
+        _rmin->setIntValue(r->tm_min);
+        _rsec->setIntValue(r->tm_sec);
+        _rwday->setIntValue(r->tm_wday);
+    }
+
+    const double magvar = _magVar->getDoubleValue();
+    const auto hdgMag = SGMiscd::normalizePeriodic(0, 360.0, _trueHeading->getDoubleValue() - magvar);
+    _headingMagnetic->setDoubleValue(hdgMag);
+    
+    const auto trackMag = SGMiscd::normalizePeriodic(0, 360.0, _trueTrack->getDoubleValue() - magvar);
+    _trackMagnetic->setDoubleValue(trackMag);
 }
 
 
-
+// Register the subsystem.
+SGSubsystemMgr::Registrant<FGProperties> registrantFGProperties;
+
 ////////////////////////////////////////////////////////////////////////
 // Save and restore.
 ////////////////////////////////////////////////////////////////////////
@@ -708,7 +494,6 @@ fgLoadProps (const std::string& path, SGPropertyNode * props, bool in_fg_root, i
 }
 
 
-
 ////////////////////////////////////////////////////////////////////////
 // Property convenience functions.
 ////////////////////////////////////////////////////////////////////////
@@ -774,7 +559,7 @@ fgGetDouble (const char * name, double defaultValue)
   return globals->get_props()->getDoubleValue(name, defaultValue);
 }
 
-const char *
+std::string
 fgGetString (const char * name, const char * defaultValue)
 {
   return globals->get_props()->getStringValue(name, defaultValue);
@@ -870,5 +655,21 @@ fgUntie(const char * name)
   }
 }
 
+void fgUntieIfDefined(const std::string& name)
+{
+    SGPropertyNode* node = globals->get_props()->getNode(name);
+    if (!node) {
+        return;
+    }
+
+    if (!node->isTied()) {
+        return;
+    }
+
+    if (!node->untie()) {
+        SG_LOG(SG_GENERAL, SG_WARN, "Failed to untie property " << name);
+    }
+}
+
 
 // end of fg_props.cxx
