@@ -1,4 +1,7 @@
-// fg_scene_commands.cxx - internal FGFS commands.
+/*
+ * SPDX-FileName: fg_scene_commands.cxx
+ * SPDX-FileComment: internal FGFS commands
+ */
 
 #include "config.h"
 
@@ -11,7 +14,7 @@
 
 #include <simgear/sg_inlines.h>
 #include <simgear/debug/logstream.hxx>
-#include <simgear/math/sg_random.h>
+#include <simgear/math/sg_random.hxx>
 #include <simgear/io/iostreams/sgstream.hxx>
 #include <simgear/scene/material/mat.hxx>
 #include <simgear/scene/material/matlib.hxx>
@@ -34,6 +37,7 @@
 #include <Sound/sample_queue.hxx>
 #include <Airports/xmlloader.hxx>
 #include <Network/HTTPClient.hxx>
+#include <Viewer/CameraGroup.hxx>
 #include <Viewer/viewmgr.hxx>
 #include <Viewer/view.hxx>
 #include <Environment/presets.hxx>
@@ -50,8 +54,6 @@
 #include "main.hxx"
 #include "positioninit.hxx"
 
-#include <boost/scoped_array.hpp>
-
 #if FG_HAVE_GPERFTOOLS
 # include <google/profiler.h>
 #endif
@@ -99,6 +101,19 @@ static bool
 do_switch_aircraft (const SGPropertyNode * arg, SGPropertyNode * root)
 {
     fgSetString("/sim/aircraft", arg->getStringValue("aircraft"));
+
+    // we could validate the new aircraft is find-able, and if not, bail out
+    // of the command at this point? Might give a better user experience?
+    // would need something like bool fgIsValidAircraft(name, dir);
+    // which could be cooked up by modifying FindAndCacheAircraft
+
+    // We assume that we are changing the aircraft, so we remove the aircraft path.
+    // this forces aircraft-dir to be recomputed by a full search as the user hopefully
+    // expects. 
+    // We could allow pasing the dir as an optional argument to this command, if that
+    // is ever needed in the future.
+    fgGetNode("/sim")->removeChild("aircraft-dir");
+
     // start a reset
     fgResetIdleState();
     return true;
@@ -123,7 +138,7 @@ do_reposition (const SGPropertyNode * arg, SGPropertyNode * root)
 static bool
 do_panel_load (const SGPropertyNode * arg, SGPropertyNode * root)
 {
-  string panel_path = arg->getStringValue("path");
+  std::string panel_path = arg->getStringValue("path");
   if (!panel_path.empty()) {
     // write to the standard property, which will force a load
     fgSetString("/sim/panel/path", panel_path.c_str());
@@ -217,6 +232,15 @@ do_hires_screen_capture (const SGPropertyNode * arg, SGPropertyNode * root)
   return true;
 }
 
+/**
+ * Reload all Compositor instances in the default CameraGroup.
+ */
+static bool
+do_reload_compositor(const SGPropertyNode*, SGPropertyNode*)
+{
+    flightgear::reloadCompositors(flightgear::CameraGroup::getDefault());
+    return true;
+}
 
 /**
  * Reload the tile cache.
@@ -231,7 +255,7 @@ do_tile_cache_reload (const SGPropertyNode * arg, SGPropertyNode * root)
         master_freeze->setBoolValue(true);
     }
 
-    globals->get_subsystem("scenery")->reinit();
+    globals->get_subsystem<FGScenery>()->reinit();
 
     if ( !freeze ) {
         master_freeze->setBoolValue(false);
@@ -249,8 +273,8 @@ do_materials_reload (const SGPropertyNode * arg, SGPropertyNode * root)
     SGMaterialLib* new_matlib =  new SGMaterialLib;
     SGPath mpath( globals->get_fg_root() );
     mpath.append( fgGetString("/sim/rendering/materials-file") );
-    bool loaded = new_matlib->load(globals->get_fg_root().local8BitStr(),
-                                  mpath.local8BitStr(),
+    bool loaded = new_matlib->load(globals->get_fg_root(),
+                                  mpath,
                                   globals->get_props());
 
     if ( ! loaded ) {
@@ -276,7 +300,7 @@ do_materials_reload (const SGPropertyNode * arg, SGPropertyNode * root)
 static bool
 do_dialog_new (const SGPropertyNode * arg, SGPropertyNode * root)
 {
-    NewGUI * gui = (NewGUI *)globals->get_subsystem("gui");
+    auto gui = globals->get_subsystem<NewGUI>();
     if (!gui) {
       return false;
     }
@@ -300,11 +324,24 @@ do_dialog_new (const SGPropertyNode * arg, SGPropertyNode * root)
 static bool
 do_dialog_show (const SGPropertyNode * arg, SGPropertyNode * root)
 {
-    NewGUI * gui = (NewGUI *)globals->get_subsystem("gui");
+    auto gui = globals->get_subsystem<NewGUI>();
     gui->showDialog(arg->getStringValue("dialog-name"));
     return true;
 }
 
+/**
+ * Built-in command: Show an XML-configured dialog.
+ *
+ * dialog-name: the name of the GUI dialog to display.
+ */
+static bool
+do_dialog_toggle (const SGPropertyNode * arg, SGPropertyNode * root)
+{
+    auto gui = globals->get_subsystem<NewGUI>();
+    gui->toggleDialog(arg->getStringValue("dialog-name"));
+    return true;
+}
+
 
 /**
  * Built-in Command: Hide the active XML-configured dialog.
@@ -312,7 +349,7 @@ do_dialog_show (const SGPropertyNode * arg, SGPropertyNode * root)
 static bool
 do_dialog_close (const SGPropertyNode * arg, SGPropertyNode * root)
 {
-    NewGUI * gui = (NewGUI *)globals->get_subsystem("gui");
+    auto gui = globals->get_subsystem<NewGUI>();
     if(arg->hasValue("dialog-name"))
         return gui->closeDialog(arg->getStringValue("dialog-name"));
     return gui->closeActiveDialog();
@@ -327,7 +364,7 @@ do_dialog_close (const SGPropertyNode * arg, SGPropertyNode * root)
 static bool
 do_dialog_update (const SGPropertyNode * arg, SGPropertyNode * root)
 {
-    NewGUI * gui = (NewGUI *)globals->get_subsystem("gui");
+    auto gui = globals->get_subsystem<NewGUI>();
     FGDialog * dialog;
     if (arg->hasValue("dialog-name"))
         dialog = gui->getDialog(arg->getStringValue("dialog-name"));
@@ -345,7 +382,7 @@ do_dialog_update (const SGPropertyNode * arg, SGPropertyNode * root)
 static bool
 do_open_browser (const SGPropertyNode * arg, SGPropertyNode * root)
 {
-    string path;
+    std::string path;
     if (arg->hasValue("path"))
         path = arg->getStringValue("path");
     else
@@ -380,14 +417,14 @@ do_open_launcher(const SGPropertyNode*, SGPropertyNode*)
 static bool
 do_dialog_apply (const SGPropertyNode * arg, SGPropertyNode * root)
 {
-    NewGUI * gui = (NewGUI *)globals->get_subsystem("gui");
-    FGDialog * dialog;
+    auto gui = globals->get_subsystem<NewGUI>();
+    FGDialog* dialog = nullptr;
     if (arg->hasValue("dialog-name"))
         dialog = gui->getDialog(arg->getStringValue("dialog-name"));
     else
         dialog = gui->getActiveDialog();
 
-    if (dialog != 0) {
+    if (dialog) {
         dialog->applyValues(arg->getStringValue("object-name"));
         return true;
     } else {
@@ -403,7 +440,7 @@ do_dialog_apply (const SGPropertyNode * arg, SGPropertyNode * root)
 static bool
 do_gui_redraw (const SGPropertyNode * arg, SGPropertyNode * root)
 {
-    NewGUI * gui = (NewGUI *)globals->get_subsystem("gui");
+    auto gui = globals->get_subsystem<NewGUI>();
     gui->redraw();
     return true;
 }
@@ -451,12 +488,12 @@ do_presets_commit (const SGPropertyNode * arg, SGPropertyNode * root)
 static bool
 do_press_cockpit_button (const SGPropertyNode * arg, SGPropertyNode * root)
 {
-  const char *prefix = arg->getStringValue("prefix");
+  const std::string prefix = arg->getStringValue("prefix");
 
-  if (arg->getBoolValue("guarded") && fgGetDouble((string(prefix) + "-guard").c_str()) < 1)
+  if (arg->getBoolValue("guarded") && fgGetDouble((prefix + "-guard").c_str()) < 1)
     return true;
 
-  string prop = string(prefix) + "-button";
+  std::string prop = prefix + "-button";
   double value;
 
   if (arg->getBoolValue("latching"))
@@ -473,10 +510,10 @@ do_press_cockpit_button (const SGPropertyNode * arg, SGPropertyNode * root)
 static bool
 do_release_cockpit_button (const SGPropertyNode * arg, SGPropertyNode * root)
 {
-  const char *prefix = arg->getStringValue("prefix");
+  const std::string prefix = arg->getStringValue("prefix");
 
   if (arg->getBoolValue("guarded")) {
-    string prop = string(prefix) + "-guard";
+    std::string prop = prefix + "-guard";
     if (fgGetDouble(prop.c_str()) < 1) {
       fgSetDouble(prop.c_str(), 1);
       return true;
@@ -484,7 +521,7 @@ do_release_cockpit_button (const SGPropertyNode * arg, SGPropertyNode * root)
   }
 
   if (! arg->getBoolValue("latching")) {
-    fgSetDouble((string(prefix) + "-button").c_str(), 0);
+    fgSetDouble((prefix + "-button").c_str(), 0);
     fgSetBool(arg->getStringValue("discrete"), false);
   }
 
@@ -518,6 +555,7 @@ static struct {
     { "tile-cache-reload", do_tile_cache_reload },
     { "dialog-new", do_dialog_new },
     { "dialog-show", do_dialog_show },
+    { "dialog-toggle", do_dialog_toggle },
     { "dialog-close", do_dialog_close },
     { "dialog-update", do_dialog_update },
     { "dialog-apply", do_dialog_apply },
@@ -532,6 +570,7 @@ static struct {
     { "print-visible-scene", do_print_visible_scene_info },
     { "reload-shaders", do_reload_shaders },
     { "reload-materials", do_materials_reload },
+    { "reload-compositor", do_reload_compositor },
     { "open-launcher", do_open_launcher },
     { 0, 0 }			// zero-terminated
 };
