@@ -84,7 +84,7 @@ public:
     }
 
   // test against the aircraft-dir property
-    const char* aircraftDir = fgGetString("/sim/aircraft-dir");
+    const std::string aircraftDir = fgGetString("/sim/aircraft-dir");
     string_list aircraftDirPieces(sgPathBranchSplit(aircraftDir));
     if (!aircraftDirPieces.empty() && (aircraftDirPieces.back() == pieces[1])) {
         // current aircraft-dir matches resource aircraft
@@ -135,17 +135,6 @@ public:
   }
 };
 
-class CompositorEffectsProvider : public simgear::ResourceProvider {
-public:
-    CompositorEffectsProvider() :
-        simgear::ResourceProvider(simgear::ResourceManager::PRIORITY_NORMAL) {
-    }
-    SGPath resolve(const std::string &aResource, SGPath&) const override {
-        const SGPath p = globals->get_fg_root() / "Compositor" / aResource;
-        return p.exists() ? p : SGPath();
-    }
-};
-
 ////////////////////////////////////////////////////////////////////////
 // Implementation of FGGlobals.
 ////////////////////////////////////////////////////////////////////////
@@ -177,9 +166,6 @@ FGGlobals::FGGlobals() :
     resMgr->addProvider(new AircraftResourceProvider());
     resMgr->addProvider(new CurrentAircraftDirProvider());
     resMgr->addProvider(new flightgear::addons::ResourceProvider());
-#ifdef ENABLE_COMPOSITOR
-    resMgr->addProvider(new CompositorEffectsProvider());
-#endif
     initProperties();
 }
 
@@ -195,6 +181,10 @@ void FGGlobals::initProperties()
     viewLat = props->getNode("sim/current-view/viewer-lat-deg", true);
     viewAlt = props->getNode("sim/current-view/viewer-elev-ft", true);
 
+    referenceOffsetX = props->getNode("sim/model/reference-offset-x", true);
+    referenceOffsetY = props->getNode("sim/model/reference-offset-y", true);
+    referenceOffsetZ = props->getNode("sim/model/reference-offset-z", true);
+
     orientPitch = props->getNode("orientation/pitch-deg", true);
     orientHeading = props->getNode("orientation/heading-deg", true);
     orientRoll = props->getNode("orientation/roll-deg", true);
@@ -210,15 +200,15 @@ FGGlobals::~FGGlobals()
     // stop OSG threading first, to avoid thread races while we tear down
     // scene-graph pieces
     // there are some scenarios where renderer is already gone.
-    osg::ref_ptr<osgViewer::Viewer> vw;
+    osg::ref_ptr<osgViewer::ViewerBase> vb;
     if (renderer) {
-        vw = renderer->getViewer();
-        if (vw) {
+        vb = renderer->getViewerBase();
+        if (vb) {
             // https://code.google.com/p/flightgear-bugs/issues/detail?id=1291
             // explicitly stop trheading before we delete the renderer or
             // viewMgr (which ultimately holds refs to the CameraGroup, and
             // GraphicsContext)
-            vw->stopThreading();
+            vb->stopThreading();
         }
     }
 
@@ -227,9 +217,10 @@ FGGlobals::~FGGlobals()
 
     // don't cancel the pager until after shutdown, since AIModels (and
     // potentially others) can queue delete requests on the pager.
-    if (vw && vw->getDatabasePager()) {
-        vw->getDatabasePager()->cancel();
-        vw->getDatabasePager()->clear();
+    osgViewer::View* v = renderer->getView();
+    if (v && v->getDatabasePager()) {
+        v->getDatabasePager()->cancel();
+        v->getDatabasePager()->clear();
     }
 
     osgDB::Registry::instance()->clearObjectCache();
@@ -245,7 +236,7 @@ FGGlobals::~FGGlobals()
 
     delete subsystem_mgr;
     subsystem_mgr = nullptr; // important so ::get_subsystem returns NULL
-    vw = nullptr;
+    vb = nullptr;
     set_matlib(NULL);
 
     delete time_params;
@@ -288,7 +279,7 @@ void FGGlobals::set_fg_root (const SGPath &root) {
                 << fg_root << "'\n***\n***");
     }
 
-    // deliberately not a tied property, for fgValidatePath security
+    // deliberately not a tied property, for SGPath::validate() security
     // write-protect to avoid accidents
     SGPropertyNode *n = fgGetNode("/sim", true);
     n->removeChild("fg-root", 0);
@@ -378,7 +369,6 @@ SGPath FGGlobals::findDataPath(const std::string& pathSuffix) const
         }
     }
 
-    SG_LOG(SG_IO, SG_WARN, "not found in any data path: '" << pathSuffix << "'");
     return SGPath{};
 }
 
@@ -412,7 +402,7 @@ void FGGlobals::append_fg_scenery (const SGPath &path)
         return;
     }
 
-    // tell the ResouceManager about the scenery path
+    // tell the ResourceManager about the scenery path
     // needed to load Models from this scenery path
     simgear::ResourceManager::instance()->addBasePath(abspath, simgear::ResourceManager::PRIORITY_DEFAULT);
 
@@ -426,6 +416,35 @@ void FGGlobals::append_fg_scenery (const SGPath &path)
 
     // temporary fix so these values survive reset
     n->setAttribute(SGPropertyNode::PRESERVE, true);
+
+    // Add any sources.xml file to the property tree so we can provide attribution through the GUI
+    SGPath sources = SGPath(abspath);
+    sources.append("sources.xml");
+    SG_LOG(SG_TERRAIN, SG_DEBUG, "Looking for source file " << sources << ". Exists ? " << sources.exists());
+    if (sources.exists()) {
+        // Determine the next free indice under /scenery/sources
+        SGPropertyNode* sourcesProp = fgGetNode("/scenery/sources", true);
+        int propIndex = 0;
+        while (sourcesProp->getChild("directory", propIndex) != NULL) {
+            ++propIndex;
+        }
+
+        sourcesProp = sourcesProp->getChild("directory", propIndex++, true);
+
+        // Add a reference to the path itself
+        sourcesProp->setStringValue("path", abspath.utf8Str());
+
+        // Now load the sources file into /scenery/sources/source[n]
+        if(!fgLoadProps(sources.utf8Str(), sourcesProp, false))
+        {
+            SG_LOG(SG_TERRAIN, SG_ALERT, "Unable to load sources file " << sources.utf8Str());
+        }
+
+        // Ensure these properties cannot be over-ridden, and preserve them across a reset.
+        sourcesProp->setAttribute(SGPropertyNode::WRITE, false);
+        sourcesProp->setAttribute(SGPropertyNode::PRESERVE, true);
+    }
+
 }
 
 void FGGlobals::append_read_allowed_paths(const SGPath &path)
@@ -477,7 +496,7 @@ void FGGlobals::set_terrasync_dir(const SGPath &path)
   }
   SGPath abspath(path.realpath());
   terrasync_dir = abspath;
-  // deliberately not a tied property, for fgValidatePath security
+  // deliberately not a tied property, for SGPath::validate() security
   // write-protect to avoid accidents
   SGPropertyNode *n = fgGetNode("/sim/terrasync/scenery-dir", true);
   n->setAttribute(SGPropertyNode::WRITE, true);
@@ -583,15 +602,6 @@ FGGlobals::get_subsystem (const char * name) const
     return subsystem_mgr->get_subsystem(name);
 }
 
-void
-FGGlobals::add_subsystem (const char * name,
-                          SGSubsystem * subsystem,
-                          SGSubsystemMgr::GroupType type,
-                          double min_time_sec)
-{
-    subsystem_mgr->add(name, subsystem, type, min_time_sec);
-}
-
 SGEventMgr *
 FGGlobals::get_event_mgr () const
 {
@@ -632,6 +642,21 @@ FGGlobals::get_view_position_cart() const
 {
   return SGVec3d::fromGeod(get_view_position());
 }
+SGVec3d FGGlobals::get_ownship_reference_position_cart() const
+{
+    SGVec3d  pos = get_aircraft_position_cart();
+
+    if (referenceOffsetX)
+        pos[0] += referenceOffsetX->getDoubleValue();
+
+    if (referenceOffsetY)
+        pos[1] += referenceOffsetY->getDoubleValue();
+
+    if (referenceOffsetZ)
+        pos[2] += referenceOffsetZ->getDoubleValue();
+
+    return pos;
+}
 
 static void treeDumpRefCounts(int depth, SGPropertyNode* nd)
 {
@@ -922,12 +947,12 @@ void FGGlobals::set_warp_delta( long int d )
 
 FGScenery* FGGlobals::get_scenery () const
 {
-    return get_subsystem<FGScenery>();
+    return subsystem_mgr->get_subsystem<FGScenery>();
 }
 
 FGViewMgr *FGGlobals::get_viewmgr() const
 {
-    return get_subsystem<FGViewMgr>();
+    return subsystem_mgr->get_subsystem<FGViewMgr>();
 }
 
 flightgear::View* FGGlobals::get_current_view () const
@@ -943,7 +968,7 @@ void FGGlobals::set_matlib( SGMaterialLib *m )
 
 FGControls *FGGlobals::get_controls() const
 {
-    return get_subsystem<FGControls>();
+    return subsystem_mgr->get_subsystem<FGControls>();
 }
 
 void FGGlobals::addListenerToCleanup(SGPropertyChangeListener* l)
