@@ -18,9 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA.
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+#include <config.h>
 
 #ifdef HAVE_WINDOWS_H
 #include <windows.h>
@@ -31,18 +29,24 @@
 #include <cstddef>              // std::size_t
 #include <cassert>
 
+#include <simgear/misc/strutils.hxx>
+#include <simgear/props/props.hxx>
 #include <simgear/props/props_io.hxx>
 #include <simgear/structure/exception.hxx>
 
 #include "fg_props.hxx"
 #include "locale.hxx"
+#include "XLIFFParser.hxx"
+
+#include <Add-ons/AddonMetadataParser.hxx>
 
 using std::vector;
 using std::string;
+namespace strutils = simgear::strutils;
 
 FGLocale::FGLocale(SGPropertyNode* root) :
-    _intl(root->getNode("/sim/intl",0, true)),
-    _defaultLocale(_intl->getChild("locale",0, true))
+	_intl(root->getNode("/sim/intl", 0, true)),
+	_defaultLocale(_intl->getChild("locale", 0, true))
 {
 }
 
@@ -67,25 +71,61 @@ string FGLocale::removeEncodingPart(const string& locale)
     return res;
 }
 
+string removeLocalePart(const string& locale)
+{
+    std::size_t pos = locale.find('_');
+    if (pos == string::npos) {
+        pos = locale.find('-');
+    }
+
+    if (pos == string::npos)
+        return {};
+
+    return locale.substr(0, pos);
+}
+
 #ifdef _WIN32
 
 
 string_list
-FGLocale::getUserLanguage()
+FGLocale::getUserLanguages()
 {
-    string_list result;
-    static wchar_t localeNameBuf[LOCALE_NAME_MAX_LENGTH];
+	unsigned long bufSize = 128;
+	wchar_t* localeNameBuf = reinterpret_cast<wchar_t*>(alloca(bufSize));
+	unsigned long numLanguages = 0;
 
-    if (GetUserDefaultLocaleName(localeNameBuf, LOCALE_NAME_MAX_LENGTH)) {
-        std::wstring ws(localeNameBuf);
-        std::string localeNameUTF8 = simgear::strutils::convertWStringToUtf8(ws);
+	bool ok = GetUserPreferredUILanguages(MUI_LANGUAGE_NAME, &numLanguages,
+		localeNameBuf, &bufSize);
+	if (!ok) {
+		// if we have a lot of languages, can fail, allocate a bigger
+		// buffer
+		if (GetLastError() == ERROR_INSUFFICIENT_BUFFER) {
+			bufSize = 0;
+			GetUserPreferredUILanguages(MUI_LANGUAGE_NAME, &numLanguages,
+				nullptr, &bufSize);
+			localeNameBuf = reinterpret_cast<wchar_t*>(alloca(bufSize));
+			ok = GetUserPreferredUILanguages(MUI_LANGUAGE_NAME, &numLanguages,
+				localeNameBuf, &bufSize);
+		}
+	}
 
-        SG_LOG(SG_GENERAL, SG_INFO, "Detected user locale:" << localeNameUTF8);
-        result.push_back(localeNameUTF8);
-        return result;
-    } else {
-        SG_LOG(SG_GENERAL, SG_WARN, "Failed to detected user locale");
-    }
+	if (!ok) {
+		SG_LOG(SG_GENERAL, SG_WARN, "Failed to detected user locale via GetUserPreferredUILanguages");
+		return{};
+	}
+
+	string_list result;
+	result.reserve(numLanguages);
+	for (unsigned int l = 0; l < numLanguages; ++l) {
+		std::wstring ws(localeNameBuf);
+		if (ws.empty())
+			break;
+
+		// skip to next string, past this string and trailing NULL
+		localeNameBuf += (ws.size() + 1);
+		result.push_back(simgear::strutils::convertWStringToUtf8(ws));
+		SG_LOG(SG_GENERAL, SG_INFO, "User langauge " << l << ":" << result.back());
+	}
 
     return result;
 }
@@ -96,7 +136,7 @@ FGLocale::getUserLanguage()
  * Determine locale/language settings on Linux/Unix.
  */
 string_list
-FGLocale::getUserLanguage()
+FGLocale::getUserLanguages()
 {
     string_list result;
     const char* langEnv = ::getenv("LANG");
@@ -141,10 +181,9 @@ FGLocale::findLocaleNode(const string& localeSpec)
     }
 
     // try country's default resource, i.e. "de_DE" => "de"
-    std::size_t pos = language.find('_');
-    if ((pos != string::npos) && (pos > 0))
-    {
-        node = findLocaleNode(language.substr(0, pos));
+    const auto justTheLanguage = removeLocalePart(language);
+    if (!justTheLanguage.empty()) {
+        node = findLocaleNode(justTheLanguage);
         if (node)
             return node;
     }
@@ -154,29 +193,32 @@ FGLocale::findLocaleNode(const string& localeSpec)
 
 // Select the language. When no language is given (nullptr),
 // a default is determined matching the system locale.
-bool
-FGLocale::selectLanguage(const char *language)
+bool FGLocale::selectLanguage(const std::string& language)
 {
-    string_list languages = getUserLanguage();
-    if (languages.empty()) {
+    _languages = getUserLanguages();
+    if (_languages.empty()) {
         // Use plain C locale if nothing is available.
         SG_LOG(SG_GENERAL, SG_WARN, "Unable to detect system language" );
-        languages.push_back("C");
+        _languages.push_back("C");
     }
 
     // if we were passed a language option, try it first
-    if ((language != nullptr) && (std::strlen(language) > 0)) {
-        languages.insert(languages.begin(), string(language));
+    if (!language.empty()) {
+        const auto normalizedLang = strutils::replace(language, "-", "_");
+        _languages.insert(_languages.begin(), normalizedLang);
     }
 
-    _currentLocaleString = removeEncodingPart(languages[0]);
+    _currentLocaleString = removeEncodingPart(_languages.front());
     if (_currentLocaleString == "C") {
         _currentLocaleString.clear();
     }
+    // Record the current locale at /sim/intl/current-locale
+    _intl->getChild("current-locale", 0, true)
+         ->setStringValue(_currentLocaleString);
 
     _currentLocale = nullptr;
 
-    for (const string& lang: languages) {
+    for (const string& lang : _languages) {
         SG_LOG(SG_GENERAL, SG_DEBUG,
                "Trying to find locale for '" << lang << "'");
         _currentLocale = findLocaleNode(lang);
@@ -189,24 +231,41 @@ FGLocale::selectLanguage(const char *language)
         }
     }
 
+    if (_currentLocale && _currentLocale->hasChild("xliff")) {
+        parseXLIFF(_currentLocale);
+    }
+   
+    
     // load resource for system messages (translations for fgfs internal messages)
     loadResource("sys");
-
-    // load resource for atc messages
     loadResource("atc");
-
     loadResource("tips");
+    loadResource("weather-scenarios");
 
-    if (!_currentLocale)
-    {
-       SG_LOG(SG_GENERAL, SG_ALERT,
-              "System locale not found or no internationalization settings specified in defaults.xml. Using default (en)." );
-       return false;
+    _inited = true;
+    if (!_currentLocale && !_currentLocaleString.empty()) {
+        SG_LOG(SG_GENERAL, SG_WARN,
+               "System locale not found or no internationalization settings specified in defaults.xml. Using default (en).");
+        return false;
     }
 
     return true;
 }
 
+void FGLocale::clear()
+{
+    _inited = false;
+    _currentLocaleString.clear();
+    _languages.clear();
+
+    if (_currentLocale && (_currentLocale != _defaultLocale)) {
+        // remove loaded strings, so we don't duplicate
+        _currentLocale->removeChild("strings");
+    }
+
+    _currentLocale.clear();
+}
+
 // Return the preferred language according to user choice and/or settings
 // (e.g., 'fr_FR', or the empty string if nothing could be found).
 std::string
@@ -215,6 +274,28 @@ FGLocale::getPreferredLanguage() const
     return _currentLocaleString;
 }
 
+void FGLocale::parseXLIFF(SGPropertyNode* localeNode)
+{
+    SGPath path(globals->get_fg_root());
+    SGPath xliffPath = path / localeNode->getStringValue("xliff");
+    if (!xliffPath.exists()) {
+        SG_LOG(SG_GENERAL, SG_ALERT, "No XLIFF file at " << xliffPath);
+    } else {
+        SG_LOG(SG_GENERAL, SG_INFO, "Loading XLIFF file at " << xliffPath);
+        SGPropertyNode_ptr stringNode = localeNode->getNode("strings", 0, true);
+        try {
+            flightgear::XLIFFParser visitor(stringNode);
+            readXML(xliffPath, visitor);
+        } catch (sg_io_exception& ex) {
+            SG_LOG(SG_GENERAL, SG_WARN, "failure parsing XLIFF: " << path <<
+                   "\n\t" << ex.getMessage() << "\n\tat:" << ex.getLocation().asString());
+        } catch (sg_exception& ex) {
+            SG_LOG(SG_GENERAL, SG_WARN, "failure parsing XLIFF: " << path <<
+                   "\n\t" << ex.getMessage());
+        }
+    }
+}
+
 // Load strings for requested resource and locale.
 // Result is stored below "strings" in the property tree of the given locale.
 bool
@@ -222,10 +303,24 @@ FGLocale::loadResource(SGPropertyNode* localeNode, const char* resource)
 {
     SGPath path( globals->get_fg_root() );
 
+    // already loaded
+    if (localeNode->hasChild("xliff")) {
+        return true;
+    }
+    
     SGPropertyNode* stringNode = localeNode->getNode("strings", 0, true);
+    SGPropertyNode* resourceNode = stringNode->getNode(resource);
 
-    const char *path_str = stringNode->getStringValue(resource, nullptr);
-    if (!path_str)
+    if (!resourceNode)
+        return false;
+
+    if (resourceNode->getBoolValue("__loaded")) {
+        // already loaded previously
+        return true;
+    }
+
+    string path_str = resourceNode->getStringValue();
+    if (path_str.empty())
     {
         SG_LOG(SG_GENERAL, SG_WARN, "No path in " << stringNode->getPath() << "/" << resource << ".");
         return false;
@@ -239,7 +334,7 @@ FGLocale::loadResource(SGPropertyNode* localeNode, const char* resource)
     // load the actual file
     try
     {
-        readProperties(path, stringNode->getNode(resource, 0, true));
+        readProperties(path, resourceNode);
     } catch (const sg_exception &e)
     {
         SG_LOG(SG_GENERAL, SG_ALERT, "Unable to read the localized strings from " << path <<
@@ -247,7 +342,9 @@ FGLocale::loadResource(SGPropertyNode* localeNode, const char* resource)
         return false;
     }
 
-   return true;
+    // set marker to indicate the data has been loaded
+    resourceNode->setBoolValue("__loaded", true);
+    return true;
 }
 
 // Load strings for requested resource (for current and default locale).
@@ -259,8 +356,7 @@ FGLocale::loadResource(const char* resource)
     bool Ok = loadResource(_defaultLocale, resource);
 
     // also load language specific resource, unless identical
-    if ((_currentLocale!=0)&&
-        (_defaultLocale != _currentLocale))
+    if ((_currentLocale != nullptr) && (_defaultLocale != _currentLocale))
     {
         Ok &= loadResource(_currentLocale, resource);
     }
@@ -269,7 +365,7 @@ FGLocale::loadResource(const char* resource)
 }
 
 std::string
-FGLocale::getLocalizedString(SGPropertyNode *localeNode, const char* id, const char* context, int index) const
+FGLocale::innerGetLocalizedString(SGPropertyNode* localeNode, const char* id, const char* context, int index) const
 {
     SGPropertyNode *n = localeNode->getNode("strings",0, true)->getNode(context);
     if (!n) {
@@ -284,21 +380,28 @@ FGLocale::getLocalizedString(SGPropertyNode *localeNode, const char* id, const c
     return std::string();
 }
 
+std::string
+FGLocale::getLocalizedString(const std::string& id, const char* resource, const std::string& defaultValue)
+{
+    return getLocalizedString(id.c_str(), resource, defaultValue.c_str());
+}
+
 std::string
 FGLocale::getLocalizedString(const char* id, const char* resource, const char* Default)
 {
+    assert(_inited);
     if (id && resource)
     {
         std::string s;
         if (_currentLocale) {
-            s = getLocalizedString(_currentLocale, id, resource, 0);
+            s = innerGetLocalizedString(_currentLocale, id, resource, 0);
             if (!s.empty()) {
                 return s;
             }
         }
 
         if (_defaultLocale) {
-            s = getLocalizedString(_defaultLocale, id, resource, 0);
+            s = innerGetLocalizedString(_defaultLocale, id, resource, 0);
             if (!s.empty()) {
                 return s;
             }
@@ -311,17 +414,18 @@ FGLocale::getLocalizedString(const char* id, const char* resource, const char* D
 std::string
 FGLocale::getLocalizedStringWithIndex(const char* id, const char* resource, unsigned int index) const
 {
+    assert(_inited);
     if (id && resource) {
         std::string s;
         if (_currentLocale) {
-            s = getLocalizedString(_currentLocale, id, resource, index);
+            s = innerGetLocalizedString(_currentLocale, id, resource, index);
             if (!s.empty()) {
                 return s;
             }
         }
 
         if (_defaultLocale) {
-            s = getLocalizedString(_defaultLocale, id, resource, index);
+            s = innerGetLocalizedString(_defaultLocale, id, resource, index);
             if (!s.empty()) {
                 return s;
             }
@@ -344,6 +448,7 @@ FGLocale::getLocalizedStrings(SGPropertyNode *localeNode, const char* id, const
 
 size_t FGLocale::getLocalizedStringCount(const char* id, const char* resource) const
 {
+    assert(_inited);
     if (_currentLocale) {
         SGPropertyNode* resourceNode = _currentLocale->getNode("strings",0, true)->getNode(resource);
         if (resourceNode) {
@@ -355,7 +460,11 @@ size_t FGLocale::getLocalizedStringCount(const char* id, const char* resource) c
     }
 
     if (_defaultLocale) {
-        size_t count = _defaultLocale->getNode("strings",0, true)->getNode(resource)->getChildren(id).size();
+        auto resourceNode = _defaultLocale->getNode("strings", 0, true)->getNode(resource);
+        if (!resourceNode)
+            return 0;
+
+        size_t count = resourceNode->getChildren(id).size();
         if (count > 0) {
             return count;
         }
@@ -367,6 +476,7 @@ size_t FGLocale::getLocalizedStringCount(const char* id, const char* resource) c
 simgear::PropertyList
 FGLocale::getLocalizedStrings(const char* id, const char* resource)
 {
+    assert(_inited);
     if (id && resource)
     {
         if (_currentLocale)
@@ -387,20 +497,21 @@ FGLocale::getLocalizedStrings(const char* id, const char* resource)
 }
 
 // Check for localized font
-const char*
+std::string
 FGLocale::getDefaultFont(const char* fallbackFont)
 {
-    const char* font = nullptr;
+    assert(_inited);
+    std::string font;
     if (_currentLocale)
     {
         font = _currentLocale->getStringValue("font", "");
-        if (font[0] != 0)
+        if (!font.empty())
             return font;
     }
     if (_defaultLocale)
     {
         font = _defaultLocale->getStringValue("font", "");
-        if (font[0] != 0)
+        if (!font.empty())
             return font;
     }
 
@@ -418,10 +529,11 @@ std::string FGLocale::localizedPrintf(const char* id, const char* resource, ...
 
 std::string FGLocale::vlocalizedPrintf(const char* id, const char* resource, va_list args)
 {
+    assert(_inited);
     std::string format = getLocalizedString(id, resource);
-    int len = ::vsprintf(nullptr, format.c_str(), args);
+    int len = ::vsnprintf(nullptr, 0, format.c_str(), args);
     char* buf = (char*) alloca(len);
-    ::vsprintf(buf, format.c_str(), args);
+    ::vsnprintf(buf, len, format.c_str(), args);
     return std::string(buf);
 }
 
@@ -488,3 +600,26 @@ std::string fgTrPrintfMsg(const char* key, ...)
     va_end(args);
     return r;
 }
+
+SGPropertyNode_ptr FGLocale::selectLanguageNode(SGPropertyNode* langs) const
+{
+    if (!langs)
+        return {};
+
+    for (auto l : _languages) {
+        // Only accept the hyphen separator in PropertyList node names between
+        // language and territory
+        const auto langNoEncoding = strutils::replace(removeEncodingPart(l),
+                                                      "_", "-");
+        if (langs->hasChild(langNoEncoding)) {
+            return langs->getChild(langNoEncoding);
+        }
+
+        const auto justLang = removeLocalePart(langNoEncoding);
+        if (langs->hasChild(justLang)) {
+            return langs->getChild(justLang);
+        }
+    }
+
+    return {};
+}
