@@ -24,6 +24,7 @@
 #include <osg/PagedLOD>
 
 // simgear
+#include <simgear/misc/strutils.hxx>
 #include <simgear/props/props_io.hxx>
 #include <simgear/structure/exception.hxx>
 #include <simgear/structure/event_mgr.hxx>
@@ -32,14 +33,18 @@
 
 #include "globals.hxx"
 #include "fg_props.hxx"
+#include "fg_io.hxx"
 
 #include <Navaids/navlist.hxx>
 #include <Airports/runways.hxx>
 #include <Airports/airport.hxx>
 #include <Airports/dynamics.hxx>
 #include <Airports/groundnetwork.hxx>
+
 #include <AIModel/AIManager.hxx>
 #include <AIModel/AICarrier.hxx>
+#include <AIModel/AIAircraft.hxx>
+#include <AIModel/AIFlightPlan.hxx>
 
 #include <Scenery/scenery.hxx>
 #include <GUI/MessageBox.hxx>
@@ -48,10 +53,11 @@
 using std::endl;
 using std::string;
 
+
 namespace flightgear
 {
-    
-    
+
+
     enum InitPosResult {
         ExactPosition,
         VicinityPosition,
@@ -65,70 +71,8 @@ namespace flightgear
 static SGTimeStamp global_finalizeTime;
 static bool global_callbackRegistered = false;
 
-static void finalizePosition();
+void finalizePosition();
 
-// Set current tower position lon/lat given an airport id
-static bool fgSetTowerPosFromAirportID( const string& id) {
-  const FGAirport *a = fgFindAirportID( id);
-  if (a) {
-    SGGeod tower = a->getTowerLocation();
-    fgSetDouble("/sim/tower/longitude-deg",  tower.getLongitudeDeg());
-    fgSetDouble("/sim/tower/latitude-deg",  tower.getLatitudeDeg());
-    fgSetDouble("/sim/tower/altitude-ft", tower.getElevationFt());
-    return true;
-  } else {
-    return false;
-  }
-
-}
-
-class FGTowerLocationListener : public SGPropertyChangeListener {
-
-  void valueChanged(SGPropertyNode* node) {
-    string id(node->getStringValue());
-    if (fgGetBool("/sim/tower/auto-position",true))
-    {
-      // enforce using closest airport when auto-positioning is enabled
-      const char* closest_airport = fgGetString("/sim/airport/closest-airport-id", "");
-      if (closest_airport && (id != closest_airport))
-      {
-        id = closest_airport;
-        node->setStringValue(id);
-      }
-    }
-    fgSetTowerPosFromAirportID(id);
-  }
-};
-
-class FGClosestTowerLocationListener : public SGPropertyChangeListener
-{
-  void valueChanged(SGPropertyNode* )
-  {
-    // closest airport has changed
-    if (fgGetBool("/sim/tower/auto-position",true))
-    {
-      // update tower position
-      const char* id = fgGetString("/sim/airport/closest-airport-id", "");
-      if (id && *id!=0)
-        fgSetString("/sim/tower/airport-id", id);
-    }
-  }
-};
-
-void initTowerLocationListener() {
-
-  SGPropertyChangeListener* tll = new FGTowerLocationListener();
-  globals->addListenerToCleanup(tll);
-  fgGetNode("/sim/tower/airport-id",  true)
-  ->addChangeListener( tll, true );
-
-  FGClosestTowerLocationListener* ntcl = new FGClosestTowerLocationListener();
-  globals->addListenerToCleanup(ntcl);
-  fgGetNode("/sim/airport/closest-airport-id", true)
-  ->addChangeListener(ntcl , true );
-  fgGetNode("/sim/tower/auto-position", true)
-  ->addChangeListener(ntcl, true );
-}
 
 static void setInitialPosition(const SGGeod& aPos, double aHeadingDeg)
 {
@@ -143,13 +87,6 @@ static void setInitialPosition(const SGGeod& aPos, double aHeadingDeg)
     fgSetDouble("/orientation/heading-deg", aHeadingDeg );
 }
 
-static bool isMPEnabled()
-{
-    // crude test, but sufficient for what we need
-    std::string txAddress = fgGetString("/sim/multiplay/txhost");
-    return !txAddress.empty();
-}
-
 static void fgApplyStartOffset(const SGGeod& aStartPos, double aHeading, double aTargetHeading = HUGE_VAL)
 {
   SGGeod startPos(aStartPos);
@@ -166,10 +103,7 @@ static void fgApplyStartOffset(const SGGeod& aStartPos, double aHeading, double
       aHeading = aTargetHeading;
     }
 
-    SGGeod offset;
-    double az2; // dummy
-    SGGeodesy::direct(startPos, offsetAzimuth + 180, offsetDistance, offset, az2);
-    startPos = offset;
+    startPos = SGGeodesy::direct(startPos, offsetAzimuth + 180, offsetDistance);
   }
 
     setInitialPosition(startPos, aHeading);
@@ -182,23 +116,33 @@ std::tuple<SGGeod, double> runwayStartPos(FGRunwayRef runway)
     double startOffset = fgGetDouble("/sim/airport/runways/start-offset-m", 5.0);
     SGGeod pos = runway->pointOnCenterline(startOffset);
 
-    if (isMPEnabled() && (fabs(offsetNm) <0.1)) {
-        SG_LOG( SG_GENERAL, SG_WARN, "Requested to start on " << runway->airport()->ident() << "/" <<
+    const bool overrideHoldShort = fgGetBool("/sim/presets/mp-hold-short-override", false);
+
+    if (!overrideHoldShort && FGIO::isMultiplayerRequested() && (fabs(offsetNm) <0.1)) {
+        SG_LOG( SG_GENERAL, SG_MANDATORY_INFO, "Requested to start on " << runway->airport()->ident() << "/" <<
                runway->ident() << ", MP is enabled so computing hold short position to avoid runway incursion");
 
         FGGroundNetwork* groundNet = runway->airport()->groundNetwork();
-        // add a margin, try to keep the entire aeroplane comfortable off the
-        // runway.
-        double margin = startOffset + (runway->widthM() * 1.5);
-        FGTaxiNodeRef taxiNode = groundNet ? groundNet->findNearestNodeOffRunway(pos, runway, margin) : 0;
-        if (taxiNode) {
-            // set this so multiplayer.nas can inform the user
-            fgSetBool("/sim/presets/avoided-mp-runway", true);
-            return std::make_tuple(taxiNode->geod(), SGGeodesy::courseDeg(taxiNode->geod(), pos));
-        }
 
-        // if we couldn't find a suitable taxi-node, give up. Guessing a position
-        // causes too much pain (starting in the water or similar bad things)
+        if (groundNet) {
+            // add a margin, try to keep the entire aeroplane comfortable off the
+            // runway.
+            double margin = startOffset + (runway->widthM() * 1.5);
+            FGTaxiNodeRef taxiNode = groundNet->findNearestNodeOffRunway(pos, runway, margin);
+            if (taxiNode) {
+                // set this so multiplayer.nas can inform the user
+                fgSetBool("/sim/presets/avoided-mp-runway", true);
+                return std::make_tuple(taxiNode->geod(), SGGeodesy::courseDeg(taxiNode->geod(), pos));
+            }
+            else {
+                // if we couldn't find a suitable taxi-node, give up. Guessing a position
+                // causes too much pain (starting in the water or similar bad things)
+                SG_LOG( SG_GENERAL, SG_ALERT, "Unable to position off runway because groundnet has no taxi node.");
+            }
+        }
+        else {
+            SG_LOG( SG_GENERAL, SG_ALERT, "Unable to position off runway because no groundnet.");
+        }
     }
 
     return std::make_tuple(pos, runway->headingDeg());
@@ -234,88 +178,42 @@ static bool setPosFromAirportIDandHdg( const string& id, double tgt_hdg ) {
     return true;
 }
 
-// Set current_options lon/lat given an airport id and parkig position name
-static bool fgSetPosFromAirportIDandParkpos( const string& id, const string& parkpos )
+static bool airportParkingSetVicinity(const string& id)
 {
-  string fltType;
-  string acOperator;
-  string acType; // Currently not used by findAvailable parking, so safe to leave empty.
-  SGPath acData;
-  if ( id.empty() )
-    return false;
-
-  const FGAirport* apt = fgFindAirportID(id);
-  if (!apt) {
-    SG_LOG( SG_GENERAL, SG_ALERT, "Failed to find airport " << id );
-    return false;
-  }
-
-  FGAirportDynamicsRef dcs = apt->getDynamics();
-  if (!dcs) {
-      // this happens during initPosition(), because the dynamics manager and
-      // ATC manager aren't running yet. We want to set an interim position
-      // so METAR and tile-loading can proceed, and then we're set the final
-      // position by call this this function again during finalizePosition
-      setInitialPosition(apt->geod(), 0.0);
-      return true;
-  }
-
-  ParkingAssignment pka;
-  double radius = fgGetDouble("/sim/dimensions/radius-m");
-  if ((parkpos == "AVAILABLE") && (radius > 0)) {
-
-    try {
-      acData = globals->get_fg_home();
-      acData.append("aircraft-data");
-      string acfile = fgGetString("/sim/aircraft") + string(".xml");
-      acData.append(acfile);
-      SGPropertyNode root;
-      readProperties(acData, &root);
-      SGPropertyNode * node = root.getNode("sim");
-      fltType    = node->getStringValue("aircraft-class", "NONE"     );
-      acOperator = node->getStringValue("aircraft-operator", "NONE"     );
-    } catch (const sg_exception &) {
-      SG_LOG(SG_GENERAL, SG_INFO,
-             "Could not load aircraft aircrat type and operator information from: " << acData << ". Using defaults");
-
-      // cout << path.str() << endl;
-    }
-    if (fltType.empty() || fltType == "NONE") {
-      SG_LOG(SG_GENERAL, SG_INFO,
-             "Aircraft type information not found in: " << acData << ". Using default value");
-      fltType = fgGetString("/sim/aircraft-class"   );
-    }
-    if (acOperator.empty() || fltType == "NONE") {
-      SG_LOG(SG_GENERAL, SG_INFO,
-             "Aircraft operator information not found in: " << acData << ". Using default value");
-      acOperator = fgGetString("/sim/aircraft-operator"   );
-    }
-
-
-    pka = dcs->getAvailableParking(radius, fltType, acType, acOperator);
-    if (pka.isValid()) {
-        // why is the following line necessary?
-      fgGetString("/sim/presets/parkpos");
-      fgSetString("/sim/presets/parkpos", pka.parking()->getName());
-    } else {
-      SG_LOG( SG_GENERAL, SG_ALERT,
-             "Failed to find a suitable parking at airport " << id );
+    const FGAirport* apt = fgFindAirportID(id);
+    if (!apt) {
+      SG_LOG( SG_GENERAL, SG_ALERT, "Failed to find airport " << id );
       return false;
     }
-  } else {
+    
+    setInitialPosition(apt->geod(), 0.0);
+    return true;
+}
 
-    pka = dcs->getParkingByName(parkpos);
-    if (!pka.isValid()) {
-      SG_LOG( SG_GENERAL, SG_ALERT,
-               "Failed to find a parking at airport " << id << ":" << parkpos);
-      return false;
+// Set current_options lon/lat given an airport id and parkig position name
+static bool finalizePositionForParkpos( const string& id, const string& parkpos )
+{
+    auto aiManager = globals->get_subsystem<FGAIManager>();
+    if (!aiManager || !aiManager->getUserAircraft()) {
+        SG_LOG( SG_GENERAL, SG_ALERT, "finalizePositionForParkpos: >> failed to find AI manager / user aircraft");
+        return false;
     }
-  }
-  // Why is the following line necessary?
-  fgGetString("/sim/presets/parkpos");
-  fgSetString("/sim/presets/parkpos", pka.parking()->getName());
-  
-  fgApplyStartOffset(pka.parking()->geod(), pka.parking()->getHeading());
+    
+    auto userAIFP = aiManager->getUserAircraft()->GetFlightPlan();
+    if (!userAIFP) {
+        SG_LOG( SG_GENERAL, SG_ALERT, "finalizePositionForParkpos: >> failed to find user aircraft AI flight-plan");
+        return false;
+    }
+    
+    auto pkr = userAIFP->getParkingGate();
+    if (!pkr) {
+        SG_LOG( SG_GENERAL, SG_ALERT,
+                "Failed to find a parking at airport " << id << ":" << parkpos);
+        return false;
+    }
+
+  fgSetString("/sim/presets/parkpos", pkr->getName());
+  fgApplyStartOffset(pkr->geod(), pkr->getHeading());
   return true;
 }
 
@@ -362,11 +260,10 @@ static bool fgSetPosFromAirportIDandRwy( const string& id, const string& rwy, bo
 }
 
 
-static void fgSetDistOrAltFromGlideSlope() {
-  // cout << "fgSetDistOrAltFromGlideSlope()" << endl;
+static void fgSetDistOrAltFromGlideSlope()
+{
   string apt_id = fgGetString("/sim/presets/airport-id");
-  double gs = fgGetDouble("/sim/presets/glideslope-deg")
-  * SG_DEGREES_TO_RADIANS ;
+  double gs = SGMiscd::deg2rad(fgGetDouble("/sim/presets/glideslope-deg"));
   double od = fgGetDouble("/sim/presets/offset-distance-nm");
   double alt = fgGetDouble("/sim/presets/altitude-ft");
 
@@ -406,15 +303,13 @@ static void fgSetDistOrAltFromGlideSlope() {
 }
 
 
-// Set current_options lon/lat given an airport id and heading (degrees)
+// Set current_options lon/lat given a Nav ID or GUID
 static bool fgSetPosFromNAV( const string& id,
                              const double& freq,
                              FGPositioned::Type type,
                              PositionedID guid)
 {
-    FGNavRecord* nav = 0;
-
-
+    FGNavRecordRef nav;
     if (guid != 0) {
         nav = FGPositioned::loadById<FGNavRecord>(guid);
         if (!nav)
@@ -432,14 +327,14 @@ static bool fgSetPosFromNAV( const string& id,
         if( navlist.size() > 1 ) {
           std::ostringstream buf;
           buf << "Ambigous NAV-ID: '" << id << "'. Specify id and frequency. Available stations:" << endl;
-          for( nav_list_type::const_iterator it = navlist.begin(); it != navlist.end(); ++it ) {
+          for( const auto& nav : navlist ) {
             // NDB stored in kHz, VOR stored in MHz * 100 :-P
-            double factor = (*it)->type() == FGPositioned::NDB ? 1.0 : 1/100.0;
-            string unit = (*it)->type() == FGPositioned::NDB ? "kHz" : "MHz";
-            buf << (*it)->ident() << " "
-            << std::setprecision(5) << (double)((*it)->get_freq() * factor) << " "
-            << (*it)->get_lat() << "/" << (*it)->get_lon()
-            << endl;
+            double factor = nav->type() == FGPositioned::NDB ? 1.0 : 1/100.0;
+            string unit = nav->type() == FGPositioned::NDB ? "kHz" : "MHz";
+            buf << nav->ident() << " "
+                << std::setprecision(5) << static_cast<double>(nav->get_freq() * factor) << " "
+                << nav->get_lat() << "/" << nav->get_lon()
+                << endl;
           }
 
           SG_LOG( SG_GENERAL, SG_ALERT, buf.str() );
@@ -456,68 +351,94 @@ static bool fgSetPosFromNAV( const string& id,
 
 static InitPosResult setInitialPosFromCarrier( const string& carrier )
 {
-#if !defined(FG_TESTLIB)
   const auto initialPos = FGAICarrier::initialPositionForCarrier(carrier);
   if (initialPos.first) {
     // set these so scenery system has a vicinity to work with, and
     // so our PagedLOD is loaded
     fgSetDouble("/sim/presets/longitude-deg",  initialPos.second.getLongitudeDeg());
     fgSetDouble("/sim/presets/latitude-deg",  initialPos.second.getLatitudeDeg());
-    SG_LOG( SG_GENERAL, SG_INFO, "Initial carrier pos = " << initialPos.second );
+
+    const std::string carrier = fgGetString("/sim/presets/carrier");
+    const std::string carrierpos = fgGetString("/sim/presets/carrier-position");
+    const std::string parkpos = fgGetString("/sim/presets/parkpos");
+
+    if (!carrier.empty())
+    {
+        std::string cpos = simgear::strutils::lowercase(carrierpos);
+
+        if (cpos == "flols" || cpos == "abeam")
+            fgSetInt("/sim/presets/carrier-course", 3);// base=1, launch=2, recovery=3
+        else if (parkpos.find("cat") != std::string::npos)
+            fgSetInt("/sim/presets/carrier-course", 2);// base=1, launch=2, recovery=3
+        else
+            fgSetInt("/sim/presets/carrier-course", 1); // base
+    }
+    else
+        fgSetInt("/sim/presets/carrier-course", 0); // not defined.
+
+    SG_LOG( SG_GENERAL, SG_DEBUG, "Initial carrier pos = " << initialPos.second << " course " << fgGetInt("/sim/presets/carrier-course"));
     return VicinityPosition;
   }
-  
+
   SG_LOG( SG_GENERAL, SG_ALERT, "Failed to locate aircraft carrier = " << carrier );
-#endif
   return Failure;
 }
 
+static InitPosResult checkCarrierSceneryLoaded(const SGSharedPtr<FGAICarrier> carrierRef)
+{
+    SGVec3d cartPos = carrierRef->getCartPos();
+    auto framestamp = globals->get_renderer()->getFrameStamp();
+    simgear::CheckSceneryVisitor csnv(globals->get_scenery()->getPager(),
+                                      toOsg(cartPos),
+                                      100.0 /* range in metres */,
+                                      framestamp);
+
+    // currently the PagedLODs will not be loaded by the DatabasePager
+    // while the splashscreen is there, so CheckSceneryVisitor force-loads
+    // missing objects in the main thread
+    carrierRef->getSceneBranch()->accept(csnv);
+    if (!csnv.isLoaded()) {
+      return ContinueWaiting;
+    }
+
+    // and then wait for the load to actually be synced to the main thread
+    if (!carrierRef->modelLoaded()) {
+      return ContinueWaiting;
+    }
+
+    return VicinityPosition;
+}
+
 // Set current_options lon/lat given an aircraft carrier id
 static InitPosResult setFinalPosFromCarrier( const string& carrier, const string& posid )
 {
-  
-#if !defined(FG_TESTLIB)
+
   SGSharedPtr<FGAICarrier> carrierRef = FGAICarrier::findCarrierByNameOrPennant(carrier);
   if (!carrierRef) {
     SG_LOG( SG_GENERAL, SG_ALERT, "Failed to locate aircraft carrier = "
            << carrier );
     return Failure;
   }
-  
-  SGVec3d cartPos = carrierRef->getCartPos();
-  auto framestamp = globals->get_renderer()->getViewer()->getFrameStamp();
-  simgear::CheckSceneryVisitor csnv(globals->get_scenery()->getPager(),
-                                    toOsg(cartPos),
-                                    100.0 /* range in metres */,
-                                    framestamp);
-  
-  // currently the PagedLODs will not be loaded by the DatabasePager
-  // while the splashscreen is there, so CheckSceneryVisitor force-loads
-  // missing objects in the main thread
-  carrierRef->getSceneBranch()->accept(csnv);
-  if (!csnv.isLoaded()) {
-    return ContinueWaiting;
+
+  auto res = checkCarrierSceneryLoaded(carrierRef);
+  if (res != VicinityPosition) {
+      return res; // either failrue or keep waiting for scenery load
   }
-  
-  // and then wait for the load to actually be synced to the main thread
-  if (carrierRef->getSceneBranch()->getNumChildren() < 1) {
-    return ContinueWaiting;
-  }
-  
+
   SGGeod geodPos;
   double heading;
   SGVec3d uvw;
   if (carrierRef->getParkPosition(posid, geodPos, heading, uvw)) {
-    
+
     ////////
     double lon = geodPos.getLongitudeDeg();
     double lat = geodPos.getLatitudeDeg();
     double alt = geodPos.getElevationFt() + 2.0;
-    
+
     SG_LOG( SG_GENERAL, SG_INFO, "Attempting to set starting position for "
            << carrier << " at lat = " << lat << ", lon = " << lon
            << ", alt = " << alt << ", heading = " << heading);
-    
+
     fgSetDouble("/sim/presets/longitude-deg",  lon);
     fgSetDouble("/sim/presets/latitude-deg",  lat);
     fgSetDouble("/sim/presets/altitude-ft", alt);
@@ -526,7 +447,7 @@ static InitPosResult setFinalPosFromCarrier( const string& carrier, const string
     fgSetDouble("/position/latitude-deg",  lat);
     fgSetDouble("/position/altitude-ft", alt);
     fgSetDouble("/orientation/heading-deg", heading);
-    
+
     fgSetString("/sim/presets/speed-set", "UVW");
     fgSetDouble("/velocities/uBody-fps", uvw(0));
     fgSetDouble("/velocities/vBody-fps", uvw(1));
@@ -534,21 +455,79 @@ static InitPosResult setFinalPosFromCarrier( const string& carrier, const string
     fgSetDouble("/sim/presets/uBody-fps", uvw(0));
     fgSetDouble("/sim/presets/vBody-fps", uvw(1));
     fgSetDouble("/sim/presets/wBody-fps", uvw(2));
-    
+
     fgSetBool("/sim/presets/onground", true);
-    
+
     /////////
     return ExactPosition;
   }
-#endif
   SG_LOG( SG_GENERAL, SG_ALERT, "Failed to locate aircraft carrier = " << carrier );
   return Failure;
 }
 
+static InitPosResult setFinalPosFromCarrierFLOLS(const string& carrier, bool abeam)
+{
+    SGSharedPtr<FGAICarrier> carrierRef = FGAICarrier::findCarrierByNameOrPennant(carrier);
+    if (!carrierRef) {
+      SG_LOG( SG_GENERAL, SG_ALERT, "Failed to locate aircraft carrier = "
+             << carrier );
+      return Failure;
+    }
+
+    auto res = checkCarrierSceneryLoaded(carrierRef);
+    if (res != VicinityPosition) {
+        SG_LOG(SG_GENERAL, SG_DEBUG, "carrier scenery not yet loaded");
+        return res; // either failure or keep waiting for scenery load
+    }
+
+    SGGeod flolsPosition;
+    double headingToFLOLS;
+    if (!carrierRef->getFLOLSPositionHeading(flolsPosition, headingToFLOLS)) {
+        SG_LOG( SG_GENERAL, SG_ALERT, "Unable to compiute FLOLS position for carrier = "
+               << carrier );
+        return Failure;
+    }
+
+    const auto flolsElevationFt = flolsPosition.getElevationFt();
+    double gs = SGMiscd::deg2rad(carrierRef->getFLOLFSGlidepathAngleDeg());
+    const double od = fgGetDouble("/sim/presets/offset-distance-nm");
+
+    // start position
+    SGGeod startPos;
+    if (abeam) {
+      // If we're starting from the abeam position, we are opposite the FLOLS, downwind on a left hand circuit
+      startPos = SGGeodesy::direct(flolsPosition, headingToFLOLS - 90, od * SG_NM_TO_METER);
+    } else {
+      startPos = SGGeodesy::direct(flolsPosition, headingToFLOLS + 180, od * SG_NM_TO_METER);
+    }
+
+    double alt = fgGetDouble("/sim/presets/altitude-ft");
+
+    if (alt < 0.0f) {
+      // No altitude set, so base on glideslope
+      const double offsetFt = od * SG_NM_TO_METER * SG_METER_TO_FEET;
+      startPos.setElevationFt(fabs(offsetFt*tan(gs)) + flolsElevationFt);
+    } else {
+      startPos.setElevationFt(alt);
+    }
+
+    fgSetDouble("/sim/presets/longitude-deg",  startPos.getLongitudeDeg());
+    fgSetDouble("/sim/presets/latitude-deg",  startPos.getLatitudeDeg());
+    fgSetDouble("/sim/presets/altitude-ft", startPos.getElevationFt());
+    fgSetDouble("/sim/presets/heading-deg", abeam ? (headingToFLOLS - 180) : headingToFLOLS);
+    fgSetDouble("/position/longitude-deg",  startPos.getLongitudeDeg());
+    fgSetDouble("/position/latitude-deg",  startPos.getLatitudeDeg());
+    fgSetDouble("/position/altitude-ft", startPos.getElevationFt());
+    fgSetDouble("/orientation/heading-deg", headingToFLOLS);
+    fgSetBool("/sim/presets/onground", false);
+
+    return ExactPosition;
+}
+
 // Set current_options lon/lat given a fix ident and GUID
 static bool fgSetPosFromFix( const string& id, PositionedID guid )
 {
-    FGPositioned* fix = NULL;
+    FGPositionedRef fix;
     if (guid != 0) {
         fix = FGPositioned::loadById<FGPositioned>(guid);
     } else {
@@ -573,22 +552,25 @@ bool initPosition()
     globals->get_event_mgr()->addTask("finalizePosition", &finalizePosition, 0.1);
     global_callbackRegistered = true;
   }
-  
-  double gs = fgGetDouble("/sim/presets/glideslope-deg")
-  * SG_DEGREES_TO_RADIANS ;
+
+  double gs = SGMiscd::deg2rad(fgGetDouble("/sim/presets/glideslope-deg"));
   double od = fgGetDouble("/sim/presets/offset-distance-nm");
   double alt = fgGetDouble("/sim/presets/altitude-ft");
-  
+
   bool set_pos = false;
-  
+
+  // clear this value, so we don't preserve an old value and confuse
+  // the ATC manager. We will set it again if it's valid
+  fgSetString("/sim/atc/runway", "");
+
   // If glideslope is specified, then calculate offset-distance or
   // altitude relative to glide slope if either of those was not
   // specified.
   if ( fabs( gs ) > 0.01 ) {
     fgSetDistOrAltFromGlideSlope();
   }
-  
-  
+
+
   // If we have an explicit, in-range lon/lat, don't change it, just use it.
   // If not, check for an airport-id and use that.
   // If not, default to the middle of the KSFO field.
@@ -602,7 +584,7 @@ bool initPosition()
   {
     set_pos = true;
   }
-  
+
   string apt = fgGetString("/sim/presets/airport-id");
   const bool apt_req = fgGetBool("/sim/presets/airport-requested");
   string rwy_no = fgGetString("/sim/presets/runway");
@@ -614,31 +596,33 @@ bool initPosition()
   string carrier = fgGetString("/sim/presets/carrier");
   string parkpos = fgGetString("/sim/presets/parkpos");
   string fix = fgGetString("/sim/presets/fix");
-  
+  const auto tacan = fgGetString("/sim/presets/tacan-id");
+
   // the launcher sets this to precisely identify a navaid
   PositionedID navaidId = fgGetInt("/sim/presets/navaid-id");
-  
+
   SGPropertyNode *hdg_preset = fgGetNode("/sim/presets/heading-deg", true);
   double hdg = hdg_preset->getDoubleValue();
-  
+  double original_hdg = hdg_preset->getDoubleValue();
+
   // save some start parameters, so that we can later say what the
   // user really requested. TODO generalize that and move it to options.cxx
   static bool start_options_saved = false;
   if (!start_options_saved) {
     start_options_saved = true;
     SGPropertyNode *opt = fgGetNode("/sim/startup/options", true);
-    
+
     opt->setDoubleValue("latitude-deg", lat_deg);
     opt->setDoubleValue("longitude-deg", lon_deg);
     opt->setDoubleValue("heading-deg", hdg);
     opt->setStringValue("airport", apt.c_str());
     opt->setStringValue("runway", rwy_no.c_str());
   }
-  
+
   if (hdg > 9990.0) {
     hdg = fgGetDouble("/environment/config/boundary/entry/wind-from-heading-deg", 270);
   }
-  
+
   if ( !set_pos && !carrier.empty() ) {
     // an aircraft carrier is requested
     const auto result = setInitialPosFromCarrier( carrier );
@@ -647,7 +631,7 @@ bool initPosition()
       set_pos = true;
     }
   }
-  
+
   if (apt_req && !rwy_req) {
     // ensure that if the users asks for a specific airport, but not a runway,
     // presumably because they want automatic selection, we do not look
@@ -655,70 +639,77 @@ bool initPosition()
     // likely missing.
     rwy_no.clear();
   }
-  
+
   if ( !set_pos && !apt.empty() && !parkpos.empty() ) {
     // An airport + parking position is requested
     // since this depends on parking, which is part of dynamics, and hence
     // also depends on ATC (the ground controller), we need to defer this
     // until position finalisation
-    if ( fgSetPosFromAirportIDandParkpos( apt, parkpos ) ) {
-      // set tower position
-      fgSetString("/sim/airport/closest-airport-id",  apt.c_str());
-      fgSetString("/sim/tower/airport-id",  apt.c_str());
+    // the rest of the work happens in finalizePosFromParkpos
+    if ( airportParkingSetVicinity( apt ) ) {
       set_pos = true;
     }
   }
-  
+
   if ( !set_pos && !apt.empty() && !rwy_no.empty() ) {
     // An airport + runway is requested
     if ( fgSetPosFromAirportIDandRwy( apt, rwy_no, rwy_req ) ) {
-      // set tower position (a little off the heading for single
-      // runway airports)
-      fgSetString("/sim/airport/closest-airport-id",  apt.c_str());
-      fgSetString("/sim/tower/airport-id",  apt.c_str());
       set_pos = true;
     }
   }
-  
+
   if ( !set_pos && !apt.empty() ) {
     // An airport is requested (find runway closest to hdg)
     if ( setPosFromAirportIDandHdg( apt, hdg ) ) {
-      // set tower position (a little off the heading for single
-      // runway airports)
-      fgSetString("/sim/airport/closest-airport-id",  apt.c_str());
-      fgSetString("/sim/tower/airport-id",  apt.c_str());
       set_pos = true;
     }
   }
-  
-  if (hdg_preset->getDoubleValue() > 9990.0)
-    hdg_preset->setDoubleValue(hdg);
-  
+
+  // if an airport ID was requested, set closest-airport-id
+  // and tower based upon it.
+  if (!apt.empty() && set_pos) {
+      // set tower position
+      fgSetString("/sim/airport/closest-airport-id", apt.c_str());
+      fgSetString("/sim/tower/airport-id", apt.c_str());
+  }
+
+  if (original_hdg < 9990.0) {
+    // The user-set heading may be overridden by the setPosFromAirportID above.
+    hdg_preset->setDoubleValue(original_hdg);
+  }
+
   if ( !set_pos && !vor.empty() ) {
     // a VOR is requested
     if ( fgSetPosFromNAV( vor, vor_freq, FGPositioned::VOR, navaidId ) ) {
       set_pos = true;
     }
   }
-  
+
   if ( !set_pos && !ndb.empty() ) {
     // an NDB is requested
     if ( fgSetPosFromNAV( ndb, ndb_freq, FGPositioned::NDB, navaidId ) ) {
       set_pos = true;
     }
   }
-  
+
   if ( !set_pos && !fix.empty() ) {
     // a Fix is requested
     if ( fgSetPosFromFix( fix, navaidId ) ) {
       set_pos = true;
     }
   }
-  
+
+  if (!set_pos & !tacan.empty()) {
+      // we don't record TACANs in the NavCache right now, instead we
+      // record DMEs, some of which have TACAN in the name.
+      if (fgSetPosFromNAV(tacan, 0.0, FGPositioned::DME, navaidId)) {
+          set_pos = true;
+      }
+  }
+
   if ( !set_pos ) {
-    // No lon/lat specified, no airport specified, use the default airport
-    // TODO: don't hardcode this. Really.
-    const FGAirport* airport = fgFindAirportID("PHNL");
+    const std::string defaultAirportId = fgGetString("/sim/presets/airport-id");
+    const FGAirport* airport = fgFindAirportID(defaultAirportId);
     if( airport ) {
       const SGGeod & airportGeod = airport->geod();
       fgSetDouble("/sim/presets/longitude-deg", airportGeod.getLongitudeDeg());
@@ -728,30 +719,31 @@ bool initPosition()
       // Let's hope KSFO still exists somehow
       fgSetDouble("/sim/presets/longitude-deg", -122.374843);
       fgSetDouble("/sim/presets/latitude-deg", 37.619002);
-      SG_LOG(SG_GENERAL, SG_ALERT, "Sorry, the default airport seems to be unknown.");
+      SG_LOG(SG_GENERAL, SG_ALERT, "Sorry, the default airport ('" << defaultAirportId
+             << "') seems to be unknown.");
     }
   }
-  
+
   fgSetDouble( "/position/longitude-deg",
               fgGetDouble("/sim/presets/longitude-deg") );
   fgSetDouble( "/position/latitude-deg",
               fgGetDouble("/sim/presets/latitude-deg") );
   fgSetDouble( "/orientation/heading-deg", hdg_preset->getDoubleValue());
-  
+
   // determine if this should be an on-ground or in-air start
   if ((fabs(gs) > 0.01 || fabs(od) > 0.1 || alt > 0.1) && carrier.empty()) {
     fgSetBool("/sim/presets/onground", false);
   } else {
     fgSetBool("/sim/presets/onground", true);
   }
-  
+
   fgSetBool("/sim/position-finalized", false);
-  
+
   // Initialize the longitude, latitude and altitude to the initial position
   fgSetDouble("/position/altitude-ft", fgGetDouble("/sim/presets/altitude-ft"));
   fgSetDouble("/position/longitude-deg", fgGetDouble("/sim/presets/longitude-deg"));
   fgSetDouble("/position/latitude-deg", fgGetDouble("/sim/presets/latitude-deg"));
-  
+
   return true;
 }
 
@@ -813,30 +805,59 @@ void finalizePosition()
      * loaded. => When requested "initial preset position" relates to a
      * carrier, recalculate the 'initial' position here
      */
-    std::string carrier = fgGetString("/sim/presets/carrier");
-    std::string parkpos = fgGetString("/sim/presets/parkpos");
-    std::string apt = fgGetString("/sim/presets/airport-id");
+    const std::string carrier = fgGetString("/sim/presets/carrier");
+    const std::string carrierpos = fgGetString("/sim/presets/carrier-position");
+    const std::string parkpos = fgGetString("/sim/presets/parkpos");
+    const std::string runway = fgGetString("/sim/presets/runway");
+    const std::string apt = fgGetString("/sim/presets/airport-id");
 
     if (!carrier.empty())
     {
-        const auto res = setFinalPosFromCarrier(carrier, parkpos);
-        if (res == ExactPosition) {
+        /* /sim/presets/carrier-position can take a number of different values
+           - name of a parking/cataputt
+           - FLOLS - to position on final approach
+           - abeam - on an abeam position from the FLOLS, heading downwind
+           - <empty> indicating a start position of a catapult
+        */
+
+        // Convert to lower case to simplify comparison
+        std::string cpos = simgear::strutils::lowercase(carrierpos);
+
+        const bool    inair = (cpos == "flols") || (cpos == "abeam");
+        const bool    abeam = (cpos == "abeam");
+        InitPosResult carrierResult;
+        if (inair) {
+            carrierResult = setFinalPosFromCarrierFLOLS(carrier, abeam);
+        } else {
+            // We don't simply use cpos as it is lower case, and the
+            // search against parking/catapult positions is case-sensitive.
+            carrierResult = setFinalPosFromCarrier(carrier, carrierpos);
+        }
+        if (carrierResult == ExactPosition) {
             done = true;
-        } else if (res == Failure) {
+        } else if (carrierResult == Failure) {
             SG_LOG(SG_GENERAL, SG_ALERT, "secondary carrier init failed");
             done = true;
         } else {
             done = false;
             // 60 second timeout on waiting for the carrier to load
             if (global_finalizeTime.elapsedMSec() > 60000) {
+                SG_LOG(SG_GENERAL, SG_ALERT, "Timeout waiting for carrier scenery to load, will start on the water.");
                 done = true;
             }
         }
-        
+
     } else if (!apt.empty() && !parkpos.empty()) {
         // parking position depends on ATC / dynamics code to assign spaces,
         // so we wait until this point to initialise
-        fgSetPosFromAirportIDandParkpos(apt, parkpos);
+        bool ok = finalizePositionForParkpos(apt, parkpos);
+        if (!ok) {
+            SG_LOG(SG_GENERAL, SG_WARN, "finalizePositionForParkPos failed, reverting to best runway");
+            
+            // clear this so finalizeMetar works as expected
+            fgSetString("/sim/presets/parkpos", "");
+            finalizeMetar();
+        }
     } else {
         done = finalizeMetar();
     }
