@@ -21,25 +21,23 @@
 #include "sentryIntegration.hxx"
 
 #include <cstring> // for strcmp
-#include <algorithm>
 
 #include <simgear/debug/LogCallback.hxx>
 #include <simgear/debug/logstream.hxx>
 #include <simgear/debug/ErrorReportingCallback.hxx>
+#include <simgear/debug/Reporting.hxx>
+
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/structure/commands.hxx>
 #include <simgear/structure/exception.hxx>
 #include <simgear/io/iostreams/sgstream.hxx>
 
-#include <Include/version.h>
 #include <Main/fg_init.hxx>
 #include <Main/fg_props.hxx>
 #include <Main/globals.hxx>
 
-#include <build.h>
-
-using namespace std;
+#include <flightgearBuildId.h>
 
 bool doesStringMatchPrefixes(const std::string& s, const std::initializer_list<const char*>& prefixes)
 {
@@ -63,22 +61,10 @@ auto OSG_messageWhitelist = {
      "Detected particle system using segment(s) with less than 2 vertices"
 };
 
-auto XML_messageWhitelist = {
-    "Cannot open file",
-    "not well-formed (invalid token)",
-    "mismatched tag",
-    "syntax error",
-    "no element found",
-    "Root element name is",
-    "XML or text declaration not at start of entity",
-    "Failed to open file",
-    "unclosed token"};
-
 auto exception_messageWhitelist = {
     "position is invalid, NaNs",    ///< avoid spam when NaNs occur
     "bad AI flight plan",           ///< adjusting logic to avoid this is tricky
     "couldn't find shader",         ///< handled seperately
-    "unable to find add-on metadata file",
 
     /// supress noise from user-entered METAR values : we special case
     /// when live metar fails to parse
@@ -90,7 +76,6 @@ auto exception_messageWhitelist = {
 #if defined(HAVE_SENTRY) && !defined(BUILDING_TESTSUITE)
 
 static bool static_sentryEnabled = false;
-thread_local bool perThread_reportXMLParseErrors = true;
 
 #include <sentry.h>
 
@@ -103,15 +88,6 @@ void sentryTraceSimgearThrow(const std::string& msg, const std::string& origin,
     if (!static_sentryEnabled)
         return;
 
-// don't report the exceptions raised by easyxml.cxx, if this per-thread
-// flag is set. This avoids frequent errors when the launcher scans
-// directories containing many aircraft of unknown origin/quality
-// if the user tries to fly with one, we'll still get an error then,
-// but that's a real failure point (from the user PoV)
-    if (!perThread_reportXMLParseErrors && doesStringMatchPrefixes(msg, XML_messageWhitelist)) {
-        return;
-    }
-
     if (doesStringMatchPrefixes(msg, exception_messageWhitelist)) {
         return;
     }
@@ -119,9 +95,8 @@ void sentryTraceSimgearThrow(const std::string& msg, const std::string& origin,
     sentry_value_t exc = sentry_value_new_object();
     sentry_value_set_by_key(exc, "type", sentry_value_new_string("Exception"));
 
-    string message = msg;
+    std::string message = msg;
     sentry_value_t info = sentry_value_new_object();
-
     if (!origin.empty()) {
         sentry_value_set_by_key(info, "origin", sentry_value_new_string(origin.c_str()));
     }
@@ -181,7 +156,7 @@ private:
     int _lastLoggedCount = 0;
 };
 
-const auto missingShaderPrefix = string{"Missing shader"};
+const auto missingShaderPrefix = std::string{"Missing shader"};
 
 string_list anon_missingShaderList;
 
@@ -196,7 +171,7 @@ bool isNewMissingShader(const std::string& path)
     return true;
 }
 
-void sentrySimgearReportCallback(const string& msg, const string& more, bool isFatal)
+void sentrySimgearReportCallback(const std::string& msg, const std::string& more, bool isFatal)
 {
     // don't duplicate reports for missing shaders, once per sessions
     // is sufficient
@@ -225,17 +200,19 @@ void sentrySimgearReportCallback(const string& msg, const string& more, bool isF
 
 void sentryReportBadAlloc()
 {
-    sentry_value_t sentryMessage = sentry_value_new_object();
-    sentry_value_set_by_key(sentryMessage, "type", sentry_value_new_string("Fatal Error"));
-    sentry_value_set_by_key(sentryMessage, "formatted", sentry_value_new_string("bad allocation"));
+    if (simgear::ReportBadAllocGuard::isSet()) {
+        sentry_value_t sentryMessage = sentry_value_new_object();
+        sentry_value_set_by_key(sentryMessage, "type", sentry_value_new_string("Fatal Error"));
+        sentry_value_set_by_key(sentryMessage, "formatted", sentry_value_new_string("bad allocation"));
 
-    sentry_value_t event = sentry_value_new_event();
-    sentry_value_set_by_key(event, "message", sentryMessage);
+        sentry_value_t event = sentry_value_new_event();
+        sentry_value_set_by_key(event, "message", sentryMessage);
 
-    sentry_event_value_add_stacktrace(event, nullptr, 0);
-    sentry_capture_event(event);
+        sentry_event_value_add_stacktrace(event, nullptr, 0);
+        sentry_capture_event(event);
+    }
 
-    std::set_new_handler(nullptr);
+    throw std::bad_alloc(); // allow normal processing
 }
 
 } // namespace
@@ -253,7 +230,7 @@ bool sentryReportCommand(const SGPropertyNode* args, SGPropertyNode* root)
     sentry_value_t exc = sentry_value_new_object();
     sentry_value_set_by_key(exc, "type", sentry_value_new_string("Report"));
 
-    const string message = args->getStringValue("message");
+    const auto message = args->getStringValue("message");
     sentry_value_set_by_key(exc, "value", sentry_value_new_string(message.c_str()));
 
     sentry_value_t event = sentry_value_new_event();
@@ -299,7 +276,12 @@ void initSentry()
     
     const auto buildString = std::to_string(JENKINS_BUILD_NUMBER);
     sentry_options_set_dist(options, buildString.c_str());
-    
+
+    // for dev / nightly builds, put Sentry in debug mode
+    if (strcmp(FG_BUILD_TYPE, "Release")) {
+        sentry_options_set_debug(options, 1);
+    }
+
     SGPath dataPath = fgHomePath() / "sentry_db";
 #if defined(SG_WINDOWS)
     const auto homePathString = dataPath.wstr();
@@ -315,7 +297,6 @@ void initSentry()
     sentry_options_add_attachment(options, logPath.c_str());
 #endif
     
-
     const auto uuidPath = fgHomePath() / "sentry_uuid.txt";
     bool generateUuid = true;
     std::string uuid;
@@ -339,22 +320,25 @@ void initSentry()
         uuid = std::string{bytes};
         // write it back to disk for next time
         sg_ofstream f(uuidPath);
-        f << uuid << endl;
+        f << uuid << std::endl;
     }
 
-    sentry_init(options);
-    static_sentryEnabled = true;
+    if (sentry_init(options) == 0) {
+        static_sentryEnabled = true;
+        sentry_value_t user = sentry_value_new_object();
+        sentry_value_t userUuidV = sentry_value_new_string(uuid.c_str());
+        sentry_value_set_by_key(user, "id", userUuidV);
+        sentry_set_user(user);
 
-    sentry_value_t user = sentry_value_new_object();
-    sentry_value_t userUuidV = sentry_value_new_string(uuid.c_str());
-    sentry_value_set_by_key(user, "id", userUuidV);
-    sentry_set_user(user);
+        sglog().addCallback(new SentryLogCallback);
+        setThrowCallback(sentryTraceSimgearThrow);
+        simgear::setErrorReportCallback(sentrySimgearReportCallback);
 
-    sglog().addCallback(new SentryLogCallback);
-    setThrowCallback(sentryTraceSimgearThrow);
-    simgear::setErrorReportCallback(sentrySimgearReportCallback);
-
-    std::set_new_handler(sentryReportBadAlloc);
+        std::set_new_handler(sentryReportBadAlloc);
+    } else {
+        SG_LOG(SG_GENERAL, SG_WARN, "Failed to init Sentry reporting");
+        static_sentryEnabled = false;
+    }
 }
 
 void delayedSentryInit()
@@ -497,11 +481,6 @@ void  sentryReportFatalError(const std::string& msg, const std::string& more)
     sentry_capture_event(event);
 }
 
-void sentryThreadReportXMLErrors(bool report)
-{
-    perThread_reportXMLParseErrors = report;
-}
-
 void sentryReportUserError(const std::string& aggregate, const std::string& details)
 {
     if (!static_sentryEnabled)
@@ -572,10 +551,6 @@ void sentryReportFatalError(const std::string&, const std::string&)
 {
 }
 
-void sentryThreadReportXMLErrors(bool)
-{
-}
-
 void sentryReportUserError(const std::string&, const std::string&)
 {
 }
@@ -597,14 +572,4 @@ void addSentryTag(const std::string& tag, const std::string& value)
     addSentryTag(tag.c_str(), value.c_str());
 }
 
-SentryXMLErrorSupression::SentryXMLErrorSupression()
-{
-    sentryThreadReportXMLErrors(false);
-}
-
-SentryXMLErrorSupression::~SentryXMLErrorSupression()
-{
-    sentryThreadReportXMLErrors(true);
-}
-
 } // of namespace flightgear
