@@ -41,7 +41,9 @@
 #include <Environment/environment_mgr.hxx>
 #include <Environment/ephemeris.hxx>
 #include <Instrumentation/instrument_mgr.hxx>
-#include <Instrumentation/HUD/HUD.hxx>
+#ifdef ENABLE_HUD
+#  include <Instrumentation/HUD/HUD.hxx>
+#endif
 #include <Systems/system_mgr.hxx>
 #include <Autopilot/route_mgr.hxx>
 #include <Autopilot/autopilotgroup.hxx>
@@ -54,6 +56,9 @@
 #include <ATC/atc_mgr.hxx>
 #include <AIModel/AIManager.hxx>
 #include <MultiPlayer/multiplaymgr.hxx>
+#if defined(ENABLE_SWIFT)
+#  include <Network/Swift/swift_connection.hxx>
+#endif
 #include <AIModel/submodel.hxx>
 #include <Aircraft/controls.hxx>
 #include <Input/input.hxx>
@@ -85,7 +90,9 @@ SGSubsystem* createSubsystemByName(const std::string& name)
     MAKE_SUB(Ephemeris, "ephemeris");
     MAKE_SUB(FGSystemMgr, "systems");
     MAKE_SUB(FGInstrumentMgr, "instrumentation");
+#ifdef ENABLE_HUD
     MAKE_SUB(HUD, "hud");
+#endif
     MAKE_SUB(flightgear::CockpitDisplayManager, "cockpit-displays");
     MAKE_SUB(FGRouteMgr, "route-manager");
     MAKE_SUB(FGIO, "io");
@@ -108,6 +115,10 @@ SGSubsystem* createSubsystemByName(const std::string& name)
 #ifdef ENABLE_IAX
     MAKE_SUB(FGCom, "fgcom");
 #endif
+#ifdef ENABLE_SWIFT
+    MAKE_SUB(swift::SwiftConnection, "swift");
+#endif
+    MAKE_SUB(FGLight, "tides");
     MAKE_SUB(FGLight, "lighting");
     MAKE_SUB(FGAircraftModel, "aircraft-model");
     MAKE_SUB(FGModelMgr, "model-manager");
@@ -188,18 +199,18 @@ static SGSubsystem* getSubsystem(const SGPropertyNode* arg, bool create)
 }
 
 static bool
-do_check_subsystem_running(const SGPropertyNode* arg)
+do_check_subsystem_running(const SGPropertyNode * arg, SGPropertyNode * root)
 {
   return getSubsystem(arg, false) != 0;
 }
 
 static bool
-do_add_subsystem (const SGPropertyNode * arg)
+do_add_subsystem (const SGPropertyNode * arg, SGPropertyNode * root)
 {
   return getSubsystem(arg, true) != 0;
 }
 
-static bool do_remove_subsystem(const SGPropertyNode * arg)
+static bool do_remove_subsystem(const SGPropertyNode * arg, SGPropertyNode * root)
 {
   std::string name = arg->getStringValue("subsystem");
 
@@ -226,7 +237,7 @@ static bool do_remove_subsystem(const SGPropertyNode * arg)
  * none is specified, reinitialize all of them.
  */
 static bool
-do_reinit (const SGPropertyNode * arg)
+do_reinit (const SGPropertyNode * arg, SGPropertyNode * root)
 {
     bool result = true;
 
@@ -235,8 +246,8 @@ do_reinit (const SGPropertyNode * arg)
         globals->get_subsystem_mgr()->reinit();
     } else {
         for ( unsigned int i = 0; i < subsystems.size(); i++ ) {
-            const char * name = subsystems[i]->getStringValue();
-            SGSubsystem * subsystem = globals->get_subsystem(name);
+            std::string name = subsystems[i]->getStringValue();
+            SGSubsystem * subsystem = globals->get_subsystem_mgr()->get_subsystem(name.c_str());
             if (subsystem == 0) {
                 result = false;
                 SG_LOG( SG_GENERAL, SG_ALERT,
@@ -258,14 +269,14 @@ do_reinit (const SGPropertyNode * arg)
  * subsystem[*] - the name(s) of the subsystem(s) to suspend.
  */
 static bool
-do_suspend (const SGPropertyNode * arg)
+do_suspend (const SGPropertyNode * arg, SGPropertyNode * root)
 {
     bool result = true;
 
     vector<SGPropertyNode_ptr> subsystems = arg->getChildren("subsystem");
     for ( unsigned int i = 0; i < subsystems.size(); i++ ) {
-        const char * name = subsystems[i]->getStringValue();
-        SGSubsystem * subsystem = globals->get_subsystem(name);
+        std::string name = subsystems[i]->getStringValue();
+        SGSubsystem * subsystem = globals->get_subsystem_mgr()->get_subsystem(name.c_str());
         if (subsystem == 0) {
             result = false;
             SG_LOG(SG_GENERAL, SG_ALERT, "Subsystem " << name << " not found");
@@ -282,14 +293,14 @@ do_suspend (const SGPropertyNode * arg)
  * subsystem[*] - the name(s) of the subsystem(s) to suspend.
  */
 static bool
-do_resume (const SGPropertyNode * arg)
+do_resume (const SGPropertyNode * arg, SGPropertyNode * root)
 {
     bool result = true;
 
     vector<SGPropertyNode_ptr> subsystems = arg->getChildren("subsystem");
     for ( unsigned int i = 0; i < subsystems.size(); i++ ) {
-        const char * name = subsystems[i]->getStringValue();
-        SGSubsystem * subsystem = globals->get_subsystem(name);
+        std::string name = subsystems[i]->getStringValue();
+        SGSubsystem * subsystem = globals->get_subsystem_mgr()->get_subsystem(name.c_str());
         if (subsystem == 0) {
             result = false;
             SG_LOG(SG_GENERAL, SG_ALERT, "Subsystem " << name << " not found");
