@@ -82,7 +82,8 @@ const char* restrictionToString(RouteRestriction aRestrict);
 typedef std::vector<FlightPlan::DelegateFactoryRef> FPDelegateFactoryVec;
 static FPDelegateFactoryVec static_delegateFactories;
   
-FlightPlan::FlightPlan() :
+FlightPlan::FlightPlan(bool isRoute) :
+    _isRoute(isRoute),
   _currentIndex(-1),
     _followLegTrackToFix(true),
     _aircraftCategory(ICAO_AIRCRAFT_CATEGORY_C),
@@ -104,7 +105,17 @@ FlightPlan::FlightPlan() :
     }
   }
 }
-  
+
+FlightPlanRef FlightPlan::create()
+{
+    return new FlightPlan(false);
+}
+
+FlightPlanRef FlightPlan::createRoute()
+{
+    return new FlightPlan(true);
+}
+
 FlightPlan::~FlightPlan()
 {
 // clean up delegates
@@ -116,9 +127,12 @@ FlightPlan::~FlightPlan()
     }
 }
   
-FlightPlan* FlightPlan::clone(const string& newIdent) const
+FlightPlanRef FlightPlan::clone(const string& newIdent, bool convertIntoFlightPlan) const
 {
-  FlightPlan* c = new FlightPlan();
+    // this is the only place we allow conversion of a route into an active FP,
+    // by design. Forces people to clone-to-a-flight-plan if they want to
+    // activate a route.
+  FlightPlanRef c = new FlightPlan(convertIntoFlightPlan ? false : _isRoute);
   c->_ident = newIdent.empty() ? _ident : newIdent;
   c->lockDelegates();
   
@@ -146,12 +160,16 @@ FlightPlan* FlightPlan::clone(const string& newIdent) const
       c->setCruiseFlightLevel(_cruiseFlightLevel);
   } else if (_cruiseAltitudeFt > 0) {
       c->setCruiseAltitudeFt(_cruiseAltitudeFt);
+  } else if (_cruiseAltitudeM > 0) {
+    c->setCruiseAltitudeM(_cruiseAltitudeM);
   }
 
   if (_cruiseAirspeedMach > 0) {
       c->setCruiseSpeedMach(_cruiseAirspeedMach);
   } else if (_cruiseAirspeedKnots > 0) {
       c->setCruiseSpeedKnots(_cruiseAirspeedKnots);
+  } else if (_cruiseAirspeedKph > 0) {
+    c->setCruiseSpeedKPH(_cruiseAirspeedKph);
   }
 
   c->_didLoadFP = true; // set the loaded flag to give delegates a chance
@@ -161,6 +179,8 @@ FlightPlan* FlightPlan::clone(const string& newIdent) const
   for (int l=0; l < numLegs(); ++l) {
     c->_legs.push_back(_legs[l]->cloneFor(c));
   }
+    
+    c->expandVias();
   c->unlockDelegates();
   return c;
 }
@@ -193,25 +213,55 @@ FlightPlan::LegRef FlightPlan::insertWayptAtIndex(Waypt* aWpt, int aIndex)
   return legAtIndex(index);
 }
   
+static WayptVec copyWaypointsExpandingVias(WayptRef preceeding, const WayptVec& wps)
+{
+    WayptVec result;
+    result.reserve(wps.size());
+    
+    for (auto wp : wps) {
+        if (wp->type() == "via") {
+            Via* via = static_cast<Via*>(wp.get());
+            WayptVec viaPoints = via->expandToWaypoints(preceeding);
+            result.insert(result.end(), viaPoints.begin(), viaPoints.end());
+        } else {
+            // everything else is copied directly
+            result.push_back(wp);
+        }
+    }
+    
+    return result;
+}
+
 void FlightPlan::insertWayptsAtIndex(const WayptVec& wps, int aIndex)
 {
   if (wps.empty()) {
     return;
   }
   
-  int index = aIndex;
-  if ((aIndex == -1) || (aIndex > (int) _legs.size())) {
-    index = _legs.size();
-  }
-  
+    int index = aIndex;
+    if ((aIndex == -1) || (aIndex > (int) _legs.size())) {
+      index = _legs.size();
+    }
+    
+    WayptVec toInsertWps = wps;
+    // catch insert of VIAs here
+    if (!_isRoute && (index > 0)) {
+        const auto pre = _legs.at(index - 1)->waypoint();
+        toInsertWps = copyWaypointsExpandingVias(pre, wps);
+    } else if (index == 0) {
+        if (wps.front()->type() == "via") {
+            SG_LOG(SG_AUTOPILOT, SG_DEV_ALERT, "Inserting a VIA at leg 0 of flight-plan, VIA cannot be expanded");
+        }
+    }
+   
   auto it = _legs.begin() + index;
-  int endIndex = index + wps.size() - 1;
+  int endIndex = index + toInsertWps.size() - 1;
   if (_currentIndex >= endIndex) {
-    _currentIndex += wps.size();
+    _currentIndex += toInsertWps.size();
   }
  
   LegVec newLegs;
-  for (WayptRef wp : wps) {
+  for (WayptRef wp : toInsertWps) {
       newLegs.push_back(LegRef{new Leg(this, wp)});
   }
   
@@ -268,8 +318,10 @@ void FlightPlan::clearAll()
 
     _cruiseAirspeedMach = 0.0;
     _cruiseAirspeedKnots = 0;
+    _cruiseAirspeedKph = 0;
     _cruiseFlightLevel = 0;
     _cruiseAltitudeFt = 0;
+    _cruiseAltitudeM = 0;
 
     clearLegs();
     unlockDelegates();
@@ -356,9 +408,16 @@ int FlightPlan::clearWayptsWithFlag(WayptFlag flag)
   unlockDelegates();
   return numDeleted;
 }
-    
+
+bool FlightPlan::isRoute() const
+{
+    return _isRoute;
+}
+
 bool FlightPlan::isActive() const
 {
+    if (_isRoute)
+        return false;
     return (_currentIndex >= 0);
 }
 
@@ -389,6 +448,10 @@ void FlightPlan::sequence()
     
 void FlightPlan::finish()
 {
+    if (_isRoute) {
+        throw sg_exception("Called finish on FlightPlan marked isRoute");
+    }
+    
   if (_currentIndex == -1) {
     return;
   }
@@ -655,12 +718,12 @@ void FlightPlan::setEstimatedDurationMinutes(int mins)
 
 void FlightPlan::computeDurationMinutes()
 {
-    if ((_cruiseAirspeedMach < 0.01) && (_cruiseAirspeedKnots < 10)) {
+    if ((_cruiseAirspeedMach < 0.01) && (_cruiseAirspeedKnots < 10) && (_cruiseAirspeedKph < 10)) {
         SG_LOG(SG_AUTOPILOT, SG_WARN, "can't compute duration, no cruise speed set");
         return;
     }
 
-    if ((_cruiseAltitudeFt < 100) && (_cruiseFlightLevel < 10)) {
+    if ((_cruiseAltitudeFt < 100) && (_cruiseAltitudeM < 100) && (_cruiseFlightLevel < 10)) {
         SG_LOG(SG_AUTOPILOT, SG_WARN, "can't compute duration, no cruise altitude set");
         return;
     }
@@ -754,6 +817,10 @@ void FlightPlan::saveToProperties(SGPropertyNode* d) const
     d->setIntValue("version", 2);
     
     // general data
+    if (_isRoute) {
+        d->setBoolValue("is-route", true);
+    }
+    
     d->setStringValue("flight-rules", static_icaoFlightRulesCode[static_cast<int>(_flightRules)]);
     d->setStringValue("flight-type", static_icaoFlightTypeCode[static_cast<int>(_flightType)]);
     if (!_callsign.empty()) {
@@ -808,12 +875,16 @@ void FlightPlan::saveToProperties(SGPropertyNode* d) const
         d->setIntValue("cruise/flight-level", _cruiseFlightLevel);
     } else if (_cruiseAltitudeFt > 0) {
         d->setIntValue("cruise/altitude-ft", _cruiseAltitudeFt);
+    } else if (_cruiseAltitudeM > 0) {
+        d->setIntValue("cruise/altitude-m", _cruiseAltitudeM);
     }
     
     if (_cruiseAirspeedMach > 0.0) {
         d->setDoubleValue("cruise/mach", _cruiseAirspeedMach);
     } else if (_cruiseAirspeedKnots > 0) {
         d->setIntValue("cruise/knots", _cruiseAirspeedKnots);
+    } else if (_cruiseAirspeedKph > 0) {
+        d->setIntValue("cruise/kph", _cruiseAirspeedKph);
     }
     
     // route nodes
@@ -858,6 +929,10 @@ bool FlightPlan::load(const SGPath& path)
         _departureChanged = true;
         Status = true;
     } else if (loadXmlFormat(path)) { // XML property data
+        if (!_isRoute) {
+            expandVias();
+        }
+        
         // we don't want to re-compute the arrival / departure after
         // a load, since we assume the flight-plan had it specified already
         // especially, the XML might have a SID/STAR embedded, which we don't
@@ -865,7 +940,7 @@ bool FlightPlan::load(const SGPath& path)
         
         // however, we do want to run the normal delegate if no procedure was
         // defined. We'll use the presence of waypoints tagged to decide
-        const bool hasArrival = anyWaypointsWithFlag(this, WPT_ARRIVAL);
+        const bool hasArrival = anyWaypointsWithFlag(this, WPT_ARRIVAL) || anyWaypointsWithFlag(this, WPT_APPROACH);
         const bool hasDeparture = anyWaypointsWithFlag(this, WPT_DEPARTURE);
         _arrivalChanged = !hasArrival;
         _departureChanged = !hasDeparture;
@@ -874,6 +949,10 @@ bool FlightPlan::load(const SGPath& path)
         _arrivalChanged = true;
         _departureChanged = true;
         Status = true;
+        
+        if (!_isRoute) {
+            expandVias(); // plain text could in principle contain VIAs
+        }
     }
     
     if (Status == true) {
@@ -919,6 +998,10 @@ bool FlightPlan::load(std::istream &stream)
         Status = false;
     }
     
+    if (!_isRoute) {
+        expandVias();
+    }
+    
     // we don't want to re-compute the arrival / departure after
      // a load, since we assume the flight-plan had it specified already
      // especially, the XML might have a SID/STAR embedded, which we don't
@@ -1104,7 +1187,6 @@ bool FlightPlan::loadXmlFormat(const SGPath& path)
 void FlightPlan::loadXMLRouteHeader(SGPropertyNode_ptr routeData)
 {
     // general info
-
     const auto rules = routeData->getStringValue("flight-rules", "V");
     auto it = std::find(static_icaoFlightRulesCode.begin(),
                         static_icaoFlightRulesCode.end(), rules);
@@ -1120,6 +1202,13 @@ void FlightPlan::loadXMLRouteHeader(SGPropertyNode_ptr routeData)
     _aircraftType = routeData->getStringValue("aircraft-type");
     _estimatedDuration = routeData->getIntValue("estimated-duration-minutes");
 
+    if (routeData->hasValue("is-route")) {
+        if (_isRoute != routeData->getBoolValue("is-route")) {
+            // this is actually okay, we will expand any VIAs
+            SG_LOG(SG_NAVAID, SG_INFO, "Loading XML marked with 'is-route' into FlightPlan with is-route not set");
+        }
+    }
+    
   // departure nodes
   SGPropertyNode* dep = routeData->getChild("departure");
   if (dep) {
@@ -1137,9 +1226,14 @@ void FlightPlan::loadXMLRouteHeader(SGPropertyNode_ptr routeData)
           // to parse this value as a SID, and look for a seperate sid_trans
           // value
           const string trans = dep->getStringValue("sid_trans");
-          const auto sid = dep->getStringValue("sid");
-          setSID(_departure->findSIDWithIdent(sid), trans);
-      }
+          const auto sidIdent = dep->getStringValue("sid");
+          auto sid = _departure->findSIDWithIdent(sidIdent);
+          if (!sid) {
+              SG_LOG(SG_NAVAID, SG_WARN, "FlightPlan specifies unknown SID:" << sidIdent);
+          } else {
+              setSID(sid, trans);
+          }
+      } // of have a SID identifier
     }
   }
   
@@ -1156,16 +1250,26 @@ void FlightPlan::loadXMLRouteHeader(SGPropertyNode_ptr routeData)
       if (dst->hasChild("star")) {
           // prior to 2020.2 we would attempt to treat 'star' as a
           // transiiton ID, but this is ambiguous. Look for a seperate value now
-          const auto star = dst->getStringValue("star");
+          const auto starIdent = dst->getStringValue("star");
           const string trans = dst->getStringValue("star_trans");
-          setSTAR(_destination->findSTARWithIdent(star), trans);
+          auto star = _destination->findSTARWithIdent(starIdent);
+
+          if (star) {
+              setSTAR(star, trans);
+          } else {
+              SG_LOG(SG_NAVAID, SG_WARN, "FlightPlan specifies unknown STAR:" << starIdent);
+          }
       } // of STAR processing
       
       if (dst->hasChild("approach")) {
           auto app = _destination->findApproachWithIdent(dst->getStringValue("approach"));
           const auto trans = dst->getStringValue("approach_trans");
-          setApproach(app, trans);
-      }
+          if (app) {
+              setApproach(app, trans);
+          } else {
+              SG_LOG(SG_NAVAID, SG_WARN, "FlightPlan specifies unknown approach:" << dst->getStringValue("approach"));
+          }
+      } // of have approach in the XML
     }
   }
   
@@ -1181,12 +1285,16 @@ void FlightPlan::loadXMLRouteHeader(SGPropertyNode_ptr routeData)
           _cruiseFlightLevel = crs->getIntValue("flight-level");
       } else if (crs->hasChild("altitude-ft")) {
           _cruiseAltitudeFt = crs->getIntValue("altitude-ft");
+      } else if (crs->hasChild("altitude-m")) {
+          _cruiseAltitudeM = crs->getIntValue("altitude-m");
       }
 
       if (crs->hasChild("mach")) {
           _cruiseAirspeedMach = crs->getDoubleValue("mach");
       } else if (crs->hasChild("knots")) {
           _cruiseAirspeedKnots = crs->getIntValue("knots");
+      } else if (crs->hasChild("kph")) {
+          _cruiseAirspeedKph = crs->getIntValue("kph");
       }
   } // of cruise data loading
 }
@@ -1208,16 +1316,7 @@ bool FlightPlan::loadVersion2XMLRoute(SGPropertyNode_ptr routeData)
             continue;
         }
 
-      LegRef l = new Leg{this, wp};
-      // sync leg restrictions with waypoint ones
-        if (wp->speedRestriction() != RESTRICT_NONE) {
-            l->setSpeed(wp->speedRestriction(), wp->speed());
-        }
-        
-        if (wp->altitudeRestriction() != RESTRICT_NONE) {
-            l->setAltitude(wp->altitudeRestriction(), wp->altitudeFt());
-        }
-        
+        LegRef l = new Leg{this, wp};
         if (wpNode->hasChild("hold-count")) {
             l->setHoldCount(wpNode->getIntValue("hold-count"));
         }
@@ -1290,7 +1389,7 @@ WayptRef FlightPlan::parseVersion1XMLWaypt(SGPropertyNode* aWP)
   
   double altFt = aWP->getDoubleValue("altitude-ft", -9999.9);
   if (altFt > -9990.0) {
-    w->setAltitude(altFt, RESTRICT_AT);
+    w->setAltitude(altFt, RESTRICT_AT, ALTITUDE_FEET);
   }
   
   return w;
@@ -1318,14 +1417,18 @@ bool FlightPlan::loadPlainTextFormat(const SGPath& path)
       if (line.empty() || (line[0] == '#')) {
         continue; // ignore empty/comment lines
       }
-      
+
       // prevent Sentry error 'FLIGHTGEAR-J6', when we try loading XML
       // data here
       if (simgear::strutils::starts_with(line, "<?xml")) {
           return false;
       }
 
-      WayptRef w = waypointFromString(line);
+      SGGeod vicinity;
+      if (!_legs.empty()) {
+          vicinity = _legs.back()->waypoint()->position();
+      }
+      WayptRef w = waypointFromString(line, vicinity);
       if (!w) {
           SG_LOG(SG_NAVAID, SG_ALERT, "Failed to create waypoint from '" << line << "' in " << path);
           _legs.clear();
@@ -1348,185 +1451,90 @@ double FlightPlan::magvarDegAt(const SGGeod& pos) const
   double jd = globals->get_time_params()->getJD();
   return sgGetMagVar(pos, jd) * SG_RADIANS_TO_DEGREES;
 }
-  
 
-namespace
+SGGeod FlightPlan::vicinityForInsertIndex(int aIndex) const
 {
-
-WayptRef intersectionFromString(FGPositionedRef p1,
-                                const SGGeod& basePosition,
-                                const double magvar,
-                                const string_list& pieces)
-{
-    assert(pieces.size() == 4);
-    // navid/radial/navid/radial notation
-    FGPositionedRef p2 = FGPositioned::findClosestWithIdent(pieces[2], basePosition);
-    if (!p2) {
-      SG_LOG( SG_NAVAID, SG_INFO, "Unable to find FGPositioned with ident:" << pieces[2]);
-      return nullptr;
-    }
-
-    double r1 = atof(pieces[1].c_str()),
-    r2 = atof(pieces[3].c_str());
-    r1 += magvar;
-    r2 += magvar;
-
-    SGGeod intersection;
-    bool ok = SGGeodesy::radialIntersection(p1->geod(), r1, p2->geod(), r2, intersection);
-    if (!ok) {
-      SG_LOG(SG_NAVAID, SG_INFO, "no valid intersection for:" << pieces[0]
-              << "/" << pieces[2]);
-      return NULL;
-    }
-
-    std::string name = p1->ident() + "-" + p2->ident();
-    return new BasicWaypt(intersection, name, nullptr);
-}
-
-WayptRef wayptFromLonLatString(const std::string& target)
-{
-    // permit lat,lon/radial/offset format
-    string_list pieces(simgear::strutils::split(target, "/"));
-    if ((pieces.size() != 1) && (pieces.size() != 3)) {
-        return {};
-    }
-    
-    SGGeod g;
-    const bool defaultToLonLat = true; // parseStringAsGeod would otherwise default to lat,lon
-    if (!simgear::strutils::parseStringAsGeod(pieces[0], &g, defaultToLonLat)) {
-        return {};
-    }
-    
-    if (pieces.size() == 3) {
-        // process offset
-        const double bearing = std::stod(pieces[1]);
-        const double distanceNm = std::stod(pieces[2]);
-        g = SGGeodesy::direct(g, bearing, distanceNm * SG_NM_TO_METER);
-    }
-    
-    // build a short name
-    const int lonDeg = static_cast<int>(g.getLongitudeDeg());
-    const int latDeg = static_cast<int>(g.getLatitudeDeg());
-
-    char buf[32];
-    char ew = (lonDeg < 0) ? 'W' : 'E';
-    char ns = (latDeg < 0) ? 'S' : 'N';
-    snprintf(buf, 32, "%c%03d%c%03d", ew, std::abs(lonDeg), ns, std::abs(latDeg));
-
-    return new BasicWaypt(g, buf, nullptr);
-}
-
-WayptRef viaFromString(const SGGeod& basePosition, const std::string& target)
-{
-    assert(target.find("VIA ") == 0);
-    string_list pieces(simgear::strutils::split(target, "/"));
-    if ((pieces.size() != 4) || (pieces[2] != "TO")) {
-        SG_LOG( SG_NAVAID, SG_WARN, "Malformed VIA specification string:" << target);
-        return nullptr;
-    }
-    
-    // TO navaid is pieces[3]
-    FGPositionedRef nav = FGPositioned::findClosestWithIdent(pieces[3], basePosition, nullptr);
-    if (!nav) {
-        SG_LOG( SG_NAVAID, SG_WARN, "TO navaid:" << pieces[3] << " unknown");
-        return nullptr;
-    }
-    
-    // airway ident is pieces[1]
-    AirwayRef airway = Airway::findByIdentAndNavaid(pieces[1], nav);
-    if (!airway) {
-        SG_LOG( SG_NAVAID, SG_WARN, "Unknown airway:" << pieces[1]);
-        return nullptr;
-    }
-
-    return new Via(nullptr, airway, nav);
-}
-
-} // of anonymous namespace
-
-WayptRef FlightPlan::waypointFromString(const string& tgt )
-{
-  string target = simgear::strutils::uppercase(tgt);
-  // extract altitude
-  double altFt = 0.0;
-  RouteRestriction altSetting = RESTRICT_NONE;
-  
-  size_t pos = target.find( '@' );
-  if ( pos != string::npos ) {
-    altFt = atof( target.c_str() + pos + 1 );
-    target = target.substr( 0, pos );
-    if ( !strcmp(fgGetString("/sim/startup/units"), "meter") )
-      altFt *= SG_METER_TO_FEET;
-    altSetting = RESTRICT_AT;
-  }
-  
-  // check for lon,lat
-    WayptRef wpt = wayptFromLonLatString(target);
-
-  SGGeod basePosition;
-  if (_legs.empty()) {
-    // route is empty, use departure position / aircraft position
-    basePosition = _departure ? _departure->geod() : globals->get_aircraft_position();
-  } else {
-    basePosition = _legs.back()->waypoint()->position();
-  }
-  
-  const double magvar = magvarDegAt(basePosition);
-
-  if (wpt) {
-      // already handled in the lat/lon test above
-  } else if (target.find("VIA ") == 0) {
-      wpt = viaFromString(basePosition, target);
-  } else {
-      string_list pieces(simgear::strutils::split(target, "/"));
-      FGPositionedRef p = FGPositioned::findClosestWithIdent(pieces.front(), basePosition);
-      if (!p) {
-        SG_LOG( SG_NAVAID, SG_INFO, "Unable to find FGPositioned with ident:" << pieces.front());
-        return nullptr;
-      }
-
-      if (pieces.size() == 1) {
-        wpt = new NavaidWaypoint(p, this);
-      } else if (pieces.size() == 3) {
-        // navaid/radial/distance-nm notation
-        double radial = atof(pieces[1].c_str()),
-        distanceNm = atof(pieces[2].c_str());
-        radial += magvar;
-        wpt = new OffsetNavaidWaypoint(p, this, radial, distanceNm);
-      } else if (pieces.size() == 2) {
-        FGAirport* apt = dynamic_cast<FGAirport*>(p.ptr());
-        if (!apt) {
-          SG_LOG(SG_NAVAID, SG_INFO, "Waypoint is not an airport:" << pieces.front());
-          return nullptr;
+    if (aIndex < 0 || aIndex >= numLegs()) { // appending, not inserting
+        const int n = numLegs();
+        if (n > 0) {
+            // if we have at least one existing leg, use its position
+            // for the search vicinity
+            return pointAlongRoute(n - 1, 0.0);
         }
 
-        if (!apt->hasRunwayWithIdent(pieces[1])) {
-          SG_LOG(SG_NAVAID, SG_INFO, "No runway: " << pieces[1] << " at " << pieces[0]);
-          return nullptr;
-        }
+        return SGGeod::invalid();
+    }
 
-        FGRunway* runway = apt->getRunwayByIdent(pieces[1]);
-        wpt = new NavaidWaypoint(runway, this);
-      } else if (pieces.size() == 4) {
-        wpt = intersectionFromString(p, basePosition, magvar, pieces);
-      }
-  }
-  
-  if (!wpt) {
-    SG_LOG(SG_NAVAID, SG_INFO, "Unable to parse waypoint:" << target);
-    return nullptr;
-  }
-  
-  if (altSetting != RESTRICT_NONE) {
-    wpt->setAltitude(altFt, altSetting);
-  }
-  return wpt;
+    // if we're somewhere in the middle of the route compute a search
+    // vicinity halfway between the previous waypoint and the one we are
+    // inserting at, i.e the middle of the leg.
+    // if we're at the beginning, just use zero of course.
+    const double normOffset = (aIndex > 0) ? -0.5 : 0.0;
+    return pointAlongRouteNorm(aIndex, normOffset);
 }
 
+WayptRef FlightPlan::waypointFromString(const string& tgt, const SGGeod& vicinity)
+{
+    SGGeod basePosition = vicinity;
+    if (!vicinity.isValid()) {
+        // compute basePosition using some heuristics
+        if (_legs.empty()) {
+            // route is empty, use departure position / aircraft position
+            basePosition = _departure ? _departure->geod() : globals->get_aircraft_position();
+        } else {
+            basePosition = _legs.back()->waypoint()->position();
+        }
+    }
+
+    return Waypt::createFromString(this, tgt, basePosition);
+}
+
+bool FlightPlan::expandVias()
+{
+    // must be called with the delegats locked, so that
+    // waypointsChanged can be set on finish
+    
+    assert(_delegateLock > 0);
+    bool didChangeAny = false;
+    
+    for (unsigned int i=1; i < _legs.size(); ) {
+      if (_legs[i]->waypoint()->type() == "via") {
+        WayptRef preceeding = _legs[i - 1]->waypoint();
+        Via* via = static_cast<Via*>(_legs[i]->waypoint());
+        WayptVec wps = via->expandToWaypoints(preceeding);
+        
+        // delete the VIA leg
+        auto it = _legs.begin() + i;
+        LegRef l = *it;
+        _legs.erase(it);
+        
+        // create new legs and insert
+          it = _legs.begin() + i;
+        
+        LegVec newLegs;
+        for (WayptRef wp : wps) {
+            newLegs.push_back(LegRef{new Leg(this, wp)});
+        }
+        
+          didChangeAny = true;
+        _legs.insert(it, newLegs.begin(), newLegs.end());
+      } else {
+        ++i; // normal case, no expansion
+      }
+    }
+    
+    return didChangeAny;
+}
 
 void FlightPlan::activate()
 {
-  FGRouteMgr* routeManager = globals->get_subsystem<FGRouteMgr>();
+    if (_isRoute) {
+        // no allowed, clone and make the non-route FP active
+        SG_LOG(SG_NAVAID, SG_DEV_ALERT, "tried to activate an is-route FlightPlan");
+        return;
+    }
+    
+  auto routeManager = globals->get_subsystem<FGRouteMgr>();
   if (routeManager) {
     if (routeManager->flightPlan() != this) {
       SG_LOG(SG_NAVAID, SG_DEBUG, "setting new flight-plan on route-manager");
@@ -1538,32 +1546,7 @@ void FlightPlan::activate()
   
   _currentIndex = 0;
   _currentWaypointChanged = true;
-  
-  for (unsigned int i=1; i < _legs.size(); ) {
-    if (_legs[i]->waypoint()->type() == "via") {
-      WayptRef preceeding = _legs[i - 1]->waypoint();
-      Via* via = static_cast<Via*>(_legs[i]->waypoint());
-      WayptVec wps = via->expandToWaypoints(preceeding);
-      
-      // delete the VIA leg
-      auto it = _legs.begin() + i;
-      LegRef l = *it;
-      _legs.erase(it);
-      
-      // create new legs and insert
-        it = _legs.begin() + i;
-      
-      LegVec newLegs;
-      for (WayptRef wp : wps) {
-          newLegs.push_back(LegRef{new Leg(this, wp)});
-      }
-      
-      _waypointsChanged = true;
-      _legs.insert(it, newLegs.begin(), newLegs.end());
-    } else {
-      ++i; // normal case, no expansion
-    }
-  }
+    _waypointsChanged = expandVias();
   
   for (auto d : _delegates) {
     d->activated();
@@ -1587,9 +1570,12 @@ FlightPlan::Leg* FlightPlan::Leg::cloneFor(FlightPlan* owner) const
 // clone local data
   c->_speed = _speed;
   c->_speedRestrict = _speedRestrict;
-  c->_altitudeFt = _altitudeFt;
+  c->_speedUnits = _speedUnits;
+  c->_altitude = _altitude;
   c->_altRestrict = _altRestrict;
-  
+  c->_altitudeUnits = _altitudeUnits;
+  c->_holdCount = c->_holdCount;
+
   return c;
 }
   
@@ -1606,36 +1592,37 @@ unsigned int FlightPlan::Leg::index() const
   return _parent->findLegIndex(this);
 }
 
-int FlightPlan::Leg::altitudeFt() const
+double FlightPlan::Leg::altitude(RouteUnits aUnits) const
 {
   if (_altRestrict != RESTRICT_NONE) {
-    return _altitudeFt;
+    return convertAltitudeUnits(_altitudeUnits, aUnits, _altitude);
   }
-  
-  return _waypt->altitudeFt();
+
+  return _waypt->altitude(aUnits);
 }
 
-int FlightPlan::Leg::speed() const
+int FlightPlan::Leg::altitudeFt() const
+{
+  return static_cast<int>(altitude(ALTITUDE_FEET));
+}
+
+double FlightPlan::Leg::speed(RouteUnits units) const
 {
   if (_speedRestrict != RESTRICT_NONE) {
-    return _speed;
+    return convertSpeedUnits(_speedUnits, units, altitudeFt(), _speed);
   }
-  
-  return _waypt->speed();
+
+  return _waypt->speed(units);
 }
 
 int FlightPlan::Leg::speedKts() const
 {
-  return speed();
+  return static_cast<int>(speed(SPEED_KNOTS));
 }
   
 double FlightPlan::Leg::speedMach() const
 {
-  if (!isMachRestrict(_speedRestrict)) {
-    return 0.0;
-  }
-  
-  return -(_speed / 100.0);
+  return speed(SPEED_MACH);
 }
 
 RouteRestriction FlightPlan::Leg::altitudeRestriction() const
@@ -1655,21 +1642,30 @@ RouteRestriction FlightPlan::Leg::speedRestriction() const
   
   return _waypt->speedRestriction();
 }
-  
-void FlightPlan::Leg::setSpeed(RouteRestriction ty, double speed)
+
+void FlightPlan::Leg::setSpeed(RouteRestriction ty, double speed, RouteUnits aUnit)
 {
   _speedRestrict = ty;
-  if (isMachRestrict(ty)) {
-    _speed = (speed * -100); 
-  } else {
-    _speed = speed;
+  if (aUnit == DEFAULT_UNITS) {
+    if (isMachRestrict(ty)) {
+      aUnit = SPEED_MACH;
+    } else {
+      // TODO: check for system in metric?
+      aUnit = SPEED_KNOTS;
+    }
   }
+  _speedUnits = aUnit;
+  _speed = speed;
 }
-  
-void FlightPlan::Leg::setAltitude(RouteRestriction ty, int altFt)
+
+void FlightPlan::Leg::setAltitude(RouteRestriction ty, double alt, RouteUnits aUnit)
 {
   _altRestrict = ty;
-  _altitudeFt = altFt;
+  if (aUnit == DEFAULT_UNITS) {
+    aUnit = ALTITUDE_FEET;
+  }
+  _altitudeUnits = aUnit;
+  _altitude = alt;
 }
 
 double FlightPlan::Leg::courseDeg() const
@@ -1745,16 +1741,24 @@ void FlightPlan::Leg::writeToProperties(SGPropertyNode* aProp) const
 {
     if (_speedRestrict != RESTRICT_NONE) {
         aProp->setStringValue("speed-restrict", restrictionToString(_speedRestrict));
-        if (_speedRestrict == SPEED_RESTRICT_MACH) {
-            aProp->setDoubleValue("speed", speedMach());
+        if (_speedUnits == SPEED_MACH) {
+      aProp->setDoubleValue("speed-mach", _speed);
+        } else if (_speedUnits == SPEED_KPH) {
+      aProp->setDoubleValue("speed-kph", _speed);
         } else {
-            aProp->setDoubleValue("speed", _speed);
+      aProp->setDoubleValue("speed", _speed);
         }
     }
   
     if (_altRestrict != RESTRICT_NONE) {
         aProp->setStringValue("alt-restrict", restrictionToString(_altRestrict));
-        aProp->setDoubleValue("altitude-ft", _altitudeFt);
+        if (_altitudeUnits == ALTITUDE_FLIGHTLEVEL) {
+      aProp->setDoubleValue("flight-level", _altitude);
+        } else if (_altitudeUnits == ALTITUDE_METER) {
+      aProp->setDoubleValue("altitude-m", _altitude);
+        } else {
+      aProp->setDoubleValue("altitude-ft", _altitude);
+        }
     }
     
     if (_holdCount > 0) {
@@ -1945,6 +1949,16 @@ bool FlightPlan::followLegTrackToFixes() const
     return _followLegTrackToFix;
 }
 
+void FlightPlan::setMaxFlyByTurnAngle(double deg)
+{
+    _maxFlyByTurnAngle = deg;
+}
+
+double FlightPlan::maxFlyByTurnAngle() const
+{
+    return _maxFlyByTurnAngle;
+}
+
 std::string FlightPlan::icaoAircraftCategory() const
 {
     std::string r;
@@ -2209,9 +2223,10 @@ ICAOFlightType FlightPlan::flightType() const
 void FlightPlan::setCruiseSpeedKnots(int kts)
 {
     lockDelegates();
+    _cruiseDataChanged = true;
     _cruiseAirspeedKnots = kts;
     _cruiseAirspeedMach = 0.0;
-    _cruiseDataChanged = true;
+    _cruiseAirspeedKph = 0;
     unlockDelegates();
 }
 
@@ -2224,8 +2239,9 @@ void FlightPlan::setCruiseSpeedMach(double mach)
 {
     lockDelegates();
     _cruiseDataChanged = true;
-    _cruiseAirspeedMach = mach;
     _cruiseAirspeedKnots = 0;
+    _cruiseAirspeedMach = mach;
+    _cruiseAirspeedKph = 0;
     unlockDelegates();
 }
 
@@ -2234,12 +2250,28 @@ double FlightPlan::cruiseSpeedMach() const
     return _cruiseAirspeedMach;
 }
 
+void FlightPlan::setCruiseSpeedKPH(int kph)
+{
+    lockDelegates();
+    _cruiseDataChanged = true;
+    _cruiseAirspeedKnots = 0;
+    _cruiseAirspeedMach = 0.0;
+    _cruiseAirspeedKph = kph;
+    unlockDelegates();
+}
+
+int FlightPlan::cruiseSpeedKPH() const
+{
+    return _cruiseAirspeedKph;
+}
+
 void FlightPlan::setCruiseFlightLevel(int flightLevel)
 {
     lockDelegates();
     _cruiseDataChanged = true;
-    _cruiseFlightLevel = flightLevel;
     _cruiseAltitudeFt = 0;
+    _cruiseAltitudeM = 0;
+    _cruiseFlightLevel = flightLevel;
     unlockDelegates();
 }
 
@@ -2253,6 +2285,7 @@ void FlightPlan::setCruiseAltitudeFt(int altFt)
     lockDelegates();
     _cruiseDataChanged = true;
     _cruiseAltitudeFt = altFt;
+    _cruiseAltitudeM = 0;
     _cruiseFlightLevel = 0;
     unlockDelegates();
 }
@@ -2262,6 +2295,21 @@ int FlightPlan::cruiseAltitudeFt() const
     return _cruiseAltitudeFt;
 }
 
+void FlightPlan::setCruiseAltitudeM(int altM)
+{
+    lockDelegates();
+    _cruiseDataChanged = true;
+    _cruiseAltitudeFt = 0;
+    _cruiseAltitudeM = altM;
+    _cruiseFlightLevel = 0;
+    unlockDelegates();
+}
+
+int FlightPlan::cruiseAltitudeM() const
+{
+    return _cruiseAltitudeM;
+}
+
 void FlightPlan::forEachLeg(const LegVisitor& lv)
 {
     std::for_each(_legs.begin(), _legs.end(), lv);
@@ -2271,9 +2319,9 @@ void FlightPlan::forEachLeg(const LegVisitor& lv)
 int FlightPlan::indexOfFirstNonDepartureWaypoint() const
 {
     const auto numLegs = _legs.size();
-    for (int i = 0; i < numLegs; ++i) {
+    for (size_t i = 0; i < numLegs; ++i) {
         if (!(_legs.at(i)->waypoint()->flags() & WPT_DEPARTURE))
-            return i ;
+            return static_cast<int>(i);
     }
     
     // all waypoints are marked as departure
@@ -2283,9 +2331,9 @@ int FlightPlan::indexOfFirstNonDepartureWaypoint() const
 int FlightPlan::indexOfFirstArrivalWaypoint() const
 {
     const auto numLegs = _legs.size();
-    for (int i = 0; i < numLegs; ++i) {
+    for (size_t i = 0; i < numLegs; ++i) {
         if (_legs.at(i)->waypoint()->flags() & WPT_ARRIVAL)
-            return i;
+            return static_cast<int>(i);
     }
     
     // no waypoints are marked as arrival
@@ -2295,9 +2343,9 @@ int FlightPlan::indexOfFirstArrivalWaypoint() const
 int FlightPlan::indexOfFirstApproachWaypoint() const
 {
     const auto numLegs = _legs.size();
-    for (int i = 0; i < numLegs; ++i) {
+    for (size_t i = 0; i < numLegs; ++i) {
         if (_legs.at(i)->waypoint()->flags() & WPT_APPROACH)
-            return i;
+            return static_cast<int>(i);
     }
     
     // no waypoints are marked as arrival
