@@ -1,28 +1,10 @@
-// navdb.cxx -- top level navaids management routines
-//
-// Written by Curtis Olson, started May 2004.
-//
-// Copyright (C) 2004  Curtis L. Olson - http://www.flightgear.org/~curt
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// $Id$
+/*
+ * SPDX-FileCopyrightText: Copyright (C) 2004  Curtis L. Olson - http://www.flightgear.org/~curt
+ * SPDX_FileComment: top level navaids management routines
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+#include "config.h"
 
 #include <string>
 #include <vector>
@@ -31,6 +13,7 @@
 #include <cstddef>              // std::size_t
 #include <cerrno>
 #include <limits>
+#include <stdexcept>
 
 #include "navdb.hxx"
 
@@ -60,6 +43,8 @@
 using std::string;
 using std::vector;
 
+namespace strutils = simgear::strutils;
+
 // Duplicate navaids with the same ident will be removed if the disance
 // between them is less than this.
 static const double DUPLICATE_DETECTION_RADIUS_NM = 10;
@@ -155,7 +140,7 @@ static bool canBeDuplicate(FGPositionedRef ref, FGPositioned::Type type,
 // corresponding data into the NavDataCache.
 PositionedID NavLoader::processNavLine(
   const string& line, const string& utf8Path, unsigned int lineNum,
-  FGPositioned::Type type, unsigned long version)
+  FGPositioned::Type type, unsigned int version)
 {
   NavDataCache* cache = NavDataCache::instance();
   int rowCode, elev_ft, freq, range;
@@ -423,10 +408,12 @@ PositionedID NavLoader::processNavLine(
 }
 
 // load and initialize the navigational databases
-void NavLoader::loadNav(const SGPath& path, std::size_t bytesReadSoFar,
+void NavLoader::loadNav(const NavDataCache::SceneryLocation& sceneryLocation,
+                        std::size_t bytesReadSoFar,
                         std::size_t totalSizeOfAllDatFiles)
 {
   NavDataCache* cache = NavDataCache::instance();
+  const SGPath path = sceneryLocation.datPath;
   const string utf8Path = path.utf8Str();
   sg_gzifstream in(path);
 
@@ -445,18 +432,33 @@ void NavLoader::loadNav(const SGPath& path, std::size_t bytesReadSoFar,
   }
 
   unsigned int lineNumber;
-  unsigned long version;
+  unsigned int version;
+  vector<string> fields(simgear::strutils::split(line, 0, 1));
 
   try {
-    vector<string> fields(simgear::strutils::split(line, 0, 1));
-    version = std::stoul(fields[0]);
-  } catch (const std::logic_error&) {
-    std::string errMsg = utf8Path + ": unable to parse version from header";
+    if (fields.empty()) {
+      throw sg_format_exception();
+    }
+    version = strutils::readNonNegativeInt<unsigned int>(fields[0]);
+  } catch (const sg_exception& exc) {
     std::string strippedLine = simgear::strutils::stripTrailingNewlines(line);
-    SG_LOG(SG_NAVAID, SG_ALERT, errMsg << ": " << strippedLine );
+    std::string errMsg = utf8Path + ": ";
+
+    if (fields.empty()) {
+      errMsg += "unable to parse format version: empty line";
+      SG_LOG(SG_NAVAID, SG_ALERT, errMsg);
+    } else {
+      errMsg += "unable to parse format version";
+      SG_LOG(SG_NAVAID, SG_ALERT,
+             errMsg << " (" << exc.what() << "): " << strippedLine);
+    }
+
     throw sg_format_exception(errMsg, strippedLine);
   }
 
+  SG_LOG(SG_NAVAID, SG_INFO,
+         "nav.dat format version (" << utf8Path << "): " << version);
+
   for (lineNumber = 3; std::getline(in, line); lineNumber++) {
     processNavLine(line, utf8Path, lineNumber, FGPositioned::INVALID, version);
 
