@@ -24,9 +24,11 @@
 
 #include <simgear/structure/exception.hxx>
 
+#include <Airports/airport.hxx>
 #include <Airports/runways.hxx>
 #include <Navaids/waypoint.hxx>
 
+#include <iterator> // required for WIN
 using std::string;
 
 namespace flightgear
@@ -50,7 +52,9 @@ Approach::Approach(const string& aIdent, ProcedureType ty) :
 {
 
 }
-    
+
+Approach::~Approach() = default;
+
 Approach* Approach::createTempApproach(const std::string& aIdent, FGRunway* aRunway, const WayptVec& aPath)
 {
     Approach* app = new Approach(aIdent, PROCEDURE_APPROACH_RNAV);
@@ -100,13 +104,22 @@ void Approach::addTransition(Transition* aTrans)
   aTrans->mark(WPT_APPROACH);
 }
 
-bool Approach::route(WayptRef aIAF, WayptVec& aWps)
+bool Approach::routeWithTransition(FGRunwayRef runway, Transition* trans, WayptVec& aWps)
+{
+    if (!trans)
+        return false;
+
+    trans->route(aWps);
+    bool ok = routeFromVectors(aWps);
+    return ok;
+}
+
+bool Approach::route(FGRunwayRef runway, WayptRef aIAF, WayptVec& aWps)
 {
   if (aIAF.valid()) {
-    WptTransitionMap::iterator it;
     bool haveTrans = false;
-    for (it = _transitions.begin(); it != _transitions.end(); ++it) {
-      Transition* t= it->second;
+    for (auto te : _transitions) {
+      auto t = te.second;
       if (t->enroute()->matches(aIAF)) {
         t->route(aWps);
         haveTrans = true;
@@ -115,13 +128,27 @@ bool Approach::route(WayptRef aIAF, WayptVec& aWps)
     } // of transitions iteration
     
     if (!haveTrans) {
-      SG_LOG(SG_NAVAID, SG_INFO, "approach " << ident() << " has no transition " <<
-        "for IAF: " << aIAF->ident());
-      return false;
+      if (_primary.front()->matches(aIAF)) {
+        // direct IAF on the approach, no transition is needed
+      } else {
+        // we couldn't find the IAF at the front of any obvious thing - either
+        // the primary waypoints or any transition we have defined.
+        // warn and just use the primary waypoints down below
+        SG_LOG(SG_NAVAID, SG_INFO, "approach " << ident() << " has no transition " <<
+               "for IAF: " << aIAF->ident());
+      }
     }
   }
   
-  return routeFromVectors(aWps);
+  bool ok = routeFromVectors(aWps);
+  
+  if (ok && !aWps.empty() && aIAF.valid() && aWps.front()->matches(aIAF)) {
+    // don't duplicate the IAF into the route we return. This avoids a
+    // duplicated waypt between the end of a STAR and the approach
+    aWps.erase(aWps.begin());
+  }
+  
+  return ok;
 }
 
 bool Approach::routeFromVectors(WayptVec& aWps)
@@ -138,7 +165,29 @@ bool Approach::isApproach(ProcedureType ty)
 {
   return (ty >= PROCEDURE_APPROACH_ILS) && (ty <= PROCEDURE_APPROACH_RNAV);
 }
-  
+
+string_list Approach::transitionIdents() const
+{
+    string_list r;
+    r.reserve(_transitions.size());
+    for (const auto& t : _transitions) {
+        r.push_back(t.second->ident());
+    }
+    return r;
+}
+
+Transition* Approach::findTransitionByName(const string& aIdent) const
+{
+    auto it = std::find_if(_transitions.begin(), _transitions.end(), [aIdent](const WptTransitionMap::value_type& t) {
+        return aIdent == t.second->ident();
+    });
+
+    if (it == _transitions.end())
+        return nullptr;
+
+    return it->second;
+}
+
 //////////////////////////////////////////////////////////////////////////////
 
 ArrivalDeparture::ArrivalDeparture(const string& aIdent, FGAirport* apt) :
@@ -243,7 +292,14 @@ bool ArrivalDeparture::commonRoute(Transition* t, WayptVec& aPath, FGRunwayRef a
   }
   
   RunwayTransitionMap::iterator r = _runways.find(aRwy);
-  assert(r != _runways.end());
+  if (r == _runways.end()) {
+      // runway doesn't match STAR/SID; this may be intentional (cf. EDDF
+      // transitions), so we have to cater for it: we will just return at this
+      // point, and trust the calling code to insert VECTORS or select an
+      // appropriate approach transition.
+      return true;
+  }
+
   if (!r->second) {
     // no transitions specified. Not great, but not
     // much we can do about it. Calling code will insert VECTORS to the approach
@@ -265,13 +321,28 @@ Transition* ArrivalDeparture::findTransitionByEnroute(Waypt* aEnroute) const
   WptTransitionMap::const_iterator eit;
   for (eit = _enrouteTransitions.begin(); eit != _enrouteTransitions.end(); ++eit) {
     if (eit->second->enroute()->matches(aEnroute)) {
-      SG_LOG(SG_NAVAID, SG_INFO, ident() << " using enroute transition " << eit->second->ident());
       return eit->second;
     }
   } // of enroute transition iteration
   
   return NULL;
 }
+    
+Transition* ArrivalDeparture::findTransitionByEnroute(FGPositioned* aEnroute) const
+{
+    if (!aEnroute) {
+        return NULL;
+    }
+    
+    WptTransitionMap::const_iterator eit;
+    for (eit = _enrouteTransitions.begin(); eit != _enrouteTransitions.end(); ++eit) {
+        if (eit->second->enroute()->matches(aEnroute)) {
+            return eit->second;
+        }
+    } // of enroute transition iteration
+    
+    return NULL;
+}
 
 WayptRef ArrivalDeparture::findBestTransition(const SGGeod& aPos) const
 {
