@@ -1,46 +1,32 @@
-// AV400WSim.cxx -- Garmin 400 series protocal class.  This AV400WSim
-// protocol generates the set of "simulator" commands a garmin 400 WAAS
-// series gps would expect as input in simulator mode.  The AV400W
-// protocol parses the set of commands that a garmin 400W series gps
-// would emit.
-// 
-// The Garmin WAAS GPS uses 2 serial channels to communicate with the
-// simulator.  These 2 channels are represented by the FGAV400WSimA and
-// the FGAV400WSimB classes.  The "A" channel is similar to the previous
-// AVSim400 protocol. The "B" channel is considered the "GPS" channel and
-// uses a different protocol than the "A" channel. The GPS unit expects
-// input on the "B" channel at two different frequencies (1hz and 5hz,
-// normally).  The "B" channel also expects responses to certain output
-// messages.
-//
-// Original AV400Sim code Written by Curtis Olson, started Janauary 2009.
-// This AV400W code written by Bruce Hellstrom, March 2011.
-//
-// Copyright (C) 2009  Curtis L. Olson - http://www.flightgear.org/~curt
-// Copyright (c) 2011  Bruce Hellstrom - http://www.celebritycc.com
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-//
-// $Id$
-
+/*
+ * SPDX-FileName: AV400WSim.cxx
+ * SPDX-FileComment: <text>Garmin 400 series protocal class.  This AV400WSim
+ * protocol generates the set of "simulator" commands a garmin 400 WAAS
+ * series gps would expect as input in simulator mode.  The AV400W
+ * protocol parses the set of commands that a garmin 400W series gps
+ * would emit.
+ * 
+ * The Garmin WAAS GPS uses 2 serial channels to communicate with the
+ * simulator.  These 2 channels are represented by the FGAV400WSimA and
+ * the FGAV400WSimB classes.  The "A" channel is similar to the previous
+ * AVSim400 protocol. The "B" channel is considered the "GPS" channel and
+ * uses a different protocol than the "A" channel. The GPS unit expects
+ * input on the "B" channel at two different frequencies (1hz and 5hz,
+ * normally).  The "B" channel also expects responses to certain output
+ * messages.
+ *
+ * Original AV400Sim code Written by Curtis Olson, started Janauary 2009.</text>
+ * SPDX-FileCopyrightText: Copyright (c) 2011  Bruce Hellstrom - http://www.celebritycc.com
+ * SPDX-FileContributor: Copyright (C) 2009  Curtis L. Olson - http://www.flightgear.org/~curt
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
+#include <cstdio>
+
 #include <simgear/debug/logstream.hxx>
 #include <simgear/math/sg_geodesy.hxx>
 #include <simgear/io/iochannel.hxx>
@@ -70,12 +56,12 @@ bool FGAV400WSimA::gen_message() {
 
     // create msg_h
     double obs = fgGetDouble( "/instrumentation/nav[0]/radials/selected-deg" );
-    sprintf( msg_h, "h%04d\r\n", (int)(obs*10) );
+    snprintf( msg_h, 32, "h%04d\r\n", (int)(obs*10) );
 
     // create msg_i
     double fuel = fgGetDouble( "/consumables/fuel/total-fuel-gals" );
     if ( fuel > 999.9 ) { fuel = 999.9; }
-    sprintf( msg_i, "i%04.0f\r\n", fuel*10.0 );
+    snprintf( msg_i, 32, "i%04.0f\r\n", fuel*10.0 );
 
     // create msg_j
     double gph = fgGetDouble( "/engines/engine[0]/fuel-flow-gph" );
@@ -83,10 +69,10 @@ bool FGAV400WSimA::gen_message() {
     gph += fgGetDouble( "/engines/engine[2]/fuel-flow-gph" );
     gph += fgGetDouble( "/engines/engine[3]/fuel-flow-gph" );
     if ( gph > 999.9 ) { gph = 999.9; }
-    sprintf( msg_j, "j%04.0f\r\n", gph*10.0 );
+    snprintf( msg_j, 32, "j%04.0f\r\n", gph*10.0 );
 
     // create msg_k
-    sprintf( msg_k, "k%04d%02d%02d%02d%02d%02d\r\n",
+    snprintf( msg_k, 32, "k%04d%02d%02d%02d%02d%02d\r\n",
              fgGetInt( "/sim/time/utc/year"),
              fgGetInt( "/sim/time/utc/month"),
              fgGetInt( "/sim/time/utc/day"),
@@ -97,13 +83,13 @@ bool FGAV400WSimA::gen_message() {
     // create msg_l
     alt = fgGetDouble( "/instrumentation/pressure-alt-ft" );
     if ( alt > 99999.0 ) { alt = 99999.0; }
-    sprintf( msg_l, "l%05.0f\r\n", alt );
+    snprintf( msg_l, 32, "l%05.0f\r\n", alt );
 
     // sentence type 2
     //sprintf( msg_type2, "w01%c\r\n", (char)65 );
 
     // assemble message
-    string sentence;
+    std::string sentence;
     sentence += '\002';         // STX
     sentence += msg_h;		// obs heading in deg (*10)
     sentence += msg_i;		// total fuel in gal (*10)
@@ -126,14 +112,14 @@ bool FGAV400WSimA::gen_message() {
 bool FGAV400WSimA::parse_message() {
     SG_LOG( SG_IO, SG_INFO, "parse AV400WSimA message" );
 
-    string msg = buf;
+    std::string msg = buf;
     msg = msg.substr( 0, length );
     SG_LOG( SG_IO, SG_INFO, "entire message = " << msg );
 
-    string ident = msg.substr(0, 1);
+    std::string ident = msg.substr(0, 1);
     if ( ident == "i" ) {
-        string side = msg.substr(1,1);
-        string num = msg.substr(2,3);
+        std::string side = msg.substr(1,1);
+        std::string num = msg.substr(2,3);
         if ( side == "-" ) {
             fgSetDouble("/instrumentation/av400w/cdi-deflection", 0.0);
         }
@@ -148,8 +134,8 @@ bool FGAV400WSimA::parse_message() {
         }
     }
     else if ( ident == "j" ) {
-        string side = msg.substr(1,1);
-        string num = msg.substr(2,3);
+        std::string side = msg.substr(1,1);
+        std::string num = msg.substr(2,3);
         if ( side == "-" ) {
             fgSetDouble("/instrumentation/av400w/gs-deflection", 0.0);
         }
@@ -165,7 +151,7 @@ bool FGAV400WSimA::parse_message() {
         }
     }
     else if ( ident == "k" ) {
-        string ind = msg.substr(1,1);
+        std::string ind = msg.substr(1,1);
         if ( ind == "T" ) {
             fgSetBool("/instrumentation/av400w/to-flag", true);
             fgSetBool("/instrumentation/av400w/from-flag", false);
@@ -182,7 +168,7 @@ bool FGAV400WSimA::parse_message() {
         //printf( "k, %s\n", ind.c_str() );
     }
     else if ( ident == "S" ) {
-        string ind = msg.substr(1,5);
+        std::string ind = msg.substr(1,5);
         //printf( "S - %s\n", ind.c_str() );
     }
     else {
@@ -261,9 +247,8 @@ FGAV400WSimB::FGAV400WSimB() :
 hz2(0.0),
 hz2count(0),
 hz2cycles(0),    
-flight_phase(0xFF),
+flight_phase(-1),
 req_hostid(true),
-req_raimap(false),
 req_sbas(false)
 {
     hal.clear();
@@ -282,7 +267,7 @@ FGAV400WSimB::~FGAV400WSimB() {
 
 bool FGAV400WSimB::gen_hostid_message() {
     char chksum = 0;
-    string data = "Cj\r\n";
+    std::string data = "Cj\r\n";
     data += "COPYRIGHT 2008 GARMIN LTD.       \r\n";
     data += "SFTW P/N #    006-B0339-0A\r\n";
     data += "SOFTWARE VER #           3\r\n";
@@ -295,18 +280,18 @@ bool FGAV400WSimB::gen_hostid_message() {
     data += "OPTIONS LIST    iiiiiiiiii";
     
     // calculate the checksum
-    for ( string::const_iterator cli = data.begin(); cli != data.end(); cli++ ) {
+    for ( std::string::const_iterator cli = data.begin(); cli != data.end(); cli++ ) {
         chksum ^= *cli;
     }
     
-    string sentence( "@@" );
+    std::string sentence( "@@" );
     sentence += data;
     sentence.push_back( chksum );
     sentence += "\x0D\n";
     
     length = sentence.length();
     char *bufptr = buf;
-    for ( string::const_iterator cli = sentence.begin(); cli != sentence.end(); cli++ ) {
+    for ( std::string::const_iterator cli = sentence.begin(); cli != sentence.end(); cli++ ) {
         *bufptr++ = *cli;
     }
 
@@ -315,23 +300,23 @@ bool FGAV400WSimB::gen_hostid_message() {
 
 bool FGAV400WSimB::gen_sbas_message() {
     char chksum = 0;
-    string data = "WA";
+    std::string data = "WA";
     data.push_back( '\0' );
     data += sbas_sel;
     
     // calculate the checksum
-    for ( string::const_iterator cli = data.begin(); cli != data.end(); cli++ ) {
+    for ( std::string::const_iterator cli = data.begin(); cli != data.end(); cli++ ) {
         chksum ^= *cli;
     }
     
-    string sentence( "@@" );
+    std::string sentence( "@@" );
     sentence += data;
     sentence.push_back( chksum );
     sentence += "\x0D\n";
     
     length = sentence.length();
     char *bufptr = buf;
-    for ( string::const_iterator cli = sentence.begin(); cli != sentence.end(); cli++ ) {
+    for ( std::string::const_iterator cli = sentence.begin(); cli != sentence.end(); cli++ ) {
         *bufptr++ = *cli;
     }
 
@@ -343,7 +328,7 @@ bool FGAV400WSimB::gen_Wh_message() {
     char chksum = 0;
     
     // generate the Wh message
-    string data = "Wh";
+    std::string data = "Wh";
     data.push_back( '\x0F' );
     data.append( "\x3f\x00\x00\x20\x00\x20", 6 );
     data.append( "\x4f\x00\x00\x28\x00\x30", 6 );
@@ -363,18 +348,18 @@ bool FGAV400WSimB::gen_Wh_message() {
     data.push_back( '0' );
     
     // calculate the checksum
-    for ( string::const_iterator cli = data.begin(); cli != data.end(); cli++ ) {
+    for ( std::string::const_iterator cli = data.begin(); cli != data.end(); cli++ ) {
         chksum ^= *cli;
     }
     
-    string sentence( "@@" );
+    std::string sentence( "@@" );
     sentence += data;
     sentence.push_back( chksum );
     sentence += "\x0D\n";
 
     length = sentence.length();
     char *bufptr = buf;
-    for ( string::const_iterator cli = sentence.begin(); cli != sentence.end(); cli++ ) {
+    for ( std::string::const_iterator cli = sentence.begin(); cli != sentence.end(); cli++ ) {
         *bufptr++ = *cli;
     }
 
@@ -387,7 +372,7 @@ bool FGAV400WSimB::gen_Wx_message() {
     char chksum = 0;
     
     // Now process the Wx message
-    string data = "Wx";
+    std::string data = "Wx";
     data.push_back( (char)( fgGetInt( "/sim/time/utc/month") & 0xFF ) );
     data.push_back( (char)( fgGetInt( "/sim/time/utc/day") & 0xFF ) );
     data.push_back( (char)( (fgGetInt( "/sim/time/utc/year") >> 8 ) & 0xFF ) );
@@ -403,11 +388,11 @@ bool FGAV400WSimB::gen_Wx_message() {
     data.push_back( '\0' );
     
     // calculate the checksum
-    for ( string::const_iterator cli = data.begin(); cli != data.end(); cli++ ) {
+    for ( std::string::const_iterator cli = data.begin(); cli != data.end(); cli++ ) {
         chksum ^= *cli;
     }
     
-    string sentence( "@@" );
+    std::string sentence( "@@" );
     sentence += data;
     sentence.push_back( chksum );
     sentence += "\x0D\n";
@@ -415,7 +400,7 @@ bool FGAV400WSimB::gen_Wx_message() {
     // cout << sentence;
     length = sentence.length();
     char *bufptr = buf;
-    for ( string::const_iterator cli = sentence.begin(); cli != sentence.end(); cli++ ) {
+    for ( std::string::const_iterator cli = sentence.begin(); cli != sentence.end(); cli++ ) {
         *bufptr++ = *cli;
     }
 
@@ -428,7 +413,7 @@ bool FGAV400WSimB::gen_Wt_message() {
     char chksum = 0;
     
     // generate the Wt message
-    string data = "Wt";
+    std::string data = "Wt";
     data.push_back( (char)( fgGetInt( "/sim/time/utc/month") & 0xFF ) );
     data.push_back( (char)( fgGetInt( "/sim/time/utc/day") & 0xFF ) );
     data.push_back( (char)( (fgGetInt( "/sim/time/utc/year") >> 8 ) & 0xFF ) );
@@ -508,18 +493,18 @@ bool FGAV400WSimB::gen_Wt_message() {
     data += "\x7F\xFF";
     
     // calculate the checksum
-    for ( string::const_iterator cli = data.begin(); cli != data.end(); cli++ ) {
+    for ( std::string::const_iterator cli = data.begin(); cli != data.end(); cli++ ) {
         chksum ^= *cli;
     }
     
-    string sentence( "@@" );
+    std::string sentence( "@@" );
     sentence += data;
     sentence.push_back( chksum );
     sentence += "\x0D\n";
 
     length = sentence.length();
     char *bufptr = buf;
-    for ( string::const_iterator cli = sentence.begin(); cli != sentence.end(); cli++ ) {
+    for ( std::string::const_iterator cli = sentence.begin(); cli != sentence.end(); cli++ ) {
         *bufptr++ = *cli;
     }
 
@@ -532,7 +517,7 @@ bool FGAV400WSimB::gen_Wm_message() {
     char chksum = 0;
     
     // generate the Wt message
-    string data = "Wm";
+    std::string data = "Wm";
 
     // flight phase
     data.push_back( flight_phase );
@@ -558,18 +543,18 @@ bool FGAV400WSimB::gen_Wm_message() {
     data.append( "\x00\x0F\x00\x0F\x00\x0F", 6 );
     
     // calculate the checksum
-    for ( string::const_iterator cli = data.begin(); cli != data.end(); cli++ ) {
+    for ( std::string::const_iterator cli = data.begin(); cli != data.end(); cli++ ) {
         chksum ^= *cli;
     }
     
-    string sentence( "@@" );
+    std::string sentence( "@@" );
     sentence += data;
     sentence.push_back( chksum );
     sentence += "\x0D\n";
 
     length = sentence.length();
     char *bufptr = buf;
-    for ( string::const_iterator cli = sentence.begin(); cli != sentence.end(); cli++ ) {
+    for ( std::string::const_iterator cli = sentence.begin(); cli != sentence.end(); cli++ ) {
         *bufptr++ = *cli;
     }
 
@@ -581,7 +566,7 @@ bool FGAV400WSimB::gen_Wv_message() {
     char chksum = 0;
     
     // generate the Wt message
-    string data = "Wv";
+    std::string data = "Wv";
 
     // data is valid
     data += "1";
@@ -611,11 +596,11 @@ bool FGAV400WSimB::gen_Wv_message() {
     data.push_back( (char)( vnup & 0xFF ) );
 
     // calculate the checksum
-    for ( string::const_iterator cli = data.begin(); cli != data.end(); cli++ ) {
+    for ( std::string::const_iterator cli = data.begin(); cli != data.end(); cli++ ) {
         chksum ^= *cli;
     }
     
-    string sentence( "@@" );
+    std::string sentence( "@@" );
     sentence += data;
     sentence.push_back( chksum );
     sentence += "\x0D\n";
@@ -623,7 +608,7 @@ bool FGAV400WSimB::gen_Wv_message() {
     // cout << sentence;
     length = sentence.length();
     char *bufptr = buf;
-    for ( string::const_iterator cli = sentence.begin(); cli != sentence.end(); cli++ ) {
+    for ( std::string::const_iterator cli = sentence.begin(); cli != sentence.end(); cli++ ) {
         *bufptr++ = *cli;
     }
 
@@ -631,12 +616,12 @@ bool FGAV400WSimB::gen_Wv_message() {
 }
 
 
-bool FGAV400WSimB::verify_checksum( string message, int datachars ) {
+bool FGAV400WSimB::verify_checksum( std::string message, int datachars ) {
     bool bRet = false;
-    string dataseg = message.substr(SOM_SIZE, datachars);
+    std::string dataseg = message.substr(SOM_SIZE, datachars);
     char chksum = 0;
     char cs = message[SOM_SIZE + datachars];
-    for ( string::const_iterator cli = dataseg.begin();
+    for ( std::string::const_iterator cli = dataseg.begin();
           cli != dataseg.end(); cli++ ) {
         chksum ^= *cli;
     }
@@ -646,7 +631,7 @@ bool FGAV400WSimB::verify_checksum( string message, int datachars ) {
     }
     else {
         SG_LOG( SG_IO, SG_INFO, "bad input checksum: " << message );
-        //string msgid = asciitize_message( message );
+        //std::string msgid = asciitize_message( message );
         //printf( "FGAV400SimB::verify_checksum bad input checksum:\n%s\n", msgid.c_str() );
     }
         
@@ -654,10 +639,10 @@ bool FGAV400WSimB::verify_checksum( string message, int datachars ) {
 }
 
 
-string FGAV400WSimB::asciitize_message( string message ) {
-    string asciimsg;
+std::string FGAV400WSimB::asciitize_message( std::string message ) {
+    std::string asciimsg;
 
-    for ( string::const_iterator cli = message.begin();
+    for ( std::string::const_iterator cli = message.begin();
           cli != message.end(); cli++ ) 
     {
         unsigned char uc = static_cast<unsigned char>(*cli);
@@ -666,7 +651,7 @@ string FGAV400WSimB::asciitize_message( string message ) {
         }
         else {
             char tempbuf[20];
-            sprintf( tempbuf, "\\x%02X", uc );
+            snprintf( tempbuf, 20, "\\x%02X", uc );
             asciimsg += tempbuf;
         }
     }
@@ -674,8 +659,8 @@ string FGAV400WSimB::asciitize_message( string message ) {
     return( asciimsg );
 }
 
-string FGAV400WSimB::buffer_to_string() {
-    string message;
+std::string FGAV400WSimB::buffer_to_string() {
+    std::string message;
     char *bufctr = buf;
     
     for ( int xctr = 0; xctr < length; xctr++ ) {
@@ -689,20 +674,20 @@ string FGAV400WSimB::buffer_to_string() {
 bool FGAV400WSimB::parse_message() {
     SG_LOG( SG_IO, SG_INFO, "parse AV400WSimB message" );
 
-    string msg = buffer_to_string();
+    std::string msg = buffer_to_string();
     
-    string som = msg.substr(0, 2);
+    std::string som = msg.substr(0, 2);
     if ( som != "@@" ) {
         SG_LOG( SG_IO, SG_INFO, "bad start message" );
         return false;
     }
 
-    string ident = msg.substr(2,2);
+    std::string ident = msg.substr(2,2);
     
     if ( ident == "AH" ) { // Flight Phase
         if ( verify_checksum( msg, 3 ) ) {
             flight_phase = msg[4];
-            //string ascmsg = asciitize_message( msg );
+            //std::string ascmsg = asciitize_message( msg );
             //printf( "%10d received AH %s\n", outputctr, ascmsg.c_str() );
             switch( flight_phase ) {
                 case FGAV400WSimB::PHASE_OCEANIC: // Oceanic
@@ -807,7 +792,7 @@ bool FGAV400WSimB::parse_message() {
         // Do nothing until we know what it does
     }
     else {
-        string unkmsg = msg.substr( 0, 4 );
+        std::string unkmsg = msg.substr( 0, 4 );
         printf( "parse_message unknown: %s\n", unkmsg.c_str() );
     }
     
@@ -1007,8 +992,8 @@ bool FGAV400WSimB::process() {
         length++;
         
         if ( gotCr && gotLf ) { // message done
-            //string msg = buffer_to_string();
-            //string ascmsg = asciitize_message( msg );
+            //std::string msg = buffer_to_string();
+            //std::string ascmsg = asciitize_message( msg );
             //printf( "Received message\n" );
             //printf( "%s\n", ascmsg.c_str() );
             //printf( "got message\n" );
