@@ -18,12 +18,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
+#include "config.h"
+
 #include "HTTPClient.hxx"
 
 #include <cassert>
 
 #include <Main/fg_props.hxx>
-#include <Include/version.h>
 
 #include <simgear/sg_inlines.h>
 
@@ -47,10 +48,10 @@ typedef nasal::Ghost<pkg::PackageRef> NasalPackage;
 typedef nasal::Ghost<pkg::CatalogRef> NasalCatalog;
 typedef nasal::Ghost<pkg::InstallRef> NasalInstall;
 
-const char* OFFICIAL_CATALOG_ID = "org.flightgear.fgaddon";
+static const char* OFFICIAL_CATALOG_ID = "org.flightgear.fgaddon.stable_2024";
 
 // fallback URL is used when looking up a version-specific catalog fails
-const char* FALLBACK_CATALOG_URL = "http://mirrors.ibiblio.org/flightgear/ftp/Aircraft/catalog.xml";
+static const char* FALLBACK_CATALOG_URL = "http://mirrors.ibiblio.org/flightgear/ftp/Aircraft-2024/catalog.xml";
 
 namespace {
 
@@ -71,7 +72,7 @@ namespace {
     std::string _getDefaultCatalogUrl()
     {
         return fgGetString("/sim/package-system/default-catalog/url",
-                    "http://mirrors.ibiblio.org/flightgear/ftp/" FLIGHTGEAR_VERSION "/catalog.xml");
+                           "http://mirrors.ibiblio.org/flightgear/ftp/" FLIGHTGEAR_MAJOR_MINOR_VERSION "/catalog.xml");
     }
 } // of anonymous namespace
 
@@ -85,6 +86,19 @@ FGHTTPClient::~FGHTTPClient()
 {
 }
 
+FGHTTPClient* FGHTTPClient::getOrCreate()
+{
+    auto ext = globals->get_subsystem<FGHTTPClient>();
+    if (ext) {
+        return ext;
+    }
+
+    ext = globals->get_subsystem_mgr()->add<FGHTTPClient>();
+    ext->init();
+    return ext;
+}
+
+
 void FGHTTPClient::init()
 {
     // launcher may need to setup HTTP access abnormally early, so
@@ -123,12 +137,15 @@ bool FGHTTPClient::isDefaultCatalogInstalled() const
     return getDefaultCatalog().valid();
 }
 
-void FGHTTPClient::addDefaultCatalog()
+pkg::CatalogRef FGHTTPClient::addDefaultCatalog()
 {
     pkg::CatalogRef defaultCatalog = getDefaultCatalog();
     if (!defaultCatalog.valid()) {
-      pkg::Catalog::createFromUrl(globals->packageRoot(), getDefaultCatalogUrl());
+        auto cat = pkg::Catalog::createFromUrl(globals->packageRoot(), getDefaultCatalogUrl());
+        return cat;
     }
+
+    return defaultCatalog;
 }
 
 std::string FGHTTPClient::getDefaultCatalogId() const
@@ -206,6 +223,21 @@ static naRef f_catalog_search(pkg::Catalog& cat, const nasal::CallContext& ctx)
     return ctx.to_nasal(result);
 }
 
+static naRef f_catalog_packages(pkg::Catalog& cat, const nasal::CallContext& ctx)
+{
+    // TODO: support package types
+    pkg::PackageList result = cat.packages();
+    return ctx.to_nasal(result);
+}
+
+
+static naRef f_catalog_installedPackages(pkg::Catalog& cat, naContext c)
+{
+    // TODO: support package types
+    pkg::PackageList result = cat.installedPackages();
+    return nasal::to_nasal(c, result);
+}
+
 static naRef f_package_variants(pkg::Package& pack, naContext c)
 {
     nasal::Hash h(c);
@@ -233,11 +265,11 @@ void FGHTTPClient::postinit()
   .member("id", &pkg::Catalog::id)
   .member("url", &pkg::Catalog::url)
   .member("description", &pkg::Catalog::description)
-  .method("packages", &pkg::Catalog::packages)
+  .method("packages", &f_catalog_packages)
   .method("packageById", &pkg::Catalog::getPackageById)
   .method("refresh", &pkg::Catalog::refresh)
   .method("needingUpdate", &pkg::Catalog::packagesNeedingUpdate)
-  .member("installed", &pkg::Catalog::installedPackages)
+  .member("installed", &f_catalog_installedPackages)
   .method("search", &f_catalog_search)
   .member("enabled", &pkg::Catalog::isEnabled);
 
@@ -274,7 +306,7 @@ void FGHTTPClient::postinit()
   
   pkg::Root* packageRoot = globals->packageRoot();
   if (packageRoot) {
-    FGNasalSys* nasalSys = (FGNasalSys*) globals->get_subsystem("nasal");
+    auto nasalSys = globals->get_subsystem<FGNasalSys>();
     nasal::Hash nasalGlobals = nasalSys->getGlobals();
     nasal::Hash nasalPkg = nasalGlobals.createHash("pkg"); // module
     nasalPkg.set("root", packageRoot);
@@ -297,3 +329,9 @@ void FGHTTPClient::makeRequest(const simgear::HTTP::Request_ptr& req)
 {
   _http->makeRequest(req);
 }
+
+
+// Register the subsystem.
+SGSubsystemMgr::Registrant<FGHTTPClient> registrantFGHTTPClient(
+    SGSubsystemMgr::GENERAL,
+    {{"nasal", SGSubsystemMgr::Dependency::HARD}});
