@@ -1,24 +1,9 @@
-// atlas.cxx -- Atlas protocal class
-//
-// Written by Curtis Olson, started November 1999.
-//
-// Copyright (C) 1999  Curtis L. Olson - http://www.flightgear.org/~curt
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// $Id$
+/*
+ * SPDX-FileName: atlas.cxx
+ * SPDX-FileComment: Atlas protocol class
+ * SPDX-FileCopyrightText: Copyright (C) 1999  Curtis L. Olson - http://www.flightgear.org/~curt
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -88,7 +73,7 @@ bool FGAtlas::gen_message() {
     SGTime *t = globals->get_time_params();
 
     char utc[10];
-    sprintf( utc, "%02d%02d%02d", 
+    snprintf( utc, sizeof(utc), "%02d%02d%02d", 
 	     t->getGmt()->tm_hour, t->getGmt()->tm_min, t->getGmt()->tm_sec );
 
     char lat[20];
@@ -101,7 +86,7 @@ bool FGAtlas::gen_message() {
     }
     deg = (int)(latd);
     min = (latd - (double)deg) * 60.0;
-    sprintf( lat, "%02d%06.3f,%c", abs(deg), min, dir);
+    snprintf( lat, sizeof(lat), "%02d%06.3f,%c", abs(deg), min, dir);
 
     char lon[20];
     double lond = fdm->get_Longitude() * SGD_RADIANS_TO_DEGREES;
@@ -113,47 +98,49 @@ bool FGAtlas::gen_message() {
     }
     deg = (int)(lond);
     min = (lond - (double)deg) * 60.0;
-    sprintf( lon, "%03d%06.3f,%c", abs(deg), min, dir);
+    snprintf( lon, sizeof(lon), "%03d%06.3f,%c", abs(deg), min, dir);
 
     char speed[10];
-    sprintf( speed, "%05.1f", fdm->get_V_equiv_kts() );
+    snprintf( speed, sizeof(speed), "%05.1f", fdm->get_V_equiv_kts() );
 
     char heading[10];
-    sprintf( heading, "%05.1f", fdm->get_Psi() * SGD_RADIANS_TO_DEGREES );
+    snprintf( heading, sizeof(heading), "%05.1f", fdm->get_Psi() * SGD_RADIANS_TO_DEGREES );
 
     char altitude_m[10];
-    sprintf( altitude_m, "%02d", 
+    snprintf( altitude_m, sizeof(altitude_m), "%02d", 
 	     (int)(fdm->get_Altitude() * SG_FEET_TO_METER) );
 
     char altitude_ft[10];
-    sprintf( altitude_ft, "%02d", (int)fdm->get_Altitude() );
+    snprintf( altitude_ft, sizeof(altitude_ft), "%02d", (int)fdm->get_Altitude() );
 
-    char date[10];
-    sprintf( date, "%02d%02d%02d", t->getGmt()->tm_mday, 
-	     t->getGmt()->tm_mon+1, t->getGmt()->tm_year );
+    char date[16];
+	unsigned short tm_mday = t->getGmt()->tm_mday;
+	unsigned short tm_mon = t->getGmt()->tm_mon + 1;
+	unsigned short tm_year = t->getGmt()->tm_year;
+    snprintf( date, sizeof(date), "%02u%02u%02u", tm_mday, tm_mon, tm_year);
 
     // $GPRMC,HHMMSS,A,DDMM.MMM,N,DDDMM.MMM,W,XXX.X,XXX.X,DDMMYY,XXX.X,E*XX
-    sprintf( rmc, "GPRMC,%s,A,%s,%s,%s,%s,%s,0.000,E",
+    snprintf( rmc, sizeof(rmc), "GPRMC,%s,A,%s,%s,%s,%s,%s,0.000,E",
 	     utc, lat, lon, speed, heading, date );
-    sprintf( rmc_sum, "%02X", calc_atlas_cksum(rmc) );
+    snprintf( rmc_sum, sizeof(rmc_sum), "%02X", calc_atlas_cksum(rmc) );
 
-    sprintf( gga, "GPGGA,%s,%s,%s,1,,,%s,F,,,,",
+    snprintf( gga, sizeof(gga), "GPGGA,%s,%s,%s,1,,,%s,F,,,,",
 	     utc, lat, lon, altitude_ft );
-    sprintf( gga_sum, "%02X", calc_atlas_cksum(gga) );
+    snprintf( gga_sum, sizeof(gga_sum), "%02X", calc_atlas_cksum(gga) );
 
-    sprintf( patla, "PATLA,%.2f,%.1f,%.2f,%.1f,%.0f",
+    snprintf( patla, sizeof(patla), "PATLA,%.2f,%.1f,%.2f,%.1f,%.0f",
              _nav1_freq->getDoubleValue(),
              _nav1_sel_radial->getDoubleValue(),
              _nav2_freq->getDoubleValue(),
              _nav2_sel_radial->getDoubleValue(),
              _adf_freq->getDoubleValue() );
-    sprintf( patla_sum, "%02X", calc_atlas_cksum(patla) );
+    snprintf( patla_sum, sizeof(patla_sum), "%02X", calc_atlas_cksum(patla) );
 
     SG_LOG( SG_IO, SG_DEBUG, rmc );
     SG_LOG( SG_IO, SG_DEBUG, gga );
     SG_LOG( SG_IO, SG_DEBUG, patla );
 
-    string atlas_sentence;
+    std::string atlas_sentence;
 
     // RMC sentence
     atlas_sentence = "$";
@@ -194,32 +181,32 @@ bool FGAtlas::gen_message() {
 bool FGAtlas::parse_message() {
     SG_LOG( SG_IO, SG_INFO, "parse atlas message" );
 
-    string msg = buf;
+    std::string msg = buf;
     msg = msg.substr( 0, length );
     SG_LOG( SG_IO, SG_INFO, "entire message = " << msg );
 
-    string::size_type begin_line, end_line, begin, end;
+    std::string::size_type begin_line, end_line, begin, end;
     begin_line = begin = 0;
 
     // extract out each line
     end_line = msg.find("\n", begin_line);
-    while ( end_line != string::npos ) {
-	string line = msg.substr(begin_line, end_line - begin_line);
+    while ( end_line != std::string::npos ) {
+	std::string line = msg.substr(begin_line, end_line - begin_line);
 	begin_line = end_line + 1;
 	SG_LOG( SG_IO, SG_INFO, "  input line = " << line );
 
 	// leading character
-	string start = msg.substr(begin, 1);
+	std::string start = msg.substr(begin, 1);
 	++begin;
 	SG_LOG( SG_IO, SG_INFO, "  start = " << start );
 
 	// sentence
 	end = msg.find(",", begin);
-	if ( end == string::npos ) {
+	if ( end == std::string::npos ) {
 	    return false;
 	}
     
-	string sentence = msg.substr(begin, end - begin);
+	std::string sentence = msg.substr(begin, end - begin);
 	begin = end + 1;
 	SG_LOG( SG_IO, SG_INFO, "  sentence = " << sentence );
 
@@ -229,31 +216,31 @@ bool FGAtlas::parse_message() {
 	if ( sentence == "GPRMC" ) {
 	    // time
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string utc = msg.substr(begin, end - begin);
+	    std::string utc = msg.substr(begin, end - begin);
 	    begin = end + 1;
 	    SG_LOG( SG_IO, SG_INFO, "  utc = " << utc );
 
 	    // junk
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string junk = msg.substr(begin, end - begin);
+	    std::string junk = msg.substr(begin, end - begin);
 	    begin = end + 1;
 	    SG_LOG( SG_IO, SG_INFO, "  junk = " << junk );
 
 	    // lat val
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string lat_str = msg.substr(begin, end - begin);
+	    std::string lat_str = msg.substr(begin, end - begin);
 	    begin = end + 1;
 
 	    lat_deg = atof( lat_str.substr(0, 2).c_str() );
@@ -261,11 +248,11 @@ bool FGAtlas::parse_message() {
 
 	    // lat dir
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string lat_dir = msg.substr(begin, end - begin);
+	    std::string lat_dir = msg.substr(begin, end - begin);
 	    begin = end + 1;
 
 	    lat = lat_deg + ( lat_min / 60.0 );
@@ -278,11 +265,11 @@ bool FGAtlas::parse_message() {
 
 	    // lon val
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string lon_str = msg.substr(begin, end - begin);
+	    std::string lon_str = msg.substr(begin, end - begin);
 	    begin = end + 1;
 
 	    lon_deg = atof( lon_str.substr(0, 3).c_str() );
@@ -290,11 +277,11 @@ bool FGAtlas::parse_message() {
 
 	    // lon dir
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string lon_dir = msg.substr(begin, end - begin);
+	    std::string lon_dir = msg.substr(begin, end - begin);
 	    begin = end + 1;
 
 	    lon = lon_deg + ( lon_min / 60.0 );
@@ -317,11 +304,11 @@ bool FGAtlas::parse_message() {
 
 	    // speed
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string speed_str = msg.substr(begin, end - begin);
+	    std::string speed_str = msg.substr(begin, end - begin);
 	    begin = end + 1;
 	    speed = atof( speed_str.c_str() );
 	    fdm->set_V_calibrated_kts( speed );
@@ -330,11 +317,11 @@ bool FGAtlas::parse_message() {
 
 	    // heading
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string hdg_str = msg.substr(begin, end - begin);
+	    std::string hdg_str = msg.substr(begin, end - begin);
 	    begin = end + 1;
 	    heading = atof( hdg_str.c_str() );
 	    fdm->set_Euler_Angles( fdm->get_Phi(), 
@@ -344,21 +331,21 @@ bool FGAtlas::parse_message() {
 	} else if ( sentence == "GPGGA" ) {
 	    // time
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string utc = msg.substr(begin, end - begin);
+	    std::string utc = msg.substr(begin, end - begin);
 	    begin = end + 1;
 	    SG_LOG( SG_IO, SG_INFO, "  utc = " << utc );
 
 	    // lat val
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string lat_str = msg.substr(begin, end - begin);
+	    std::string lat_str = msg.substr(begin, end - begin);
 	    begin = end + 1;
 
 	    lat_deg = atof( lat_str.substr(0, 2).c_str() );
@@ -366,11 +353,11 @@ bool FGAtlas::parse_message() {
 
 	    // lat dir
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string lat_dir = msg.substr(begin, end - begin);
+	    std::string lat_dir = msg.substr(begin, end - begin);
 	    begin = end + 1;
 
 	    lat = lat_deg + ( lat_min / 60.0 );
@@ -383,11 +370,11 @@ bool FGAtlas::parse_message() {
 
 	    // lon val
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string lon_str = msg.substr(begin, end - begin);
+	    std::string lon_str = msg.substr(begin, end - begin);
 	    begin = end + 1;
 
 	    lon_deg = atof( lon_str.substr(0, 3).c_str() );
@@ -395,11 +382,11 @@ bool FGAtlas::parse_message() {
 
 	    // lon dir
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string lon_dir = msg.substr(begin, end - begin);
+	    std::string lon_dir = msg.substr(begin, end - begin);
 	    begin = end + 1;
 
 	    lon = lon_deg + ( lon_min / 60.0 );
@@ -412,17 +399,17 @@ bool FGAtlas::parse_message() {
 
 	    // junk
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string junk = msg.substr(begin, end - begin);
+	    std::string junk = msg.substr(begin, end - begin);
 	    begin = end + 1;
 	    SG_LOG( SG_IO, SG_INFO, "  junk = " << junk );
 
 	    // junk
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
@@ -432,7 +419,7 @@ bool FGAtlas::parse_message() {
 
 	    // junk
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
@@ -442,21 +429,21 @@ bool FGAtlas::parse_message() {
 
 	    // altitude
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string alt_str = msg.substr(begin, end - begin);
+	    std::string alt_str = msg.substr(begin, end - begin);
 	    altitude = atof( alt_str.c_str() );
 	    begin = end + 1;
 
 	    // altitude units
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string alt_units = msg.substr(begin, end - begin);
+	    std::string alt_units = msg.substr(begin, end - begin);
 	    begin = end + 1;
 
 	    if ( alt_units != "F" ) {
@@ -470,51 +457,51 @@ bool FGAtlas::parse_message() {
 	} else if ( sentence == "PATLA" ) {
 	    // nav1 freq
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string nav1_freq = msg.substr(begin, end - begin);
+	    std::string nav1_freq = msg.substr(begin, end - begin);
 	    begin = end + 1;
 	    SG_LOG( SG_IO, SG_INFO, "  nav1_freq = " << nav1_freq );
 
 	    // nav1 selected radial
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string nav1_rad = msg.substr(begin, end - begin);
+	    std::string nav1_rad = msg.substr(begin, end - begin);
 	    begin = end + 1;
 	    SG_LOG( SG_IO, SG_INFO, "  nav1_rad = " << nav1_rad );
 
 	    // nav2 freq
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string nav2_freq = msg.substr(begin, end - begin);
+	    std::string nav2_freq = msg.substr(begin, end - begin);
 	    begin = end + 1;
 	    SG_LOG( SG_IO, SG_INFO, "  nav2_freq = " << nav2_freq );
 
 	    // nav2 selected radial
 	    end = msg.find(",", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string nav2_rad = msg.substr(begin, end - begin);
+	    std::string nav2_rad = msg.substr(begin, end - begin);
 	    begin = end + 1;
 	    SG_LOG( SG_IO, SG_INFO, "  nav2_rad = " << nav2_rad );
 
 	    // adf freq
 	    end = msg.find("*", begin);
-	    if ( end == string::npos ) {
+	    if ( end == std::string::npos ) {
 		return false;
 	    }
     
-	    string adf_freq = msg.substr(begin, end - begin);
+	    std::string adf_freq = msg.substr(begin, end - begin);
 	    begin = end + 1;
 	    SG_LOG( SG_IO, SG_INFO, "  adf_freq = " << adf_freq );
 	}
