@@ -1,24 +1,9 @@
-// generic.cxx -- generic protocol class
-//
-// Written by Curtis Olson, started November 1999.
-//
-// Copyright (C) 1999  Curtis L. Olson - http://www.flightgear.org/~curt
-//
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License as
-// published by the Free Software Foundation; either version 2 of the
-// License, or (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-//
-// $Id$
+/*
+ * SPDX-FileName: generic.cxx
+ * SPDX-FileComment: generic protocol class
+ * SPDX-FileCopyrightText: Copyright (C) 1999  Curtis L. Olson - http://www.flightgear.org/~curt
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -38,11 +23,12 @@
 #include <simgear/props/props_io.hxx>
 #include <simgear/math/SGMath.hxx>
 
-#include <Main/globals.hxx>
-#include <Main/fg_props.hxx>
-#include <Main/fg_os.hxx>
-#include <Main/util.hxx>
 #include "generic.hxx"
+#include <Main/fg_os.hxx>
+#include <Main/fg_props.hxx>
+#include <Main/globals.hxx>
+#include <Main/sentryIntegration.hxx>
+#include <Main/util.hxx>
 
 using simgear::strutils::unescape;
 
@@ -221,7 +207,7 @@ int FGSTXETXWrapper::unwrap( size_t n, uint8_t * buf )
   return n;
 }
 
-FGGeneric::FGGeneric(vector<string> tokens) : exitOnError(false), initOk(false), wrapper(NULL)
+FGGeneric::FGGeneric(std::vector<std::string> tokens) : exitOnError(false), initOk(false), wrapper(NULL)
 {
     size_t configToken;
     if (tokens[1] == "socket") {
@@ -238,13 +224,12 @@ FGGeneric::FGGeneric(vector<string> tokens) : exitOnError(false), initOk(false),
        return;
     }
 
-    string config = tokens[ configToken ];
+    std::string config = tokens[ configToken ];
     file_name = config+".xml";
-    direction = tokens[2];
+    set_direction(tokens[2]);
 
-    if (direction != "in" && direction != "out" && direction != "bi") {
-        SG_LOG(SG_NETWORK, SG_ALERT, "Unsuported protocol direction: "
-               << direction);
+    if (get_direction() == SG_IO_NONE) {
+        SG_LOG(SG_NETWORK, SG_ALERT, "Unsupported protocol direction: " << tokens[2]);
         return;
     }
 
@@ -267,7 +252,7 @@ union u64 {
 
 // generate the message
 bool FGGeneric::gen_message_binary() {
-    string generic_sentence;
+    std::string generic_sentence;
     length = 0;
 
     double val;
@@ -357,8 +342,8 @@ bool FGGeneric::gen_message_binary() {
         }
 
         default: // SG_STRING
-            const char *strdata = _out_message[i].prop->getStringValue();
-            int32_t strlength = strlen(strdata);
+            std::string strdata = _out_message[i].prop->getStringValue();
+            size_t strlength = strdata.length();
 
             if (binary_byte_order == BYTE_ORDER_NEEDS_CONVERSION) {
                 SG_LOG( SG_IO, SG_ALERT, "Generic protocol: "
@@ -372,7 +357,7 @@ bool FGGeneric::gen_message_binary() {
             }
             memcpy(&buf[length], &strlength, sizeof(int32_t));
             length += sizeof(int32_t);
-            strncpy(&buf[length], strdata, strlength);
+            strncpy(&buf[length], strdata.c_str(), strlength);
             length += strlength; 
             /* FIXME padding for alignment? Something like: 
              * length += (strlength % 4 > 0 ? sizeof(int32_t) - strlength % 4 : 0;
@@ -408,7 +393,7 @@ bool FGGeneric::gen_message_binary() {
 }
 
 bool FGGeneric::gen_message_ascii() {
-    string generic_sentence;
+    std::string generic_sentence;
     char tmp[255];
     length = 0;
 
@@ -419,7 +404,7 @@ bool FGGeneric::gen_message_ascii() {
             generic_sentence += var_separator;
         }
         
-        string format = simgear::strutils::sanitizePrintfFormat(_out_message[i].format);
+        std::string format = simgear::strutils::sanitizePrintfFormat(_out_message[i].format);
 
         switch (_out_message[i].type) {
         case FG_BYTE:
@@ -455,7 +440,7 @@ bool FGGeneric::gen_message_ascii() {
 
         default: // SG_STRING
             snprintf(tmp, 255, format.c_str(),
-                                   _out_message[i].prop->getStringValue());
+                                   _out_message[i].prop->getStringValue().c_str());
         }
 
         generic_sentence += tmp;
@@ -753,13 +738,14 @@ bool FGGeneric::close() {
 void
 FGGeneric::reinit()
 {
-    SGPath path( globals->get_fg_root() );
-    path.append("Protocol");
-    path.append(file_name.c_str());
-
-    SG_LOG(SG_NETWORK, SG_INFO, "Reading communication protocol from "
-                                << path);
-
+    SGPath path = globals->findDataPath("Protocol/" + file_name);
+    if (!path.exists()) {
+        SG_LOG(SG_NETWORK, SG_WARN, "Couldn't find protocol file for '" << file_name << "'");
+        return;
+    }
+    
+    SG_LOG(SG_NETWORK, SG_INFO, "Reading communication protocol from " << path);
+   
     SGPropertyNode root;
     try {
         readProperties(path, &root);
@@ -769,7 +755,8 @@ FGGeneric::reinit()
          return;
     }
 
-    if (direction == "out") {
+    const auto dir = get_direction();
+    if ((dir == SG_IO_OUT) || (dir == SG_IO_BI)) {
         SGPropertyNode *output = root.getNode("generic/output");
         if (output) {
             _out_message.clear();
@@ -779,7 +766,9 @@ FGGeneric::reinit()
                 return;
             }
         }
-    } else if (direction == "in") {
+    }
+
+    if ((dir == SG_IO_IN) || (dir == SG_IO_BI)) {
         SGPropertyNode *input = root.getNode("generic/input");
         if (input) {
             _in_message.clear();
@@ -803,7 +792,7 @@ FGGeneric::reinit()
 
 
 bool
-FGGeneric::read_config(SGPropertyNode *root, vector<_serial_prot> &msg)
+FGGeneric::read_config(SGPropertyNode *root, std::vector<_serial_prot> &msg)
 {
     binary_mode = root->getBoolValue("binary_mode");
 
@@ -864,7 +853,7 @@ FGGeneric::read_config(SGPropertyNode *root, vector<_serial_prot> &msg)
         }
 
         if ( root->hasValue("binary_footer") ) {
-            string footer_type = root->getStringValue("binary_footer");
+            std::string footer_type = root->getStringValue("binary_footer");
             if ( footer_type == "length" ) {
                 binary_footer_type = FOOTER_LENGTH;
             } else if ( footer_type.substr(0, 5) == "magic" ) {
@@ -883,7 +872,7 @@ FGGeneric::read_config(SGPropertyNode *root, vector<_serial_prot> &msg)
         }
 
         if ( root->hasValue("byte_order") ) {
-            string byte_order = root->getStringValue("byte_order");
+            std::string byte_order = root->getStringValue("byte_order");
             if (byte_order == "network" ) {
                 if ( sgIsLittleEndian() ) {
                     binary_byte_order = BYTE_ORDER_NEEDS_CONVERSION;
@@ -900,7 +889,7 @@ FGGeneric::read_config(SGPropertyNode *root, vector<_serial_prot> &msg)
         }
 
         if( root->hasValue( "wrapper" ) ) {
-            string w = root->getStringValue( "wrapper" );
+            std::string w = root->getStringValue( "wrapper" );
             if( w == "kiss" )  wrapper = new FGKissWrapper();
             else if( w == "stxetx" )  wrapper = new FGSTXETXWrapper();
             else SG_LOG(SG_IO, SG_ALERT,
@@ -909,7 +898,7 @@ FGGeneric::read_config(SGPropertyNode *root, vector<_serial_prot> &msg)
     }
 
     int record_length = 0; // Only used for binary protocols.
-    vector<SGPropertyNode_ptr> chunks = root->getChildren("chunk");
+    std::vector<SGPropertyNode_ptr> chunks = root->getChildren("chunk");
 
     for (unsigned int i = 0; i < chunks.size(); i++) {
 
@@ -928,11 +917,11 @@ FGGeneric::read_config(SGPropertyNode *root, vector<_serial_prot> &msg)
             chunk.prop = new SGPropertyNode();
             chunk.prop->setStringValue( chunks[i]->getStringValue("const", "" ) );
         } else {
-            string node = chunks[i]->getStringValue("node", "/null");
+            std::string node = chunks[i]->getStringValue("node", "/null");
             chunk.prop = fgGetNode(node.c_str(), true);
         }
 
-        string type = chunks[i]->getStringValue("type");
+        std::string type = chunks[i]->getStringValue("type");
 
         // Note: officially the type is called 'bool' but for backward
         //       compatibility 'boolean' will also be supported.
