@@ -33,7 +33,9 @@
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
 
-#include <3rdparty/cjson/cJSON.h>
+#include <cJSON.h>
+
+//#define MIRROR_DEBUG 1
 
 namespace flightgear {
 namespace http {
@@ -75,7 +77,7 @@ using std::string;
                 break;
 
             default:
-                SG_LOG(SG_NETWORK, SG_INFO, "implement me!" << type);
+                SG_LOG(SG_NETWORK, SG_DEV_ALERT, "MirrorPropTree PropertyValue : implement me!" << type);
                 break;
             }
         }
@@ -115,26 +117,22 @@ using std::string;
     };
 
 
-    struct RecentlyRemovedNode
+    struct RemovedNode
     {
-        RecentlyRemovedNode() : type(simgear::props::NONE), id(0) { }
-
-        RecentlyRemovedNode(SGPropertyNode* node, unsigned int aId) :
-            type(node->getType()),
+        RemovedNode(SGPropertyNode* node, unsigned int aId) :
             path(node->getPath()),
             id(aId)
         {}
 
-        simgear::props::Type type;
         std::string path;
-        unsigned int id;
-
-        bool operator==(const RecentlyRemovedNode& other) const
+        unsigned int id = 0;
+        
+        bool operator==(const RemovedNode& other) const
         {
-            return (type == other.type) && (path == other.path);
+            return (path == other.path);
         }
     };
-
+    
     class MirrorTreeListener : public SGPropertyChangeListener
     {
     public:
@@ -145,10 +143,9 @@ using std::string;
 
         virtual ~MirrorTreeListener()
         {
-            SG_LOG(SG_NETWORK, SG_INFO, "destroying MirrorTreeListener");
         }
 
-        virtual void valueChanged(SGPropertyNode* node) override
+        void valueChanged(SGPropertyNode* node) override
         {
             auto it = idHash.find(node);
             if (it == idHash.end()) {
@@ -164,26 +161,55 @@ using std::string;
             }
         }
 
-        virtual void childAdded(SGPropertyNode* parent, SGPropertyNode* child) override
+        void childAdded(SGPropertyNode* parent, SGPropertyNode* child) override
         {
-            // this works becuase custom operator== overload on RecentlyRemovedNode
-            // ignores the ID value.
-            RecentlyRemovedNode m(child, 0);
-            auto rrIt = std::find(recentlyRemoved.begin(), recentlyRemoved.end(), m);
+            SG_UNUSED(parent);
+            recursiveAdd(child);            
+        }
+
+        void recursiveAdd(SGPropertyNode* node)
+        {
+            RemovedNode r(node, 0 /* id not actually used */);
+#if defined (MIRROR_DEBUG)
+            SG_LOG(SG_NETWORK, SG_INFO, "looking for RR:" << r.path);
+#endif
+            auto rrIt = std::find(recentlyRemoved.begin(), recentlyRemoved.end(), r);
             if (rrIt != recentlyRemoved.end()) {
+#if defined (MIRROR_DEBUG)
+                SG_LOG(SG_NETWORK, SG_INFO, "recycling node:" << node->getPath());
+#endif
+                const auto id = rrIt->id;
                 // recycle nodes which get thrashed from Nasal (deleted + re-created
                 // each time a Nasal timer fires)
-                removedNodes.erase(rrIt->id); // don't remove it!
-                idHash.insert(std::make_pair(child, rrIt->id));
-                changedNodes.insert(child);
+                removedNodes.erase(id); // don't remove it!
+                idHash.insert(std::make_pair(node, id));
+                
+                // we can still do change compression here, but this also
+                // deals with type mutation when removing + re-adding with a
+                // different type
+                PropertyValue newVal(node);
+                if (!previousValues[id].equals(node, newVal)) {
+                    previousValues[id] = newVal;
+                    changedNodes.insert(node);
+#if defined (MIRROR_DEBUG)
+                    SG_LOG(SG_NETWORK, SG_INFO, "\tand will actually change" << node->getPath());
+#endif
+                }
+                
                 recentlyRemoved.erase(rrIt);
                 return;
             }
-
-            newNodes.insert(child);
+#if defined (MIRROR_DEBUG)
+            SG_LOG(SG_NETWORK, SG_INFO, "new node:" << node->getPath());
+#endif
+            newNodes.insert(node);
+            int child = 0;
+            for (; child < node->nChildren(); ++child) {
+                recursiveAdd(node->getChild(child));
+            }
         }
 
-        virtual void childRemoved(SGPropertyNode* parent, SGPropertyNode* child) override
+        void childRemoved(SGPropertyNode* parent, SGPropertyNode* child) override
         {
             changedNodes.erase(child); // have to do this here with the pointer valid
             newNodes.erase(child);
@@ -192,16 +218,24 @@ using std::string;
             if (it != idHash.end()) {
                 removedNodes.insert(it->second);
                 idHash.erase(it);
-
                 // record so we can map removed+add of the same property into
                 // a simple value change (this happens commonly with the canvas
                 // due to lazy Nasal scripting)
-                recentlyRemoved.push_back(RecentlyRemovedNode(child, it->second));
+                recentlyRemoved.emplace_back(child, it->second);
+#if defined (MIRROR_DEBUG)
+                SG_LOG(SG_NETWORK, SG_INFO, "adding RR:" << recentlyRemoved.back().path);
+#endif
             }
+#if defined (MIRROR_DEBUG)
+            SG_LOG(SG_NETWORK, SG_INFO, "saw remove of:" << child->getPath());
+#endif
         }
 
         void registerSubtree(SGPropertyNode* node)
         {
+#if defined (MIRROR_DEBUG)              
+            SG_LOG(SG_NETWORK, SG_INFO, "register subtree:" << node->getPath());
+#endif
             valueChanged(node);
 
             // and recurse
@@ -214,7 +248,7 @@ using std::string;
         std::set<SGPropertyNode*> newNodes;
         std::set<SGPropertyNode*> changedNodes;
         std::set<PropertyId> removedNodes;
-
+        
         PropertyId idForProperty(SGPropertyNode* prop)
         {
             auto it = idHash.find(prop);
@@ -227,18 +261,23 @@ using std::string;
 
         cJSON* makeJSONData()
         {
+#if defined (MIRROR_DEBUG)
             SGTimeStamp st;
             st.stamp();
-
-            cJSON* result = cJSON_CreateObject();
-
+            
             int newSize = newNodes.size();
             int changedSize = changedNodes.size();
             int removedSize = removedNodes.size();
-
+#endif
+            cJSON* result = cJSON_CreateObject();
             if (!newNodes.empty()) {
                 cJSON * newNodesArray = cJSON_CreateArray();
-
+                
+                // cJSON_AddItemToArray performance is O(N) due to use of a linked
+                // list, which dominates the performance here. To fix this we maintan
+                // a point to the tail of the array, keeping appends O(1)
+                cJSON* arrayTail = nullptr;
+                
                 for (auto prop : newNodes) {
                     changedNodes.erase(prop); // avoid duplicate send
                     cJSON* newPropData = cJSON_CreateObject();
@@ -250,7 +289,15 @@ using std::string;
                     if (prop->getType() != simgear::props::NONE) {
                         cJSON_AddItemToObject(newPropData, "value", JSON::valueToJson(prop));
                     }
-                    cJSON_AddItemToArray(newNodesArray, newPropData);
+                    
+                    if (arrayTail) {
+                        arrayTail->next = newPropData;
+                        newPropData->prev = arrayTail;
+                        arrayTail = newPropData;
+                    } else {
+                        cJSON_AddItemToArray(newNodesArray, newPropData);
+                        arrayTail = newPropData;
+                    }
                 }
 
                 newNodes.clear();
@@ -269,19 +316,32 @@ using std::string;
 
             if (!changedNodes.empty()) {
                 cJSON * changedNodesArray = cJSON_CreateArray();
-
+                
+                // see comment above about cJSON_AddItemToArray
+                cJSON* tail = nullptr;
+                
                 for (auto prop : changedNodes) {
                     cJSON* propData = cJSON_CreateArray();
                     cJSON_AddItemToArray(propData, cJSON_CreateNumber(idForProperty(prop)));
                     cJSON_AddItemToArray(propData, JSON::valueToJson(prop));
-                    cJSON_AddItemToArray(changedNodesArray, propData);
+                    
+                    
+                    if (tail) {
+                        tail->next = propData;
+                        propData->prev = tail;
+                        tail = propData;
+                    } else {
+                        cJSON_AddItemToArray(changedNodesArray, propData);
+                        tail = propData;
+                    }
                 }
 
                 changedNodes.clear();
                 cJSON_AddItemToObject(result, "changed", changedNodesArray);
             }
-
+#if defined (MIRROR_DEBUG)
             SG_LOG(SG_NETWORK, SG_INFO, "making JSON data took:" << st.elapsedMSec() << " for " << newSize << "/" << changedSize << "/" << removedSize);
+#endif
             recentlyRemoved.clear();
 
             return result;
@@ -299,7 +359,7 @@ using std::string;
         /// track recently removed nodes in case they are re-created imemdiately
         /// after with the same type, since we can make this much more efficient
         /// when sending over the wire.
-        std::vector<RecentlyRemovedNode> recentlyRemoved;
+        std::vector<RemovedNode> recentlyRemoved;
     };
 
 #if 0
@@ -358,32 +418,43 @@ static void handleExecCommand(cJSON* json)
 #endif
 
 MirrorPropertyTreeWebsocket::MirrorPropertyTreeWebsocket(const std::string& path) :
+    _rootPath(path),
     _listener(new MirrorTreeListener),
     _minSendInterval(100)
 {
-    _subtreeRoot = globals->get_props()->getNode(path, true);
-    _subtreeRoot->addChangeListener(_listener.get());
-    _listener->registerSubtree(_subtreeRoot);
-    _lastSendTime = SGTimeStamp::now();
+    checkNodeExists();
 }
 
 MirrorPropertyTreeWebsocket::~MirrorPropertyTreeWebsocket()
 {
-    SG_LOG(SG_NETWORK, SG_INFO, "shutting down MirrorPropertyTreeWebsocket");
 }
 
 void MirrorPropertyTreeWebsocket::close()
 {
-    _subtreeRoot->removeChangeListener(_listener.get());
+    if (_subtreeRoot) {
+        _subtreeRoot->removeChangeListener(_listener.get());
+    }
+}
 
-  #if 0
-  SG_LOG(SG_NETWORK, SG_INFO, "closing PropertyChangeWebsocket #" << id);
-  _watchedNodes.clear();
-  #endif
+void MirrorPropertyTreeWebsocket::checkNodeExists()
+{
+    _subtreeRoot = globals->get_props()->getNode(_rootPath, false);
+    if (_subtreeRoot) {
+        _subtreeRoot->addChangeListener(_listener.get());
+        _listener->registerSubtree(_subtreeRoot);
+        _lastSendTime = SGTimeStamp::now();
+    }
 }
 
 void MirrorPropertyTreeWebsocket::handleRequest(const HTTPRequest & request, WebsocketWriter &writer)
 {
+    if (!_subtreeRoot) {
+        checkNodeExists();
+        if (!_subtreeRoot) {
+            return; // still no node exists, we can't process this
+        }
+    }
+
   if (request.Content.empty()) return;
 #if 0
   /*
@@ -442,6 +513,13 @@ void MirrorPropertyTreeWebsocket::handleRequest(const HTTPRequest & request, Web
 
 void MirrorPropertyTreeWebsocket::poll(WebsocketWriter & writer)
 {
+    if (!_subtreeRoot) {
+        checkNodeExists();
+        if (!_subtreeRoot) {
+            return;
+        }
+    }
+
     if (!_listener->haveChangesToSend()) {
         return;
     }
