@@ -24,7 +24,6 @@
 #include <Main/fg_props.hxx>
 #include <simgear/debug/logstream.hxx>
 #include <simgear/misc/strutils.hxx>
-#include <boost/lexical_cast.hpp>
 
 #include <vector>
 #include <map>
@@ -75,7 +74,7 @@ private:
   class CompareNodes {
   public:
     bool operator() (const SGPropertyNode *a, const SGPropertyNode *b) const {
-        int r = strcmp(a->getName(), b->getName());
+        int r = strcmp(a->getNameString().c_str(), b->getNameString().c_str());
         return r ? r < 0 : a->getIndex() < b->getIndex();
     }
   };
@@ -186,15 +185,15 @@ static DOMElement * renderPropertyValueElement( SGPropertyNode_ptr node )
         root->setAttribute( "type", "text" );
         root->setAttribute( "name", node->getDisplayName() );
         root->setAttribute( "value", htmlSpecialChars(value) );
-        root->setAttribute( "size", boost::lexical_cast<std::string>( len ) );
+        root->setAttribute( "size", std::to_string(len) );
         root->setAttribute( "maxlength", "2047" );
     } else {
         int rows = (len / 60)+1;
         int cols = 60;
         root = new DOMNode( "textarea" );
         root->setAttribute( "name", node->getDisplayName() );
-        root->setAttribute( "cols", boost::lexical_cast<std::string>( cols ) );
-        root->setAttribute( "rows", boost::lexical_cast<std::string>( rows ) );
+        root->setAttribute( "cols", std::to_string( cols ) );
+        root->setAttribute( "rows", std::to_string( rows ) );
         root->setAttribute( "maxlength", "2047" );
         root->addChild( new DOMTextElement( htmlSpecialChars(value) ) );
     }
@@ -217,7 +216,7 @@ bool PropertyUriHandler::handleGetRequest( const HTTPRequest & request, HTTPResp
   }
 
   // skip trailing '/' - not very efficient but shouldn't happen too often
-  while( false == propertyPath.empty() && propertyPath[ propertyPath.length()-1 ] == '/' )
+  while( !propertyPath.empty() && propertyPath[ propertyPath.length()-1 ] == '/' )
     propertyPath = propertyPath.substr(0,propertyPath.length()-1);
 
   if( request.RequestVariables.get("submit") == "update" ) {
@@ -274,7 +273,7 @@ bool PropertyUriHandler::handleGetRequest( const HTTPRequest & request, HTTPResp
   catch( string & s ) { 
     SG_LOG(SG_NETWORK,SG_WARN, "httpd: reading '" << propertyPath  << "' failed: " << s );
   }
-  if( false == node.valid() ) {
+  if( !node.valid() ) {
     DOMNode * headline = new DOMNode( "h3" );
     body->addChild( headline );
     headline->addChild( new DOMTextElement( "Non-existent node requested!" ) );
