@@ -24,7 +24,7 @@
 #include <simgear/props/props.hxx>
 #include <simgear/structure/commands.hxx>
 #include <Main/globals.hxx>
-#include <3rdparty/cjson/cJSON.h>
+#include <cJSON.h>
 
 
 using std::string;
@@ -50,7 +50,7 @@ bool RunUriHandler::handleRequest( const HTTPRequest & request, HTTPResponse & r
   SG_LOG( SG_NETWORK, SG_INFO, "RunUriHandler("<< request.Content << "): command='" << command << "', arg='" << JSON::toJsonString(false,args,5) << "'");
 
   cJSON_Delete( json );
-  if ( globals->get_commands()->execute(command.c_str(), args) ) {
+  if ( globals->get_commands()->execute(command.c_str(), args, nullptr) ) {
     response.Content = "ok.";
     return true;
   } 
