@@ -18,6 +18,8 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
+#include "config.h"
+
 #include "httpd.hxx"
 #include "HTTPRequest.hxx"
 #include "PropertyChangeWebsocket.hxx"
@@ -31,9 +33,9 @@
 #include "NavdbUriHandler.hxx"
 #include "PropertyChangeObserver.hxx"
 #include <Main/fg_props.hxx>
-#include <Include/version.h>
-#include <3rdparty/mongoose/mongoose.h>
-#include <3rdparty/cjson/cJSON.h>
+
+#include <mongoose.h>
+#include <cJSON.h>
 
 #include <string>
 #include <vector>
@@ -166,73 +168,55 @@ public:
  *
  * Mongoose API is documented here: http://cesanta.com/docs/API.shtml
  */
-class MongooseHttpd: public FGHttpd {
+class MongooseHttpd : public FGHttpd
+{
 public:
+    /**
+     * Construct a MongooseHttpd object from options in a PropertyNode
+     */
+    MongooseHttpd(SGPropertyNode_ptr);
 
-  /**
-   * Construct a MongooseHttpd object from options in a PropertyNode
-   */
-  MongooseHttpd(SGPropertyNode_ptr);
+    /**
+     * Cleanup et.al.
+     */
+    ~MongooseHttpd();
 
-  /**
-   * Cleanup et.al.
-   */
-  ~MongooseHttpd();
+    // Subsystem API.
+    void bind() override;            // Currently a noop
+    void init() override;            // Reads the configuration PropertyNode, installs URIHandlers and configures mongoose
+    void unbind() override;          // shutdown of mongoose, clear connections, unregister URIHandlers
+    void update(double dt) override; // poll connections, check for changed properties
 
-  /**
-   * override SGSubsystem::init()
-   *
-   * Reads the configuration PropertyNode, installs URIHandlers and configures mongoose
-   */
-  void init();
+    // Subsystem identification.
+    static const char* staticSubsystemClassId() { return "mongoose-httpd"; }
 
-  /**
-   * override SGSubsystem::bind()
-   *
-   * Currently a noop
-   */
-  void bind();
+    /**
+     * Returns a URIHandler for the given uri
+     *
+     * @see URIHandlerMap::findHandler( const std::string & uri )
+     */
+    SGSharedPtr<URIHandler> findHandler(const std::string & uri) {
+        return _uriHandler.findHandler(uri);
+    }
 
-  /**
-   * override SGSubsystem::unbind()
-   * shutdown of mongoose, clear connections, unregister URIHandlers
-   */
-  void unbind();
-
-  /**
-   * overrride SGSubsystem::update()
-   * poll connections, check for changed properties
-   */
-  void update(double dt);
-
-  /**
-   * Returns a URIHandler for the given uri
-   *
-   * @see URIHandlerMap::findHandler( const std::string & uri )
-   */
-  SGSharedPtr<URIHandler> findHandler(const std::string & uri)
-  {
-    return _uriHandler.findHandler(uri);
-  }
-
-  Websocket * newWebsocket(const string & uri);
+    Websocket * newWebsocket(const string & uri);
 
 private:
-  int poll(struct mg_connection * connection);
-  int auth(struct mg_connection * connection);
-  int request(struct mg_connection * connection);
-  int onConnect(struct mg_connection * connection);
-  void close(struct mg_connection * connection);
+    int poll(struct mg_connection * connection);
+    int auth(struct mg_connection * connection);
+    int request(struct mg_connection * connection);
+    int onConnect(struct mg_connection * connection);
+    void close(struct mg_connection * connection);
 
-  static int staticRequestHandler(struct mg_connection *, mg_event event);
+    static int staticRequestHandler(struct mg_connection *, mg_event event);
 
-  struct mg_server *_server;
-  SGPropertyNode_ptr _configNode;
+    struct mg_server *_server;
+    SGPropertyNode_ptr _configNode;
 
-  typedef int (MongooseHttpd::*handler_t)(struct mg_connection *);
-  URIHandlerMap _uriHandler;
+    typedef int (MongooseHttpd::*handler_t)(struct mg_connection *);
+    URIHandlerMap _uriHandler;
 
-  PropertyChangeObserver _propertyChangeObserver;
+    PropertyChangeObserver _propertyChangeObserver;
 };
 
 class MongooseConnection: public Connection {
@@ -338,7 +322,7 @@ int RegularConnection::request(struct mg_connection * connection)
 
   // find a handler for the uri and remember it for possible polls on this connection
   _handler = _httpd->findHandler(request.Uri);
-  if (false == _handler.valid()) {
+  if (!_handler.valid()) {
     // uri not registered - pass false to indicate we have not processed the request
     return MG_FALSE;
   }
@@ -366,7 +350,7 @@ int RegularConnection::request(struct mg_connection * connection)
     if (name.empty() || value.empty()) continue;
     mg_send_header(connection, name.c_str(), value.c_str());
   }
-  if (done || false == response.Content.empty()) {
+  if (done || !response.Content.empty()) {
     SG_LOG(SG_NETWORK, SG_INFO,
         "RegularConnection::request() responding " << response.Content.length() << " Bytes, done=" << done);
     mg_send_data(connection, response.Content.c_str(), response.Content.length());
@@ -377,7 +361,7 @@ int RegularConnection::request(struct mg_connection * connection)
 int RegularConnection::poll(struct mg_connection * connection)
 {
   setConnection(connection);
-  if (false == _handler.valid()) return MG_FALSE;
+  if (!_handler.valid()) return MG_FALSE;
   // only return MG_TRUE if we handle this request
   return _handler->poll(this) ? MG_TRUE : MG_MORE;
 }
@@ -426,8 +410,16 @@ int WebsocketConnection::onConnect(struct mg_connection * connection)
 int WebsocketConnection::request(struct mg_connection * connection)
 {
   setConnection(connection);
+  if ((connection->wsbits & 0x0f) >= 0x8) {
+    // control opcode (close/ping/pong)
+    if ((connection->wsbits & 0x0f) == WEBSOCKET_OPCODE_PING) {
+      mg_websocket_write(connection, WEBSOCKET_OPCODE_PONG, NULL, 0);
+    }
+    return MG_MORE;
+  }
+
   MongooseHTTPRequest request(connection);
-  SG_LOG(SG_NETWORK, SG_INFO, "WebsocketConnection::request for " << request.Uri);
+  SG_LOG(SG_NETWORK, SG_DEBUG, "WebsocketConnection::request for " << request.Uri);
 
   if ( NULL == _websocket) {
     SG_LOG(SG_NETWORK, SG_ALERT, "httpd: unhandled websocket uri: " << request.Uri);
@@ -453,39 +445,39 @@ void MongooseHttpd::init()
 {
   SGPropertyNode_ptr n = _configNode->getNode("uri-handler");
   if (n.valid()) {
-    const char * uri;
+    string uri;
 
-    if ((uri = n->getStringValue("screenshot"))[0] != 0) {
+    if (!(uri = n->getStringValue("screenshot")).empty()) {
       SG_LOG(SG_NETWORK, SG_INFO, "httpd: adding screenshot uri handler at " << uri);
       _uriHandler.push_back(new flightgear::http::ScreenshotUriHandler(uri));
     }
 
-    if ((uri = n->getStringValue("property"))[0] != 0) {
+    if (!(uri = n->getStringValue("property")).empty()) {
       SG_LOG(SG_NETWORK, SG_INFO, "httpd: adding property uri handler at " << uri);
       _uriHandler.push_back(new flightgear::http::PropertyUriHandler(uri));
     }
 
-    if ((uri = n->getStringValue("json"))[0] != 0) {
+    if (!(uri = n->getStringValue("json")).empty()) {
       SG_LOG(SG_NETWORK, SG_INFO, "httpd: adding json uri handler at " << uri);
       _uriHandler.push_back(new flightgear::http::JsonUriHandler(uri));
     }
 
-    if ((uri = n->getStringValue("pkg"))[0] != 0) {
+    if (!(uri = n->getStringValue("pkg")).empty()) {
       SG_LOG(SG_NETWORK, SG_INFO, "httpd: adding pkg uri handler at " << uri);
       _uriHandler.push_back(new flightgear::http::PkgUriHandler(uri));
     }
 
-    if ((uri = n->getStringValue("flighthistory"))[0] != 0) {
+    if (!(uri = n->getStringValue("flighthistory")).empty()) {
       SG_LOG(SG_NETWORK, SG_INFO, "httpd: adding flighthistory uri handler at " << uri);
       _uriHandler.push_back(new flightgear::http::FlightHistoryUriHandler(uri));
     }
 
-    if ((uri = n->getStringValue("run"))[0] != 0) {
+    if (!(uri = n->getStringValue("run")).empty()) {
       SG_LOG(SG_NETWORK, SG_INFO, "httpd: adding run uri handler at " << uri);
       _uriHandler.push_back(new flightgear::http::RunUriHandler(uri));
     }
 
-    if ((uri = n->getStringValue("navdb"))[0] != 0) {
+    if (!(uri = n->getStringValue("navdb")).empty()) {
       SG_LOG(SG_NETWORK, SG_INFO, "httpd: adding navdb uri handler at " << uri);
       _uriHandler.push_back(new flightgear::http::NavdbUriHandler(uri));
     }
@@ -502,7 +494,7 @@ void MongooseHttpd::init()
 
     mg_set_option(_server, "document_root", docRoot.c_str());
 
-    mg_set_option(_server, "listening_port", n->getStringValue("listening-port", "8080"));
+    mg_set_option(_server, "listening_port", n->getStringValue("listening-port", "8080").c_str());
     {
       // build url rewrites relative to fg-root
       string rewrites = n->getStringValue("url-rewrites", "");
@@ -519,7 +511,7 @@ void MongooseHttpd::init()
         }
         string & lhs = rw_entries[0];
         string & rhs = rw_entries[1];
-        if (false == rewrites.empty()) rewrites.append(1, ',');
+        if (!rewrites.empty()) rewrites.append(1, ',');
         rewrites.append(lhs).append(1, '=');
         SGPath targetPath(rhs);
         if (targetPath.isAbsolute() ) {
@@ -529,23 +521,20 @@ void MongooseHttpd::init()
           rewrites.append(fgRoot).append(1, '/').append(rhs);
         }
       }
-      if (false == rewrites.empty()) mg_set_option(_server, "url_rewrites", rewrites.c_str());
+      if (!rewrites.empty()) mg_set_option(_server, "url_rewrites", rewrites.c_str());
     }
-    mg_set_option(_server, "enable_directory_listing", n->getStringValue("enable-directory-listing", "yes"));
-    mg_set_option(_server, "idle_timeout_ms", n->getStringValue("idle-timeout-ms", "30000"));
-    mg_set_option(_server, "index_files", n->getStringValue("index-files", "index.html"));
-    mg_set_option(_server, "extra_mime_types", n->getStringValue("extra-mime-types", ""));
-    mg_set_option(_server, "access_log_file", n->getStringValue("access-log-file", ""));
+    mg_set_option(_server, "enable_directory_listing", n->getStringValue("enable-directory-listing", "yes").c_str());
+    mg_set_option(_server, "idle_timeout_ms", n->getStringValue("idle-timeout-ms", "30000").c_str());
+    mg_set_option(_server, "index_files", n->getStringValue("index-files", "index.html").c_str());
+    mg_set_option(_server, "extra_mime_types", n->getStringValue("extra-mime-types", "").c_str());
+    mg_set_option(_server, "access_log_file", n->getStringValue("access-log-file", "").c_str());
 
-    if( sglog().would_log(SG_NETWORK,SG_INFO) ) {
-      SG_LOG(SG_NETWORK,SG_INFO,"starting mongoose with these options: ");
-      const char ** optionNames = mg_get_valid_option_names();
-      for( int i = 0; optionNames[i] != NULL; i+= 2 ) {
-        SG_LOG(SG_NETWORK,SG_INFO, "  > " << optionNames[i] << ": '" << mg_get_option(_server, optionNames[i]) << "'" );
-      }
-      SG_LOG(SG_NETWORK,SG_INFO,"end of mongoose options.");
+    SG_LOG(SG_NETWORK,SG_INFO,"starting mongoose with these options: ");
+    const char ** optionNames = mg_get_valid_option_names();
+    for( int i = 0; optionNames[i] != NULL; i+= 2 ) {
+      SG_LOG(SG_NETWORK,SG_INFO, "  > " << optionNames[i] << ": '" << mg_get_option(_server, optionNames[i]) << "'" );
     }
-
+    SG_LOG(SG_NETWORK,SG_INFO,"end of mongoose options.");
   }
 
   _configNode->setBoolValue("running",true);
@@ -610,8 +599,9 @@ Websocket * MongooseHttpd::newWebsocket(const string & uri)
     SG_LOG(SG_NETWORK, SG_INFO, "new PropertyChangeWebsocket for: " << uri);
     return new PropertyChangeWebsocket(&_propertyChangeObserver);
   } else if (uri.find("/PropertyTreeMirror/") == 0) {
-      SG_LOG(SG_NETWORK, SG_INFO, "new MirrorPropertyTreeWebsocket for: " << uri);
-    return new MirrorPropertyTreeWebsocket(uri.substr(20));
+    const auto path = uri.substr(20);
+    SG_LOG(SG_NETWORK, SG_INFO, "new MirrorPropertyTreeWebsocket for: " << path);
+    return new MirrorPropertyTreeWebsocket(path);
   }
   return NULL;
 }
@@ -651,11 +641,17 @@ int MongooseHttpd::staticRequestHandler(struct mg_connection * connection, mg_ev
 FGHttpd * FGHttpd::createInstance(SGPropertyNode_ptr configNode)
 {
 // only create a server if a port has been configured
-  if (false == configNode.valid()) return NULL;
+  if (!configNode.valid()) return NULL;
   string port = configNode->getStringValue("options/listening-port", "");
   if (port.empty()) return NULL;
   return new MongooseHttpd(configNode);
 }
 
+
+// Register the subsystem.
+#if 0
+SGSubsystemMgr::Registrant<MongooseHttpd> registrantMongooseHttpd;
+#endif
+
 } // namespace http
 } // namespace flightgear
