@@ -32,22 +32,22 @@
 
 #include <simgear/bucket/newbucket.hxx>
 #include <simgear/debug/logstream.hxx>
+#include <simgear/scene/tgdb/SGOceanTile.hxx>
 
 #include "tileentry.hxx"
 
 using std::string;
 
-// Constructor
+// Base constructor
 TileEntry::TileEntry ( const SGBucket& b )
     : tile_bucket( b ),
-      tileFileName(b.gen_index_str()),
       _node( new osg::LOD ),
       _priority(-FLT_MAX),
       _current_view(false),
       _time_expired(-1.0)
 {
-    tileFileName += ".stg";
-    _node->setName(tileFileName);
+    _create_orthophoto();
+    
     // Give a default LOD range so that traversals that traverse
     // active children (like the groundcache lookup) will work before
     // tile manager has had a chance to update this node.
@@ -62,6 +62,8 @@ TileEntry::TileEntry( const TileEntry& t )
   _current_view(t._current_view),
   _time_expired(t._time_expired)
 {
+    _create_orthophoto();
+
     _node->setName(tileFileName);
     // Give a default LOD range so that traversals that traverse
     // active children (like the groundcache lookup) will work before
@@ -69,6 +71,16 @@ TileEntry::TileEntry( const TileEntry& t )
     _node->setRange(0, 0.0, 10000.0);
 }
 
+void TileEntry::_create_orthophoto() {
+    bool use_photoscenery = fgGetBool("/sim/rendering/photoscenery/enabled");
+    if (use_photoscenery) {
+        _orthophoto = simgear::Orthophoto::fromBucket(tile_bucket, globals->get_fg_scenery());
+        if (_orthophoto) {
+            simgear::OrthophotoManager::instance()->registerOrthophoto(tile_bucket.gen_index(), _orthophoto);
+        }
+    }
+}
+
 // Destructor
 TileEntry::~TileEntry ()
 {
@@ -101,8 +113,6 @@ TileEntry::addToSceneGraph(osg::Group *terrain_branch)
 void
 TileEntry::removeFromSceneGraph()
 {
-    SG_LOG( SG_TERRAIN, SG_DEBUG, "disconnecting TileEntry nodes" );
-
     if (! is_loaded()) {
         SG_LOG( SG_TERRAIN, SG_DEBUG, "removing a not-fully loaded tile!" );
     } else {
@@ -119,3 +129,65 @@ TileEntry::removeFromSceneGraph()
     }
 }
 
+// Constructor - STG Variant
+STGTileEntry::STGTileEntry ( const SGBucket& b ) : TileEntry(b)
+{
+    tileFileName = b.gen_index_str() + ".stg";
+    _node->setName(tileFileName);
+}
+
+// Destructor - STG Variant
+STGTileEntry::~STGTileEntry ()
+{
+}
+
+// Constructor - VPB version
+VPBTileEntry::VPBTileEntry ( const SGBucket& b, osg::ref_ptr<simgear::SGReaderWriterOptions> options ) : TileEntry(b)
+{
+    tileFileName = "vpb/" + b.gen_vpb_base() + ".osgb";
+    std::string zipFileName = "vpb/" + b.gen_base_path() + ".zip";
+
+    bool found = false;
+    auto filePathList = options->getDatabasePathList();
+    for (auto path : filePathList) {
+        SGPath p(path, tileFileName);
+        if (p.exists()) {
+            found = true;
+            break;
+        }
+
+        SGPath archive(path, zipFileName);
+        if (archive.exists()) {
+            found = true;
+            break;
+        }
+
+    }
+
+    if (found) {
+        // File exists - set it up for loading later
+        _node->setName(tileFileName);
+
+        // Give a default LOD range so that traversals that traverse
+        // active children (like the groundcache lookup) will work before
+        // tile manager has had a chance to update this node.
+        _node->setRange(0, 0.0, 160000.0);
+    } else {
+        // File doesn't exist, so add a 1x1 degree Ocean tile.
+        double lat = floor(b.get_center_lat()) + 0.5;
+        double lon = floor(b.get_center_lon()) + 0.5;
+        SG_LOG( SG_TERRAIN, SG_DEBUG, "Generating Ocean Tile for " << lat << ", " << lon);
+
+        // Standard for WS2.0 is 5 points per bucket (~30km), or 8km spacing.  
+        // 1 degree latitude and 1 degree of longitude at the equator is 111km, 15 points
+        // are equivalent resolution.
+        osg::Node* oceanTile = SGOceanTile(lat, lon, 1.0, 1.0, options->getMaterialLib(), 15, 15);
+        _node->addChild(oceanTile, 0, 250000.0);
+    }
+}
+
+// Destructor - VPB Variant
+VPBTileEntry::~VPBTileEntry ()
+{
+}
+
