@@ -28,8 +28,6 @@
 #include <algorithm>
 #include <functional>
 
-#include <boost/lexical_cast.hpp>
-
 #include <osgViewer/Viewer>
 #include <osgDB/Registry>
 
@@ -38,16 +36,19 @@
 #include <simgear/structure/exception.hxx>
 #include <simgear/scene/model/modellib.hxx>
 #include <simgear/scene/util/SGReaderWriterOptions.hxx>
+#include <simgear/scene/tgdb/VPBTechnique.hxx>
+#include <simgear/scene/tgdb/VPBLineFeatureRenderer.hxx>
 #include <simgear/scene/tsync/terrasync.hxx>
 #include <simgear/misc/strutils.hxx>
 #include <simgear/scene/material/matlib.hxx>
 
-#include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
+#include <Main/globals.hxx>
+#include <Main/sentryIntegration.hxx>
+#include <Scripting/NasalModelData.hxx>
+#include <Scripting/NasalSys.hxx>
 #include <Viewer/renderer.hxx>
 #include <Viewer/splash.hxx>
-#include <Scripting/NasalSys.hxx>
-#include <Scripting/NasalModelData.hxx>
 
 #include "scenery.hxx"
 #include "SceneryPager.hxx"
@@ -62,66 +63,98 @@ public:
         _manager(manager),
         _useVBOsProp(fgGetNode("/sim/rendering/use-vbos", true)),
         _enableCacheProp(fgGetNode("/sim/tile-cache/enable", true)),
-        _pagedLODMaximumProp(fgGetNode("/sim/rendering/max-paged-lod", true))
+        _pagedLODMaximumProp(fgGetNode("/sim/rendering/max-paged-lod", true)),
+        _lodDetailed(fgGetNode("/sim/rendering/static-lod/detailed", true)),
+        _lodRoughDelta(fgGetNode("/sim/rendering/static-lod/rough-delta", true)),
+        _lodBareDelta(fgGetNode("/sim/rendering/static-lod/bare-delta", true)),
+        _lodRough(fgGetNode("/sim/rendering/static-lod/rough", true)),
+        _lodBare(fgGetNode("/sim/rendering/static-lod/bare", true))
     {
         _useVBOsProp->addChangeListener(this, true);
-      
+
         _enableCacheProp->addChangeListener(this, true);
         if (_enableCacheProp->getType() == simgear::props::NONE) {
             _enableCacheProp->setBoolValue(true);
         }
-      
+
         if (_pagedLODMaximumProp->getType() == simgear::props::NONE) {
             // not set, use OSG default / environment value variable
-            osg::ref_ptr<osgViewer::Viewer> viewer(globals->get_renderer()->getViewer());
-            int current = viewer->getDatabasePager()->getTargetMaximumNumberOfPageLOD();
+            osg::ref_ptr<osgViewer::View> view(globals->get_renderer()->getView());
+            int current = view->getDatabasePager()->getTargetMaximumNumberOfPageLOD();
             _pagedLODMaximumProp->setIntValue(current);
         }
         _pagedLODMaximumProp->addChangeListener(this, true);
+        _lodDetailed->addChangeListener(this, true);
+        _lodBareDelta->addChangeListener(this, true);
+        _lodRoughDelta->addChangeListener(this, true);
     }
-    
+
     ~TileManagerListener()
     {
         _useVBOsProp->removeChangeListener(this);
         _enableCacheProp->removeChangeListener(this);
         _pagedLODMaximumProp->removeChangeListener(this);
+        _lodDetailed->removeChangeListener(this);
+        _lodBareDelta->removeChangeListener(this);
+        _lodRoughDelta->removeChangeListener(this);
     }
-    
+
     virtual void valueChanged(SGPropertyNode* prop)
     {
         if (prop == _useVBOsProp) {
             bool useVBOs = prop->getBoolValue();
             _manager->_options->setPluginStringData("SimGear::USE_VBOS",
-                                                useVBOs ? "ON" : "OFF");
+                                                    useVBOs ? "ON" : "OFF");
         } else if (prop == _enableCacheProp) {
             _manager->_enableCache = prop->getBoolValue();
         } else if (prop == _pagedLODMaximumProp) {
-          int v = prop->getIntValue();
-          osg::ref_ptr<osgViewer::Viewer> viewer(globals->get_renderer()->getViewer());
-          viewer->getDatabasePager()->setTargetMaximumNumberOfPageLOD(v);
+            int                             v = prop->getIntValue();
+            osg::ref_ptr<osgViewer::View> view(globals->get_renderer()->getView());
+            if (view) {
+              osgDB::DatabasePager* pager = view->getDatabasePager();
+              if (pager) pager->setTargetMaximumNumberOfPageLOD(v);
+            }
+        } else if (prop == _lodDetailed || prop == _lodBareDelta || prop == _lodRoughDelta) {
+            // compatibility with earlier versions; set the static lod ranges appropriately as otherwise (bad) self managed
+            // LOD on scenery with range animations doesn't work.
+            // see also /sim/rendering/enable-range-lod-animations - which is false by default in > 2019.2 which also fixes
+            // the scenery but in a more efficient way.
+            _lodRough->setDoubleValue(_lodDetailed->getDoubleValue() + _lodRoughDelta->getDoubleValue());
+            _lodBare->setDoubleValue(_lodRough->getDoubleValue() + _lodBareDelta->getDoubleValue());
         }
+
+        flightgear::addSentryBreadcrumb("Property:" + prop->getNameString() + " is now " +
+                                            prop->getStringValue(),
+                                        "info");
     }
-    
+
 private:
     FGTileMgr* _manager;
     SGPropertyNode_ptr _useVBOsProp,
       _enableCacheProp,
-      _pagedLODMaximumProp;
+      _pagedLODMaximumProp,
+      _lodDetailed,
+      _lodRoughDelta,
+      _lodBareDelta,
+      _lodRough,
+      _lodBare
+        ;
 };
 
 FGTileMgr::FGTileMgr():
     state( Start ),
     last_state( Running ),
     scheduled_visibility(100.0),
-    _terra_sync(NULL),
-    _listener(NULL),
     _visibilityMeters(fgGetNode("/environment/visibility-m", true)),
-    _maxTileRangeM(fgGetNode("/sim/rendering/static-lod/bare", true)),
+    _lodDetailed(fgGetNode("/sim/rendering/static-lod/detailed", true)),
+    _lodRoughDelta(fgGetNode("/sim/rendering/static-lod/rough-delta", true)),
+    _lodBareDelta(fgGetNode("/sim/rendering/static-lod/bare-delta", true)),
     _disableNasalHooks(fgGetNode("/sim/temp/disable-scenery-nasal", true)),
     _scenery_loaded(fgGetNode("/sim/sceneryloaded", true)),
     _scenery_override(fgGetNode("/sim/sceneryloaded-override", true)),
     _pager(FGScenery::getPagerSingleton()),
-    _enableCache(true)
+    _enableCache(true),
+    _use_vpb(false)
 {
 }
 
@@ -138,8 +171,7 @@ void FGTileMgr::init()
 
 void FGTileMgr::shutdown()
 {
-    delete _listener;
-    _listener = NULL;
+    _listener.reset();
 
     FGScenery* scenery = globals->get_scenery();
     if (scenery && scenery->get_terrain_branch()) {
@@ -154,34 +186,42 @@ void FGTileMgr::shutdown()
 void FGTileMgr::reinit()
 {
     SG_LOG( SG_TERRAIN, SG_INFO, "Initializing Tile Manager subsystem." );
-    _terra_sync = static_cast<simgear::SGTerraSync*> (globals->get_subsystem("terrasync"));
+    auto terraSync = globals->get_subsystem<simgear::SGTerraSync>();
 
-  // drops the previous options reference
+    // drops the previous options reference
     _options = new simgear::SGReaderWriterOptions;
-    _listener = new TileManagerListener(this);
-    
+    _listener.reset(new TileManagerListener(this));
+
     materialLibChanged();
     _options->setPropertyNode(globals->get_props());
-    
+
     osgDB::FilePathList &fp = _options->getDatabasePathList();
     const PathList &sc = globals->get_fg_scenery();
     fp.clear();
-    PathList::const_iterator it;
-    for (it = sc.begin(); it != sc.end(); ++it) {
-        fp.push_back(it->local8BitStr());
+    for (auto it = sc.begin(); it != sc.end(); ++it) {
+        fp.push_back(it->utf8Str());
     }
-    _options->setPluginStringData("SimGear::FG_ROOT", globals->get_fg_root().local8BitStr());
-    
-    if (_terra_sync) {
-      _options->setPluginStringData("SimGear::TERRASYNC_ROOT", globals->get_terrasync_dir().local8BitStr());
+    _options->setPluginStringData("SimGear::FG_ROOT", globals->get_fg_root().utf8Str());
+
+    if (terraSync) {
+        _options->setPluginStringData("SimGear::TERRASYNC_ROOT", globals->get_terrasync_dir().utf8Str());
     }
-    
+
     if (!_disableNasalHooks->getBoolValue())
       _options->setModelData(new FGNasalModelDataProxy);
 
-    _options->setPluginStringData("SimGear::LOD_RANGE_BARE", fgGetString("/sim/rendering/static-lod/bare", boost::lexical_cast<string>(SG_OBJECT_RANGE_BARE)));
-    _options->setPluginStringData("SimGear::LOD_RANGE_ROUGH", fgGetString("/sim/rendering/static-lod/rough", boost::lexical_cast<string>(SG_OBJECT_RANGE_ROUGH)));
-    _options->setPluginStringData("SimGear::LOD_RANGE_DETAILED", fgGetString("/sim/rendering/static-lod/detailed", boost::lexical_cast<string>(SG_OBJECT_RANGE_DETAILED)));
+    double detailed = fgGetDouble("/sim/rendering/static-lod/detailed", SG_OBJECT_RANGE_DETAILED);
+    double rough    = fgGetDouble("/sim/rendering/static-lod/rough-delta", SG_OBJECT_RANGE_ROUGH) + detailed;
+    double bare     = fgGetDouble("/sim/rendering/static-lod/bare", SG_OBJECT_RANGE_BARE) + rough;
+    double tile_min_expiry = fgGetDouble("/sim/rendering/plod-minimum-expiry-time-secs", SG_TILE_MIN_EXPIRY);
+    flightgear::addSentryBreadcrumb("PLod-minimum-expiry time=" + std::to_string(tile_min_expiry), "info");
+
+    _use_vpb = fgGetBool("/scenery/use-vpb");
+
+    _options->setPluginStringData("SimGear::LOD_RANGE_BARE", std::to_string(bare));
+    _options->setPluginStringData("SimGear::LOD_RANGE_ROUGH", std::to_string(rough));
+    _options->setPluginStringData("SimGear::LOD_RANGE_DETAILED", std::to_string(detailed));
+    _options->setPluginStringData("SimGear::PAGED_LOD_EXPIRY", std::to_string(tile_min_expiry));
 
     string_list scenerySuffixes;
     for (auto node : fgGetNode("/sim/rendering/", true)->getChildren("scenery-path-suffix")) {
@@ -195,8 +235,11 @@ void FGTileMgr::reinit()
         scenerySuffixes = {"Objects", "Terrain"}; // defaut values
     }
 
+    if (terraSync) {
+        terraSync->setSceneryPathSuffixes(scenerySuffixes);
+    }
     _options->setSceneryPathSuffixes(scenerySuffixes);
-  
+
     if (state != Start)
     {
       // protect against multiple scenery reloads and properly reset flags,
@@ -206,25 +249,25 @@ void FGTileMgr::reinit()
         return;
       }
     }
-  
+
     _scenery_loaded->setBoolValue(false);
     fgSetDouble("/sim/startup/splash-alpha", 1.0);
-    
+
     materialLibChanged();
 
     // remove all old scenery nodes from scenegraph and clear cache
     osg::Group* group = globals->get_scenery()->get_terrain_branch();
     group->removeChildren(0, group->getNumChildren());
     tile_cache.init();
-    
+
     // clear OSG cache, except on initial start-up
     if (state != Start)
     {
         osgDB::Registry::instance()->clearObjectCache();
     }
-    
+
     state = Inited;
-    
+
     previous_bucket.make_bad();
     current_bucket.make_bad();
     scheduled_visibility = 100.0;
@@ -243,13 +286,12 @@ void FGTileMgr::materialLibChanged()
 bool FGTileMgr::sched_tile( const SGBucket& b, double priority, bool current_view, double duration)
 {
     // see if tile already exists in the cache
-    TileEntry *t = tile_cache.get_tile( b );
+    STGTileEntry *t = tile_cache.get_stg_tile( b );
     if (!t)
     {
         // create a new entry
-        t = new TileEntry( b );
-        SG_LOG( SG_TERRAIN, SG_INFO, "sched_tile: new tile entry for:" << b );
-
+        t = new STGTileEntry( b );
+        SG_LOG( SG_TERRAIN, SG_INFO, "sched_tile: new STG tile entry for:" << b );
 
         // insert the tile into the cache, update will generate load request
         if ( tile_cache.insert_tile( t ) )
@@ -271,6 +313,36 @@ bool FGTileMgr::sched_tile( const SGBucket& b, double priority, bool current_vie
     // update tile's properties
     tile_cache.request_tile(t,priority,current_view,duration);
 
+    if (_use_vpb) {
+        VPBTileEntry *v = tile_cache.get_vpb_tile( b );
+
+        if (!v)
+        {
+            // create a new entry
+            v = new VPBTileEntry( b, _options );
+            SG_LOG( SG_TERRAIN, SG_INFO, "sched_tile: new VPB tile entry for:" << b );
+
+            // insert the tile into the cache, update will generate load request
+            if ( tile_cache.insert_tile( v ) )
+            {
+                // Attach to scene graph
+                v->addToSceneGraph(globals->get_scenery()->get_terrain_branch());
+            } else {
+                // insert failed (cache full with no available entries to
+                // delete.)  Try again later
+                delete v;
+                return false;
+            }
+
+            SG_LOG( SG_TERRAIN, SG_DEBUG, "  New tile cache size " << (int)tile_cache.get_size() );
+        }
+
+        // update tile's properties.  We ensure VPB tiles have maximum priority - priority is calculated as
+        // _negative_ the square of the distance from the viewer to the tile.
+        // so by multiplying by 0.1 we increase the number towards 0.
+        tile_cache.request_tile(v,priority * 0.1,current_view,duration);
+    }
+
     return t->is_loaded();
 }
 
@@ -292,11 +364,14 @@ void FGTileMgr::schedule_needed(const SGBucket& curr_bucket, double vis)
             "scheduling needed tiles for " << curr_bucket
            << ", tile-width-m:" << tile_width << ", tile-height-m:" << tile_height);
 
-    
+
     // cout << "tile width = " << tile_width << "  tile_height = "
     //      << tile_height << endl;
+    // starting with 2018.3 we will use deltas rather than absolutes as it is more intuitive for the user
+    // and somewhat easier to visualise
+    double maxTileRange = _lodDetailed->getDoubleValue() + _lodRoughDelta->getDoubleValue() + _lodBareDelta->getDoubleValue();
 
-    double tileRangeM = std::min(vis,_maxTileRangeM->getDoubleValue());
+    double tileRangeM = std::min(vis, maxTileRange);
     int xrange = (int)(tileRangeM / tile_width) + 1;
     int yrange = (int)(tileRangeM / tile_height) + 1;
     if ( xrange < 1 ) { xrange = 1; }
@@ -309,17 +384,18 @@ void FGTileMgr::schedule_needed(const SGBucket& curr_bucket, double vis)
     // cout << "max cache size = " << tile_cache.get_max_cache_size()
     //      << " current cache size = " << tile_cache.get_size() << endl;
 
-    // clear flags of all tiles belonging to the previous view set 
+    // clear flags of all tiles belonging to the previous view set
     tile_cache.clear_current_view();
 
     // update timestamps, so all tiles scheduled now are *newer* than any tile previously loaded
     osg::FrameStamp* framestamp
-            = globals->get_renderer()->getViewer()->getFrameStamp();
+            = globals->get_renderer()->getFrameStamp();
     tile_cache.set_current_time(framestamp->getReferenceTime());
 
     SGBucket b;
 
     int x, y;
+    auto terraSync = globals->get_subsystem<simgear::SGTerraSync>();
 
     /* schedule all tiles, use distance-based loading priority,
      * so tiles are loaded in innermost-to-outermost sequence. */
@@ -331,12 +407,12 @@ void FGTileMgr::schedule_needed(const SGBucket& curr_bucket, double vis)
             if (!b.isValid()) {
                 continue;
             }
-            
+
             float priority = (-1.0) * (x*x+y*y);
             sched_tile( b, priority, true, 0.0 );
-            
-            if (_terra_sync) {
-                _terra_sync->scheduleTile(b);
+
+            if (terraSync) {
+                terraSync->scheduleTile(b);
             }
         }
     }
@@ -348,14 +424,13 @@ void FGTileMgr::schedule_needed(const SGBucket& curr_bucket, double vis)
  */
 void FGTileMgr::update_queues(bool& isDownloadingScenery)
 {
-    osg::FrameStamp* framestamp
-        = globals->get_renderer()->getViewer()->getFrameStamp();
+    osg::FrameStamp* framestamp = globals->get_renderer()->getFrameStamp();
     double current_time = framestamp->getReferenceTime();
     double vis = _visibilityMeters->getDoubleValue();
     TileEntry *e;
     int loading=0;
     int sz=0;
-    
+
     tile_cache.set_current_time( current_time );
     tile_cache.reset_traversal();
 
@@ -368,7 +443,7 @@ void FGTileMgr::update_queues(bool& isDownloadingScenery)
             // Set the ssg transform and update it's range selector
             // based on current visibilty
             e->prep_ssg_node(vis);
-            
+
             if (!e->is_loaded()) {
                 bool nonExpiredOrCurrent = !e->is_expired(current_time) || e->is_current_view();
                 bool downloading = isTileDirSyncing(e->tileFileName);
@@ -399,7 +474,7 @@ void FGTileMgr::update_queues(bool& isDownloadingScenery)
       dropTiles = true;
       drop_count = sz; // no limit on tiles to drop
     }
-  
+
     if (dropTiles)
     {
         long drop_index = _enableCache ? tile_cache.get_drop_tile() :
@@ -411,14 +486,19 @@ void FGTileMgr::update_queues(bool& isDownloadingScenery)
             SG_LOG(SG_TERRAIN, SG_DEBUG, "Dropping:" << old->get_tile_bucket());
 
             tile_cache.clear_entry(drop_index);
-            
+
+            if (_use_vpb) {
+                // Clear out any VPB data - e.g. roads
+                simgear::VPBLineFeatureRenderer::unloadFeatures(old->get_tile_bucket());
+            }
+
             osg::ref_ptr<osg::Object> subgraph = old->getNode();
             old->removeFromSceneGraph();
             delete old;
             // zeros out subgraph ref_ptr, so subgraph is owned by
             // the pager and will be deleted in the pager thread.
             _pager->queueDeleteRequest(subgraph);
-          
+
             if (!_enableCache)
                 drop_index = tile_cache.get_first_expired_tile();
             // limit tiles dropped to drop_count
@@ -447,8 +527,8 @@ void FGTileMgr::update(double)
         bool fdmInited = fgGetBool("sim/fdm-initialized");
         bool positionFinalized = fgGetBool("sim/position-finalized");
         bool sceneryOverride = _scenery_override->getBoolValue();
-        
-        
+
+
     // we are done if final position is set and the scenery & FDM are done.
     // scenery-override can ignore the last two, but not position finalization.
         if (positionFinalized && (sceneryOverride || (isSceneryLoaded() && fdmInited)))
@@ -465,7 +545,7 @@ void FGTileMgr::update(double)
             } else {
                 fgSplashProgress("loading-scenery");
             }
-            
+
             // be nice to loader threads while waiting for initial scenery, reduce to 20fps
             SGTimeStamp::sleepForMSec(50);
         }
@@ -505,7 +585,7 @@ void FGTileMgr::schedule_tiles_at(const SGGeod& location, double range_m)
             scheduled_visibility = range_m;
             schedule_needed(current_bucket, range_m);
         }
-        
+
         // save bucket
         previous_bucket = current_bucket;
     } else if ( state == Start || state == Inited ) {
@@ -532,7 +612,7 @@ bool FGTileMgr::schedule_scenery(const SGGeod& position, double range_m, double
 
     SGBucket bucket(position);
     available = sched_tile( bucket, priority, false, duration );
-  
+
     if ((!available)&&(duration==0.0)) {
         SG_LOG( SG_TERRAIN, SG_DEBUG, "schedule_scenery: Scheduling tile at bucket:" << bucket << " return false" );
         return false;
@@ -546,7 +626,7 @@ bool FGTileMgr::schedule_scenery(const SGGeod& position, double range_m, double
     double tile_r = 0.5*sqrt(tile_width*tile_width + tile_height*tile_height);
     double max_dist = tile_r + range_m;
     double max_dist2 = max_dist*max_dist;
-    
+
     int xrange = (int)fabs(range_m / tile_width) + 1;
     int yrange = (int)fabs(range_m / tile_height) + 1;
 
@@ -561,7 +641,7 @@ bool FGTileMgr::schedule_scenery(const SGGeod& position, double range_m, double
                 if (!b.isValid()) {
                     continue;
                 }
-                
+
                 double distance2 = distSqr(cartPos, SGVec3d::fromGeod(b.get_center()));
                 // Do not ask if it is just the next tile but way out of range.
                 if (distance2 <= max_dist2)
@@ -589,14 +669,20 @@ bool FGTileMgr::isSceneryLoaded()
 
 bool FGTileMgr::isTileDirSyncing(const std::string& tileFileName) const
 {
-    if (!_terra_sync) {
+    auto terraSync = globals->get_subsystem<simgear::SGTerraSync>();
+    if (!terraSync) {
         return false;
     }
-    
+
+    // if Models is syncing, also wait for it, since otherwise
+    // we get load errors
+    if (terraSync->isDataDirPending("Models")) {
+        return true;
+    }
+
     std::string nameWithoutExtension = tileFileName.substr(0, tileFileName.size() - 4);
     long int bucketIndex = simgear::strutils::to_int(nameWithoutExtension);
     SGBucket bucket(bucketIndex);
-    
-    return _terra_sync->isTileDirPending(bucket.gen_base_path());
-}
 
+    return terraSync->isTileDirPending(bucket.gen_base_path());
+}
