@@ -18,14 +18,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+#include "config.h"
 
 #include "NasalCanvas.hxx"
 #include <Canvas/canvas_mgr.hxx>
 #include <Canvas/gui_mgr.hxx>
 #include <Main/globals.hxx>
+#include <Scripting/NasalCondition.hxx> // for NasalBinding
 #include <Scripting/NasalSys.hxx>
 
 #include <osgGA/GUIEventAdapter>
@@ -36,11 +35,14 @@
 #include <simgear/canvas/CanvasWindow.hxx>
 #include <simgear/canvas/elements/CanvasElement.hxx>
 #include <simgear/canvas/elements/CanvasText.hxx>
-#include <simgear/canvas/layout/BoxLayout.hxx>
-#include <simgear/canvas/layout/NasalWidget.hxx>
+#include <simgear/canvas/events/CanvasKeyBinding.hxx>
 #include <simgear/canvas/events/CustomEvent.hxx>
 #include <simgear/canvas/events/KeyboardEvent.hxx>
 #include <simgear/canvas/events/MouseEvent.hxx>
+#include <simgear/canvas/layout/BoxLayout.hxx>
+#include <simgear/canvas/layout/GridLayout.hxx>
+#include <simgear/canvas/layout/NasalWidget.hxx>
+#include <simgear/canvas/layout/SpacerItem.hxx>
 
 #include <simgear/nasal/cppbind/from_nasal.hxx>
 #include <simgear/nasal/cppbind/to_nasal.hxx>
@@ -62,6 +64,7 @@ typedef nasal::Ghost<sc::CustomEventPtr> NasalCustomEvent;
 typedef nasal::Ghost<sc::DeviceEventPtr> NasalDeviceEvent;
 typedef nasal::Ghost<sc::KeyboardEventPtr> NasalKeyboardEvent;
 typedef nasal::Ghost<sc::MouseEventPtr> NasalMouseEvent;
+typedef nasal::Ghost<sc::KeyBindingRef> NasalKeyBinding;
 
 struct CustomEventDetailWrapper;
 typedef SGSharedPtr<CustomEventDetailWrapper> CustomEventDetailPtr;
@@ -72,10 +75,13 @@ typedef nasal::Ghost<sc::CanvasPtr> NasalCanvas;
 typedef nasal::Ghost<sc::ElementPtr> NasalElement;
 typedef nasal::Ghost<sc::GroupPtr> NasalGroup;
 typedef nasal::Ghost<sc::TextPtr> NasalText;
+typedef nasal::Ghost<sc::ImagePtr> NasalImage;
 
 typedef nasal::Ghost<sc::LayoutItemRef> NasalLayoutItem;
 typedef nasal::Ghost<sc::LayoutRef> NasalLayout;
 typedef nasal::Ghost<sc::BoxLayoutRef> NasalBoxLayout;
+typedef nasal::Ghost<sc::GridLayoutRef> NasalGridLayout;
+using NasalSpacerItem = nasal::Ghost<sc::SpacerItemRef>;
 
 typedef nasal::Ghost<sc::WindowPtr> NasalWindow;
 
@@ -99,22 +105,20 @@ SGPropertyNode* from_nasal_helper(naContext c, naRef ref, SGPropertyNode**)
   return props;
 }
 
-CanvasMgr& requireCanvasMgr(naContext c)
+CanvasMgr& requireCanvasMgr(const nasal::ContextWrapper& ctx)
 {
-  CanvasMgr* canvas_mgr =
-    static_cast<CanvasMgr*>(globals->get_subsystem("Canvas"));
+  auto canvas_mgr = globals->get_subsystem<CanvasMgr>();
   if( !canvas_mgr )
-    naRuntimeError(c, "Failed to get Canvas subsystem");
+    ctx.runtimeError("Failed to get Canvas subsystem");
 
   return *canvas_mgr;
 }
 
-GUIMgr& requireGUIMgr(naContext c)
+GUIMgr& requireGUIMgr(const nasal::ContextWrapper& ctx)
 {
-  GUIMgr* mgr =
-    static_cast<GUIMgr*>(globals->get_subsystem("CanvasGUI"));
+  auto mgr = globals->get_subsystem<GUIMgr>();
   if( !mgr )
-    naRuntimeError(c, "Failed to get CanvasGUI subsystem");
+    ctx.runtimeError("Failed to get CanvasGUI subsystem");
 
   return *mgr;
 }
@@ -124,7 +128,7 @@ GUIMgr& requireGUIMgr(naContext c)
  */
 static naRef f_createCanvas(const nasal::CallContext& ctx)
 {
-  return ctx.to_nasal(requireCanvasMgr(ctx.c).createCanvas());
+  return ctx.to_nasal(requireCanvasMgr(ctx).createCanvas());
 }
 
 /**
@@ -132,10 +136,9 @@ static naRef f_createCanvas(const nasal::CallContext& ctx)
  */
 static naRef f_createWindow(const nasal::CallContext& ctx)
 {
-  return nasal::to_nasal<sc::WindowWeakPtr>
+  return ctx.to_nasal<sc::WindowWeakPtr>
   (
-    ctx.c,
-    requireGUIMgr(ctx.c).createWindow( ctx.getArg<std::string>(0) )
+    requireGUIMgr(ctx).createWindow( ctx.getArg<std::string>(0) )
   );
 }
 
@@ -145,7 +148,7 @@ static naRef f_createWindow(const nasal::CallContext& ctx)
 static naRef f_getCanvas(const nasal::CallContext& ctx)
 {
   SGPropertyNode& props = *ctx.requireArg<SGPropertyNode*>(0);
-  CanvasMgr& canvas_mgr = requireCanvasMgr(ctx.c);
+  CanvasMgr& canvas_mgr = requireCanvasMgr(ctx);
 
   sc::CanvasPtr canvas;
   if( canvas_mgr.getPropertyRoot() == props.getParent() )
@@ -175,27 +178,27 @@ naRef f_canvasCreateGroup(sc::Canvas& canvas, const nasal::CallContext& ctx)
 /**
  * Get group containing all gui windows
  */
-naRef f_getDesktop(naContext c, naRef me, int argc, naRef* args)
+naRef f_getDesktop(const nasal::CallContext& ctx)
 {
-  return nasal::to_nasal(c, requireGUIMgr(c).getDesktop());
+  return ctx.to_nasal(requireGUIMgr(ctx).getDesktop());
 }
 
 naRef f_setInputFocus(const nasal::CallContext& ctx)
 {
-  requireGUIMgr(ctx.c).setInputFocus(ctx.requireArg<sc::WindowPtr>(0));
+  requireGUIMgr(ctx).setInputFocus(ctx.requireArg<sc::WindowPtr>(0));
   return naNil();
 }
 
 naRef f_grabPointer(const nasal::CallContext& ctx)
 {
   return ctx.to_nasal(
-    requireGUIMgr(ctx.c).grabPointer(ctx.requireArg<sc::WindowPtr>(0))
+    requireGUIMgr(ctx).grabPointer(ctx.requireArg<sc::WindowPtr>(0))
   );
 }
 
 naRef f_ungrabPointer(const nasal::CallContext& ctx)
 {
-  requireGUIMgr(ctx.c).ungrabPointer(ctx.requireArg<sc::WindowPtr>(0));
+  requireGUIMgr(ctx).ungrabPointer(ctx.requireArg<sc::WindowPtr>(0));
   return naNil();
 }
 
@@ -210,15 +213,29 @@ static sc::ElementPtr f_groupGetChild(sc::Group& group, SGPropertyNode* node)
   return group.getChild(node);
 }
 
+static naRef f_groupAddKeyBinding(sc::Group& group, const nasal::CallContext& ctx)
+{
+  auto keyBinding = ctx.requireArg<sc::KeyBindingRef>(0);
+  group.getOrCreateFocusScope()->addKeyBinding(keyBinding);
+  return naNil();
+}
+
+static naRef f_windowAddKeyBinding(sc::Window& window, const nasal::CallContext& ctx)
+{
+  auto keyBinding = ctx.requireArg<sc::KeyBindingRef>(0);
+  window.focusScope()->addKeyBinding(keyBinding);
+  return naNil();
+}
+
 static void propElementSetData( simgear::PropertyBasedElement& el,
                                 const std::string& name,
-                                naContext c,
+                                const nasal::ContextWrapper& ctx,
                                 naRef ref )
 {
   if( naIsNil(ref) )
     return el.removeDataProp(name);
 
-  std::string val = nasal::from_nasal<std::string>(c, ref);
+  std::string val = ctx.from_nasal<std::string>(ref);
 
   char* end = NULL;
 
@@ -266,7 +283,7 @@ static naRef f_propElementData( simgear::PropertyBasedElement& el,
     // Add/delete properties given as hash
     nasal::Hash obj = ctx.requireArg<nasal::Hash>(0);
     for(nasal::Hash::iterator it = obj.begin(); it != obj.end(); ++it)
-      propElementSetData(el, it->getKey(), ctx.c, it->getValue<naRef>());
+      propElementSetData(el, it->getKey(), ctx, it->getValue<naRef>());
 
     return ctx.to_nasal(&el);
   }
@@ -286,7 +303,7 @@ static naRef f_propElementData( simgear::PropertyBasedElement& el,
     else
     {
       // only name -> get property
-      propElementSetData(el, name, ctx.c, ctx.requireArg<naRef>(1));
+      propElementSetData(el, name, ctx, ctx.requireArg<naRef>(1));
       return ctx.to_nasal(&el);
     }
   }
@@ -361,21 +378,38 @@ static naRef f_customEventGetDetail( sc::CustomEvent& event,
   );
 }
 
+static naRef f_layoutItemSetVisible(sc::LayoutItem& item,
+                                    const nasal::CallContext& ctx)
+{
+  item.setVisible(ctx.getArg<bool>(0, true));
+  return ctx.me;
+}
+
 static naRef f_boxLayoutAddItem( sc::BoxLayout& box,
                                  const nasal::CallContext& ctx )
 {
-  box.addItem( ctx.requireArg<sc::LayoutItemRef>(0),
-               ctx.getArg<int>(1),
-               ctx.getArg<int>(2, sc::AlignFill) );
+  const auto item = ctx.requireArg<sc::LayoutItemRef>(0);
+  if (!item) {
+    ctx.runtimeError("BoxLayout.addItem: argument 0 is not a layout item");
+  }
+
+  box.addItem(item,
+              ctx.getArg<int>(1),
+              ctx.getArg<int>(2, sc::AlignFill));
   return naNil();
 }
 static naRef f_boxLayoutInsertItem( sc::BoxLayout& box,
                                     const nasal::CallContext& ctx )
 {
-  box.insertItem( ctx.requireArg<int>(0),
-                  ctx.requireArg<sc::LayoutItemRef>(1),
-                  ctx.getArg<int>(2),
-                  ctx.getArg<int>(3, sc::AlignFill) );
+  const auto item = ctx.requireArg<sc::LayoutItemRef>(1);
+  if (!item) {
+    ctx.runtimeError("BoxLayout.insertItem: argument 1 is not a layout item");
+  }
+
+  box.insertItem(ctx.requireArg<int>(0),
+                 item,
+                 ctx.getArg<int>(2),
+                 ctx.getArg<int>(3, sc::AlignFill));
   return naNil();
 }
 
@@ -399,6 +433,80 @@ static naRef f_newAsBase(const nasal::CallContext& ctx)
   return ctx.to_nasal<Base*>(new Type());
 }
 
+static naRef f_imageFillRect(sc::Image& img, const nasal::CallContext& ctx)
+{
+    const SGRecti r = ctx.requireArg<SGRecti>(0);
+    if (ctx.isString(1)) {
+        img.fillRect(r, ctx.getArg<std::string>(1));
+    } else {
+        img.fillRect(r, ctx.requireArg<osg::Vec4>(1));
+    }
+    return naNil();
+}
+
+static naRef f_imageSetPixel(sc::Image& img, const nasal::CallContext& ctx)
+{
+    const int s = ctx.requireArg<int>(0);
+    const int t = ctx.requireArg<int>(1);
+    if (ctx.isString(2)) {
+        img.setPixel(s, t, ctx.getArg<std::string>(2));
+    } else {
+        img.setPixel(s, t, ctx.requireArg<osg::Vec4>(2));
+    }
+    return naNil();
+}
+
+static naRef f_canvasImageSize(sc::Image& img, const nasal::CallContext& ctx)
+{
+    auto osgImage = img.getImage();
+    osg::Vec2f sz{0.0f, 0.0f};
+    if (osgImage) {
+        sz = osg::Vec2f{static_cast<float>(osgImage->s()),
+                        static_cast<float>(osgImage->t())};
+    }
+
+    return ctx.to_nasal(sz);
+}
+
+static naRef f_gridLayoutAddItem(sc::GridLayout& grid,
+                                 const nasal::CallContext& ctx)
+{
+    const auto item = ctx.requireArg<sc::LayoutItemRef>(0);
+    if (!item) {
+        ctx.runtimeError("GridLayout.addItem: argument 0 is not a layout item");
+    }
+
+
+    grid.addItem(item,
+                 ctx.requireArg<int>(1),
+                 ctx.requireArg<int>(2),
+                 ctx.getArg<int>(3, 1),
+                 ctx.getArg<int>(4, 1));
+    return naNil();
+}
+
+static naRef f_newGridLayout(const nasal::CallContext& ctx)
+{
+    return ctx.to_nasal(new sc::GridLayout);
+}
+
+static naRef f_newSpacerItem(const nasal::CallContext& ctx)
+{
+    return ctx.to_nasal(new sc::SpacerItem);
+}
+
+static naRef f_keyBindingAddBinding(sc::KeyBinding& keyBinding, const nasal::CallContext& ctx)
+{
+    auto cb = ctx.requireArg<NasalBinding::NasalCallback>(0);
+    keyBinding.addBinding(new NasalBinding{cb});
+    return naNil();
+}
+
+static naRef f_newKeyBinding(const nasal::CallContext& ctx)
+{
+    return ctx.to_nasal(new sc::KeyBinding);
+}
+
 naRef initNasalCanvas(naRef globals, naContext c)
 {
   nasal::Hash globals_module(globals, c),
@@ -458,6 +566,15 @@ naRef initNasalCanvas(naRef globals, naContext c)
     .member("buttons", &sc::MouseEvent::getButtonMask)
     .member("click_count", &sc::MouseEvent::getCurrentClickCount);
 
+
+  NasalKeyBinding::init("canvas.KeyBinding")
+      .member("key", &sc::KeyBinding::key, &sc::KeyBinding::setKey)
+      .member("keyCode", &sc::KeyBinding::keyCode, &sc::KeyBinding::setKeyCode)
+      .member("modifiers", &sc::KeyBinding::modifiers, &sc::KeyBinding::setModifiers)
+      .method("addBinding", &f_keyBindingAddBinding);
+  canvas_module.createHash("KeyBinding")
+      .set("new", &f_newKeyBinding);
+
   //----------------------------------------------------------------------------
   // Canvas & elements
 
@@ -484,21 +601,26 @@ naRef initNasalCanvas(naRef globals, naContext c)
   canvas_module.set("_getCanvasGhost", f_getCanvas);
 
   NasalElement::init("canvas.Element")
-    .bases<NasalPropertyBasedElement>()
-    .member("_node_ghost", &elementGetNode<sc::Element>)
-    .method("_getParent", &sc::Element::getParent)
-    .method("_getCanvas", &sc::Element::getCanvas)
-    .method("addEventListener", &sc::Element::addEventListener)
-    .method("setFocus", &sc::Element::setFocus)
-    .method("dispatchEvent", &sc::Element::dispatchEvent)
-    .method("getBoundingBox", &sc::Element::getBoundingBox)
-    .method("getTightBoundingBox", &sc::Element::getTightBoundingBox);
+      .bases<NasalPropertyBasedElement>()
+      .member("_node_ghost", &elementGetNode<sc::Element>)
+      .method("_getParent", &sc::Element::getParent)
+      .method("_getCanvas", &sc::Element::getCanvas)
+      .method("addEventListener", &sc::Element::addEventListener)
+      .method("setFocus", &sc::Element::setFocus)
+      .method("dispatchEvent", &sc::Element::dispatchEvent)
+      .method("getBoundingBox", &sc::Element::getBoundingBox)
+      .method("getTightBoundingBox", &sc::Element::getTightBoundingBox)
+      .method("_posToLocal", &sc::Element::posToLocal)
+      .method("_posFromLocal", &sc::Element::posFromLocal)
+      .method("canvasToLocal", &sc::Element::canvasToLocal)
+      .method("localToCanvas", &sc::Element::localToCanvas);
 
   NasalGroup::init("canvas.Group")
-    .bases<NasalElement>()
-    .method("_createChild", &f_groupCreateChild)
-    .method( "_getChild", &f_groupGetChild)
-    .method("_getElementById", &sc::Group::getElementById);
+      .bases<NasalElement>()
+      .method("_createChild", &f_groupCreateChild)
+      .method("_getChild", &f_groupGetChild)
+      .method("_getElementById", &sc::Group::getElementById)
+      .method("addKeyBinding", &f_groupAddKeyBinding);
   NasalText::init("canvas.Text")
     .bases<NasalElement>()
     .method("heightForWidth", &sc::Text::heightForWidth)
@@ -508,6 +630,13 @@ naRef initNasalCanvas(naRef globals, naContext c)
     .method("getNearestCursor", &sc::Text::getNearestCursor)
     .method("getCursorPos", &sc::Text::getCursorPos);
 
+  NasalImage::init("canvas.Image")
+      .bases<NasalElement>()
+      .method("fillRect", &f_imageFillRect)
+      .method("setPixel", &f_imageSetPixel)
+      .method("dirtyPixels", &sc::Image::dirtyPixels)
+      .method("imageSize", &f_canvasImageSize);
+
   //----------------------------------------------------------------------------
   // Layouting
 
@@ -519,27 +648,30 @@ naRef initNasalCanvas(naRef globals, naContext c)
     = &sc::LayoutItem::setContentsMargins;
 
   NasalLayoutItem::init("canvas.LayoutItem")
-    .method("getCanvas", &sc::LayoutItem::getCanvas)
-    .method("setCanvas", &sc::LayoutItem::setCanvas)
-    .method("getParent", &sc::LayoutItem::getParent)
-    .method("setParent", &sc::LayoutItem::setParent)
-    .method("setContentsMargins", f_layoutItemSetContentsMargins)
-    .method("setContentsMargin", &sc::LayoutItem::setContentsMargin)
-    .method("sizeHint", &sc::LayoutItem::sizeHint)
-    .method("minimumSize", &sc::LayoutItem::minimumSize)
-    .method("maximumSize", &sc::LayoutItem::maximumSize)
-    .method("hasHeightForWidth", &sc::LayoutItem::hasHeightForWidth)
-    .method("heightForWidth", &sc::LayoutItem::heightForWidth)
-    .method("minimumHeightForWidth", &sc::LayoutItem::minimumHeightForWidth)
-    .method("setAlignment", &sc::LayoutItem::setAlignment)
-    .method("alignment", &sc::LayoutItem::alignment)
-    .method("setVisible", &sc::LayoutItem::setVisible)
-    .method("isVisible", &sc::LayoutItem::isVisible)
-    .method("isExplicitlyHidden", &sc::LayoutItem::isExplicitlyHidden)
-    .method("show", &sc::LayoutItem::show)
-    .method("hide", &sc::LayoutItem::hide)
-    .method("setGeometry", &sc::LayoutItem::setGeometry)
-    .method("geometry", &sc::LayoutItem::geometry);
+      .method("getCanvas", &sc::LayoutItem::getCanvas)
+      .method("setCanvas", &sc::LayoutItem::setCanvas)
+      .method("getParent", &sc::LayoutItem::getParent)
+      .method("setParent", &sc::LayoutItem::setParent)
+      .method("setContentsMargins", f_layoutItemSetContentsMargins)
+      .method("setContentsMargin", &sc::LayoutItem::setContentsMargin)
+      .method("sizeHint", &sc::LayoutItem::sizeHint)
+      .method("minimumSize", &sc::LayoutItem::minimumSize)
+      .method("maximumSize", &sc::LayoutItem::maximumSize)
+      .method("hasHeightForWidth", &sc::LayoutItem::hasHeightForWidth)
+      .method("heightForWidth", &sc::LayoutItem::heightForWidth)
+      .method("minimumHeightForWidth", &sc::LayoutItem::minimumHeightForWidth)
+      .method("setAlignment", &sc::LayoutItem::setAlignment)
+      .method("alignment", &sc::LayoutItem::alignment)
+      .method("setVisible", f_layoutItemSetVisible)
+      .method("isVisible", &sc::LayoutItem::isVisible)
+      .method("isExplicitlyHidden", &sc::LayoutItem::isExplicitlyHidden)
+      .method("show", &sc::LayoutItem::show)
+      .method("hide", &sc::LayoutItem::hide)
+      .method("setGeometry", &sc::LayoutItem::setGeometry)
+      .method("geometry", &sc::LayoutItem::geometry)
+      .method("setGridLocation", &sc::LayoutItem::setGridLocation)
+      .method("setGridSpan", &sc::LayoutItem::setGridSpan);
+
   sc::NasalWidget::setupGhost(canvas_module);
 
   NasalLayout::init("canvas.Layout")
@@ -565,26 +697,41 @@ naRef initNasalCanvas(naRef globals, naContext c)
     .method("setStretchFactor", &sc::BoxLayout::setStretchFactor)
     .method("stretch", &sc::BoxLayout::stretch);
 
-  canvas_module.createHash("HBoxLayout")
-               .set("new", &f_newAsBase<sc::HBoxLayout, sc::BoxLayout>);
-  canvas_module.createHash("VBoxLayout")
-               .set("new", &f_newAsBase<sc::VBoxLayout, sc::BoxLayout>);
+  NasalGridLayout::init("canvas.GridLayout")
+      .bases<NasalLayout>()
+      .method("addItem", &f_gridLayoutAddItem)
+      .method("setRowStretch", &sc::GridLayout::setRowStretch)
+      .method("setColumnStretch", &sc::GridLayout::setColumnStretch);
+
+    NasalSpacerItem::init("canvas.SpacerItem")
+        .bases<NasalLayoutItem>();
+    
+    canvas_module.createHash("HBoxLayout")
+                 .set("new", &f_newAsBase<sc::HBoxLayout, sc::BoxLayout>);
+    canvas_module.createHash("VBoxLayout")
+                 .set("new", &f_newAsBase<sc::VBoxLayout, sc::BoxLayout>);
+    canvas_module.createHash("GridLayout")
+                 .set("new", &f_newGridLayout);
+    canvas_module.createHash("Spacer")
+                 .set("new", &f_newSpacerItem);
 
   //----------------------------------------------------------------------------
   // Window
 
-  NasalWindow::init("canvas.Window")
-    .bases<NasalElement>()
-    .bases<NasalLayoutItem>()
-    .member("_node_ghost", &elementGetNode<sc::Window>)
-    .method("_getCanvasDecoration", &sc::Window::getCanvasDecoration)
-    .method("setLayout", &sc::Window::setLayout);
+    NasalWindow::init("canvas.Window")
+        .bases<NasalElement>()
+        .bases<NasalLayoutItem>()
+        .member("_node_ghost", &elementGetNode<sc::Window>)
+        .method("_getCanvasDecoration", &sc::Window::getCanvasDecoration)
+        .method("setLayout", &sc::Window::setLayout)
+        .method("toScreenPosition", &sc::Window::toScreenPosition)
+        .method("addKeyBinding", &f_windowAddKeyBinding);
 
-  canvas_module.set("_newWindowGhost", f_createWindow);
-  canvas_module.set("_getDesktopGhost", f_getDesktop);
-  canvas_module.set("setInputFocus", f_setInputFocus);
-  canvas_module.set("grabPointer", f_grabPointer);
-  canvas_module.set("ungrabPointer", f_ungrabPointer);
+    canvas_module.set("_newWindowGhost", f_createWindow);
+    canvas_module.set("_getDesktopGhost", f_getDesktop);
+    canvas_module.set("setInputFocus", f_setInputFocus);
+    canvas_module.set("grabPointer", f_grabPointer);
+    canvas_module.set("ungrabPointer", f_ungrabPointer);
 
-  return naNil();
+    return naNil();
 }
