@@ -13,8 +13,6 @@
 #include <simgear/scene/util/OsgMath.hxx>
 #include <simgear/debug/logstream.hxx>
 
-#include <boost/bind.hpp>
-
 #include <algorithm>
 #include <cstring> // for strlen
 
@@ -23,9 +21,6 @@
 // destructor the <unload> script. The latter happens when the model branch
 // is removed from the scene graph.
 
-unsigned int FGNasalModelData::_max_module_id = 0;
-FGNasalModelDataList FGNasalModelData::_loaded_models;
-
 typedef osg::ref_ptr<osg::Node> NodeRef;
 typedef nasal::Ghost<NodeRef> NasalNode;
 
@@ -58,7 +53,7 @@ static naRef f_node_getPose( const osg::Node& node,
     double hdg, pitch, roll;
     rotate.getEulerDeg(hdg, pitch, roll);
 
-    nasal::Hash pose(ctx.to_nasal(coord), ctx.c);
+    nasal::Hash pose(ctx.to_nasal(coord), ctx.c_ctx());
     pose.set("heading", hdg);
     pose.set("pitch", pitch);
     pose.set("roll", roll);
@@ -78,9 +73,10 @@ FGNasalModelData::FGNasalModelData( SGPropertyNode *root,
   _path(path),
   _root(root), _prop(prop),
   _load(load), _unload(unload),
-  _branch(branch),
-  _module_id( _max_module_id++ )
+  _branch(branch)
 {
+  const std::lock_guard<std::mutex> lock(FGNasalModelData::_loaded_models_mutex);
+  _module_id = _max_module_id++;
   _loaded_models.push_back(this);
 
   SG_LOG
@@ -96,6 +92,7 @@ FGNasalModelData::FGNasalModelData( SGPropertyNode *root,
 //------------------------------------------------------------------------------
 FGNasalModelData::~FGNasalModelData()
 {
+  const std::lock_guard<std::mutex> lock(FGNasalModelData::_loaded_models_mutex);
   _loaded_models.remove(this);
 
   SG_LOG
@@ -115,8 +112,8 @@ void FGNasalModelData::load()
 
   SG_LOG(SG_NASAL, SG_DEBUG, "Loading nasal module " << _module.c_str());
 
-  const char *s = _load ? _load->getStringValue() : "";
-  FGNasalSys* nasalSys = (FGNasalSys*) globals->get_subsystem("nasal");
+  const std::string s = _load ? _load->getStringValue() : "";
+  auto nasalSys = globals->get_subsystem<FGNasalSys>();
 
   // Add _module_id to script local hash to allow placing canvasses on objects
   // inside the model.
@@ -130,7 +127,7 @@ void FGNasalModelData::load()
   naRef arg[2];
   arg[0] = nasalSys->propNodeGhost(_root);
   arg[1] = nasalSys->propNodeGhost(_prop);
-  nasalSys->createModule(_module.c_str(), _path.c_str(), s, strlen(s),
+  nasalSys->createModule(_module.c_str(), _path.c_str(), s.c_str(), s.length(),
                          _root, 2, arg);
 }
 
@@ -140,7 +137,7 @@ void FGNasalModelData::unload()
     if (_module.empty())
         return;
 
-    FGNasalSys* nasalSys = (FGNasalSys*) globals->get_subsystem("nasal");
+    auto nasalSys = globals->get_subsystem<FGNasalSys>();
     if(!nasalSys) {
         SG_LOG(SG_NASAL, SG_WARN, "Trying to run an <unload> script "
                "without Nasal subsystem present.");
@@ -151,8 +148,8 @@ void FGNasalModelData::unload()
 
     if (_unload)
     {
-        const char *s = _unload->getStringValue();
-        nasalSys->createModule(_module.c_str(), _module.c_str(), s, strlen(s), _root);
+        const std::string s = _unload->getStringValue();
+        nasalSys->createModule(_module.c_str(), _module.c_str(), s.c_str(), s.length(), _root);
     }
 
     nasalSys->deleteModule(_module.c_str());
@@ -167,12 +164,13 @@ osg::Node* FGNasalModelData::getNode()
 //------------------------------------------------------------------------------
 FGNasalModelData* FGNasalModelData::getByModuleId(unsigned int id)
 {
+  const std::lock_guard<std::mutex> lock(FGNasalModelData::_loaded_models_mutex);
   FGNasalModelDataList::iterator it = std::find_if
   (
     _loaded_models.begin(),
     _loaded_models.end(),
-    boost::bind(&FGNasalModelData::_module_id, _1) == id
-  );
+    [id] (const FGNasalModelData* const data) {
+        return data->_module_id == id; });
 
   if( it != _loaded_models.end() )
     return *it;
@@ -183,11 +181,14 @@ FGNasalModelData* FGNasalModelData::getByModuleId(unsigned int id)
 //------------------------------------------------------------------------------
 FGNasalModelDataProxy::~FGNasalModelDataProxy()
 {
-    FGNasalSys* nasalSys = (FGNasalSys*) globals->get_subsystem("nasal");
-    // when necessary, register Nasal module to be destroyed/unloaded
-    // in the main thread.
-    if ((_data.valid())&&(nasalSys))
-        nasalSys->registerToUnload(_data);
+    if (globals) {
+      auto nasalSys = globals->get_subsystem<FGNasalSys>();
+
+      // when necessary, register Nasal module to be destroyed/unloaded
+      // in the main thread.
+      if ((_data.valid())&&(nasalSys))
+          nasalSys->registerToUnload(_data);
+    }
 }
 
 //------------------------------------------------------------------------------
@@ -205,7 +206,7 @@ void FGNasalModelDataProxy::modelLoaded( const std::string& path,
     if(!nasal)
         return;
     
-    FGNasalSys* nasalSys = (FGNasalSys*) globals->get_subsystem("nasal");
+    auto nasalSys = globals->get_subsystem<FGNasalSys>();
     if(!nasalSys)
     {
         SG_LOG
