@@ -1,6 +1,6 @@
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+// SPDX-License-Identifier: GPL-2.0-or-later
+
+#include "config.h"
 
 #include <cstring>
 
@@ -11,6 +11,7 @@
 #include <Main/globals.hxx>
 
 #include "NasalSys.hxx"
+#include "NasalSys_private.hxx"
 
 using namespace std;
 
@@ -30,7 +31,7 @@ static void propNodeGhostDestroy(void* ghost)
   if (!SGPropertyNode::put(prop)) delete prop;
 }
 
-naGhostType PropNodeGhostType = { propNodeGhostDestroy, "prop" };
+naGhostType PropNodeGhostType = { propNodeGhostDestroy, "prop", nullptr, nullptr };
 
 naRef propNodeGhostCreate(naContext c, SGPropertyNode* ghost)
 {
@@ -41,7 +42,7 @@ naRef propNodeGhostCreate(naContext c, SGPropertyNode* ghost)
 
 naRef FGNasalSys::propNodeGhost(SGPropertyNode* handle)
 {
-    return propNodeGhostCreate(_context, handle);
+    return propNodeGhostCreate(d->_context, handle);
 }
 
 SGPropertyNode* ghostToPropNode(naRef ref)
@@ -125,6 +126,41 @@ static naRef f_getType(naContext c, naRef me, int argc, naRef* args)
     return NASTR(t);
 }
 
+// Check if type of a property is numeric (returns 0 or 1).
+// Forms:
+//    props.Node.isNumeric(string relative_path);
+//    props.Node.isNumeric();
+static naRef f_isNumeric(naContext c, naRef me, int argc, naRef* args)
+{
+    using namespace simgear;
+    NODEARG();
+    MOVETARGET(naVec_size(argv) > 0, false);
+    switch(node->getType()) {
+    case props::INT:
+    case props::LONG:
+    case props::FLOAT:
+    case props::DOUBLE: return naNum(true);
+    default:
+        break;
+    }
+    return naNum(false);
+}
+
+// Check if type of a property is integer (returns 0 or 1).
+// Forms:
+//    props.Node.isInt(string relative_path);
+//    props.Node.isInt();
+static naRef f_isInt(naContext c, naRef me, int argc, naRef* args)
+{
+    using namespace simgear;
+    NODEARG();
+    MOVETARGET(naVec_size(argv) > 0, false);
+    if ((node->getType() == props::INT) || (node->getType() == props::LONG)) {
+        return naNum(true);
+    }
+    return naNum(false);
+}
+
 
 // Get an attribute of a property by name (returns true/false).
 // Forms:
@@ -154,6 +190,11 @@ static naRef f_getAttribute(naContext c, naRef me, int argc, naRef* args)
     else if(!strcmp(a, "trace-write")) attr = SGPropertyNode::TRACE_WRITE;
     else if(!strcmp(a, "userarchive")) attr = SGPropertyNode::USERARCHIVE;
     else if(!strcmp(a, "preserve"))    attr = SGPropertyNode::PRESERVE;
+    else if(!strcmp(a, "protected"))   attr = SGPropertyNode::PROTECTED;
+    else if(!strcmp(a, "listener-safe"))        attr = SGPropertyNode::LISTENER_SAFE;
+    else if(!strcmp(a, "value-changed-up"))     attr = SGPropertyNode::VALUE_CHANGED_UP;
+    else if(!strcmp(a, "value-changed-down"))   attr = SGPropertyNode::VALUE_CHANGED_DOWN;
+
     else {
         naRuntimeError(c, "props.getAttribute() with invalid attribute");
         return naNil();
@@ -173,11 +214,19 @@ static naRef f_getAttribute(naContext c, naRef me, int argc, naRef* args)
 static naRef f_setAttribute(naContext c, naRef me, int argc, naRef* args)
 {
     NODEARG();
+    if (node->getAttribute(SGPropertyNode::PROTECTED)) {
+        naRuntimeError(c, "props.setAttribute() called on protected property %s",
+                       node->getPath().c_str());
+        return naNil();
+    }
+
     MOVETARGET(naVec_size(argv) > 2, false);
     naRef val = naVec_get(argv, 0);
     if(naVec_size(argv) == 1 && naIsNum(val)) {
         naRef ret = naNum(node->getAttributes());
-        node->setAttributes((int)val.num);
+        // prevent Nasal modifying PROTECTED
+        int attrs = static_cast<int>(val.num) & (~SGPropertyNode::PROTECTED);
+        node->setAttributes(attrs);
         return ret;
     }
     SGPropertyNode::Attribute attr;
@@ -190,6 +239,7 @@ static naRef f_setAttribute(naContext c, naRef me, int argc, naRef* args)
     else if(!strcmp(a, "trace-write")) attr = SGPropertyNode::TRACE_WRITE;
     else if(!strcmp(a, "userarchive")) attr = SGPropertyNode::USERARCHIVE;
     else if(!strcmp(a, "preserve"))    attr = SGPropertyNode::PRESERVE;
+    // explicitly don't allow "protected" to be modified here
     else {
         naRuntimeError(c, "props.setAttribute() with invalid attribute");
         return naNil();
@@ -206,7 +256,7 @@ static naRef f_setAttribute(naContext c, naRef me, int argc, naRef* args)
 static naRef f_getName(naContext c, naRef me, int argc, naRef* args)
 {
     NODENOARG();
-    return NASTR(node->getName());
+    return NASTR(node->getNameString().c_str());
 }
 
 
@@ -256,30 +306,7 @@ static naRef f_getValue(naContext c, naRef me, int argc, naRef* args)
     using namespace simgear;
     NODEARG();
     MOVETARGET(naVec_size(argv) > 0, false);
-    switch(node->getType()) {
-    case props::BOOL:   case props::INT:
-    case props::LONG:   case props::FLOAT:
-    case props::DOUBLE:
-    {
-        double dv = node->getDoubleValue();
-        if (SGMisc<double>::isNaN(dv)) {
-          SG_LOG(SG_NASAL, SG_ALERT, "Nasal getValue: property " << node->getPath() << " is NaN");
-          return naNil();
-        }
-
-        return naNum(dv);
-    }
-
-    case props::STRING:
-    case props::UNSPECIFIED:
-        return NASTR(node->getStringValue());
-    case props::VEC3D:
-        return makeVectorFromVec(c, node->getValue<SGVec3d>());
-    case props::VEC4D:
-        return makeVectorFromVec(c, node->getValue<SGVec4d>());
-    default:
-        return naNil();
-    }
+    return FGNasalSys::getPropertyValue(c, node);
 }
 
 template<typename T>
@@ -299,6 +326,50 @@ T makeVecFromVector(naRef vector)
     return vec;
 }
 
+static std::string s_val_description(naRef val)
+{
+    std::ostringstream message;
+    if (naIsNil(val))           message << "nil";
+    else if (naIsNum(val))      message << "num:" << naNumValue(val).num;
+    else if (naIsString(val))   message << "string:" << naStr_data(val);
+    else if (naIsScalar(val))   message << "scalar";
+    else if (naIsVector(val))   message << "vector";
+    else if (naIsHash(val))     message << "hash";
+    else if (naIsFunc(val))     message << "func";
+    else if (naIsCode(val))     message << "code";
+    else if (naIsCCode(val))    message << "ccode";
+    else if (naIsGhost(val))    message << "ghost";
+    else message << "?";
+    return message.str();
+}
+
+// Helper function to set the value of a node; returns true if it succeeded or
+// false if it failed.  <val> can be a string, number, or a
+// vector or numbers (for SGVec3D/4D types).
+static naRef f_setValueHelper(naContext c, SGPropertyNode_ptr node, naRef val) {
+    bool result = false;
+    if(naIsString(val)) {
+        result = node->setStringValue(naStr_data(val));
+    } else if(naIsVector(val)) {
+        if(naVec_size(val) == 3)
+            result = node->setValue(makeVecFromVector<SGVec3d>(val));
+        else if(naVec_size(val) == 4)
+            result = node->setValue(makeVecFromVector<SGVec4d>(val));
+        else
+            naRuntimeError(c, "props.setValue() vector value has wrong size");
+    } else if(naIsNum(val)) {
+        double d = naNumValue(val).num;
+        if (SGMisc<double>::isNaN(d)) {
+            naRuntimeError(c, "props.setValue() passed a NaN");
+        }
+
+        result = node->setDoubleValue(d);
+    } else {
+        naRuntimeError(c, "props.setValue() called with unsupported value %s", s_val_description(val).c_str());
+    }
+    return naNum(result);
+}
+
 
 // Set the value of a node; returns true if it succeeded or
 // false if it failed. <val> can be a string, number, or a
@@ -312,28 +383,7 @@ static naRef f_setValue(naContext c, naRef me, int argc, naRef* args)
     NODEARG();
     MOVETARGET(naVec_size(argv) > 1, true);
     naRef val = naVec_get(argv, 0);
-    bool result = false;
-    if(naIsString(val)) {
-         result = node->setStringValue(naStr_data(val));
-    } else if(naIsVector(val)) {
-        if(naVec_size(val) == 3)
-            result = node->setValue(makeVecFromVector<SGVec3d>(val));
-        else if(naVec_size(val) == 4)
-            result = node->setValue(makeVecFromVector<SGVec4d>(val));
-        else
-            naRuntimeError(c, "props.setValue() vector value has wrong size");
-    } else {
-        if(!naIsNum(val))
-            naRuntimeError(c, "props.setValue() with non-number");
-
-        double d = naNumValue(val).num;
-        if (SGMisc<double>::isNaN(d)) {
-          naRuntimeError(c, "props.setValue() passed a NaN");
-        }
-
-        result = node->setDoubleValue(d);
-    }
-    return naNum(result);
+    return f_setValueHelper(c, node, val);
 }
 
 static naRef f_setIntValue(naContext c, naRef me, int argc, naRef* args)
@@ -362,6 +412,20 @@ static naRef f_setBoolValue(naContext c, naRef me, int argc, naRef* args)
     return naNum(node->setBoolValue(naTrue(val) ? true : false));
 }
 
+static naRef f_toggleBoolValue(naContext c, naRef me, int argc, naRef* args)
+{
+    using namespace simgear;
+
+    NODEARG();
+    MOVETARGET(naVec_size(argv) > 0, false);
+    if (node->getType() != props::BOOL) {
+        naRuntimeError(c, "props.toggleBoolValue() on non-bool prop");
+    }
+
+    const auto val = node->getBoolValue();
+    return naNum(node->setBoolValue(val ? false : true));
+}
+
 static naRef f_setDoubleValue(naContext c, naRef me, int argc, naRef* args)
 {
     NODEARG();
@@ -377,6 +441,120 @@ static naRef f_setDoubleValue(naContext c, naRef me, int argc, naRef* args)
     return naNum(node->setDoubleValue(r.num));
 }
 
+static naRef f_adjustValue(naContext c, naRef me, int argc, naRef* args)
+{
+    using namespace simgear;
+
+    NODEARG();
+    MOVETARGET(naVec_size(argv) > 1, false);
+    naRef r = naNumValue(naVec_get(argv, 0));
+    if (naIsNil(r))
+        naRuntimeError(c, "props.adjustValue() with non-number");
+
+    if (SGMisc<double>::isNaN(r.num)) {
+        naRuntimeError(c, "props.adjustValue() passed a NaN");
+    }
+
+    switch(node->getType()) {
+    case props::BOOL:
+    case props::INT:
+    case props::LONG:
+    case props::FLOAT:
+    case props::DOUBLE:
+        // fall through
+        break;
+
+    default:
+        naRuntimeError(c, "props.adjustValue() called on non-numeric type");
+        return naNil();
+    }
+
+    const auto dv = node->getDoubleValue();
+    return naNum(node->setDoubleValue(dv + r.num));
+}
+
+// Forward declaration
+static naRef f_setChildrenHelper(naContext c, SGPropertyNode_ptr node, char* name, naRef val);
+
+static naRef f_setValuesHelper(naContext c, SGPropertyNode_ptr node, naRef hash)
+{
+    if (!naIsHash(hash)) {
+        naRuntimeError(c, "props.setValues() with non-hash");
+    }
+
+    naRef keyvec = naNewVector(c);
+    naHash_keys(keyvec, hash);
+    naRef ret;
+
+    for (int i = 0; i < naVec_size(keyvec); i++) {
+        naRef key = naVec_get(keyvec, i);
+        if (! naIsScalar(key)) {
+            naRuntimeError(c, "props.setValues() with non-scalar key value");
+        }
+        char* keystr = naStr_data(naStringValue(c, key));
+        ret = f_setChildrenHelper(c, node, keystr, naHash_cget(hash, keystr));
+    }
+
+    return ret;
+}
+
+static naRef f_setValues(naContext c, naRef me, int argc, naRef* args)
+{
+    NODEARG();
+    MOVETARGET(naVec_size(argv) > 1, true);
+    naRef val = naVec_get(argv, 0);
+    return f_setValuesHelper(c, node, val);
+}
+
+static naRef f_setChildrenHelper(naContext c, SGPropertyNode_ptr node, char* name, naRef val)
+{
+    naRef ret;
+    try {
+        SGPropertyNode_ptr subnode = node->getNode(name, true);
+
+        if (naIsScalar(val)) {
+            ret =  f_setValueHelper(c, subnode, val);
+        } else if (naIsHash(val)) {
+            ret = f_setValuesHelper(c, subnode, val);
+        } else if (naIsVector(val)) {
+            char nameBuf[1024];
+            for (int i = 0; i < naVec_size(val); i++) {
+                const auto len = ::snprintf(nameBuf, sizeof(nameBuf), "%s[%i]", name, i);
+                if ((len < 0) || (len >= (int) sizeof(nameBuf))) {
+                  naRuntimeError(c, "Failed to create buffer for property name in setChildren");
+                }
+                ret = f_setChildrenHelper(c, node, nameBuf, naVec_get(val, i));
+            }
+        } else if (naIsNil(val)) {
+            // Nil value OK - no-op
+        } else {
+            // We have an error, but throwing a runtime error will prevent certain things from
+            // working (such as the pilot list)
+            // The nasal version would fail silently with invalid data - a runtime error will dump the stack and
+            // stop execution.
+            // Overall to be safer the new method should be functionally equivalent to keep compatibility.
+            //
+            //REMOVED: naRuntimeError(c, "props.setChildren() with unknown type");
+        }
+    } catch(const string& err) {
+        naRuntimeError(c, (char *)err.c_str());
+        return naNil();
+    }
+
+    return ret;
+}
+
+static naRef f_setChildren(naContext c, naRef me, int argc, naRef* args)
+{
+    NODEARG();
+    if(! naIsString(naVec_get(argv, 0))) {
+      naRuntimeError(c, "props.setChildren() with non-string first argument");
+    }
+
+    char* name = naStr_data(naVec_get(argv, 0));
+    naRef val = naVec_get(argv, 1);
+    return f_setChildrenHelper(c, node, name, val);
+}
 
 // Get the parent of this node as a ghost.
 // Forms:
@@ -536,6 +714,12 @@ static naRef f_removeChild(naContext c, naRef me, int argc, naRef* args)
     if(!naIsString(child) || !naIsNum(index)) return naNil();
     SGPropertyNode_ptr n;
     try {
+        n = node->getChild(naStr_data(child), (int)index.num);
+        if (n && n->getAttribute(SGPropertyNode::PROTECTED)) {
+            naRuntimeError(c, "props.Node.removeChild() called on protected child %s of %s",
+                           naStr_data(child), node->getPath().c_str());
+            return naNil();
+        }
         n = node->removeChild(naStr_data(child), (int)index.num);
     } catch (const string& err) {
         naRuntimeError(c, (char *)err.c_str());
@@ -555,17 +739,32 @@ static naRef f_removeChildren(naContext c, naRef me, int argc, naRef* args)
     naRef result = naNewVector(c);
     if(naIsNil(argv) || naVec_size(argv) == 0) {
         // Remove all children
-        for(int i = node->nChildren() - 1; i >=0; i--)
-            naVec_append(result, propNodeGhostCreate(c, node->removeChild(i)));
+        for(int i = node->nChildren() - 1; i >=0; i--) {
+            SGPropertyNode_ptr n = node->getChild(i);
+            if (n->getAttribute(SGPropertyNode::PROTECTED)) {
+                SG_LOG(SG_NASAL, SG_ALERT, "props.Node.removeChildren: node " <<
+                       n->getPath() << " is protected");
+                continue;
+            }
+
+            node->removeChild(i);
+            naVec_append(result, propNodeGhostCreate(c, n));
+        }
     } else {
         // Remove all children of a specified name
         naRef name = naVec_get(argv, 0);
         if(!naIsString(name)) return naNil();
         try {
-            vector<SGPropertyNode_ptr> children
-                = node->removeChildren(naStr_data(name));
-            for(unsigned int i=0; i<children.size(); i++)
-                naVec_append(result, propNodeGhostCreate(c, children[i]));
+            auto children = node->getChildren(naStr_data(name));
+            for (auto cn : children) {
+                if (cn->getAttribute(SGPropertyNode::PROTECTED)) {
+                    SG_LOG(SG_NASAL, SG_ALERT, "props.Node.removeChildren: node " <<
+                        cn->getPath() << " is protected");
+                    continue;
+                }
+                node->removeChild(cn);
+                naVec_append(result, propNodeGhostCreate(c, cn));
+            }
         } catch (const string& err) {
             naRuntimeError(c, (char *)err.c_str());
             return naNil();
@@ -593,17 +792,28 @@ static naRef f_removeAllChildren(naContext c, naRef me, int argc, naRef* args)
 static naRef f_alias(naContext c, naRef me, int argc, naRef* args)
 {
     NODEARG();
+    if (node->getAttribute(SGPropertyNode::PROTECTED)) {
+        naRuntimeError(c, "props.Node.alias() called on protected property %s",
+                       node->getPath().c_str());
+        return naNil();
+    }
     SGPropertyNode* al;
     naRef prop = naVec_get(argv, 0);
     try {
         if(naIsString(prop)) al = globals->get_props()->getNode(naStr_data(prop), true);
         else if(naIsGhost(prop)) al = static_cast<SGPropertyNode*>(naGhost_ptr(prop));
-        else throw string("props.alias() with bad argument");
-    } catch (const string& err) {
-        naRuntimeError(c, (char *)err.c_str());
+        else
+            throw sg_exception("props.alias() with bad argument");
+    } catch (sg_exception& err) {
+        naRuntimeError(c, err.what());
         return naNil();
     }
-    return naNum(node->alias(al));
+
+    bool withListeners = false;
+    if (naVec_size(argv) > 1) {
+        withListeners = static_cast<int>(naVec_get(argv, 1).num) != 0;
+    }
+    return naNum(node->alias(al, withListeners));
 }
 
 
@@ -681,8 +891,10 @@ static struct {
     { f_equals,             "_equals"             },
     { f_getValue,           "_getValue"           },
     { f_setValue,           "_setValue"           },
+    { f_setValues,          "_setValues"          },
     { f_setIntValue,        "_setIntValue"        },
     { f_setBoolValue,       "_setBoolValue"       },
+    { f_toggleBoolValue,    "_toggleBoolValue"    },
     { f_setDoubleValue,     "_setDoubleValue"     },
     { f_getParent,          "_getParent"          },
     { f_getChild,           "_getChild"           },
@@ -692,20 +904,56 @@ static struct {
     { f_removeChild,        "_removeChild"        },
     { f_removeChildren,     "_removeChildren"     },
     { f_removeAllChildren,  "_removeAllChildren"  },
+    { f_setChildren,        "_setChildren"        },
     { f_alias,              "_alias"              },
     { f_unalias,            "_unalias"            },
     { f_getAliasTarget,     "_getAliasTarget"     },
     { f_getNode,            "_getNode"            },
     { f_new,                "_new"                },
     { f_globals,            "_globals"            },
+    { f_isNumeric,          "_isNumeric"          },
+    { f_isInt,              "_isInt"              },
+    { f_adjustValue,        "_adjustValue"        },
     { 0, 0 }
 };
 
 naRef FGNasalSys::genPropsModule()
 {
-    naRef namespc = naNewHash(_context);
+    naRef namespc = naNewHash(d->_context);
     for(int i=0; propfuncs[i].name; i++)
         hashset(namespc, propfuncs[i].name,
-                naNewFunc(_context, naNewCCode(_context, propfuncs[i].func)));
+                naNewFunc(d->_context, naNewCCode(d->_context, propfuncs[i].func)));
     return namespc;
 }
+
+naRef FGNasalSys::getPropertyValue(naContext c, SGPropertyNode* node)
+{
+    using namespace simgear;
+    if (!node)
+        return naNil();
+    
+    switch(node->getType()) {
+    case props::BOOL:   case props::INT:
+    case props::LONG:   case props::FLOAT:
+    case props::DOUBLE:
+    {
+        double dv = node->getDoubleValue();
+        if (SGMisc<double>::isNaN(dv)) {
+          SG_LOG(SG_NASAL, SG_ALERT, "Nasal getValue: property " << node->getPath() << " is NaN");
+          return naNil();
+        }
+
+        return naNum(dv);
+    }
+
+    case props::STRING:
+    case props::UNSPECIFIED:
+        return NASTR(node->getStringValue().c_str());
+    case props::VEC3D:
+        return makeVectorFromVec(c, node->getValue<SGVec3d>());
+    case props::VEC4D:
+        return makeVectorFromVec(c, node->getValue<SGVec4d>());
+    default:
+        return naNil();
+    }
+}
