@@ -16,14 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
+
+#include <utility>
+
 #include "VoiceSynthesizer.hxx"
 #include <Main/globals.hxx>
 #include <Main/fg_props.hxx>
 #include <simgear/sg_inlines.h>
 #include <simgear/debug/logstream.hxx>
-#include <simgear/sound/readwav.hxx>
 #include <simgear/misc/sg_path.hxx>
-#include <OpenThreads/Thread>
+#include <simgear/threads/SGThread.hxx>
+
 #include <flite_hts_engine.h>
 
 using std::string;
@@ -33,7 +36,8 @@ static const char * VOICE_FILES[] = {
   "cstr_uk_female-1.0.htsvoice"
 };
 
-class FLITEVoiceSynthesizer::WorkerThread: public OpenThreads::Thread {
+class FLITEVoiceSynthesizer::WorkerThread : public SGThread
+{
 public:
   WorkerThread(FLITEVoiceSynthesizer * synthesizer)
       : _synthesizer(synthesizer)
@@ -48,6 +52,13 @@ void FLITEVoiceSynthesizer::WorkerThread::run()
 {
   for (;;) {
     SynthesizeRequest request = _synthesizer->_requests.pop();
+
+    // marker value indicating termination requested
+    if ((request.speed < 0.0) && (request.volume < 0.0)) {
+      SG_LOG(SG_SOUND, SG_DEBUG, "FLITE synthesis thread exiting");
+      return;
+    }
+
     if ( NULL != request.listener) {
       SGSharedPtr<SGSoundSample> sample = _synthesizer->synthesize(request.text, request.volume, request.speed, request.pitch);
       request.listener->SoundSampleReady( sample );
@@ -55,11 +66,19 @@ void FLITEVoiceSynthesizer::WorkerThread::run()
   }
 }
 
+SynthesizeRequest SynthesizeRequest::cancelThreadRequest()
+{
+  SynthesizeRequest marker;
+  marker.volume = -999.0;
+  marker.speed = -999.0;
+  return marker;
+}
+
 string FLITEVoiceSynthesizer::getVoicePath( voice_t voice )
 {
   if( voice < 0 || voice >= VOICE_UNKNOWN ) return string("");
   SGPath voicePath = globals->get_fg_root() / "ATC" /  VOICE_FILES[voice];
-  return voicePath.local8BitStr();
+  return voicePath.utf8Str();
 }
 
 string FLITEVoiceSynthesizer::getVoicePath( const string & voice )
@@ -76,6 +95,7 @@ void FLITEVoiceSynthesizer::synthesize( SynthesizeRequest & request)
 }
 
 FLITEVoiceSynthesizer::FLITEVoiceSynthesizer(const std::string & voice)
+    // REVIEW: Memory Leak - 1,696 bytes in 4 blocks are definitely lost in loss record 6,145 of 6,440
     : _engine(new Flite_HTS_Engine), _worker(new FLITEVoiceSynthesizer::WorkerThread(this)), _volume(6.0)
 {
   _volume = fgGetDouble("/sim/sound/voice-synthesizer/volume", _volume );
@@ -86,7 +106,8 @@ FLITEVoiceSynthesizer::FLITEVoiceSynthesizer(const std::string & voice)
 
 FLITEVoiceSynthesizer::~FLITEVoiceSynthesizer()
 {
-  _worker->cancel();
+  // push the special marker value
+  _requests.push(SynthesizeRequest::cancelThreadRequest());
   _worker->join();
   Flite_HTS_Engine_clear(_engine);
 }
@@ -94,18 +115,22 @@ FLITEVoiceSynthesizer::~FLITEVoiceSynthesizer()
 SGSoundSample * FLITEVoiceSynthesizer::synthesize(const std::string & text, double volume, double speed, double pitch )
 {
   SG_CLAMP_RANGE( volume, 0.0, 1.0 );
-  SG_CLAMP_RANGE( speed, 0.0, 1.0 );
-  SG_CLAMP_RANGE( pitch, 0.0, 1.0 );
+  SG_CLAMP_RANGE( speed, 0.0, 10.0 );
+  SG_CLAMP_RANGE( pitch, 0.0, 10.0 );
   HTS_Engine_set_volume( &_engine->engine, _volume );
   HTS_Engine_set_speed( &_engine->engine, 0.8 + 0.4 * speed );
   HTS_Engine_add_half_tone(&_engine->engine, -4.0 + 8.0 * pitch );
 
     
-  ALvoid* data;
-  ALsizei rate, count;
+  void* data;
+  int rate, count;
   if ( FALSE == Flite_HTS_Engine_synthesize_samples_mono16(_engine, text.c_str(), &data, &count, &rate)) return NULL;
 
-  return new SGSoundSample(&data,
+  auto buf = std::unique_ptr<unsigned char, decltype(free)*>{
+    reinterpret_cast<unsigned char*>( data ),
+    free
+  };
+  return new SGSoundSample(std::move(buf),
                            count * sizeof(short),
                            rate,
                            SG_SAMPLE_MONO16);
