@@ -33,11 +33,14 @@
 
 #include <Main/fg_props.hxx>
 #include <Main/globals.hxx>
+#include <Main/sentryIntegration.hxx>
 #include <Sound/soundmanager.hxx>
+#include <algorithm>
 
+#include <simgear/debug/ErrorReportingCallback.hxx>
+#include <simgear/misc/sg_path.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/props/props_io.hxx>
-#include <simgear/misc/sg_path.hxx>
 #include <simgear/sound/xmlsound.hxx>
 
 FGFX::FGFX ( const std::string &refname, SGPropertyNode *props ) :
@@ -61,6 +64,13 @@ FGFX::FGFX ( const std::string &refname, SGPropertyNode *props ) :
     _avionics_ext = _props->getNode("sim/sound/avionics/external-view", true);
     _internal = _props->getNode("sim/current-view/internal", true);
 
+    _atc_enabled = _props->getNode("sim/sound/atc/enabled", true);
+    _atc_volume = _props->getNode("sim/sound/atc/volume", true);
+    _atc_ext = _props->getNode("sim/sound/atc/external-view", true);
+
+    _machwave_active = _props->getNode("sim/sound/machwave/active", true);
+    _machwave_volume = _props->getNode("sim/sound/machwave/offset-m", true);
+
     _smgr = globals->get_subsystem<FGSoundManager>();
     if (!_smgr) {
       return;
@@ -70,32 +80,28 @@ FGFX::FGFX ( const std::string &refname, SGPropertyNode *props ) :
     _refname = refname;
     _smgr->add(this, refname);
 
-    if (!_is_aimodel)
+    if (!_is_aimodel) // only for the main aircraft
     {
         _avionics = _smgr->find("avionics", true);
         _avionics->tie_to_listener();
+        
+        _atc = _smgr->find("atc", true);
+        _atc->tie_to_listener();
     }
-    else
-        _avionics = NULL;
 }
 
-void FGFX::unbind()
+void FGFX::shutdown()
 {
-    if (_smgr)
-    {
-        _smgr->remove(_refname);
-    }
+    _smgr->remove(_refname);
+    _xmlSounds.clear();
 }
 
 FGFX::~FGFX ()
 {
-    for (unsigned int i = 0; i < _sound.size(); i++ ) {
-        delete _sound[i];
-    }
-    _sound.clear();
+    // check that shutdown has been called
+    assert(!_smgr->exists(_refname));
 }
 
-
 void
 FGFX::init()
 {
@@ -114,54 +120,49 @@ FGFX::init()
     SGPath path = globals->resolve_aircraft_path(path_str);
     if (path.isNull())
     {
+        simgear::reportFailure(simgear::LoadFailure::NotFound, simgear::ErrorCode::AudioFX,
+                               "Failed to find FX XML file:" + path_str, sg_location{path_str});
         SG_LOG(SG_SOUND, SG_ALERT,
                "File not found: '" << path_str);
         return;
     }
-    SG_LOG(SG_SOUND, SG_INFO, "Reading sound " << node->getName()
+    SG_LOG(SG_SOUND, SG_INFO, "Reading sound " << node->getNameString()
            << " from " << path);
 
     SGPropertyNode root;
     try {
         readProperties(path, &root);
-    } catch (const sg_exception &) {
-        SG_LOG(SG_SOUND, SG_ALERT,
-               "Error reading file '" << path << '\'');
+    } catch (const sg_exception& e) {
+        simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::AudioFX,
+                               "Failure loading FX XML:" + e.getFormattedMessage(), e.getLocation());
         return;
     }
 
     node = root.getNode("fx");
     if(node) {
         for (int i = 0; i < node->nChildren(); ++i) {
-            SGXmlSound *soundfx = new SGXmlSound();
-  
+            SGXmlSoundRef soundfx{new SGXmlSound};
+
             try {
-                soundfx->init( _props, node->getChild(i), this, _avionics,
+                bool ok = soundfx->init( _props, node->getChild(i), this, _avionics,
                                path.dir() );
-                _sound.push_back( soundfx );
+                if (ok) {
+                    _xmlSounds.push_back(soundfx);
+                }
             } catch ( sg_exception &e ) {
                 SG_LOG(SG_SOUND, SG_ALERT, e.getFormattedMessage());
-                delete soundfx;
+                simgear::reportFailure(simgear::LoadFailure::BadData, simgear::ErrorCode::AudioFX,
+                                       "Failure creating Audio FX:" + e.getFormattedMessage(), path);
             }
         }
     }
 }
 
-
-void
-FGFX::reinit()
-{
-    for ( unsigned int i = 0; i < _sound.size(); i++ ) {
-        delete _sound[i];
-    }
-    _sound.clear();
-    init();
-}
-
-
 void
 FGFX::update (double dt)
 {
+    // FGFX::update is called via the SGSoundManager update(), which
+    // calls update on all its sample-groups.
     if (!_smgr) {
         return;
     }
@@ -169,16 +170,17 @@ FGFX::update (double dt)
     if (!_active && _smgr->is_active())
     {
         _active = true;
-        for ( unsigned int i = 0; i < _sound.size(); i++ ) {
-            _sound[i]->start();
+        for (auto s : _xmlSounds) {
+            s->start();
         }
     }
 
       
     if ( _enabled->getBoolValue() ) {
-        if ( _avionics && _avionics_enabled->getBoolValue())
+        if ( _avionics)
         {
-            if (_avionics_ext->getBoolValue() || _internal->getBoolValue()) {
+            const bool e = _avionics_enabled->getBoolValue();
+            if (e && (_avionics_ext->getBoolValue() || _internal->getBoolValue())) {
                 // avionics sound is enabled
                 _avionics->resume(); // no-op if already in resumed state
                 _avionics->set_volume( _avionics_volume->getFloatValue() );
@@ -186,13 +188,28 @@ FGFX::update (double dt)
             else
                 _avionics->suspend();
         }
+        
+        if ( _atc)
+        {
+            const bool e = _atc_enabled->getBoolValue();
+            if (e && (_atc_ext->getBoolValue() || _internal->getBoolValue())) {
+                // ATC sound is enabled
+                _atc->resume(); // no-op if already in resumed state
+                _atc->set_volume( _atc_volume->getFloatValue() );
+            }
+            else
+                _atc->suspend();
+        }
+
+        _machwave_active->setBoolValue(_mInCone);
+        _machwave_volume->setFloatValue(_mOffset_m);
 
         set_volume( _volume->getDoubleValue() );
         resume();
 
         // update sound effects if not paused
-        for ( unsigned int i = 0; i < _sound.size(); i++ ) {
-            _sound[i]->update(dt);
+        for (auto xs : _xmlSounds) {
+            xs->update(dt);
         }
 
         SGSampleGroup::update(dt);
