@@ -7,16 +7,14 @@
 #include <osg/Viewport>
 #include <osgViewer/Viewer>
 
-#include <plib/pu.h>
+#include <GUI/FlightGear_pu.h>
 #include <Main/fg_props.hxx>
 #include "CameraGroup.hxx"
 #include "FGEventHandler.hxx"
 #include "WindowSystemAdapter.hxx"
+#include "WindowBuilder.hxx"
 #include "renderer.hxx"
-
-#if !defined(X_DISPLAY_MISSING)
-#define X_DOUBLE_SCROLL_BUG 1
-#endif
+#include "sview.hxx"
 
 #ifdef SG_MAC
 // hack - during interactive resize on Mac, OSG queues and then flushes
@@ -45,7 +43,8 @@ FGEventHandler::FGEventHandler() :
     resizable(true),
     mouseWarped(false),
     scrollButtonPressed(false),
-    changeStatsCameraRenderOrder(false)
+    changeStatsCameraRenderOrder(false),
+    m_composite_viewer_enabled(fgGetNode("/sim/rendering/composite-viewer-enabled", true))
 {
     using namespace osgGA;
     statsHandler->setKeyEventTogglesOnScreenStats(displayStatsKey);
@@ -59,6 +58,12 @@ FGEventHandler::FGEventHandler() :
     _print = fgGetNode("/sim/rendering/print-statistics", true);
 }
 
+void FGEventHandler::clear()
+{
+    _display.clear();
+    _print.clear();
+}
+
 void FGEventHandler::reset()
 {
     _display = fgGetNode("/sim/rendering/on-screen-statistics", true);
@@ -76,26 +81,50 @@ void FGEventHandler::init(const osgGA::GUIEventAdapter& ea,
 #endif
 
 // Calculate event coordinates in the viewport of the GUI camera, if
-// possible. Otherwise return false and (-1, -1).
-namespace
-{
-bool
-eventToViewport(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter& us,
+// possible. Otherwise sets (x, y) to (-1, -1).
+FGEventHandler::WindowType
+FGEventHandler::eventToViewport(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter& us,
                 int& x, int& y)
 {
+    WindowType  ret = WindowType_NONE;
+    
+    flightgear::WindowBuilder* window_builder = flightgear::WindowBuilder::getWindowBuilder();
+    flightgear::GraphicsWindow* main_window = window_builder->getDefaultWindow();
+
     x = -1;
     y = -1;
 
     const osg::GraphicsContext* eventGC = ea.getGraphicsContext();
     if( !eventGC )
-      return false; // TODO how can this happen?
+      return WindowType_NONE; // TODO how can this happen?
     const osg::GraphicsContext::Traits* traits = eventGC->getTraits();
-    osg::Camera* guiCamera = getGUICamera(CameraGroup::getDefault());
-    if (!guiCamera)
-        return false;
-    osg::Viewport* vport = guiCamera->getViewport();
-    if (!vport)
-        return false;
+    
+    osg::Viewport* vport;
+    
+    if (m_composite_viewer_enabled->getBoolValue()
+            && eventGC != main_window->gc.get()) {
+        // CompositeViewer is enabled and this is not the main window.
+        simgear::compositor::Compositor* compositor = SviewGetEventViewport(ea);
+        if (!compositor) {
+            SG_LOG(SG_GENERAL, SG_DEBUG, "SviewGetEventViewport() returned nullptr");
+            return WindowType_NONE;
+        }
+        vport = compositor->getViewport();
+        if (!vport) {
+            SG_LOG(SG_GENERAL, SG_ALERT, "compositor->getViewport() is nullptr");
+            return WindowType_NONE;
+        }
+        ret = WindowType_SVIEW;
+    }
+    else {
+        osg::Camera* guiCamera = getGUICamera(CameraGroup::getDefault());
+        if (!guiCamera)
+            return WindowType_NONE;
+        vport = guiCamera->getViewport();
+        if (!vport)
+            return WindowType_NONE;
+        ret = WindowType_MAIN;
+    }
     
     // Scale x, y to the dimensions of the window
     double wx = (((ea.getX() - ea.getXmin()) / (ea.getXmax() - ea.getXmin()))
@@ -108,11 +137,40 @@ eventToViewport(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter& us,
         // downwards".
         x = wx - vport->x();
         y = vport->height() - (wy - vport->y());
-        return true;
-    } else {
-        return false;
     }
+    return ret;
 }
+
+/* A hack for when we are linked with OSG-3.4 and CompositeViewer is
+enabled. It seems that OSG-3.4 incorrectly calls our event handler for
+extra view windows (e.g. resize/close events), so we try to detect
+this. Unfortunately OSG also messes up <ea>'s graphics context pointer so this
+does't alwys work. */
+bool FGEventHandler::isMainWindow(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter& us)
+{
+    int x;
+    int y;
+    return eventToViewport(ea, us, x, y) == WindowType_MAIN;
+}
+
+void FGEventHandler::setWindowRectangleInteriorWithCorrection(osgViewer::GraphicsWindow* window, int x, int y, int width, int height)
+{
+    // Store (x y) in our state so that our handle() event handler can
+    // compare the requested position with the actual position and update
+    // (m_setWindowRectangle_delta_x m_setWindowRectangle_delta_y) accordingly.
+    //
+    SG_LOG(SG_VIEW, SG_DEBUG, "FGEventHandler::setWindowRectangle(): pos=(" << x << " " << y << ")"
+            << " delta=(" << m_setWindowRectangle_delta_x << " " << m_setWindowRectangle_delta_y << ")"
+            );
+    m_setWindowRectangle_called = true;
+    m_setWindowRectangle_called_x = x;
+    m_setWindowRectangle_called_y = y;
+    window->setWindowRectangle(
+            x - m_setWindowRectangle_delta_x,
+            y - m_setWindowRectangle_delta_y,
+            width,
+            height
+            );
 }
 
 bool FGEventHandler::handle(const osgGA::GUIEventAdapter& ea,
@@ -121,15 +179,19 @@ bool FGEventHandler::handle(const osgGA::GUIEventAdapter& ea,
     // Event handlers seem to be called even if the according event has already
     // been handled. Already handled events shouldn't be handled multiple times
     // so we need to exit here manually.
-    if(    ea.getHandled()
-           // Let mouse move events pass to correctly handle mouse cursor hide
-           // timeout while moving just on the canvas gui.
-           // TODO We should clean up the whole mouse input and make hide
-           //      timeout independent of the event handler which consumed the
-           //      event.
-        && ea.getEventType() != osgGA::GUIEventAdapter::MOVE
-        && ea.getEventType() != osgGA::GUIEventAdapter::DRAG )
-      return false;
+    if (ea.getHandled()
+        // Let mouse move events pass to correctly handle mouse cursor hide
+        // timeout while moving just on the canvas gui.
+        // TODO We should clean up the whole mouse input and make hide
+        //      timeout independent of the event handler which consumed the
+        //      event.
+
+        // if we see a release, still process for active pick callbacks
+        // https://sourceforge.net/p/flightgear/codetickets/2347/
+
+        && ea.getEventType() != osgGA::GUIEventAdapter::MOVE && ea.getEventType() != osgGA::GUIEventAdapter::DRAG && ea.getEventType() != osgGA::GUIEventAdapter::RELEASE) {
+        return false;
+    }
 
     int x = 0;
     int y = 0;
@@ -152,7 +214,8 @@ bool FGEventHandler::handle(const osgGA::GUIEventAdapter& ea,
     case osgGA::GUIEventAdapter::PUSH:
     case osgGA::GUIEventAdapter::RELEASE:
     {
-        bool mainWindow = eventToViewport(ea, us, x, y);
+        WindowType window_type = eventToViewport(ea, us, x, y);
+        bool mainWindow = (window_type == WindowType_MAIN);
         int button = 0;
         switch (ea.getButton()) {
         case osgGA::GUIEventAdapter::LEFT_MOUSE_BUTTON:
@@ -173,12 +236,9 @@ bool FGEventHandler::handle(const osgGA::GUIEventAdapter& ea,
     }
     case osgGA::GUIEventAdapter::SCROLL:
     {
-        bool mainWindow = eventToViewport(ea, us, x, y);
-#ifdef X_DOUBLE_SCROLL_BUG
-        scrollButtonPressed = !scrollButtonPressed;
-        if (!scrollButtonPressed) // Drop the button release event
-            return true;
-#endif
+        WindowType window_type = eventToViewport(ea, us, x, y);
+        bool mainWindow = (window_type == WindowType_MAIN);
+        
         int button;
         if (ea.getScrollingMotion() == osgGA::GUIEventAdapter::SCROLL_2D) {
             if (ea.getScrollingDeltaY() > 0)
@@ -217,14 +277,38 @@ bool FGEventHandler::handle(const osgGA::GUIEventAdapter& ea,
         // that with osgViewer.
         if (mouseWarped)
             return true;
-        if (eventToViewport(ea, us, x, y) && mouseMotionHandler)
+        if (eventToViewport(ea, us, x, y) != WindowType_NONE && mouseMotionHandler)
             (*mouseMotionHandler)(x, y, &ea);
         return true;
     case osgGA::GUIEventAdapter::RESIZE:
         SG_LOG(SG_VIEW, SG_DEBUG, "FGEventHandler::handle: RESIZE event " << ea.getWindowHeight() << " x " << ea.getWindowWidth() << ", resizable: " << resizable);
+        if (!isMainWindow(ea, us)) {
+            return true;
+        }
         CameraGroup::getDefault()->resized();
-        if (resizable)
-          globals->get_renderer()->resize(ea.getWindowWidth(), ea.getWindowHeight());
+        if (resizable) {
+            if (m_setWindowRectangle_called) {
+                // Update m_setWindowRectangle_delta_x and
+                // m_setWindowRectangle_delta_y so that our
+                // setWindowRectangle() compensates for the window furniture
+                // differences in future calls.
+                //
+                m_setWindowRectangle_called = false;
+                int error_x = ea.getWindowX() - m_setWindowRectangle_called_x;
+                int error_y = ea.getWindowY() - m_setWindowRectangle_called_y;
+                SG_LOG(SG_VIEW, SG_BULK, "m_setWindowRectangle_called is set:"
+                        << " m_setWindowRectangle_delta=("
+                        << m_setWindowRectangle_delta_x << " " << m_setWindowRectangle_delta_y << ")"
+                        << " m_setWindowRectangle_called_=("
+                        << m_setWindowRectangle_called_x << " " << m_setWindowRectangle_called_y << ")"
+                        << " ea.getWindow=(" << ea.getWindowX() << " " << ea.getWindowY() << ")"
+                        << " error=(" << error_x << " " << error_y << ")"
+                        );
+                m_setWindowRectangle_delta_x += error_x;
+                m_setWindowRectangle_delta_y += error_y;
+          }
+          globals->get_renderer()->resize(ea.getWindowWidth(), ea.getWindowHeight(), ea.getWindowX(), ea.getWindowY());
+        }
         statsHandler->handle(ea, us);
       #ifdef SG_MAC
         // work around OSG Cocoa-Viewer issue with resize event handling,
@@ -234,6 +318,10 @@ bool FGEventHandler::handle(const osgGA::GUIEventAdapter& ea,
       #endif
         return true;
      case osgGA::GUIEventAdapter::CLOSE_WINDOW:
+        if (!isMainWindow(ea, us)) {
+            return true;
+        }
+        // Fall through.
     case osgGA::GUIEventAdapter::QUIT_APPLICATION:
         fgOSExit(0);
         return true;
@@ -435,11 +523,15 @@ void FGEventHandler::handleStats(osgGA::GUIActionAdapter& us)
     }
 }
 
-void eventToWindowCoords(const osgGA::GUIEventAdapter* ea,
+bool eventToWindowCoords(const osgGA::GUIEventAdapter* ea,
                          double& x, double& y)
 {
     using namespace osg;
     const GraphicsContext* gc = ea->getGraphicsContext();
+    if (!gc || !gc->getTraits()) {
+        return false;
+    }
+
     const GraphicsContext::Traits* traits = gc->getTraits() ;
     // Scale x, y to the dimensions of the window
     x = (((ea->getX() - ea->getXmin()) / (ea->getXmax() - ea->getXmin()))
@@ -448,23 +540,8 @@ void eventToWindowCoords(const osgGA::GUIEventAdapter* ea,
          * (double)traits->height);
     if (ea->getMouseYOrientation() == osgGA::GUIEventAdapter::Y_INCREASING_DOWNWARDS)
         y = (double)traits->height - y;
+
+    return true;
 }
 
-#if 0
-void eventToWindowCoordsYDown(const osgGA::GUIEventAdapter* ea,
-                              double& x, double& y)
-{
-    using namespace osg;
-    const GraphicsContext* gc = ea->getGraphicsContext();
-    const GraphicsContext::Traits* traits = gc->getTraits() ;
-    // Scale x, y to the dimensions of the window
-    x = (((ea->getX() - ea->getXmin()) / (ea->getXmax() - ea->getXmin()))
-         * (double)traits->width);
-    y = (((ea->getY() - ea->getYmin()) / (ea->getYmax() - ea->getYmin()))
-         * (double)traits->height);
-    if (ea->getMouseYOrientation() == osgGA::GUIEventAdapter::Y_INCREASING_UPWARDS)
-        y = (double)traits->height - y;
-}
-#endif
-    
 }
