@@ -18,7 +18,13 @@
  */
 
 #include "test_ls_matrix.hxx"
+#include "testAeroElement.hxx"
+#include "testYASimAtmosphere.hxx"
+#include "testYASimGear.hxx"
 
 
 // Set up the unit tests.
+CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(AeroElementTests, "Unit tests");
 CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(LaRCSimMatrixTests, "Unit tests");
+CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(YASimAtmosphereTests, "Unit tests");
+CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(YASimGearTests, "Unit tests");
