@@ -1,6 +1,5 @@
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+
+#include <config.h>
 
 #ifdef HAVE_WINDOWS_H
 #  include <windows.h>
@@ -8,10 +7,12 @@
 #  include <netinet/in.h>       // htonl() ntohl()
 #endif
 
+#include <cstdlib>
+
+#include <chrono>
 #include <iostream>
 #include <string>
-
-#include <plib/sg.h>
+#include <thread>
 
 #include <simgear/constants.h>
 #include <simgear/io/lowlevel.hxx> // endian tests
@@ -73,7 +74,7 @@ bool inited = false;
 // an exception whenever a "bad" floating point value is loaded into a
 // floating point register.  Solaris is notorious for this, but then
 // so is LynxOS on the PowerPC.  By translating the data in place,
-// there is no need to load a FP register with the "corruped" floating
+// there is no need to load a FP register with the "corrupted" floating
 // point value.  By doing the BIG_ENDIAN test, I can optimize the
 // routine for big-endian processors so it can be as efficient as
 // possible
@@ -148,9 +149,9 @@ static void midg2fg( const MIDGpos pos, const MIDGatt att,
     fdm->v_north = 0.0;
     fdm->v_east = 0.0;
     fdm->v_down = 0.0;
-    fdm->v_wind_body_north = 0.0;
-    fdm->v_wind_body_east = 0.0;
-    fdm->v_wind_body_down = 0.0;
+    fdm->v_body_u = 0.0;
+    fdm->v_body_v = 0.0;
+    fdm->v_body_w = 0.0;
     fdm->stall_warning = 0.0;
 
     fdm->A_X_pilot = 0.0;
@@ -224,9 +225,9 @@ static void midg2fg( const MIDGpos pos, const MIDGatt att,
     htonf(fdm->v_north);
     htonf(fdm->v_east);
     htonf(fdm->v_down);
-    htonf(fdm->v_wind_body_north);
-    htonf(fdm->v_wind_body_east);
-    htonf(fdm->v_wind_body_down);
+    htonf(fdm->v_body_u);
+    htonf(fdm->v_body_v);
+    htonf(fdm->v_body_w);
 
     htonf(fdm->A_X_pilot);
     htonf(fdm->A_Y_pilot);
@@ -545,7 +546,7 @@ int main( int argc, char **argv ) {
             double elapsed_us = (current_time_stamp - last_time_stamp).toUSecs();
             if ( elapsed_us < (frame_us - 2000) ) {
                 double requested_us = (frame_us - elapsed_us) - 2000 ;
-                ulMilliSecondSleep ( (int)(requested_us / 1000.0) ) ;
+                std::this_thread::sleep_for(std::chrono::milliseconds((int) (requested_us / 1000.0)));
             }
             current_time_stamp.stamp();
             while ( (current_time_stamp - last_time_stamp).toUSecs() < frame_us ) {
@@ -574,19 +575,19 @@ int main( int argc, char **argv ) {
         SGSerialPort input( serialdev, 115200 );
         if ( !input.is_enabled() ) {
             cout << "Cannot open: " << serialdev << endl;
-            return false;
+            return EXIT_FAILURE;
         }
 
         // open up the data log file if requested
         if ( !outfile.length() ) {
             cout << "no --outfile <name> specified, cannot capture data!"
                  << endl;
-            return false;
+            return EXIT_FAILURE;
         }
         SGFile output( outfile );
         if ( !output.open( SG_IO_OUT ) ) {
             cout << "Cannot open: " << outfile << endl;
-            return false;
+            return EXIT_FAILURE;
         }
 
         while ( input.is_enabled() ) {
