@@ -1,6 +1,5 @@
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
+
+#include <config.h>
 
 #ifdef HAVE_WINDOWS_H
 #  include <windows.h>
@@ -8,15 +7,12 @@
 #  include <netinet/in.h>       // htonl() ntohl()
 #endif
 
-#ifndef _WIN32
-#  include <strings.h>		// for bzero()
-#else
-#  define bzero(a,b) memset(a,0,b)
-#endif
+#include <cstdlib>
+
+#include <chrono>
 #include <iostream>
 #include <string>
-
-#include <plib/sg.h>
+#include <thread>
 
 #include <simgear/constants.h>
 #include <simgear/io/lowlevel.hxx> // endian tests
@@ -964,7 +960,7 @@ int main( int argc, char **argv ) {
                 double elapsed_us = (current_time_stamp - last_time_stamp).toUSecs();
                 if ( elapsed_us < (frame_us - 2000) ) {
                     double requested_us = (frame_us - elapsed_us) - 2000 ;
-                    ulMilliSecondSleep ( (int)(requested_us / 1000.0) ) ;
+                    std::this_thread::sleep_for(std::chrono::milliseconds((int) (requested_us / 1000.0)));
                 }
                 current_time_stamp.stamp();
                 while ( (current_time_stamp - last_time_stamp).toUSecs() < frame_us ) {
@@ -995,15 +991,14 @@ int main( int argc, char **argv ) {
     } else if ( serialdev.length() ) {
         // process incoming data from the serial port
 
-        int count = 0;
         double current_time = 0.0;
         double last_time = 0.0;
 
-        gps gpspacket; bzero( &gpspacket, sizeof(gpspacket) );
-	imu imupacket; bzero( &imupacket, sizeof(imupacket) );
-	nav navpacket; bzero( &navpacket, sizeof(navpacket) );
-	servo servopacket; bzero( &servopacket, sizeof(servopacket) );
-	health healthpacket; bzero( &healthpacket, sizeof(healthpacket) );
+        gps gpspacket; memset( &gpspacket, 0, sizeof gpspacket );
+	imu imupacket; memset( &imupacket, 0, sizeof imupacket );
+	nav navpacket; memset( &navpacket, 0, sizeof navpacket );
+	servo servopacket; memset( &servopacket, 0, sizeof servopacket );
+	health healthpacket; memset( &healthpacket, 0, sizeof healthpacket );
 
         double gps_time = 0.0;
         double imu_time = 0.0;
@@ -1017,19 +1012,19 @@ int main( int argc, char **argv ) {
         SGSerialPort uavcom( serialdev, 115200 );
         if ( !uavcom.is_enabled() ) {
             cout << "Cannot open: " << serialdev << endl;
-            return false;
+            return EXIT_FAILURE;
         }
 
         // open up the data log file if requested
         if ( !outfile.length() ) {
             cout << "no --outfile <name> specified, cannot capture data!"
                  << endl;
-            return false;
+            return EXIT_FAILURE;
         }
         SGFile log( outfile );
         if ( !log.open( SG_IO_OUT ) ) {
             cout << "Cannot open: " << outfile << endl;
-            return false;
+            return EXIT_FAILURE;
         }
 
         // add some test commands
@@ -1044,7 +1039,6 @@ int main( int argc, char **argv ) {
 					 &imupacket, &navpacket, &servopacket,
 					 &healthpacket, ignore_checksum );
             // cout << "message id = " << id << endl;
-            count++;
 
             telnet.process();
 
